function outputParameters=VidaCenterlineRestitch(inputParameters)

fileRootName = inputParameters.fileRootName;
intermediateResultsFolder = inputParameters.intermediateResultsFolder;
processedDataFolder = inputParameters.processedDataFolder;
vascEnhancedMaskFileName = inputParameters.vascEnhancedMaskFileName;
subVolumeOverlapSize = inputParameters.subVolumeOverlapSize;
fullStackSize = inputParameters.fullStackSize;
blockGrid = inputParameters.blockGrid;
vascBlockCenterlineMaskFileNameList = inputParameters.vascBlockCenterlineMaskFileNameList;
%-----
[pname,bname,ext]=fileparts(vascEnhancedMaskFileName);
vascCenterlineMaskFileName=fullfile(pname,[fileRootName,'-VascCenterline.mat']);
%-----


disp(['Restitching centerline Blocks']);


centerlineMask = false(fullStackSize);
overlapOffset = ceil(subVolumeOverlapSize/2);

minX1 = Inf; minX2 = Inf; minX3 = Inf;
maxX1 = 0; maxX2 = 0; maxX3 = 0;
for gridIter = 1:length(blockGrid);
    blockIterLims=blockGrid{gridIter};
    minX1 = min(minX1,blockIterLims.x1min);
    minX2 = min(minX2,blockIterLims.x2min);
    minX3 = min(minX3,blockIterLims.x3min);
    maxX1 = max(maxX1,blockIterLims.x1max);
    maxX2 = max(maxX2,blockIterLims.x2max);
    maxX3 = max(maxX3,blockIterLims.x3max);
end


%Concatenate the lists of centerline point with appropriate block offsets
compiledPixelList = [];
for blockIter = 1:length(blockGrid)
    blockIterLims = blockGrid{blockIter};
    %-----
    tempLoadVar = load(vascBlockCenterlineMaskFileNameList{blockIter});
    tempFieldNames = fieldnames(tempLoadVar);
    vascBlockCenterlineMask = tempLoadVar.(tempFieldNames{1});
    clear tempLoadVar tempFieldNames;
    if isstruct(vascBlockCenterlineMask),
        currentPixelList = vascBlockCenterlineMask.pixelList;
        currentMaskSize = vascBlockCenterlineMask.maskSize;
    else%if isstruct(vascBlockCenterlineMask),
        currentPixelList = find(vascBlockCenterlineMask);
        currentMaskSize = size(vascBlockCenterlineMask);
    end%if-else isstruct(vascBlockCenterlineMask),
    %-----
    [x1List,x2List,x3List] = ind2sub(currentMaskSize,currentPixelList);
    %Crop the list to minimize overlap
    x1MinSubset = ones(size(x1List));
    x1MaxSubset = ones(size(x1List));
    x2MinSubset = ones(size(x2List));
    x2MaxSubset = ones(size(x2List));
    x3MinSubset = ones(size(x3List));
    x3MaxSubset = ones(size(x3List));
    %---
    if blockIterLims.x1min ~= minX1, x1MinSubset = x1List >= (1+overlapOffset); end
    if blockIterLims.x1max ~= maxX1, x1MaxSubset = x1List < (currentMaskSize(1)-overlapOffset); end
    if blockIterLims.x2min ~= minX2, x2MinSubset = x2List >= (1+overlapOffset); end
    if blockIterLims.x2max ~= maxX2, x2MaxSubset = x2List < (currentMaskSize(2)-overlapOffset); end
    if blockIterLims.x3min ~= minX3, x3MinSubset = x3List >= (1+overlapOffset); end
    if blockIterLims.x3max ~= maxX3, x3MaxSubset = x3List < (currentMaskSize(3)-overlapOffset); end
    %-----
    x1Sub = and(x1MinSubset,x1MaxSubset);
    x2Sub = and(x2MinSubset,x2MaxSubset);
    x3Sub = and(x3MinSubset,x3MaxSubset);
    allSub = and(and(x1Sub,x2Sub),x3Sub);
    x1ListSub = x1List(allSub);
    x2ListSub = x2List(allSub);
    x3ListSub = x3List(allSub);
    %-----
    x1ListSub = x1ListSub + blockIterLims.x1min - 1;
    x2ListSub = x2ListSub + blockIterLims.x2min - 1;
    x3ListSub = x3ListSub + blockIterLims.x3min - 1;
    %-----
    adjustedPixelList = sub2ind(fullStackSize,x1ListSub,x2ListSub,x3ListSub);
    compiledPixelList = cat(1,compiledPixelList,adjustedPixelList);
end%for blockIter = 1:length(blockGrid)
%-----
vascCenterlineMask = [];
vascCenterlineMask.pixelList = compiledPixelList;
vascCenterlineMask.maskSize = fullStackSize;
saveSize = (getfield(whos('vascCenterlineMask'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save(vascCenterlineMaskFileName,'vascCenterlineMask',scs);
%---

%-----
outputParameters = inputParameters;
outputParameters.vascCenterlineMaskFileName = vascCenterlineMaskFileName; %note the difference between vascCenterlineMaskFileName and centerlineBlockFileNames
