function outExp = VidaChannelSub(inExp)

%this function runs through a sequence of channel subtraction
%e.g., let channelSubtractionSequence = 
%   {'FITC-*' , 'HORD-*', 'vasc-*'; ...
%    'NEUN-*' , 'vasc-*', 'temp-*'; ...
%    'temp-*' , 'HORD-*', 'neur-*'};
%this will first subtract hord from fitc and place it in vasc
%then subtract vasc from neun and place it in temp
%and finally subtract hord from temp and place it in neur
%then bleeding channel (the second entry) must be relatively pure
%channel subtraction factors are determined automatically
%number and size of files for each channel must match
%no other files that fit the file name string description can
%be in the named folder.
%-----
processedDataFolder = inExp.globalParameters.processedDataFolder;
intermediateResultsFolder = inExp.globalParameters.intermediateResultsFolder;
voxelSizeInMicrons = inExp.globalParameters.voxelSizeInMicrons;
%-----
channelSubtractionSequence = inExp.channelSubParameters.channelSubtractionSequence;
bleedMaskTopFractionList = inExp.channelSubParameters.bleedMaskTopFractionList;
maxSubtractionFactorList = inExp.channelSubParameters.maxSubtractionFactorList;
numFactors = inExp.channelSubParameters.numFactors;  
skipFirstNFiles = inExp.channelSubParameters.skipFirstNFiles; 
skipLastNFiles = inExp.channelSubParameters.skipLastNFiles;  
YXZsubBlockSizeInMicrons = inExp.channelSubParameters.YXZsubBlockSizeInMicrons;  
startFrameFraction = inExp.channelSubParameters.startFrameFraction; 
numFramesToProject = inExp.channelSubParameters.numFramesToProject;  
fitPolynomialDegree = inExp.channelSubParameters.fitPolynomialDegree; 
%-----
subBlockSize = YXZsubBlockSizeInMicrons ./ voxelSizeInMicrons;


numSequences = size(channelSubtractionSequence,1);
for sequenceIter = 1:numSequences,
    %pull out the strings for the files to be analyzed in this sequence
    mainFileString = [channelSubtractionSequence{sequenceIter,1},'-*'];
    bleedFileString = [channelSubtractionSequence{sequenceIter,2},'-*'];
    outputFileString = [channelSubtractionSequence{sequenceIter,3},'-*'];
    bleedMaskTopFraction = bleedMaskTopFractionList(sequenceIter);
    maxSubtractionFactor = maxSubtractionFactorList(sequenceIter);
    disp(['Channel Subtracting ',bleedFileString,' from ',mainFileString, ' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
    
    %Determine the subtraction factor for this sequence
    subFactorParameters.processedDataFolder = processedDataFolder;
    subFactorParameters.mainFileString = mainFileString;
    subFactorParameters.bleedFileString = bleedFileString;
    subFactorParameters.bleedMaskTopFraction = bleedMaskTopFraction;
    subFactorParameters.maxSubtractionFactor = maxSubtractionFactor;
    subFactorParameters.numFactors = numFactors;
    subFactorParameters.skipFirstNFiles = skipFirstNFiles;
    subFactorParameters.skipLastNFiles = skipLastNFiles;
    subFactorParameters.subBlockSize = subBlockSize;
    subFactorParameters.startFrameFraction = startFrameFraction;
    subFactorParameters.numFramesToProject = numFramesToProject;
    subFactorParameters.fitPolynomialDegree = fitPolynomialDegree;
        subFactorParameters = VidaGetChannelSubFactor(subFactorParameters);
    %-----
    medianSubtractionFactor = subFactorParameters.medianSubtractionFactor;
    subtractionFactorList{sequenceIter} = medianSubtractionFactor; 
    
    %Perform the Channel Subtraction for this sequence
    PerformChannelSubParameters.processedDataFolder = processedDataFolder;
    PerformChannelSubParameters.mainFileString = mainFileString;
    PerformChannelSubParameters.bleedFileString = bleedFileString;
    PerformChannelSubParameters.outputFileString = outputFileString;
    PerformChannelSubParameters.medianSubtractionFactor = medianSubtractionFactor;
        PerformChannelSubParameters = VidaPerformChannelSub(PerformChannelSubParameters);
    %-----
    outputFileNameList{sequenceIter} = PerformChannelSubParameters.outputFileNameList;
  
end%for sequenceIter = 1:numSequences,
%----

outExp = inExp;
outExp.channelSubParameters.subtractionFactorList = subtractionFactorList;
outExp.channelSubParameters.outputFileNameList = outputFileNameList;
outExp.completionDates.channelSubCompletionDate = datestr(now); 
return


