function outputParameters = VidaPerformChannelSub(inputParameters)
%this function performs the channel subtraction with the provided channels
%and the provided subtraction factor.

processedDataFolder = inputParameters.processedDataFolder;
mainFileString = inputParameters.mainFileString;
bleedFileString = inputParameters.bleedFileString ;
outputFileString = inputParameters.outputFileString;
medianSubtractionFactor = inputParameters.medianSubtractionFactor;

mainFileInfo = dir(fullfile(processedDataFolder,mainFileString));
bleedFileInfo = dir(fullfile(processedDataFolder,bleedFileString));
totalNumFiles = size(mainFileInfo,1);

for fileIter = 1:totalNumFiles,
    disp(['Subtracting File # ',num2str(fileIter),' of ',num2str(totalNumFiles),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
    mainFileName = fullfile(processedDataFolder,mainFileInfo(fileIter).name);
    bleedFileName= fullfile(processedDataFolder,bleedFileInfo(fileIter).name);

    %Generate an output file name
    mainRoot = mainFileString(find(mainFileString~='*'));
    outputRoot = outputFileString(find(outputFileString~='*'));
    startRootPos = strfind(mainFileName,mainRoot);
    stopRootPos = startRootPos+length(mainRoot)-1;
    filePrefix = mainFileName(1:startRootPos-1);
    fileSuffix = mainFileName(stopRootPos+1:end);
    outputFileName = [filePrefix,outputRoot,fileSuffix];
    outputFileNameList{fileIter} = outputFileName;
    
    mainFileImfInfo = imfinfo(mainFileName);
    bleedFileImfInfo = imfinfo(bleedFileName);
    errorFlag = 0;
    if length(mainFileImfInfo)~=length(bleedFileImfInfo), errorFlag = 1; end
    if mainFileImfInfo(1).Width~=bleedFileImfInfo(1).Width, errorFlag = 1; end
    if mainFileImfInfo(1).Height~=bleedFileImfInfo(1).Height, errorFlag = 1; end

    if errorFlag == 1,
        disp('Error in Channel Subtraction Files');
        keyboard
    end

    mainFrame = imread(mainFileName,fileIter);
    bleedFrame = imread(bleedFileName,fileIter);
    corrFrame = double(mainFrame)-(double(bleedFrame)*medianSubtractionFactor);
    imwrite(uint16(corrFrame),outputFileName,'tiff','WriteMode','overwrite','Compression','none');
    for frameIter = 2:length(mainFileImfInfo);
        mainFrame = imread(mainFileName,frameIter);
        bleedFrame = imread(bleedFileName,frameIter);
        corrFrame = double(mainFrame)-(double(bleedFrame)*medianSubtractionFactor);
        imwrite(uint16(corrFrame),outputFileName,'tiff','WriteMode','append','Compression','none');
    end% for frameIter = 2:length(mainFileImfInfo);
    
end%for fileIter = 1:totalNumFiles,

outputParameters = inputParameters;
outputParameters.outputFileNameList = outputFileNameList;
outExp.completionDates.channelSubCompletionDate = datestr(now);
return


   