function [GapConnectingMask,GapConfidence,EPMask] = ChooseGapConnectingMask(E,R,B,BacktrackMask,EPMask,pt1,pt2,Thresholds,x,y,z,minvvol,morphfiltsz)

%Copyright 2008 J.Kaufhold

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.

dispon=0;

% get lattice point locations in local coordinate systems (may be different from original pt locations)
lpt1=round([pt1(1)-min(y(:))+1,pt1(2)-min(x(:))+1,pt1(3)-min(z(:))+1]);
lpt2=round([pt2(1)-min(y(:))+1,pt2(2)-min(x(:))+1,pt2(3)-min(z(:))+1]);

% relax backtracking mask to account for morphological filtering in vessel cascade (which can make points on the cusp of
% the mask lose their membership in the connecting mask)
BacktrackMask = imdilate(BacktrackMask,ones(7,7,7));

%% Get mask that connects both endpoints by stepwise reducing threshold
% initialize gap as unconnected
connected=false;
% initialize threshold at highest value
TIND=length(Thresholds.Enhanced);
% initialize threshold index for unconnected and connected at TIND
uconnTIND=TIND;
connTIND=TIND;
while ~connected
    % Start at highest threshold and relax until lpt1 and lpt2 are connected
    L1=ThresholdEnhancedVesselMasks(E,Thresholds.Enhanced(TIND),minvvol,morphfiltsz);
    L1=bwlabeln(L1 & BacktrackMask,6);
    % if points have the same binary label, and label isnt 0... They are connected at that threshold
    L1conn = (L1(lpt2(1),lpt2(2),lpt2(3))==L1(lpt1(1),lpt1(2),lpt1(3))) && (L1(lpt1(1),lpt1(2),lpt1(3))~=0);
    if L1conn
        connected=true; % if the 2 points share the same contiguous label in L0, mark them connected
        connTIND=TIND;
    else
        uconnTIND=TIND;
    end
    if TIND>=2
        TIND=TIND-1; % lower threshold
    else
        disp('threshold lowered with no connection made');
        disp('handle exception');
        keyboard;
    end
    if dispon
        figure;clf;hold on;
        patch(isosurface(x,y,z,BacktrackMask, .5), 'FaceColor', 'yellow', 'EdgeColor', 'none','facealpha',.1);
        patch(isosurface(x,y,z,L1, .5), 'FaceColor', 'black', 'EdgeColor', 'none','facealpha',.1);
        plot3(pt1(2),pt1(1),pt1(3),'ro');
        plot3(pt2(2),pt2(1),pt2(3),'mo');
        title('connected mask (black)');
    end
end

%% REFINE THRESHOLD TO A TIGHTER VOLUME VIA 10 STEPS OF BINARY SEARCH
UThresh=[Thresholds.Enhanced(uconnTIND),Thresholds.Raw(uconnTIND)];
LThresh=[Thresholds.Enhanced(connTIND),Thresholds.Raw(connTIND)];
nBINARYSEARCHROUNDS=5;
if connected
    for iBS=1:nBINARYSEARCHROUNDS
        EMidThresh=(UThresh+LThresh)/2; % choose threshold between L and U
        LMid=ThresholdEnhancedVesselMasks(E,EMidThresh(1),minvvol,morphfiltsz);
        LMid=bwlabeln(LMid & BacktrackMask,6);
        Mconn=(LMid(lpt2(1),lpt2(2),lpt2(3))==LMid(lpt1(1),lpt1(2),lpt1(3)))&&(LMid(lpt1(1),lpt1(2),lpt1(3))~=0);
        % if points have the same binary label, and label isnt 0... They are connected at that threshold
        if Mconn
            %disp('raising low thresh');
            LThresh=EMidThresh; % if the 2 points share the same contiguous label in LMid, EMidThresh becomes LThresh
        else
            %disp('relaxing high thresh');
            UThresh=EMidThresh; % if the 2 points share the same contiguous label in LMid, EMidThresh becomes LThresh
        end
    end
else
    error('Couldnt bridge gap... Exiting');
end
TightThresh=LThresh;

% Get raw gap-connecting mask, RGCM
RGCM=ThresholdEnhancedVesselMasks(E,TightThresh(1),minvvol,morphfiltsz);

RGCM=((RGCM==RGCM(lpt1(1),lpt1(2),lpt1(3))) | (RGCM==RGCM(lpt2(1),lpt2(2),lpt2(3)))) & (RGCM~=0);

%% Trim gap connecting mask
LTGCM = bwlabeln(RGCM,6);
LTGCM = BacktrackMask & ((LTGCM==LTGCM(lpt1(1),lpt1(2),lpt1(3))) & (LTGCM==LTGCM(lpt2(1),lpt2(2),lpt2(3)))) & (LTGCM~=0);
LTGCM = bwlabeln(LTGCM,6); 
GapConnectingMask = ((LTGCM==LTGCM(lpt1(1),lpt1(2),lpt1(3))) & (LTGCM==LTGCM(lpt2(1),lpt2(2),lpt2(3)))) & (LTGCM~=0);

% % Old confidence value was just an integer...
polyfitINDs = Thresholds.Enhanced>1;
P1 = polyfit(log(Thresholds.Enhanced(polyfitINDs)),Thresholds.zrange(polyfitINDs),1);
if dispon
    figure;plot(Thresholds.zrange(polyfitINDs),log(Thresholds.Enhanced(polyfitINDs)),'r.');
end

if TightThresh(1) > 0
    GapConfidence = polyval(P1,log(TightThresh(1)));
else
    GapConfidence = -inf;
end


if dispon
    figure;clf;hold on;
    patch(isosurface(x,y,z,EPMask, .5), 'FaceColor', 'magenta', 'EdgeColor', 'none','facealpha',.1);
    patch(isosurface(x,y,z,RGCM, .5), 'FaceColor', 'black', 'EdgeColor', 'none','facealpha',.1);
    plot3(pt1(2),pt1(1),pt1(3),'ro');
    plot3(pt2(2),pt2(1),pt2(3),'ro');
    title('gap connecting mask (black)');

    figure;clf;hold on;
    patch(isosurface(x,y,z,B, .5), 'FaceColor', 'blue', 'EdgeColor', 'none','facealpha',.1);
    patch(isosurface(x,y,z,BacktrackMask, .5), 'FaceColor', 'yellow', 'EdgeColor', 'none','facealpha',.1);
    patch(isosurface(x,y,z,GapConnectingMask, .5), 'FaceColor', 'black', 'EdgeColor', 'none','facealpha',.1);
    plot3(pt1(2),pt1(1),pt1(3),'ro');
    plot3(pt2(2),pt2(1),pt2(3),'ro');
    title('gap connecting mask (black)');
end
