function [Vertices,Strands] = ConnectAndReindexBridgingStrands(Vertices,Strands,BridgingStrands)

%% Make sure all vertices have free ends appropriately labeled
Vertices = UpdateVerticesByStrands(Vertices.AllVerts,Strands);
%% Given: 3D bridging strands
BridgingStrands = BridgingStrands(~cellfun(@isempty,BridgingStrands)); % remove empty bridging strands
BridgingStrands = EliminateOppositeStrands(BridgingStrands);           % remove antisense strands with same endpoints
%% SHOW ALL STRANDS CONSTRUCTED FROM VERTS
for iBRIDGE = 1:numel(BridgingStrands)
    disp(['reindexing bridging strand ',num2str(iBRIDGE),' of ',num2str(numel(BridgingStrands))]);
    %% Get closest vertex indices to Bridging strand endpoints...
    [VS,VE] = GetClosestVSVEIndices(BridgingStrands{iBRIDGE},Vertices,Strands);
    %% Get Connection Type Based on VS (start vertex index) and VE (end vertex index) of a Connection
    ConnectionType = GetBridgeConnectionType(Vertices,VS,VE);
    switch lower(ConnectionType)
        case {'endpointtoendpoint'}
            [Strands,Vertices] = ConnectEndPointToEndpoint(Strands,Vertices,VS,VE,BridgingStrands{iBRIDGE});
        case {'endpointtomidpoint'}
            [Strands,Vertices] = ConnectEndPointToMidPoint(Strands,Vertices,VS,VE,BridgingStrands{iBRIDGE});
        case {'endpointtojunctionpoint'}
            [Strands,Vertices] = ConnectEndPointToJunctionPoint(Strands,Vertices,VS,VE,BridgingStrands{iBRIDGE});
        otherwise
            error(['Unknown connection type: ',ConnectionType]);
    end
end
