function [Strands,Vertices] = ConnectEndPointToEndpoint(Strands,Vertices,VS,VE,BridgingStrand)

nstrands = numel(Strands);
N = nstrands+1;

Nverts = size(Vertices.AllVerts);
NNewVerts = size(BridgingStrand,1);
BridgingStrandVertexIndices = Nverts(1)+(1:NNewVerts);

%% UPDATE STRAND INDEXING
% Step 1: Create new strand, N > largest strand index
Strands(N).StartVertexIndex = [];                % starts at index of V2
Strands(N).EndVertexIndex = [];                  % ends at index of V5
Strands(N).StartVertexNeighborStrands = [];      % strands C&D&E touch V2
Strands(N).EndVertexNeighborStrands = [];        % no other strands touch V5
% % % % Strands(N).InteriorVertices = [];                % strands interior (degree 2 chain) vertices ordered closest to Start to End
Strands(N).StartToEndIndices = [];

% Step 2: Get bridging strand's neighbor strand indices
% % VSStrand = Vertices.StrandIndices{VS};
% % VEStrand = Vertices.StrandIndices{VE};
VSStrand = Vertices.StrandIndices(VS,:);
VSStrand = VSStrand(VSStrand~=0);
VEStrand = Vertices.StrandIndices(VE,:);
VEStrand = VEStrand(VEStrand~=0);


% Step 3: Get neighbor strand endpoints opposite VS 
if Strands(VSStrand).StartVertexIndex == VS       % VS is the start vertex
    Strands(N).StartVertexIndex = Strands(VSStrand).EndVertexIndex;
    VSDirection = 'EndToStart';
    % Step 4: Get all strands except the VSStrand neighboring VS
    Strands(N).StartVertexNeighborStrands = Strands(VSStrand).EndVertexNeighborStrands;
else
    Strands(N).StartVertexIndex = Strands(VSStrand).StartVertexIndex;
    VSDirection = 'StartToEnd';
    % Step 4: Get all strands except the VSStrand neighboring VS
    Strands(N).StartVertexNeighborStrands = Strands(VSStrand).StartVertexNeighborStrands;
end

% Step 3: Get neighbor strand endpoints opposite VE
if Strands(VEStrand).StartVertexIndex == VE       % VE is the start vertex
    Strands(N).EndVertexIndex = Strands(VEStrand).EndVertexIndex;
    VEDirection = 'EndToStart';
    % Step 4: Get all strands except the VEStrand neighboring VE
    Strands(N).EndVertexNeighborStrands = Strands(VEStrand).EndVertexNeighborStrands;
else
    Strands(N).EndVertexIndex = Strands(VEStrand).StartVertexIndex;
    VEDirection = 'StartToEnd';
    % Step 4: Get all strands except the VEStrand neighboring VE
    Strands(N).EndVertexNeighborStrands = Strands(VEStrand).StartVertexNeighborStrands;
end

% Step5: Infer centerline point order from Strand(VS) to Strand(VE)
if strcmp(VSDirection,'StartToEnd')
    VSorder = Strands(VSStrand).StartToEndIndices;

else
    VSorder = fliplr(Strands(VSStrand).StartToEndIndices);

end

if strcmp(VEDirection,'StartToEnd')
    VEorder = fliplr(Strands(VEStrand).StartToEndIndices);

else
    VEorder = Strands(VEStrand).StartToEndIndices;

end

Strands(N).StartToEndIndices = [VSorder,BridgingStrandVertexIndices,VEorder];
% % Strands(N).InteriorVertices = Strands(N).StartToEndIndices(2:end-1);
% % stei = [VSorder,BridgingStrandVertexIndices,VEorder];
% % Strands(N).InteriorVertices = stei(2:end-1);

% Step 6: Replace links to VSStrand in VS neighbor strands with links to N
for iNSTRAND = Strands(N).StartVertexNeighborStrands
    stmp = Strands(iNSTRAND).StartVertexNeighborStrands;
    stmp(stmp == VSStrand) = N;
    Strands(iNSTRAND).StartVertexNeighborStrands = stmp;

    stmp = Strands(iNSTRAND).EndVertexNeighborStrands;
    stmp(stmp == VSStrand) = N;
    Strands(iNSTRAND).EndVertexNeighborStrands = stmp;
end

% Step 7: Replace links to VEStrand in VE neighbor strands with links to N
for iNSTRAND = Strands(N).EndVertexNeighborStrands
    stmp = Strands(iNSTRAND).StartVertexNeighborStrands;
    stmp(stmp == VEStrand) = N;
    Strands(iNSTRAND).StartVertexNeighborStrands = stmp;

    stmp = Strands(iNSTRAND).EndVertexNeighborStrands;
    stmp(stmp == VEStrand) = N;
    Strands(iNSTRAND).EndVertexNeighborStrands = stmp;
end

% Step 8: Empty strand definitions for strands VSStrand and VEStrand
% (BUT DO NOT REMOVE THEM FROM THE LIST WITHOUT CORRECTLY REINDEXING
%  ...THIS SHOULD EVENTUALLY BE DONE, BUT NOT CRITICAL)
Strands(VSStrand).StartVertexIndex = [];
Strands(VSStrand).EndVertexIndex = [];
Strands(VSStrand).StartVertexNeighborStrands = [];
Strands(VSStrand).EndVertexNeighborStrands = [];
% % % % Strands(VSStrand).InteriorVertices = [];
Strands(VSStrand).StartToEndIndices = [];

Strands(VEStrand).StartVertexIndex = [];
Strands(VEStrand).EndVertexIndex = [];
Strands(VEStrand).StartVertexNeighborStrands = [];
Strands(VEStrand).EndVertexNeighborStrands = [];
% % % % Strands(VEStrand).InteriorVertices = [];
Strands(VEStrand).StartToEndIndices = [];

% Step 9: Update vertex strand memberships
Vertices = UpdateVerticesByStrands([Vertices.AllVerts;BridgingStrand],Strands);

Strands(N).VOIkeepstrands = true;
