function [Strands,Vertices] = ConnectEndPointToJunctionPoint(Strands,Vertices,VS,VE,BridgingStrand)

nstrands = numel(Strands);
N = nstrands+1;

Nverts = size(Vertices.AllVerts);
NNewVerts = size(BridgingStrand,1);
BridgingStrandVertexIndices = Nverts(1)+(1:NNewVerts);

%% UPDATE STRAND INDEXING
% Step 1: Create new strand, N > largest strand index
Strands(N).StartVertexIndex = [];                % starts at index of V2
Strands(N).EndVertexIndex = [];                  % ends at index of V5
Strands(N).StartVertexNeighborStrands = [];      % strands C&D&E touch V2
Strands(N).EndVertexNeighborStrands = [];        % no other strands touch V5
% % Strands(N).InteriorVertices = [];                % strands interior (degree 2 chain) vertices ordered closest to Start to End
Strands(N).StartToEndIndices = [];

% Step 2: Get bridging strand's neighbor strand indices
% ENFORCE THAT VS IS ALWAYS THE DEGREE 1 VERTEX (i.e. the free endpoint)
if Vertices.FreeEndPoints(VE) & Vertices.JunctionPoints(VS) % VE is degree 1 and VS is degree 3 or higher
    % Swap VS and VE
    VStmp = VE;
    VE = VS;
    VS = VStmp;
    BridgingStrandVertexIndices = fliplr(BridgingStrandVertexIndices);
else
    if ~(Vertices.FreeEndPoints(VS) & Vertices.JunctionPoints(VE)) % NOT ( VS is degree 1 and VE is degree 3 or higher )
        keyboard
        error('VS and VE do not compose a FreeEndPoint/JunctionPoint pair');
    end
end
% Step 2: Get bridging strand's neighbor strand indices
% % VSStrand = Vertices.StrandIndices{VS};
% % VEStrand = Vertices.StrandIndices{VE};
VSStrand = Vertices.StrandIndices(VS,:);
VSStrand = VSStrand(VSStrand~=0);
VEStrand = Vertices.StrandIndices(VE,:);
VEStrand = VEStrand(VEStrand~=0);

% Step 3: Get neighbor strand endpoints opposite VS 
if Strands(VSStrand).StartVertexIndex == VS       % VS is the start vertex
    Strands(N).StartVertexIndex = Strands(VSStrand).EndVertexIndex;
    VSDirection = 'EndToStart';
    % Step 4: Get all strands except the VSStrand neighboring VS
    Strands(N).StartVertexNeighborStrands = Strands(VSStrand).EndVertexNeighborStrands;
else
    Strands(N).StartVertexIndex = Strands(VSStrand).StartVertexIndex;
    VSDirection = 'StartToEnd';
    % Step 4: Get all strands except the VSStrand neighboring VS
    Strands(N).StartVertexNeighborStrands = Strands(VSStrand).StartVertexNeighborStrands;
end

% Step5: Infer centerline point order from Strand(VS) to VE
if strcmp(VSDirection,'StartToEnd')
    VSorder = Strands(VSStrand).StartToEndIndices;

else
    VSorder = fliplr(Strands(VSStrand).StartToEndIndices);

end

Strands(N).StartToEndIndices = [VSorder,BridgingStrandVertexIndices,VE];
% % Strands(N).InteriorVertices = Strands(N).StartToEndIndices(2:end-1);
% % stei = [VSorder,BridgingStrandVertexIndices,VE];
% % Strands(N).InteriorVertices = stei(2:end-1);
Strands(N).EndVertexIndex = VE;

% Step : Get all strands neighboring VE
% Strands(N).EndVertexNeighborStrands = Vertices.StrandIndices{VE};
tempStrands = Vertices.StrandIndices(VE,:);
tempStrands = tempStrands(tempStrands~=0);
Strands(N).EndVertexNeighborStrands = tempStrands;

% Step 6: Replace links to VSStrand in VS neighbor strands with links to N
for iNSTRAND = Strands(N).StartVertexNeighborStrands
    stmp = Strands(iNSTRAND).StartVertexNeighborStrands;
    stmp(stmp == VSStrand) = N;
    Strands(iNSTRAND).StartVertexNeighborStrands = stmp;

    stmp = Strands(iNSTRAND).EndVertexNeighborStrands;
    stmp(stmp == VSStrand) = N;
    Strands(iNSTRAND).EndVertexNeighborStrands = stmp;
end

% Step 7: Add links to N to VE neighbor strands (take into account start/end sense)
for iNSTRAND = VEStrand     % For each index in VEStrand
    if Strands(iNSTRAND).StartVertexIndex == VE   % if VE == Start Point
        % Add N to the list of StartVertexNeighborStrands
        Strands(iNSTRAND).StartVertexNeighborStrands = [Strands(iNSTRAND).StartVertexNeighborStrands,N];
    else
        % Add N to the list of EndVertexNeighborStrands
        Strands(iNSTRAND).EndVertexNeighborStrands = [Strands(iNSTRAND).EndVertexNeighborStrands,N];
    end
end

% Step 8: Empty strand definitions for strand VSStrand
% (BUT DO NOT REMOVE THEM FROM THE LIST WITHOUT CORRECTLY REINDEXING
%  ...THIS SHOULD EVENTUALLY BE DONE, BUT NOT CRITICAL)
Strands(VSStrand).StartVertexIndex = [];
Strands(VSStrand).EndVertexIndex = [];
Strands(VSStrand).StartVertexNeighborStrands = [];
Strands(VSStrand).EndVertexNeighborStrands = [];
% % % % Strands(VSStrand).InteriorVertices = [];
Strands(VSStrand).StartToEndIndices = [];

% Step 9: Update vertex strand memberships
Vertices = UpdateVerticesByStrands([Vertices.AllVerts;BridgingStrand],Strands);

Strands(N).VOIkeepstrands = true;
