function [Strands,Vertices] = ConnectEndPointToMidPoint(Strands,Vertices,VS,VE,BridgingStrand)

nstrands = numel(Strands);
A = nstrands+1;
B = A+1;

Nverts = size(Vertices.AllVerts);
NNewVerts = size(BridgingStrand,1);
BridgingStrandVertexIndices = Nverts(1)+(1:NNewVerts);

% Step 1: Make sure VS is degree 1 and VE is degree 2
% ENFORCE THAT VS IS ALWAYS THE DEGREE 1 VERTEX (i.e. the free endpoint)
if Vertices.FreeEndPoints(VE) & ...                                  % VE is degree 1 and
        (~Vertices.JunctionPoints(VS) & ~Vertices.FreeEndPoints(VS)) % VS is not degree 3 or higher or degree 1
    % Swap VS and VE
    VStmp = VE;
    VE = VS;
    VS = VStmp;
    BridgingStrandVertexIndices = fliplr(BridgingStrandVertexIndices);
else
    if ~(Vertices.FreeEndPoints(VS) & ... % NOT ( VS is degree 1 and VE is degree 2 )
            (~Vertices.JunctionPoints(VE) & ~Vertices.FreeEndPoints(VE))) 
        disp('VS and VE do not compose a FreeEndPoint/MidPoint pair');
% %         if isempty(setdiff(Vertices.StrandIndices{VE},Vertices.StrandIndices{VS}))
        if isempty(setdiff(Vertices.StrandIndices(VE,:),Vertices.StrandIndices(VS,:)))
            disp('but VS and VE are already on the same strand... skipping');
            return;
        else
            disp('and VS and VE are not already on the same strand... fix...');
            keyboard;
        end
    end
end
% Step 2: Get bridging strand's neighbor strand indices
tempStrand = Vertices.StrandIndices(VE,:);
tempStrand = tempStrand(tempStrand~=0);
VEStrand = unique(tempStrand);
if numel(VEStrand)>1
    disp('too many endpoint strands')
    keyboard;
end

%% UPDATE STRAND INDEXING
% Step 3: Create 2 new strands, A & B
% Create new strand, A > largest strand index
%  Strand A's StartVertexIndex is the start vertex for VEStrand
try
    Strands(A).StartVertexIndex = Strands(VEStrand).StartVertexIndex;                
catch
    disp('problem in ConnectEndPointToMidPoint');
    keyboard;
end

%  Strand A's EndVertexIndex is VE
Strands(A).EndVertexIndex = VE;                  
%  Strand A's StartVertexNeighborStrands are the StartVertexNeighborStrands for VEStrand
Strands(A).StartVertexNeighborStrands = Strands(VEStrand).StartVertexNeighborStrands;      
%  Strand A's EndVertexNeighborStrands is StrandB
Strands(A).EndVertexNeighborStrands = B;        
%  Strand A's StartToEndVertices are the StartToEndVertices up to and including VE
Strands(A).StartToEndIndices = Strands(VEStrand).StartToEndIndices(1:find(Strands(VEStrand).StartToEndIndices==VE));
% % Strands(A).InteriorVertices = Strands(A).StartToEndIndices(2:end-1);
% % Strands(A).InteriorVertices = Strands(A).StartToEndIndices(2:end-1);

% Create new strand, B > A
%  Strand B's StartVertexIndex is VE
Strands(B).StartVertexIndex = VE;                
%  Strand B's EndVertexIndex is the end vertex for VEStrand
Strands(B).EndVertexIndex = Strands(VEStrand).EndVertexIndex;                  
%  Strand B's StartVertexNeighborStrands is StrandA
Strands(B).StartVertexNeighborStrands = A;      
%  Strand B's EndVertexNeighborStrands are the EndVertexNeighborStrands for VEStrand
Strands(B).EndVertexNeighborStrands = Strands(VEStrand).EndVertexNeighborStrands;        
%  Strand B's StartToEndVertices are the StartToEndVertices starting with VE up to the end
Strands(B).StartToEndIndices = Strands(VEStrand).StartToEndIndices(find(Strands(VEStrand).StartToEndIndices==VE):end);
% % Strands(B).InteriorVertices = Strands(B).StartToEndIndices(2:end-1);                

% % Strands(B).InteriorVertices = Strands(B).StartToEndIndices(2:end-1);


% Step : Empty strand definitions for strand VSStrand
% (BUT DO NOT REMOVE THEM FROM THE LIST WITHOUT CORRECTLY REINDEXING
%  ...THIS SHOULD EVENTUALLY BE DONE, BUT NOT CRITICAL)
Strands(VEStrand).StartVertexIndex = [];
Strands(VEStrand).EndVertexIndex = [];
Strands(VEStrand).StartVertexNeighborStrands = [];
Strands(VEStrand).EndVertexNeighborStrands = [];
% % Strands(VEStrand).InteriorVertices = [];
Strands(VEStrand).StartToEndIndices = [];

% Step : Replace links to VEStrand in A's StartVertexNeighborStrands with links to A
for iNSTRAND = Strands(A).StartVertexNeighborStrands
    stmp = Strands(iNSTRAND).StartVertexNeighborStrands;
    stmp(stmp == VEStrand) = A;
    Strands(iNSTRAND).StartVertexNeighborStrands = stmp;

    stmp = Strands(iNSTRAND).EndVertexNeighborStrands;
    stmp(stmp == VEStrand) = A;
    Strands(iNSTRAND).EndVertexNeighborStrands = stmp;
end

% Step : Replace links to VEStrand in B's EndVertexNeighborStrands with links to B
for iNSTRAND = Strands(B).EndVertexNeighborStrands
    stmp = Strands(iNSTRAND).StartVertexNeighborStrands;
    stmp(stmp == VEStrand) = B;
    Strands(iNSTRAND).StartVertexNeighborStrands = stmp;

    stmp = Strands(iNSTRAND).EndVertexNeighborStrands;
    stmp(stmp == VEStrand) = B;
    Strands(iNSTRAND).EndVertexNeighborStrands = stmp;
end

% Step : Update vertex strand memberships
Vertices = UpdateVerticesByStrands(Vertices.AllVerts,Strands);

% "FAKE junction" VE in Vertices should be marked as a junction
Vertices.JunctionPoints(VE) = 1;

% Set strands to "on" for testing
Strands(A).VOIkeepstrands = true;
Strands(B).VOIkeepstrands = true;

% Use ConnectEndPointToJunctionPoint to connect VS to VE
[Strands,Vertices] = ConnectEndPointToJunctionPoint(Strands,Vertices,VS,VE,BridgingStrand);
