function BridgingStrands = EliminateOppositeStrands(BridgingStrands)

% This function finds pairs of BridgingStrands that are the same except for
% their direction (one of the pair goes from VS to VE and the other goes
% from VE to VS). It leaves only one unique BridgingStrand per pair.

% use a hash function on vertex pair coordinates to find repeated start and end vertices
maxBridgeDim = 1;
for iBRIDGE = 1:numel(BridgingStrands)
    maxBridgeDim = max(maxBridgeDim,size(BridgingStrands{iBRIDGE},2));
end
hashvec = zeros(numel(BridgingStrands),2 * maxBridgeDim);
for iBRIDGE = 1:numel(BridgingStrands)
    try
        hashvec(iBRIDGE,:) = [BridgingStrands{iBRIDGE}(1,:),BridgingStrands{iBRIDGE}(end,:)];
    catch
        keyboard;
    end
end

% find the vertices in the start list that also appear in the end list
[RepeatPoints,h1,h2] = intersect(hashvec(:,1:3),hashvec(:,4:6),'rows');
% keep all the BridgingStrand start and end vertex pairs that are not repeated
uniquepairs = setdiff(1:numel(BridgingStrands),h1);
uniquepairs = uniquepairs(:);

% assuming all repeated pairs occur exactly 2x, this keeps only the
% unique pairs
repeatedpairs = sortrows(sort([h1,h2]',1)');
halfrepeatedpairs = unique(repeatedpairs,'rows');
halfrepeatedpairs = halfrepeatedpairs(:,1);

% This eliminates all the non-unique bridging strands
BridgingStrands = BridgingStrands(unique([uniquepairs;halfrepeatedpairs]));
