function [AM,cxyz] = GetCenterlinePointsAndAdjacency(centerlinePts,maskSize)

%GETCENTERLINEPOINTSANDADJACENCY Get the adjacency matrix and centerline points of a 26 connected binary mask of centerlines.
%   [AM,cxyz] = GetCenterlinePointsAndAdjacency(CenterlineMask)
%   Computes the adjacency matrix, AM, and centerline locations in 3D of a
%   3D 26-connected binary mask input of centerlines.
%
%   The output, AM, is a sparse matrix of 1s where a centerline point index
%   is connected to another centerline point index.
%   
%   See also SPY, TREEPLOT.

%   John Kaufhold

% count the total number of points
nCpts = numel(centerlinePts);
% Create a block of centerline point indices for adjacency matrix computation
fprintf(1,'Building array of sparse indexing matrices...\n');
[cxyz,sparseIndexBlock] = BuildSparseIndexBlock(centerlinePts,maskSize);
% Create a cell array of pairs of connected indices
pairsAM1 = cell(nCpts,1);
pairsAM2 = cell(nCpts,1);
cptNbhood = zeros(3,3,3);
fprintf(1,'Building sparse adjacency matrix...\n');

for iCPT = 1:nCpts
    cptNbhood(1:3,1:3,1) = full(sparseIndexBlock{cxyz(iCPT,3)-1}(cxyz(iCPT,1)+(-1:1),cxyz(iCPT,2)+(-1:1)));
    cptNbhood(1:3,1:3,2) = full(sparseIndexBlock{cxyz(iCPT,3)}(cxyz(iCPT,1)+(-1:1),cxyz(iCPT,2)+(-1:1)));
    cptNbhood(1:3,1:3,3) = full(sparseIndexBlock{cxyz(iCPT,3)+1}(cxyz(iCPT,1)+(-1:1),cxyz(iCPT,2)+(-1:1)));
    cPttmp = cptNbhood(14);
    neighborPts = [cptNbhood(1:13),cptNbhood(15:27)];
    neighborPts = neighborPts(neighborPts>0);
    pairstmp1 = neighborPts(:)';
    pairstmp2 = repmat(cPttmp,1,numel(neighborPts(:)));
    pairsAM1{iCPT} = pairstmp1;
    pairsAM2{iCPT} = pairstmp2;
end
% create sparse adjacency matrix, AM
sparseAMFill = double([[pairsAM1{:}]',[pairsAM2{:}]',ones(numel([pairsAM1{:}]),1)]);
AM = sparse(sparseAMFill(:,1),sparseAMFill(:,2),sparseAMFill(:,3));
