function [centerlineStats,L,clm1] = GetCenterlineStats(centerLineMask)

clmsz = size(centerLineMask);
clm1 = zeros(size(centerLineMask)+2,'uint8');            % preallocate with padding
clm1(2:end-1,2:end-1,2:end-1) = logical(centerLineMask); % place mask inside padding
clm2 = uint8(convn(clm1,ones(3,3,3),'same'));            % convolve with 3x3x3 ones to find junctions
clms = clm2(2:end-1,2:end-1,2:end-1);                    % trim back to remove padding
clms = clms .* uint8(centerLineMask);                    % restrict graph counts to centerline locations

singlevoxelislands = (clms == 1);                % this counts the number of endpoints that are isolated islands (should be 0)
connectedendpoints = (clms == 2);                % this counts the number of endpoints that are connected to a strand
lengthpoints = (clms == 3);                      % this counts the number of "regular" centerline points (clms ==3)
junctionpoints = (clms >= 4);                    % this counts the number of junction points (clms > 3)
[L,njunctionislands] = bwlabeln(junctionpoints); % this counts the number of islands made up of only adjacent junction points

centerlineStats.nsinglevoxelislands = sum(singlevoxelislands(:));
centerlineStats.nconnectedendpoints = sum(connectedendpoints(:));
centerlineStats.nlengthpoints = sum(lengthpoints(:));
centerlineStats.njunctionpoints = sum(junctionpoints(:));
centerlineStats.njunctionislands = njunctionislands;

fprintf(1,'there are %d nsinglevoxelislands.\n',centerlineStats.nsinglevoxelislands);
fprintf(1,'there are %d nconnectedendpoints.\n',centerlineStats.nconnectedendpoints);
fprintf(1,'there are %d nlengthpoints.\n',centerlineStats.nlengthpoints);
fprintf(1,'there are %d njunctionpoints.\n',centerlineStats.njunctionpoints);
fprintf(1,'there are %d njunctionislands.\n',centerlineStats.njunctionislands);
