function [CenterlineStrands, strandIDi] = GetCenterlinesAndStrandStruct(AM)

disp('Strandifying...');
% define the number of centerline points
nCpts = size(AM,1);
% keep track of which centerline points have been indexed already
nodeDegrees = sum(AM);
% compute indices of endpts, connectors, and junctions...
endptsi=find(nodeDegrees==1);
juncti=find(nodeDegrees>2);
% LUT for each node is as follows: 1=endpt, 2=conni, 3=junct
degLUT = full(min(nodeDegrees,3));

% initialize list of visited nodes to all unvisited (0s)
visitedi=zeros(nCpts,1);
% initialize list strand IDs to unlabeled (0s)
strandIDi=cell(nCpts,1);

% Build adjacent referencing structure
linkstruct(nCpts).links = [];
for iCPT = 1:nCpts
    linkstruct(iCPT).links = find(AM(iCPT,:));          % follow strand to adjacent centerline points
end

strandcells = cell(1,length(endptsi));
nchar = 0;
% first connect all endpts up to terminal junctions
for i=1:length(endptsi)
    if any(i== ceil((.1:.1:.9)*length(endptsi)))
        fprintf(repmat('\b',1,nchar))
        nchar = fprintf(1,'Strandifying endpoint vertex %d of %d\n',i,length(endptsi));
    end
    
    cpt=endptsi(i);                    % get the current point index
    if visitedi(cpt)==0                % If current endpoint is unvisited...
        strandIDi{cpt}=i;                % label the ID for the ith strand's beginning node
        visitedi(cpt)=1;                 % mark endpoint node as visited

        linki = linkstruct(cpt).links; % follow strand
        linkstruct(cpt).links = setdiff(linkstruct(cpt).links,linki);   % remove old links
        linkstruct(linki).links = setdiff(linkstruct(linki).links,cpt); % remove old links
        strandcells{i}=cpt;
        while ~isempty(linki)
            cpt=linki;                     % get new cpt (linki)
            if length(cpt)>1; disp('ERROR: cpt has more than one index');keyboard; end
            strandcells{i}=[strandcells{i},cpt];
            visitedi(cpt)=1+visitedi(cpt); % mark endpoint node as visited
            if degLUT(linki)==2            % check to see that linki is a connector (degree=2)
                % disp('connector')
                strandIDi{cpt}=i;            % label it and mark it
                linki = linkstruct(cpt).links; % follow strand
                linkstruct(cpt).links = setdiff(linkstruct(cpt).links,linki);   % remove old links
                linkstruct(linki).links = setdiff(linkstruct(linki).links,cpt); % remove old links
            else % if degLUT==2
                if degLUT(linki)==1          % if not a connector and (degree=1),
                    % disp('terminus')
                    strandIDi{cpt}=i;          % label it and mark it
                    linki=[];                  % end following
                else  % if degLUT==1
                    if degLUT(linki)==3                 % if not a connector and (degree=3),
                        % disp('junction');
                        strandIDi{cpt}=unique([strandIDi{cpt},i]);% label it and mark it
                        linki=[];                         % end following
                    else % if degLUT==3
                        disp('ERROR: vessel node degree LUT corrupted');keyboard;
                    end % if degLUT==3
                end % if degLUT==1
            end % if degLUT==2
        end % while still more links (linki not empty)
    end % if cpt is unvisited
end % for all endpts

% assume that some single-emanation junctions can be marked...
junctionsmarkable=1;

% follows same general flow as above, only this knows there will be no
% endpoints in its search and it only goes junction to junction. It also
% chooses the first unmarked route out of a junction whereas above, the
% link following only had one direction to go.

fprintf(1,'Strandifying %d junction vertices\n',length(juncti));
while junctionsmarkable
    junctionsmarked=0; % For each high degree node (every node in juncti)
    for i=1:length(juncti)
        nextstrandID=max([strandIDi{:}])+1;
        cpt=juncti(i);                  % get the current point index
        UVneighbors = linkstruct(cpt).links; % UVneighbors is list of all unmarked neighbor nodes
        ntomark=length(UVneighbors);    % compute the number of connected nodes to mark
        if (ntomark>=1)                 % if there's at least one connection out of junction...
            strandcells{nextstrandID}=cpt;% update strandcells
            linki=UVneighbors(1);         % Only take first unmarked path each iteration
            junctionsmarked=junctionsmarked+1;% note that junctionmarking not finished
            linkstruct(cpt).links = setdiff(linkstruct(cpt).links,linki);   % remove old links
            linkstruct(linki).links = setdiff(linkstruct(linki).links,cpt); % remove old links
            strandIDi{cpt}=unique([strandIDi{cpt},nextstrandID]); % label the ID for the ith strand's beginning node
            while ~isempty(linki)
                strandcells{nextstrandID}=[strandcells{nextstrandID},linki];
                if degLUT(linki)==2          % check to see that linki is a connector (degree=2)
                    % disp('connector')
                    cpt=linki;                 % get new cpt (linki)
                    if length(cpt)>1; disp('more than one cpt'); keyboard; end
                    strandIDi{cpt}=nextstrandID;% label it and mark it
                    visitedi(cpt)=1+visitedi(cpt);% mark endpoint node as visited
                    linki = linkstruct(cpt).links; % UVneighbors is list of all unmarked neighbor nodes
                    linkstruct(cpt).links = setdiff(linkstruct(cpt).links,linki);   % remove old links
                    linkstruct(linki).links = setdiff(linkstruct(linki).links,cpt); % remove old links
                else
                    if degLUT(linki)==3       % if not a connector and (degree=3),
                        % disp('junction');
                        cpt=linki;                                    % get new cpt (linki)
                        if length(cpt)>1; disp('more than one cpt'); keyboard; end
                        strandIDi{cpt}=unique([strandIDi{cpt},nextstrandID]); % label it and mark it
                        visitedi(cpt)=1+visitedi(cpt);                % mark endpoint node as visited
                        linki=[];
                    else
                        disp('check vessel node degree');
                        keyboard;
                        error('vessel node degree LUT corrupted');
                    end
                end
            end
        end
        % Are all its neighbors strand IDs in this nodes strandID struct?
    end
    junctionsmarkable=(junctionsmarked>0);
end

% remove any unused labelIDs (needs to reindex, but there are few, so this picks off one at a time)
unusedIDsmissing=1;
while unusedIDsmissing
    unusedstrandlabels=setdiff(1:max([strandIDi{:}]),unique([strandIDi{:}]));
    if ~isempty(unusedstrandlabels) % if there's at least one unlabeled strand, remove one
        unusedIDsmissing=1;
        i=1; % only take off the first missing label each time
        removelabeli=unusedstrandlabels(i);
        tmpcells=[strandcells(1:removelabeli-1),strandcells(removelabeli+1:end)];
        for j=1:length(strandIDi)
            stmp=strandIDi{j};
            % decrement any index over removelabeli
            stmp(find(stmp>removelabeli))=stmp(find(stmp>removelabeli))-1;
            tmpstrandIDi{j}=stmp;
        end
        strandcells=tmpcells;
        strandIDi=tmpstrandIDi;
    else
        unusedIDsmissing=0;
    end
end

nStrands0 = numel(strandcells);
% Indicate unlabeled/unfinished junctions... And make sure all points have as many
% labels as their degree indicates (or correctly count loops)
nodeDegrees_cksum=nodeDegrees;                   % connectors should only have one ID
nodeDegrees_cksum(nodeDegrees_cksum==2)=1;    % Find all remaining unlabeled points
ul = zeros(1,nCpts);
uf = zeros(1,nCpts);
for i=1:numel(strandIDi)               % Check all node ID lists and see that they match their nodeDegrees
    ul(i)=isempty(strandIDi{i});
    uf(i)=(length(unique(strandIDi{i}))~=nodeDegrees_cksum(i));
end
unlabeled=find(ul);
unfinished=find(uf);

% FIND AND ADD ISLANDS TO strandcells
nonislandunlabeled = zeros(size(unlabeled));
islandcount = 0;
for iUL = 1:numel(unlabeled)
    currentUnlabeled = unlabeled(iUL);
    if nodeDegrees(currentUnlabeled)==0        % node is an island
        islandcount = islandcount + 1;
        strandcells{nStrands0+islandcount} = currentUnlabeled;
    else
        nonislandunlabeled(iUL) = 1;
    end
end

%nStrands1 = numel(strandcells);

if islandcount>0
    fprintf(1,'Warning: some nodes were islands\n');
end

if ~isempty(unfinished)
    fprintf(1,'Warning: some nodes dont account for their graph degree\n');
    fprintf(1,'Warning: likely these are self-loops\n');
    disp(['check nodes: ',num2str(unfinished)]);
end

nStrands2 = numel(strandcells);

% All strands at this point have only degree 2 nodes in them (i.e. "rings")
unusedlinks = find(~cellfun(@isempty,{linkstruct(:).links}));
isolatedcyclecount = 0;
while ~isempty(unusedlinks)
    isolatedcyclecount = isolatedcyclecount + 1;
    nextstrandID = nStrands2 + isolatedcyclecount;
    cpt=unusedlinks(1);                  % get the first unused point
    UVneighbors = linkstruct(cpt).links; % UVneighbors is list of all unmarked neighbor nodes
    %ntomark=length(UVneighbors);         % compute the number of connected nodes to mark

    strandcells{nextstrandID}=cpt;% update strandcells
    linki=UVneighbors(1);         % Only take first unmarked path each iteration
    linkstruct(cpt).links = setdiff(linkstruct(cpt).links,linki);   % remove old links
    linkstruct(linki).links = setdiff(linkstruct(linki).links,cpt); % remove old links
    strandIDi{cpt}=unique([strandIDi{cpt},nextstrandID]); % label the ID for the ith strand's beginning node
    while ~isempty(linki)
        strandcells{nextstrandID}=[strandcells{nextstrandID},linki];
        % disp('connector')
        cpt=linki;                 % get new cpt (linki)
        if length(cpt)>1; disp('more than one cpt'); keyboard; end
        strandIDi{cpt}=nextstrandID;% label it and mark it
        visitedi(cpt)=1+visitedi(cpt);% mark endpoint node as visited
        linki = linkstruct(cpt).links; % UVneighbors is list of all unmarked neighbor nodes
        linkstruct(cpt).links = setdiff(linkstruct(cpt).links,linki);   % remove old links
        if ~isempty(linki)
            linkstruct(linki).links = setdiff(linkstruct(linki).links,cpt); % remove old links
        end
    end
    unusedlinks = find(~cellfun(@isempty,{linkstruct(:).links}));
end

%maxStrandID = max([strandIDi{:}]);
%CenterlineStrands = strandcells(1:maxStrandID);
CenterlineStrands = strandcells;

disp('Strandification successful.');
