function [IslandCenterPoints] = GetIslandPointsFromMask(VesselRepresentations)


maskpath = VesselRepresentations.IsotropicVesselMaskMatFname;
MV3Dfname = VesselRepresentations.VectorizedMV3DFilename;
maskSize = VesselRepresentations.maskSize;

%Copyright 2008 J.Kaufhold

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.

clobberislands = false;
%% Load and Parse strand points
strandSet = amira2mat_parseFile(MV3Dfname);
% compensate strand locations for 0-based to 1-based indexing
tmpxyz=strandSet.strandData(:,2:4)+1;

%% Load mask's .tif info

% % % keyboard
% % % RVWVTFinfo = imfinfo(maskpath);
% % % % Get size of .tif
% % % vsz = [RVWVTFinfo(1).Height,RVWVTFinfo(1).Width,length(RVWVTFinfo)];
% % % % Get block indices of .tif
vsz = maskSize;


% THESE OVERLAP PARAMETERS MAY NEED TO FLOW UP TO CALLING ROUTINE...
overlapsz=15;n1=5;n2=5;
[blockranges]=gridupblock(vsz,overlapsz,n1,n2);

% Make a filename for each block of islands
islandblockfname=cell(1,length(blockranges));
for iBR=1:length(blockranges)
    islandblockfname{iBR}=['islandpts',num2str(iBR),'.mat'];
end

%% RESTITCH... REQUIRES MASK RESIDENT IN MEMORY... 
% try
%     UnvecVascIslandsMask = zeros(vsz,'uint8');
% catch
%     disp(['Cannot allocate mask of size ',num2str(vsz),', revert to streaming']);
% end

%% RESTITCH... USES DISK FOR STORAGE (will scale to larger volumes)
islandlistbyblock = cell(1,length(blockranges));
fprintf(1,'Getting islands from block (of %d): ',length(blockranges));
%-----
%Load the full mask here
loadedVariables = load(maskpath);
variableNames = fieldnames(loadedVariables);
maskVariableName = variableNames{1};
fullMask = loadedVariables.(maskVariableName);
clear loadedVariables variableNames maskVariableName;
fullMask = struct2mask(fullMask);
%-----

for iBR=1:length(blockranges)
    fprintf(1,'%d... ',iBR);
    if clobberislands
        % Get each block of raw vasculature
        br = blockranges{iBR};
        maskblk = fullMask(br.x1min:br.x1max,br.x2min:br.x2max,br.x3min:br.x3max);
        % Get island mask for each block
        islandcptmask = GetIslandsFromBlock(maskblk,tmpxyz,blockranges{iBR});
        % save each block of raw vasculature
        save(islandblockfname{iBR},'islandcptmask','-mat');
    else
        try
            load(islandblockfname{iBR},'islandcptmask','-mat');
            fprintf(1,'found islands... loading...\n');
        catch
            % Get each block of raw vasculature
             br = blockranges{iBR};
             maskblk = fullMask(br.x1min:br.x1max,br.x2min:br.x2max,br.x3min:br.x3max);
            % Get island mask for each block
            islandcptmask = GetIslandsFromBlock(maskblk,tmpxyz,blockranges{iBR});
            % save each block of raw vasculature
            save(islandblockfname{iBR},'islandcptmask','-mat');
        end
    end
    % put islands in large island mask
    [x1s,x2s,x3s] = rangeselect(blockranges{iBR});
    % UnvecVascIslandsMask(x1s,x2s,x3s) = max(UnvecVascIslandsMask(x1s,x2s,x3s),islandcptmask);
    [x1tmp,x2tmp,x3tmp] = ind2sub(size(islandcptmask),find(islandcptmask));
    islandlistbyblock{iBR} = [x1s(x1tmp)',x2s(x2tmp)',x3s(x3tmp)'];
end

% Get centerpoint list for all islands
nislandsbyblock = cellfun(@numel,islandlistbyblock)/3;
nislandsbyblock = [[1,1+cumsum(nislandsbyblock(1:end-1))];cumsum(nislandsbyblock)];
IslandCenterPoints = zeros(nislandsbyblock(end),3);
for iBR = 1:numel(blockranges)
    IslandCenterPoints(nislandsbyblock(1,iBR):nislandsbyblock(2,iBR),1:3) = islandlistbyblock{iBR};
end
IslandCenterPoints = unique(IslandCenterPoints,'rows');
