function SAM = GetStrandAdjacencyMatrix(Strands,Vertices)

% Get all vertices of junctions
JunctionVerts = cellfun(@length,Vertices.StrandIndices)>1;
% Get all strand indices incident on all junctions
ConnectedStrands = Vertices.StrandIndices(JunctionVerts);
% Compute the highest connectivity of any junction vertex
MaxStrandConnectivity = max(cellfun(@length,ConnectedStrands));

% Preallocate the LUT for indexing pairs of strands
AdjacencyLUT = cell(1,MaxStrandConnectivity);
for iAL = 1:MaxStrandConnectivity; AdjacencyLUT{iAL} = getAllPairs(iAL); end

% Preallocate the pairs adjacency matrix
AMpairs = cell(1,numel(ConnectedStrands));
for iCS = 1:numel(ConnectedStrands)
    CStmp = ConnectedStrands{iCS};
    ipairs = AdjacencyLUT{numel(CStmp)};
    AMpairs{iCS} = reshape(CStmp(ipairs(:)),size(ipairs));
end

% Create sparse indexing matrix from cell array of strand index pairs
Sij = cell2mat(AMpairs');
% Make it symmetric
SAM = sparse([Sij(:,1);Sij(:,2)],[Sij(:,2);Sij(:,1)],ones(2*size(Sij,1),1));

function indexpairs = getAllPairs(npairs)

indexpairs = zeros((npairs^2 - npairs)/2, 2);
icounter = 1;
for i = 1:npairs - 1
    for j = i+1:npairs
        indexpairs(icounter,1:2) = [i, j];
        icounter = icounter + 1;
    end
end
