function [strandSet connections2add connections2retest] = PairNearestNeighborEndpoints(strandSet,prmts)

%Copyright 2008 J.Kaufhold

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.

%% parse behavior parameters
% boundBoxMinDist = 10;
% boundBoxDimXYZ = max(strandSet.strandData(:,2:4)); %FOR DEV ONLY!!!!!!
% maxGapDist_um = 10; %1pxl = 1um
% minGapDist_um = 0; %just for slicing the range of possible gaps, set to 0 otherwise

dispon = 0;

boundBoxMinDist = prmts.boundBoxMinDist;
boundBoxDimXYZ = prmts.boundBoxDimXYZ; %FOR DEV ONLY!!!!!!
maxGapDist_um = prmts.maxGapDist_um ; %1pxl = 1um
%minGapDist_um = prmts.minGapDist_um; %just for slicing the range of possible gaps, set to 0 otherwise

%% prepare graphic output
% h2Lines = amira2mat_displayStrandSet(strandSet);
% set(h2Lines,'color',[.5 .5 .5],'linewidth',.5)

%% gather unconnected point coordinates and strand identification
unconCoord = strandSet.strandData(strandSet.ptr2uncon,2:4);        % coordinates of interior endpoints
unconStrandId = strandSet.strandData(strandSet.ptr2uncon,1);       % strandIDs of interior endpoints

%% consider only endpoints away from the face of the bounding box
keepBoundBoxDimXYZ = boundBoxDimXYZ - boundBoxMinDist;
keepIdx = find(unconCoord(:,1) > boundBoxMinDist & unconCoord(:,2) >= boundBoxMinDist & unconCoord(:,3) >= boundBoxMinDist & unconCoord(:,2) <= keepBoundBoxDimXYZ(1) & unconCoord(:,1) <= keepBoundBoxDimXYZ(2) & unconCoord(:,3) <= keepBoundBoxDimXYZ(3));

if dispon
    % % show all strands with all endpoints...
    figure(1);clf;
    plot3(-1,-1,-1,'r.');
    hold on;
    plot3([-1,-1.01],[-1,-1.01],[-1 -1],'r-','linewidth',2);
    plot3([-1,-1.01],[-1,-1.01],[-1 -1],'k-','linewidth',2);
    plot3([-1,-1.01],[-1,-1.01],[-1 -1],'b-','linewidth',2);
    plot3(unconCoord(keepIdx,1),unconCoord(keepIdx,2),unconCoord(keepIdx,3),'r.')
    for iEPT=1:length(keepIdx)
        p1=keepIdx(iEPT);
        strandID1=unconStrandId(p1);
        strand1ptsidx = strandSet.strandSetInfo(strandID1).ptr2coord;
        s1xyz=strandSet.strandData(strand1ptsidx,2:4);
        plot3(s1xyz(:,1),s1xyz(:,2),s1xyz(:,3),'r-');
    end
    title('Nearest neighbors');
end

%% Do a brute force pdist. A local approach might be faster....
DistMtx=squareform(pdist(unconCoord(keepIdx,:)));
[sortedNeighDist sortedIdx]= sort(DistMtx,2);
%this results in a square matrix with distance to further neighbours increasing across
%columns. First column is all zeros (diagonal elements in unsorted matrix). Idx is an
%index of the sorted columns (Idx(:,2) is the nearest neighbour).

% remember max distance
mxDist=max(sortedNeighDist(:));

%% CONNECT NEAREST NEIGHBORS
nextconn=1;
marked=[];
nextconn2retest=1;
unmarked=[];
nndists=sortedNeighDist(:,2);
nnIdx=sortedIdx(:,2);
while sum(nndists >= mxDist) <= length(nndists)-1
    %% Get the closest pair of interior endpoints to connect...
    eps=1e-8;
    [smallestGap] = min(nndists);
    %disp(['distance before indexing: ',num2str(smallestGap)]);   % useful for checking that indexing is correct
    % Are they mutually nearest neighbors?
    smallestGapPairCandidateIdx = find(nndists<=smallestGap+eps);
    PairsIdx = sort([smallestGapPairCandidateIdx nnIdx(smallestGapPairCandidateIdx)],2); % if points are mutually nearest each other, index pairs will appear 2x (switched order)
    PairedIdx = unique(PairsIdx,'rows');       % keeping only unique pairs may not include only mutually nearest neighbors
    PairedIdx = PairedIdx(1,:);                % keep only the first returned pair
    % ONLY CONSIDER PairedIdx from here on...
    MutualPair = (sum((PairsIdx(:,1)==PairedIdx(1))+(PairsIdx(:,2)==PairedIdx(2))==2)>=2);     % check that PairedIdx appeared in PairsIdx 2x
    % Are they on different strands?
    p1 = keepIdx(PairedIdx(1));
    p2 = keepIdx(PairedIdx(2));
    p1xyz=unconCoord(p1,:);
    p2xyz=unconCoord(p2,:);
    %disp(['distance after indexing: ',num2str(sqrt(sum((p1xyz-p2xyz).^2)))]);     % useful for checking that indexing
    %is correct
    strandID1=unconStrandId(p1);
    strandID2=unconStrandId(p2);
    DifferentStrands = (strandID1 ~= strandID2);     % not the same strandID
    % Are the strands on either side "pointed" at each other (high dot product?)
    % Is the connection pointed at each free end?
    strand1ptsidx = strandSet.strandSetInfo(strandID1).ptr2coord;
    strand2ptsidx = strandSet.strandSetInfo(strandID2).ptr2coord;
    s1xyz=strandSet.strandData(strand1ptsidx,2:4);
    s2xyz=strandSet.strandData(strand2ptsidx,2:4);
    % s2sdot is the dot product of the free ends
    % scdot is the average dot product of the free ends with the connection
    [s2sdot,scdot,s1cdot,s2cdot] = strand2stranddot(s1xyz,s2xyz,p1xyz,p2xyz);
    disp(['strand2strand dot: ',num2str(s2sdot)]);
    disp(['strand2conn dot  : ',num2str(scdot)]);
    disp(['strand1-conn dot : ',num2str(s1cdot)]);
    disp(['strand2-conn dot : ',num2str(s2cdot)]);
    % If so, add connection and remove all potential connections to those points
    if MutualPair && DifferentStrands && (smallestGap <= maxGapDist_um) && (s2sdot>0) && (scdot>0) && (s1cdot>0) && (s2cdot>0)
        connections2add(nextconn).pt2pt=[strandSet.ptr2uncon(p1),strandSet.ptr2uncon(p2)];
        connections2add(nextconn).strand2strand=[strandID1,strandID2];
        if dispon
            plot3([p1xyz(:,1),p2xyz(:,1)],[p1xyz(:,2),p2xyz(:,2)],[p1xyz(:,3),p2xyz(:,3)],'k-','linewidth',2);
        end
        % if a connection is made, completely remove those nodes from further candidate connections
        DistMtx(PairedIdx(1),:)=mxDist+eps;
        DistMtx(PairedIdx(2),:)=mxDist+eps;
        DistMtx(:,PairedIdx(1))=mxDist+eps;
        DistMtx(:,PairedIdx(2))=mxDist+eps;
        marked=[marked,PairedIdx];
        nextconn=nextconn+1;
        disp(['connected index ',num2str(PairedIdx(1)),' with ',num2str(PairedIdx(2))]);
        disp('---------------------------------------------------------------------------');
    else
        connections2retest(nextconn2retest).pt2pt=[strandSet.ptr2uncon(p1),strandSet.ptr2uncon(p2)];
        connections2retest(nextconn2retest).strand2strand=[strandID1,strandID2];
        % if a connection is NOT made, only remove that pair of nodes from further candidate connections
        if ~MutualPair; disp('not mutual pair');end
        if ~DifferentStrands; disp('same strand');end
        if ~(smallestGap <= maxGapDist_um); disp('gap too large');end
        if ~(s2sdot>0); disp('strands dont point at each other');end
        if ~(s1cdot); disp('strand 1 not pointed with connection');end
        if ~(s2cdot); disp('strand 2 not pointed with connection');end
        if dispon
            plot3([p1xyz(:,1),p2xyz(:,1)],[p1xyz(:,2),p2xyz(:,2)],[p1xyz(:,3),p2xyz(:,3)],'b-','linewidth',2);
        end
        DistMtx(PairedIdx(1),PairedIdx(2))=mxDist+eps;
        DistMtx(PairedIdx(2),PairedIdx(1))=mxDist+eps;
        unmarked=[unmarked,PairedIdx];
        nextconn2retest=nextconn2retest+1;
        disp(['couldnt connect index ',num2str(PairedIdx(1)),' with ',num2str(PairedIdx(2))]);
        disp('---------------------------------------------------------------------------');
    end
    % now mark the temporary sorted distance and index matrices to remove the pair from consideration in the next set of candidates
    nndists(nnIdx==PairedIdx(1))=mxDist+eps;
    nndists(nnIdx==PairedIdx(2))=mxDist+eps;
end

if dispon
    legend('endpoint','endpoint strand','connection accepted','connection rejected');
    axis([0 300 0 300 0 300]);
end

