function outExp = ReconstructVascFromRadii(vidaProject,outputMatFileName,outputTifFileName)

%GENERAL REQUIRED PARAMETERS 

maskSize = vidaProject.registeredExperiment.strandifyParameters.centerlineRadiusStructure.maskSize;
centerlinePixelList = vidaProject.registeredExperiment.strandifyParameters.centerlineRadiusStructure.pixelList;
radiusListCalc =vidaProject.registeredExperiment.strandifyParameters.centerlineRadiusStructure.radiusList;
numCenterlinePixels = numel(centerlinePixelList);
ySize = maskSize(1);
xSize = maskSize(2);
zSize = maskSize(3);

        %RECONSTRUCT VASC MASK FROM THE CENTERLINE MASK AND BWDIST RADII
        registeredVascReconstructedMask = false(maskSize);
        numCenterlinePixels = size(centerlinePixelList,1);
        %Create a storage space for spheres to avoid re-creating them
        sphereCell = cell([10000,1]);
        for pixelIter = 1:numCenterlinePixels,
            currentPixel = centerlinePixelList(pixelIter);
            currentRadius = radiusListCalc(pixelIter);
            [cY,cX,cZ] = ind2sub(maskSize,currentPixel);
            sphereRadius = double(currentRadius);
            sphereRadiusIndex = int16(ceil(sphereRadius*10));
            if sphereRadius>0,
                %Fill in with a sphere
                if isempty(sphereCell{sphereRadiusIndex}),
                    sphereDiameter = sphereRadius * 2;
                    myhalf = 1+round(sphereDiameter-1)/2;
                    [x,y,z] = meshgrid(-myhalf:myhalf,-myhalf:myhalf,-myhalf:myhalf);
                    myR = sqrt(x.^2 + y.^2 +z.^2);
                    mySphere = myR<=(sphereDiameter/2);
                    if sphereDiameter <= 2,mySphere = cat(3,[1 1; 1 1],[1 1; 1 1]);end
                    if sphereDiameter <= 1,mySphere = 1; end
                    sphereCell{sphereRadiusIndex} = mySphere;
                end%if isempty(sphereCell(sphereRadiusIndex))
                mySphere = sphereCell{sphereRadiusIndex};
                cubeSize = size(mySphere,1);
                halfCubeMinus = int16(floor(cubeSize/2));
                halfCubePlus = cubeSize-halfCubeMinus-1;
                %---
                xminus = cX-halfCubeMinus;
                x1Block = max(1,xminus);
                x1Sphere = max(1,((halfCubeMinus + 2) - cX));
                xplus = cX+halfCubePlus;
                x2Block = min(xplus,xSize);
                x2Sphere = min(cubeSize,xSize+cubeSize-xplus);
                %---
                yminus = cY-halfCubeMinus;
                y1Block = max(1,yminus);
                y1Sphere = max(1,((halfCubeMinus + 2 ) - cY));
                yplus = cY+halfCubePlus;
                y2Block = min(yplus,ySize);
                y2Sphere = min(cubeSize,ySize+cubeSize-yplus);
                %---
                zminus = cZ-halfCubeMinus;
                z1Block = max(1,zminus);
                z1Sphere = max(1,((halfCubeMinus + 2) - cZ));
                zplus = cZ+halfCubePlus;
                z2Block = min(zplus,zSize);
                z2Sphere = min(cubeSize,zSize+cubeSize-zplus);
                %---
                cutSphere = mySphere(y1Sphere:y2Sphere,x1Sphere:x2Sphere,z1Sphere:z2Sphere);
                cutBlock = registeredVascReconstructedMask(y1Block:y2Block,x1Block:x2Block,z1Block:z2Block);
                unionCutBlock = or(cutSphere,cutBlock);
                registeredVascReconstructedMask(y1Block:y2Block,x1Block:x2Block,z1Block:z2Block) = unionCutBlock;
            end%if sphereRadius>0
        end%for pixelIter = 1:numCenterlinePixels,
        %----------
saveSize = (getfield(whos('registeredVascReconstructedMask'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save(outputMatFileName,'registeredVascReconstructedMask',scs);
maketiff(registeredVascReconstructedMask,outputTifFileName)
        
% outExp = inExp;
% outExp.globalParameters.registeredVascReconstructedMaskFileName = registeredVascReconstructedMaskFileName;
return

