function centerlinePts = ReduceCenterlineMaskJunctions(centerlinePts,maskSize)

% centerLineMask : a mask of 1s on centerline locations (some junction
%                  points are clustered)
% CLtrimmed      : a mask of 1s on centerline locations (with junction
%                  clusters reduced to have minimal points--but may
%                  include some additional morphological noise, i.e.: hairs)

centerLineMask = false(maskSize);
centerLineMask(centerlinePts) = true;
[centerlineStats,L,paddedCL] = GetCenterlineStats(centerLineMask); % count junctions and junction islands and created padded versions

%% STEP 1: TRIM JUNCTION CLUSTERS VIA THINNING:
Lpad = zeros(size(L)+2);                         % pad junction island labels
Lpad(2:end-1,2:end-1,2:end-1) = L;               % create padded version of junction labels

RP = regionprops(Lpad);
nchar = 0;
for iRP = 1:numel(RP)
    if any(iRP == round(linspace(numel(RP)/10,numel(RP),10)))
        fprintf(repmat('\b',1,nchar))
        nchar = fprintf(1,'Thinning reduction of junction cluster %d of %d...\n',iRP,numel(RP));
        pause(.01);
    end
    if RP(iRP).Area > 1  % If the volume of the label is 1, ignore it--it's an isolated junction point
        bb = RP(iRP).BoundingBox;
        JIbb1 = (bb(2) - 1 : bb(2) + bb(5)) + .5;   % get the bounding box by dimension
        JIbb2 = (bb(1) - 1 : bb(1) + bb(4)) + .5;   % get the bounding box by dimension
        JIbb3 = (bb(3) - 1 : bb(3) + bb(6)) + .5;   % get the bounding box by dimension
        bbclm = paddedCL(JIbb1,JIbb2,JIbb3);        % bbclm is the original centerline mask in the bb
        bbL = Lpad(JIbb1,JIbb2,JIbb3) == iRP;       % bbL is the junction island labelmap in the bb
        paddedCL(JIbb1,JIbb2,JIbb3) = ThinJunctionIsland(bbclm,bbL);
        while sum(sum(sum(abs(single(paddedCL(JIbb1,JIbb2,JIbb3)) - single(bbclm))))) ~= 0  % iterate until irreducible
            bbclm = paddedCL(JIbb1,JIbb2,JIbb3);
            bbL = bbL & bbclm; % restrict to only remaining points in bbL
            paddedCL(JIbb1,JIbb2,JIbb3) = ThinJunctionIsland(bbclm,bbL);
        end
    end
end
CLtrimmed = paddedCL(2:end-1,2:end-1,2:end-1); % trim padded mask back

centerlinePts = find(CLtrimmed);
