function outputParameters = RunTRPrime(inputParameters);

if ~exist('inputParameters'),
    inputParameters = [];
    rootFolder = cd;
    myDir = dir('*VidaProject.mat');
    vidaProjectFileName = fullfile(rootFolder,myDir(1).name);
    disp(['Loading VidaProject : ',datestr(now,'mmddyy_HH:MM.SS')]);
    load(vidaProjectFileName);
    inputParameters = vidaProject;
end%if ~exist(inputParameters)

vidaProject = inputParameters;
clear inputParameters

rootName = vidaProject.vidaProjectRootName;
rootCodeFolder = vidaProject.codeFolders.rootCodeFolder;
rootFolder = vidaProject.vidaProjectRootFolder;
addpath(fullfile(rootCodeFolder,'GapConnectionFunctionsFolder'));


%% define file names 

EnhancedVesselVolumeMatFname = vidaProject.globalParameters.registeredVascEnhancedGrayscaleFileName;
GrayscaleVesselVolumeMatFname = vidaProject.globalParameters.registeredVascGrayscaleFileName;
IsotropicVesselMaskMatFname = vidaProject.globalParameters.registeredVascReconstructedMaskFileName;
registeredFilesFolder = vidaProject.globalParameters.registeredFilesFolder;
maskSize = vidaProject.globalParameters.registeredStackSize;
concatenatedFileFolder = registeredFilesFolder;

% % % 
% % % %mat file names
% % % EnhancedVesselVolumeMatFname = fullfile(concatenatedFileFolder, [rootName '-registeredVascEnhancedGrayscale.mat']);
% % % GrayscaleVesselVolumeMatFname = fullfile(concatenatedFileFolder, [rootName '-registeredVascGrayscale.mat']);
% % % IsotropicVesselMaskMatFname = fullfile(concatenatedFileFolder, [rootName '-registeredVascReconstructedMask.mat']);
% % % 
% % % %tif counterpart
% % % EnhancedVesselVolumeTifFname = strrep(EnhancedVesselVolumeMatFname,'.mat','.tif');
% % % GrayscaleVesselVolumeTiftFname = strrep(GrayscaleVesselVolumeMatFname,'.mat','.tif');
% % % IsotropicVesselMaskTifFname = strrep(IsotropicVesselMaskMatFname,'.mat','.tif');

%vectorization (mv3d) 
VectorizedMV3DFilename =  fullfile(rootFolder, [rootName '-StrandSet.mv3d']);

%% 0 - Extract Strands and Vertices and write mv3d file

maskSize = vidaProject.globalParameters.centerlineRadiusStructure.maskSize;
vsz1 = maskSize(1);
vsz2 = maskSize(2);
nslices = maskSize(3);

Strands = vidaProject.globalParameters.vectorizedStructure.Strands;
Vertices = vidaProject.globalParameters.vectorizedStructure.Vertices;







disp('Augmenting points with edge points - RunTRPrime-line58');
augmentedFileName = fullfile(rootFolder,[rootName,'-AugmentedPoints.mat']);
if exist(augmentedFileName,'file');
    disp(['Loading Previous edge augmented file : ',datestr(now,'mmddyy_HH:MM.SS')]);
    load(augmentedFileName),
else
    skipSize = 15;
    tempBlock = false(maskSize);
    tempBlock(2,1:skipSize:end,1:skipSize:end) = 1; tempBlock(end-1,1:skipSize:end,1:skipSize:end) = 1;
    tempBlock(1:skipSize:end,2,1:skipSize:end) = 1; tempBlock(1:skipSize:end,end-1,1:skipSize:end) = 1;
    tempBlock(1:skipSize:end,1:skipSize:end,2) = 1; tempBlock(1:skipSize:end,1:skipSize:end,end-1) = 1;
    [qy,qx,qz] = ind2sub(size(tempBlock),find(tempBlock));
    edgePoints = int16([qy,qx,qz]);
    numOldVerts = size(Vertices.AllVerts,1);
    numNewVerts = size(qy,1);
    numStrands = size(Strands,2);
    q0 = single(qy*0);
    q1 = single(qy*0+1);
    q00 = false(size(q0));
    Vertices.AllVerts = [Vertices.AllVerts;edgePoints];
    Vertices.AllRadii = [Vertices.AllRadii;q1];
    Vertices.AllStrandEndPoints = [Vertices.AllStrandEndPoints;q00];
    Vertices.JunctionPoints = [Vertices.JunctionPoints;q00];
    Vertices.FreeEndPoints = [Vertices.FreeEndPoints;q00];
    temp = Vertices.StrandIndices;
    for k = numOldVerts+1:numOldVerts+numNewVerts,
        if mod(k,1000)==0, disp([k,numOldVerts+numNewVerts]);end
        temp{k} = numStrands+1;
    end;
    Vertices.StrandIndices = temp;
    Strands(numStrands+1).StartVertexIndex = numOldVerts+1;
    Strands(numStrands+1).EndVertexIndex = numOldVerts+numNewVerts;
% %     Strands(numStrands+1).InteriorVertices = [numOldVerts+2:numOldVerts+numNewVerts-1];
    Strands(numStrands+1).StartToEndIndices = [numOldVerts+1:numOldVerts+numNewVerts];
    Strands(numStrands+1).StartVertexNeighborStrands = [];
    Strands(numStrands+1).EndVertexNeighborStrands = [];
    Strands(numStrands+1).Active = 1;
    save(augmentedFileName,'Vertices','Strands');
end













if ~exist(VectorizedMV3DFilename,'file');
    if ~WriteMV3DFile(Strands,Vertices,VectorizedMV3DFilename);
        fprintf('\nFailed to wirte %s!\nHalting',VectorizedMV3DFilename);
    end
end
%% 1 - create tiff files required for TR (skip vectorization step)

% % % fprintf('\nCreating Enhanced Gray Scale tif. Loading mat file....');
% % % load(EnhancedVesselVolumeMatFname);
% % % if ~exist('registeredVascEnhancedGrayscale'),
% % %     registeredVascEnhancedGrayscale = vascEnhancedGrayscale;
% % % end
% % % fprintf('Writing tif....');
% % % maketiff(registeredVascEnhancedGrayscale,EnhancedVesselVolumeTifFname);
% % % fprintf('Done.');
% % % clear registeredVascEnhancedGrayscale
% % % 
% % % fprintf('\nCreating Vessel Gray Scale tif. Loading mat file....');
% % % load(GrayscaleVesselVolumeMatFname);
% % % if ~exist('registeredVascGrayscale'),
% % %     registeredVascGrayscale = vascBlock;
% % % end
% % % fprintf('Writing tif....');
% % % maketiff(registeredVascGrayscale,GrayscaleVesselVolumeTiftFname);
% % % fprintf('Done.');
% % % clear registeredVascGrayscale
% % % 
% % % % create 'balls' from radii
% % %  ReconstructVascFromRadii(vidaProject,IsotropicVesselMaskMatFname,IsotropicVesselMaskTifFname)



%% ConnectGaps
%% Set parameters for test suite
VesselRepresentations.RawVesselVolumeMatFname = GrayscaleVesselVolumeMatFname;
VesselRepresentations.EnhancedVesselVolumeMatFname = EnhancedVesselVolumeMatFname;





% % % 
% % % disp('RunTRPrime - line 84')
% % % VesselRepresentations.EnhancedVesselVolumeMatFname = GrayscaleVesselVolumeMatFname
% % % 




TRFilesFolder = fullfile(rootFolder,'TRFiles');
[success,message,messageID] = mkdir(rootFolder,'TRFiles');

VesselRepresentations.IsotropicVesselMaskMatFname = IsotropicVesselMaskMatFname;
VesselRepresentations.VectorizedMV3DFilename = VectorizedMV3DFilename;
VesselRepresentations.maskSize = maskSize;
VesselRepresentations.rootFolder = rootFolder;
VesselRepresentations.rootName = rootName;
VesselRepresentations.TRFilesFolder = TRFilesFolder;


GapConnectionParameters.prmts.boundBoxMinDist = 10;
GapConnectionParameters.prmts.boundBoxDimXYZ = [vsz1 vsz2 nslices];
GapConnectionParameters.prmts.maxGapDist_um = 30;
GapConnectionParameters.prmts.minGapDist_um = 0;
GapConnectionParameters.hopislands = 1;
GapConnectionParameters.PreprocessNNConnections = 1;
GapConnectionParameters.BBmargin = [20 35 50];
GapConnectionParameters.thresholdGapConfidence = 0.33;
GapConnectionParameters.threshparameters.minvvol = 5;
GapConnectionParameters.threshparameters.morphfiltsz = 3;
VesselRepresentations.GapConnectionParameters = GapConnectionParameters;

[GapConnectionStruct] = LocalThresholdRelaxationConnectionPrime(VesselRepresentations);
vidaProject.globalParameters.GapConnectionStruct = GapConnectionStruct;
outputParameters = vidaProject;

