function [BridgeStrandEP1EP2] = ShortestBridgeEP1ToEP2(pt1xyz,pt2xyz,GCM,x,y,z)

%Copyright 2008 J.Kaufhold

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.

dispon=0;

% get nearest lattice points for bridging strand computations on lattice graph
latpt1xyz=round(pt1xyz);
latpt2xyz=round(pt2xyz);

% Save originals...
GCM0=GCM;x0=x;y0=y;z0=z;

%% Trim out only a part of the data 4r from the midpoint between the endpoints to connect
% compute the midpoint between the two endpoints to connect
latmidptxyz = round(.5 * (latpt1xyz + latpt2xyz));
% compute 3x the radius to the midpoint between the two endpoints to connect
rcent = 4*ceil(max ( sqrt(sum((latmidptxyz-latpt1xyz).^2)),sqrt(sum((latmidptxyz-latpt2xyz).^2))));

tmpmask = (x >= latmidptxyz(2)-rcent) & (x <= latmidptxyz(2)+rcent) & (y >= latmidptxyz(1)-rcent) & (y <= latmidptxyz(1)+rcent) & (z >= latmidptxyz(3)-rcent) & (z <= latmidptxyz(3)+rcent);
x = x(tmpmask);
y = y(tmpmask);
z = z(tmpmask);
GCM = GCM(tmpmask);
xrange = max(x(:))-min(x(:))+1;
yrange = max(y(:))-min(y(:))+1;
zrange = max(z(:))-min(z(:))+1;
x = reshape(x,yrange,xrange,zrange);
y = reshape(y,yrange,xrange,zrange);
z = reshape(z,yrange,xrange,zrange);
GCM = reshape(GCM,yrange,xrange,zrange);

%% Get lattice indices of pt1 and pt2
pt1IND=find((latpt1xyz(2)==x(:))&(latpt1xyz(1)==y(:))&(latpt1xyz(3)==z(:)));
pt2IND=find((latpt2xyz(2)==x(:))&(latpt2xyz(1)==y(:))&(latpt2xyz(3)==z(:)));

L = bwlabeln(GCM);
if L(pt1IND) ~= L(pt2IND)
    disp('Mask does not connect candidate centerline points... reverting to original size');
    GCM=GCM0;x=x0;y=y0;z=z0;
    pt1IND=find((latpt1xyz(2)==x(:))&(latpt1xyz(1)==y(:))&(latpt1xyz(3)==z(:)));
    pt2IND=find((latpt2xyz(2)==x(:))&(latpt2xyz(1)==y(:))&(latpt2xyz(3)==z(:)));
    L = bwlabeln(GCM);
    if L(pt1IND) ~= L(pt2IND)
        disp('Original mask does not connect candidate centerline points... exiting...');
    end
end
clear L
clear GCM0 x0 y0 z0

[lpt1x,lpt1y,lpt1z] = ind2sub(size(GCM),pt1IND);
[lpt2x,lpt2y,lpt2z] = ind2sub(size(GCM),pt2IND);

% Get pathlength distance from pt1
[D1]=ChamferDT(GCM,[lpt1x,lpt1y,lpt1z]);
% Get pathlength distance from pt2
[D2]=ChamferDT(GCM,[lpt2x,lpt2y,lpt2z]);
% Adding them together puts a ridge of ~constant distance on the minimum path
% (except for the hardcoded -10s outside mask)
Dreciprocal = D1+D2;
clear D1 D2
% Find the max distance
maxDr = max(Dreciprocal(:));
% Replace pts outside the mask with that maxDr, so the minimum distance is approximately where the strand is
Dreciprocal(Dreciprocal==-20) = maxDr;

% minGCM is the "minimal" gap-connecting mask... It should connect the two points
minGCM = Dreciprocal < min(Dreciprocal(:))+3;
minGCM = imdilate(minGCM,true(3,3,3),'same');
clear Dreciprocal

%% Get all pairs of neighbor indices in minGCM + GCM if needed
save('largevars_tmp.mat','x','y','z','minGCM','GCM','-mat');
szmGCM = size(minGCM);
clear x y z minGCM GCM
[D1,D2,D3]=DG3D(szmGCM);
load('largevars_tmp.mat','x','y','z','minGCM','GCM','-mat');

% Get all pairs of neighbor indices in minGCM
v=minGCM(:);
D1=abs(D1);linkrows1=(D1*v)==2;
D2=abs(D2);linkrows2=(D2*v)==2;
D3=abs(D3);linkrows3=(D3*v)==2;
[alllinksi,alllinksj]=find([D1(linkrows1,:);D2(linkrows2,:);D3(linkrows3,:)]');
linkedpairs=reshape(alllinksi,2,numel(alllinksi)/2)';

%% Find shortest path from pt1 to pt2 using only all pairs
% organize input to dijkstra
nodes=[(1:numel(x))',x(:),y(:),z(:)];
segments=[(1:size(linkedpairs,1))',linkedpairs];
% get the shortest path
[dist, shortestpath] = dijkstra(nodes, segments, pt1IND, pt2IND);

try
    BridgeStrandEP1EP2=[x(shortestpath)',y(shortestpath)',z(shortestpath)'];
catch
    disp('error in BridgeStrandEP1EP2 in ShortestBridgeEP1ToEP2');
    disp('falling back to larger mask');
    % Get all pairs of neighbor indices in GCM
    vG=GCM(:);
    linkrows1G=(D1*vG)==2;
    linkrows2G=(D2*vG)==2;
    linkrows3G=(D3*vG)==2;
    [alllinksiG,alllinksjG]=find([D1(linkrows1G,:);D2(linkrows2G,:);D3(linkrows3G,:)]');
    linkedpairsG=reshape(alllinksiG,2,numel(alllinksiG)/2)';
    % organize input to dijkstra
    nodesG=[(1:numel(x))',x(:),y(:),z(:)];
    segmentsG=[(1:size(linkedpairsG,1))',linkedpairsG];
    % get the shortest path
    [distG, shortestpathG] = dijkstra(nodesG, segmentsG, pt1IND, pt2IND);
    try
        BridgeStrandEP1EP2=[x(shortestpathG)',y(shortestpathG)',z(shortestpathG)'];
    catch
        disp('problem finding bridging strand after fallback to larger mask');
        keyboard;
        error('problem finding bridging strand after fallback to larger mask');
    end
end

pt1rounded=sum(abs(BridgeStrandEP1EP2(1,:)-[pt1xyz(2),pt1xyz(1),pt1xyz(3)]))>1e-10;
if pt1rounded
    % add first point
    BridgeStrandEP1EP2=[[pt1xyz(2),pt1xyz(1),pt1xyz(3)];BridgeStrandEP1EP2];
end

pt2rounded=sum(abs(BridgeStrandEP1EP2(end,:)-[pt2xyz(2),pt2xyz(1),pt2xyz(3)]))>1e-10;
if pt2rounded
    % add last point
    BridgeStrandEP1EP2=[BridgeStrandEP1EP2;[pt2xyz(2),pt2xyz(1),pt2xyz(3)]];
end

if dispon
    figure;clf;hold on;
    patch(isosurface(x,y,z,GCM, .5), 'FaceColor', 'black', 'EdgeColor', 'none','facealpha',.1);
    patch(isosurface(x,y,z,minGCM, .5), 'FaceColor', 'green', 'EdgeColor', 'none','facealpha',.1);
    plot3([x(linkedpairs(:,1)),x(linkedpairs(:,2))]',[y(linkedpairs(:,1)),y(linkedpairs(:,2))]',[z(linkedpairs(:,1)),z(linkedpairs(:,2))]','r-')
    plot3(x(pt1IND),y(pt1IND),z(pt1IND),'m.','markersize',25);
    plot3(x(pt2IND),y(pt2IND),z(pt2IND),'m.','markersize',25);
    plot3(BridgeStrandEP1EP2(:,1),BridgeStrandEP1EP2(:,2),BridgeStrandEP1EP2(:,3),'k-','linewidth',3)
    try
        plot3([x(linkedpairsG(:,1)),x(linkedpairsG(:,2))]',[y(linkedpairsG(:,1)),y(linkedpairsG(:,2))]',[z(linkedpairsG(:,1)),z(linkedpairsG(:,2))]','m-')
    catch
        disp('some linked pairs could not plot');
    end
    axis equal;
    title('gap connecting mask (black)');
end
