function [AMjunctionreplaced,cxyz] = SimplifyJunctionClustersToSingleJunctions(AM,cxyz)

dispon = 0;
AM0 = AM;
fprintf('Replacing irreducible but connected junction clusters...\n');

if dispon
    figure(1);
    set(1,'position',[93 145 567 547]);
    gplot3(AM0,cxyz);
    hold on;
end

junctionnodes = full(sum(AM)>=3);
JCTclustercandidates = find(junctionnodes);
nJCTpts = numel(JCTclustercandidates);
JCTvisited = zeros(1,size(AM,1));
nchar = 0;
for iJCT = 1:nJCTpts
    if ~JCTvisited(JCTclustercandidates(iJCT))
        % Get a list of all junctions adjacent to JCTclustercandidates(iJCT)
        AdjNodeList = full(AM(JCTclustercandidates(iJCT),:))>0;
        AdjNodes = find(AdjNodeList);
        AdjJunctions = junctionnodes(AdjNodeList);
        NewJunctionClusterMembers = AdjNodes(AdjJunctions); % initialize with the first set of adjacent junction cluster members
        AllJunctionClusterMembers = [JCTclustercandidates(iJCT),NewJunctionClusterMembers];
        while ~isempty(NewJunctionClusterMembers)
            MoreJunctionClusterMembers = []; % Initialize the list of junctions to grow the cluster by to none until observed
            for iNJCM = 1:numel(NewJunctionClusterMembers)
                AdjNodeList = full(AM(NewJunctionClusterMembers(iNJCM),:))>0;
                AdjNodes = find(AdjNodeList);
                AdjJunctions = junctionnodes(AdjNodeList);
                AdditionalClusterMembers = setdiff(AdjNodes(AdjJunctions),AllJunctionClusterMembers); % add any cluster members that weren't in the original cluster membership
                MoreJunctionClusterMembers = [MoreJunctionClusterMembers,AdditionalClusterMembers]; % accumulate all the new junctions in the cluster
            end
            NewJunctionClusterMembers = MoreJunctionClusterMembers; % update the NewJunctionClusterMembers list
            AllJunctionClusterMembers = [AllJunctionClusterMembers,NewJunctionClusterMembers];
        end
        AllJunctionClusterMembers = unique(AllJunctionClusterMembers); % in case some junctions may be overcounted (probably redundant)
        % if there are more than 2 junction nodes touching, replace all connections into that cluster with a connection to one of the nodes
        if numel(AllJunctionClusterMembers) >= 2
            % Get all connections into cluster
            ConnectionsIntoCluster = setdiff(find(sum(full(AM(AllJunctionClusterMembers,:)))>0),AllJunctionClusterMembers);

            % Remove all connections among and to all elements of AllJunctionClusterMembers
            for iAJCM = 1:numel(AllJunctionClusterMembers);
                AM(AllJunctionClusterMembers(iAJCM),:) = 0;
                AM(:,AllJunctionClusterMembers(iAJCM)) = 0;
            end
            
            % Identify 1 of the junction points as the "replaced junction point"
            ReplacedJunctionPoint = AllJunctionClusterMembers(1);
            
            % Move that junction point location to the mean of the junction
            % point locations in the cluster
            cxyz(ReplacedJunctionPoint,:) = mean(cxyz(AllJunctionClusterMembers,:));
            
            % Move all the other old centerline locations to 0,0,0 (to flag for
            % removal)
            EliminatedJunctionClusterMembers = AllJunctionClusterMembers(2:end);
            cxyz(EliminatedJunctionClusterMembers,:) = 0;
            
            % Make connections between all centerline points formerly
            % incident on the junction cluster (ConnectionsIntoCluster) to the new replaced junction
            % point
            for iCIC = 1:numel(ConnectionsIntoCluster);
                % remove all outside connections only to the junction cluster members
                AM(ConnectionsIntoCluster(iCIC),AllJunctionClusterMembers) = 0;
                AM(AllJunctionClusterMembers,ConnectionsIntoCluster(iCIC)) = 0;
            end  
            
            for iCIC = 1:numel(ConnectionsIntoCluster);   
                % make connections only to the junction that replaced the
                % cluster
                AM(ConnectionsIntoCluster(iCIC),ReplacedJunctionPoint) = 1;
                AM(ReplacedJunctionPoint,ConnectionsIntoCluster(iCIC)) = 1;
            end
            %   mark all those junction nodes as visited
            JCTvisited(AllJunctionClusterMembers) = true; % mark all the junction cluster junctions as visited
        else
            % otherwise {it's a single junction}
            JCTvisited(JCTclustercandidates(iJCT)) = true;                      % mark the single junction as visited
        end
    end
    fprintf(repmat('\b',1,nchar))
    nchar = fprintf('%d / %d junction nodes examined...\n',iJCT,nJCTpts);
end

if dispon
    gplot3(AM,cxyz + .1*randn(size(cxyz)),'r');
end

% Clean up graph (remove empty adjacency matrix rows + columns and
% corresponding centerline points)
FlaggedForGraphRemoval = sum(AM) == 0;
AM(FlaggedForGraphRemoval,:) = [];
AM(:,FlaggedForGraphRemoval) = [];
cxyz(FlaggedForGraphRemoval,:) = [];

if dispon
    gplot3(AM,cxyz + .1*randn(size(cxyz)),'k');
end

AMjunctionreplaced= AM;
