%input vidaProject
clear all
close all
clc

rootFolder = '/home/pablo/Data/Run7pbCP_400x400x400/';
rootName = 'pbCP';
concatenatedFileFolder = [rootFolder 'RegisteredFiles' filesep 'ConcatenatedFilesFolder'];
load(fullfile(rootFolder,[rootName '-vidaProject.mat']));
%% 0 - Extract Strands and Vertices
Strands = vidaProject.globalParameters.vectorizedStructure.Strands;
Vertices = vidaProject.globalParameters.vectorizedStructure.Vertices;

%% 1 - create tiff files required for TR (skip vectorization step)
load(fullfile(concatenatedFileFolder, [rootName '-registeredVascGrayscale.mat']))
maketiff(registeredVascGrayscale,
%%

VesselMaskFname = 'QF02-Stacked-FITC-mask-RawFractionReconstructed.tif';
CenterlineMaskFname = 'QF02-Stacked-FITC-mask-CenterLine.tif';
GrayscaleVesselVolumeFname = 'QF02-Stacked-FITC.tif';
VectorizationFname = 'QF02-Stacked-FITC-mask-RawFractionReconstructed-filt1.mv3d';
% VectorizationFname = 'amira-QF02-vec-result.mv3d';
GapFilledVesselMaskFname = 'QF02-Stacked-FITC-mask-GapFilled.tif';

% Inputs:
% =======================
% VesselMaskFname            : 1s where vessels are "on" (uint8 tif)
% CenterlineMaskFname        : 1s where the vessel mask's centerline is "on" (i.e. no simple points) (uint8 tif)
% GrayscaleVesselVolumeFname : grayscale .tif filename
% 
% Outputs:
% =======================
% GapFilledVesselMaskFname   : 1s where vessels are "on" after gap filling (uint8 tif)

%% 0. Should zero edges of both CenterlineMask and binary VesselMask

%% 1. Read centerline mask with 0 edges
load('QF02-Stacked-FITC-mask-CenterLine-zeroedges.mat','centerLineMask','-mat');
vsz1 = size(centerLineMask,1);
vsz2 = size(centerLineMask,2);
nslices = size(centerLineMask,3);
clear centerLineMask;


load('QF02-VectorizedCenterlines.mat','Strands','Vertices','AMCenterlines','centerlinePts','maskSize','-mat');
% 
% %% Show all endpoint vertices
% nStrands = numel(Strands);
% figure;clf;hold on;
% epcount = 0;
% for iSTRAND = 1:nStrands
%     plot3(Vertices.AllVerts(Strands(iSTRAND).StartVertexIndex,2),Vertices.AllVerts(Strands(iSTRAND).StartVertexIndex,1),Vertices.AllVerts(Strands(iSTRAND).StartVertexIndex,3),'ro');
%     plot3(Vertices.AllVerts(Strands(iSTRAND).EndVertexIndex,2),Vertices.AllVerts(Strands(iSTRAND).EndVertexIndex,1),Vertices.AllVerts(Strands(iSTRAND).EndVertexIndex,3),'ro');
%     if isempty(Strands(iSTRAND).StartVertexNeighborStrands)
%         plot3(Vertices.AllVerts(Strands(iSTRAND).StartVertexIndex,2),Vertices.AllVerts(Strands(iSTRAND).StartVertexIndex,1),Vertices.AllVerts(Strands(iSTRAND).StartVertexIndex,3),'b.');
%         epcount = epcount + 1;
%     end
%     if isempty(Strands(iSTRAND).EndVertexNeighborStrands)
%         plot3(Vertices.AllVerts(Strands(iSTRAND).EndVertexIndex,2),Vertices.AllVerts(Strands(iSTRAND).EndVertexIndex,1),Vertices.AllVerts(Strands(iSTRAND).EndVertexIndex,3),'b.');
%         epcount = epcount + 1;
%     end
% end

%% 3. Convert centerline mask to vectorized structure amenable to Threshold Relaxation
% i.e. build .mv3d file from Strands and Vertices structure

[Strands,Vertices] = FilterFragmentStrands(Strands,Vertices);
done = WriteMV3DFile(Strands,Vertices,VectorizationFname);

%% 4. Connect Gaps via Threshold Relaxation (i.e. compute BridgingStrands)
path(path,'C:\Documents and Settings\kaufholdj\Desktop\ManipulatingNeuralTissue1\ThresholdRelaxationDevelopment');

%% Set parameters for test suite
VesselRepresentations.RawVesselVolumeTifFname = GrayscaleVesselVolumeFname;
VesselRepresentations.EnhancedVesselVolumeTifFname = GrayscaleVesselVolumeFname;
VesselRepresentations.IsotropicVesselMaskTifFname = VesselMaskFname;
VesselRepresentations.VectorizedMV3DFilename = VectorizationFname;
GapConnectionParameters.prmts.boundBoxMinDist = 10;
GapConnectionParameters.prmts.boundBoxDimXYZ = [vsz1 vsz2 nslices];
GapConnectionParameters.prmts.maxGapDist_um = 30;
GapConnectionParameters.prmts.minGapDist_um = 0;
GapConnectionParameters.hopislands = 1;
GapConnectionParameters.PreprocessNNConnections = 0;
GapConnectionParameters.BBmargin = 50;
GapConnectionParameters.threshparameters.minvvol = 5;
GapConnectionParameters.threshparameters.morphfiltsz = 3;
VesselRepresentations.GapConnectionParameters = GapConnectionParameters;

%[GapConnectionStruct] = LocalThresholdRelaxationConnection(VesselRepresentations);
% QF02_GCS.mat contains the bridging strands associated with Phil's centerlining
% load('QF02_GCS.mat','GapConnectionStruct');
load('QF02_GCS_v2.mat','GapConnectionStruct');
%load('QF02_amira_GCS.mat','GapConnectionStruct');
%load('QF02_amira_GCS_v2.mat','GapConnectionStruct');

%% 5. Reindex vectorization with BridgingStrands
%BridgingStrandVectorizationFname = 'amira-QF02-bridgingstrand-vec-result.mv3d';
BridgingStrandVectorizationFname = 'QF02-Stacked-FITC-mask-BridgingStrands-v2.mv3d';

done = WriteMV3DBridgingStrands(GapConnectionStruct.ThreshRelax.BridgingStrands,BridgingStrandVectorizationFname);

% sGC = sort(GapConnectionStruct.ThreshRelax.BridgingConfidences);
% HiConfINDs = GapConnectionStruct.ThreshRelax.BridgingConfidences > sGC(fix(2*numel(sGC)/3));
% LoConfINDs = GapConnectionStruct.ThreshRelax.BridgingConfidences < sGC(fix(numel(sGC)/3));
% MedConfINDs = ~(HiConfINDs|LoConfINDs);
% HighConfBridgingStrandVectorizationFname = 'HighConf-amira-QF02-bridgingstrand-vec-result.mv3d';
% MedConfBridgingStrandVectorizationFname = 'MedConf-amira-QF02-bridgingstrand-vec-result.mv3d';
% LowConfBridgingStrandVectorizationFname = 'LowConf-amira-QF02-bridgingstrand-vec-result.mv3d';
% 
% done = WriteMV3DBridgingStrands(GapConnectionStruct.ThreshRelax.BridgingStrands(HiConfINDs),HighConfBridgingStrandVectorizationFname);
% done = WriteMV3DBridgingStrands(GapConnectionStruct.ThreshRelax.BridgingStrands(MedConfINDs),MedConfBridgingStrandVectorizationFname);
% done = WriteMV3DBridgingStrands(GapConnectionStruct.ThreshRelax.BridgingStrands(LoConfINDs),LowConfBridgingStrandVectorizationFname);
% 
% figure;
% hsGC = histc(sGC,linspace(-1,4,50));
% plot(linspace(-1,4,50),hsGC);
% hold on;
% plot([sGC(fix(2*numel(sGC)/3)) sGC(fix(2*numel(sGC)/3))],[0 max(hsGC)],'k');
% plot([sGC(fix(numel(sGC)/3)) sGC(fix(numel(sGC)/3))],[0 max(hsGC)],'k');
% xlabel('z score');ylabel('frequency');
% 
% BridgeStrands = GapConnectionStruct.ThreshRelax.BridgingStrands;
% for iBRS = 1:numel(BridgeStrands)
%     if ~isempty(BridgeStrands{iBRS})
%         BridgeStrandStepLengths(iBRS) = sum(sum(abs(diff(BridgeStrands{iBRS}))));
%         BridgeStrandDists(iBRS) = sqrt(sum((BridgeStrands{iBRS}(1,:)-BridgeStrands{iBRS}(end,:)).^2));
%     end
% end
% 
% figure;
% ConfidentStrandsByLength = BridgeStrandStepLengths < 100*(GapConnectionStruct.ThreshRelax.BridgingConfidences - .5);
% plot(GapConnectionStruct.ThreshRelax.BridgingConfidences(~ConfidentStrandsByLength),BridgeStrandStepLengths(~ConfidentStrandsByLength),'r.');
% hold on;
% plot(GapConnectionStruct.ThreshRelax.BridgingConfidences(ConfidentStrandsByLength),BridgeStrandStepLengths(ConfidentStrandsByLength),'r.');
% plot([.5 2],[0,150],'k');
% xlabel('z-score of Bridging Strand');
% ylabel('Length of Bridging Strand');
% 
% LengthConfBridgingStrandVectorizationFname = 'LengthConf-amira-QF02-bridgingstrand-vec-result.mv3d';
% done = WriteMV3DBridgingStrands(GapConnectionStruct.ThreshRelax.BridgingStrands(ConfidentStrandsByLength),LengthConfBridgingStrandVectorizationFname);
% NotLengthConfBridgingStrandVectorizationFname = 'NotLengthConf-amira-QF02-bridgingstrand-vec-result.mv3d';
% done = WriteMV3DBridgingStrands(GapConnectionStruct.ThreshRelax.BridgingStrands(~ConfidentStrandsByLength),NotLengthConfBridgingStrandVectorizationFname);
% 
% %% Show bridging strands only
% figure;clf;hold on;
% nBridgingStrands = numel(GapConnectionStruct.ThreshRelax.BridgingStrands);
% for iSTRAND = 1:nBridgingStrands
%     if ~isempty(GapConnectionStruct.ThreshRelax.BridgingStrands{iSTRAND})
%         strandxyz = GapConnectionStruct.ThreshRelax.BridgingStrands{iSTRAND};
%         plot3(strandxyz(:,2),strandxyz(:,1),strandxyz(:,3));
%     end
% end


%% 6. Build GapFilledVesselMask from original binary mask and BridgingStrands
%% 7. Revectorize GapFilledVesselMask
%% 8. Compare the results of 5 & 7
