%input vidaProject
clear all
close all
clc

rootFolder = '/home/pablo/Data/Run7pbCP_400x400x400/';
rootName = 'pbCP';
concatenatedFileFolder = [rootFolder 'RegisteredFiles' filesep 'ConcatenatedFilesFolder'];
load(fullfile(rootFolder,[rootName '-vidaProject.mat']));

%add specific paths
addpath /media/sda1/MATLAB/work/VidaSuite/VascRadiiFunctionsFolder

%% define file names 

%mat file names
EnhancedVesselVolumeMatFname = fullfile(concatenatedFileFolder, [rootName '-registeredVascEnhancedGrayscale.mat']);
GrayscaleVesselVolumeMatFname = fullfile(concatenatedFileFolder, [rootName '-registeredVascGrayscale.mat']);
IsotropicVesselMaskMatFname = fullfile(concatenatedFileFolder, [rootName '-registeredVascReconstructedMask.mat']);

%tif counterpart
EnhancedVesselVolumeTifFname = strrep(EnhancedVesselVolumeMatFname,'.mat','.tif');
GrayscaleVesselVolumeTiftFname = strrep(GrayscaleVesselVolumeMatFname,'.mat','.tif');
IsotropicVesselMaskTifFname = strrep(IsotropicVesselMaskMatFname,'.mat','.tif');

%vectorization (mv3d) 
VectorizedMV3DFilename =  fullfile(concatenatedFileFolder, [rootName '-StrandSet.mv3d']);

%% 0 - Extract Strands and Vertices and write mv3d file

maskSize = vidaProject.registeredExperiment.strandifyParameters.centerlineRadiusStructure.maskSize;
vsz1 = maskSize(1);
vsz2 = maskSize(2);
nslices = maskSize(3);

Strands = vidaProject.globalParameters.vectorizedStructure.Strands;
Vertices = vidaProject.globalParameters.vectorizedStructure.Vertices;

if ~WriteMV3DFile(Strands,Vertices,VectorizedMV3DFilename);
    fprintf('\nFailed to wirte %s!\nHalting',VectorizedMV3DFilename);
end
%% 1 - create tiff files required for TR (skip vectorization step)

fprintf('\nCreating Enhanced Gray Scale tif. Loading mat file....');
load(EnhancedVesselVolumeMatFname);
fprintf('Writing tif....');
maketiff(registeredVascEnhancedGrayscale,EnhancedVesselVolumeTifFname);
fprintf('Done.');
clear registeredVascEnhancedGrayscale

fprintf('\nCreating Vessel Gray Scale tif. Loading mat file....');
load(GrayscaleVesselVolumeMatFname);
fprintf('Writing tif....');
maketiff(registeredVascGrayscale,GrayscaleVesselVolumeTiftFname);
fprintf('Done.');
clear registeredVascGrayscale

% create 'balls' from radii
 ReconstructVascFromRadii(vidaProject,IsotropicVesselMaskMatFname,IsotropicVesselMaskTifFname)



%% ConnectGaps
%% Set parameters for test suite
VesselRepresentations.RawVesselVolumeTifFname = GrayscaleVesselVolumeTiftFname;
VesselRepresentations.EnhancedVesselVolumeTifFname = EnhancedVesselVolumeTifFname;
VesselRepresentations.IsotropicVesselMaskTifFname = IsotropicVesselMaskTifFname;
VesselRepresentations.VectorizedMV3DFilename = VectorizedMV3DFilename;
GapConnectionParameters.prmts.boundBoxMinDist = 10;
GapConnectionParameters.prmts.boundBoxDimXYZ = [vsz1 vsz2 nslices];
GapConnectionParameters.prmts.maxGapDist_um = 30;
GapConnectionParameters.prmts.minGapDist_um = 0;
GapConnectionParameters.hopislands = 1;
GapConnectionParameters.PreprocessNNConnections = 0;
GapConnectionParameters.BBmargin = 50;
GapConnectionParameters.threshparameters.minvvol = 5;
GapConnectionParameters.threshparameters.morphfiltsz = 3;
VesselRepresentations.GapConnectionParameters = GapConnectionParameters;

[GapConnectionStruct] = LocalThresholdRelaxationConnection(VesselRepresentations);