function strandSet = amira2mat_buildStrandAdjacency(strandSet)
%Build graph based on strand connectivity
%Label also unconnected endpoints
%Pablo Blinder, J.Kaufhold - 30Sept2007

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.

%% find bifurcations coordinates
%strategy - bifurcation appear more than once (aka non-unique)
[uniqueCoord,ii_unique] = unique(strandSet.strandData(:,2:4) ,'rows');
nCtrPoints = size(strandSet.strandData,1);
ctrPointIdx = 1: nCtrPoints;
bifPointIdx = setdiff(ctrPointIdx,ii_unique)';
nbif = numel(bifPointIdx);

%now collect all strand endpoints, will be used to find unconnected ends
endPointIdx = [strandSet.ptr2endpoints;[strandSet.ptr2endpoints(2:end-1) + 1]];
endPointIdx(1) = 1;
ctrPointDeg = zeros(nCtrPoints,1) + 2;
ctrPointDeg(endPointIdx) = 1;

%% Build strand-based adjacency matrix
nStrands = numel(strandSet.strandSetInfo);
Adj = sparse(nStrands,nStrands);
LUT = strandSet.ctrPoint2StrandLUT; 
%LUT(:,1) running id of each center point, LUT(:,2) strand id assigned to center point
for ni = 1 : nbif
   
    %look up the coordinates ids that match each bifurcation
    bifPointIdxInStrands = ...
        find(strandSet.strandData(:,2) == strandSet.strandData(bifPointIdx(ni),2) ...
        & strandSet.strandData(:,3) == strandSet.strandData(bifPointIdx(ni),3) ...
        & strandSet.strandData(:,4) == strandSet.strandData(bifPointIdx(ni),4));
    ctrPointDeg(bifPointIdxInStrands) = 3;
    %for each of these coordinates, find strand identity based on the ctrpoint-strand
    %lookup table
    strandIds = interp1(LUT(:,1),LUT(:,2),bifPointIdxInStrands);
    
    %update adjacency
    pairs = amira2mat_permPairs(numel(strandIds));
    idx = sub2ind([nStrands,nStrands],[strandIds(pairs(:,1)); strandIds(pairs(:,2))],...
        [strandIds(pairs(:,2)); strandIds(pairs(:,1))]);
    Adj(idx) = 1;
    
end %cycling bifurcation points

%find unconnected strand ends based on center line degree
ptr2unconCtrPoint = find(ctrPointDeg == 1);

%% update strandSet data structure
strandSet.strandAdj = Adj;
strandSet.prt2bif = bifPointIdx;
strandSet.ptr2uncon = ptr2unconCtrPoint;

%% -----------------------------------------------------------------------------------------
function [nsubgs,subglabels] = countAndLabelSubgraphs(R)

% R          : the symmetric adjacency matrix
% subglabels : an integer label indexed into every point R relates
% nsubgs     : the number of subgraphs in R
% modified from John Kaufhold
R0=R;
subglabels=zeros(1,size(R,1));
R=spones(R)+speye(size(R,1));
[p,q,r] = dmperm(R);
for iP=1:length(r)-1; subglabels(p(r(iP):r(iP+1)-1))=iP; end
nsubgs=max(subglabels(:));