function [s2sdot,scdot,s1cdot,s2cdot] = strand2stranddot(s1,s2,p1,p2)

%Copyright 2008 J.Kaufhold

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.

% showing the gap distance helps confirm indexing is correct compared the calling function
gd=sqrt(sum((p1-p2).^2));
disp(['the gap distance in the function is: ',num2str(gd)]);

% figure;clf;
% plot3(s1(:,1),s1(:,2),s1(:,3),'r.');
% hold on;
% plot3(s2(:,1),s2(:,2),s2(:,3),'r.');
% plot3(p1(:,1),p1(:,2),p1(:,3),'ro');
% plot3(p2(:,1),p2(:,2),p2(:,3),'ro');

if all(p1==p2)
    disp('physical gap distance=0, setting dot products to 1');
    s2sdot=1;s1cdot=1;s2cdot=1;scdot=1;
else
    % get the index on the strand associated with each of p1 and p2
    [minp1s1,minp1s1idx]=min(sum((repmat(p1,size(s1,1),1)-s1).^2,2));
    [minp2s2,minp2s2idx]=min(sum((repmat(p2,size(s2,1),1)-s2).^2,2));
    if minp1s1idx>1; s1=flipud(s1); end
    if minp2s2idx>1; s2=flipud(s2); end
    % get s1 vector from endpoint to next point location
    try
        s1v1=s1(1,:)-s1(2,:);
    catch
        s1v1 = zeros(1,size(s1,2));
    end
        
    if all(s1v1==0) & (size(s1,1)>2)           % if first and 2nd points identical, find vector
        [s1v1] = get_nonzero_vector(s1)
    end

    % get s2 vector from endpoint to next point location
    try
        s2v1 = s2(2,:)-s2(1,:);
    catch
        s2v1 = zeros(1,size(s2,2));
    end
    
    if all(s2v1==0) & (size(s2,1)>2)            % if first and 2nd points identical, find vector
        [s2v1] = get_nonzero_vector(s2)
    end
    % compute dot product of vectors on either side of putative connection
    if ( (norm(s1v1)~=0) & (norm(s2v1)~=0) )
        s2sdot=sum(s1v1.*s2v1)/(norm(s1v1)*norm(s2v1));
    else
        s2sdot = 0;
    end
    % compute the connection vector
    p2pv=p2-p1;
    % compute dot product of connection and vector on s1 side of connection
    if ( (norm(s1v1)~=0) & (norm(p2pv)~=0) )
        s1cdot = sum(s1v1.*p2pv)/(norm(s1v1)*norm(p2pv));
    else
        s1cdot = 0;
    end
        
    % compute dot product of connection and vector on s2 side of connection
    if ( (norm(s2v1)~=0) & (norm(p2pv)~=0) )
        s2cdot = sum(s2v1.*p2pv)/(norm(s2v1)*norm(p2pv));
    else
        s2cdot = 0;
    end
    % average the 2 dot products
    scdot=.5*(s1cdot+s2cdot);
end

if any(isnan([s2sdot,scdot,s1cdot,s2cdot]))
    disp(['nan in dot products']);
    keyboard;
end

function [vnz] = get_nonzero_vector(s)
vnz=s(1,:)-s(2,:);
stranddepth=3;
try
    while ~all(vnz==0)
        vnz=s(1,:)-s(stranddepth,:);
    end
catch
    vnz=NaN;
end
