function output_parameters = fit_n_gaussians(input_parameters);
global mylimits;

%fit n gaussians
%first fit single gaussians one_by_one to maximal peaks
%  considering only a 2 half-stdev's from the peak.
%   the subtract the fitted gaussian, forcing positive
%next, analyze the remaining curve for the next gaussian
%these gaussians are used as seeds for the iterative fitting
%routine MULTI_FIT.m

cut_data = readparameters(input_parameters,'cut_data',[]);
max_gaussians = readparameters(input_parameters,'max_gaussians',3);
first_fit_std = readparameters(input_parameters,'first_fit_std',2);
median_filter_size = readparameters(input_parameters,'median_filter_size',5);
min_amplitude_factor = readparameters(input_parameters,'min_amplitude_factor',0.1);
autohist_bins = readparameters(input_parameters,'autohist_bins',100);

% % X = [1:autohist_bins];
% % y = histc(cut_data,X)';
% % 
% % [X,y] = autohist(cut_data,autohist_bins);
% % whos X y

X = [1:length(cut_data)]';
y = cut_data;

X = X';
y = y';
yfilt = medfilt1(y,median_filter_size);
xmax = max(X);
xmin = min(X);


n = 1;
ynew = yfilt;
keep_going = 1;
clear betan;
while keep_going == 1,
    %fit a single gaussian to the peak of what's left
    maxval = max(ynew);
    maxpos = mean(X(find(ynew==maxval)));
    temp1 = cut_data(find(cut_data>=maxpos));
    temp2 = cut_data(find(cut_data<=maxpos));
    tempstd = first_fit_std * min(std(temp1),std(temp2));
    x1 = maxpos-tempstd; x2 = maxpos+tempstd;
    ycut = ynew;
    ycut(find(X<x1)) = 0;
    ycut(find(X>x2)) = 0;
    %figure(7); plot(X,ycut);
    
    A1 = maxval; xo1 = maxpos; s1 = tempstd;
    beta0 = [A1 xo1 s1];
    [beta1,r,j] = nlinfit(X,ycut,@single_gaussian_function,beta0);
    betan(n,:) = beta1;

    %figure(3); plot(X,ynew);
    %hold on
    g = single_gaussian_function(beta1,X);
    %figure(3);plot(X,g,'k-');
    %plot(X,g,'g-');
    %hold off
    %pause(1);
    
    ynew = max(0,ynew - g);
    
    if n==1,
        max_amp = max(g);
        n = n +1;
    else
        current_amp = max(g);
        if current_amp < min_amplitude_factor * max_amp,
            keep_going = 0;
        else
            n = n + 1;
        end
    end
    if n>max_gaussians, keep_going = 0; end
end%while

g = zeros(size(y));
figure(2); plot(X,y,'b-'); hold on
for k = 1:n-1,
    gn = single_gaussian_function(betan(k,:),X);
    g = g + gn;
    plot(X,gn,'r-');
end
plot(X,g,'k-');
hold off
pause(1);




%Now allow each gaussian to be fit iteratively

mycoeff = betan(1:n-1,:);
mylimits = [max(betan(:,1))*0.1, max(betan(:,1))*2,...
    X(1), X(length(X)),...
    2*(X(2)-X(1)) ,(0.25/sqrt(2))*(X(length(X))-X(1))];

myparameters.dataX = X;
myparameters.dataY = yfilt;
myparameters.seed_coeff = mycoeff;
%myparameters.function_handle = @mygaussian_with_limits;
myparameters.function_handle = @mygaussian;
myparameters.max_iterations = 500;
myparameters.residual_limit = 1e-50;
myparameters.display_curves = 'yes';
myparameters.manual_pause = 'no';
myparameters.display_result = 'yes';
output_parameters = multi_fit(myparameters);
pause(1);

%Make high-resolution versions of output curves
fit_coeff = output_parameters.fit_coeff;
[num_curves,junk] = size(fit_coeff);
left_curve = find(fit_coeff(:,2) == min(fit_coeff(:,2)));
X2 = linspace(min(X),max(X),1000*length(X));
g2 = zeros(size(X2));
for k = 1:num_curves;
    if k~=left_curve,
        g2 = g2 + mygaussian(fit_coeff(k,:),X2);
    end%if
end%for k = 1:num_curves;
g1 = mygaussian(fit_coeff(left_curve,:),X2);

%Determine Crossing point of two curves
g_diff = g1-g2;
g_diff_sign = sign(g_diff);
g_zero_cross = diff(g_diff_sign);
where_cross = find(g_zero_cross~=0);
switch length(where_cross),
    case 0,
        crossing_point = fit_coeff(1,2); %use peak of left gaussian
    case 1,
        crossing_point = X2(where_cross);
    case 2,
         left_max_pos = find(g1==max(g1));
         cross_on_right = find(where_cross>left_max_pos(1));
         switch length(cross_on_right)
             case 1
                 crossing_point = X2(where_cross(2));
             case 2
                 crossing_point = X2(where_cross(1));
             otherwise
                 crossing_point = ( fit_coeff(1,2) + fit_coeff(2,2) )./2
         end %switch length(cross_on_right);
end %switch length(where_cross);

output_parameters = input_parameters;
output_parameters.g1 = g1;
output_parameters.g2 = g2;
output_parameters.crossing_point = crossing_point;
    


% % % 
% % % figure(8); plot(X,g1,'r-');
% % % hold on; plot(X,g2,'k-');
% % % hold off


return






function g = single_gaussian_function(mycoeff,myx);
global pos_cutoff
global xmax xmin
global max_std1;
global maxpos

    A1 = abs(mycoeff(1));
    xo1 = mycoeff(2);
    s1 = abs(mycoeff(3));
        
    g = A1.*exp(-(myx-xo1).^2 / (2*(s1.^2)));
    
return



function g = n_gaussians_function(mycoeff,myx);
global pos_cutoff
global xmax xmin
global max_std1;
global maxpos

[myn,junk] = size(mycoeff);
g = zeros(size(myx));
for k = 1:myn;
    An = abs(mycoeff(k,1));
    xon = mycoeff(k,2);
    sn = abs(mycoeff(k,3));
    termn = An.*exp(-(-myx-xon).^2 / (2*(sn.^2)));
    g = g + termn;
end
    
return



