function outputStructure = MultiThreadClassicParentVidaSuite(varargin)

%if input variable is a string, assume it is the name of the .m setup
%function file to run. - the strings "current" and "default" default to
%using the setup function: SuperVidaParamatersCurrent.m
%if input variable is a structure, assume the call is from the GUI
%and the setup parameters are stored in a .mat file
outputStructure = [];
nin = nargin;
if nin == 0,
    commandVariable = 'current';
else%if nin==0
    commandVariable = varargin{1};
end%if nin==0
%---
% % % if isstruct(commandVariable),
% % %     calledFromGuiFlag = 1;
% % %     setupMatFileName = commandVariable.setupMatFileName; 
% % %     parameterSetupFunctionName = 'current';
% % % else%if isstruct(commandVariable)
% % %     calledFromGuiFlag = 0;
% % %     parameterSetupFunctionName = commandVariable;
% % % end%if-else isstruct(commandVariable)
% % % %---

parameterSetupFunctionName = commandVariable;

switch parameterSetupFunctionName
    case {'default','Default','DEFAULT','def','Def','DEF'};
        parameterSetupFunctionName = 'DefaultClassicParameters.m';
    case {'current','CURRENT','Current'}
        parameterSetupFunctionName = 'DefaultClassicParameters.m';
    otherwise
        parameterSetupFunctionName = parameterSetupFunctionName;
end%switch parameterSetupFunctionName
%-----------
%Automatically detect rootCodeFolder and add setupFunctions
rootCodeFolder = fileparts(which('ClassicVidaSuite'));
addpath(fullfile(rootCodeFolder,'SetupFunctionsFolder'))
%-----
    outputProject = VidaAutoDectectVidaProjectRootFolder([]);
vidaProjectRootName = outputProject.vidaProjectRootName;
vidaProjectRootFolder = outputProject.vidaProjectRootFolder;
%----------
%SetupParameters
defaultOverwriteFilesStateFlag = 0;
vidaProjectStyleFlag = 1;
useTifStorageFlag = 0; % use variable imageStack in Mat file
skipAutoUpgradeFlag = 0;
debugModeAvoidOverwriteFlag = 1; %set to 1 to avoid overwriting all files experiment file is deleted

deleteAfterCompletedFlags.deleteOffsetTif = 1;
deleteAfterCompletedFlags.deleteSubBlockFiles = 1;
deleteAfterCompletedFlags.deleteVascBlockChannelSubFiles = 1;
deleteAfterCompletedFlags.deleteNeunBlockChannelSubFiles = 1;
deleteAfterCompletedFlags.deleteCellSeg = 1;
deleteAfterCompletedFlags.deleteVascSegFiles = 0;
deleteAfterCompletedFlags.deleteCenterlineFiles = 0;
deleteAfterCompletedFlags.deleteVascRadiiFiles = 0;

createTifFlags.createRawMaskTif = 0;
createTifFlags.createEnhMaskTif = 0;
createTifFlags.createCenterlineMaskTif = 0;

%-----

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp([' ';' ';' ']);
disp(['VIDA Script Started at ',datestr(now)]);
disp([' ';' ';' ']);
%--------------------------------------------------------------------------
multiThreadParentRunningFlagFileName = fullfile(vidaProjectRootFolder,[vidaProjectRootName,'-MultiThreadParentRunningFlag.mat']);
%-----
%Check if another Parent is running
if exist(multiThreadParentRunningFlagFileName,'file'),
    myWarning = 'Another Parent Process appears to be running on this dataset.  Proceed?';
    proceedFlag = questdlg(myWarning,'Parent Duplicate Warning','Proceed','Cancel','Cancel');
    pause(1);
    switch proceedFlag,
        case {'Cancel'}
            return%exit back to calling function (VIDA)
        otherwise
            disp('Proceeding with potential duplicate Parent process');
    end%switch proceedFlag
else%if exist(multiThreadParentRunningFlagFileName,'file'),
    multiThreadParentRunningFlag = 1;
    save(multiThreadParentRunningFlagFileName,'multiThreadParentRunningFlag');
end%if-else exist(multiThreadParentRunningFlagFileName,'file'),

%-----


vidaProject.vidaProjectRootName = vidaProjectRootName;
vidaProject.vidaProjectRootFolder = vidaProjectRootFolder;
vidaProject.rootCodeFolder = rootCodeFolder;
%---
vidaProject.progressFlags.initialized = 1;
vidaProject.progressFlags.sortOriginalTifs = 0;
vidaProject.progressFlags.createUniformityFilter = 0;
vidaProject.progressFlags.processImageStacksCompleted = 0;
vidaProject.progressFlags.multiStackProcessingCompleted = 0;


%Initialize the VidaProject file with folder names and setup Parameters
%This step initializes the vidaProject structure, but does NOT save to disk
vidaProject.setupParameters.vidaProjectRootName = vidaProjectRootName;
vidaProject.setupParameters.vidaProjectRootFolder = vidaProjectRootFolder;
vidaProject.setupParameters.parameterSetupFunctionName = parameterSetupFunctionName;
vidaProject.setupParameters.defaultOverwriteFilesStateFlag = defaultOverwriteFilesStateFlag;
vidaProject.setupParameters.useTifStorageFlag = useTifStorageFlag;
vidaProject.setupParameters.skipAutoUpgradeFlag = skipAutoUpgradeFlag;
vidaProject.setupParameters.defaultOverwriteFilesStateFlag = defaultOverwriteFilesStateFlag;
    vidaProject = VidaSetupVidaProjectFile(vidaProject);
revisedCodeFolders = vidaProject.setupParameters.revisedCodeFolders;
vidaProjectFileName = vidaProject.vidaProjectFileName;

%-----
experimentTemplate = vidaProject;
experimentTemplate.deleteAfterCompletedFlags = deleteAfterCompletedFlags;
experimentTemplate.createTifFlags = createTifFlags;
%-----

% experimentTemplate.globalParameters.storePixelListInExperimentFlag = 1;%noteToSelf
%-----
vidaProject.experimentTemplate = experimentTemplate;


%Add all code Folder to Matlab path
codeFolders = experimentTemplate.codeFolders;
codeFolderList = fieldnames(experimentTemplate.codeFolders);
numCodeFolders = size(codeFolderList,1);
for codeFolderIter = 1:numCodeFolders,
    currentField = codeFolderList{codeFolderIter};
    currentCodePath = experimentTemplate.codeFolders.(currentField);
    addpath(currentCodePath)
end
    


%LOAD PARAMETERS
%this step further initializes the vidaProject, but does NOT save to disk
%first, check to see if parameterSetupFunctionName is a .m file or .mat file
continueFlag = 0;
if strcmpi(parameterSetupFunctionName(end-1:end),'.m');
    [junkPath,fileNameOnly] = fileparts(parameterSetupFunctionName);
    loadParamCommand = ['experimentTemplate = ',fileNameOnly,'(experimentTemplate);'];
    eval(loadParamCommand)
    disp(['LOADING PARAMETER SET: ',parameterSetupFunctionName]);
    continueFlag = 1;
end%if parameterSetupFunctionName(end-1:end) == '.m',
%---
if strcmpi(parameterSetupFunctionName(end-3:end),'.mat'),
    load(parameterSetupFunctionName,'experimentTemplate');
    continueFlag = 1;
end%if parameterSetupFunctionName(end-3:end) == '.mat',
%---
if continueFlag == 0,
    disp('NOT A VALID PARAMETER SETUP FILE');
    keyboard
end

experimentTemplate.globalParameters = experimentTemplate.basicParameters;
vidaProject.globalParameters = experimentTemplate.basicParameters;
vidaProject.experimentTemplate = experimentTemplate;

%Some specifc global-scale parameters may have been overwritten when the generic experiment Template
%was loaded.  Re-write these specific parameters to the experiment_Template
experimentTemplate.vidaProjectRootName = vidaProjectRootName;
experimentTemplate.vidaProjectRootFolder = vidaProjectRootFolder;
experimentTemplate.rootCodeFolder = rootCodeFolder;
experimentTemplate.setupParameters.vidaProjectRootName = vidaProjectRootName;
experimentTemplate.setupParameters.vidaProjectRootFolder = vidaProjectRootFolder;
experimentTemplate.setupParameters.parameterSetupFunctionName = parameterSetupFunctionName;
experimentTemplate.setupParameters.defaultOverwriteFilesStateFlag = defaultOverwriteFilesStateFlag;
experimentTemplate.setupParameters.useTifStorageFlag = useTifStorageFlag;
experimentTemplate.setupParameters.skipAutoUpgradeFlag = skipAutoUpgradeFlag;
experimentTemplate.setupParameters.defaultOverwriteFilesStateFlag = defaultOverwriteFilesStateFlag;
experimentTemplate.setupParameters.revisedCodeFolders = revisedCodeFolders;
experimentTemplate.vidaProjectFileName = vidaProjectFileName;
%---
experimentTemplate.deleteAfterCompletedFlags = deleteAfterCompletedFlags;
experimentTemplate.createTifFlags = createTifFlags;
experimentTemplate.codeFolders = codeFolders;
%-----
%-----



%AutoAdjust the rootfolder in any existing VidaProject file in case folder has moved
%this step saves the changes in the vidaProject file to disk
    vidaProject = VidaAutoAdjustVidaProjectRootFolder(vidaProject);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%DATE STAMP VIDAPROJECT FILE OR LOAD EXISTING VIDAPROJECT FILE
%This step overloads the iniatilzed vidaProject file if one exists on disk
vidaProject.setupParameters.debugModeAvoidOverwriteFlag = debugModeAvoidOverwriteFlag;
    vidaProject = VidaLoadVidaProjectFile(vidaProject);
vidaProject.codeFolders = revisedCodeFolders;
experimentTemplate = vidaProject.experimentTemplate;
experimentTemplate.codeFolders = revisedCodeFolders;
%-----
vidaProject.experimentTemplate = experimentTemplate;
%----------
%Copy the parameters relating to the full Project to the vidaProject structure
if ~isfield(vidaProject,'registrationParameters'),
    vidaProject.registrationParameters = experimentTemplate.registrationParameters;
end%if ~isfield(vidaProject,'registrationParameters'),
%---
if ~isfield(vidaProject,'strandifyParameters'),
    vidaProject.strandifyParameters = experimentTemplate.strandifyParameters;
end%if ~isfield(vidaProject,'strandifyParameters'),
%---
saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save(vidaProjectFileName,'vidaProject',scs);
preMatRoot = vidaProjectFileName(1:findstr(vidaProjectFileName,'.mat')-1);
save([preMatRoot,'Backup',datestr(now,'mmddyy-HHMM'),'.mat'],'vidaProject',scs);
%----------
%--------
%STAGE 1 : Determine number of image stacks - allocate each to a folder
%   SORT .TIF FILES INTO INDIVIDUAL FOLDERS IF NECESSARY
disp('Sorting Files');
if vidaProject.progressFlags.sortOriginalTifs == 0,
    vidaProject.globalParameters.reverseXBlockNumberingFlag = vidaProject.registrationParameters.reverseXBlockNumberingFlag;
    vidaProject.globalParameters.reverseYBlockNumberingFlag = vidaProject.registrationParameters.reverseYBlockNumberingFlag;
        vidaProject = VidaSortOriginalTifs(vidaProject);
    vidaProject.progressFlags.sortOriginalTifs = 1; 
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
end%if vidaProject.progressFlags.sortOriginalTifs == 0,
imageStackFolderNameList = vidaProject.imageStackFolderNameList;
numImageStacks = size(imageStackFolderNameList,2);
%--------
%--------
%STAGE 2 : Create uniformity filter for preprocessing
disp('Uniformity files');
if vidaProject.progressFlags.createUniformityFilter == 0,
        vidaProject = VidaCreateUniformityFilter(vidaProject);
    vidaProject.progressFlags.createUniformityFilter = 1;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
end%if vidaProject.progressFlags.createUniformityFilter == 0,
%--------
%--------
%STAGE 3 : Create MultiThreadProgress Structure
%First check for a MultiThreadParentRunningFlagFile.  If exists, exit program, otherwise:
%Create a MultiThreadProgress file to track and control multithread progress.
%   flags to track current state: initializing, imageStackProcessing, registering, stitching, subBlocking,
%       subBlockProcessing, subBlockRestitching,strandifying.
%   structure for each multi-thread segment with fields:
%       ID, checkedOutFlag, checkedOutByString, checkedOutDateString, completedFlag,
%Create a MultiThreadAccess file to quickly turn on and off access to the larger MultiThreadProgress file
%   to avoid simultaneous read/writes to the larger file. 
%-----
if ~isfield(vidaProject,'multiThreadProgress'),
    multiThreadProgress = [];
    %progressFlags: 0 = not yet, 1 = currently running; 2 = completed.
    multiThreadProgress.initializingFlag = 1;
    multiThreadProgress.imageStackProcessingFlag = 0;
    multiThreadProgress.registeringFlag = 0;
    multiThreadProgress.stitchingFlag = 0;
    multiThreadProgress.subBlockingFlag = 0;
    multiThreadProgress.subBlockProcessingFlag = 0;
    multiThreadProgress.subBlockRestitchingFlag = 0;
    multiThreadProgress.srandifyingFlag = 0;
    vidaProject.multiThreadProgress = multiThreadProgress;
end%if ~isfield(vidaProject,'multiThreadProgress'),
%-----
multiThreadRequestFolder = fullfile(vidaProjectRootFolder,'MultiThreadRequestFiles');
if ~exist(multiThreadRequestFolder,'file'),
    [mds,mdm,mdmid] = mkdir(vidaProjectRootFolder,'MultiThreadRequestFiles');
end%if ~exist(multiThreadRequestFolder,'file'),
vidaProject.multiThreadProgress.multiThreadRequestFolder = multiThreadRequestFolder;
vidaProject.multiThreadProgress.initializingFlag = 2;
%-----
%Save the VidaProject so that the multiThreadParameters are saved
if vidaProject.multiThreadProgress.imageStackProcessingFlag == 0,
    vidaProject.multiThreadProgress.imageStackProcessingFlag = 1;
end%if vidaProject.multiThreadProgress.imageStackProcessingFlag == 0,
%-----
%-----
%STAGE 4 : Check out an individual image stack, and run processing.
if vidaProject.multiThreadProgress.imageStackProcessingFlag == 2,
    fprintf('\nProcessing of individual image stacks previously completed\n\n');
else%if vidaProject.multiThreadProgress.imageStackProcessingFlag == 2,  
    fprintf('\nBegin Processing of individual image stacks\n\n');
    vidaProject.multiThreadProgress.imageStackProcessingFlag = 1;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
        vidaProject = VidaMultiThreadImageStackParent(vidaProject);
    vidaProject.multiThreadProgress.imageStackProcessingFlag = 2;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
end%if vidaProject.multiThreadProgress.imageStackProcessingFlag == 2,
    
%STAGE 5 : Sort Stack Order if necessary
tempFolderNameList = vidaProject.imageStackFolderNameList;
rczMax = vidaProject.globalParameters.rczMax;
rczMin = vidaProject.globalParameters.rczMin;
zList = [rczMin(1):rczMax(1)];
rList = [rczMin(2):rczMax(2)];
cList = [rczMin(3):rczMax(3)];



%STAGE 6 : Collect necessary filenames
disp('Collecting filenames');
if ~isfield(vidaProject.registrationParameters,'blockRegistration'),
    pathDelim = '\';if isunix; pathDelim = '/';end %make platform independent - Pablo
    for stackIter = 1:numImageStacks,
        charCount = fprintf(num2str(stackIter));
        tempMainFolderName = vidaProject.imageStackFolderNameList{stackIter};
        tempStackName = tempMainFolderName(findstr(tempMainFolderName,[pathDelim 'ImageStack-'])+12:end);
        tempProcessedDataFolderName = fullfile(tempMainFolderName,'ProcessedData');
        tempExperimentFileName = fullfile(tempProcessedDataFolderName,[tempStackName,'-Experiment.mat']);
        load(tempExperimentFileName,'experiment');
        vidaProject.globalParameters.vascGrayscaleFileNameList{stackIter} = experiment.globalParameters.vascGrayscaleFileName;
        vidaProject.globalParameters.vascEnhancedGrayscaleFileNameList{stackIter} = experiment.globalParameters.vascEnhancedGrayscaleFileName;
        vidaProject.globalParameters.vascEnhancedMaskFileNameList{stackIter} = experiment.globalParameters.vascEnhancedMaskFileName;
        vidaProject.globalParameters.vascCenterlineMaskFileNameList{stackIter} = experiment.globalParameters.vascCenterlineMaskFileName;
        vidaProject.globalParameters.vascReconstructedMaskFileNameList{stackIter} = experiment.globalParameters.vascReconstructedMaskFileName;
        vidaProject.globalParameters.vascRadiusMapFileNameList{stackIter} = experiment.globalParameters.vascRadiusMapFileName;
        vidaProject.globalParameters.stackSizeList(stackIter,:) = experiment.globalParameters.fullStackSize;
        fprintf(repmat('\b',[1,charCount]));
    end%for stackIter = 1:numImageStacks,
end%if ~isfield(vidaProject.registrationParameters,'blockRegistration'),


%Create a subFolder for registered results
rootDataFolder = vidaProject.vidaProjectRootFolder;
registeredFilesFolder = fullfile(rootDataFolder,'RegisteredFiles');
vidaProject.globalParameters.registeredFilesFolder = registeredFilesFolder;
if ~exist(registeredFilesFolder,'dir'),
    mkdir(registeredFilesFolder);
end
%-----
%Create a sub-directory for adjacent pair registration results
adjacentPairRegistrationDataFolder = fullfile(vidaProjectRootFolder,'RegistrationDataFiles');
if ~exist(adjacentPairRegistrationDataFolder,'file'),
    [mds,mdm,mdmid] = mkdir(vidaProjectRootFolder,'RegistrationDataFiles');
end%if ~exist(adjacentPairRegistrationDataFolder,'file'),
vidaProject.globalParameters.adjacentPairRegistrationDataFolder = adjacentPairRegistrationDataFolder;
%-----
saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save(vidaProjectFileName,'vidaProject',scs);



%-----
%STAGE 7 : Register multiple stacks into single global coordinate sytem.
if vidaProject.multiThreadProgress.registeringFlag == 2,
    fprintf('\nRegistration of adjacent pairs previously completed\n\n');
else%if vidaProject.multiThreadProgress.registeringFlag == 2,
    fprintf('\nBegin Processing of adjacent pairs\n\n');
    vidaProject.multiThreadProgress.registeringFlag = 1;
        vidaProject = VidaRegisterBlocksParent(vidaProject);
    vidaProject.multiThreadProgress.registeringFlag = 2;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
end%if-else vidaProject.multiThreadProgress.registeringFlag == 2,


%-----
%STAGE 8 : Choose a Multi-stack processing pathway, and run.
registeredPathway = 'StitchCenterlineStrandify';%noteToSelf
switch registeredPathway,
    case{'StitchCenterlineStrandify','StitchCenterlineStrandify'}
        vidaProject = VidaRegisteredProcessStitchCenterlineStrandifyParent(vidaProject);
    case{'ConcatenateStrandify'}
        vidaProject = VidaRegisteredProcessConcatenateStrandify(vidaProject);
end%switch multiStackPathway
saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save(vidaProjectFileName,'vidaProject',scs);
%-----
if exist(multiThreadParentRunningFlagFileName),
    delete(multiThreadParentRunningFlagFileName);
end%if exist(multiThreadParentRunningFlagFileName),
%-----

disp(['SuperVidaSuite Completed at ',datestr(now)]);
outputStructure.vidaProjectFileName = vidaProjectFileName;






