function outputParameters = VidaNeunMakeHalfSizeVisualizationTif(inputParameters)


intermediateResultsFolder = inputParameters.intermediateResultsFolder;
cellCentroidAndLabelList = inputParameters.cellCentroidAndLabelList;
useTifStorageFlag = inputParameters.useTifStorageFlag;
stackedFileNames = inputParameters.stackedFileNames;
vascMaskSize = inputParameters.vascMaskSize;
%-----


%MAKE GIANT TIFF OF NEURONS VS NON_NEURONS
disp(['Creating Giant Cell .tif files : ',datestr(now,'mm/dd/yy HH:MM:SS')]);tic;

HORDfname = stackedFileNames{1};
NEUNfname = stackedFileNames{3};

%----
ysize = ceil(vascMaskSize(1)./2);
xsize = ceil(vascMaskSize(2)./2);
zsize = ceil(vascMaskSize(3)./2);
halfCellCentroidAndLabelList = cellCentroidAndLabelList;
halfCellCentroidAndLabelList(:,1:3) = ceil(halfCellCentroidAndLabelList(:,1:3)./2);
%-----
if useTifStorageFlag ~= 1,
    load(HORDfname,'tiledStack');
%    tiledStack = imfilter(tiledStack,ones(3,3,3))./27;
    tiledStackHORD = tiledStack(1:2:end,1:2:end,1:2:end);
    clear tiledStack;
    load(NEUNfname,'tiledStack');
%     tiledStack = imfilter(tiledStack,ones(3,3,3))./27;
    tiledStackNEUN = tiledStack(1:2:end,1:2:end,1:2:end);
    clear tiledStack;
end%if useTifStorageFlag ~= 1,


writeMode = 'overwrite';
for frameIter = 1:zsize;
    if useTifStorageFlag == 1,
        NFrame = imread(NEUNfname,'tiff',frameIter);
        NFrame = NFrame(1:2:end,1:2:end,:);
        HFrame = imread(HORDfname,'tiff',frameIter);
        HFrame = HFrame(1:2:end,1:2:end,:);
    else%if useTifStorageFlag == 1,
        NFrame = tiledStackNEUN(:,:,frameIter);
        HFrame = tiledStackHORD(:,:,frameIter);
    end%if useTifStorageFlag == 1,
    neuronMask = logical(0.*HFrame);
    nonNeuronMask = logical(0.*HFrame);

    extraZExtent = 2;
    whereCells = find(halfCellCentroidAndLabelList(:,3)>=frameIter-extraZExtent & halfCellCentroidAndLabelList(:,3)<=frameIter+extraZExtent);
    if ~isempty(whereCells),
        cellList = ceil(halfCellCentroidAndLabelList(whereCells,:));
        neuronList = cellList((cellList(:,4)==1),1:3);
        nonNeuronList = cellList(~cellList(:,4),1:3);
        extraXYExtent = 0;
        for k = 1:size(nonNeuronList,1),
            y1 = max(1,nonNeuronList(k,1)-extraXYExtent);
            y2 = min(ysize,nonNeuronList(k,1)+extraXYExtent+1);
            x1 = max(1,nonNeuronList(k,2)-extraXYExtent);
            x2 = min(xsize,nonNeuronList(k,2)+extraXYExtent+1);
            nonNeuronMask(y1:y2,x1:x2) = 1;
        end
        for k = 1:size(neuronList,1),
            y1 = max(1,neuronList(k,1)-extraXYExtent);
            y2 = min(ysize,neuronList(k,1)+extraXYExtent+1);
            x1 = max(1,neuronList(k,2)-extraXYExtent);
            x2 = min(xsize,neuronList(k,2)+extraXYExtent+1);
            neuronMask(y1:y2,x1:x2) = 1;
        end
    end    
            
% %     HFrame = imfilter(HFrame,ones(3,3),'replicate')./9;
% %     NFrame = imfilter(NFrame,ones(3,3),'replicate')./9;

        neuronMaskFrame = neuronMask;
        nonNeuronMaskFrame = nonNeuronMask;
        bothMaskFrame = or(neuronMaskFrame,nonNeuronMaskFrame);
      
        
        HORDFrame = double(HFrame);
        NEUNFrame = double(NFrame);
        HORDFrame = HORDFrame./1204;
        HORDFrame = HORDFrame.^0.25;
        HORDNormFrame = min(HORDFrame.*312 , 256);
        NEUNFrame = NEUNFrame./2048;
        NEUNFrame = NEUNFrame.^0.2;
        NEUNNormFrame = min(NEUNFrame.*256 , 256);


        factor = 0.8;
        semiBoth = uint8([HORDNormFrame.*factor.*(bothMaskFrame) , NEUNNormFrame.*factor.*(bothMaskFrame)]);
        semi_n = uint8([HORDNormFrame.*factor.*(neuronMaskFrame) , NEUNNormFrame.*factor.*(neuronMaskFrame)]);
        semi_nn = uint8([HORDNormFrame.*factor.*(nonNeuronMaskFrame) , NEUNNormFrame.*factor.*(nonNeuronMaskFrame)]);

        ChR = uint8([HORDNormFrame.*(~nonNeuronMaskFrame) , NEUNNormFrame.*(~nonNeuronMaskFrame)])+semi_nn;
        ChG = uint8([HORDNormFrame.*(~neuronMaskFrame) , NEUNNormFrame.*(~neuronMaskFrame)])+semi_n;
        ChB = uint8([HORDNormFrame.*(~bothMaskFrame) , NEUNNormFrame.*(~bothMaskFrame)]) + semiBoth;

        ChRCentroids = uint8([neuronMaskFrame*255,neuronMaskFrame*255]);
        ChR = max(ChR,ChRCentroids);
        ChGCentroids = uint8([nonNeuronMaskFrame*255,nonNeuronMaskFrame*255]);
        ChG = max(ChG,ChGCentroids);
        
        
        ChR(:,xsize) = 255;
        ChG(:,xsize) = 255;
        ChB(:,xsize) = 255;

        RGB = cat(3,cat(3,ChR,ChG),ChB);
 

            ChR2 = uint8([HORDNormFrame , NEUNNormFrame]);
            ChG2 = uint8([HORDNormFrame , NEUNNormFrame]);
            ChB2 = uint8([HORDNormFrame , NEUNNormFrame]);
             
            RGBOriginal = cat(3,cat(3,ChR2,ChG2),ChB2);
            RGBOriginal(:,xsize,:) = 255;
            
            imwrite(RGB,fullfile(intermediateResultsFolder,'RGBHalfSize_alt.tif'),'tiff','WriteMode',writeMode,'Compression','none','Resolution',1);
            writeMode = 'append';
            imwrite(RGBOriginal,fullfile(intermediateResultsFolder,'RGBHalfSize_alt.tif'),'tiff','WriteMode',writeMode,'Compression','none','Resolution',1);
            
end%for frameIter = 1:zsize

outputParameters = inputParameters;
