function outputParameters = VidaNeunMakeVisualizationTif(inputParameters)


intermediateResultsFolder = inputParameters.intermediateResultsFolder;
cellCentroidAndLabelList = inputParameters.cellCentroidAndLabelList;
stackedFileNames = inputParameters.stackedFileNames;
vascMaskSize = inputParameters.vascMaskSize;
%-----

%MAKE GIANT TIFF OF NEURONS VS NON_NEURONS
disp(['Creating Giant Cell .tif files : ',datestr(now,'mm/dd/yy HH:MM:SS')]);tic;

HORDfname = stackedFileNames{1};
NEUNfname = stackedFileNames{3};

%----
ysize = vascMaskSize(1);
xsize = vascMaskSize(2);
zsize = vascMaskSize(3);
%-----
load(HORDfname,'tiledStack');
tiledStackHORD = tiledStack(:,:,1:2:end);
clear tiledStack
load(NEUNfname,'tiledStack');
tiledStackNEUN = tiledStack(:,:,1:2:end);
clear tiledStack


writeMode = 'overwrite';
for frameIter = 1:2:zsize;
    NFrame = tiledStackNEUN(:,:,(frameIter+1)/2);
    HFrame = tiledStackHORD(:,:,(frameIter+1)/2);
    neuronMask = logical(0.*HFrame);
    nonNeuronMask = logical(0.*HFrame);

    whereCells = find(cellCentroidAndLabelList(:,3)>=frameIter-3 & cellCentroidAndLabelList(:,3)<=frameIter+3);
    if ~isempty(whereCells),
        cellList = cellCentroidAndLabelList(whereCells,:);
        neuronList = cellList((cellList(:,4)==1),1:3);
        nonNeuronList = cellList(~cellList(:,4),1:3);
        extraZExtent = 1;
        for k = 1:size(nonNeuronList,1),
            y1 = max(1,nonNeuronList(k,1)-extraZExtent);
            y2 = min(ysize,nonNeuronList(k,1)+extraZExtent+1);
            x1 = max(1,nonNeuronList(k,2)-extraZExtent);
            x2 = min(xsize,nonNeuronList(k,2)+extraZExtent+1);
            nonNeuronMask(y1:y2,x1:x2) = 1;
        end
        for k = 1:size(neuronList,1),
            y1 = max(1,neuronList(k,1)-extraZExtent);
            y2 = min(ysize,neuronList(k,1)+extraZExtent+1);
            x1 = max(1,neuronList(k,2)-extraZExtent);
            x2 = min(xsize,neuronList(k,2)+extraZExtent+1);
            neuronMask(y1:y2,x1:x2) = 1;
        end
    end    
            
    HSmoothedFrame = imfilter(HFrame,ones(3,3),'replicate')./9;
    NSmoothedFrame = imfilter(NFrame,ones(3,3),'replicate')./9;

        neuronMaskFrame = neuronMask;
        nonNeuronMaskFrame = nonNeuronMask;
        bothMaskFrame = or(neuronMaskFrame,nonNeuronMaskFrame);
        HORDFrame = double(HSmoothedFrame);
        NEUNFrame = double(NSmoothedFrame);
        HORDFrame = HORDFrame./1204;
        HORDFrame = HORDFrame.^0.25;
        HORDNormFrame = min(HORDFrame.*312 , 256);
        NEUNFrame = NEUNFrame./2048;
        NEUNFrame = NEUNFrame.^0.2;
        NEUNNormFrame = min(NEUNFrame.*256 , 256);


        factor = 0.5;
        semiBoth = uint8([HORDNormFrame.*factor.*(bothMaskFrame) , NEUNNormFrame.*factor.*(bothMaskFrame)]);
        semi_n = uint8([HORDNormFrame.*factor.*(neuronMaskFrame) , NEUNNormFrame.*factor.*(neuronMaskFrame)]);
        semi_nn = uint8([HORDNormFrame.*factor.*(nonNeuronMaskFrame) , NEUNNormFrame.*factor.*(nonNeuronMaskFrame)]);

        ChR = uint8([HORDNormFrame.*(~nonNeuronMaskFrame) , NEUNNormFrame.*(~nonNeuronMaskFrame)])+semi_nn;
        ChG = uint8([HORDNormFrame.*(~neuronMaskFrame) , NEUNNormFrame.*(~neuronMaskFrame)])+semi_n;
        ChB = uint8([HORDNormFrame.*(~bothMaskFrame) , NEUNNormFrame.*(~bothMaskFrame)]) + semiBoth;

        ChR(:,xsize) = 255;
        ChG(:,xsize) = 255;
        ChB(:,xsize) = 255;

        RGB = cat(3,cat(3,ChR,ChG),ChB);
 
% % %             RGBHalfSize = RGB(1:2:ysize,1:2:2*xsize,:);

            ChR2 = uint8([max(HORDNormFrame,double(neuronMaskFrame)*256) , max(NEUNNormFrame,double(neuronMaskFrame)*256)]);
            ChG2 = uint8([max(HORDNormFrame,double(nonNeuronMaskFrame)*256) , max(NEUNNormFrame,double(nonNeuronMaskFrame)*256)]);
            ChB2 = uint8([HORDNormFrame , NEUNNormFrame]);
            RGB2 = cat(3,cat(3,ChR2,ChG2),ChB2);
            RGB2(:,xsize,:) = 255;
            RGB2HalfSize  = RGB2;

            smoothedHORDFrame = double(HSmoothedFrame);
            smoothedNEUNFrame = double(NSmoothedFrame);
            smoothedHORDFrame = smoothedHORDFrame./1204;
            smoothedHORDFrame = smoothedHORDFrame.^0.25;
            smoothedHORDNormFrame = min(smoothedHORDFrame.*312 , 256);
            smoothedNEUNFrame = smoothedNEUNFrame./2048;
            smoothedNEUNFrame = smoothedNEUNFrame.^0.2;
            smoothedNEUNNormFrame = min(smoothedNEUNFrame.*256 , 256);
            smoothPic = uint8([smoothedHORDNormFrame , smoothedNEUNNormFrame]);

            RGBOriginal = cat(3,cat(3,smoothPic,smoothPic),smoothPic);
            RGBOriginal(:,xsize,:) = 255;
            RGBOriginalHalfSize = RGBOriginal;

                    imwrite(RGB2HalfSize,fullfile(intermediateResultsFolder,'RGBNeun_alt.tif'),'tiff','WriteMode',writeMode,'Compression','none','Resolution',1);
                    imwrite(RGBOriginalHalfSize,fullfile(intermediateResultsFolder,'RGBNeun_alt.tif'),'tiff','WriteMode',writeMode,'Compression','none','Resolution',1);
                    writeMode = 'append';
   
end%for frameIter = 1:zsize

outputParameters = inputParameters;
