function [Offset_minE,Emin,Eoffsets] = LocalOffsetSearch(B_center,OffsetSearch,Blocks,Offsets,MarginalOffsetEnergies,registrationDataFileNameList)

if isfield(MarginalOffsetEnergies,'Offset3DEnergy'),
    Offset3DEnergyStoredFlag = 1;
else%if isfield(MarginalOffsetEnergies,'Offset3DEnergy'),
    Offset3DEnergyStoredFlag = 0;
end%if-else isfield(MarginalOffsetEnergies,'Offset3DEnergy'),
%-----
%-----
% First initialize block with energy from block centers
adj_b = Blocks(B_center).all_block_pair_indices;
Center_ij = zeros(1,numel(adj_b));                            % First compute which block the center block is for each adjacent block index
for iAB = 1:numel(adj_b)
    BiBj_ind = adj_b(iAB);                                    % Get the block pair index for pairwise indexing
    BiBj = Offsets(BiBj_ind).offs_blocks;                     % Get the ordering of blocks for offset polarity bookkeeping
    Center_ij(iAB) = find(BiBj == B_center);                  % Identify which block plays the role of center for each pair
    if Offset3DEnergyStoredFlag == 0,
        load(registrationDataFileNameList{BiBj_ind},'adjacentPairEnergy');
        MarginalOffsetEnergies(BiBj_ind).Offset3DEnergy = adjacentPairEnergy.Offset3DEnergy;
    end%if Offset3DEnergyStoredFlag == 0,
end

Eoffsets = zeros(1,size(OffsetSearch,1));   % Preallocate offset energies
for iOS = 1:size(OffsetSearch,1)
    off_cand = OffsetSearch(iOS,:);         % get the first offset candidate
    E_iAB = zeros(1,numel(adj_b));
    for iAB = 1:numel(adj_b)
        BiBj_ind = adj_b(iAB);                                    % Get the block pair index for pairwise indexing
        phi_BiBj = Offsets(BiBj_ind).offs;                        % Look up the 3D offset that corresponds to 0 offset in MOES calculations
        BiBj = Offsets(BiBj_ind).offs_blocks;                     % Get the ordering of blocks for offset polarity bookkeeping
        Bi = BiBj(1);                                             % Identify block i
        Bj = BiBj(2);                                             % Identify block j
        if Center_ij(iAB) == 1  % B_center is Bi
            Delta_ij = (Blocks(Bi).corner_coords + off_cand) - Blocks(Bj).corner_coords - phi_BiBj + MarginalOffsetEnergies(BiBj_ind).ZeroOffsetLoc3D;
        else                    % B_center is Bj
            Delta_ij = Blocks(Bi).corner_coords - (Blocks(Bj).corner_coords + off_cand) - phi_BiBj + MarginalOffsetEnergies(BiBj_ind).ZeroOffsetLoc3D;
        end
        InBounds = all(Delta_ij <= size(MarginalOffsetEnergies(BiBj_ind).Offset3DEnergy)) & all(Delta_ij >= [1 1 1]);
        if InBounds
            E_iAB(iAB) = MarginalOffsetEnergies(BiBj_ind).Offset3DEnergy(Delta_ij(1),Delta_ij(2),Delta_ij(3));
        else
            % disp('CAUTION: at least one offset drifted out of bounds...');
            fprintf('\nOUT OF BOUNDS for blocks %d,%d',Bi,Bj);
            E_iAB(iAB) = inf;
        end
    end
    Eoffsets(iOS) = sum(E_iAB);
end
[Emin,Emin_ij] = min(Eoffsets);
Offset_minE = OffsetSearch(Emin_ij,:);
