function outputParameters = MaximizeProjectedBlockCorrelationSum(inputParameters)


blockFileNameList = inputParameters.blockFileNameList;
adjacentBlockPairList = inputParameters.adjacentBlockPairList;
adjacentBlockPairOffsetList = inputParameters.adjacentBlockPairOffsetList;
emptyBlockCorrectionFlag = inputParameters.emptyBlockCorrectionFlag;
emptyBlockThreshold = inputParameters.emptyBlockThreshold;
    
%path(path,'C:\Documents and Settings\kaufholdj\Desktop\ManipulatingNeuralTissue1\VidaSuite070709\GeneralFunctionsFolder');
%path(path,'C:\Documents and Settings\kaufholdj\Desktop\ManipulatingNeuralTissue1\GapConnectionsTesting2_071009');
%path(path,'C:\Documents and Settings\kaufholdj\Desktop\ManipulatingNeuralTissue1\BlockRegPrototype1\VolumeRendering');

%% COMPUTE CORNER LOCATIONS ASSUMING BLOCK 1 IS AT 1,1,1
% Preallocate corner locations
cornercoords1 = zeros(numel(blockFileNameList),1);
cornercoords2 = zeros(numel(blockFileNameList),1);
cornercoords3 = zeros(numel(blockFileNameList),1);

% Set the first corner to be at 1,1,1
cornercoords1(1) = 1;
cornercoords2(1) = 1;
cornercoords3(1) = 1;

% Estimate all other corner locations for initial config based on corner
% 1--assume all subsequent corner locations of blocks in the 2nd column of
% adjacentBlockPairList are defined by prior corner locations.
for iAB = 1:size(adjacentBlockPairList,1)
    cornercoords1(adjacentBlockPairList(iAB,2)) = cornercoords1(adjacentBlockPairList(iAB,1)) - adjacentBlockPairOffsetList(iAB,1);
    cornercoords2(adjacentBlockPairList(iAB,2)) = cornercoords2(adjacentBlockPairList(iAB,1)) - adjacentBlockPairOffsetList(iAB,2);
    cornercoords3(adjacentBlockPairList(iAB,2)) = cornercoords3(adjacentBlockPairList(iAB,1)) - adjacentBlockPairOffsetList(iAB,3);
end

%% GENERATE BLOCKCONFIG
% % % % % % % % % % Infer BlockConfig from adjacency and offsets (ASSUMES REGULAR GRID
% % % % % % % % % % OFFSETTING... If Offsets are not nearly regular, this indexing will not
% % % % % % % % % % work
% % % % % % % % % MaxAbsAdjacentOffsets = max(abs(adjacentBlockPairOffsetList));
% % % % % % % % % % Minimum offset determines what range of corner coordinates will lead to a
% % % % % % % % % % specific block configuration
% % % % % % % % % MinimumOffsetRequiredForBlockIndexing = 20;
% % % % % % % % % 
% % % % % % % % % if MaxAbsAdjacentOffsets(1) > MinimumOffsetRequiredForBlockIndexing
% % % % % % % % %     BlockInds1 = round(cornercoords1/MaxAbsAdjacentOffsets(1)) + 1;
% % % % % % % % % else
% % % % % % % % %     BlockInds1 = ones(size(cornercoords1));
% % % % % % % % % end
% % % % % % % % % 
% % % % % % % % % if MaxAbsAdjacentOffsets(2) > MinimumOffsetRequiredForBlockIndexing
% % % % % % % % %     BlockInds2 = round(cornercoords2/MaxAbsAdjacentOffsets(2)) + 1;
% % % % % % % % % else
% % % % % % % % %     BlockInds2 = ones(size(cornercoords2));
% % % % % % % % % end
% % % % % % % % % 
% % % % % % % % % if MaxAbsAdjacentOffsets(3) > MinimumOffsetRequiredForBlockIndexing
% % % % % % % % %     BlockInds3 = round(cornercoords3/MaxAbsAdjacentOffsets(3)) + 1;
% % % % % % % % % else
% % % % % % % % %     BlockInds3 = ones(size(cornercoords3));
% % % % % % % % % end

% Infer BlockConfig from adjacency and offsets (ASSUMES REGULAR GRID
% OFFSETTING ON DIMENSIONS 1 AND 2) 3rd dimension can be nonuniformly
% spaced (must be monotonic).
% Assumes grid forms a lattice aligned with 3 coordinate directions (i.e.
% nominal offsets do not skew (at all)). Some drift is acceptable, but
% total drift over any dimension cannot exceed the spacing for that
% direction.

MaxAbsAdjacentOffsets = max(abs(adjacentBlockPairOffsetList),[],1);
% Minimum offset determines what range of corner coordinates will lead to a
% specific block configuration
MinimumOffsetRequiredForBlockIndexing = 15;

if MaxAbsAdjacentOffsets(1) > MinimumOffsetRequiredForBlockIndexing
    BlockInds1 = round(cornercoords1/MaxAbsAdjacentOffsets(1)) + 1;
else
    BlockInds1 = ones(size(cornercoords1));
end


if MaxAbsAdjacentOffsets(2) > MinimumOffsetRequiredForBlockIndexing
    BlockInds2 = round(cornercoords2/MaxAbsAdjacentOffsets(2)) + 1;
else
    BlockInds2 = ones(size(cornercoords2));
end

if MaxAbsAdjacentOffsets(3) > MinimumOffsetRequiredForBlockIndexing
    %BlockInds3 = round(cornercoords3/MaxAbsAdjacentOffsets(3)) + 1;
    BlockInds3 = round(cornercoords3/MinimumOffsetRequiredForBlockIndexing) + 1;
    [bis3,bis3s] = unique(BlockInds3);
    if bis3s(1) == min(bis3s)    % if when sorted the first number is the lowest (closest to index 1), then indexing is "positive"
        [ubis3,ubis3s] = sort(bis3s,'ascend');
    else
        [ubis3,ubis3s] = sort(bis3s,'descend');
    end
    BlockInds3 = ubis3s;
else
    BlockInds3 = ones(size(cornercoords3));
end

 

% Get block indices for grid


% Break up volume into 4x4x4 grid
BlockConfig.nblks1 = numel(unique(BlockInds1));
BlockConfig.nblks2 = numel(unique(BlockInds2));
BlockConfig.nblks3 = numel(unique(BlockInds3));
BlockConfig.nblks = [BlockConfig.nblks1,BlockConfig.nblks2,BlockConfig.nblks3];
BlockConfig.BlockArrangement = reshape(1:prod(BlockConfig.nblks),BlockConfig.nblks);
% Make a list of all block pairs
BlockConfig.all_block_pairs = adjacentBlockPairList;

% Compute sizes and ranges of blocks (ASSUME BLOCKS ARE REGULAR AND BLOCK 1
% IS REPRESENTATIVE)
%[blockData] = GetBlockVolumeData(blockFileNameList{1},'VascMask');
loadedStruct = load(blockFileNameList{1});
loadedVariables = fieldnames(loadedStruct);
blockData = loadedStruct.(loadedVariables{1});
if isstruct(blockData),
    tempPixelList = blockData.pixelList;
    tempMaskSize = blockData.maskSize;
    blockData = false(tempMaskSize);
    blockData(tempPixelList) = 1;
    clear tempPixelList tempMaskSize
end%if isstruct(blockData),
%-----    
bsz = size(blockData);
BlockConfig.bsz1 = bsz(1);
BlockConfig.bsz2 = bsz(2);
BlockConfig.bsz3 = bsz(3);
BlockConfig.blksz = [BlockConfig.bsz1,BlockConfig.bsz2,BlockConfig.bsz3];
BlockConfig.blksz1 = 0:(BlockConfig.bsz1 - 1);
BlockConfig.blksz2 = 0:(BlockConfig.bsz2 - 1);
BlockConfig.blksz3 = 0:(BlockConfig.bsz3 - 1);

% % Create 3D cross for neighborhood computation
% BlockConfig.cross3D = Make3DCross;

%% GENERATE BLOCKS STRUCT
Blocks = struct([]);
% Initialize block corner locations
for iBLK = 1:numel(BlockConfig.BlockArrangement)
    Blocks(iBLK).corner_coords = [cornercoords1(iBLK) cornercoords2(iBLK) cornercoords3(iBLK)];
end

for iBLOCK = 1:numel(Blocks)
    loadedStruct = load(blockFileNameList{1});
    loadedVariables = fieldnames(loadedStruct);
    blockData = loadedStruct.(loadedVariables{1});
    if isstruct(blockData),
        tempPixelList = blockData.pixelList;
        tempMaskSize = blockData.maskSize;
        blockData = false(tempMaskSize);
        blockData(tempPixelList) = 1;
        clear tempPixelList tempMaskSize
    end%if isstruct(blockData),
    %-----
    % Blocks(iBLOCK).block_data_fname = ['Block_',num2str(iBLOCK),'.tif'];
    Blocks(iBLOCK).block_data_fname = blockFileNameList{iBLOCK};
    Blocks(iBLOCK).block_size = size(blockData);
    
    Blocks(iBLOCK).all_block_pair_indices = find(sum((BlockConfig.all_block_pairs == iBLOCK),2) > 0);
    abtmp = BlockConfig.all_block_pairs(Blocks(iBLOCK).all_block_pair_indices,:);
    Blocks(iBLOCK).adjacent_blocks = abtmp(abtmp ~= iBLOCK);
    for iAB = 1:numel(Blocks(iBLOCK).adjacent_blocks)
        Blocks(iBLOCK).MOESindices(iAB).adjacent_block = Blocks(iBLOCK).adjacent_blocks(iAB);
        Blocks(iBLOCK).MOESindices(iAB).x1x2 = 1;
        Blocks(iBLOCK).MOESindices(iAB).x1x3 = 1;
        Blocks(iBLOCK).MOESindices(iAB).x2x3 = 1;
    end
end
BlocksNominal = Blocks;

%% INITIALIZE OFFSETS
Offsets = InitializeOffsets(Blocks,BlockConfig);

%% INITIALIZE MOES WITH REAL OVERLAP CORRELATION

%noteToSelf
MOESsz = 41;
MarginalOffsetEnergies = PopulateMOESWithOverlapCrossCorrelation(Blocks,BlockConfig,Offsets,MOESsz);


%% COMPUTE 3D OFFSET ENERGY LUT (CONSTANT STRUCT)
for iMOES = 1:numel(MarginalOffsetEnergies),
    MarginalOffsetEnergies(iMOES).emptyBlockCorrectionFlag = emptyBlockCorrectionFlag;
    MarginalOffsetEnergies(iMOES).emptyBlockThreshold = emptyBlockThreshold;
end
    MarginalOffsetEnergies = Make3DOffsetEnergies(MarginalOffsetEnergies);
%StepThroughMarginalOffsetEnergies(MarginalOffsetEnergies);

%% USE MOES MINIMA TO ESTIMATE INITIAL BLOCK LOCATIONS
[Blocks,A,r,dML] = InitializeBlockCornersViaMOES(Blocks,BlockConfig,Offsets,MarginalOffsetEnergies);

% compute the residual deltas from the least squares solution
dLS = A*[Blocks.corner_coords]' - r;

BlockConfig.cross3D = Make3DCross;
[a1,a2,a3] = ind2sub(size(BlockConfig.cross3D),find(BlockConfig.cross3D));
a1=a1-2;a2=a2-2;a3=a3-2;
a1=[a1;0];a2=[a2;0];a3=[a3;0];
OffsetSearch = [a1(:),a2(:),a3(:)];

disp('Skipping Nosearchdims step in MaximizeProjectedBlockCorrelationSum.m line203');
%noteToSelf
% % % % nosearchdims = BlockConfig.nblks == 1;
% % % % OffsetSearch((abs(OffsetSearch(:,nosearchdims)) ~= 0),:) = [];  % remove any search dimension that only has one block layer

Blocks0 = Blocks;   % Set the initial configuration of Blocks to be the least squares solution
num_it = 12;

%% RUN DESCENT FROM BlocksPert num_ensembles TIMES, TAKING THE BEST RESULT (by E)
num_ensembles = 10;
%A_trials = cell(1,num_ensembles);
CornerEnsembles = cell(1,num_ensembles);
E_iEA = zeros(1,num_ensembles);
tic
E_init = ComputeTotalEnergy(Blocks0,Offsets,MarginalOffsetEnergies);    % Contains the energy of the initial configuration
for iEA = 1:num_ensembles
    disp(['search ',num2str(iEA)]);
    %[Blocks,AO,TotalMotion,E_it] = BlockDescentByRandomBlockLocalOffsetSearch(OffsetSearch,Blocks0,Offsets,MarginalOffsetEnergies,num_it);
    [Blocks] = BlockDescentByRandomBlockLocalOffsetSearch(OffsetSearch,Blocks0,Offsets,MarginalOffsetEnergies,num_it);
    [E_iEA(iEA)] = ComputeTotalEnergy(Blocks,Offsets,MarginalOffsetEnergies);
    CornerEnsembles{iEA} = reshape([Blocks.corner_coords],3,prod(BlockConfig.nblks))';
end
toc

% Get the lowest energy config of all the ensemble trials; store that config in Blocks
[minE,minE_configi] = min(E_iEA);
best_corner_config = CornerEnsembles{minE_configi};
for iB = 1:numel(Blocks); Blocks(iB).corner_coords = best_corner_config(iB,:); end

% compute the residual deltas from the least squares solution
dLS_Search = A*[Blocks.corner_coords]' - r;

% Set the returned datastructure
estimatedRegistration.BlocksNominal = BlocksNominal;
estimatedRegistration.Blocks0 = Blocks0;
estimatedRegistration.Blocks = Blocks;
estimatedRegistration.BlockConfig = BlockConfig;
estimatedRegistration.Offsets = Offsets;
estimatedRegistration.MarginalOffsetEnergies = MarginalOffsetEnergies;
estimatedRegistration.dML = dML;
estimatedRegistration.dLS = dLS;
estimatedRegistration.dLS_Search = dLS_Search;

outputParameters = inputParameters;
outputParameters.estimatedRegistration = estimatedRegistration;

