function MarginalOffsetEnergies = PopulateMOESWithOverlapCrossCorrelation(Blocks,BlockConfig,Offsets,MOESsz)

MarginalOffsetEnergies = struct([]);
MOESmidpt = ceil(MOESsz/2);

USE_LOWDIM_INDEXING = true;

for iOFF = 1:numel(Offsets)
    fprintf(1,'Populating Marginal Offset Energy %d of %d. %s\n',iOFF,numel(Offsets),datestr(now));
    % Get indices of both blocks
    Bi = Offsets(iOFF).offs_blocks(1);
    Bj = Offsets(iOFF).offs_blocks(2);
    % Get offset block i to block j
    phi_BiBj = Offsets(iOFF).offs;
    % Get each block's size
    blksz = Blocks(Bi).block_size;
    % Get each block's data
    %blockIData = loadmovie(Blocks(Bi).block_data_fname);
    %blockJData = loadmovie(Blocks(Bj).block_data_fname);
% % %     blockIData = GetBlockVolumeData(Blocks(Bi).block_data_fname,'VascMask');
% % %     blockJData = GetBlockVolumeData(Blocks(Bj).block_data_fname,'VascMask');
    %------
    loadedStruct = load(Blocks(Bi).block_data_fname);
    loadedVariables = fieldnames(loadedStruct);
    blockIData = loadedStruct.(loadedVariables{1});
    if isstruct(blockIData),
        tempPixelList = blockIData.pixelList;
        tempMaskSize = blockIData.maskSize;
        blockIData = false(tempMaskSize);
        blockIData(tempPixelList) = 1;
        clear tempPixelList tempMaskSize
    end%if isstruct(blockIData),
    %-----
    loadedStruct = load(Blocks(Bj).block_data_fname);
    loadedVariables = fieldnames(loadedStruct);
    blockJData = loadedStruct.(loadedVariables{1});
    if isstruct(blockJData),
        tempPixelList = blockJData.pixelList;
        tempMaskSize = blockJData.maskSize;
        blockJData = false(tempMaskSize);
        blockJData(tempPixelList) = 1;
        clear tempPixelList tempMaskSize
    end%if isstruct(blockJData),
    %-----
    
    %         % Visualize each block in the same space
    %         figure;
    %         patch(isosurface(x1i,x2i,x3i,blockIData, mean(blockIData(:))),'facecolor','blue','facealpha',.2,'edgealpha',0);
    %         patch(isosurface(x1j,x2j,x3j,blockJData, mean(blockJData(:))),'facecolor','red','facealpha',.2,'edgealpha',0);
    %         view(40,30);
    %         axis image; grid on;
    
    if ~USE_LOWDIM_INDEXING
        % Choose block i's local coordinate system to begin at 1 1 1
        [x1i,x2i,x3i] = ndgrid(1:blksz(1),1:blksz(2),1:blksz(3));
        % Choose block j's local coordinate system in reference to i's
        [x1j,x2j,x3j] = ndgrid((1:blksz(1))-phi_BiBj(1),(1:blksz(2))-phi_BiBj(2),(1:blksz(3))-phi_BiBj(3));
        % Compute the overlap region
        [Overlaps,Oi,Oj] = intersect([x1i(:),x2i(:),x3i(:)],[x1j(:),x2j(:),x3j(:)],'rows');
        %clear x1i x2i x3i x1j x2j x3j
        % Compute the overlap region dimensions
        Overlap_r1 = max(Overlaps(:,1)) - min(Overlaps(:,1)) + 1;
        Overlap_r2 = max(Overlaps(:,2)) - min(Overlaps(:,2)) + 1;
        Overlap_r3 = max(Overlaps(:,3)) - min(Overlaps(:,3)) + 1;
        %clear Overlaps
        % Get the overlap data in block i
        blockIData = single(reshape(blockIData(Oi),[Overlap_r3,Overlap_r2,Overlap_r1]));
        %clear Oi
        % Get the overlap data in block j
        blockJData = single(reshape(blockJData(Oj),[Overlap_r3,Overlap_r2,Overlap_r1]));
        %clear Oj
        
        blockIData = permute(blockIData,[3 2 1]);  % fix dimension mix-up with reshape
        blockJData = permute(blockJData,[3 2 1]);  % fix dimension mix-up with reshape
    else
        x1i = [1:blksz(1)];
        x2i = [1:blksz(2)];
        x3i = [1:blksz(3)];
        x1j = x1i-phi_BiBj(1);
        x2j = x2i-phi_BiBj(2);
        x3j = x3i-phi_BiBj(3);
        [x1O,x1Oi,x1Oj] = intersect(x1i,x1j);
        [x2O,x2Oi,x2Oj] = intersect(x2i,x2j);
        [x3O,x3Oi,x3Oj] = intersect(x3i,x3j);
        blockIData = blockIData(x1Oi,x2Oi,x3Oi);
        blockJData = blockJData(x1Oj,x2Oj,x3Oj);
    end
    
    % Get projections of each block in the overlap region
    Pi_12 = sum(blockIData,3);
    Pi_13 = sum(blockIData,2);
    Pi_23 = sum(blockIData,1);
    Pj_12 = sum(blockJData,3);
    Pj_13 = sum(blockJData,2);
    Pj_23 = sum(blockJData,1);
    
    % Slide projections according to MOES energy lookup tables -- populate MOES
    MOES_x1x2 = GetXCorrelationViaSlidingWindows(Pi_12,Pj_12,MOESsz);
    MOES_x1x3 = GetXCorrelationViaSlidingWindows(Pi_13,Pj_13,MOESsz);
    MOES_x2x3 = GetXCorrelationViaSlidingWindows(Pi_23,Pj_23,MOESsz);
    %     MOES_x1x2 = GetXCorrelationViaSlidingWindows_withViz(Pi_12,Pj_12,MOESsz,iOFF);
    %     MOES_x1x3 = GetXCorrelationViaSlidingWindows_withViz(Pi_13,Pj_13,MOESsz,iOFF);
    %     MOES_x2x3 = GetXCorrelationViaSlidingWindows_withViz(Pi_23,Pj_23,MOESsz,iOFF);
    
    MarginalOffsetEnergies(iOFF).MOES_x1x2 = MOES_x1x2;
    MarginalOffsetEnergies(iOFF).MOES_x1x3 = MOES_x1x3;
    MarginalOffsetEnergies(iOFF).MOES_x2x3 = MOES_x2x3;
    MarginalOffsetEnergies(iOFF).blocks = BlockConfig.all_block_pairs(iOFF,:);
    MarginalOffsetEnergies(iOFF).dims.x1x2 = [1 2]; % indices of dimensions for MOES_x1x2
    MarginalOffsetEnergies(iOFF).dims.x1x3 = [1 3]; % indices of dimensions for MOES_x1x3
    MarginalOffsetEnergies(iOFF).dims.x2x3 = [2 3]; % indices of dimensions for MOES_x2x3
    MarginalOffsetEnergies(iOFF).dim_names = {'x1x2','x1x3','x2x3'}; % dimension name for MOES_x1x2, MOES_x1x3, MOES_x2x3
    MarginalOffsetEnergies(iOFF).offs_to_MOESloc_x1x2.MOESloc = [MOESmidpt MOESmidpt 1];
    MarginalOffsetEnergies(iOFF).offs_to_MOESloc_x1x3.MOESloc = [MOESmidpt 1 MOESmidpt];
    MarginalOffsetEnergies(iOFF).offs_to_MOESloc_x2x3.MOESloc = [1 MOESmidpt MOESmidpt];
    MarginalOffsetEnergies(iOFF).offs_to_MOESloc_x1x2.offs = [blksz(1) blksz(2) 0];
    MarginalOffsetEnergies(iOFF).offs_to_MOESloc_x1x3.offs = [blksz(1) 0 blksz(3)];
    MarginalOffsetEnergies(iOFF).offs_to_MOESloc_x2x3.offs = [0 blksz(2) blksz(3)];
end
