function outputParameters = VidaRegisterBlocks(inputParameters)
vidaProject = inputParameters;
imageStackFolderNameList = vidaProject.imageStackFolderNameList;
experimentTemplate = vidaProject.experimentTemplate;
currentExperiment = vidaProject.currentExperiment;
fullStackSize = currentExperiment.globalParameters.fullStackSize;
registrationParameters = experimentTemplate.registrationParameters;
reverseYBlockNumberingFlag = registrationParameters.reverseYBlockNumberingFlag;
reverseXBlockNumberingFlag = registrationParameters.reverseXBlockNumberingFlag;
xcorrCh = registrationParameters.xcorrCh;
nominalBlockStepSizeRCInMicrons = registrationParameters.nominalBlockStepSizeRCInMicrons;
nominalBlockStepSizeZInMicrons = registrationParameters.nominalBlockStepSizeZInMicrons;
variableZBlockStepSizeFlag = registrationParameters.variableZBlockStepSizeFlag;
ySearchSizeInMicrons = registrationParameters.ySearchSizeInMicrons;
xSearchSizeInMicrons = registrationParameters.xSearchSizeInMicrons;
zSearchSizeInMicrons = registrationParameters.zSearchSizeInMicrons;
experimentTemplate.registrationParameters.emptyBlockCorrectionFlag = 1; %set to 1 to default empty blocks to nominal position
emptyBlockCorrectionFlag = registrationParameters.emptyBlockCorrectionFlag;
emptyBlockThreshold = registrationParameters.emptyBlockThreshold;
vidaProjectRootName = vidaProject.vidaProjectRootName;
vidaProjectRootFolder = vidaProject.vidaProjectRootFolder;
rczMin = vidaProject.globalParameters.rczMin;
rczMax = vidaProject.globalParameters.rczMax;
voxelSizeInMicrons = vidaProject.globalParameters.voxelSizeInMicrons;
%-----
%Convert microns to voxels
nominalBlockStepSizeRC = nominalBlockStepSizeRCInMicrons ./ voxelSizeInMicrons;
nominalBlockStepSizeZ = nominalBlockStepSizeZInMicrons ./ voxelSizeInMicrons;
ySearchSize = ySearchSizeInMicrons ./ voxelSizeInMicrons;
xSearchSize = xSearchSizeInMicrons ./ voxelSizeInMicrons;
zSearchSize = zSearchSizeInMicrons ./ voxelSizeInMicrons;
%-----
%Add converted or global variables to registrationParameters for ease of
%passing information to subfunction calls
registrationParameters.vidaProjectRootName = vidaProjectRootName;
registrationParameters.vidaProjectRootFolder = vidaProjectRootFolder;
registrationParameters.nominalBlockStepSizeRC = nominalBlockStepSizeRC;
registrationParameters.nominalBlockStepSizeZ = nominalBlockStepSizeZ;
registrationParameters.ySearchSize = ySearchSize;
registrationParameters.xSearchSize = xSearchSize;
registrationParameters.zSearchSize = zSearchSize;
%-----
registrationParameters.maskFileToUseForRegistration = 'VascEnhancedMask';
%-----
%-----

numStacks = size(imageStackFolderNameList);
if numStacks == 1,
    registrationParameters.fullStackSize = fullStackSize;
    registrationParameters.blockRegistration = [1 1 1];
    registrationParameters.numBlocksRCZ = [1 1 1];
    registrationParameters.adjacentBlockPairList = [];
    vidaProject.registrationParameters = registrationParameters;
    vidaProject.globalParameters.blockRegistration = [1 1 1];
    vidaProject.globalParameters.RCZList = [1 1 1];
    vidaProject.globalParameters.adjacentBlockPairList = [];
    outputParameters = vidaProject;
    return% skip the rest of the function
end%if numStacks == 1,



%% SET BLOCK CONFIGURATION (CONSTANT STRUCT)

numRBlocks = rczMax(1) - rczMin(1) +1;
numCBlocks = rczMax(2) - rczMin(2) +1;
numZBlocks = rczMax(3) - rczMin(3) +1;
%-----
registrationParameters.nblks1 = numRBlocks;
registrationParameters.nblks2 = numCBlocks;
registrationParameters.nblks3 = numZBlocks;
registrationParameters.firstBlkRow = rczMin(1);
registrationParameters.firstBlkCol = rczMin(2);
registrationParameters.firstBlkZpos = rczMin(3);

%Build structure
    registrationParameters = GeneratePointerToBlocks(registrationParameters);
blockFileNameList = registrationParameters.blockFileNameList;
RCZList = registrationParameters.RCZList;
    registrationParameters = GetBlockAdjacency(registrationParameters);
adjacentBlockPairList = registrationParameters.adjacentBlockPairList;
    registrationParameters = GetBlockOffsets(registrationParameters);
adjacentBlockPairOffsetList = registrationParameters.adjacentBlockPairOffsetList;
blockCornerCoordList = registrationParameters.blockCornerCoordList;
    registrationParameters = MaximizeProjectedBlockCorrelationSum(registrationParameters);
estimatedRegistration = registrationParameters.estimatedRegistration;




%StepThroughMarginalOffsetEnergies(estimatedRegistration.MarginalOffsetEnergies);

% compute the total motion from the least squares answer to the final
% answer after energy minimization:
TotalMotion_LS_EMin = sum(abs([estimatedRegistration.Blocks.corner_coords] - [estimatedRegistration.Blocks0.corner_coords]));
fprintf('After the least squares solution, the entire set of blocks moved a total of %d voxels to the min energy solution found in the search.\n',TotalMotion_LS_EMin);

%ShowAllPairwiseRegisteredBlocks(estimatedRegistration.BlocksNominal,estimatedRegistration.Offsets);
%ShowAllPairwiseRegisteredBlocks(estimatedRegistration.Blocks0,estimatedRegistration.Offsets);
% ShowAllPairwiseRegisteredBlocks(estimatedRegistration.Blocks,estimatedRegistration.Offsets);


%----------



blockRegistration = reshape([estimatedRegistration.Blocks.corner_coords],[3,length(blockFileNameList)])';
numBlocksRCZ = [numRBlocks,numCBlocks,numZBlocks];

registrationParameters.fullStackSize = fullStackSize;
registrationParameters.blockRegistration = blockRegistration;
registrationParameters.numBlocksRCZ = numBlocksRCZ;
vidaProject.registrationParameters = registrationParameters;
vidaProject.globalParameters.blockRegistration = blockRegistration;
vidaProject.globalParameters.RCZList = RCZList;
vidaProject.globalParameters.adjacentBlockPairList = adjacentBlockPairList;
outputParameters = vidaProject;

return








%% print out for sanity check
% % % clc
% % % for iPAIR = 1 : size(adjacentBlockPairList,1)
% % %     Bi = adjacentBlockPairList(iPAIR,1);
% % %     Bj = adjacentBlockPairList(iPAIR,2);
% % % 
% % %     [path2block,BiName] = fileparts(blockFileNameList{Bi});
% % %     [path2block,BjName] = fileparts(blockFileNameList{Bj});
% % %     fprintf('\nPair%d\n\tBi = %s\t(%d,%d,%d)\n\tBj = %s\t(%d,%d,%d)',iPAIR,BiName,blockCornerCoordList(Bi,:),BjName,blockCornerCoordList(Bj,:));
% % %     fprintf('\n\t\t\t\t\t\t\t\t\t\t\t\t(%d,%d,%d)', adjacentBlockPairOffsetList(iPAIR,:));
% % % end
% % % fprintf('\n')


%%

