function outProject = VidaRegisterBlocksParent(inProject)

vidaProject = inProject;
vidaProjectRootName = vidaProject.vidaProjectRootName;
vidaProjectFileName = vidaProject.vidaProjectFileName;
multiThreadProgress = vidaProject.multiThreadProgress;
imageStackFolderNameList = vidaProject.imageStackFolderNameList;
%-----
experimentTemplate = vidaProject.experimentTemplate;
currentExperiment = vidaProject.currentExperiment;
fullStackSize = currentExperiment.globalParameters.fullStackSize;
registrationParameters = experimentTemplate.registrationParameters;
reverseYBlockNumberingFlag = registrationParameters.reverseYBlockNumberingFlag;
reverseXBlockNumberingFlag = registrationParameters.reverseXBlockNumberingFlag;
xcorrCh = registrationParameters.xcorrCh;
nominalBlockStepSizeRCInMicrons = registrationParameters.nominalBlockStepSizeRCInMicrons;
nominalBlockStepSizeZInMicrons = registrationParameters.nominalBlockStepSizeZInMicrons;
variableZBlockStepSizeFlag = registrationParameters.variableZBlockStepSizeFlag;
ySearchSizeInMicrons = registrationParameters.ySearchSizeInMicrons;
xSearchSizeInMicrons = registrationParameters.xSearchSizeInMicrons;
zSearchSizeInMicrons = registrationParameters.zSearchSizeInMicrons;
experimentTemplate.registrationParameters.emptyBlockCorrectionFlag = 1; %set to 1 to default empty blocks to nominal position
emptyBlockCorrectionFlag = registrationParameters.emptyBlockCorrectionFlag;
emptyBlockThreshold = registrationParameters.emptyBlockThreshold;
vidaProjectRootName = vidaProject.vidaProjectRootName;
rczMin = vidaProject.globalParameters.rczMin;
rczMax = vidaProject.globalParameters.rczMax;
voxelSizeInMicrons = vidaProject.globalParameters.voxelSizeInMicrons;
%-----
%Convert microns to voxels
nominalBlockStepSizeRC = nominalBlockStepSizeRCInMicrons ./ voxelSizeInMicrons;
nominalBlockStepSizeZ = nominalBlockStepSizeZInMicrons ./ voxelSizeInMicrons;
ySearchSize = ySearchSizeInMicrons ./ voxelSizeInMicrons;
xSearchSize = xSearchSizeInMicrons ./ voxelSizeInMicrons;
zSearchSize = zSearchSizeInMicrons ./ voxelSizeInMicrons;
yxzSearchSize = [ySearchSize,xSearchSize,zSearchSize];
%-----
%Add converted or global variables to registrationParameters for ease of
%passing information to subfunction calls
registrationParameters.vidaProjectRootName = vidaProjectRootName;
registrationParameters.nominalBlockStepSizeRC = nominalBlockStepSizeRC;
registrationParameters.nominalBlockStepSizeZ = nominalBlockStepSizeZ;
registrationParameters.yxzSearchSize = yxzSearchSize;
%-----
RCZList = registrationParameters.RCZList;
adjacentBlockPairList = registrationParameters.adjacentBlockPairList;
adjacentBlockPairOffsetList = registrationParameters.adjacentBlockPairOffsetList;
blockCornerCoordList = registrationParameters.blockCornerCoordList;
%-----

numImageStacks = size(imageStackFolderNameList);
if numImageStacks == 1,
    registrationParameters.fullStackSize = fullStackSize;
    registrationParameters.blockRegistration = [1 1 1];
    registrationParameters.numBlocksRCZ = [1 1 1];
    registrationParameters.adjacentBlockPairList = [];
    vidaProject.registrationParameters = registrationParameters;
    vidaProject.globalParameters.blockRegistration = [1 1 1];
    vidaProject.globalParameters.RCZList = [1 1 1];
    vidaProject.globalParameters.adjacentBlockPairList = [];
    outputParameters = vidaProject;
    return% skip the rest of the function
end%if numImageStacks == 1,


%% SET BLOCK CONFIGURATION (CONSTANT STRUCT)

numRBlocks = rczMax(1) - rczMin(1) +1;
numCBlocks = rczMax(2) - rczMin(2) +1;
numZBlocks = rczMax(3) - rczMin(3) +1;
%-----
registrationParameters.nblks1 = numRBlocks;
registrationParameters.nblks2 = numCBlocks;
registrationParameters.nblks3 = numZBlocks;
registrationParameters.firstBlkRow = rczMin(1);
registrationParameters.firstBlkCol = rczMin(2);
registrationParameters.firstBlkZpos = rczMin(3);


%% MODIFY LOCAL PARAMETERS. 

remoteVidaProjectRootFolder = vidaProject.remoteVidaProjectRootFolder;
originalVidaProjectRootFolder = vidaProject.originalVidaProjectRootFolder;
%-----
vidaProjectRootFolder = remoteVidaProjectRootFolder;
registrationParameters.vidaProjectRootFolder = vidaProjectRootFolder;




%Modify the necessary local VidaProject parameters to reflect remote filename on another computer
multiThreadRequestFolder = fullfile(remoteVidaProjectRootFolder,'MultiThreadRequestFiles');
registeredFilesFolder = fullfile(remoteVidaProjectRootFolder,'RegisteredFiles');
adjacentPairRegistrationDataFolder =  fullfile(remoteVidaProjectRootFolder,'RegistrationDataFiles');
vidaProject.vidaProjectRootFolder = remoteVidaProjectRootFolder;
vidaProject.experimentTemplate.globalParameters.vidaProjectRootFolder = remoteVidaProjectRootFolder;
vidaProject.currentExperiment.globalParameters.vidaProjectRootFolder = remoteVidaProjectRootFolder;
vidaProject.multiThreadProgress.multiThreadRequestFolder = multiThreadRequestFolder;
vidaProject.globalParameters.registeredFilesFolder = registeredFilesFolder;
vidaProject.globalParameters.adjacentPairRegistrationDataFolder = adjacentPairRegistrationDataFolder;
%---
pathLength = size(originalVidaProjectRootFolder,2);
numImageStacks = size(vidaProject.imageStackFolderNameList,2);
for stackIter = 1:numImageStacks,
    currentFolderName = vidaProject.imageStackFolderNameList{stackIter};
    vidaProject.imageStackFolderNameList{stackIter} = [remoteVidaProjectRootFolder,currentFolderName(pathLength+1:end)];
end%for stackIter = 1:numImageStacks,



%%

%BUILD STRUCTURE
blockFileNameList = registrationParameters.blockFileNameList;
numBlocks = size(blockFileNameList,1);
for blockIter = 1:numBlocks,
    currentBlockFileName = blockFileNameList{blockIter};
    blockFileNameList{blockIter} = [remoteVidaProjectRootFolder,currentBlockFileName(pathLength+1:end)];
end%for blockIter = 1:numBlocks,
registrationParameters.blockFileNameList = blockFileNameList;
%-----
%-----
Blocks = registrationParameters.Blocks;
BlocksNominal = registrationParameters.BlocksNominal;
BlockConfig = registrationParameters.BlockConfig;
Offsets = registrationParameters.Offsets;
%-----
MOESParameters.Blocks = Blocks;
MOESParameters.BlockConfig = BlockConfig;
MOESParameters.Offsets = Offsets;
MOESParameters.yxzSearchSize = yxzSearchSize;
%-----
numAdjacentPairs = size(adjacentBlockPairList,1);
adjacentPairRegistrationResultsFilePrefix = fullfile(adjacentPairRegistrationDataFolder,[vidaProjectRootName,'-RegData']);

%%
%Begin Multi-thread processing of registration
%STAGE 7 - Child : Request a SubBlock for processing
%   Generate a 6-digit random number as a processor ID
rand('twister', (sum(1000*clock)+(cputime*10000)));
childProcessorID = ceil(rand(1)*(999998));
childProcessorIDString = num2str(childProcessorID,'%06d');
alladjacentPairsCompletedSignal = 0;
waitForParentCounter = 0;
waitForParentMaxLoops = 100;
waitForParentDelayInSec = 5;%seconds
completedResponseIDList = [];
%-----
disp(['Child-Processor ID : ',childProcessorIDString]);
while alladjacentPairsCompletedSignal == 0,
    %Check for a response file - if none exists, create a request file
    requestFileName = fullfile(multiThreadRequestFolder,['MultiThreadRequest',childProcessorIDString,'.mat']);
    responseFileName = fullfile(multiThreadRequestFolder,['MultiThreadResponse',childProcessorIDString,'.mat']);
    deliveryFileName = fullfile(multiThreadRequestFolder,['MultiThreadDelivery',childProcessorIDString,'.mat']);
    if ~exist(responseFileName,'file'),
        if ~exist(requestFileName,'file'),
            %Generate a request file
            disp(['Sending multiThread request as child-processor # ',childProcessorIDString,' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            requestVariable = [];
            save(requestFileName,'requestVariable');
            pause(3);
        end%if ~exist(requestFileName,'file'),
    end%if ~exist(responseFileName,'file'),
    %-----
    %Check for a response file, and process the SuBlock.
    %Look in request folder for delivery files
    if exist(responseFileName,'file'),
        try
            disp('Detected Response File. Attempting Load.');
            load(responseFileName,'multiThreadResponse');
            assignmentIndex = multiThreadResponse.assignmentIndex;
            assignmentString = multiThreadResponse.assignmentString;
            responseID = multiThreadResponse.responseID;
            disp(['Load successful, Assignment Index = ',num2str(assignmentIndex)])
        catch
            assignmentIndex = 0;
            responseID = 0;
            assignmentString = '';
            disp('Load Failed');
        end%try-catch
        %-----
        if assignmentIndex == -1,
            %This stage is completed.  Move on to next multi-thread stage
            disp(['Adjacent Pair Stage completion signal received : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            alladjacentPairsCompletedSignal = 1;
        end%if assignmentIndex == -1,
        %-----
        %If this reponse has already been delivered, just wait for next response.
        if any(responseID == completedResponseIDList)
            assignmentIndex = 0;
        end%if any(responseID == completedResponseIDList)
        %-----
        if assignmentIndex == 0,
            %No stack assigned right now - wait and keep requesting
            disp(['Resending and waiting for reponse to Adjacent Pair request as child-processor #',childProcessorIDString,' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            requestVariable = [];
            save(requestFileName,'requestVariable');
            pause(waitForParentDelayInSec);
            waitForParentCounter = waitForParentCounter + 1;
        end%if assignmentIndex == 0,
        %-----
        if waitForParentCounter >= waitForParentMaxLoops,
            %No stack assigned right now - wait and keep requesting
            disp(['Parent process failing to repond.  Moving to next stage : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            pause(waitForParentDelayInSec);
            assignmentIndex = -1;
        end%if if waitForParentCounter >= waitForParentMaxLoops,
        %-----
        if assignmentIndex > 0,
            waitForParentCounter = 0;
            %-----
            %Before processing, send the next request file
            %This eliminates excessive waiting for parent process.
            if ~exist(requestFileName,'file'),
                disp(['Sending next multiThread request as child-processor # ',childProcessorIDString,' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
                requestVariable = [];
                save(requestFileName,'requestVariable');
                pause(3);
            end%if ~exist(requestFileName,'file'),
            %-----
            %*******HERE IS WHERE YOU ACTUALLY PROCESS THE IMAGE BLOCK*****
            %*******HERE IS WHERE YOU ACTUALLY PROCESS THE IMAGE BLOCK*****
            %*******HERE IS WHERE YOU ACTUALLY PROCESS THE IMAGE BLOCK*****
            fprintf('\n\n');
            disp(['Child-Processing Adjacent Pair # ',num2str(assignmentIndex),' of ',num2str(numAdjacentPairs),' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            MOESParameters.currentAdjacentPairIndex = assignmentIndex;
            multiThreadParameters.MOESParameters = MOESParameters;
            multiThreadParameters.registrationParameters = registrationParameters;
            multiThreadParameters.registrationDataFileName = [adjacentPairRegistrationResultsFilePrefix,num2str(assignmentIndex,'%06d'),'.mat'];
                multiThreadParameters = VidaCalculateSingleMOES(multiThreadParameters);
            %**************************************************************
            %**************************************************************
            %**************************************************************
            %----- 
            %Generate delivery file
            deliveryData = [];
            deliveryData.IDNum = assignmentIndex;
            deliveryCompleted = 0;
            while deliveryCompleted == 0,
                if ~exist(deliveryFileName,'file'),
                    save(deliveryFileName,'deliveryData');
                    deliveryCompleted = 1;
                else%if ~exist(deliveryFileName,'file'),
                    pause(10);
                end%if-else ~exist(deliveryFileName,'file'),
            end%while deliveryCompleted == 0,
            %-----
            completedResponseIDList = [completedResponseIDList;responseID];
            assignmentIndex = 0 ; %set this processor to wait.
        end%if assignmentIndex > 0,
    end%if exist(responseFileName,'file'),
end%while alladjacentPairsCompletedSignal == 0,
%-----
%All Image Stacks Completed.
outProject = vidaProject;












