%This code performs a circuit analysis on a labeled, trimmed vessel set

%Load a globalDataStruct
%
%Note:  no connectivity changes can have occured since the latest restrandify round
%Changes to labels are okay.
%
%Labels: 100=Unlabeled; 200=Temp ; 300=Surface ; 400=Plunging ; 500=Micro ;
%         10=Unlabeled ; 20=Temp ; 30=Supply ; 40=Draining ;  50=Connecting ;
%Therefore: SurfaceArtery=330; SurfaceVein = 340;
%
%cpMatrix:
%   cols1-3:coords, col4:endPointFlag, col5:junctionFlag, col6:activeFlag
%   col7:label

%---
if ~(exist('voltageStrandLabelList','var'))
    voltageStrandLabelList = [330 430];%labels to be considerred voltages voltages
end%if ~exist...    
%---
if ~(exist('groundStrandLabelList','var'))
    groundStrandLabelList = [340 440];%labels to be considered ground
end%if ~exist...    
%---
if ~(exist('Ev','var'))
    Ev = [100];%potential at voltage voltage connection
end%if ~exist...    
%---
if ~(exist('Eg','var'))
    Eg = 50;%potential at ground connection
end%if ~exist...
%----
if ~(exist('conductanceMultiplier','var'))
    conductanceMultiplier = 1e9;
end%if ~exist...
%----
if ~exist('variableCleanupFlag','var'), 
    variableCleanupFlag = 0;
end
%---

maskSize = globalDataStruct.maskSize;
strands = globalDataStruct.strands;
cpMatrix = globalDataStruct.cpMatrix;
si = globalDataStruct.si;
ar = globalDataStruct.ar;
numStrands = size(strands,2);

%Generate a node structure
nodes = [];
nodeIndexList = [];
nodeCounter = 0;
strandConductanceList = [];
for strandIter = 1:numStrands,
    currentStrandID = strandIter;
    currentStrand = strands(currentStrandID);
% %     iv = currentStrand.InteriorVertices;
    svns = currentStrand.StartVertexNeighborStrands;
    evns = currentStrand.EndVertexNeighborStrands;
    activeFlag = currentStrand.Active;
    majorityLabel = median(cpMatrix(iv,7));
    strands(currentStrandID).majorityLabel = majorityLabel;
% %     iv = currentStrand.InteriorVertices;
    iv = currentStrand.StartToEndIndices(2:end-1);
    strandLength = numel(iv);
    if strandLength==0, keyboard;end
    strandConductanceList(currentStrandID) = conductanceMultiplier/strandLength;

    svnsSort = sort(svns);
    evnsSort = sort(evns);
    if isempty(svnsSort),
        nodeCounter = nodeCounter + 1;
        nodes{nodeCounter} = currentStrandID;
        nodeIndexList(nodeCounter) = currentStrand.StartVertexIndex;
    else%if isempty(svnsSort),
        if svnsSort(1)==currentStrandID,
            nodeCounter=nodeCounter+1;
            nodes{nodeCounter} = svnsSort;
            nodeIndexList(nodeCounter) = currentStrand.StartVertexIndex;
        end%if svnsSort(1)==currentStrandID,
    end%sif-else isempty(svnsSort),
    %---
    if currentStrand.StartVertexIndex ~= currentStrand.EndVertexIndex
    %this is to avoid double counting of end_loops.  ---o
        if isempty(evnsSort),
            nodeCounter = nodeCounter + 1;
            nodes{nodeCounter} = currentStrandID;
            nodeIndexList(nodeCounter) = currentStrand.EndVertexIndex;
        else%if isempty(evnsSort),
            if evnsSort(1)==currentStrandID,
                nodeCounter=nodeCounter+1;
                nodes{nodeCounter} = evnsSort;
                nodeIndexList(nodeCounter) = currentStrand.EndVertexIndex;
            end%if evnsSort(1)==currentStrandID,
        end%end%if-else isempty(evnsSort),
    end%if currentStrand.StartVertexIndex ~= currentStrand.EndVertexIndex
end%for strandIter = 1:numStrands,
    


strandConductanceList = strandConductanceList + rand(size(strandConductanceList));


numNodes = size(nodes,2);
nodesConnectedByStrand = zeros(numStrands,2);
for nodeIter = 1:numNodes,
    currentNodeID = nodeIter;
    attachedStrands = unique(nodes{currentNodeID});
    numAttached = size(attachedStrands,2);
    for attachedIter = 1:numAttached;
        currentStrandID = attachedStrands(attachedIter);
        currentConnection = nodesConnectedByStrand(currentStrandID,:);
        emptySpaces = find(currentConnection == 0);
        firstEmptySlot = emptySpaces(1);
        nodesConnectedByStrand(currentStrandID,firstEmptySlot) = currentNodeID;
    end%for attachedIter = 1:numAttached;
end%for nodeIter = 1:numNodes,
    
loopStrands = find(nodesConnectedByStrand(:,2)==0);
nodesConnectedByStrand(loopStrands,2) = nodesConnectedByStrand(loopStrands,1);
    

for k = 1:numNodes,
    f1 = find(nodesConnectedByStrand(:,1)==k);
    f2 = find(nodesConnectedByStrand(:,2)==k);
    ff = unique([f1;f2]);
    %-----
    stei = strands(f1).StartToEndIndices;
    %-----
    if numel(ff)<2,
        f1 = ff(1);
        if isempty(strands(f1).StartVertexNeighborStrands),
            nodeCPID(k) = stei(end);
        else
            nodeCPID(k) = stei(1);
        end
    else
        f1 = ff(1);
        f2 = ff(2);
        stei = strands(f1).StartToEndIndices;
        s1 = strands(f1).StartVertexIndex;
        s2 = strands(f2).StartVertexIndex;
        e2 = strands(f2).EndVertexIndex;
        if or(s1 == s2 , s1==e2),
            nodeCPID(k) = stei(1);
        else
            nodeCPID(k) = stei(end);
        end
    end
end


%Make a list of nodes that are ON the special vessels (surface&plunging) and thus held at Ev or Eg.
voltageNodeList = [];
groundNodeList = [];
%-----
for strandIter = 1:numStrands,
    currentStrandID = strandIter;
    currentStrand = strands(currentStrandID);
    majorityLabel = currentStrand.majorityLabel;
    attachedNodes = nodesConnectedByStrand(currentStrandID,:);
    if ismember(majorityLabel,voltageStrandLabelList),
        voltageNodeList = [voltageNodeList,attachedNodes];
    end%if ismember(majorityLabel,voltageStrandLabelList],  
    %-----
    if ismember(majorityLabel,groundStrandLabelList),
        groundNodeList = [groundNodeList,attachedNodes];
    end%if ismember(majorityLabel,voltageStrandLabelList],  
end%for strandIter = 1:numStrands,
voltageNodeList = unique(voltageNodeList);
groundNodeList = unique(groundNodeList);
numVoltageNodes = size(voltageNodeList,2);
numGroundNodes = size(groundNodeList,2);



%Make a list of strands that are connected to the voltage nodes.
%   there are two types : those that are PART of the special vessels
%   and those that are attached to them by a special node.
voltageStrandList = [];
groundStrandList = [];
voltageConnectStrandList = [];
groundConnectStrandList = [];
for nodeIter = 1:numVoltageNodes,
    currentVoltageNode = voltageNodeList(nodeIter);
    strandsAttached = nodes{currentVoltageNode};
    numStrandsAttached = size(strandsAttached,2);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentMajorityLabel = strands(currentStrandID).majorityLabel;
        if ismember(currentMajorityLabel,voltageStrandLabelList),
            voltageStrandList = [voltageStrandList,currentStrandID];
        else%if ismember(currentMajorityLabel,voltageStrandLabelList),
            voltageConnectStrandList = [voltageConnectStrandList,currentStrandID];
        end%if-else ismember(currentMajorityLabel,voltageStrandLabelList),
    end%for strandIter = 1:numStrands,
end%for nodeIter = 1:numVoltageNodes,
%-----
for nodeIter = 1:numGroundNodes,
    currentGroundNode = groundNodeList(nodeIter);
    strandsAttached = nodes{currentGroundNode};
    numStrandsAttached = size(strandsAttached,2);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentMajorityLabel = strands(currentStrandID).majorityLabel;
        if ismember(currentMajorityLabel,groundStrandLabelList),
            groundStrandList = [groundStrandList,currentStrandID];
        else%if ismember(currentMajorityLabel,groundStrandLabelList),
            groundConnectStrandList = [groundConnectStrandList,currentStrandID];
        end%if-else ismember(currentMajorityLabel,groundStrandLabelList),
    end%for strandIter = 1:numStrands,
end%for nodeIter = 1:numGroundNodes,
%-----
voltageStrandList = unique(voltageStrandList);
groundStrandList = unique(groundStrandList);
numVoltageConnectStrands = unique(voltageConnectStrandList);
groundConnectStrandList = unique(groundConnectStrandList);
%---
numVoltageStrands = size(voltageStrandList,2);
numGroundStrands = size(groundStrandList,2);
numVoltageConnectStrands = size(voltageConnectStrandList,2);
numGroundConnectStrands = size(groundConnectStrandList,2);

tempNodeList = [1:numNodes];
freeNodeList = tempNodeList(~ismember(tempNodeList,[voltageNodeList,groundNodeList]));
numFreeNodes = size(freeNodeList,2);
sourceConductance = inf;

%Populate the conductance matrix
gMatrix = sparse(zeros(numFreeNodes,numFreeNodes));
forcingVector = zeros(numFreeNodes,1);
for freeNodeIter = 1:numFreeNodes,
    currentNodeID = freeNodeList(freeNodeIter);
    strandsAttached = nodes{currentNodeID};
    numStrandsAttached = size(strandsAttached,2);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentStrand = strands(currentStrandID);
        strandConductance = strandConductanceList(currentStrandID);
        activeFlag = currentStrand.Active;
        majorityLabel = median(cpMatrix(iv,7));
        connectedNodes = nodesConnectedByStrand(currentStrandID,:);
        otherNodeID = connectedNodes(connectedNodes~=currentNodeID);
        if ~isempty(otherNodeID),
            otherFreeNodeID = find(freeNodeList==otherNodeID);
            if ~isempty(otherFreeNodeID),
                gMatrix(freeNodeIter,otherFreeNodeID) =  -1 *  strandConductance;
                gMatrix(otherFreeNodeID,freeNodeIter) =  -1 *  strandConductance;
            end%if ~isempty(otherFreeNodeID),
        end%if ~isempty(otherNodeID),
    end%for strandIter = 1:numStrandsAttached,
end%for freeNodeIter = 1:numFreeNodes,
            
for freeNodeIter = 1:numFreeNodes,
    gMatrix(freeNodeIter,freeNodeIter) = -1 * sum(gMatrix(freeNodeIter,:));
end

forcingVector = zeros(numFreeNodes,1);
for strandIter = 1:numVoltageConnectStrands,
    currentVoltageConnectStrandID = voltageConnectStrandList(strandIter);
    currentNodes = nodesConnectedByStrand(currentVoltageConnectStrandID,:);
    currentStrand = strands(currentVoltageConnectStrandID);
    iv = currentStrand.StartToEndIndices(2:end-1);
    strandConductance = strandConductanceList(currentVoltageConnectStrandID);
    if ismember(currentNodes(1),voltageNodeList),
        freeNodeID = find(freeNodeList==currentNodes(2));
    else%if ismember(currentNodes(1),voltageNodeList),
        freeNodeID = find(freeNodeList==currentNodes(1));
    end%if-else ismember(currentNodes(1),voltageNodeList),
    gMatrix(freeNodeID,freeNodeID) = gMatrix(freeNodeID,freeNodeID) + strandConductance;
    forcingVector(freeNodeID) = forcingVector(freeNodeID) + Ev.*strandConductance;
end%for strandIter = 1:numVoltageConnectStrands,

for strandIter = 1:numGroundConnectStrands,
    currentGroundConnectStrandID = groundConnectStrandList(strandIter);
    currentNodes = nodesConnectedByStrand(currentGroundConnectStrandID,:);
    currentStrand = strands(currentGroundConnectStrandID);
    iv = currentStrand.StartToEndIndices(2:end-1);
    strandConductance = strandConductanceList(currentGroundConnectStrandID);
    if ismember(currentNodes(1),groundNodeList),
        freeNodeID = find(freeNodeList==currentNodes(2));
    else%if ismember(currentNodes(1),voltageNodeList),
        freeNodeID = find(freeNodeList==currentNodes(1));
    end%if-else ismember(currentNodes(1),voltageNodeList),
    gMatrix(freeNodeID,freeNodeID) = gMatrix(freeNodeID,freeNodeID) + strandConductance;
    forcingVector(freeNodeID) = forcingVector(freeNodeID) + Eg.*strandConductance;
end%for strandIter = 1:numGroundConnectStrands,



tic
gInv = inv(gMatrix);
toc
voltageVector = gInv*forcingVector;


analysisResults.cropYXZRange = cropYXZRange;
analysisResults.groundLabelList = groundLabelList;
analysisResults.sourceLabelList = sourceLabelList;
analysisResults.sourceVoltage = sourceVoltage;
analysisResults.groundVoltage = groundVoltage;
analysisResults.startStrandID = startStrandID;
%-----
analysisResults.forcingVector = forcingVector;
analysisResults.voltageVector = voltageVector;
analysisResults.voltageNodeList = voltageNodeList;
analysisResults.gMatrix = gMatrix;
analysisResults.gInv = gInv;
analysisResults.strandsConnectedByNode = nodes;
analysisResults.nodesConnectedByStrand = nodesConnectedByStrand;
analysisResults.strandConductanceList = strandConductanceList;
analysisResults.nodeCPID = nodeCPID;
analysisResults.groundStrandList = groundStrandList;
analysisResults.voltageStrandList = voltageStrandList;
analysisResults.groundConnectStrandList = groundConnectStrandList;
analysisResults.voltageConnectStrandList  = voltageConnectStrandList ;
analysisResults.freeNodeList  = freeNodeList ;
analysisResults.groundNodeList = groundNodeList;
analysisResults.voltageNodeList = voltageNodeList;




figure(1);plot(voltageVector)
figure(2);plot(forcingVector)
figure(3);plot(gMatrix*voltageVector)



globalDataStruct.cpMatrix = cpMatrix;
globalDataStruct.strands = strands;
fileRootName = globalDataStruct.fileRootName;
dateString = datestr(now,'yyyy_mm_dd-HH_MM_SS');
[pathName,fileName,ext] = fileparts(fileRootName);
saveFileName = fullfile(pathName,['SE',dateString,'-AnalysisResults-',fileName,ext]);
save(saveFileName,'analysisResults','-v7.3');




if variableCleanupFlag == 1,
    clear activeFlag ar attachedIter attachedNodes attachedStrands conductanceMultiplier 
    clear connectedNodes cpMatrix currentConnection currentGroundConnectStrandID 
    clear currentGroundNode currentMajorityLabel currentNodeID currentNodes 
    clear currentStrand currentStrandID currentVoltageConnectStrandID currentVoltageNode dateString
    clear e2 emptySpaces evns evnsSort ext f1 f2 ff fileName fileRootName firstEmptySlot 
    clear forcingVector freeNodeID freeNodeIter freeNodeList
    clear gInv gMatrix groundConnectStrandList groundNodeList groundStrandList iv k
    clear loopStrands majorityLabel maskSize nodeCPID nodeCounter nodeIndexList nodeIter
    clear nodes nodesConnectedByStrand numAttached numFreeNodes numGroundConnectStrands 
    clear numGroundNodes numGroundStrands 
    clear numNodes numStrands numStrandsAttached numVoltageConnectStrands 
    clear numVoltageNodes numVoltageStrands otherFreeNodeID otherNodeID pathName
    clear s1 s2 safeLabelList saveFileName si sourceConductance stei strandConductance
    clear strandConductanceList strandIter strandLength strands strandsAttached svns 
    clear svnsSort tempNodeList   voltageConnectStrandList 
    clear voltageNodeList voltageStrandList voltageVector 
end%if variableCleanupFlag == 1


