%This code performs a circuit analysis on a labeled, trimmed vessel set
%Load a globalDataStruct
%
%Note:  no connectivity changes can have occured since the latest restrandify round
%Changes to labels are okay.
%
%Labels: 100=Unlabeled; 200=Temp ; 300=Surface ; 400=Plunging ; 500=Micro ;
%         10=Unlabeled ; 20=Temp ; 30=Supply ; 40=Draining ;  50=Connecting ;
%Therefore: SurfaceArtery=330; SurfaceVein = 340;
%
%cpMatrix:
%   cols1-3:coords, col4:endPointFlag, col5:junctionFlag, col6:activeFlag
%   col7:label



%In this version, only free endpoints of surface vessels are given fixed voltages
%The four rectangular edges are allowed different fixed voltage levels
%i.e., left edge artery endpoints = 50V, right edge artery endpoints = 25V





% % load('F:\Phil\AOH Datasets\Dataset_DB\ThirdSize_2\SE2010_10_22-13_34_08-Restrandified-StrandEdit-pbDB_subSample3-crop-rotate-crop.mat')





%---
if ~(exist('voltageStrandLabelList','var'))
    voltageStrandLabelList = [330];%labels to be considerred voltages voltages
end%if ~exist...    
%---
if ~(exist('groundStrandLabelList','var'))
    groundStrandLabelList = [340];%labels to be considered ground
end%if ~exist...    
%---
if ~(exist('Ev','var'))
    Ev = [100];%potential at voltage voltage connection
end%if ~exist...    
%---
if ~(exist('Eg','var'))
    Eg = 50;%potential at ground connection
end%if ~exist...
%----
if ~(exist('conductanceMultiplier','var'))
    conductanceMultiplier = 1e9;
end%if ~exist...
%----
if ~exist('variableCleanupFlag','var'), 
    variableCleanupFlag = 0;
end
%---

maskSize = globalDataStruct.maskSize;
strands = globalDataStruct.strands;
cpMatrix = globalDataStruct.cpMatrix;
si = globalDataStruct.si;
ar = globalDataStruct.ar;
numStrands = size(strands,2);

radiusPower = 4;
minRadius = 0.25;
minStrandLength = 4;

highVoltageCoordinateYX = [1 1];
lowGroundCoordinateYX = [1119 1];
voltageRange = [990 1000];
groundRange = [99 100];

voltageMap = zeros(maskSize(1),maskSize(2));
voltageMap(highVoltageCoordinateYX(1),highVoltageCoordinateYX(2)) = 1;
voltageMap = bwdist(voltageMap);
mapMin = min(voltageMap(:));
mapMax = max(voltageMap(:));
normVoltageMap = 1 - ((voltageMap-mapMin)/mapMax);
voltageSpread = voltageRange(2)-voltageRange(1);
voltageMap = (normVoltageMap * voltageSpread) + voltageRange(1);
%---
groundMap = zeros(maskSize(1),maskSize(2));
groundMap(lowGroundCoordinateYX(1),lowGroundCoordinateYX(2)) = 1;
groundMap = bwdist(groundMap);
mapMin = min(groundMap(:));
mapMax = max(groundMap(:));
normGroundMap = (groundMap-mapMin)/mapMax;
groundSpread = groundRange(2)-groundRange(1);
groundMap = (normGroundMap * groundSpread) + groundRange(1);



disp(['Generating Node Struture : ',datestr(now)]);
%Generate a node structure
nodes = [];
nodeIndexList = [];
nodeCounter = 0;
strandConductanceList = [];
for strandIter = 1:numStrands,
    currentStrandID = strandIter;
    currentStrand = strands(currentStrandID);
    iv = currentStrand.StartToEndIndices(2:end-1);
    svns = currentStrand.StartVertexNeighborStrands;
    evns = currentStrand.EndVertexNeighborStrands;
    activeFlag = currentStrand.Active;
    majorityLabel = median(cpMatrix(iv,7));
    strands(currentStrandID).majorityLabel = majorityLabel;
    iv = currentStrand.StartToEndIndices(2:end-1);
    strandLength = numel(iv);
    strandLength = max(strandLength,minStrandLength);
    
    
    if strandLength==0, keyboard;end
    thisRadiusList = ar(iv);
    thisRadiusList = max(thisRadiusList,minRadius);
   
    
    
% % %     thisStrandRadius = median(thisRadiusList(thisRadiusList>minRadius));
    thisStrandRadius = median(thisRadiusList);
    
    
    
    %Add a random amount to the strandlength to avoid discretation and 
    %potential matrix singularity???
    
    
    strandLength = max(minStrandLength,strandLength) + rand(1);
% % %     strandConductanceList(currentStrandID) = conductanceMultiplier/strandLength;
    strandConductanceList(currentStrandID) = conductanceMultiplier * thisStrandRadius.^radiusPower / strandLength;
    
    
    

    
    
% % % % disp('USING EXPONENTIAL RADIAL DEPENDENCE!!!!');    
% % % % disp('USING EXPONENTIAL RADIAL DEPENDENCE!!!!');    
% % % % disp('USING EXPONENTIAL RADIAL DEPENDENCE!!!!');    
% % % % disp('USING EXPONENTIAL RADIAL DEPENDENCE!!!!');    
% % % % ro =  3;   
% % % % strandConductanceList(currentStrandID) = conductanceMultiplier * thisStrandRadius * (1-exp(-thisStrandRadius/ro)) / strandLength;

    
    
    svnsSort = sort(svns);
    evnsSort = sort(evns);
    if isempty(svnsSort),
        nodeCounter = nodeCounter + 1;
        nodes{nodeCounter} = currentStrandID;
        nodeIndexList(nodeCounter) = currentStrand.StartVertexIndex;
    else%if isempty(svnsSort),
        if svnsSort(1)==currentStrandID,
            nodeCounter=nodeCounter+1;
            nodes{nodeCounter} = svnsSort;
            nodeIndexList(nodeCounter) = currentStrand.StartVertexIndex;
        end%if svnsSort(1)==currentStrandID,
    end%sif-else isempty(svnsSort),
    %---
    if currentStrand.StartVertexIndex ~= currentStrand.EndVertexIndex
    %this is to avoid double counting of end_loops.  ---o
        if isempty(evnsSort),
            nodeCounter = nodeCounter + 1;
            nodes{nodeCounter} = currentStrandID;
            nodeIndexList(nodeCounter) = currentStrand.EndVertexIndex;
        else%if isempty(evnsSort),
            if evnsSort(1)==currentStrandID,
                nodeCounter=nodeCounter+1;
                nodes{nodeCounter} = evnsSort;
                nodeIndexList(nodeCounter) = currentStrand.EndVertexIndex;
            end%if evnsSort(1)==currentStrandID,
        end%end%if-else isempty(evnsSort),
    end%if currentStrand.StartVertexIndex ~= currentStrand.EndVertexIndex
end%for strandIter = 1:numStrands,
    
numNodes = nodeCounter;
%Determine the label at each node
nodeLabelList = zeros(size(nodes))+110;
for nodeIter = 1:numNodes,
    attachedStrands = nodes{nodeIter};
    numAttachedStrands = numel(attachedStrands);
    %Check to see if this node is an endpoint
    if numAttachedStrands < 2,
        thisStrand = strands(attachedStrands(1));
        svi = thisStrand.StartVertexIndex;
        evi = thisStrand.EndVertexIndex;
        svns = thisStrand.StartVertexNeighborStrands;
        evns = thisStrand.EndVertexNeighborStrands;
        if or(isempty(svns),isempty(evns)),
            if isempty(svns),
                endPointIndex = svi;
            else%if isempty(svns),
                endPointIndex = evi;
            end%if isempty(svns),
            nodeLabelList(nodeIter) = cpMatrix(endPointIndex,7);
        end%if or(isempty(svns),isempty(evns)),
    end%if numAttachedStrands < 2,
end
clear svi evi svns evns thisNode thisStrand endPointIndex

%All nodes in nodeLabel have a label of 110 (undetermined) UNLESS
%the node is a free endpoint, in which case it takes the label of its only
%attached strand


% % % % 
% % % % strandConductanceList = strandConductanceList + rand(size(strandConductanceList));
% % % % 


%Find which strands are connected by each node
numNodes = size(nodes,2);
nodesConnectedByStrand = zeros(numStrands,2);
for nodeIter = 1:numNodes,
    currentNodeID = nodeIter;
    attachedStrands = unique(nodes{currentNodeID});
    numAttached = size(attachedStrands,2);
    for attachedIter = 1:numAttached;
        currentStrandID = attachedStrands(attachedIter);
        currentConnection = nodesConnectedByStrand(currentStrandID,:);
        emptySpaces = find(currentConnection == 0);
        firstEmptySlot = emptySpaces(1);
        nodesConnectedByStrand(currentStrandID,firstEmptySlot) = currentNodeID;
    end%for attachedIter = 1:numAttached;
end%for nodeIter = 1:numNodes,
  
loopStrands = find(nodesConnectedByStrand(:,2)==0);
nodesConnectedByStrand(loopStrands,2) = nodesConnectedByStrand(loopStrands,1);
    
%Find the index in CPMatrix for each node point
for k = 1:numNodes,
    attachedStrandList1 = find(nodesConnectedByStrand(:,1)==k);
    attachedStrandList2 = find(nodesConnectedByStrand(:,2)==k);
    allAttachedStrandList = unique([attachedStrandList1;attachedStrandList2]);
    if numel(allAttachedStrandList)<2,
        attachedStrandID = allAttachedStrandList(1);
        onlyAttachedStrand = strands(attachedStrandID);
        stei = onlyAttachedStrand.StartToEndIndices;

        if isempty(onlyAttachedStrand.StartVertexNeighborStrands),
            nodeCPID(k) = stei(1);
        else
            nodeCPID(k) = stei(end);
        end
    else%if numel(allAttachedStrandList)<2,
        attachedStrand1ID = allAttachedStrandList(1);
        attachedStrand2ID = allAttachedStrandList(2);
        attachedStrand1 = strands(attachedStrand1ID);
        attachedStrand2 = strands(attachedStrand2ID);
        stei1 = attachedStrand1.StartToEndIndices;

        s1 = attachedStrand1.StartVertexIndex;
        s2 = attachedStrand2.StartVertexIndex;
        e2 = attachedStrand2.EndVertexIndex;
        if or(s1 == s2 , s1==e2),
            nodeCPID(k) = stei1(1);
        else
            nodeCPID(k) = stei1(end);
        end
    end%if numel(allAttachedStrandList)<2,
end


%Make a list of free end nodes that are ON the special vessels (surface) 
% and thus held at Ev or Eg.
%Consider only those nodes that are free end points
voltageNodeList = [];
groundNodeList = [];
%-----
for nodeIter = 1:numNodes,
    nodeID = nodeIter;
    thisNodeLabel = nodeLabelList(nodeIter);
    if ismember(thisNodeLabel,voltageStrandLabelList),
        voltageNodeList = [voltageNodeList,nodeID];
    end
    %---
    if ismember(thisNodeLabel,groundStrandLabelList),
        groundNodeList = [groundNodeList,nodeID];
    end
end



numVoltageNodes = size(voltageNodeList,2);
numGroundNodes = size(groundNodeList,2);



%Make a list of strands that are connected to the voltage nodes.
%   there are two types : those that are PART of the special vessels
%   and those that are attached to them by a special node.
voltageStrandList = [];
groundStrandList = [];
voltageConnectStrandList = [];
groundConnectStrandList = [];
for nodeIter = 1:numVoltageNodes,
    currentVoltageNode = voltageNodeList(nodeIter);
    strandsAttached = nodes{currentVoltageNode};
    numStrandsAttached = size(strandsAttached,2);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentMajorityLabel = strands(currentStrandID).majorityLabel;
        voltageConnectStrandList = [voltageConnectStrandList,currentStrandID];
    end%for strandIter = 1:numStrands,
end%for nodeIter = 1:numVoltageNodes,
%-----
for nodeIter = 1:numGroundNodes,
    currentGroundNode = groundNodeList(nodeIter);
    strandsAttached = nodes{currentGroundNode};
    numStrandsAttached = size(strandsAttached,2);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentMajorityLabel = strands(currentStrandID).majorityLabel;
        groundConnectStrandList = [groundConnectStrandList,currentStrandID];
    end%for strandIter = 1:numStrands,
end%for nodeIter = 1:numGroundNodes,
%-----
numVoltageConnectStrands = unique(voltageConnectStrandList);
groundConnectStrandList = unique(groundConnectStrandList);
%---
numVoltageConnectStrands = size(voltageConnectStrandList,2);
numGroundConnectStrands = size(groundConnectStrandList,2);

tempNodeList = [1:numNodes];
freeNodeList = tempNodeList(~ismember(tempNodeList,[voltageNodeList,groundNodeList]));
numFreeNodes = size(freeNodeList,2);

disp(['Populating gMatrix : ',datestr(now)]);
%Populate the conductance matrix
gMatrix = sparse(zeros(numFreeNodes,numFreeNodes));
forcingVector = zeros(numFreeNodes,1);
for freeNodeIter = 1:numFreeNodes,
    currentNodeID = freeNodeList(freeNodeIter);
    strandsAttached = nodes{currentNodeID};
    numStrandsAttached = size(strandsAttached,2);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentStrand = strands(currentStrandID);
        strandConductance = strandConductanceList(currentStrandID);
        activeFlag = currentStrand.Active;
        majorityLabel = median(cpMatrix(iv,7));
        connectedNodes = nodesConnectedByStrand(currentStrandID,:);
        otherNodeID = connectedNodes(connectedNodes~=currentNodeID);
        if ~isempty(otherNodeID),
            otherFreeNodeID = find(freeNodeList==otherNodeID);
            if ~isempty(otherFreeNodeID),
                gMatrix(freeNodeIter,otherFreeNodeID) =  -1 *  strandConductance;
                gMatrix(otherFreeNodeID,freeNodeIter) =  -1 *  strandConductance;
            end%if ~isempty(otherFreeNodeID),
        end%if ~isempty(otherNodeID),
    end%for strandIter = 1:numStrandsAttached,
end%for freeNodeIter = 1:numFreeNodes,
            
for freeNodeIter = 1:numFreeNodes,
    gMatrix(freeNodeIter,freeNodeIter) = -1 * sum(gMatrix(freeNodeIter,:));
end

forcingVector = zeros(numFreeNodes,1);
for strandIter = 1:numVoltageConnectStrands,
    currentVoltageConnectStrandID = voltageConnectStrandList(strandIter);
    currentNodes = nodesConnectedByStrand(currentVoltageConnectStrandID,:);
    currentStrand = strands(currentVoltageConnectStrandID);
    iv = currentStrand.StartToEndIndices(2:end-1);
    strandConductance = strandConductanceList(currentVoltageConnectStrandID);
    if ismember(currentNodes(1),voltageNodeList),
        freeNodeID = find(freeNodeList==currentNodes(2));
    else%if ismember(currentNodes(1),voltageNodeList),
        freeNodeID = find(freeNodeList==currentNodes(1));
    end%if-else ismember(currentNodes(1),voltageNodeList),
    nodeID = freeNodeList(freeNodeID);
    yxCoords = cpMatrix(nodeCPID(nodeID),1:2);
    Ev = voltageMap(yxCoords(1),yxCoords(2));
    gMatrix(freeNodeID,freeNodeID) = gMatrix(freeNodeID,freeNodeID) + strandConductance;
    forcingVector(freeNodeID) = forcingVector(freeNodeID) + Ev.*strandConductance;
end%for strandIter = 1:numVoltageConnectStrands,

for strandIter = 1:numGroundConnectStrands,
    currentGroundConnectStrandID = groundConnectStrandList(strandIter);
    currentNodes = nodesConnectedByStrand(currentGroundConnectStrandID,:);
    currentStrand = strands(currentGroundConnectStrandID);
    iv = currentStrand.StartToEndIndices(2:end-1);
    strandConductance = strandConductanceList(currentGroundConnectStrandID);
    if ismember(currentNodes(1),groundNodeList),
        freeNodeID = find(freeNodeList==currentNodes(2));
    else%if ismember(currentNodes(1),voltageNodeList),
        freeNodeID = find(freeNodeList==currentNodes(1));
    end%if-else ismember(currentNodes(1),voltageNodeList),
    nodeID = freeNodeList(freeNodeID);
    yxCoords = cpMatrix(nodeCPID(nodeID),1:2);
    Eg = groundMap(yxCoords(1),yxCoords(2));
    gMatrix(freeNodeID,freeNodeID) = gMatrix(freeNodeID,freeNodeID) + strandConductance;
    forcingVector(freeNodeID) = forcingVector(freeNodeID) + Eg.*strandConductance;
end%for strandIter = 1:numGroundConnectStrands,



disp(['Inverting gMatrix : ',datestr(now)]);
tic
gInv = inv(gMatrix);
toc
voltageVector = gInv*forcingVector;


% analysisResults.cropYXZRange = cropYXZRange;
% analysisResults.groundLabelList = groundLabelList;
% analysisResults.sourceLabelList = sourceLabelList;
% analysisResults.sourceVoltage = sourceVoltage;
% analysisResults.groundVoltage = groundVoltage;
% analysisResults.startStrandID = startStrandID;
%-----
analysisResults.forcingVector = forcingVector;
analysisResults.voltageVector = voltageVector;
analysisResults.voltageNodeList = voltageNodeList;
% analysisResults.gMatrix = gMatrix;
% analysisResults.gInv = gInv;
analysisResults.strandsConnectedByNode = nodes;
analysisResults.nodesConnectedByStrand = nodesConnectedByStrand;
analysisResults.strandConductanceList = strandConductanceList;
analysisResults.nodeCPID = nodeCPID;
analysisResults.groundStrandList = groundStrandList;
analysisResults.voltageStrandList = voltageStrandList;
analysisResults.groundConnectStrandList = groundConnectStrandList;
analysisResults.voltageConnectStrandList  = voltageConnectStrandList ;
analysisResults.freeNodeList  = freeNodeList ;
analysisResults.groundNodeList = groundNodeList;
analysisResults.voltageNodeList = voltageNodeList;
analysisResults.radiusPower = radiusPower;
analysisResults.minRadius = minRadius;
analysisResults.minStrandLength = minStrandLength;




figure(1);plot(voltageVector)
figure(2);plot(forcingVector)
figure(3);plot(gMatrix*voltageVector)



globalDataStruct.cpMatrix = cpMatrix;
globalDataStruct.strands = strands;
fileRootName = globalDataStruct.fileRootName;
dateString = datestr(now,'yyyy_mm_dd-HH_MM_SS');
[pathName,fileName,ext] = fileparts(fileRootName);
saveFileName = fullfile(pathName,['SE',dateString,'-AnalysisResults-',fileName,ext]);
save(saveFileName,'analysisResults');




% % % % % if variableCleanupFlag == 1,
% % % % %     clear activeFlag ar attachedIter attachedNodes attachedStrands conductanceMultiplier 
% % % % %     clear connectedNodes cpMatrix currentConnection currentGroundConnectStrandID 
% % % % %     clear currentGroundNode currentMajorityLabel currentNodeID currentNodes 
% % % % %     clear currentStrand currentStrandID currentVoltageConnectStrandID currentVoltageNode dateString
% % % % %     clear e2 emptySpaces evns evnsSort ext f1 f2 ff fileName fileRootName firstEmptySlot 
% % % % %     clear forcingVector freeNodeID freeNodeIter freeNodeList
% % % % %     clear gInv gMatrix groundConnectStrandList groundNodeList groundStrandList iv k
% % % % %     clear loopStrands majorityLabel maskSize nodeCPID nodeCounter nodeIndexList nodeIter
% % % % %     clear nodes nodesConnectedByStrand numAttached numFreeNodes numGroundConnectStrands 
% % % % %     clear numGroundNodes numGroundStrands 
% % % % %     clear numNodes numStrands numStrandsAttached numVoltageConnectStrands 
% % % % %     clear numVoltageNodes numVoltageStrands otherFreeNodeID otherNodeID pathName
% % % % %     clear s1 s2 safeLabelList saveFileName si sourceConductance stei strandConductance
% % % % %     clear strandConductanceList strandIter strandLength strands strandsAttached svns 
% % % % %     clear svnsSort tempNodeList   voltageConnectStrandList 
% % % % %     clear voltageNodeList voltageStrandList voltageVector 
% % % % % end%if variableCleanupFlag == 1
% % % 
% % % %Stage 8) Create Masks for visualization
% % % disp(['Stage 8 : Create masks for visulization',datestr(now)]);
% % % disp(['Stage 8a : VoltageMask',datestr(now)]);
% % % %---
% % % GenerateCircuitVoltageMask
% % % disp(['Writing Mask : ',datestr(now)]);
% % % MakeHalfSizeMasks
% % % maketiff(halfMask,'HalfVoltageMask.tif');
% % % %---
% % % GenerateCircuitCurrentMask
% % % disp(['Writing Mask : ',datestr(now)]);
% % % MakeHalfSizeMasks
% % % maketiff(halfMask,'HalfCurrentMask.tif');
% % % %---
% % % CalculateTransitTime

