%This code removes free ends from a vessel set until no free ends remain (only closed
%loops)

%Load a globalDataStruct
%
%Note:  no connectivity changes can have occured since the latest restrandify round
%Changes to labels are okay.
%
%Labels: 100=Unlabeled; 200=Temp ; 300=Surface ; 400=Plunging ; 500=Micro ;
%         10=Unlabeled ; 20=Temp ; 30=Supply ; 40=Draining ;  50=Connecting ;
%Therefore: SurfaceArtery=330; SurfaceVein = 340;
%
%cpMatrix:
%   cols1-3:coords, col4:endPointFlag, col5:junctionFlag, col6:activeFlag
%   col7:label

if ~exist('variableCleanupFlag','var'), 
    variableCleanupFlag = 0;
end
%---
if (~exist('safeLabelList','var'))
    safeLabelList = [330 430 340 440];%the list labels that will not be trimmed even if free end
end%if ~exist...    

maskSize = globalDataStruct.maskSize;
strands = globalDataStruct.strands;
cpMatrix = globalDataStruct.cpMatrix;
si = globalDataStruct.si;
ar = globalDataStruct.ar;
numStrands = size(strands,2);

keepCyclingFlag = 1;
while keepCyclingFlag == 1,
    removalCounter = 0;
    %Cycle through each strand, check for dead-ends
    for strandIter = 1:numStrands,
        currentStrandID = strandIter;
        currentStrand = strands(currentStrandID);
        iv = currentStrand.StartToEndIndices(2:end-1);
        svns = currentStrand.StartVertexNeighborStrands;
        evns = currentStrand.EndVertexNeighborStrands;
        svns = svns(svns~=currentStrandID);
        evns = evns(evns~=currentStrandID);
        activeFlag = currentStrand.Active;
        majorityLabel = median(cpMatrix(iv,7));
        if ismember(majorityLabel,safeLabelList), activeFlag = 0; end
        if and(or(isempty(svns),isempty(evns)),activeFlag==1),
            %make all cp's on this strand inactive
            removalCounter = removalCounter + 1;
            cpMatrix(iv,6) = 0;
            strands(currentStrandID).Active = 0;
            if isempty(svns), cpMatrix(currentStrand.StartVertexIndex,6) = 0; end
            if isempty(evns), cpMatrix(currentStrand.EndVertexIndex,6) = 0; end
            %remove this strand from the neighborList of all its neighbors
            neighborStrandList = [svns,evns];
            numNeighbors = size(neighborStrandList,2);
            for neighborIter = 1:numNeighbors,
                currentNeighborID = neighborStrandList(neighborIter);
                currentNeighbor = strands(currentNeighborID);
                nsvns = currentNeighbor.StartVertexNeighborStrands;
                nevns = currentNeighbor.EndVertexNeighborStrands;
                if ismember(currentStrandID,nsvns),
                    adjustedList = nsvns(nsvns~=currentStrandID);
                    adjustedList = adjustedList(adjustedList~=currentNeighborID);
                    strands(currentNeighborID).StartVertexNeighborStrands = adjustedList;
                end%if ismember(currentStrandID,nsvns),
                %---
                if ismember(currentStrandID,nevns),
                    adjustedList = nevns(nevns~=currentStrandID);
                    adjustedList = adjustedList(adjustedList~=currentNeighborID);
                    strands(currentNeighborID).EndVertexNeighborStrands = adjustedList;
                end%if ismember(currentStrandID,esvns),
                %---
            end%for neighborIter = 1:numNeighbors,
        end%if and(or(isempty(svns),isempty(evns)),activeFlag==1),
    end%for strandIter = 1:numStrands,
    disp(removalCounter);
    if removalCounter == 0,
        keepCyclingFlag = 0;
    end%if removalCounter == 0,
end%while keepCyclingFlag = 1;


globalDataStruct.cpMatrix = cpMatrix;
globalDataStruct.strands = strands;
fileRootName = globalDataStruct.fileRootName;
dateString = datestr(now,'yyyy_mm_dd-HH_MM_SS');
[pathName,fileName,ext] = fileparts(fileRootName);
saveFileName = fullfile(pathName,['SE',dateString,'-DeadEndsTrimmed','-StrandEdit-',fileName,ext]);
save(saveFileName,'globalDataStruct');


if variableCleanupFlag == 1,
    clear activeFlag adjustedList ar cpMatrix currentNeighbor currentNeighborID
    clear currentStrand currentStrandID dateString evns ext fileName fileRootName
    clear iv keepCyclingFlag majorityLabel neighborIter neighborStrandList nevns
    clear nsvns numNeighbors numStrands pathName removalCounter saveFileName si
    clear strandIter strands svns 
end%if variableCleanupFlag == 1



    