%This code tracks vessels down from the closest labeled surface vessel.

%Load a globalDataStruct
%
%Note:  no connectivity changes can have occured since the latest restrandify round
%Changes to labels are okay.
%

%Labels: 100=Unlabeled; 200=Temp ; 300=Surface ; 400=Plunging ; 500=Micro ;
%         10=Unlabeled ; 20=Temp ; 30=Supply ; 40=Draining ;  50=Connecting ;
%
%Therefore: SurfaceArtery=330; SurfaceVein = 340;

tic
startingLabelList = [340];%the list of labels you are tracking to
associatedLabelList = [110 110];%the list of label to associate with strands tracked to startingLabelList
maxSearchDepth = 200;

% distanceMetric = 'Strands';
distanceMetric = 'VesselLength';


strands = globalDataStruct.strands;
cpMatrix = globalDataStruct.cpMatrix;
numStrands = size(strands,2);
%-----
%Generate a label for each strand based on the cpMatrix
strandTracker = [];
%neighborStruct will keep track of the search, arranged by strandID
neighborStruct = [];
for strandIter = 1:numStrands,
    neighborStruct(strandIter).searchDepth = Inf;
    neighborStruct(strandIter).completedFlag = 0;
    currentStrand = strands(strandIter);
    interiorVerts = currentStrand.StartToEndIndices(2:end-1);
    interiorLabels = cpMatrix(interiorVerts,7);
    if isempty(interiorLabels),
        uniqueLabels = 110;
    else%if isempty(interiorLabels),
        uniqueLabels = unique(interiorLabels);
    end%if isempty(interiorLabels),
    %-----
    tempDepthToTarget = Inf;
    tempBestLabel = 110;
    tempBestPath = [];
    %-----
    if size(uniqueLabels,1) > 1,
        labelHist = hist(interiorLabels,uniqueLabels);
        [histMax,maxPos] = max(labelHist);
        majorityLabel = uniqueLabels(maxPos);
        currentLabel = majorityLabel;
    else%if size(uniqueLabels,1) > 1,
        currentLabel = uniqueLabels(1);
    end%if-else size(uniqueLabels,1) > 1,
    strandTracker(strandIter).originalLabel = currentLabel;
    strandTracker(strandIter).trackedLabel = 110;
    strandTracker(strandIter).distanceFromStart = Inf;
    strandTracker(strandIter).upstreamNeighbor = Inf;
    currentTracker = strandTracker(strandIter);
    if ismember(currentTracker.originalLabel,startingLabelList),
%         currentTracker.trackedLabel = associatedLabelList(currentTracker.originalLabel==startingLabelList);
        currentTracker.trackedLabel = currentLabel;
%         currentTracker.distanceFromStart = 0;
        currentTracker.upstreamNeighbor = 0;
        strandTracker(strandIter) = currentTracker;
    end%if ismember(currentTracker.originalLabel,startingLabelList),
end%for strandIter = 1:numStrands,
clear strandIter
%----
%Find all strands with the starting labels
strandLabelList = [strandTracker(:).originalLabel];
startingStrandList = find(ismember(strandLabelList,startingLabelList));
numStartingStrands = size(startingStrandList,2);


%-----
%Begin tracking down 
toc
disp('Begin Tracking');
% disp('WARNING! NOT STARTING AT STRAND 1 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!');
attachmentList = zeros(1,numStrands);
sourceList = zeros(1,numStrands);
for startStrandIter = 1:numStartingStrands,
%     disp([num2str(startStrandIter),' of ',num2str(numStartingStrands)]);
    startingStrandID = startingStrandList(startStrandIter);
    startingStrand = strands(startingStrandID);
    startingLabel = strandTracker(startingStrandID).originalLabel;
    applyLabel = associatedLabelList(startingLabel == startingLabelList);
    currentLevelStrandIDList = startingStrandID;
    upstreamNeighborIDList = [0];
    cummulativeStrandList = [];
    totalAttached = 0;
    for searchDepth = 0:maxSearchDepth,
        cummulativeStrandList = [cummulativeStrandList,currentLevelStrandIDList];
        numCurrentLevelStrands = size(currentLevelStrandIDList,2);
        continueTrackingFlagList = false(size(currentLevelStrandIDList));
        attachmentCounter = 0;
        for kk = 1:numCurrentLevelStrands,
            currentStrandID = currentLevelStrandIDList(kk);
            currentStrand = strands(currentStrandID);
            upstreamNeighborID = upstreamNeighborIDList(kk);
            if upstreamNeighborID~=0,
                upstreamNeighborDistance = strandTracker(upstreamNeighborID).distanceFromStart;
            else
                upstreamNeighborDistance = 0;
            end
            storedDistance = strandTracker(currentStrandID).distanceFromStart;
            switch distanceMetric,
                case {'strands','Strands','StrandNum','strandNum'},
                    currentDistance = searchDepth;
                case {'vesselLength','length','VesselLength','Length'}
                    if searchDepth < 2, 
                        vesselLength = 0;
                    else%if searchDepth < 2,
                        vesselLength = size(strands(upstreamNeighborID).StartToEndIndices(2:end-1),2);
                    end%if searchDepth < 2,
                    currentDistance = upstreamNeighborDistance + vesselLength;
            end%switch distanceMetric,
            
            if currentDistance < storedDistance,
                strandTracker(currentStrandID).distanceFromStart = currentDistance;
                strandTracker(currentStrandID).upstreamNeighbor = upstreamNeighborID;
                strandTracker(currentStrandID).trackedLabel = applyLabel;
                if searchDepth == 0 ,
                    strandTracker(currentStrandID).trackedLabel = startingLabel;
                end%if searchDepth == 0 ,
                attachedVertices = [currentStrand.StartVertexNeighborStrands,currentStrand.EndVertexNeighborStrands];
                trueAttachments = unique(attachedVertices(attachedVertices~=currentStrandID));
                sourceVertices = repmat(currentStrandID,size(trueAttachments));
                numAttached = size(trueAttachments,2);
                attachmentList(attachmentCounter+1:attachmentCounter+numAttached) = trueAttachments;
                sourceList(attachmentCounter+1:attachmentCounter+numAttached) = sourceVertices;
                attachmentCounter = attachmentCounter+numAttached;
            end%if searchDepth < storedDistance,
        end%for kk = 1:numCurrentLevelStrands,
        totalAttached = totalAttached+attachmentCounter;
%         disp([attachmentCounter,totalAttached]);
        newStrandFlag = ~ismember(attachmentList(1:attachmentCounter),cummulativeStrandList);
        currentLevelStrandIDList = attachmentList(newStrandFlag);
        upstreamNeighborIDList = sourceList(newStrandFlag);
    end%for searchDepth = 1:maxSearchDepth,
end%for startStrandIter = 1:numStartingStrands,
toc

for strandIter = 1:numStrands;
    currentStrand = strands(strandIter);
    iv = currentStrand.StartToEndIndices(2:end-1);
    currentLabel = strandTracker(strandIter).trackedLabel;
    originalLabel = strandTracker(strandIter).originalLabel;
    currentUpstreamNeighbor = strandTracker(strandIter).upstreamNeighbor;
    currentDistanceFromStart = strandTracker(strandIter).distanceFromStart;
    strands(strandIter).upstreamNeighbor = currentUpstreamNeighbor;
    strands(strandIter).distanceFromStart = currentDistanceFromStart;
    if ismember(originalLabel,startingLabelList),
        currentLabel = originalLabel;
        strandTracker(strandIter).distanceFromStart = 0;
        strandTracker(strandIter).upstreamNeighbor = 0;
    end%if ismember(currentStrand.originalLabel,startingLabelList),
    cpMatrix(iv,7) = currentLabel;
end

%assign a distance to each pixel based on distance along strand as well as strand location
cpDistFromTarget = zeros(size(cpMatrix(:,1)))+Inf;
for strandIter = 1:numStrands,
    currentStrand = strands(strandIter);
    iv = currentStrand.StartToEndIndices(2:end-1);
    currentUpstreamNeighbor = strandTracker(strandIter).upstreamNeighbor;
    currentDistanceFromStart = strandTracker(strandIter).distanceFromStart;
    EndVertexNeighborStrands = currentStrand.EndVertexNeighborStrands;
    if ismember(currentUpstreamNeighbor,EndVertexNeighborStrands),
        iv = fliplr(iv);
    end%if ismember(currentUpstreamNeighbor,EndVertexNeighborStrands),
    %-----
    if ~isempty(iv),
        cpDistFromTarget(iv) = currentDistanceFromStart;
        addDistance = [1:size(iv,2)];
        if currentUpstreamNeighbor == 0,
            addDistance(:) = 0;
        end
        cpDistFromTarget(iv) = cpDistFromTarget(iv) + addDistance';
    end%if ~isempty(iv),
end%for strandIter = 1:numStrands,
    
        
        
        
        
        
        
        
        
globalDataStruct.cpMatrix = cpMatrix;
globalDataStruct.strands = strands;
globalDataStruct.cpDistFromTarget = cpDistFromTarget;
fileRootName = globalDataStruct.fileRootName;
dateString = datestr(now,'yyyy_mm_dd-HH_MM_SS');
[pathName,fileName,ext] = fileparts(fileRootName);
saveFileName = fullfile(pathName,['SE',dateString,'-SurfVeinDist','-StrandEdit-',fileName,ext]);
save(saveFileName,'globalDataStruct');


    
