function outputParameters = VidaCollapseNodeClusters(inputParameters)
nodeSegStructure = inputParameters.nodeSegStructure;
    cpList = nodeSegStructure.cpList;
    segmentList = nodeSegStructure.segmentList;
    nodeList = nodeSegStructure.nodeList;
    maskSize = nodeSegStructure.maskSize;
minSegmentLength = inputParameters.minSegmentLength;
%-----    

% save tempStage02


%COLLAPSE NODE CLUSTERS
disp(['Collapsing Node Clusters : ',datestr(now)]);tic
%-----
numSegments = size(segmentList,2)
numNodes = size(nodeList,2);
numCenterlinePoints = size(cpList,1);
%-----
numCollapsed = 0;
startClusterFindPoint = 1;
continueClusterCollapseFlag = 1;
numChar = 0;
tic

%pad the cpList to allow for new points
cpCounter = numCenterlinePoints;
cpList(numCenterlinePoints*2,5) = 0;


while continueClusterCollapseFlag,
    continueClusterCollapseFlag = 0;
% % % %     if mod(startClusterFindPoint,10)==0,
% % % %         fprintf(repmat('\b',[1,numChar]));
% % % %         t1 = toc;
% % % %         remainingTime = t1 * (numSegments-startClusterFindPoint)/startClusterFindPoint;
% % % %         pause(0.001);
% % % %         numChar = fprintf(['Segment ',num2str(startClusterFindPoint),' of ',num2str(numSegments),'. Est.Time: ',num2str(remainingTime),' sec.']);
% % % %     end%if mod(startClusterFindPoint,10)==0,
    for segmentIter = startClusterFindPoint:numSegments,
        currentSegment = segmentList{segmentIter};
        segmentLength = size(currentSegment,2);
        if segmentLength > 0 && segmentLength < minSegmentLength,
            continueClusterCollapseFlag = 1;
            break%break out of segmentIter loop
        end%if segmentLength>0 && segmentLength<minSegmentLength
        %no more short segments found.
        continueClusterCollapseFlag = 0;
    end%for segmentIter = startClusterFindPoint:numSegments,
    startClusterFindPoint = segmentIter + 1;
    if continueClusterCollapseFlag == 1,
        %-----
        %once a single short segment has been found
        %hunt for a short segments connected to it and mark the nodes
        firstPoint = currentSegment(1);
        lastPoint = currentSegment(end);
        clusterNodeList = [];
        if cpList(firstPoint,3)<0,
            clusterNodeList = [clusterNodeList,cpList(firstPoint,3)*-1];
        end%if cpList(firstPoint,3)<0,
        %---
        if cpList(lastPoint,3)<0,
            clusterNodeList = [clusterNodeList,cpList(lastPoint,3)*-1];
        end%if cpList(lastPoint,3)<0,
        currentClusterNodeListSize = size(clusterNodeList,2);
        currentClusterNodePointer = 1;
        moreNodesInClusterFlag = currentClusterNodeListSize>0;
        %disp('a');toc;
        while moreNodesInClusterFlag == 1,
            currentNode = clusterNodeList(currentClusterNodePointer);
            %find all nodes attached to this node by short segments
            segmentsAttachedList = nodeList{currentNode};
            numAttached = size(segmentsAttachedList,2);
            for attachedIter = 1:numAttached
                currentAttached = segmentsAttachedList(attachedIter);
                segmentPoints = segmentList{currentAttached};
                if size(segmentPoints,2) < minSegmentLength,
                    frontPoint = segmentPoints(1);
                    rearPoint = segmentPoints(end);
                    if cpList(frontPoint,3)<0 && ~ismember(cpList(frontPoint,3)*-1,clusterNodeList),
                        clusterNodeList = [clusterNodeList,cpList(frontPoint,3)*-1];
                    end%if cpList(frontPoint)<0 ...
                    %---
                    if cpList(rearPoint,3)<0 && ~ismember(cpList(rearPoint,3)*-1,clusterNodeList),
                        clusterNodeList = [clusterNodeList,cpList(rearPoint,3)*-1];
                    end%if cpList(rearPoint)<0 ...
                end%if size(segmentPoints,2) < minSegmentLength
            end%for attachedIter = 1:numAttached
            currentClusterNodePointer = currentClusterNodePointer + 1;
            if currentClusterNodePointer > size(clusterNodeList,2),
                moreNodesInClusterFlag = 0;
            end%if currentClusterNodePointer > size...
        end%while moreNodesInClusterFlag == 1,
        %disp('b');toc;
        %-----
        numNodesInCluster = size(clusterNodeList,2);
        if numNodesInCluster > 1
            %FOUND A CLUSTER!  COLLAPSE IT!
            numCollapsed = numCollapsed+1;
            %To account for small single-node loops where the front node
            %and the back node are the same node, unqiue the list.
            uniqueClusterNodeList = unique(clusterNodeList);
            uniqueNumNodesInCluster = size(uniqueClusterNodeList,2);
            %First find all segments attached to these nodes
            attachedSegmentsList = [];
            for nodeIter = 1:uniqueNumNodesInCluster,
                thisNodeSegments = nodeList{uniqueClusterNodeList(nodeIter)};
                attachedSegmentsList = [attachedSegmentsList,thisNodeSegments];
            end

            %----
            %Note, segments attached between nodes within the cluster
            %   will show up twice,  segments to the outside world show up once.
            uniqueSegmentList = unique(attachedSegmentsList);
            numUniqueSegments = size(uniqueSegmentList,2);
            internalSegments = [];
            externalSegments = [];
            for uniqueIter = 1:numUniqueSegments,
                currentSegment = uniqueSegmentList(uniqueIter);
                numOccurences = sum(attachedSegmentsList == currentSegment);
                if numOccurences == 1,
                    externalSegments = [externalSegments,currentSegment];
                else%if numOccurences == 1,
                    internalSegments = [internalSegments,currentSegment];
                end%if-else numOccurences == 1,
            end%for uniqueIter...
            %Step 1: in cpList: remove all centerline points on internal segments and nodes.
            %Step 2: in segmentList : remove all internal segments
            %Step 3: in nodeList : remove all cluster nodes
            %Step 4: in nodeList : create new "supernode" - attach it to all external segments.
            %Step 5: in cpList: create new centerpoint at cluster center, assign it to supernode.
            %Step 6: in all external segments, replace reference to cluster nodes with supernode.
            %---------
            %Step0a - find all the internal centerline points
            numInternalSegments = size(internalSegments,2);
            allInternalPoints = [];
            allInternalSegments = [];
            for internalSegmentIter = 1:numInternalSegments;
                thisInternalSegment = internalSegments(internalSegmentIter);
                internalPoints = segmentList{thisInternalSegment};
                allInternalSegments = [allInternalSegments,thisInternalSegment];
                allInternalPoints = [allInternalPoints,internalPoints];
            end%for internalSegmentIter = 1:numInternalSegments;
            %disp('c');toc;
            %---
            %Step0b - find all the external centerline points attached to
            %the cluster
            numExternalSegments = size(externalSegments,2);
            attachedExternalPoints = [];
            for externalSegmentIter = 1:numExternalSegments,
                thisExternalSegment = externalSegments(externalSegmentIter);
                externalPoints = segmentList{thisExternalSegment};
                if sum(externalPoints(1) == allInternalPoints),
                    %attachment point is front node
                    thisExternalPoint = externalPoints(2);
                else%if sum(enternalPoints(1) == allInternalPoints),
                    %attachment point is rear node
                    thisExternalPoint = externalPoints(end-1);
                end%if-else sum(enternalPoints(1) == allInternalPoints),
                attachedExternalPoints = [attachedExternalPoints;thisExternalPoint];
            end%for externalSegmentIter = 1:numExternalSegments,
            uniqueInternalPoints = unique(allInternalPoints);
            attachedExternalPointIndexList = cpList(attachedExternalPoints,1);
            internalPointIndexList = cpList(allInternalPoints,1);
            uniqueInternalPointIndexList = cpList(uniqueInternalPoints,1);
            internalRadii = cpList(uniqueInternalPoints,5);
            [ec1,ec2,ec3] = ind2sub(maskSize,attachedExternalPointIndexList);
            externalCoords = [ec1,ec2,ec3];
            [ic1,ic2,ic3] = ind2sub(maskSize,uniqueInternalPointIndexList);
            internalCoords = [ic1,ic2,ic3];
            %disp('d');toc;
            %---
            %Step0c - cycle through the internal points and find the one
            %that minimizes the sum distance to all external attachments.
            %This helps center the "supernode" near any true junction
            numUniqueInternalPoints = size(internalCoords,1);
            sumDist = [];
            for internalIter = 1:numUniqueInternalPoints,
                currentCoords = internalCoords(internalIter,:);
                diffList = repmat(currentCoords,numExternalSegments,1) - externalCoords;
                sumDist(internalIter) = sum(sqrt(sum(diffList.^2,2)));
            end%for internalIter = 1:numUniqueInternalPoints,
            %-----
            minDistPosList = find(sumDist == min(sumDist));
            minDistCoordList = internalCoords(minDistPosList,:);
            meanMinDistInternalCoord = round(mean(minDistCoordList,1));
            %-----
            %decide on a best coordinate for the supernode
            if numExternalSegments >1,
                bestSuperNodeCoord = meanMinDistInternalCoord;
                bestRadius = median(internalRadii);
            else%if numExternalSegments >1,
                %only one connection to the graph - use to mean coord to avoid excess trimming.
                bestSuperNodeCoord = round(mean(internalCoords,1));
                bestRadius = median(internalRadii);
            end%if-else numExternalSegments >1,
            %disp('e');toc;
            %---------
            %Step 1: in cpList: remove all centerline points on internal segments and nodes.
            %Step 2: in segmentList : remove all internal segments
            allInternalPoints = [];
            for internalSegmentIter = 1:size(internalSegments,2);
                thisInternalSegment = internalSegments(internalSegmentIter);
                internalPoints = segmentList{thisInternalSegment};
                segmentList{thisInternalSegment} = [];
                allInternalPoints = [allInternalPoints,internalPoints];
                cpList(internalPoints,3) = 0;
            end
            %disp('f');toc;
            %---
            %Step 3: in nodeList : remove all cluster nodes
            for clusterNodeIter = 1:numNodesInCluster,
                thisNode = clusterNodeList(clusterNodeIter);
                nodeList{thisNode} = [];
            end
            %disp('g');toc;
            %Step 4: in nodeList : create new "supernode" - attach it to all external segments.
            numNodes = numNodes+1;
            nodeList{numNodes} = externalSegments;
            %disp('h');toc;
            %Step 5: in cpList: create new centerpoint at cluster center, assign it to supernode.
            numCenterlinePoints = numCenterlinePoints+1;
            superNodeIndex = sub2ind(maskSize,bestSuperNodeCoord(1),bestSuperNodeCoord(2),bestSuperNodeCoord(3));
            numNeighbors = size(externalSegments,2);
            nodeIDTag = numNodes * -1;
            superNodePointer = numCenterlinePoints;
            cpList(superNodePointer,:) = [superNodeIndex, numNeighbors, nodeIDTag, numNeighbors bestRadius];
            %disp('i');toc;
            %Step 6: in all external segments, replace reference to cluster nodes with supernode.
            for externalSegmentIter = 1:size(externalSegments,2);
                thisExternalSegment = externalSegments(externalSegmentIter);
                externalPoints = segmentList{thisExternalSegment};
                [val,indEP,indAIP] = intersect(externalPoints,allInternalPoints);
                externalPoints(indEP) = numCenterlinePoints;
                segmentList{thisExternalSegment} = externalPoints;
            end%for for externalSegmentIter =...
            %disp('J');toc;
            %-----
            %Step 7: if there is a disconnect between the external segments and the supernode.
            %   Add centerline points to the segment.
            %First make a list of all coords of attached segments
            nearbyPointList = [];
            for externalSegmentIter = 1:numExternalSegments;
                currentSegment = externalSegments(externalSegmentIter);
                nearbyPointList = [nearbyPointList,segmentList{currentSegment}];
            end
            nearbyIndexList = cpList(unique(nearbyPointList),1);
            [nc1,nc2,nc3] = ind2sub(maskSize,nearbyIndexList);
            nearbyCoordList = [nc1,nc2,nc3];
            %-----
            pointOffsets = externalCoords - repmat(bestSuperNodeCoord,[numExternalSegments,1]);
            pointSeparations = sqrt(sum(pointOffsets.^2,2));
            disconnectIndex = find(pointSeparations>1.8);
            numDisconnects = size(disconnectIndex,1);
            for connectIter = 1:numDisconnects;
                currentDisconnectIndex = disconnectIndex(connectIter);
                currentSegment = externalSegments(currentDisconnectIndex);
                currentSegmentList = segmentList{currentSegment};
                reattachParameters.coord1 = bestSuperNodeCoord;
                reattachParameters.coord2 = externalCoords(currentDisconnectIndex,:);
                reattachParameters.nearbyCoordList = nearbyCoordList;
                    reattachParameters = VidaReattachStrand(reattachParameters);
                addedCoords = reattachParameters.addedCoords;
                numAddedCoords = size(addedCoords,1);
                %add to cpList
                addedPointers = [];
                for newCPIter = 1:numAddedCoords;
                    numCenterlinePoints = numCenterlinePoints + 1;
                    ac = addedCoords(newCPIter,:);
                    currentAddedIndex = sub2ind(maskSize,ac(1),ac(2),ac(3));
                    cpList(numCenterlinePoints,:) = [currentAddedIndex, 2, currentSegment, 2, bestRadius];
                    addedPointers = [addedPointers,numCenterlinePoints];%#ok<AGROW>
                end%for newCPIter = 1:numAddedCoords;
                %-----
                %insert into segmentList
                if currentSegmentList(1) == superNodePointer,
                    reattachedSegmentList = [superNodePointer,addedPointers,currentSegmentList(2:end)];
                else%if currentSegmentList(1) == superNodePointer,
                    if currentSegmentList(end)~=superNodePointer,'ooops',keyboard;end
                    reattachedSegmentList = [currentSegmentList(1:end-1),fliplr(addedPointers),superNodePointer];
                end%if-else currentSegmentList(1) == superNodePointer,
                segmentList{currentSegment} = reattachedSegmentList;
            end%for connectIter = 1:numDisconnects;
            %disp('K');toc;
            
        end%if numNodesInCluster>1
        

        
    end% if continueClusterCollapseFlag == 1,
end%while continueClusterCollapseFlag,

%Remove Padding on cpList:
cpList = cpList(cpList(:,1)~=0,:);

%-----
%Occasionally, an isolated cluster of nodes is collasped to a single unconnected "supernode".
%It will have a zero-neighbor listing in cpList with a (-)node# in column3
%but the correpsonding row in nodeList will be empty and cause errors.
%Remove the offending unconnected centerpoint/supernode 
cpList(cpList(:,2)==0,3) = 0;
fprintf(repmat('\b',[1,numChar]));
fprintf('\n\n');        
disp(['   Collapsed ',num2str(numCollapsed),' node clusters.']);
fprintf('   Completed. ');toc
%-----
nodeSegStructure.cpList = cpList;
nodeSegStructure.segmentList = segmentList;
nodeSegStructure.nodeList = nodeList;
%---



outputParameters = inputParameters;
outputParameters.nodeSegStructure = nodeSegStructure;
