function outExp = VidaTiling(inExp)
%Sample Script for full tiling of 3-channel strip data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

processedDataFolder = inExp.globalParameters.processedDataFolder;
fileRootName = inExp.globalParameters.fileRootName;
lastRerunStartDateNum = inExp.globalParameters.lastRerunStartDateNum;
voxelSizeInMicrons = inExp.globalParameters.voxelSizeInMicrons;
generateOutputTifsFlag = inExp.globalParameters.generateOutputTifsFlag;
%---
tilingParameters = inExp.tilingParameters;
registrationChannelName = tilingParameters.registrationChannelName;
inputFileRoots = tilingParameters.inputFileRoots;
outputFileRoots = tilingParameters.outputFileRoots;
subVolumeTileVerticalOverlapFraction = tilingParameters.subVolumeTileVerticalOverlapFraction;
subVolumeTileHorizontalOverlapFraction = tilingParameters.subVolumeTileHorizontalOverlapFraction;
marginFraction = tilingParameters.marginFraction;
reverseXDirectionFlag = tilingParameters.reverseXDirectionFlag;
reverseYDirectionFlag = tilingParameters.reverseYDirectionFlag;
searchRangeYXZInMicrons = tilingParameters.searchRangeYXZInMicrons;
filterSizeYXZInMicrons = tilingParameters.filterSizeYXZInMicrons;
projectionMethod = tilingParameters.projectionMethod;
defaultVerticalYXZOffsetInMicrons = tilingParameters.defaultVerticalYXZOffsetInMicrons;
defaultHorizontalYXZOffsetInMicrons = tilingParameters.defaultHorizontalYXZOffsetInMicrons;
twoSidedAlignmentFlag = tilingParameters.twoSidedAlignmentFlag;
sampleFileSuffix = tilingParameters.sampleFileSuffix;
%---
relativeOffsetTextFile = fullfile(processedDataFolder,[fileRootName,'-RigidTrans.txt']);
tilingFilesFolder = fullfile(processedDataFolder,'TilingFiles');
[mkdirSuccess,mkdirMessage,mkdirId] = mkdir(processedDataFolder,'TilingFiles');
%-----
registrationRoot = [registrationChannelName,'-',fileRootName];
numChannels = size(inputFileRoots,2);
%---
corrParameters = tilingParameters;
corrParameters.processedDataFolder = processedDataFolder;
corrParameters.fileRootName = fileRootName;
corrParameters.projectionMethod = projectionMethod;
corrParameters.generateOutputTifsFlag = generateOutputTifsFlag;
%----
defaultVerticalYXZOffsetInVoxels = defaultVerticalYXZOffsetInMicrons ./ voxelSizeInMicrons;
defaultHorizontalYXZOffsetInVoxels = defaultHorizontalYXZOffsetInMicrons ./ voxelSizeInMicrons;
searchRangeYXZInVoxels = searchRangeYXZInMicrons ./ voxelSizeInMicrons;
filterSizeYXZInVoxels = filterSizeYXZInMicrons ./ voxelSizeInMicrons;
%-----
corrParameters.searchRangeYXZInVoxels = searchRangeYXZInVoxels;
corrParameters.filterSizeYXZInVoxels = filterSizeYXZInVoxels;

%-----
%determine number of fields
numXFields = 1;
numYFields = 1;
fileNames = dir(fullfile(processedDataFolder,[inputFileRoots{1},'*']));
numMatchFiles = size(fileNames,1);
for nameFileIter = 1:numMatchFiles,
    currentName = fileNames(nameFileIter).name;
    cLetterPos = findstr(currentName,'-C');
    rLetterPos = findstr(currentName,'-R');
    cLetterPos = cLetterPos(end);
    rLetterPos = rLetterPos(end);
    extPos = findstr(currentName,'.tif');
    tempXPos = str2double(currentName(cLetterPos+2:rLetterPos-1));
    tempYPos = str2double(currentName(rLetterPos+2:extPos-1));
    numXFields = max(numXFields,tempXPos);
    numYFields = max(numYFields,tempYPos);
end%for nameFileIter = 1:numMatchFiles,
corrParameters.numXFields = numXFields;
corrParameters.numYFields = numYFields;
totalNumFields = numXFields.*numYFields;
%---
if totalNumFields>1,
    rigidTransFileName = relativeOffsetTextFile;
    rigidTransExists = exist(rigidTransFileName,'file');
    overwriteFlag = 0;
    fileExistsFlag = 0;
    if exist(rigidTransFileName,'file'),
        fileExistsFlag = 1;
        fileProps = dir(rigidTransFileName);
        fileDateNum = datenum(fileProps.date);
        if fileDateNum < lastRerunStartDateNum,
            overwriteFlag = 1;
            %Move the pre-tile files back to ProcessedData Folder
            preTileFiles = dir([tilingFilesFolder,'\*-',fileRootName,'-Z*']);
            numFiles = size(preTileFiles,1);
            for fileIter = 1:numFiles,
                currentFileName = preTileFiles(fileIter).name;
                fullOriginalName = fullfile(tilingFilesFolder,currentFileName);
                fullFinalName = fullfile(processedDataFolder,currentFileName);
                dosCommand = ['move "',fullOriginalName,'" "',fullFinalName,'"'];
                dos(dosCommand); 
            end%for fileIter = 1:numFiles,
        end%if fileDateNum < lastRerunStartDateNum,
    end%if exist(rigidTransFileName,'file'),
    if and(fileExistsFlag==1,overwriteFlag==0),
        %Load results of previously calculated correlations (offsets)
        disp('...Correlations already performed, skipping...');
        disp(['     Delete file : ',relativeOffsetTextFile,' to force re-calculation']);
        %---
        YXZOffsetMatrix = dlmread(relativeOffsetTextFile);
        OffsetMatrix = [YXZOffsetMatrix(:,1),YXZOffsetMatrix(:,2),YXZOffsetMatrix(:,3)];
        relativeYXZ{1} = [0 0 0];
        for pairIter = 2:size(YXZOffsetMatrix,1)+1;
            relativeYXZ{pairIter} = OffsetMatrix(pairIter-1,:);
        end%for pairIter    
    else%if and(fileExistsFlag==1,overwriteFlag==0),
        disp(['Correlating Stacks for tiling,  ',datestr(now,'mm/dd/yy HH:MM:SS')]);
        %determine order of files to open;
        columnNum = (1:numXFields);
        rowNum = (1:numYFields);
        if reverseXDirectionFlag == 1, columnNum = (numXFields:-1:1); end
        if reverseYDirectionFlag == 1, rowNum = (numYFields:-1:1); end
        %---
        %Now use correlateStacks.m to get all the relative position
        sampleFileName = fullfile(processedDataFolder,[registrationRoot,sampleFileSuffix]);
        postname = '.tif';
        %---
        relativeYXZ = [];
        relativeYXZDouble = [];
        for xIter = 1:numXFields
            for yIter = 1:numYFields
                corrParameters.inputFileName2 = [registrationRoot,'-Z01-C',num2str(columnNum(xIter),'%02d'),'-R',num2str(rowNum(yIter),'%02d'),postname];
                corrParameters.inputFileName2 = fullfile(processedDataFolder,corrParameters.inputFileName2);
                fileInfo = imfinfo(corrParameters.inputFileName2);
                zSize = size(fileInfo,2);
                ySize = fileInfo(1).Height;
                xSize = fileInfo(1).Width;
                if yIter == 1, %top of column
                    if xIter == 1, %very first stack
                        relativeYXZ{1,1} = [0 0 0];
                    else%if xIter == 1,
                        %ALIGN TO STACK ON LEFT
                        overlapSize = round(subVolumeTileHorizontalOverlapFraction * xSize);
                        subVolumeYXZLimits(1) = ceil(marginFraction*ySize);
                        subVolumeYXZLimits(2) = ySize - floor(marginFraction*ySize);
                        subVolumeYXZLimits(3) = ceil(marginFraction*xSize);
                        subVolumeYXZLimits(4) = overlapSize - floor(marginFraction*xSize);
                        subVolumeYXZLimits(5) = ceil(marginFraction*zSize);
                        subVolumeYXZLimits(6) = zSize - floor(marginFraction*zSize);
                        subVolumeTileHorizontalYXZLimits = subVolumeYXZLimits;
                        %-----
                        corrParameters.inputFileName1 = [registrationRoot,'-Z01-C',num2str(columnNum(xIter-1),'%02d'),'-R',num2str(rowNum(yIter),'%02d'),postname];
                        corrParameters.inputFileName1 = fullfile(processedDataFolder,corrParameters.inputFileName1);
                        corrParameters.yxzGuessInVoxels = defaultHorizontalYXZOffsetInVoxels;
                        corrParameters.subVolumeYXZLimits = subVolumeYXZLimits;
                            relativeYXZ{yIter,xIter} = VidaCorrelateStacks(corrParameters);
                        disp([num2str([yIter,xIter]),'    YXZ  =  ',num2str([relativeYXZ{yIter,xIter}])]);  
                    end%if xIter==1,
                else%if yIter==1,
                    %ALIGN TO STACK ABOVE
                    overlapSize = round(subVolumeTileVerticalOverlapFraction * ySize);
                    subVolumeYXZLimits(1) = ceil(marginFraction*ySize);
                    subVolumeYXZLimits(2) = overlapSize - floor(marginFraction*ySize);
                    subVolumeYXZLimits(3) = ceil(marginFraction*ySize);
                    subVolumeYXZLimits(4) = xSize - floor(marginFraction*xSize);
                    subVolumeYXZLimits(5) = ceil(marginFraction*zSize);
                    subVolumeYXZLimits(6) = zSize - floor(marginFraction*zSize);
                    %------
                    corrParameters.inputFileName1 = [registrationRoot,'-Z01-C',num2str(columnNum(xIter),'%02d'),'-R',num2str(rowNum(yIter-1),'%02d'),postname];
                    corrParameters.inputFileName1 = fullfile(processedDataFolder,corrParameters.inputFileName1);
                    corrParameters.yxzGuessInVoxels = defaultVerticalYXZOffsetInVoxels;
                    corrParameters.subVolumeYXZLimits = subVolumeYXZLimits;
                        relativeYXZ{yIter,xIter} = VidaCorrelateStacks(corrParameters);
                    %---
                    %DOUBLE ALIGN (Align to left) IF REQUESTED
                    if twoSidedAlignmentFlag == 1,
                        if xIter > 1,
                        %ALIGN TO STACK ON LEFT
                        overlapSize = round(subVolumeTileHorizontalOverlapFraction * xSize);
                        subVolumeYXZLimits(1) = ceil(marginFraction*ySize);
                        subVolumeYXZLimits(2) = ySize - floor(marginFraction*ySize);
                        subVolumeYXZLimits(3) = ceil(marginFraction*xSize);
                        subVolumeYXZLimits(4) = overlapSize - floor(marginFraction*xSize);
                        subVolumeYXZLimits(5) = ceil(marginFraction*zSize);
                        subVolumeYXZLimits(6) = zSize - floor(marginFraction*zSize);
                        %-----
                        corrParameters.inputFileName1 = [registrationRoot,'-Z01-C',num2str(columnNum(xIter-1),'%02d'),'-R',num2str(rowNum(yIter),'%02d'),postname];
                        corrParameters.inputFileName1 = fullfile(processedDataFolder,corrParameters.inputFileName1);
                        corrParameters.yxzGuessInVoxels = defaultHorizontalYXZOffsetInVoxels;
                        corrParameters.subVolumeYXZLimits = subVolumeYXZLimits;
                            relativeYXZDouble{yIter,xIter} = VidaCorrelateStacks(corrParameters);
                        end %if xIter>1
                    end%if twoSidedAlignmentFlag ==1
                    %---
                    disp([num2str([yIter,xIter]),'    YXZ  =  ',num2str([relativeYXZ{yIter,xIter}])]); 
                    pause(0.1);
                end%if yIter==1,
                %-----
                %Calculate the absolute position
                r = relativeYXZ{yIter,xIter};
                if yIter == 1, %top of column
                    if xIter == 1, %very first stack
                        abspos{yIter,xIter} = [0 0 0];
                    else%if xIter == 1, 
                        abspos{yIter,xIter} = abspos{yIter,xIter-1} + r;
                    end%if xIter==1,
                else%if yIter==1,
                    abspos{yIter,xIter} = abspos{yIter-1,xIter} + r;
                end%if-else yIter==1,
                %---
                if twoSidedAlignmentFlag == 1,
                    if (xIter>1 && yIter>1),
                        absposDouble = abspos{yIter,xIter-1} + c{yIter,xIter};
                        averageAbspos = (abspos{yIter,xIter} + absposDouble)/2;
                        abspos{yIter,xIter} = double(floor(averageAbspos));
                        disp(['             ',num2str([relativeYXZDouble{yIter,xIter}])]); 
                    end%if AND(xIter>1,yIter>1),
                end%if twoSidedAlignmentFlag == 1,
                %---
                temp = abspos{yIter,xIter};
                xpos(yIter,xIter) = temp(1);
                ypos(yIter,xIter) = temp(2);
                zpos(yIter,xIter) = temp(3);
            end%for yIter=1:numy
        end%for xIter=1:numx
        corrParameters.relativeYXZ = relativeYXZ;
        corrParameters.relativeYXZDouble = relativeYXZDouble;
        %-----
        %Now calculate required size of montage and final coordinates of each stack
        minx = abs(min(min(xpos)))+1;
        miny = abs(min(min(ypos)))+1;
        minz = abs(min(min(zpos)))+1;
        for xIter = 1:numXFields
            for yIter = 1:numYFields
                finalYXZ{yIter,xIter} = abspos{yIter,xIter} + [miny, minx, minz];
                temp = finalYXZ{yIter,xIter};
                finalX(yIter,xIter) = temp(1);
                finalY(yIter,xIter) = temp(2);
                finalZ(yIter,xIter) = temp(3);
            end
        end
        %-----
        %Calculate necessary size for final montage
        myinfo = imfinfo(sampleFileName);
        zSize = length(myinfo);
        xSize = myinfo(1).Width;
        ySize = myinfo(1).Height;
        %---
        finalXSize = xSize + max(max(finalX))-1;
        finalYSize = ySize + max(max(finalY))-1;
        finalZSize = zSize + max(max(finalZ))-1;
        %---
        corrParameters.finalX = finalX;
        corrParameters.finalY = finalY;
        corrParameters.finalZ = finalZ;
        corrParameters.finalYXZ = finalYXZ;
        corrParameters.finalYXZSize = [finalXSize,finalYSize,finalZSize];
        %-----
        %Write the relativeOffestTextFile
        offsetMatrix = [];
        YXZOffsetMatrix = [];
        for pairIter = 2:numel(corrParameters.relativeYXZ);
            offsetMatrix = [offsetMatrix; corrParameters.relativeYXZ{pairIter}];
            YXZOffsetMatrix = [offsetMatrix(:,1),offsetMatrix(:,2),offsetMatrix(:,3)];
        end%for pairIter
        dlmwrite(relativeOffsetTextFile,YXZOffsetMatrix,' ');
        OffsetMatrix = [YXZOffsetMatrix(:,1),YXZOffsetMatrix(:,2),YXZOffsetMatrix(:,3)];
        relativeYXZ{1} = [0 0 0];
        for pairIter = 2:size(YXZOffsetMatrix,1)+1;
            relativeYXZ{pairIter} = OffsetMatrix(pairIter-1,:);
        end%for pairIter   
    end%if and(fileExistsFlag==1,overwriteFlag==0),
else%if totalNumFields>1,
    relativeYXZ{1} = [0 0 0];
end%if else totalNumFields>1,

%-----
%Write the Montage Files
numChannels = size(outputFileRoots,2);
for channelIter = 1:numChannels,
    temp_counter = min(channelIter+1,numChannels);
    currentChannelName = outputFileRoots{channelIter};
    currentInputFileRoot = inputFileRoots{channelIter};
    currentOutputFileRoot = outputFileRoots{channelIter};
    nextOutputFileRoot = outputFileRoots{temp_counter};
    sampleFileName = fullfile(processedDataFolder,[currentInputFileRoot,'-',fileRootName,'-Z01-C01-R01.tif']);
    outputFileNameSub = [fileRootName,'-Stacked-',currentOutputFileRoot,'.mat'];
    outputFileName = fullfile(processedDataFolder,outputFileNameSub);
    nextOutputFileName = regexprep(outputFileName,currentOutputFileRoot,nextOutputFileRoot);
    currentFullInputFileRoot = fullfile(processedDataFolder,[currentInputFileRoot,'-',fileRootName,]);
    disp(['Tiling ', currentOutputFileRoot, ' Channel']);
    %-----
    overwriteFlag = 0;
    fileExistsFlag = 0;
    myquery1 = exist(outputFileName,'file');%2 if file exists
    myquery2 = exist(nextOutputFileName,'file');%2 if file exists
    if channelIter == numChannels, myquery2 = 0; end
    if (myquery1+myquery2 == 4), 
        fileExistsFlag = 1;
        fileProps = dir(outputFileName);
        fileDateNum = datenum(fileProps.date);
        if fileDateNum < lastRerunStartDateNum,
            overwriteFlag = 1;
            %Move the pre-tile files back to ProcessedData Folder
            preTileFiles = dir([tilingFilesFolder,'\*-',fileRootName,'-Z*']);
            numFiles = size(preTileFiles,1);
            for fileIter = 1:numFiles,
                currentFileName = preTileFiles(fileIter).name;
                fullOriginalName = fullfile(tilingFilesFolder,currentFileName);
                fullFinalName = fullfile(processedDataFolder,currentFileName);
                dosCommand = ['move "',fullOriginalName,'" "',fullFinalName,'"'];
                dos(dosCommand); 
            end%for fileIter = 1:numFiles,
        end%if fileDateNum < lastRerunStartDateNum,
    end%if (myquery1+myquery2 == 4), 
    if and(fileExistsFlag==1,overwriteFlag==0),
        disp('...Skipping-Previously calculated');
        corrParameters.stackedVolumeFileNames{channelIter} = outputFileName;
    else%if and(fileExistsFlag==1,overwriteFlag==0),
        corrParameters.sampleFileName = sampleFileName;
        corrParameters.outputFileName = outputFileName;
        corrParameters.currentFullInputFileRoot = currentFullInputFileRoot;
        corrParameters.relativeYXZ = relativeYXZ;
        disp(['Writing Montaged .tif file, ',outputFileNameSub]);
            corrParameters = VidaTileImages(corrParameters);
        corrParameters.stackedVolumeFileNames{channelIter} = corrParameters.outputFileName;
        corrParameters.stackedVolumeFileNamesSub{channelIter} = outputFileNameSub;
        corrParameters.stackedVolumeFolders{channelIter} = processedDataFolder;
    end%if and(fileExistsFlag==1,overwriteFlag==0),
end%for channelIter = 1:numChannels,

%-----
%DEFINE CHANNEL NAMES
stackedFileNames = corrParameters.stackedVolumeFileNames;
imageStackSize = corrParameters.imageStackSize;

[tempPath,tempFileName,tempExt,tempVer] = fileparts(stackedFileNames{1});
channels.cellChannelString = tempFileName(findstr(tempFileName,'-')+1:end);
[tempPath,tempFileName,tempExt,tempVer] = fileparts(stackedFileNames{2});
channels.vascChannelString = tempFileName(findstr(tempFileName,'-')+1:end);
channels.neunChannelString = '';
if numChannels >2,
    [tempPath,tempFileName,tempExt,tempVer] = fileparts(stackedFileNames{3});
    channels.neunChannelString = tempFileName(findstr(tempFileName,'-')+1:end);
end
%-----
%Clean up original individual files to TilingFiles Folder
cleanUpFiles = dir([processedDataFolder,'\*-',fileRootName,'-Z*']);
numFiles = size(cleanUpFiles,1);
for fileIter = 1:numFiles,
    currentFileName = cleanUpFiles(fileIter).name;
    fullOriginalName = fullfile(processedDataFolder,currentFileName);
    fullFinalName = fullfile(tilingFilesFolder,currentFileName);
    dosCommand = ['move "',fullOriginalName,'" "',fullFinalName,'"'];
    dos(dosCommand); 
end%for fileIter = 1:numFiles,
%-----
outExp = inExp;
outExp.tilingParameters.channelNames = outputFileRoots;
outExp.tilingParameters.relativeOffsetTextFile = relativeOffsetTextFile;
outExp.tilingParameters.tilingFilesFolder = tilingFilesFolder;
%---
outExp.globalParameters.numChannels = numChannels;
outExp.globalParameters.stackedFileNames = stackedFileNames;
outExp.globalParameters.channels = channels;
outExp.globalParameters.cellMaskSize = imageStackSize;
outExp.globalParameters.vascMaskSize = imageStackSize;
outExp.completionDates.tilingCompletionDate = datestr(now);



return






