function outputParameters=VidaReconstructionRestitch(inputParameters)

fileRootName = inputParameters.fileRootName;
intermediateResultsFolder = inputParameters.intermediateResultsFolder;
processedDataFolder = inputParameters.processedDataFolder;
vascBlockRadiusMapFileNameList = inputParameters.vascBlockRadiusMapFileNameList;
fullStackSize = inputParameters.fullStackSize;
blockGrid = inputParameters.blockGrid;
subVolumeOverlapSize = inputParameters.subVolumeOverlapSize;
%-----
vascReconstructedMaskFileName = fullfile(processedDataFolder,[fileRootName,'-VascReconstructedMask.mat']);
vascRadiusMapFileName = fullfile(processedDataFolder,[fileRootName,'-VascRadiusMap.mat']);
%-----
if isfield(inputParameters,'overrideVascReconstructedMaskFullFileName'),
    vascReconstructedMaskFileName = inputParameters.overrideVascReconstructedMaskFullFileName;
end%if isfield(inputParameters,'overrideVascReconstructedMaskFullFileName')
%-----
if isfield(inputParameters,'overrideVascRadiusMapFullFileName'),
    vascRadiusMapFileName = inputParameters.overrideVascRadiusMapFullFileName;
end%if isfield(inputParameters,'overrideVascRadiusMapFullFileName')
%-----



disp(['Restitching Reconstructed Blocks']);

pixelRedundancy = 1;

mybit = uint8(0)>0;
vascReconstructedMask = repmat(mybit,fullStackSize);
vascRadiusMap = zeros(size(vascReconstructedMask));
overlapOffset = ceil(subVolumeOverlapSize/2) - pixelRedundancy;

minX1 = Inf; minX2 = Inf; minX3 = Inf;
maxX1 = 0; maxX2 = 0; maxX3 = 0;
for gridIter = 1:length(blockGrid);
    blockIterLims=blockGrid{gridIter};
    minX1 = min(minX1,blockIterLims.x1min);
    minX2 = min(minX2,blockIterLims.x2min);
    minX3 = min(minX3,blockIterLims.x3min);
    maxX1 = max(maxX1,blockIterLims.x1max);
    maxX2 = max(maxX2,blockIterLims.x2max);
    maxX3 = max(maxX3,blockIterLims.x3max);
end


%Concatenate the lists of centerline point with appropriate block offsets
compiledPixelList = [];
compiledRadiusList = [];
for blockIter = 1:length(blockGrid)
    blockIterLims = blockGrid{blockIter};    fullPixelList = [];
    load(vascBlockRadiusMapFileNameList{blockIter},'vascBlockRadiusMap','extraData','-mat');
    if isstruct(vascBlockRadiusMap),
        pixelList = vascBlockRadiusMap.pixelList;
        maskSize = vascBlockRadiusMap.maskSize;
        radiusList = vascBlockRadiusMap.radiusList;
    else%if isstruct(vascBlockRadiusMap);
        pixelList = extraData.centerlinePixelList;
        maskSize = size(vascBlockRadiusMap);
        radiusList = extraData.radiusList;
    end%if isstruct(vascBlockRadiusMap);
    %-----
    [x1List,x2List,x3List] = ind2sub(maskSize,pixelList);
    %Crop the list to minimize overlap
    x1MinSubset = ones(size(x1List));
    x1MaxSubset = ones(size(x1List));
    x2MinSubset = ones(size(x2List));
    x2MaxSubset = ones(size(x2List));
    x3MinSubset = ones(size(x3List));
    x3MaxSubset = ones(size(x3List));
    %---
    if blockIterLims.x1min ~= minX1, x1MinSubset = x1List >= (1+overlapOffset); end
    if blockIterLims.x1max ~= maxX1, x1MaxSubset = x1List < (maskSize(1)-overlapOffset); end
    if blockIterLims.x2min ~= minX2, x2MinSubset = x2List >= (1+overlapOffset); end
    if blockIterLims.x2max ~= maxX2, x2MaxSubset = x2List < (maskSize(2)-overlapOffset); end
    if blockIterLims.x3min ~= minX3, x3MinSubset = x3List >= (1+overlapOffset); end
    if blockIterLims.x3max ~= maxX3, x3MaxSubset = x3List < (maskSize(3)-overlapOffset); end
    %-----
    x1Sub = and(x1MinSubset,x1MaxSubset);
    x2Sub = and(x2MinSubset,x2MaxSubset);
    x3Sub = and(x3MinSubset,x3MaxSubset);
    allSub = and(and(x1Sub,x2Sub),x3Sub);
    x1ListSub = x1List(allSub);
    x2ListSub = x2List(allSub);
    x3ListSub = x3List(allSub);
    %-----
    radiusListSub = radiusList(allSub);
    %-----
    x1ListSub = x1ListSub + blockIterLims.x1min - 1;
    x2ListSub = x2ListSub + blockIterLims.x2min - 1;
    x3ListSub = x3ListSub + blockIterLims.x3min - 1;
    %-----
    adjustedPixelList = sub2ind(fullStackSize,x1ListSub,x2ListSub,x3ListSub);
    compiledPixelList = cat(1,compiledPixelList,adjustedPixelList);
    compiledRadiusList = cat(1,compiledRadiusList,radiusListSub);
    %-----
end%for blockIter = 1:length(blockGrid)

vascRadiusMap = [];
vascRadiusMap.pixelList = compiledPixelList;
vascRadiusMap.maskSize = fullStackSize;
vascRadiusMap.radiusList = compiledRadiusList;
saveSize = (getfield(whos('vascRadiusMap'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save(vascRadiusMapFileName,'vascRadiusMap',scs);

    
%-----
outputParameters = inputParameters;
outputParameters.vascReconstructedMaskFileName = vascReconstructedMaskFileName; 
outputParameters.vascRadiusMapFileName = vascRadiusMapFileName;






