function outputParameters = VidaIsolateMicroVessels(inputParameters)
intermediateResultsFolder = inputParameters.intermediateResultsFolder;
fileRootName = inputParameters.fileRootName;
vascRadiusFilesFolder = inputParameters.vascRadiusFilesFolder;
lastRerunStartDateNum = inputParameters.lastRerunStartDateNum;
vascReconstructedMaskFileName = inputParameters.vascReconstructedMaskFileName;
displayMicroVesselsFlag = inputParameters.displayMicroVesselsFlag;
maxBlockSizeInMicrons = inputParameters.maxBlockSizeInMicrons;
blockOverlapSizeInMicrons = inputParameters.blockOverlapSizeInMicrons;
morphologicalCloseSizeInMicrons = inputParameters.morphologicalCloseSizeInMicrons;
radiusListStartInMicrons = inputParameters.radiusListInMicrons;
radiusListStepInMicrons = inputParameters.radiusListStepInMicrons;
radiusListStopInMicrons = inputParameters.radiusListStopInMicrons;
deltaRInMicrons = inputParameters.deltaRInMicrons;
voxelSizeInMicrons = inputParameters.voxelSizeInMicrons;
%-----
radiusListInMicrons = [radiusListStartInMicrons:radiusListStepInMicrons:radiusListStopInMicrons];
%-----
maxBlockSize = maxBlockSizeInMicrons ./ voxelSizeInMicrons;
blockOverlapSize = blockOverlapSizeInMicrons ./ voxelSizeInMicrons;
morphologicalCloseSize = morphologicalCloseSizeInMicrons ./ voxelSizeInMicrons;
radiusList = radiusListInMicrons ./ voxelSizeInMicrons;
deltaR = deltaRInMicrons ./ voxelSizeInMicrons;
%-----

load (vascReconstructedMaskFileName,'vascReconstructedMask');
%noteToSelf : must implement the VidaReconstructVascFromRadii code


keyboard

fullStackSize = size(VascMask);
numBlocks = ceil(fullStackSize./maxBlockSize);
blockSize = ceil(fullStackSize./numBlocks);
startY = [1:blockSize(1):fullStackSize(1)];
startY = startY(1:numBlocks(1));
stopY = startY+blockSize(1)+blockOverlapSize;
stopY(end) = fullStackSize(1);
startX = [1:blockSize(2):fullStackSize(2)];
startX = startX(1:numBlocks(2));
stopX = startX+blockSize(2)+blockOverlapSize;
stopX(end) = fullStackSize(2);
startZ = [1:blockSize(3):fullStackSize(3)];
startZ = startZ(1:numBlocks(3));
stopZ = startZ+blockSize(3)+blockOverlapSize;
stopZ(end) = fullStackSize(3);

skipSize = round(blockOverlapSize/2);


blockCounter = 0;
for zIter = 1:numBlocks(3),
    for xIter = 1:numBlocks(2),
        for yIter = 1:numBlocks(1),
            blockCounter = blockCounter + 1;
            subBlock{blockCounter}.startPos = [startY(yIter) startX(xIter) startZ(zIter)];
            subBlock{blockCounter}.stopPos = [stopY(yIter) stopX(xIter) stopZ(zIter)];
            firstY = startY(yIter) + skipSize;
            firstX = startX(xIter) + skipSize;
            firstZ = startZ(zIter) + skipSize;
            internalY1 = 1+skipSize;
            internalX1 = 1+skipSize;
            internalZ1 = 1+skipSize;
            internalY2 = stopY(yIter)-startY(yIter)+1;
            internalX2 = stopX(xIter)-startX(xIter)+1;
            internalZ2 = stopZ(zIter)-startZ(zIter)+1;
            if yIter==1, firstY = 1; internalY1 = 1; end;
            if xIter==1, firstX = 1; internalX1 = 1; end;
            if zIter==1, firstZ = 1; internalZ1 = 1; end;            
            subBlock{blockCounter}.firstPos = [firstY, firstX, firstZ];
            subBlock{blockCounter}.internalPos1 = [internalY1, internalX1, internalZ1];
            subBlock{blockCounter}.internalPos2 = [internalY2, internalX2, internalZ2];
            subBlock{blockCounter}.fileName = fullfile(vascRadiusFilesFolder,[fileRootName,'-VM-corr-BWDist-v3-block',num2str(blockCounter),'.mat']);
        end
    end
end

numSubBlocks = blockCounter;

for blockIter = 1:numSubBlocks,
    disp(['Closing and Calculating bwDist on Block ',num2str(blockIter),' of ',num2str(numSubBlocks),':',datestr(now,'mm/dd/yy HH:MM:SS')]);
    startPos = subBlock{blockIter}.startPos;
    stopPos =  subBlock{blockIter}.stopPos;
    firstPos = subBlock{blockIter}.firstPos;
    internalPos1 = subBlock{blockIter}.internalPos1;
    internalPos2 = subBlock{blockIter}.internalPos2;
    subBlockFileName = subBlock{blockIter}.fileName;
    vascmaskcorrblock = VascMask(startPos(1):stopPos(1),startPos(2):stopPos(2),startPos(3):stopPos(3));
    vascmaskcorrfnames{blockIter} = subBlockFileName;
    overwriteFlag = 0;
    fileExistsFlag = 0;
    if exist(subBlockFileName,'file'),
        fileExistsFlag = 1;
        fileProps = dir(subBlockFileName);
        fileDateNum = datenum(fileProps.date);
        if fileDateNum < lastRerunStartDateNum,
            overwriteFlag = 1;
        end%if fileDateNum < lastRerunStartDateNum,
    end%if exist(subBlockFileName,'file'),
    if and(fileExistsFlag==1,overwriteFlag==0),
        disp('...Skipping-Previously calculated');
    else%if and(fileExistsFlag==1,overwriteFlag==0),
        %first close the mask
        closedVascBlock = imclose(vascmaskcorrblock,circle3d(morphologicalCloseSize));
        D = bwdist(~closedVascBlock);
        saveSize = (getfield(whos('D'),'bytes')>0.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(subBlockFileName,'D','vascmaskcorrblock',scs);
    end%if and(fileExistsFlag==1,overwriteFlag==0),
end%for blockIter = 1:numSubBlocks,   


vesselSum = [];
for Ri=1:length(radiusList)
    % Rcap is the acceptable capillary vessel radius
    Rcap=radiusList(Ri);
    disp(['Finding capillary only (r<=',num2str(Rcap),') blocks']);
    clear VascMask;
    microVesselMaskFileNameList{Ri}=regexprep(vascReconstructedMaskFileName,'\.mat',['-R=',num2str(Rcap),'.mat']);
    overwriteFlag = 0;
    fileExistsFlag = 0;
    
% % %     if exist(microVesselMaskFileNameList{Ri},'file'),
% % %         fileExistsFlag = 1;
% % %         fileProps = dir(microVesselMaskFileNameList{Ri});
% % %         fileDateNum = datenum(fileProps.date);
% % %         if fileDateNum < lastRerunStartDateNum,
% % %             overwriteFlag = 1;
% % %         end%if fileDateNum < lastRerunStartDateNum,
% % %     end%if exist(microVesselMaskFileNameList{Ri},'file'),
    
    if and(fileExistsFlag==1,overwriteFlag==0),
        load(microVesselMaskFileNameList{Ri},'CapMask','-mat');
        disp('...Skipping - previously calculated');
    else%if and(fileExistsFlag==1,overwriteFlag==0),
        CapMask=logical(zeros(fullStackSize(1),fullStackSize(2),fullStackSize(3),'uint8'));
        for jBLK=1:numSubBlocks,
            % load each vascular binary block
            load(vascmaskcorrfnames{jBLK},'D','vascmaskcorrblock','-mat');
            disp(['finding large vessel interiors of block ',num2str(jBLK),' - ',datestr(now,'mm/dd/yy HH:MM:SS')]);
            Q = (D >= Rcap);
            RMB = imdilate(Q,circle3d(ceil(2*(Rcap+deltaR))));
            capillaryMask = and(vascmaskcorrblock,~RMB);
            startPos = subBlock{jBLK}.startPos;
            stopPos = subBlock{jBLK}.stopPos;
            firstPos = subBlock{jBLK}.firstPos;
            internalPos1 = subBlock{jBLK}.internalPos1;
            internalPos2 = subBlock{jBLK}.internalPos2;
            CapMask(firstPos(1):stopPos(1),firstPos(2):stopPos(2),firstPos(3):stopPos(3))...
                =capillaryMask(internalPos1(1):internalPos2(1),internalPos1(2):internalPos2(2),internalPos1(3):internalPos2(3));
            pause(.1)

            
        end% for jBLK=1:numSubBlocks,
    end%if and(fileExistsFlag==1,overwriteFlag==0),
    clear vascmaskcorrblock;
    clear D Q;
    clear RMB;
    clear capillaryMask
    saveSize = (getfield(whos('CapMask'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(microVesselMaskFileNameList{Ri},'CapMask',scs);
    microVesselSum(Ri) = sum(CapMask(:));
    miroVesselFraction(Ri) = sum(CapMask(:))./numel(CapMask(:));
%     figure(1);plot(radiusList(1:Ri),microVesselSum(1:Ri));
    figure(1);plot(radiusList(1:Ri),miroVesselFraction(1:Ri));
    pause(0.1);
    clear CapMask
end%for Ri=1:length(radiusList)

outputParameters = inputParameters;
outputParameters.microVesselMaskFileNameList = microVesselMaskFileNameList;
outputParameters.microVesselSum = microVesselSum;


return

%Notes:
%R = 1:  everything taken out
%R = 2:  diameters 3 and above taken out; 3 and below survive;
%R = 3:  diameters 5 and above taken out; 4 and below survive;
%R = 4:  diameters 7 and above taken out; 6 and below survive;
%R = 5;  diameters 9 and abobe taken out; 8 and below survive;


