function outExp = VidaVascRadii(inExp)
%GENERAL REQUIRED PARAMETERS 
%-----
fileRootName = inExp.globalParameters.fileRootName;
processedDataFolder = inExp.globalParameters.processedDataFolder;
intermediateResultsFolder = inExp.globalParameters.intermediateResultsFolder;
lastRerunStartDateNum = inExp.globalParameters.lastRerunStartDateNum;
vascBlockFileNameList = inExp.globalParameters.vascBlockFileNameList;
vascBlockRawMaskFileNameList = inExp.globalParameters.vascBlockRawMaskFileNameList;
fullStackSize = inExp.globalParameters.fullStackSize;
blockGrid = inExp.globalParameters.blockGrid;
subVolumeOverlapSize = inExp.globalParameters.subVolumeOverlapSize;
vascBlockCenterlineMaskFileNameList = inExp.globalParameters.vascBlockCenterlineMaskFileNameList;
channels = inExp.globalParameters.channels;
voxelSizeInMicrons = inExp.globalParameters.voxelSizeInMicrons;
%-----
radiusListStartInMicrons = inExp.vascRadiiParameters.microVesselIsolationParameters.radiusListStartInMicrons;
radiusListStopInMicrons = inExp.vascRadiiParameters.microVesselIsolationParameters.radiusListStopInMicrons;
radiusListStepInMicrons = inExp.vascRadiiParameters.microVesselIsolationParameters.radiusListStepInMicrons;
%-----
radiusListInMicrons = [radiusListStartInMicrons:radiusListStepInMicrons,radiusListStopInMicrons];

%--------------------------------------------------
%REQUIRED PARAMETERS FOR RAW VARIABLE PERCENT RECONSTRUCTION METHOD
%-----
fractionalIntensityReconstructionParameters = inExp.vascRadiiParameters.fractionalIntensityReconstructionParameters;
rawVarPercentReconstructParameters.vascBlockRawMaskFileNameList = inExp.globalParameters.vascBlockRawMaskFileNameList;
%--------------------------------------------------
%REQUIRED PARAMETERS FOR MICROVESSEL ISOLATION
microVesselIsolationParameters = inExp.vascRadiiParameters.microVesselIsolationParameters;
%--------------------------------------------------
%--------------------------------------------------
displayFlag = 0;
%-----


vascRadiusFilesFolder = fullfile(processedDataFolder,'VascRadiusFiles');
[mkdirSuccess,mkdirMessage,mkdirId] = mkdir(vascRadiusFilesFolder);
%-----
disp('Running Fractional Local Intensity Reconstruction of Vasc Mask');
fractionalIntensityReconstructionParameters.fileRootName = fileRootName;
fractionalIntensityReconstructionParameters.processedDataFolder = processedDataFolder;
fractionalIntensityReconstructionParameters.intermediateResultsFolder = intermediateResultsFolder;
fractionalIntensityReconstructionParameters.lastRerunStartDateNum = lastRerunStartDateNum;
fractionalIntensityReconstructionParameters.vascBlockRawMaskFileNameList = vascBlockRawMaskFileNameList;
fractionalIntensityReconstructionParameters.vascBlockFileNameList = vascBlockFileNameList;
fractionalIntensityReconstructionParameters.vascBlockCenterlineMaskFileNameList = vascBlockCenterlineMaskFileNameList;
fractionalIntensityReconstructionParameters.lastRerunStartDateNum = lastRerunStartDateNum;
fractionalIntensityReconstructionParameters.vascRadiusFilesFolder = vascRadiusFilesFolder;
fractionalIntensityReconstructionParameters.voxelSizeInMicrons = voxelSizeInMicrons;

    fractionalIntensityReconstructionParameters = VidaFractionalIntensityReconstruction(fractionalIntensityReconstructionParameters);
%-----
fractionalIntensityReconstructionParameters.fullStackSize = fullStackSize;
fractionalIntensityReconstructionParameters.blockGrid = blockGrid;
fractionalIntensityReconstructionParameters.subVolumeOverlapSize = subVolumeOverlapSize;
    fractionalIntensityReconstructionParameters = VidaRadiusMapRestitch(fractionalIntensityReconstructionParameters);
%-----
vascReconstructedMaskFileName = fractionalIntensityReconstructionParameters.vascReconstructedMaskFileName;
vascRadiusMapFileName = fractionalIntensityReconstructionParameters.vascRadiusMapFileName;
%-----          


processMicroVesselFlag = 0;%noteToSelf : skip

if processMicroVesselFlag == 1,
    %ISOLATE MICROVESSELS (CAPILLARIES) BY SIZE;
    disp('Isolating Microvessels by Size');
    %---
    microVesselIsolationParameters.fileRootName = fileRootName;
    microVesselIsolationParameters.vascRadiusFilesFolder = vascRadiusFilesFolder;
    microVesselIsolationParameters.intermediateResultsFolder = intermediateResultsFolder;
    microVesselIsolationParameters.lastRerunStartDateNum = lastRerunStartDateNum;
    microVesselIsolationParameters.vascReconstructedMaskFileName = vascReconstructedMaskFileName;
    microVesselIsolationParameters.displayMicroVesselsFlag = displayFlag;
    microVesselIsolationParameters.voxelSizeInMicrons = voxelSizeInMicrons;
    %---
        microVesselIsolationParameters = VidaIsolateMicroVessels(microVesselIsolationParameters);
    microVesselMaskFileNameList = microVesselIsolationParameters.microVesselMaskFileNameList;
    microVesselSum = microVesselIsolationParameters.microVesselSum;
    radiusListInMicrons = microVesselIsolationParameters.radiusListInMicrons;
    %---
else%if processMicroVesselFlag = 1,
    microVesselMaskFileNameList = [];
    microVesselSum = [];  
end%if processMicroVesselFlag = 1,

    
outExp = inExp;
outExp.vascRadiiParameters.fractionalIntensityReconstructionParameters = fractionalIntensityReconstructionParameters;
outExp.vascRadiiParameters.microVesselIsolationParameters = microVesselIsolationParameters;
%-----
outExp.globalParameters.vascReconstructedMaskFileName = vascReconstructedMaskFileName;
outExp.globalParameters.vascRadiusMapFileName = vascRadiusMapFileName;
%-----
outExp.globalParameters.radiusList = radiusListInMicrons;
outExp.globalParameters.microVesselMaskFileNameList = microVesselMaskFileNameList;
outExp.globalParameters.microVesselSum = microVesselSum;
outExp.completionDates.vascRadiiCompletionDate = datestr(now);


