function outputParameters = VidaProcessImageStack(inputParameters)
vidaProject = inputParameters;

setupParameters = vidaProject.setupParameters;
currentImageStackFolderName = setupParameters.currentImageStackFolderName;
debugModeAvoidOverwriteFlag = setupParameters.debugModeAvoidOverwriteFlag;
%-----
experiment = vidaProject.experimentTemplate;
vidaProjectRootFolder = vidaProject.vidaProjectRootFolder;
vidaProjectRootName = vidaProject.vidaProjectRootName;
rootCodeFolder = experiment.codeFolders.rootCodeFolder;
uniformityFileNameList = vidaProject.globalParameters.uniformityFileNameList;
fluoresceinFileNameList = vidaProject.globalParameters.fluoresceinFileNameList;
darkFileNameList = vidaProject.globalParameters.darkFileNameList;
darkIntensityMode = vidaProject.globalParameters.darkIntensityMode;
%-----
%Set up internal folder and file names
rootDataFolder = currentImageStackFolderName;
fileRootName = rootDataFolder(findstr(rootDataFolder,'ImageStack-')+11:end);
originalDataFolder = fullfile(rootDataFolder,'OriginalData');
processedDataFolder = fullfile(rootDataFolder,'ProcessedData');
intermediateResultsFolder = fullfile(rootDataFolder,'IntermediateResults');
% experimentFileName = fullfile(rootDataFolder,[fileRootName,'-Experiment.mat']);
%-----
experiment.globalParameters.vidaProjectRootFolder = vidaProjectRootFolder;
experiment.globalParameters.vidaProjectRootName = vidaProjectRootName;
experiment.globalParameters.fileRootName = fileRootName;
experiment.globalParameters.rootDataFolder = rootDataFolder;
experiment.globalParameters.originalDataFolder = originalDataFolder;
experiment.globalParameters.processedDataFolder = processedDataFolder;
experiment.globalParameters.intermediateResultsFolder = intermediateResultsFolder;
experiment.globalParameters.uniformityFileNameList = uniformityFileNameList;
experiment.globalParameters.fluoresceinFileNameList = fluoresceinFileNameList;
experiment.globalParameters.darkFileNameList = darkFileNameList;
experiment.globalParameters.darkIntensityMode = darkIntensityMode;
%----------
if ~exist(experiment.globalParameters.originalDataFolder,'file'),
    [mds,mdm,mdmid] = mkdir(rootDataFolder,'OriginalData');
end%if ~exist(experiment.globalParameters.processedDataFolder,'file'),
%-----
if ~exist(experiment.globalParameters.processedDataFolder,'file'),
    [mds,mdm,mdmid] = mkdir(rootDataFolder,'ProcessedData');
end%if ~exist(experiment.globalParameters.processedDataFolder,'file'),
%-----
if ~exist(experiment.globalParameters.intermediateResultsFolder,'file'),
    [mds,mdm,mdmid] = mkdir(rootDataFolder,'IntermediateResults');
end%if ~exist(experiment.globalParameters.processedDataFolder,'file'),
%----------

experimentFileName = fullfile(experiment.globalParameters.processedDataFolder,[fileRootName,'-Experiment.mat']);


%Automatically adjust rootDataFolder in experiment file
if exist(experimentFileName,'file'),
    load(experimentFileName,'experiment');
    backupExpFileName = fullfile(processedDataFolder,[fileRootName,'-ExperimentBackup',datestr(now,'mmddyy_HHMMSS'),'.mat']);
    oldPath = experiment.globalParameters.rootDataFolder;
    oldCodeFolder = experiment.codeFolders.rootCodeFolder;
    if strcmpi(oldPath,rootDataFolder)~= 1,
        saveSize = (getfield(whos('experiment'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(backupExpFileName,'experiment',scs);
        %Update all the field names
        experiment = replacefields(experiment,oldPath,rootDataFolder,'no');
        experiment = replacefields(experiment,oldCodeFolder,rootCodeFolder,'no');
        saveSize = (getfield(whos('experiment'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(experimentFileName,'experiment',scs);
    end%if strcmpi(oldPath,rootDataFolder)~= 1,
end%if exist(experimentFileName,'file'),

%--------------------------------------------------------------------------

%DATE STAMP EXPERIMENT FILE OR LOAD EXISTING EXPERIMENT FILE
%This step overloads the iniatilzed experiment file if one exists on disk
vidaProject.setupParameters.currentExperiment = experiment;
vidaProject.setupParameters.currentExperimentFileName = experimentFileName;
    vidaProject = VidaLoadExperimentFile(vidaProject);
experiment = vidaProject.currentExperiment;

format compact;

% % % %Add all code Folder to Matlab path
% % % codeFolderList = fieldnames(experiment.codeFolders);
% % % numCodeFolders = size(codeFolderList,1);
% % % for codeFolderIter = 1:numCodeFolders,
% % %     currentField = codeFolderList{codeFolderIter};
% % %     currentCodePath = experiment.codeFolders.(currentField);
% % %     addpath(currentCodePath)
% % % end
% % %    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%FUNCTION CALLS
%-----

%Save Experiment file and a backup
saveSize = (getfield(whos('experiment'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save(experimentFileName,'experiment',scs);
preMatRoot = experimentFileName(1:findstr(experimentFileName,'.mat')-1);
saveSize = (getfield(whos('experiment'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save([preMatRoot,'Backup',datestr(now,'mmddyy-HHMM'),'.mat'],'experiment',scs);
%----------
%----------
stageParameters.preProcessing.currentStageString = 'PreProcessing';
stageParameters.preProcessing.dataStructureVariableName = 'experiment';
stageParameters.preProcessing.dataStructureFileName = experimentFileName;
stageParameters.preProcessing.mainFunctionCallString = 'VidaPreProcessing';
%----
stageParameters.subBlock.currentStageString = 'SubBlock';
stageParameters.subBlock.dataStructureVariableName = 'experiment';
stageParameters.subBlock.dataStructureFileName = experimentFileName;
stageParameters.subBlock.mainFunctionCallString = 'VidaSubBlock';
%----
stageParameters.cellSeg.currentStageString = 'CellSeg';
stageParameters.cellSeg.dataStructureVariableName = 'experiment';
stageParameters.cellSeg.dataStructureFileName = experimentFileName;
stageParameters.cellSeg.mainFunctionCallString = 'VidaCellSeg';
%----
stageParameters.vascBlockChannelSub.currentStageString = 'VascBlockChannelSub';
stageParameters.vascBlockChannelSub.dataStructureVariableName = 'experiment';
stageParameters.vascBlockChannelSub.dataStructureFileName = experimentFileName;
stageParameters.vascBlockChannelSub.mainFunctionCallString = 'VidaVascBlockChannelSub';
%----
stageParameters.vascSeg.currentStageString = 'VascSeg';
stageParameters.vascSeg.dataStructureVariableName = 'experiment';
stageParameters.vascSeg.dataStructureFileName = experimentFileName;
stageParameters.vascSeg.mainFunctionCallString = 'VidaVascSeg';
%----
stageParameters.centerline.currentStageString = 'Centerline';
stageParameters.centerline.dataStructureVariableName = 'experiment';
stageParameters.centerline.dataStructureFileName = experimentFileName;
stageParameters.centerline.mainFunctionCallString = 'VidaCenterline';
%----
stageParameters.cellCentroid.currentStageString = 'CellCentroid';
stageParameters.cellCentroid.dataStructureVariableName = 'experiment';
stageParameters.cellCentroid.dataStructureFileName = experimentFileName;
stageParameters.cellCentroid.mainFunctionCallString = 'VidaCellCentroid';
%----
stageParameters.neunBlockChannelSub.currentStageString = 'NeunBlockChannelSub';
stageParameters.neunBlockChannelSub.dataStructureVariableName = 'experiment';
stageParameters.neunBlockChannelSub.dataStructureFileName = experimentFileName;
stageParameters.neunBlockChannelSub.mainFunctionCallString = 'VidaNeunBlockChannelSub';
%----
stageParameters.neunQuantify.currentStageString = 'NeunQuantify';
stageParameters.neunQuantify.dataStructureVariableName = 'experiment';
stageParameters.neunQuantify.dataStructureFileName = experimentFileName;
stageParameters.neunQuantify.mainFunctionCallString = 'VidaNeunQuantify';
%----
stageParameters.neuronClassify.currentStageString = 'NeuronClassify';
stageParameters.neuronClassify.dataStructureVariableName = 'experiment';
stageParameters.neuronClassify.dataStructureFileName = experimentFileName;
stageParameters.neuronClassify.mainFunctionCallString = 'VidaNeuronClassify';
%----
stageParameters.vascRadii.currentStageString = 'VascRadii';
stageParameters.vascRadii.dataStructureVariableName = 'experiment';
stageParameters.vascRadii.dataStructureFileName = experimentFileName;
stageParameters.vascRadii.mainFunctionCallString = 'VidaVascRadii';
%----
stageParameters.cleanUpFiles.currentStageString = 'cleanUpFiles';
stageParameters.cleanUpFiles.dataStructureVariableName = 'experiment';
stageParameters.cleanUpFiles.dataStructureFileName = experimentFileName;
stageParameters.cleanUpFiles.mainFunctionCallString = 'VidaCleanUpFiles';
%----

%RUN UNCOMPLETED STAGES
disp(['RootDataFolder = ',rootDataFolder]);
disp(['FileRootName = ',fileRootName]);
[experiment] = VidaRunSaveOrSkip3(experiment,stageParameters.preProcessing);
[experiment] = VidaRunSaveOrSkip3(experiment,stageParameters.subBlock);
[experiment] = VidaRunSaveOrSkip3(experiment,stageParameters.cellSeg);
[experiment] = VidaRunSaveOrSkip3(experiment,stageParameters.vascBlockChannelSub);
[experiment] = VidaRunSaveOrSkip3(experiment,stageParameters.vascSeg);
% [experiment] = VidaRunSaveOrSkip3(experiment,stageParameters.centerline);
[experiment] = VidaRunSaveOrSkip3(experiment,stageParameters.cellCentroid);
[experiment] = VidaRunSaveOrSkip3(experiment,stageParameters.neunBlockChannelSub);
[experiment] = VidaRunSaveOrSkip3(experiment,stageParameters.neunQuantify);
[experiment] = VidaRunSaveOrSkip3(experiment,stageParameters.neuronClassify);
% [experiment] = VidaRunSaveOrSkip3(experiment,stageParameters.vascRadii);
% [experiment] = VidaRunSaveOrSkip3(experiment,stageParameters.cleanUpFiles);%noteToSelf

vidaProject.currentExperiment = experiment;
outputParameters = vidaProject;
end