function outProject = VidaRegisteredThresholdRelaxationParentSL(inProject)

vidaProject = inProject;
vidaProjectFileName = vidaProject.vidaProjectFileName;
vidaProjectRootName = vidaProject.vidaProjectRootName;
vidaProjectRootFolder = vidaProject.vidaProjectRootFolder;
globalParameters = vidaProject.globalParameters;
registrationParameters = vidaProject.registrationParameters;
strandifyParameters = vidaProject.strandifyParameters;
imageStackFolderNameList = vidaProject.imageStackFolderNameList;
registeredFilesFolder = globalParameters.registeredFilesFolder;
voxelSizeInMicrons = globalParameters.voxelSizeInMicrons;
%-----

%-----
%Create a subFolder for stitched volumes
stitchedFilesFolder = fullfile(registeredFilesFolder,'StitchedFilesFolder');
globalParameters.stitchedFilesFolder = stitchedFilesFolder;
vidaProject.globalParameters = globalParameters;
if ~exist(stitchedFilesFolder,'dir'),
    mkdir(stitchedFilesFolder);
end
    



if vidaProject.multiThreadProgress.thresholdRelaxingFlag == 2,
    fprintf('\nThreshold Relaxation previously completed\n\n');
else%if vidaProject.multiThreadProgress.thresholdRelaxingFlag == 2,
    fprintf('\nBegin Threshold Relaxation\n\n');
    disp(['Running Threshold Relaxation: ',datestr(now)]);
    vidaProject = RunTRPrime(vidaProject);
    vidaProject.multiThreadProgress.thresholdRelaxingFlag = 2;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
end%if vidaProject.multiThreadProgress.thresholdRelaxingFlag == 2,





%STAGE 9 : Restrandify vascular data
if vidaProject.multiThreadProgress.restrandifyingFlag == 2,
    fprintf('\nSecond round strandifying previously completed\n\n');
else%if vidaProject.multiThreadProgress.restrandifyingFlag == 2,
    fprintf('\nBegin secound round strandifying of vascular data\n\n');
    %Generate new centerlineRadiusStructure from the vectorizedStructure and the new strands from threshold relaxation.
    load(registeredExperimentFileName,'registeredExperiment');
    centerlineRadiusStructure = [];
    vectorizedStructure = vidaProject.globalParameters.vectorizedStructure;
    maskSize = vectorizedStructure.maskSize;
    av = vectorizedStructure.Vertices.AllVerts;
    origPixelList = sub2ind(maskSize,av(:,1),av(:,2),av(:,3));
    origRadiusList = vectorizedStructure.Vertices.AllRadii;
    BridgingStrands = vidaProject.globalParameters.GapConnectionStruct.ThreshRelax.BridgingStrands;
    BridgingConfidences = vidaProject.globalParameters.GapConnectionStruct.ThreshRelax.BridgingConfidences;
    numBridgingStrands = size(BridgingStrands,2);
    newPixelList = [];
    %-----
    minConfidence = 0.66
    disp('_____________Hardcode_VidaRegisteredProcessStitchCenterlineStrandifyParentSL.m - line 385');
    disp(['_____________Setting min threshold confidence to',num2str(minConfidence)]);
    disp(['_____________Setting min threshold confidence to',num2str(minConfidence)]);
    disp(['_____________Setting min threshold confidence to',num2str(minConfidence)]);
    disp(['_____________Setting min threshold confidence to',num2str(minConfidence)]);
    %-----
    for strandIter = 1:numBridgingStrands,
        cs = BridgingStrands{strandIter};
        if ~isempty(cs),
            if BridgingConfidences(strandIter) >= minConfidence,
                currentPixelList = sub2ind(maskSize,cs(:,1),cs(:,2),cs(:,3));
                newPixelList = cat(1,newPixelList,currentPixelList);
            end%if BridgingConfidences(strandIter) >= minConfidence,
        end%if ~isempty(cs),
    end
    newRadiusList = zeros(size(newPixelList));
    pixelList = cat(1,origPixelList,newPixelList);
    radiusList = cat(1,origRadiusList,newRadiusList);
    centerlineRadiusStructure.pixelList = pixelList;
    centerlineRadiusStructure.maskSize = maskSize;
    centerlineRadiusStructure.radiusList = radiusList;
    %-----
    stageParameters.regRestrandify.currentStageString = 'RegisteredVascReStrandify';
    stageParameters.regRestrandify.dataStructureVariableName = 'registeredExperiment';
    stageParameters.regRestrandify.dataStructureFileName = registeredExperimentFileName;
    stageParameters.regRestrandify.mainFunctionCallString = 'VidaStrandify';
    %----
    registeredExperiment.strandifyParameters = strandifyParameters;
    registeredExperiment.globalParameters.centerlineRadiusStructure = centerlineRadiusStructure;
        [registeredExperiment] = VidaRunSaveOrSkip3(registeredExperiment,stageParameters.regRestrandify);
        load(stageParameters.regRestrandify.dataStructureFileName,stageParameters.regRestrandify.dataStructureVariableName);
    centerlineRadiusStructure = registeredExperiment.globalParameters.centerlineRadiusStructure;
    vectorizedStructure = registeredExperiment.strandifyParameters.vectorizedStructure;
    %-----
%     vidaProject.registeredExperiment = registeredExperiment;
    vidaProject.globalParameters.centerlineRadiusStructure = centerlineRadiusStructure;
    vidaProject.globalParameters.vectorizedStructure = vectorizedStructure;
    vidaProject.globalParameters.registeredCellCentroidAndLabelList = registeredCellCentroidAndLabelList;
    vidaProject.globalParameters.minConfidence = minConfidence;
    %---
    saveSize = (getfield(whos('registeredExperiment'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(registeredExperimentFileName,'registeredExperiment',scs);
    %---
    vectorizedStructureOutputFileName = fullfile(vidaProjectRootFolder,[vidaProjectRootName,'-PostThresholdVectorizedStructure.mat']);
    saveSize = (getfield(whos('vectorizedStructure'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vectorizedStructureOutputFileName,'vectorizedStructure',scs);
    %---
    vidaProject.multiThreadProgress.restrandifyingFlag = 2;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
end%if-else vidaProject.multiThreadProgress.restrandifyingFlag == 2,


%-----
disp([' ';' ';' ']);
disp(['Completed Processing of Registered Mask : ',datestr(now)]);
disp([' ';' ';' ']);
%----


outProject = vidaProject;
