%Calculate Cell Density against barrel locations
%Preload vidaProject and barrel hand-trace file
%Assumes Z-axis is the depth into brain.  If data is taken at a different
%orintation, then the data must be permuted so that [axis1, axis2, axis3]
%represents [horizontalDim1, horizontalDim2, depthIntoBrain]



registeredCellCentroidAndLabelList = vidaProject.globalParameters.registeredCellCentroidAndLabelList;
maskSize = vidaProject.globalParameters.registeredStackSize;
allVerts = vidaProject.globalParameters.vectorizedStructure.Vertices.AllVerts;
allRadii = vidaProject.globalParameters.vectorizedStructure.Vertices.AllRadii;

clear vidaProject;

%%
%Generate a 2D barrel map from the hand traces
padZ1 = 100;
padZ2 = 100;
cropStart1 =60;
cropStop1 = 1100;
cropStart2 = 20;
cropStop2 = 1050;
maxZCoord = 0;
minZCoord = inf;
mapSize1 = maskSize(1);
mapSize2 = maskSize(2);
numBarrels = size(barrels,2);
allBarrelsMap = zeros(mapSize1,mapSize2);
for barrelIter = 1:numBarrels;
    currentBarrelOutlinePoints = unique(barrels(barrelIter).userTraced.top,'rows');
    tempCoordList = currentBarrelOutlinePoints(:,1:3);
    firstCoord = tempCoordList(1,:);
    tempCoordList = tempCoordList(2:end,:);
    numCoords = size(tempCoordList,1);
    previousCoord = firstCoord;
    myImage = zeros(mapSize1,mapSize2);
    for coordIter = 1:numCoords+1
        tempSize = size(tempCoordList,1);
        sepList = tempCoordList - repmat(previousCoord,[tempSize,1]);
        distList = sqrt(sum(sepList.^2,2));
        [minVal,minPos] = min(distList);
        nextCoord = tempCoordList(minPos,:);
        d1 = sepList(minPos,1);
        d2 = sepList(minPos,2);
        numSteps = round(minVal*2);
        currentCoord = previousCoord;
        minZCoord = min(minZCoord,currentCoord(3));
        maxZCoord = max(maxZCoord,currentCoord(3));
        for stepIter = 1:numSteps,
            delta1 = d1/numSteps;
            delta2 = d2/numSteps;
            currentCoord = currentCoord + [delta1,delta2,0];
            myImage(round(currentCoord(1)),round(currentCoord(2))) = 1;
        end
        tempCoordList = tempCoordList([1:minPos-1,minPos+1:tempSize],:);
        previousCoord = nextCoord;
        if coordIter == 30,
            %after a few rounds, add the starting point back to the list
            tempCoordList = [tempCoordList;firstCoord];
        end
    end%for coordIter = 1:numCoords
    filledBarrel = imfill(myImage,'holes');
    figure(1);imagesc(filledBarrel);
    allBarrelsMap = or(allBarrelsMap,filledBarrel);
    figure(2);imagesc(allBarrelsMap);
    pause(0.5);
end%for barrelIter = 1:numBarrels
labeledBarrels = bwlabeln(allBarrelsMap);
croppedLabeledBarrels = labeledBarrels(cropStart1:cropStop1,cropStart2:cropStop2);
figure(2);imagesc(croppedLabeledBarrels);
save barrelsTempStage01a
        
   
%%
%Compute the projected 2D vessel length density across chosen z-layers
% load barrelsTempStage01a


z1 = floor(minZCoord) - padZ1;
z2 = ceil(maxZCoord) + padZ2;

z1 = 400;%400
z2 = 575;%500


allVerts(:,4) = allRadii;
subFlag1 = allVerts(:,3)>=z1;
subFlag2 = allVerts(:,3)<=z2;
subZFlag = and(subFlag1,subFlag2);
subVerts = allVerts(subZFlag,:);
subVerts(:,3) = subVerts(:,3)-z1+1;
subMask = zeros(maskSize(1),maskSize(2));
subRadii = zeros(maskSize(1),maskSize(2));
numSubVerts = size(subVerts,1);
for vertIter = 1 : numSubVerts,
    subMask(subVerts(vertIter,1),subVerts(vertIter,2)) = subMask(subVerts(vertIter,1),subVerts(vertIter,2)) + 1;
    subRadii(subVerts(vertIter,1),subVerts(vertIter,2)) = subRadii(subVerts(vertIter,1),subVerts(vertIter,2)) + subVerts(vertIter,4);
end

subVascMap = sum(subMask,3);
subRadiiMap = sum(subRadii,3);
clear subMask subRadii
croppedSubVascMap = subVascMap(cropStart1:cropStop1,cropStart2:cropStop2);
croppedSubRadiiMap = subRadiiMap(cropStart1:cropStop1,cropStart2:cropStop2);
% figure(3);imagesc(log(croppedSubVascMap+1));
figure(3);imagesc(imfilter(croppedSubVascMap,double(circle2d(7))));
% figure(4);imagesc(log(croppedSubVascMap+1));
figure(4);imagesc(imfilter(croppedSubRadiiMap,double(circle2d(7))));

%-----

z1 = 400
z2 = 575

subFlag1 = registeredCellCentroidAndLabelList(:,3)>=z1;
subFlag2 = registeredCellCentroidAndLabelList(:,3)<=z2;
subZFlag = and(subFlag1,subFlag2);
subCells = registeredCellCentroidAndLabelList(subZFlag,:);
subCells(:,3) = subCells(:,3)-z1+1;
subMask = zeros(maskSize(1),maskSize(2));
numSubCells = size(subCells,1);
for cellIter = 1:numSubCells,
    subMask(subCells(cellIter,1),subCells(cellIter,2)) = subMask(subCells(cellIter,1),subCells(cellIter,2)) +  1;
end
subCellMap = sum(subMask,3);
croppedSubCellMap = subCellMap(cropStart1:cropStop1,cropStart2:cropStop2);
figure(5);imagesc(log(croppedSubCellMap+1));
figure(5);imagesc(imfilter(croppedSubCellMap,double(circle2d(21))));




%%


allInsideMap = croppedLabeledBarrels>0;
allOutsideMap = croppedLabeledBarrels == 0;
allInsideVesselMap = croppedSubVascMap(allInsideMap);
allOutsideVesselMap = croppedSubVascMap(allOutsideMap);
allInsideMeanVesselLength = sum(allInsideVesselMap(:))./numel(allInsideVesselMap);
allOutsideMeanVesselLength = sum(allOutsideVesselMap(:))./numel(allOutsideVesselMap);



% useMap = croppedSubVascMap;
useMap = croppedSubCellMap;

dilateSize = 5;
erodeSize = 30;

insideMeanVesselLength = [];
surroundMeanVesselLenth = [];
allInsideMask = false(size(croppedLabeledBarrels));
allSurroundMask = false(size(croppedLabeledBarrels));
for barrelIter = 1:numBarrels,
    currentMask = (croppedLabeledBarrels == barrelIter);
    currentMask(1,:) = 0; currentMask(:,1) = 0;
    currentMask(end,:) = 0 ; currentMask(:,end) = 0;
    currentInside = currentMask;
    for erodeIter = 1:erodeSize,
        currentInside = imerode(currentInside,ones(3,3));
    end
    allInsideMask = or(allInsideMask,currentInside);
    %---
    currentDilate = currentMask;
    for dilateIter = 1:dilateSize;
    currentDilate = imdilate(currentDilate,ones(3,3));
    end
    currentSurround = (currentDilate - currentMask)>0;
    allSurroundMask = or(allSurroundMask,currentSurround);
    %---
    insideVesselMap = useMap(currentInside);
    surroundVesselMap = useMap(currentSurround);
    insideMeanVesselLength(barrelIter) = sum(insideVesselMap(:))./numel(insideVesselMap);
    surroundMeanVesselLenth(barrelIter) = sum(surroundVesselMap(:))./numel(surroundVesselMap);
end%for barrelIter = 1:numBarrels,

figure(4);plot(insideMeanVesselLength,'r.');hold on
figure(4);plot(surroundMeanVesselLenth,'k.');hold off
figure(5);imagesc(allInsideMask);
figure(6);imagesc(allSurroundMask);

%%


% useMap = croppedSubVascMap;
% useMap = croppedSubCellMap;
filterSize = 11;
useMap = imfilter(croppedSubCellMap,double(circle2d(filterSize)));
exclusionFilterSize = 11;
allCellRingSize = [];
allCellMeanLength = [];
for barrelIter = 1:numBarrels;
    maxErode = 60;
    maxDilate = 60;
    currentMask = (croppedLabeledBarrels == barrelIter);
    currentMask(1,:) = 0; currentMask(:,1) = 0;
    currentMask(end,:) = 0 ; currentMask(:,end) = 0;
    previousMask = currentMask;
    meanLength = [];
    ringSize = [];
    counter = 0;
    for erodeIter = 1:maxErode,
        counter = counter + 1;
        tempMask = imerode(previousMask,ones(3,3)>0);
        currentRingMask = (previousMask-tempMask)>0;
        ringValues = useMap(currentRingMask);
        previousMask = tempMask;
        ringSize(counter) = -1 * erodeIter;
        meanLength(counter) = sum(ringValues(:))./numel(ringValues);
    end
    %---
    exclusionMask = ((croppedLabeledBarrels>0) - currentMask)>0;
    exclusionMask = imdilate(exclusionMask,circle2d(exclusionFilterSize));
    previousMask = currentMask;
    for dilateIter = 1:maxDilate,
        counter = counter + 1;
        tempMask = imdilate(previousMask,ones(3,3)>0);
        currentRingMask = (tempMask - previousMask)>0;
        exclusiveRingMask = and(currentRingMask,~exclusionMask);
        ringValues = useMap(exclusiveRingMask);
        previousMask = tempMask;
        ringSize(counter) = dilateIter;
        meanLength(counter) = sum(ringValues(:))./numel(ringValues);
    end


    figure(7);plot(ringSize,meanLength,'r.');
    figure(8);plot(ringSize,meanLength,'r.');hold on
    pause(0.5);
    allCellRingSize = ringSize;
    allCellMeanLength(barrelIter,:) = meanLength;
end%for barrelIter = 1:numBarrels;
hold off


%%
%EVALUATE ONE BARREL AT A TIME
% useMap = croppedSubVascMap;
% useMap = croppedSubCellMap;
filterSize = 11;
useMap = imfilter(croppedSubVascMap,double(circle2d(filterSize)));
exclusionFilterSize = 11;
allVascRingSize = [];
allVascMeanLength = [];
for barrelIter = 1:numBarrels;
    maxErode = 60;
    maxDilate = 60;
    currentMask = (croppedLabeledBarrels == barrelIter);
    currentMask(1,:) = 0; currentMask(:,1) = 0;
    currentMask(end,:) = 0 ; currentMask(:,end) = 0;
    previousMask = currentMask;
    meanLength = [];
    ringSize = [];
    counter = 0;
    for erodeIter = 1:maxErode,
        counter = counter + 1;
        tempMask = imerode(previousMask,ones(3,3)>0);
        currentRingMask = (previousMask-tempMask)>0;
        ringValues = useMap(currentRingMask);
        previousMask = tempMask;
        ringSize(counter) = -1 * erodeIter;
        meanLength(counter) = sum(ringValues(:))./numel(ringValues);
    end
    %---
    exclusionMask = ((croppedLabeledBarrels>0) - currentMask)>0;
    exclusionMask = imdilate(exclusionMask,circle2d(exclusionFilterSize));
    previousMask = currentMask;
    for dilateIter = 1:maxDilate,
        counter = counter + 1;
        tempMask = imdilate(previousMask,ones(3,3)>0);
        currentRingMask = (tempMask - previousMask)>0;
        exclusiveRingMask = and(currentRingMask,~exclusionMask);
        ringValues = useMap(exclusiveRingMask);
        previousMask = tempMask;
        ringSize(counter) = dilateIter;
        meanLength(counter) = sum(ringValues(:))./numel(ringValues);
        
    end


    figure(17);plot(ringSize,meanLength,'r.');
    figure(18);plot(ringSize,meanLength,'r.');hold on
    allVascRingSize = ringSize;
    allVascMeanLength(barrelIter,:) = meanLength;
    pause(0.5);
end%for barrelIter = 1:numBarrels;
hold off


save barrelsTempStage01b

%%
% % % %%
% % % %Get centroid of each barrel
% % % filtVesselImage = imfilter(croppedSubVascMap,double(circle2d(25)));
% % % for barrelIter = 1:numBarrels;
% % %     currentBarrelBW = bwdist(~(croppedLabeledBarrels==barrelIter));
% % %     maxDist = max(currentBarrelBW(:));
% % %     currentBarrelBW = currentBarrelBW./maxDist;
% % %     r1 = rand(size(currentBarrelBW))*0.01;
% % %     r2 = rand(size(currentBarrelBW));
% % %     figure(1);p1 = plot(currentBarrelBW(:)+r1(:),filtVesselImage(:)+r2(:),'g.');
% % %     set(p1,'MarkerSize',1)
% % %     pause
% % % end%for barrelIter = 1:numBarrels;
% % % 

%%

%%
%Visualize Barrels
for zIter = 300:25:1000;
    disp(zIter);
    myImage = zeros(size(allBarrelsMap));
    sub = registeredCellCentroidAndLabelList(find(registeredCellCentroidAndLabelList(:,3) > zIter),:);
    sub = sub(find(sub(:,3)<(zIter +100)),:);
    for cellIter = 1:size(sub,1);
        myImage(sub(cellIter,1),sub(cellIter,2)) =  myImage(sub(cellIter,1),sub(cellIter,2)) + 1;
    end
    filtImage = imfilter(myImage,double(circle2d(15)));
    figure(77);imagesc(filtImage);
    filtImage = filtImage + allBarrelsMap * 5;
    figure(78);imagesc(filtImage);
    pause
end


%%

stripY1 = 500;
stripY2 = 600;

for stripY1 = 301:50:600;
    stripY2 = stripY1+100;

% useMap = croppedSubVascMap;
% useMap = croppedSubCellMap;



filtVascMap = imfilter(croppedSubVascMap,double(circle2d(63)));
filtCellMap = imfilter(croppedSubCellMap,double(circle2d(63)));

rotVascMap = imrotate(filtVascMap,36,'crop');
rotCellMap = imrotate(filtCellMap,36,'crop');

figure(101);imagesc(rotVascMap);
figure(102);imagesc(rotCellMap);


vascLine = sum(rotVascMap(stripY1:stripY2,:),1);
cellLine = sum(rotCellMap(stripY1:stripY2,:),1);
figure(11);plot(vascLine./max(vascLine(:)),'r-');hold off
figure(12);plot(cellLine./max(cellLine(:)),'k-');hold off

myXCorr = zeros(size(vascLine));
tempVascLine= vascLine;
for shiftIter = 1:size(filtVascMap,2);
    myCorrCoef = corrcoef(tempVascLine,cellLine);
    myXCorr(shiftIter) = myCorrCoef(1,2);
    tempVascLine = circshift(tempVascLine,[0,1]);
end
figure(13);plot(myXCorr,'g-');
pause
end%for stripY1 =...


%%
%EVALULATE ALL BARRELS AT ONCE

barrelFrame = croppedLabeledBarrels>0;
vascFrame = croppedSubVascMap; 




    
% barrelFrame = barrelFrame(100:450,25:450);
% vascFrame = vascFrame(100:450,25:450);



    figure(3);imagesc(uint8(barrelFrame*128) + 128*uint8(vascFrame>0));
    pause(0.5);
    
    %serially erode the barrelMask until no pixels are left
    keepGoing = 1;
    distanceMap = zeros(size(barrelFrame));
    tempMask = barrelFrame;
    erosionCounter = 0;
    while keepGoing,
        erosionCounter=erosionCounter+1;
        if sum(tempMask(:)) == 0,
            keepGoing = 0;
        else%if sum(tempMask(:)) == 0,
            erodedMask = imerode(tempMask,ones(3,3));
            shellMask = (tempMask-erodedMask)>0;
            distanceMap(shellMask) = -1 * erosionCounter;
            tempMask = erodedMask;
        end%if-else sum(tempMask(:)) == 0,
    end%while keepGoing,
    minDist = min(distanceMap(:));
    
% % %     %normalize the erosion distance
% % %     L = bwlabeln(barrelFrame>0);
% % %     for kk = 1:max(L(:))
% % %         subDist = distanceMap(L==kk);
% % %         normDist = (subDist./min(subDist(:)) ) * minDist;
% % %         distanceMap(L==kk) = normDist;
% % %     end
% % %         
% % %     
% % %     
    
    %now serially dilate the barrelMask until no pixels are left
    keepGoing = 1;
    tempMask = barrelFrame;
    dilationCounter = 0;
    while keepGoing,
        if sum(~tempMask(:)) == 0,
            keepGoing = 0;
        else%if sum(~tempMask(:)) == 0,
            dilatedMask = imdilate(tempMask,ones(3,3));
            shellMask = (dilatedMask-tempMask)>0;
            distanceMap(shellMask) = dilationCounter;
            tempMask = dilatedMask;
            dilationCounter=dilationCounter+1;
        end%if sum(~tempMask(:)) == 0,
    end%while keepGoing,

    %Calculate the distribution of all points in the distance map
    maxDist = max(distanceMap(:));
    distRange = [minDist:maxDist];
distRange = [-50:10:100];   
    spaceHist = hist(distanceMap(:),distRange);
    figure(1);bar(distRange,spaceHist);

    %Calculate the distribution of vasc vessel points in the distance map
    vascDistMap = distanceMap(vascFrame>0);
    vascRepMap = vascFrame(vascFrame>0);
    vascDist = zeros(1,numel(vascFrame));
    nonZeroPos = find(vascFrame>0);
    counter = 0;
    for kk = 1:length(nonZeroPos);
        myRep =  vascFrame(nonZeroPos(kk))]
        myVals = repmat(distanceMap(nonZeroPos(kk)),[1,myRep]);
        vascDist(counter+1:counter+myRep-1) = myVals;
        counter = counter+myRep;
    end
        
        
    end%kk
        
        
        
    vascHist = hist(vascDist,distRange);
    figure(2);bar(distRange,vascHist);
    
    allSpace(repCounter,:) = spaceHist;
    allVasc(repCounter,:) = vascHist;
    
    figure(11);bar(distRange,sum(allSpace,1));
    figure(12);bar(distRange,sum(allVasc,1));
    pause(1);
    
end%for frameNum = 180:10:280;






































