function outputParameters=VidaGetCentroidsFromMask(inputParameters)
%written by Phil to replace GetCentroidsFromMask
%09/21/07

%Note: this function PURPOSEFULLY, evaluates blocks that overlap by 1 pixel
%allowing multiple centroids per cell at the block junction. 
%this will then be cleaned up by merging close centroids
%this is done to avoid missing any cells at the block junction
%where the top block puts it 1 pixel too low, and the lower block puts it 1
%pixel too high.

cellCentroidFilesFolder = inputParameters.cellCentroidFilesFolder;
lastRerunStartDateNum = inputParameters.lastRerunStartDateNum;
cellMaskFileName = inputParameters.cellMaskFileName;
cellCentroidSubBlockSizeY = inputParameters.cellCentroidSubBlockSizeY;
cellCentroidSubBlockBufferSizeY = inputParameters.cellCentroidSubBlockBufferSizeY;
%-----
load(cellMaskFileName,'-mat','cellMask');
if isstruct(cellMask),
    pixelList = cellMask.pixelList;
    maskSize = cellMask.maskSize;
    cellMask = false(maskSize);
    cellMask(pixelList) = 1;
end%if isstruct(vascBlockCenterlineMask),
%-----
cellMaskYSize = size(cellMask,1);
blockStep = cellCentroidSubBlockSizeY - (2*cellCentroidSubBlockBufferSizeY);
numBlocks = ceil(cellMaskYSize/blockStep);

%-----
allCentroidList = [];



for blockIter = 1:numBlocks,
    currentCentroidBlockFileName = fullfile(cellCentroidFilesFolder,['CellCentroidsUpToBlock',num2str(blockIter,'%04d'),'.mat']);
    disp(['Cell centroids on block ',num2str(blockIter),' of ',num2str(numBlocks),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
    %-----
    overwriteFlag = 0;
    fileExistsFlag = 0;
    if exist(currentCentroidBlockFileName,'file'),
        fileExistsFlag = 1;
        fileProps = dir(currentCentroidBlockFileName);
        fileDateNum = datenum(fileProps.date);
        if fileDateNum < lastRerunStartDateNum,
            overwriteFlag = 1;
        end%if fileDateNum < lastRerunStartDateNum,
    end%if exist(currentCentroidBlockFileName,'file'),
    %---
    if and(fileExistsFlag==1,overwriteFlag==0),
        disp('...previously computed');
        load(currentCentroidBlockFileName,'allCentroidList');
    else%if and(fileExistsFlag==1,overwriteFlag==0),
        startY = max(((blockIter-1)*blockStep) + 1 , 1);
        stopY = min(cellMaskYSize,(startY + cellCentroidSubBlockSizeY-1));
        trueBlockSizeY = stopY-startY+1;
        cellBlock = cellMask(startY:stopY,:,:);
        BWUE = bwulterode(cellBlock,26);
        [L,estimateN] = bwlabeln(BWUE,26);
        clear BWUE estimateN
        cellProps = regionprops(L,'centroid');
        clear L
        centroidsTemp = cat(1,cellProps.Centroid);
        blockCentroids = [centroidsTemp(:,2),centroidsTemp(:,1),centroidsTemp(:,3)];
        switch blockIter
            case 1
                keepStart = 1;
                keepStop = trueBlockSizeY - cellCentroidSubBlockBufferSizeY;
            case numBlocks
                keepStart = cellCentroidSubBlockBufferSizeY;
                keepStop = trueBlockSizeY+1;
                previousStartY = max(((blockIter-2)*blockStep) + 1 , 1);
                previousStopY = min(cellMaskYSize,(previousStartY + cellCentroidSubBlockSizeY-1));
                previousTrueBlockSizeY = previousStopY-previousStartY+1;
                previousKeepStart = cellCentroidSubBlockBufferSizeY;
                previousKeepStop = previousTrueBlockSizeY - cellCentroidSubBlockBufferSizeY;
                previousLastY = previousStartY+previousKeepStop;
                if startY+keepStart > previousLastY,
                    diffY = startY+keepStart-previousLastY;
                    keepStart = keepStart-diffY+1;
                end
            otherwise
                keepStart = cellCentroidSubBlockBufferSizeY;
                keepStop = trueBlockSizeY - cellCentroidSubBlockBufferSizeY;
        end%switch blockIter
        keepCentroids = blockCentroids(find((blockCentroids(:,1)>=keepStart)&(blockCentroids(:,1)<keepStop)),:);
        trueCentroids = keepCentroids;
        trueCentroids(:,1) = keepCentroids(:,1) + startY -1;
        allCentroidList = cat(1,allCentroidList,trueCentroids);
        pause(0.1);
        saveSize = (getfield(whos('allCentroidList'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(currentCentroidBlockFileName,'allCentroidList',scs);
    end%if and(fileExistsFlag==1,overwriteFlag==0),
end%for blockIter = 1:numBlocks,
%-----
uniqueCentroids = unique(round(allCentroidList)-1,'rows');
%the minus one is to account of what is probably an asymmetric filtering
%that causes the mask to be shifted by one pixel compared to the raw data
outputParameters = inputParameters;
outputParameters.cellCentroidList=uniqueCentroids;
%-----
return






