function outExp = VidaMultiStackMergeCellCentroids(inExp)
%Stitch cell centroid list from single blocks
%Pablo - 18/Nov/2009
%Phil - 12/09/2009
%Pablo - 20/Dec/2010 - parametrized overlapping region marging 

overlapYXZmargin = 10; %% Note to self - hardcode this should move out


globalParameters = inExp.globalParameters;
registrationParameters = inExp.registrationParameters;
cellCentroidParameters = inExp.cellCentroidParameters;
rootFolder = globalParameters.vidaProjectRootFolder;
imageStackFolderNameList = globalParameters.imageStackFolderNameList;
adjacentBlockPairList = registrationParameters.adjacentBlockPairList;
blockStacking = registrationParameters.blockStacking;
zeroBasedOffsets = blockStacking.zerobasedoffsets;
cellCentroidOverlayCircleDiameterInMicrons = cellCentroidParameters.cellCentroidOverlayCircleDiameterInMicrons;
voxelSizeInMicrons = globalParameters.voxelSizeInMicrons;
mergedist = cellCentroidOverlayCircleDiameterInMicrons./voxelSizeInMicrons;

displayFlag = 0;

numBlocks = size(imageStackFolderNameList,2);



performance = zeros(size(adjacentBlockPairList,1),3);
mergedCellCentList=[];
ptr2pair = 0;
nchar = 0;
fprintf('\nMerging cells centroids\n');

for iBLK = 1: numBlocks;
    fprintf(repmat('\b',1,nchar));
    nchar = fprintf('Merging block %d out of %d, started @ %s\n',iBLK,numBlocks,datestr(now));
    %patch to get folder names - should be deleted when integrated
    blockName = imageStackFolderNameList{iBLK};
    delimPos =  findstr(blockName,'ImageStack-');
    blockName = blockName(delimPos+11:end);

    %load experiment file and extract labeled cell centroid list
    load(fullfile([rootFolder filesep 'ImageStack-' blockName filesep 'ProcessedData' ],[blockName '-Experiment.mat']))
    cclist=experiment.globalParameters.cellCentroidAndLabelList;
    fullStackSize = experiment.globalParameters.fullStackSize;

    %get block position in global coordinate system
    blockGlobalYXZ = zeroBasedOffsets(iBLK,:);
    mergedCellCentList = [mergedCellCentList ; cclist + repmat([blockGlobalYXZ 0],size(cclist,1),1)]; %#ok<AGROW>

    %now find already placed neighbour blocks (have lower block id)
    if ~isempty(adjacentBlockPairList)
        adjBlocks = [adjacentBlockPairList(adjacentBlockPairList(:,1) == iBLK,2) ;adjacentBlockPairList(adjacentBlockPairList(:,2) == iBLK,1) ];
        adjBlocks = adjBlocks(adjBlocks<iBLK);
        nAdjBlocks = numel(adjBlocks);
    else
        nAdjBlocks = [];
    end
    if ~isempty(nAdjBlocks)
        BiYXZStart = blockGlobalYXZ;
        BiYXZEnd = BiYXZStart + fullStackSize;

        %deal with overlapping regions, one at the time
        for iADJ = 1 : nAdjBlocks
            t0 = clock;
            mergedListSize = size(mergedCellCentList,1);

            BjYXZStart = zeroBasedOffsets(adjBlocks(iADJ),:);
            BjYXZEnd = BjYXZStart +  fullStackSize;
            overlapYXZStart = max([BiYXZStart;BjYXZStart]) - overlapYXZmargin;
            overlapYXZEnd = min([BiYXZEnd;BjYXZEnd]) + overlapYXZmargin;
            overlapYXZsize = overlapYXZEnd - overlapYXZStart;

            %find cells in current overlap
            idx = mergedCellCentList(:,1) >= overlapYXZStart(1) & ...
                mergedCellCentList(:,2) >= overlapYXZStart(2) & ...
                mergedCellCentList(:,3) >= overlapYXZStart(3) & ...
                mergedCellCentList(:,1)<= overlapYXZEnd(1) & ...
                mergedCellCentList(:,2)<= overlapYXZEnd(2) & ...
                mergedCellCentList(:,3)<= overlapYXZEnd(3);

            cclistOverlap = VidaMergeCellCentroidsCore(mergedCellCentList(idx,:),mergedist);
            overlapListSize = size(cclistOverlap,1);

            mergedCellCentList = mergedCellCentList(~idx,:);
            mergedCellCentList = [mergedCellCentList ; cclistOverlap]; %#ok<AGROW>

            ptr2pair = ptr2pair + 1;
            performance(ptr2pair,:) = [mergedListSize overlapListSize etime(clock,t0)];


            if displayFlag == 1,
                figure
                h2Bi = plot3Dcube(BiYXZStart,blockStacking);set(h2Bi,'FaceVertexCData',[.1 .1 .9]);
                h2Bj = plot3Dcube(BjYXZStart,blockStacking);set(h2Bj,'FaceVertexCData',[ .9 .1 .1]);
                h2Overlap = plot3Dcube(overlapYXZStart,overlapYXZsize);set(h2Overlap,'FaceColor','none','LineWidth',2,'edgeColor','k');
                view(3);set(gca,'ydir','reverse','zdir','reverse')
                axis equal
                hold on
                plot3(mergedCellCentList(:,2),mergedCellCentList(:,1),mergedCellCentList(:,3),'k.')
                plot3(mergedCellCentList(idx,2),mergedCellCentList(idx,1),mergedCellCentList(idx,3),'go')
            end%if displayFlag == 1,
            
        end %cyling adjacent blocks
    end %dealing with overlaps

end
outExp = inExp;
outExp.globalParameters.cellCentroidAndLabelList = mergedCellCentList;

% %
% % %%
% % plot3(mergedCellCentList(:,1),mergedCellCentList(:,2),mergedCellCentList(:,3),'r.')


