function outputParameters=VidaCellBinarize2(inputParameters)
%previously called OctoberCellBinarize2(EXPERIMENT)

fileRootName = inputParameters.fileRootName;
vidaProjectRootFolder = inputParameters.vidaProjectRootFolder;
lastRerunStartDateNum = inputParameters.lastRerunStartDateNum;
intermediateResultsFolder = inputParameters.intermediateResultsFolder;
cellSegFilesFolder = inputParameters.cellSegFilesFolder;
numSigma = inputParameters.sigZ;
backgroundSigma=inputParameters.bgSigma;
cellFilterSigma=inputParameters.cellMFModelSigma;
minVol=inputParameters.minVol;
maxVolPercent=inputParameters.maxVolPercent;
%---
blockGrid = inputParameters.blockGrid;
cellBlockFileNameList = inputParameters.cellBlockFileNameList;            
%-----
displayFlag = 0;


% %     blockSizes = blockGrid;
% %     for k = 1:length(blockSizes{1});
% %         yBlockSizes(k) = blockSizes{k}.x1max - blockSizes{k}.x1min + 1;
% %         xBlockSizes(k) = blockSizes{k}.x2max - blockSizes{k}.x2min + 1;
% %         zBlockSizes(k) = blockSizes{k}.x3max - blockSizes{k}.x3min + 1;
% %     end
% %     maxBlockSize = [max(yBlockSizes),max(xBlockSizes)];

    
%% Define filenames for enhancement and binarization
MFcode=['MFsz',num2str(cellFilterSigma)];
zcode=['z',num2str(numSigma)];
numFiles = length(cellBlockFileNameList);
cellBlockMaskFileNameList = cell(1,numFiles);
cellBlockMaskPixelListFileNameList = cell(1,numFiles);
cellBlockEnhancedGrayscaleFileNameList = cell(1,numFiles);
for blockIter=1:numFiles
    [ptmp,btmp,etmp]=fileparts(cellBlockFileNameList{blockIter});
    cellBlockMaskFileNameList{blockIter}=fullfile(cellSegFilesFolder,[btmp,'-mask-',MFcode,'-',zcode,etmp]);
    cellBlockMaskPixelListFileNameList{blockIter}=fullfile(cellSegFilesFolder,[btmp,'-EnhancedMaskPixelList.mat']);
    cellBlockEnhancedGrayscaleFileNameList{blockIter}=fullfile(cellSegFilesFolder,[btmp,'-enhanced-',MFcode,'-',zcode,etmp]);
end

%% Enhance, binarize, and log the sensitivity analysis per block


%Determine the maximum block size
numBlocks = length(blockGrid);
dy = zeros(1,numBlocks);
dx = zeros(1,numBlocks);
dz = zeros(1,numBlocks);
for k = 1:numBlocks,
    dy(k) = blockGrid{k}.x1max - blockGrid{k}.x1min + 1;
    dx(k) = blockGrid{k}.x2max - blockGrid{k}.x2min + 1;
    dz(k) = blockGrid{k}.x3max - blockGrid{k}.x3min + 1;
end
ySize = max(dy);
xSize = max(dx);
zSize = max(dz);
maxBlockSize = [ySize,xSize,zSize];
%-----
disp(['Generating Cell and Bkg Filters : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
if ~isempty(vidaProjectRootFolder),
    cellFilterFolder = fullfile(vidaProjectRootFolder,'CellFilterFiles');
    if ~exist(cellFilterFolder,'dir'),
        mkdir(cellFilterFolder);
    end%if ~exist(cellFilterFolder,'dir'),
    cellFilterFFTFileName = fullfile(cellFilterFolder,'CellFilterFFT.mat');
    backgroundFilterFFTFileName = fullfile(cellFilterFolder,'BackgroundFilterFFT.mat');
    %---
    if exist(cellFilterFFTFileName,'file'),
        load(cellFilterFFTFileName,'cellFilterFFT');
        disp('...Loaded Previous cell filter');
    else%if exist(cellFilterFFTFileName,'file'),
        cellFilterBlock = VidaGenerateCellFilter(maxBlockSize,cellFilterSigma);
        cellFilterFFT = fftn(single(cellFilterBlock));
        clear cellFilterBlock;
        saveSize = (getfield(whos('cellFilterFFT'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(cellFilterFFTFileName,'cellFilterFFT',scs);
    end%if-else exist(cellFilterFFTFileName,'file'),
    %---
    if exist(backgroundFilterFFTFileName,'file'),
        load(backgroundFilterFFTFileName,'backgroundFilterFFT');
        disp('...Loaded Previous background filter');
    else%if exist(backgroundFilterFFTFileName,'file'),
        backgroundFilterBlock = VidaGenerateCellFilter(maxBlockSize,backgroundSigma);
        backgroundFilterFFT = fftn(single(backgroundFilterBlock));
        clear backgroundFilterBlock;
        saveSize = (getfield(whos('backgroundFilterFFT'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(backgroundFilterFFTFileName,'backgroundFilterFFT',scs);
    end%if-else exist(backgroundFilterFFTFileName,'file'),
else%if ~isempty(vidaProjectRootFolder),
    cellFilterBlock = VidaGenerateCellFilter(maxBlockSize,cellFilterSigma);
    cellFilterFFT = fftn(single(cellFilterBlock));
    clear cellFilterBlock;
    %---
    backgroundFilterBlock = VidaGenerateCellFilter(maxBlockSize,backgroundSigma);
    backgroundFilterFFT = fftn(single(backgroundFilterBlock));
    clear backgroundFilterBlock;
end%if-else ~isempty(vidaProjectRootFolder)
%------
numFiles = length(cellBlockFileNameList);
cellHist = cell(1,numFiles);
for blockIter=1:numFiles;
    currentMaskPixelListFileName = cellBlockMaskPixelListFileNameList{blockIter};
    currentMaskFileName = cellBlockMaskFileNameList{blockIter};
    %-----
    overwriteFlag = 0;
    fileExistsFlag = (exist(currentMaskFileName,'file')==2);
    if fileExistsFlag,
        fileProps = dir(currentMaskFileName);
        fileDateNum = datenum(fileProps.date);
        if fileDateNum < lastRerunStartDateNum,
            overwriteFlag = 1;
        end%if fileDateNum < lastRerunStartDateNum,
    end%if if fileExistsFlag,
    %-----
    if and(fileExistsFlag==1,overwriteFlag==0),
        disp(['Cell mask and cell enhanced volume for block ',num2str(blockIter),' exists... Skipping...']);
    else%if and(fileExistsFlag==1,overwriteFlag==0),
        disp(['Binarizing cellBlock ',num2str(blockIter),' of ',num2str(length(cellBlockFileNameList)),' : ', datestr(now,'mm/dd/yy HH:MM:SS')]);
        loadedStruct = load(cellBlockFileNameList{blockIter});
        loadedVariables = fieldnames(loadedStruct);
        cellBlock = loadedStruct.(loadedVariables{1});
        if isstruct(cellBlock),
            tempPixelList = cellBlock.pixelList;
            tempMaskSize = cellBlock.maskSize;
            cellBlock = false(tempMaskSize);
            cellBlock(tempPixelList) = 1;
            clear tempPixelList tempMaskSize
        end%if isstruct(cellBlock),
        currentBlockSize = size(cellBlock);
        maxVol=maxVolPercent*numel(cellBlock);
        %-----
        paddedCellBlock = zeros(maxBlockSize,'single');
        paddedCellBlock(1:currentBlockSize(1),1:currentBlockSize(2),1:currentBlockSize(3)) = cellBlock;
        cellBlockNorm = norm(single(cellBlock(:)));
        clear cellBlock
        
        resultsStruct = [];
        %Filter the cell block with the cell-matched filter in fourier space
        cellBlockFFT = fftn(single(paddedCellBlock));
        clear paddedCellBlock;

        
        
        try
        CPXARR2=single(cellBlockFFT.*conj(cellFilterFFT));
        catch
            keyboard
        end
        
        
        
        filteredCellBlock=single(fftshift(ifftn(CPXARR2)));
        filteredCellBlock = filteredCellBlock(1:currentBlockSize(1),1:currentBlockSize(2),1:currentBlockSize(3));
        clear CPXARR2
        %-----
        if cellBlockNorm==0,
            filteredCellBlock(:) = 0;
            disp('cellMatchedFilter zero singular value detected');
        else
            %renormalize the filtered subblock by comparing to the unfiltered values
            filteredCellBlock=filteredCellBlock*(cellBlockNorm/norm(filteredCellBlock(:)));
        end
        %----------
        %----------
        %Filter the cell block with the background filter in fourier space
        CPXARR2 = single(cellBlockFFT.*conj(backgroundFilterFFT));
        clear cellBlockFFT
        filteredBackgroundBlock = single(fftshift(ifftn(CPXARR2)));
        filteredBackgroundBlock = filteredBackgroundBlock(1:currentBlockSize(1),1:currentBlockSize(2),1:currentBlockSize(3));
        clear CPXARR2
        %-----
        if cellBlockNorm==0,
            filteredBackgroundBlock = 0;
            disp('backgroundMatchedFilter zero singular value detected');
        else
            %renormalize the filtered subblock by comparing to the unfiltered values
            filteredBackgroundBlock=filteredBackgroundBlock*(cellBlockNorm/norm(filteredBackgroundBlock(:)));
        end
        %-----
        % filteredCellBlock is the matched filtered cells minus the background
        cellEnhanced=filteredCellBlock-filteredBackgroundBlock;
        clear filteredBackgroundBlock filteredCellBlock
        %---
        [cellBlockMask,histanalysis,resultsStruct]=VidaBinarizeSmoothCell2(cellEnhanced,numSigma,minVol,maxVol,displayFlag,resultsStruct);
        cellHist{blockIter}=histanalysis;
        saveSize = (getfield(whos('cellEnhanced'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(cellBlockEnhancedGrayscaleFileNameList{blockIter},'cellEnhanced','-mat',scs);
        %---
        pixelList = find(cellBlockMask);
        maskSize = size(cellBlockMask);
        cellBlockMask = [];
        cellBlockMask.pixelList = pixelList;
        cellBlockMask.maskSize = maskSize;
        saveSize = (getfield(whos('cellBlockMask'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(cellBlockMaskFileNameList{blockIter},'cellBlockMask','-mat','scs');

        
        
    end%if and(fileExistsFlag==1,overwriteFlag==0),
end%for blockIter=1:length(cellBlockFileNameList)


outputParameters = inputParameters;
outputParameters.cellBlockMaskFileNameList = cellBlockMaskFileNameList; 
outputParameters.cellBlockEnhancedGrayscaleFileNameList = cellBlockEnhancedGrayscaleFileNameList;
return












%RemovedCode (for debugging)
% % % %         %----------
% % % %         
% % % %         if displayFlag == 1
% % % %             middleFrame = floor(size(cellBlock,3)/2);
% % % %             rawCellSample = uint16(cellBlock(:,:,middleFrame));
% % % %             filtCellSample = uint16(filteredCellBlock(:,:,middleFrame));
% % % %             bkgSubCellSample = filtCellSample - uint16(filteredBackgroundBlock(:,:,middleFrame));
% % % %             figure(1);imagesc(rawCellSample);
% % % %             figure(2);imagesc(filtCellSample);
% % % %             figure(3);imagesc(middleFrame);
% % % %             pause(0.01);
% % % %         end%if displayFlag == 1


% % %         if saveFlag == 1,
% % %             resultsStruct.rawCellSample = rawCellSample;
% % %             resultsStruct.filtCellSample = filtCellSample;
% % %             resultsStruct.bkgSubCellSample = bkgSubCellSample;
% % %         end%if saveFlag == 1
% % %         
% % % 
% % %         %Save intermediate results if asked for
% % %         if saveFlag == 1,
% % %             [ySize,xSize] = size(resultsStruct.rawCellSample);
% % %             rawCellSample = zeros(maxBlockSize,'uint16');
% % %             filtCellSample = zeros(maxBlockSize,'uint16');
% % %             bkgSubCellSample = zeros(maxBlockSize,'uint16');
% % %             rawCellSample(1:ySize,1:xSize) = resultsStruct.rawCellSample;
% % %             filtCellSample(1:ySize,1:xSize) = resultsStruct.filtCellSample;
% % %             bkgSubCellSample(1:ySize,1:xSize) = resultsStruct.bkgSubCellSample;
% % %             outlineSampleFrame = resultsStruct.outlineSampleFrame;
% % %             switch blockIter,
% % %                 case 1
% % %                     imwrite(rawCellSample,fullfile(intermediateResultsFolder,[fileRootName,'_rawCellSample.tif']),'tiff','WriteMode','overwrite','Compression','none','Resolution',1);
% % %                     imwrite(filtCellSample,fullfile(intermediateResultsFolder,[fileRootName,'_filtCellSample.tif']),'tiff','WriteMode','overwrite','Compression','none','Resolution',1);
% % %                     imwrite(bkgSubCellSample,fullfile(intermediateResultsFolder,[fileRootName,'_bkgSubCellSample.tif']),'tiff','WriteMode','overwrite','Compression','none','Resolution',1);
% % %                     imwrite(outlineSampleFrame,fullfile(intermediateResultsFolder,[fileRootName,'_outlineCellSample.tif']),'tiff','WriteMode','overwrite','Compression','none','Resolution',1);
% % %                 otherwise
% % %                     imwrite(rawCellSample,fullfile(intermediateResultsFolder,[fileRootName,'_rawCellSample.tif']),'tiff','WriteMode','append','Compression','none','Resolution',1);
% % %                     imwrite(filtCellSample,fullfile(intermediateResultsFolder,[fileRootName,'_filtCellSample.tif']),'tiff','WriteMode','append','Compression','none','Resolution',1);
% % %                     imwrite(bkgSubCellSample,fullfile(intermediateResultsFolder,[fileRootName,'_bkgSubCellSample.tif']),'tiff','WriteMode','append','Compression','none','Resolution',1);
% % %                     imwrite(outlineSampleFrame,fullfile(intermediateResultsFolder,[fileRootName,'_outlineCellSample.tif']),'tiff','WriteMode','append','Compression','none','Resolution',1);
% % %             end%switch blockIter
% % %         end%if saveFlag == 1,
% % %         pause(0.1)
