function outExp = VidaCellSeg(inExp)
%last modified 08/27/07 by Phil Tsai

%-----
fileRootName = inExp.globalParameters.fileRootName;
rootDataFolder = inExp.globalParameters.rootDataFolder;
lastRerunStartDateNum = inExp.globalParameters.lastRerunStartDateNum;
processedDataFolder = inExp.globalParameters.processedDataFolder;
intermediateResultsFolder = inExp.globalParameters.intermediateResultsFolder;
voxelSizeInMicrons = inExp.globalParameters.voxelSizeInMicrons;
stackedFileNameList = inExp.globalParameters.stackedFileNameList;
fullStackSize = inExp.globalParameters.fullStackSize;
blockGrid = inExp.globalParameters.blockGrid;
subVolumeOverlapSize = inExp.globalParameters.subVolumeOverlapSize;
cellBlockFileNameList = inExp.globalParameters.cellBlockFileNameList;
%-----
cellMatchFilterModelVarianceInMicronsSquared = inExp.cellSegParameters.cellMatchFilterModelVarianceInMicronsSquared;             
cellMatchFilterBackgroundVarianceInMicronsSquared = inExp.cellSegParameters.cellMatchFilterBackgroundVarianceInMicronsSquared;   
cellMatchFilterNumSigma = inExp.cellSegParameters.cellMatchFilterNumSigma;                       
cellMatchFilterMinimumVolumeInCubicMicrons = inExp.cellSegParameters.cellMatchFilterMinimumVolumeInCubicMicrons;             
cellMatchFilterMaxVolumePercent = inExp.cellSegParameters. cellMatchFilterMaxVolumePercent;       
numVoxelsMarginBlank = inExp.cellSegParameters.numVoxelsMarginBlank;
%-----
cellMatchFilterModelVariance = cellMatchFilterModelVarianceInMicronsSquared ./ (voxelSizeInMicrons.^2);
cellMatchFilterBackgroundVariance = cellMatchFilterBackgroundVarianceInMicronsSquared ./ (voxelSizeInMicrons.^2);
cellMatchFilterMinimumVolume = cellMatchFilterMinimumVolumeInCubicMicrons ./ (voxelSizeInMicrons.^3);
cellSegFilesFolder = fullfile(processedDataFolder,'CellSegFiles');
[mkdirSuccess,mkdirMessage,mkdirId] = mkdir(cellSegFilesFolder);


%-----
% Produce a binary nuclei volume from each raw HORD block
disp(['Starting CellBinarize : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
if isfield(inExp,'vidaProjectRootFolder'),
    cellBinarizeParameters.vidaProjectRootFolder = inExp.vidaProjectRootFolder;
else%if isfield(inExp,'vidaProjectRootFolder'),
    cellBinarizeParameters.vidaProjectRootFolder = [];
end%if isfield(inExp,'vidaProjectRootFolder'),  
cellBinarizeParameters.lastRerunStartDateNum = lastRerunStartDateNum;
cellBinarizeParameters.fileRootName = fileRootName;
cellBinarizeParameters.intermediateResultsFolder = intermediateResultsFolder;
cellBinarizeParameters.cellSegFilesFolder = cellSegFilesFolder;
%---
cellBinarizeParameters.sigZ = cellMatchFilterNumSigma;              % # of stdev's above mean of peak distribution to call cell
cellBinarizeParameters.bgSigma = cellMatchFilterBackgroundVariance;          %Background estimate variance
cellBinarizeParameters.cellMFModelSigma = cellMatchFilterModelVariance;  %Cell matched filter variance 
cellBinarizeParameters.minVol = cellMatchFilterMinimumVolume;          %minimum # contiguous voxels in a cell region 
cellBinarizeParameters.maxVolPercent = cellMatchFilterMaxVolumePercent;    %max contiguous %size of subvolume that can be masked as a cell
%---
cellBinarizeParameters.cellBlockFileNameList = cellBlockFileNameList;
cellBinarizeParameters.blockGrid = blockGrid;
%-----
    cellBinarizeParameters = VidaCellBinarize2(cellBinarizeParameters);  %previously called OctoberCellBinarize2(EXPERIMENT);
cellBlockMaskFileNameList = cellBinarizeParameters.cellBlockMaskFileNameList; 
cellBlockEnhancedGrayscaleFileNameList = cellBinarizeParameters.cellBlockEnhancedGrayscaleFileNameList;

%-----    
% Stitch together all binarized cell subvolume blocks
disp(['Starting BinaryCellRestitch Filtering : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
cellRestitchParameters.fileRootName = fileRootName;
cellRestitchParameters.processedDataFolder = processedDataFolder;
cellRestitchParameters.intermediateResultsFolder = intermediateResultsFolder;
cellBinarizeParameters.cellSegFilesFolder = cellSegFilesFolder;
cellRestitchParameters.cellFileName = stackedFileNameList{1};
cellRestitchParameters.fullStackSize = fullStackSize;
cellRestitchParameters.overlapsz = subVolumeOverlapSize;
cellRestitchParameters.numVoxelsMarginBlank = numVoxelsMarginBlank;
%---
cellRestitchParameters.blockGrid = blockGrid;
cellRestitchParameters.cellBlockMaskFileNameList = cellBlockMaskFileNameList;
%---
    cellRestitchParameters = VidaBinaryCellRestitch(cellRestitchParameters);   %previously called MarchBinaryCellRestitch(EXPERIMENT);
%---
cellMaskFileName = cellRestitchParameters.cellMaskFileName; 
%-----
outExp = inExp;
outExp.globalParameters.cellBlockMaskFileNameList = cellBlockMaskFileNameList;
outExp.globalParameters.cellBlockEnhancedGrayscaleFileNameList = cellBlockEnhancedGrayscaleFileNameList;
outExp.globalParameters.cellMaskFileName = cellMaskFileName;
outExp.completionDates.cellSegCompletionDate = datestr(now);
%---
outExp.cellSegParameters.cellBinarizeParameters = cellBinarizeParameters;
outExp.cellSegParameters.cellRestitchParameters = cellRestitchParameters;
outExp.cellSegParameters.cellSegFilesFolder = cellSegFilesFolder;
%---

return



  


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%ADDITIONAL NOTES AND COMMENTS
%
%
%EXPERIMENT.vascthresh and EXPERIMENT.cellthresh
%   These two values set the saturation intensities for both cell and
%   vascular data. This partially mitigates the widening of structures
%   from the smoothing of very intense structures in the same subvolume
%   as smoothed structures that were not as intense.

%EXPERIMENT.sz3D and EXPERIMENT.overlapsz
%   These two values indicate the "max" of a 3D subvolume dimension
%   (200x200x200, e.g.), and how much subvolumes should overlap


%EXPERIMENT.CellMFModelSigma and EXPERIMENT.BGsigma
%   These parameters govern the binarization of the cell data
%   Although the labels here are CellMFSigma and BGsigma, they are really
%   *variances*, not standard deviations, of the gaussians that govern the
%   cell matched filtering and background estimation, respectively.

%EXPERIMENT.sigz
%   sigz is the number of standard deviations above the mean of the peak
%   distribution a voxel in the BG-subtracted, cell-enhanced sobvolume a
%   voxel must be to be labeled as part of a cell nucleus in the CellMask.

%EXPERIMENT.minvol
%   All labeled nuclei must also pass a size threshold (a 5x5x5 nucleus,
%   e.g., would correspond to ~5^3, or 100 voxels). Smaller contiguous 
%   collections of "on" voxels are rejected.


%EXPERIMENT.maxvolpercent
%    In some failure modes of this method, large volumes of very smooth
%    intensity hypersurfaces can be labeled as nuclei. Any connected
%    object that is larger than maxvolpercent as a proportion of the total
%    subvolume size is rejected.

