function outputParameters = VidaCenterlineEnhancedByConnectivity(inputParameters)

lastRerunStartDateNum = inputParameters.lastRerunStartDateNum;
vascBlockFileNameList = inputParameters.vascBlockFileNameList;
vascBlockRawMaskFileNameList = inputParameters.vascBlockRawMaskFileNameList;
vascBlockEnhancedMaskFileNameList = inputParameters.vascBlockEnhancedMaskFileNameList;

centerlineFilesFolder = inputParameters.centerlineFilesFolder;
bwFiltKernelSizeInMicrons = inputParameters.bwFiltKernelSizeInMicrons;  
maxFlips = inputParameters.maxFlips;
centeringFieldCircleDiameterInMicrons = inputParameters.centeringFieldCircleDiameterInMicrons;
hairTrimRadiusMultiplier = inputParameters.hairTrimRadiusMultiplier;
hairTrimRadiusAdditiveInMicrons = inputParameters.hairTrimRadiusAdditiveInMicrons;
maxCenteringFieldFilterRangeInMicrons = inputParameters.maxCenteringFieldFilterRangeInMicrons;
numCenteringFieldFilterCycles = inputParameters.numCenteringFieldFilterCycles;
%-----
centerlineRecenteringFlag = inputParameters.centerlineRecenteringFlag;
preCloseDiameterInMicrons = inputParameters.preCloseDiameterInMicrons;
centeringIterMultiplier = inputParameters.centeringIterMultiplier;
smallLoopCleanupSizeInMicrons = inputParameters.smallLoopCleanupSizeInMicrons;
minVolForConnectedCenterlineInMicrons = inputParameters.minVolForConnectedCenterlineInMicrons;
voxelSizeInMicrons = inputParameters.voxelSizeInMicrons;
%-----
bwFiltKernelSize = inputParameters.bwFiltKernelSizeInMicrons./voxelSizeInMicrons;  
centeringFieldCircleDiameter = centeringFieldCircleDiameterInMicrons ./ voxelSizeInMicrons;
hairTrimRadiusAdditive = hairTrimRadiusAdditiveInMicrons./ voxelSizeInMicrons;
maxCenteringFieldFilterRange = maxCenteringFieldFilterRangeInMicrons./ voxelSizeInMicrons;
smallLoopCleanupSize = round(smallLoopCleanupSizeInMicrons./ voxelSizeInMicrons);
minVolForConnectedCenterline = round(minVolForConnectedCenterlineInMicrons./ voxelSizeInMicrons);
centeringFieldCircleDiameter = centeringFieldCircleDiameterInMicrons./ voxelSizeInMicrons;
preCloseDiameter = preCloseDiameterInMicrons./voxelSizeInMicrons;
%-----
numCenteringFieldFilterCyclesOrig = numCenteringFieldFilterCycles;
%-----
overrideFileNumber = 0;
if isfield(inputParameters,'overrideFileNumber'),
    overrideFileNumber = inputParameters.overrideFileNumber;
end
%-----
outputFilePrefix = fullfile(centerlineFilesFolder,'VascBlockCenterlineMask');
if isfield(inputParameters,'overrideOutputFilePrefix'),
    outputFilePrefix = inputParameters.overrideOutputFilePrefix;
end
%-----


 
%Load the connectivity look-up-table
load('ConnectivityLUT.mat','connectivityLUT');
numBlocks = size(vascBlockRawMaskFileNameList,2);
vascBlockCenterlineMaskFileNameList = cell(1,numBlocks);
for blockIter = 1:numBlocks,
    
    vascBlockEnhancedMaskFileName = vascBlockEnhancedMaskFileNameList{blockIter};
    vascBlockRawMaskFileName = vascBlockRawMaskFileNameList{blockIter};
    vascBlockCenterlineMaskFileName = [outputFilePrefix,num2str(blockIter),'.mat'];
    %---
    if overrideFileNumber ~=0,
        vascBlockCenterlineMaskFileName = [outputFilePrefix,num2str(overrideFileNumber),'.mat'];
    end%if overrideFileNumber ~=0,
    %---
    vascBlockCenterlineMaskFileNameList{blockIter} = vascBlockCenterlineMaskFileName;
    %-----
    disp(['Centerline Vasc Mask Block ',num2str(blockIter),' of ',num2str(numBlocks),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
    overwriteFlag = 0;
    fileExistsFlag = (exist(vascBlockCenterlineMaskFileName,'file')==2);
    if fileExistsFlag,
        fileProps = dir(vascBlockCenterlineMaskFileName);
        fileDateNum = datenum(fileProps.date);
        if fileDateNum < lastRerunStartDateNum,
            overwriteFlag = 1;
        end%if fileDateNum < lastRerunStartDateNum,
    end%if if fileExistsFlag,
    %---
    if and(fileExistsFlag==1,overwriteFlag==0),
        disp(['...previously computed on ',datestr(fileDateNum,'mm/dd/yy HH:MM:SS')]);
        load(vascBlockCenterlineMaskFileName,'vascBlockCenterlineMask');
        if isstruct(vascBlockCenterlineMask),
            tempPixelList = vascBlockCenterlineMask.pixelList;
            tempMaskSize = vascBlockCenterlineMask.maskSize;
            vascBlockCenterlineMask = false(tempMaskSize);
            vascBlockCenterlineMask(tempPixelList) = 1;
            clear tempPixelList tempMaskSize
        end
    else%if and(fileExistsFlag==1,overwriteFlag==0),
        
%%        
        %STEP ONE - CALCULATING DISTANCES
        disp(['Step 1: Calculating Distances : ',datestr(now,'mm/dd/yy HH:MM:SS')]);

        %Load the original vascular masks
        blockStruct = load(vascBlockEnhancedMaskFileName);
        blockVariables = fieldnames(blockStruct);
        vascBlockEnhancedMask = blockStruct.(blockVariables{1});
        if isstruct(vascBlockEnhancedMask),
            pixelList = vascBlockEnhancedMask.pixelList;
            maskSize = vascBlockEnhancedMask.maskSize;
            vascBlockEnhancedMask = false(maskSize);
            vascBlockEnhancedMask(pixelList) = 1;
        end%if isstruct(vascBlockEnhancedMask),
        %-----
        blockStruct = load(vascBlockRawMaskFileName);
        blockVariables = fieldnames(blockStruct);
        vascBlockRawMask = blockStruct.(blockVariables{1});
        if isstruct(vascBlockRawMask),
            pixelList = vascBlockRawMask.pixelList;
            maskSize = vascBlockRawMask.maskSize;
            vascBlockRawMask = false(maskSize);
            vascBlockRawMask(pixelList) = 1;
        end%if isstruct(VascBlockEnhancedMask),
        
        
        blockSize = size(vascBlockEnhancedMask);
        closedvascBlockEnhancedMask = vascBlockEnhancedMask;
        if preCloseDiameter>1,
            numIter = floor(preCloseDiameter/2);
            for dilateIter = 1:numIter,
                closedvascBlockEnhancedMask = imdilate(closedvascBlockEnhancedMask,circle3d(3));    
            end
            for erodeIter = 1:numIter,
                closedvascBlockEnhancedMask = imerode(closedvascBlockEnhancedMask,circle3d(3));
            end
        end%if preCloseDiameter>1,
        closedvascBlockEnhancedMask = or(closedvascBlockEnhancedMask,vascBlockRawMask);        % "combinedMask" only used to form BW3D
        vascBlockCenterlineMask = closedvascBlockEnhancedMask;
        [ySize,xSize,zSize] = size(vascBlockCenterlineMask);
        
        
        %Remove edge pixels to avoid matrix indexing errors
        vascBlockCenterlineMask(1,:,:) = 0;
        vascBlockCenterlineMask(end,:,:) = 0;
        vascBlockCenterlineMask(:,1,:) = 0;
        vascBlockCenterlineMask(:,end,:) = 0;
        vascBlockCenterlineMask(:,:,1) = 0;
        vascBlockCenterlineMask(:,:,end) = 0;
        mybit = single(0);
           
        
        %disp('Performing 3x 1D bwdists on vascBlockEnhancedMask');
        %Perform a 3x 1D bwdist; take the median of the three values at
        %each pixel.
        bs1 = blockSize(1);bs2 = blockSize(2);bs3 = blockSize(3);
        numPixels = bs1*bs2*bs3;
        %Y-axis
        myline = reshape(vascBlockCenterlineMask,[1,numPixels]);
        bwLine = uint8(bwdist(~myline));
        bwMatrixY = reshape(bwLine,[bs1,bs2,bs3]);
        %X-axis 
        permuteBlock = permute(vascBlockCenterlineMask,[2,3,1]);
        myline = reshape(permuteBlock,[1,numPixels]);
        bwLine = uint8(bwdist(~myline));
        bwMatrixPermute = reshape(bwLine,[bs2,bs3,bs1]);
        bwMatrixX = permute(bwMatrixPermute,[3,1,2]);
        %Z-axis 
        permuteBlock = permute(vascBlockCenterlineMask,[3,1,2]);
        myline = reshape(permuteBlock,[1,numPixels]);
        bwLine = uint8(bwdist(~myline));
        bwMatrixPermute = reshape(bwLine,[bs3,bs1,bs2]);
        bwMatrixZ = permute(bwMatrixPermute,[2,3,1]);
        clear myline bwLine permuteBlock bwMatrixPermute
        %---
        %this 3x1D bwdist of the Enhanced Mask is used to determine
        %the order of pixel removal for trimming
        clear bwMatrixX bwMatrixY bwMatrixZ
        

        %a full 3D bwdist of the enhanced mask is used to seed the
        %centering field.
% % %         bw3D = bwdist(~closedvascBlockEnhancedMask);
bw3D = fastbwdist(~closedvascBlockEnhancedMask)./3;
        
  
        
        %Intersect the Raw and Enhanced Masks;
        %the intersected mask is used to determine the number of cycles
        %used to refine the centering field.
        intMask = and(vascBlockRawMask,vascBlockEnhancedMask);
 
              
% % %  bw3DRaw = bwdist(~intMask);
bw3DRaw = fastbwdist(~intMask)/3;       
        
   
        %Remove edge pixels to avoid matrix indexing errors
        intMask(1,:,:) = 0;
        intMask(end,:,:) = 0;
        intMask(:,1,:) = 0;
        intMask(:,end,:) = 0;
        intMask(:,:,1) = 0;
        intMask(:,:,end) = 0;
       
        
        
        %disp('Performing 3x 1D bwdists on intMask');
        %Perform a 3x 1D bwdist; take the median of the three values at
        %each pixel.
        bs1 = blockSize(1);bs2 = blockSize(2);bs3 = blockSize(3);
        numPixels = bs1*bs2*bs3;
        %Y-axis
        myline = reshape(intMask,[1,numPixels]);
        bwLine = uint8(bwdist(~myline));
        bwMatrixY = reshape(bwLine,[bs1,bs2,bs3]);
        %X-axis 
        permuteBlock = permute(intMask,[2,3,1]);
        myline = reshape(permuteBlock,[1,numPixels]);
        bwLine = uint8(bwdist(~myline));
        bwMatrixPermute = reshape(bwLine,[bs2,bs3,bs1]);
        bwMatrixX = permute(bwMatrixPermute,[3,1,2]);
        %Z-axis 
        permuteBlock = permute(intMask,[3,1,2]);
        myline = reshape(permuteBlock,[1,numPixels]);
        bwLine = uint8(bwdist(~myline));
        bwMatrixPermute = reshape(bwLine,[bs3,bs1,bs2]);
        bwMatrixZ = permute(bwMatrixPermute,[2,3,1]);
        clear myline bwLine permuteBlock bwMatrixPermute
        clear intMask
        %---
        %this 3x1D bwdist of the Intersected Mask is used to calculate
        %the number cycles for centering field refinement, and also
        %for determining associated radii for hair trimming 
        medianbw1DInt = double(median(cat(4,bwMatrixX,bwMatrixY,bwMatrixZ),4));
        clear bwMatrixX bwMatrixY bwMatrixZ
        
        
     

        %Generate a centering field by iterative filtering and masking    
% %         centeringField = bw3DRaw;

centeringField = bw3DRaw * 10 + bw3D;        
        
        
        filteringCircle = double(gaussian3d(centeringFieldCircleDiameter,centeringFieldCircleDiameter/3));
        circlesum = sum(filteringCircle(:));
        maxVesselDiameter = 2*max(medianbw1DInt(:));
        filterRange = min(maxCenteringFieldFilterRange,maxVesselDiameter);
  
        if numCenteringFieldFilterCyclesOrig == -1,
            numCenteringFieldFilterCycles = ceil(centeringIterMultiplier.*filterRange./(centeringFieldCircleDiameter-1));
        end%if numCenteringFieldFilterCycles = -1,

        
        for filterIter = 1:numCenteringFieldFilterCycles,
            centeringField = closedvascBlockEnhancedMask.*imfilter(centeringField,filteringCircle)./circlesum;
        end%for filterIter = 1:numCenteringFieldFilterCycles,

%         centeringField = max(1e-10,centeringField);             
        centeringField = centeringField.*vascBlockCenterlineMask;
        
clear vascBlockEnhancedMask vascBlockRawMask closedvascBlockEnhancedMask       
        
 %removeMe=1;maketiff(uint8(255*centeringField./max(centeringField(:))),'centeringField.tif');

%%

        %STEP TWO - FIRST ROUND TRIMMING
        disp(['Step 2: First Round Trimming : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
        

        
        %disp('Filtering bwDist')
        %Filter the 1-D bwDist in 3D to better eliminate "webbing"
        bwFiltKernel = gaussian3d(bwFiltKernelSize,bwFiltKernelSize);   
        kernelSum = sum(bwFiltKernel(:));

        distanceOrderedBW = bw3DRaw*100 + bw3D;
        clear bw3DRaw
        subbedBW = imfilter(bw3D,ones(3,3,3),'replicate')./100000;
        clear bw3D
        distanceOrderedBW = distanceOrderedBW-subbedBW;
        clear subbedBW
        distanceOrderedBW = distanceOrderedBW.*vascBlockCenterlineMask;

        
        %Remove edge pixels to avoid matrix indexing errors
        vascBlockCenterlineMask(1,:,:) = 0;
        vascBlockCenterlineMask(end,:,:) = 0;
        vascBlockCenterlineMask(:,1,:) = 0;
        vascBlockCenterlineMask(:,end,:) = 0;
        vascBlockCenterlineMask(:,:,1) = 0;
        vascBlockCenterlineMask(:,:,end) = 0;
        mybit = single(0);
        
     
        
        %Sort the pixels by their filtered BWdist values.
        pixelIndex = find(vascBlockCenterlineMask>0);
        myList = [pixelIndex,distanceOrderedBW(pixelIndex)];
        sortList = sortrows(myList,2);
        clear distanceOrderedBW
        
        %disp('Trimming pixels');
        %Trim pixels based on connectivity, going outside-in
        [w1List,w2List,w3List] = ind2sub(blockSize,sortList(:,1));
        powersOfTwo = pow2(26:-1:0);
        for pixelIter = 1:size(sortList,1),
            w1 = round(w1List(pixelIter));
            w2 = round(w2List(pixelIter));
            w3 = round(w3List(pixelIter));
            try
            subBlock = vascBlockCenterlineMask(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1);
            catch
                keyboard
            end
                
            subBlockLine = subBlock(:)';
             if sum(subBlockLine) == 2,
                 %Do not trim endpoints
                vascBlockCenterlineMask(w1,w2,w3) = 1;
            else%if sum(subBlockLine) == 2,
                decNum = sum(subBlockLine.*powersOfTwo , 2);
                vascBlockCenterlineMask(w1,w2,w3) = connectivityLUT(decNum);
            end%if sum(subBlockLine) == 2,
        end%for pixelIter = 1:size(sortList,1);
        
        
        clear checkerboard myList pixelIndex sortList w1List w2List w3List
        
%removeMe=1;maketiff(vascBlockCenterlineMask,'postRound1Trim.tif');pause(0.25);        
        


        
%% 





        %STEP THREE - RECENTERING BASED ON DILATED ENHANCED MASK        
        
        disp(['Step 3: First Round Recentering pixels : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
        %Recenter the skeleton pixel by pixel; find the off-neighbor
        %   with a 3D-bwDist value that is maximally greater than the current pixel
        %   Add that pixel, and try to remove the original pixel. If
        %   possible, make the swap; if not retain original config
        
     

        %Re-darken Edges
        vascBlockCenterlineMask(1,:,:) = 0; vascBlockCenterlineMask(end,:,:) = 0;
        vascBlockCenterlineMask(:,1,:) = 0; vascBlockCenterlineMask(:,end,:) = 0;
        vascBlockCenterlineMask(:,:,1) = 0; vascBlockCenterlineMask(:,:,end) = 0;

        %Arrange pixel evaluation order : Y then X then Z
% %         orig = vascBlockCenterlineMask;
        [ySize,xSize,zSize] = size(vascBlockCenterlineMask);
        indexBlock = find(ones(size(vascBlockCenterlineMask)));



        
        centerlineHistory = uint8(vascBlockCenterlineMask);
        centerlineChangedNeighbors = vascBlockCenterlineMask;
        keepGoing = 1;
        cycleCounter = 0;


        while keepGoing == 1,
            starterBlock = vascBlockCenterlineMask;
            cycleCounter = cycleCounter+1;
%             activePixels = and(vascBlockCenterlineMask,centerlineChangedNeighbors)>0;
            safeBlock = vascBlockCenterlineMask;
            safeBlock(2,:,:) = 0; safeBlock(end-1,:,:) = 0;
            safeBlock(:,2,:) = 0; safeBlock(:,end-1,:) = 0;
            safeBlock(:,:,2) = 0; safeBlock(:,:,end-1) = 0;
            activePixels = and(safeBlock,centerlineChangedNeighbors)>0;
            clear safeBlock
            pixelList = indexBlock(activePixels);
            numPixels = numel(pixelList);
            pixelRank = centeringField(pixelList);
            [sortRank,rankOrder] = sort(pixelRank);
            sortedPixelList = pixelList(rankOrder);
            [w1List,w2List,w3List] = ind2sub(blockSize,sortedPixelList);
            %------
            crossBlock = zeros(3,3,3);
            crossBlock(:,2,2) = 1; crossBlock(2,:,2) = 1; crossBlock(2,2,:) = 1;
            centerlineChangedNeighbors(:) = 0;

            for pixelIter = 1:numPixels,
                w1 = round(w1List(pixelIter));
                w2 = round(w2List(pixelIter));
                w3 = round(w3List(pixelIter));
                subBlock = vascBlockCenterlineMask(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1);
                if sum(subBlock(:))>2,


                    %First : can the pixel be removed?
                    subBlockLine = subBlock(:)';
                    decNum = sum(subBlockLine.*powersOfTwo , 2);
                    saveBit = connectivityLUT(decNum);%if saveBit == 0, then the bit is removable
                    if sum(subBlockLine) == 2,saveBit = 1; end %Do not trim endpoints

                        if w1 == 1,saveBit = 1;end
                        if w2 == 1,saveBit = 1;end
                        if w3 == 1,saveBit = 1;end
                        if w1 == ySize,saveBit = 1;end
                        if w2 == xSize,saveBit = 1;end
                        if w3 == zSize,saveBit = 1;end


                    if saveBit == 0 %removable,
                        subBlock(2,2,2) = 0;
                        vascBlockCenterlineMask(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1) = subBlock;
                        centerlineHistory(w1,w2,w3) = centerlineHistory(w1,w2,w3) + 1;
                    else%if saveBit == 0 % (non-removable),
                        %Second : if not, can the pixel be moved?
                        subHistory = centerlineHistory(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1);
                        subBlockDist = centeringField(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1);
                        subBlockGreater = subBlockDist>subBlockDist(2,2,2);
                        subBlockUnused = (subHistory < maxFlips);
                        subBlockAvailable = and(and(~subBlock,subBlockGreater),subBlockUnused)>0;
                        subBlockAvailableVals = subBlockDist(subBlockAvailable);
                        %of the available greater pixels, pick the greatest valued one 
                        %that will allow a move that maintains connectivity
                        numAvailable = numel(subBlockAvailableVals);
                        sortVals = sort(subBlockAvailableVals,'descend');
                        foundMoveFlag = 0;
                        if sum(subBlockLine) > 2, %only search if not an endpoint
                            for moveCheckIter = 1:numAvailable,
                                currentVal = sortVals(moveCheckIter);
                                subBlockAdd = (subBlockDist==currentVal);
                                testBlock = or(subBlock,subBlockAdd);
                                testBlockLine = testBlock(:)';
                                decNum = sum(testBlockLine.*powersOfTwo , 2);
                                keepBit = connectivityLUT(decNum);%if keepBit == 0, then the bit is movable
                                if keepBit == 0,
                                    foundMoveFlag = 1;
                                    replacementBlock = testBlock;
                                    replacementBlock(2,2,2) = 0;
                                    break
                                end%if keepBit == 0,
                            end%for moveCheckIter = 1:numAvailable,
                        end%if sum(subBlockLine) > 2,

                        if foundMoveFlag == 1, %movable



                            %add the new (brighter) pixel, remove the old
                            vascBlockCenterlineMask(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1) = replacementBlock;
                            centerlineHistory(w1,w2,w3) = centerlineHistory(w1,w2,w3) + 1;
                            centerlineChangedNeighbors(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1) = 1;
                        else %if foundMoveFlag == 1, %not movable
                            %Third: if not, add the brightest pixel along one of 3 axes
                            optionBlock = and(and(subBlockAvailable,crossBlock),subBlockUnused)>0;
                            optionBlockVals = subBlockDist(optionBlock);
                            numAddOptions = sum(optionBlock(:));
                            if numAddOptions>0,
                                maxNeighborVal = max(optionBlockVals);
                                addBlock = (subBlockDist==maxNeighborVal);
                                vascBlockCenterlineMask(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1) = or(addBlock,subBlock);
                                centerlineChangedNeighbors(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1) = 1;
                            else%if numAddOptions>0,
                                %do nothing - already at proper centerline
                            end%if numAddOptions>0,
                        end %if foundMoveFlag == 1,
                    end%if saveBit == 0
                end%if sum(subBlock(:))>2,
            end %for pixelIter = 1:numPixels,            

            
            %Re-darken Edges
            vascBlockCenterlineMask(1,:,:) = 0; vascBlockCenterlineMask(end,:,:) = 0;
            vascBlockCenterlineMask(:,1,:) = 0; vascBlockCenterlineMask(:,end,:) = 0;
            vascBlockCenterlineMask(:,:,1) = 0; vascBlockCenterlineMask(:,:,end) = 0;


            changedPixels = xor(vascBlockCenterlineMask,starterBlock);
            numChanged = sum(changedPixels(:));
            clear changedPixels
            if numChanged == 0,keepGoing = 0;end
        end%while keepGoing == 1,
        clear starterBlock indexBlock activePixels centerlineChangedNeighbors
        clear centerlineHistory
 


%removeMe=1;maketiff(vascBlockCenterlineMask,'postRound1Recenter.tif');pause(0.25);        
%removeMe=1;maketiff(uint8(centeringField*10),'centeringFieldRound1Recenter.tif');pause(0.25);        
        
        


        %Hair removal
        pixelIndex = find(vascBlockCenterlineMask>0);
        myList = [pixelIndex,centeringField(pixelIndex)];
        sortList = sortrows(myList,2);
        hairBlock = false(size(vascBlockCenterlineMask));
        
        
        %disp('Locating Junctions and Placing Spheres of Trimmable Voxels');
        %
        [w1List,w2List,w3List] = ind2sub(blockSize,pixelIndex(:,1));
        for pixelIter = 1:size(sortList,1),
            w1 = round(w1List(pixelIter));
            w2 = round(w2List(pixelIter));
            w3 = round(w3List(pixelIter));
            subBlock = vascBlockCenterlineMask(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1);
            subBlockLine = subBlock(:)';
            if sum(subBlockLine) > 3, %junction point
                
                associatedRadius = ceil(medianbw1DInt(w1,w2,w3)*hairTrimRadiusMultiplier)+hairTrimRadiusAdditive;
                q1i = max(1,w1-associatedRadius);
                q2i = max(1,w2-associatedRadius);
                q3i = max(1,w3-associatedRadius);
                q1f = min(ySize,w1+associatedRadius);
                q2f = min(xSize,w2+associatedRadius);
                q3f = min(zSize,w3+associatedRadius);
                [meshY,meshX,meshZ] = meshgrid([q2i-w2:q2f-w2],[q1i-w1:q1f-w1],[q3i-w3:q3f-w3]);
                hairGrid = sqrt(meshY.^2+meshX.^2+meshZ.^2);
                hairGridMask = hairGrid<=associatedRadius;
                hairBlock(q1i:q1f,q2i:q2f,q3i:q3f) = or(hairBlock(q1i:q1f,q2i:q2f,q3i:q3f),hairGridMask);
            end%if sum(subBlockLine) > 3,
        end%for pixelIter = 1:size(sortList,1)
        
        
 %removeMe=1;maketiff(hairBlock,'hairBlock.tif');
        
        
        %Preserve Edges
        vascBlockCenterlineMask(1,:,:) = 1; vascBlockCenterlineMask(end,:,:) = 1;
        vascBlockCenterlineMask(:,1,:) = 1; vascBlockCenterlineMask(:,end,:) = 1;
        vascBlockCenterlineMask(:,:,1) = 1; vascBlockCenterlineMask(:,:,end) = 1;
  
            
        keepGoing = 1;
        while keepGoing,
            %disp('Removing Hairs');
            %Trim hairs based on proximity to junctions
            keepGoing = 0;
            [w1List,w2List,w3List] = ind2sub(blockSize,sortList(:,1));
            for pixelIter = 1:size(sortList,1),
                w1 = round(w1List(pixelIter));
                w2 = round(w2List(pixelIter));
                w3 = round(w3List(pixelIter));
                subBlock = vascBlockCenterlineMask(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1);
                subHair = hairBlock(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1);
                if and( sum(subBlock(:)) == 2 , subHair(2,2,2) == 1),
                    %Trim the hair
                    vascBlockCenterlineMask(w1,w2,w3) = 0;
                    keepGoing = 1;
                end%if and( sum(subBlockLine) == 2 , subHair(2,2,2) == 1);
            end%for pixelIter = 1:size(sortList,1),
            keepGoing = 0;
        end%while keepGoing    
        clear hairBlock
        
        %Re-darken Edges
        vascBlockCenterlineMask(1,:,:) = 0; vascBlockCenterlineMask(end,:,:) = 0;
        vascBlockCenterlineMask(:,1,:) = 0; vascBlockCenterlineMask(:,end,:) = 0;
        vascBlockCenterlineMask(:,:,1) = 0; vascBlockCenterlineMask(:,:,end) = 0;
        
        
        

%removeMe=1;maketiff(vascBlockCenterlineMask,'postRound1HairRemoval.tif');pause(0.25);        
        


      
        
        
        
        

%% 





        %STEP FOUR - DILATE AND RECENTER TO REMOVE SMALL LOOPS
        
        disp(['Step 4: Second Round Recentering pixels : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
        %Recenter the skeleton pixel by pixel; find the off-neighbor
        %   with a 3D-bwDist value that is maximally greater than the current pixel
        %   Add that pixel, and try to remove the original pixel. If
        %   possible, make the swap; if not retain original config
        

        dilatedCenterline = vascBlockCenterlineMask;
        for dilateIter = 1:smallLoopCleanupSize;
            dilatedCenterline = imfilter(dilatedCenterline,double(circle3d(3)))>0;
        end%for dilateIter = 1:smallLoopCleanupSize;
        
% % %         centeringField = bwdist(~dilatedCenterline);
centeringField = fastbwdist(~dilatedCenterline)./3;  
        clear dilatedCenterline
        

        mygaussian = gaussian3d(5,3);
        centeringField = imfilter(centeringField,mygaussian,'replicate');
        
        
        centeringField(1,:,:) = 0; centeringField(end,:,:) = 0;
        centeringField(:,1,:) = 0; centeringField(:,end,:) = 0;
        centeringField(:,:,1) = 0; centeringField(:,:,end) = 0;

        

        %Re-darken Edges
        vascBlockCenterlineMask(1,:,:) = 0; vascBlockCenterlineMask(end,:,:) = 0;
        vascBlockCenterlineMask(:,1,:) = 0; vascBlockCenterlineMask(:,end,:) = 0;
        vascBlockCenterlineMask(:,:,1) = 0; vascBlockCenterlineMask(:,:,end) = 0;
    

        
        
% %         orig = vascBlockCenterlineMask;
        [ySize,xSize,zSize] = size(vascBlockCenterlineMask);
        indexBlock = find(ones(size(vascBlockCenterlineMask)));



        
        if centerlineRecenteringFlag == 1,
            centerlineHistory = uint8(vascBlockCenterlineMask);
            centerlineChangedNeighbors = vascBlockCenterlineMask;
            keepGoing = 1;
            cycleCounter = 0;
            while keepGoing == 1,
                starterBlock = vascBlockCenterlineMask;
                cycleCounter = cycleCounter+1;
                activePixels = and(vascBlockCenterlineMask,centerlineChangedNeighbors)>0;
                pixelList = indexBlock(activePixels);
                numPixels = numel(pixelList);
                pixelRank = centeringField(pixelList);
                [sortRank,rankOrder] = sort(pixelRank);
                sortedPixelList = pixelList(rankOrder);
                [w1List,w2List,w3List] = ind2sub(blockSize,sortedPixelList);
                %------
                crossBlock = zeros(3,3,3);
                crossBlock(:,2,2) = 1; crossBlock(2,:,2) = 1; crossBlock(2,2,:) = 1;
                centerlineChangedNeighbors(:) = 0;
                
                for pixelIter = 1:numPixels,
                    w1 = round(w1List(pixelIter));
                    w2 = round(w2List(pixelIter));
                    w3 = round(w3List(pixelIter));
                    subBlock = vascBlockCenterlineMask(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1);
                    if sum(subBlock(:))>2,
                        
                       
                        %First : can the pixel be removed?
                        subBlockLine = subBlock(:)';
                        decNum = sum(subBlockLine.*powersOfTwo , 2);
                        saveBit = connectivityLUT(decNum);%if saveBit == 0, then the bit is removable
                        if sum(subBlockLine) == 2,saveBit = 1; end %Do not trim endpoints
                        
                            if w1 == 1,saveBit = 1;end
                            if w2 == 1,saveBit = 1;end
                            if w3 == 1,saveBit = 1;end
                            if w1 == ySize,saveBit = 1;end
                            if w2 == xSize,saveBit = 1;end
                            if w3 == zSize,saveBit = 1;end

                        
                        if saveBit == 0 %removable,
                            subBlock(2,2,2) = 0;
                            vascBlockCenterlineMask(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1) = subBlock;
                            centerlineHistory(w1,w2,w3) = centerlineHistory(w1,w2,w3) + 1;
                        else%if saveBit == 0 % (non-removable),
                            %Second : if not, can the pixel be moved?
                            subHistory = centerlineHistory(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1);
                            subBlockDist = centeringField(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1);
                            subBlockGreater = subBlockDist>subBlockDist(2,2,2);
                            subBlockUnused = (subHistory < maxFlips);
                            subBlockAvailable = and(and(~subBlock,subBlockGreater),subBlockUnused)>0;
                            subBlockAvailableVals = subBlockDist(subBlockAvailable);
                            %of the available greater pixels, pick the greatest valued one 
                            %that will allow a move that maintains connectivity
                            numAvailable = numel(subBlockAvailableVals);
                            sortVals = sort(subBlockAvailableVals,'descend');
                            foundMoveFlag = 0;
                            if sum(subBlockLine) > 2, %only search if not an endpoint
                                for moveCheckIter = 1:numAvailable,
                                    currentVal = sortVals(moveCheckIter);
                                    subBlockAdd = (subBlockDist==currentVal);
                                    testBlock = or(subBlock,subBlockAdd);
                                    testBlockLine = testBlock(:)';
                                    decNum = sum(testBlockLine.*powersOfTwo , 2);
                                    keepBit = connectivityLUT(decNum);%if keepBit == 0, then the bit is movable
                                    if keepBit == 0,
                                        foundMoveFlag = 1;
                                        replacementBlock = testBlock;
                                        replacementBlock(2,2,2) = 0;
                                        break
                                    end%if keepBit == 0,
                                end%for moveCheckIter = 1:numAvailable,
                            end%if sum(subBlockLine) > 2,

                            if foundMoveFlag == 1, %movable
                                
                               
                                
                                %add the new (brighter) pixel, remove the old
                                vascBlockCenterlineMask(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1) = replacementBlock;
                                centerlineHistory(w1,w2,w3) = centerlineHistory(w1,w2,w3) + 1;
                                centerlineChangedNeighbors(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1) = 1;
                            else %if foundMoveFlag == 1, %not movable
                                %Third: if not, add the brightest pixel along one of 3 axes
                                optionBlock = and(and(subBlockAvailable,crossBlock),subBlockUnused)>0;
                                optionBlockVals = subBlockDist(optionBlock);
                                numAddOptions = sum(optionBlock(:));
                                if numAddOptions>0,
                                    maxNeighborVal = max(optionBlockVals);
                                    addBlock = (subBlockDist==maxNeighborVal);
                                    addedBlock = or(addBlock,subBlock);
                                    vascBlockCenterlineMask(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1) = addedBlock;
                                    centerlineChangedNeighbors(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1) = 1;
                                else%if numAddOptions>0,
                                    %do nothing - already at proper centerline
                                end%if numAddOptions>0,
                            end %if foundMoveFlag == 1,
                        end%if saveBit == 0
                    end%if sum(subBlock(:))>2,
                end %for pixelIter = 1:numPixels,            
                
               
               
                 %Re-darken Edges
                vascBlockCenterlineMask(1,:,:) = 0; vascBlockCenterlineMask(end,:,:) = 0;
                vascBlockCenterlineMask(:,1,:) = 0; vascBlockCenterlineMask(:,end,:) = 0;
                vascBlockCenterlineMask(:,:,1) = 0; vascBlockCenterlineMask(:,:,end) = 0;

                    
                changedPixels = xor(vascBlockCenterlineMask,starterBlock);
                numChanged = sum(changedPixels(:));
                clear changedPixels
                if numChanged == 0,keepGoing = 0;end
% % %                 disp([sum(centerlineHistory(:)),sum(vascBlockCenterlineMask(:))]);
            end%while keepGoing == 1,
        end%if centerlineRecenteringFlag == 1,
        clear powersOfTwo activePixels indexBlock centerlineChangedNeighbors
        clear centerlineHistory starterBlock


%removeMe=1;maketiff(vascBlockCenterlineMask,'postRound2Recenter.tif');pause(0.25);        
        
        


        %Hair removal
        pixelIndex = find(vascBlockCenterlineMask>0);
        myList = [pixelIndex,centeringField(pixelIndex)];
        clear centeringField
        sortList = sortrows(myList,2);
        hairBlock = (vascBlockCenterlineMask==-1);
       
        
        %disp('Locating Junctions');
        %
        [w1List,w2List,w3List] = ind2sub(blockSize,pixelIndex(:,1));
        clear pixelIndex
        for pixelIter = 1:size(sortList,1),
            w1 = round(w1List(pixelIter));
            w2 = round(w2List(pixelIter));
            w3 = round(w3List(pixelIter));
            subBlock = vascBlockCenterlineMask(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1);
            subBlockLine = subBlock(:)';
            if sum(subBlockLine) > 3, %junction point
                
                associatedRadius = ceil(medianbw1DInt(w1,w2,w3)*hairTrimRadiusMultiplier)+hairTrimRadiusAdditive;
                q1i = max(1,w1-associatedRadius);
                q2i = max(1,w2-associatedRadius);
                q3i = max(1,w3-associatedRadius);
                q1f = min(ySize,w1+associatedRadius);
                q2f = min(xSize,w2+associatedRadius);
                q3f = min(zSize,w3+associatedRadius);
                [meshY,meshX,meshZ] = meshgrid([q2i-w2:q2f-w2],[q1i-w1:q1f-w1],[q3i-w3:q3f-w3]);
                hairGrid = sqrt(meshY.^2+meshX.^2+meshZ.^2);
                hairGridMask = hairGrid<=associatedRadius;
                hairBlock(q1i:q1f,q2i:q2f,q3i:q3f) = max(hairBlock(q1i:q1f,q2i:q2f,q3i:q3f),hairGridMask);
            end%if sum(subBlockLine) > 3,
        end%for pixelIter = 1:size(sortList,1),
        clear medianbw1DInt
        
 %removeMe=1;maketiff(hairBlock,'hairBlock.tif');
        
        
        %Preserve Edges
        vascBlockCenterlineMask(1,:,:) = 1; vascBlockCenterlineMask(end,:,:) = 1;
        vascBlockCenterlineMask(:,1,:) = 1; vascBlockCenterlineMask(:,end,:) = 1;
        vascBlockCenterlineMask(:,:,1) = 1; vascBlockCenterlineMask(:,:,end) = 1;
  
            
        keepGoing = 1;
        while keepGoing,
            %disp('Removing Hairs');
            %Trim hairs based on proximity to junctions
            keepGoing = 0;
            [w1List,w2List,w3List] = ind2sub(blockSize,sortList(:,1));
            for pixelIter = 1:size(sortList,1),
                w1 = round(w1List(pixelIter));
                w2 = round(w2List(pixelIter));
                w3 = round(w3List(pixelIter));
                subBlock = vascBlockCenterlineMask(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1);
                subHair = hairBlock(w1-1:w1+1,w2-1:w2+1,w3-1:w3+1);
                subBlockLine = subBlock(:)';
                if and( sum(subBlockLine) == 2 , subHair(2,2,2) == 1),
                    %Trim the hair
                    vascBlockCenterlineMask(w1,w2,w3) = 0;
                    keepGoing = 1;
                end%if and( sum(subBlockLine) == 2 , subHair(2,2,2) == 1);
            end%for pixelIter = 1:size(sortList,1),
        end%while keepGoing   
        clear hairBlock        
        
            
        %Re-darken Edges
        vascBlockCenterlineMask(1,:,:) = 0; vascBlockCenterlineMask(end,:,:) = 0;
        vascBlockCenterlineMask(:,1,:) = 0; vascBlockCenterlineMask(:,end,:) = 0;
        vascBlockCenterlineMask(:,:,1) = 0; vascBlockCenterlineMask(:,:,end) = 0;
        
        
        

%removeMe=1;maketiff(vascBlockCenterlineMask,'postRound2HairRemoval.tif');pause(0.25);        
        


%% 

        %Eliminate contiguous centerline regions of size  [minvol] or less
        L=bwlabeln(vascBlockCenterlineMask,26);
        clear vascBlockCenterlineMask
        stmp=regionprops(L,'Area');
        LUT=[0,[stmp.Area]>minVolForConnectedCenterline];
        L=LUT(L+1);
        vascBlockCenterlineMask = L>0;
        clear L LUT



        
      


%%       
%removeMe=1;maketiff(vascBlockCenterlineMask,['CLMask',num2str(blockIter),'.tif']);
      
        %-----
        pixelList = find(vascBlockCenterlineMask);
        maskSize = size(vascBlockCenterlineMask);
        vascBlockCenterlineMask = [];
        vascBlockCenterlineMask.pixelList = pixelList;
        vascBlockCenterlineMask.maskSize = maskSize;
        saveSize = (getfield(whos('vascBlockCenterlineMask'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(vascBlockCenterlineMaskFileName,'vascBlockCenterlineMask',scs);

        
        %-----
        clear vascBlockCenterlineMask pixelList
        
    end %if-else and(fileExistsFlag==1,overwriteFlag==0),
end% for blockIter = 1:numBlocks,


outputParameters = inputParameters;
outputParameters.vascBlockCenterlineMaskFileNameList = vascBlockCenterlineMaskFileNameList;







