function outExp = VidaNeunBlockChannelSub(inExp)
%last modified 05/15/08 by Phil Tsai


%-----
fileRootName = inExp.globalParameters.fileRootName;
rootDataFolder = inExp.globalParameters.rootDataFolder;
lastRerunStartDateNum = inExp.globalParameters.lastRerunStartDateNum;
processedDataFolder = inExp.globalParameters.processedDataFolder;
intermediateResultsFolder = inExp.globalParameters.intermediateResultsFolder;
cellBlockFileNameList = inExp.globalParameters.cellBlockFileNameList;
vascBlockFileNameList = inExp.globalParameters.vascBlockFileNameList;
neunBlockFileNameList = inExp.globalParameters.neunBlockFileNameList;
cellBlockMaskFileNameList = inExp.globalParameters.cellBlockMaskFileNameList;
vascBlockMaskFileNameList = inExp.globalParameters.vascBlockRawMaskFileNameList;
voxelSizeInMicrons = inExp.globalParameters.voxelSizeInMicrons;
%-----
maxSubFraction = inExp.neunBlockChannelSubParameters.maxSubFraction;
subFractionStepSize = inExp.neunBlockChannelSubParameters.subFractionStepSize;
channelSubFitPolynomialDegree = inExp.neunBlockChannelSubParameters.channelSubFitPolynomialDegree;
channelSubMaskSubSampleFactor = inExp.neunBlockChannelSubParameters.channelSubMaskSubSampleFactor;
channelSubNonMaskSubSampleFactor = inExp.neunBlockChannelSubParameters.channelSubNonMaskSubSampleFactor;
topCellNeuNRatioFraction = inExp.neunBlockChannelSubParameters.topCellNeuNRatioFraction;
neunCellFilterSizeInMicrons = inExp.neunBlockChannelSubParameters.neunCellFilterSizeInMicrons;
%-----
neunCellFilterSize = round(neunCellFilterSizeInMicrons ./ voxelSizeInMicrons);
%-----
displayFlag = 0;
%-----
channelSubFactorList1 = [subFractionStepSize:subFractionStepSize:maxSubFraction];
channelSubFactorList2 = [subFractionStepSize:subFractionStepSize:maxSubFraction];
%---
fullStackSize = inExp.globalParameters.fullStackSize;
subVolumeOverlapSize = inExp.globalParameters.subVolumeOverlapSize;
blockGrid = inExp.globalParameters.blockGrid;

%---
%---
neunBlockChannelSubFilesFolder = fullfile(processedDataFolder,'neunBlockChannelSubFiles');
[mkdirSuccess,mkdirMessage,mkdirId] = mkdir(neunBlockChannelSubFilesFolder);
disp(['Performing Neun Block Channel Subtraction - stage 1 : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
bestFactorList = zeros(1,length(neunBlockFileNameList));
outputFileNameList = cell(1,length(neunBlockFileNameList));
numBlocks = length(neunBlockFileNameList);
for blockIter = 1:numBlocks;
    currentNeunBlockFileName = neunBlockFileNameList{blockIter};
    currentVascBlockFileName = vascBlockFileNameList{blockIter};
    currentVascBlockMaskFileName = vascBlockMaskFileNameList{blockIter};
    [pathName,fileName,ext,versn] = fileparts(currentNeunBlockFileName);
    newFileName = regexprep(fileName,'neunBlock-','neunBlock-temp-subch-');
    outputBlockFileName = fullfile(neunBlockChannelSubFilesFolder,[newFileName,ext]); 
    %-----
    overwriteFlag = 0;
    fileExistsFlag = 0;
    if exist(outputBlockFileName,'file'),
        fileExistsFlag = 1;
        fileProps = dir(outputBlockFileName);
        fileDateNum = datenum(fileProps.date);
        if fileDateNum < lastRerunStartDateNum,
            overwriteFlag = 1;
        end%if fileDateNum < lastRerunStartDateNum,
    end%if exist(outputBlockFileName,'file'),
    %---
    if and(fileExistsFlag==1,overwriteFlag==0),
        disp(['skipping block ',num2str(blockIter),' - previously completed...']); %---added by Phil 03/02/07
        outputFileNameList{blockIter}=outputBlockFileName;
        load(outputBlockFileName,'neunBlock','extendedFactorList','bestFactorList');
    else%if and(fileExistsFlag==1,overwriteFlag==0),
        disp(['block ',num2str(blockIter),' of ',num2str(length(neunBlockFileNameList)),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
        % Perform Channel Subtraction of the Vasc signal from
        % the neun data using the previously computed cell masks.
        %-----
        loadedStruct = load(currentVascBlockFileName);
        loadedVariables = fieldnames(loadedStruct);
        vascBlock = loadedStruct.(loadedVariables{1});
        if isstruct(vascBlock),
            tempPixelList = vascBlock.pixelList;
            tempMaskSize = vascBlock.maskSize;
            vascBlock = false(tempMaskSize);
            vascBlock(tempPixelList) = 1;
            clear tempPixelList tempMaskSize
        end%if isstruct(vascBlock),
        vascBlock = single(vascBlock);
        %-----
        loadedStruct = load(currentNeunBlockFileName);
        loadedVariables = fieldnames(loadedStruct);
        neunBlock = loadedStruct.(loadedVariables{1});
        if isstruct(neunBlock),
            tempPixelList = neunBlock.pixelList;
            tempMaskSize = neunBlock.maskSize;
            neunBlock = false(tempMaskSize);
            neunBlock(tempPixelList) = 1;
            clear tempPixelList tempMaskSize
        end%if isstruct(ZZZZZ),
        neunBlock = single(neunBlock);
        %-----
        load(currentVascBlockMaskFileName,'vascBlockRawMask','-mat');
        if isstruct(vascBlockRawMask),
            currentPixelList = vascBlockRawMask.pixelList;
            currentMaskSize = vascBlockRawMask.maskSize;
            vascBlockRawMask = false(currentMaskSize);
            vascBlockRawMask(currentPixelList) = 1;
        end%if isstruct(vascBlockCenterlineMask),
        %-----
        numFactors = length(channelSubFactorList1);
        maskPos = find(vascBlockRawMask(:));
        nonMaskPos = find(~vascBlockRawMask(:));
        maskPos = maskPos(1:channelSubMaskSubSampleFactor:end);
        nonMaskPos = nonMaskPos(1:channelSubNonMaskSubSampleFactor:end);
        %-----
        maskedMedianValue = zeros(1,numFactors);
        notMaskedMedianValue = zeros(1,numFactors);
        neunMasked = neunBlock(maskPos);
        vascMasked = vascBlock(maskPos);
        neunNotMasked = neunBlock(nonMaskPos);
        vascNotMasked = vascBlock(nonMaskPos);
        for iterFactor = 1:numFactors;
             currentFactor = channelSubFactorList1(iterFactor);
             subtractedMasked = neunMasked-currentFactor.*vascMasked;
             subtractedNotMasked = neunNotMasked-currentFactor.*vascNotMasked;
            
             %Calculate mode of masked regions:
             medianVal = median(single(subtractedMasked));
             maskedMedianValue(iterFactor) = medianVal;
             %Calculate mode of non-masked regions:
             medianVal = median(subtractedNotMasked);
             notMaskedMedianValue(iterFactor) = medianVal;
        end%for iterFactor = 1:numFactors;
        diffMedianValue = maskedMedianValue - notMaskedMedianValue;
        %-----
        [p,S] = polyfit(channelSubFactorList1,diffMedianValue,channelSubFitPolynomialDegree);
        extendedFactorList = [0:0.0025:channelSubFactorList1(end)];
        f = polyval(p,extendedFactorList);
        closestToZero = find(abs(f) == min(abs(f)));


        if isempty(closestToZero),
            bestFactor = extendedFactorList(1);
        else
            bestFactor = extendedFactorList(closestToZero(1));
        end

        neunBlock = max(0,neunBlock-bestFactor.*vascBlock);
        neunBlock=single(neunBlock);
        %-----
% % %         disp(['Channel Subtraction Factor : ',num2str(bestFactor)]);
% % %         figure(1); plot(extendedFactorList,f);
% % %         pause(0.1);
        %-----
        outputFileNameList{blockIter}=outputBlockFileName;
        bestFactorList(blockIter) = bestFactor;
        saveSize = (getfield(whos('neunBlock'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(outputBlockFileName,'neunBlock','extendedFactorList','bestFactorList',scs);
    end%if and(fileExistsFlag==1,overwriteFlag==0),
end%for blockIter=1:numBlocks
%------
%Now replace any maxed-out subtraction factors with the median of the
%other factors.
maxFactor = max(extendedFactorList);
maxedOutList = find(bestFactorList==maxFactor);
notMaxedOutList = find(bestFactorList~=maxFactor);


%-----
if isempty(notMaxedOutList),
    medianFactor = 0;
    disp('WARNING! ALL subtraction factors maxed out - setting factor to 0');%noteToSelf
    disp('WARNING! ALL subtraction factors maxed out - setting factor to 0');
    disp('WARNING! ALL subtraction factors maxed out - setting factor to 0');
    disp('WARNING! ALL subtraction factors maxed out - setting factor to 0');
    disp('WARNING! ALL subtraction factors maxed out - setting factor to 0');
    disp('WARNING! ALL subtraction factors maxed out - setting factor to 0');
end
%-----
medianFactor = median(bestFactorList(notMaxedOutList));
disp('Replacing maxed out subtraction factor with a median factor');
disp(['Median Factor : ',num2str(medianFactor)]);
for replaceIter = 1:length(maxedOutList);
    blockIter = maxedOutList(replaceIter);
    disp(['Re-running block ',num2str(blockIter),' : ',datestr(now,'mm:dd:yy HH:MM:SS')]);

    currentNeunBlockFileName = neunBlockFileNameList{blockIter};
    currentVascBlockFileName = vascBlockFileNameList{blockIter};
    load(currentVascBlockFileName,'vascBlock','-mat');
    vascBlock = single(vascBlock);
    load(currentNeunBlockFileName,'neunBlock','-mat');
    neunBlock = single(neunBlock);
    [pathName,fileName,ext,versn] = fileparts(currentNeunBlockFileName);
    newFileName = regexprep(fileName,'neunBlock-','neunBlock-temp-subch-');
    outputBlockFileName = fullfile(neunBlockChannelSubFilesFolder,[newFileName,ext]); 
    subtractedBlock = neunBlock - medianFactor.*vascBlock;
    neunBlock = subtractedBlock;
    outputFileNameList{blockIter}=outputBlockFileName;
    bestFactorList(blockIter) = medianFactor;
    saveSize = (getfield(whos('neunBlock'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(outputBlockFileName,'neunBlock','extendedFactorList','bestFactorList',scs);

end%for replaceIter = 1:length(maxedOutList);
%---
stage1BestFactorList = bestFactorList;
stage1MedianFactor = medianFactor;
stage1MaxedOutList = maxedOutList;
stage1OutputFileNameList = outputFileNameList;
%----------
%----------
disp(['Performing Neun Block Channel Subtraction - stage 2 : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
bestFactorList = zeros(1,length(neunBlockFileNameList));
outputFileNameList = cell(1,length(neunBlockFileNameList));
for blockIter = 1:length(neunBlockFileNameList);
    currentNeunBlockFileName = stage1OutputFileNameList{blockIter};
    currentCellBlockFileName = cellBlockFileNameList{blockIter};
    currentCellBlockMaskFileName = cellBlockMaskFileNameList{blockIter};
    [pathName,fileName,ext,versn] = fileparts(currentNeunBlockFileName);
    newFileName = regexprep(fileName,'neunBlock-temp-subch','neunBlock-subch');
    outputBlockFileName = fullfile(neunBlockChannelSubFilesFolder,[newFileName,ext]); 
    %-----
    overwriteFlag = 0;
    fileExistsFlag = 0;
    if exist(outputBlockFileName,'file'),
        fileExistsFlag = 1;
        fileProps = dir(outputBlockFileName);
        fileDateNum = datenum(fileProps.date);
        if fileDateNum < lastRerunStartDateNum,
            overwriteFlag = 1;
        end%if fileDateNum < lastRerunStartDateNum,
    end%if exist(outputBlockFileName,'file'),
    %---
    if and(fileExistsFlag==1,overwriteFlag==0),
        disp(['skipping block ',num2str(blockIter),' - previously completed...']); %---added by Phil 03/02/07
        outputFileNameList{blockIter}=outputBlockFileName;
        load(outputBlockFileName,'neunBlock','extendedFactorList','bestFactorList');
    else%if and(fileExistsFlag==1,overwriteFlag==0),
        disp(['block ',num2str(blockIter),' of ',num2str(length(neunBlockFileNameList)),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
        load(currentCellBlockFileName);
        cellBlock = single(cellBlock);
        load(currentNeunBlockFileName,'neunBlock');
        neunBlock = single(neunBlock);
        load(currentCellBlockMaskFileName,'cellBlockMask');
        if isstruct(cellBlockMask),
            currentPixelList = cellBlockMask.pixelList;
            currentMaskSize = cellBlockMask.maskSize;
            cellBlockMask = false(currentMaskSize);
            cellBlockMask(currentPixelList) = 1;
        end%if isstruct(cellBlockMask),
        %-----
        medianNeunVal = median(neunBlock(:));
        ncfs = neunCellFilterSize;
        filtCell = imfilter(cellBlock,ones(ncfs,ncfs,ncfs))./ncfs^3;
        filtNeun = imfilter(neunBlock,ones(ncfs,ncfs,ncfs))./ncfs^3;
        filtNeun = max(filtNeun,medianNeunVal);


        medianFiltVal = max(median(filtNeun(:)),1);
        filtNeun(find(filtNeun==0)) = medianFiltVal;
        filtRatio = filtCell./filtNeun; %assuming "calibrated" gray levels (bkg,bright ~ 1,1000 for either channel)  
                    %expect ratio to be (high/low+bleed) = high where there are cells that are not neurons
                    %cells that are neurons should have (high/high+bleed) ~1
                    %non cell regions should have (low/low) ~1
        %-----
        %find the top 1% of the ratio (sample some of the non-neuron cell ratios)
        cutOffIndex = round((1-topCellNeuNRatioFraction)*numel(filtRatio));
        sortRatio = sort(filtRatio(:));
        cutOffValue = sortRatio(cutOffIndex);
        spillOverMask = filtRatio>cutOffValue; %contains voxel ratios that correspond to nonNeurons with high confidence
        %-----    
        dilatedSpillOverMask = imdilate(spillOverMask,circle3d(11));
        dilatedCellMask = imdilate(cellBlockMask,circle3d(3));
        bkgMask = (dilatedSpillOverMask - dilatedCellMask)>0; %attempt to make a mask that is the perimeter around the nuclei of non-neurons
        %-----
        numFactors = length(channelSubFactorList2);
        diffMedianValue = zeros(1,numFactors);
        neunSpillover = neunBlock(spillOverMask);
        cellSpillover = cellBlock(spillOverMask);
        neunBkg = neunBlock(bkgMask);
        cellBkg = cellBlock(bkgMask);
        for factorIter = 1:numFactors,
            currentFactor = channelSubFactorList2(factorIter);
            subSpillover = neunSpillover - currentFactor.*cellSpillover;
            subBkg = neunBkg - currentFactor.*cellBkg;
            sigMedian = median(subSpillover);
            bkgMedian = median(subBkg);
            diffMedianValue(factorIter) = sigMedian-bkgMedian; %estimate of delta over "perinuclear-bkg" for non-neurons in the Neun-label channel
        end
        %-----
        [p,S] = polyfit(channelSubFactorList2,diffMedianValue,channelSubFitPolynomialDegree);
        extendedFactorList = [0:0.0025:channelSubFactorList2(end)];
        f = polyval(p,extendedFactorList);
        closestToZero = find(abs(f) == min(abs(f)));

        if isempty(closestToZero),
            bestFactor = extendedFactorList(1);
        else
            bestFactor = extendedFactorList(closestToZero(1));
        end



        neunBlock = max(0,neunBlock-bestFactor.*cellBlock);
        neunBlock=single(neunBlock);
        %-----
        disp(['Channel Subtraction Factor : ',num2str(bestFactor)]);
        if displayFlag == 1,
            figure(1); plot(extendedFactorList,f);
            pause(0.1);
        end%if displayFlag==1,
        %-----
        outputFileNameList{blockIter}=outputBlockFileName;
        bestFactorList(blockIter) = bestFactor;
        saveSize = (getfield(whos('neunBlock'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(outputBlockFileName,'neunBlock','extendedFactorList','bestFactorList',scs);


    end%if and(fileExistsFlag==1,overwriteFlag==0),
end%for blockIter=1:length(neunBlockFileNameList)
%Now replace any maxed-out subtraction factors with the median of the
%other factors.
maxFactor = max(extendedFactorList);
maxedOutList = find(bestFactorList==maxFactor);
notMaxedOutList = find(bestFactorList~=maxFactor);
%-----
if isempty(notMaxedOutList),
    medianFactor = 0;
    disp('WARNING! ALL subtraction factors maxed out - setting factor to 0');
    disp('WARNING! ALL subtraction factors maxed out - setting factor to 0');
    disp('WARNING! ALL subtraction factors maxed out - setting factor to 0');
    disp('WARNING! ALL subtraction factors maxed out - setting factor to 0');
    disp('WARNING! ALL subtraction factors maxed out - setting factor to 0');

end
%-----
medianFactor = median(bestFactorList(notMaxedOutList));
disp('Replacing maxed out subtraction factor with a median factor');
disp(['Median Factor : ',num2str(medianFactor)]);
for replaceIter = 1:length(maxedOutList);
    blockIter = maxedOutList(replaceIter);
    disp(['Re-running block ',num2str(blockIter),' : ',datestr(now,'mm:dd:yy HH:MM:SS')]);
    currentNeunBlockFileName = stage1OutputFileNameList{blockIter};
    currentCellBlockFileName = cellBlockFileNameList{blockIter};
    load(currentCellBlockFileName,'cellBlock','-mat');
    cellBlock = single(cellBlock);
    load(currentNeunBlockFileName,'neunBlock','-mat');
    neunBlock = single(neunBlock);
    [pathName,fileName,ext,versn] = fileparts(currentNeunBlockFileName);
    newFileName = regexprep(fileName,'neunBlock-temp-subch-','neunBlock-subch-');
    outputBlockFileName = fullfile(neunBlockChannelSubFilesFolder,[newFileName,ext]); 
    subtractedBlock = neunBlock - medianFactor.*cellBlock;
    neunBlock = subtractedBlock;
    outputFileNameList{blockIter}=outputBlockFileName;
    bestFactorList(blockIter) = medianFactor;
    saveSize = (getfield(whos('neunBlock'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(outputBlockFileName,'neunBlock','extendedFactorList','bestFactorList',scs);

end%for replaceIter = 1:length(maxedOutList);
%---



%Restitch the channel-subtracted grayscale blocks into a single volume
if numBlocks == 1,
    neunGrayscaleFileName = outputFileNameList{1};
else%if numBlocks == 1,
    neunGrayscaleFileName = fullfile(processedDataFolder,[fileRootName,'-NeunGrayscale.mat']);
    disp(['Restitching channel subtracted grayscale Blocks']);
    neunGrayscale = zeros(fullStackSize,'uint16');
    overlapOffset = ceil(subVolumeOverlapSize/2);

    minX1 = Inf; minX2 = Inf; minX3 = Inf;
    maxX1 = 0; maxX2 = 0; maxX3 = 0;
    for gridIter = 1:numBlocks;
        blockIterLims=blockGrid{gridIter};
        minX1 = min(minX1,blockIterLims.x1min);
        minX2 = min(minX2,blockIterLims.x2min);
        minX3 = min(minX3,blockIterLims.x3min);
        maxX1 = max(maxX1,blockIterLims.x1max);
        maxX2 = max(maxX2,blockIterLims.x2max);
        maxX3 = max(maxX3,blockIterLims.x3max);
    end
    %---
    for blockIter=1:numBlocks,
        blockIterLims=blockGrid{blockIter};
        load(outputFileNameList{blockIter},'neunBlock','-mat');
        x1a = blockIterLims.x1min;
        x2a = blockIterLims.x2min;
        x3a = blockIterLims.x3min;
        x1b = blockIterLims.x1max;
        x2b = blockIterLims.x2max;
        x3b = blockIterLims.x3max;
        dW1a = 0; dW2a = 0; dW3a = 0;
        dW1b = 0; dW2b = 0; dW3b = 0;
        if x1a > minX1, x1a=x1a + overlapOffset; dW1a = overlapOffset; end
        if x2a > minX2, x2a=x2a + overlapOffset; dW2a = overlapOffset; end
        if x3a > minX3, x3a=x3a + overlapOffset; dW3a = overlapOffset; end
        if x1b < maxX1, x1b=x1b - overlapOffset; dW1b = overlapOffset; end
        if x2b < maxX2, x2b=x2b - overlapOffset; dW2b = overlapOffset; end
        if x3b < maxX3, x3b=x3b - overlapOffset; dW3b = overlapOffset; end
        %---
        x1r=[x1a:x1b];
        x2r=[x2a:x2b];
        x3r=[x3a:x3b];
        neunBlock = neunBlock(1+dW1a:end-dW1b,1+dW2a:end-dW2b,1+dW3a:end-dW3b);
        neunGrayscale(x1r,x2r,x3r)=max(neunGrayscale(x1r,x2r,x3r),uint16(neunBlock));
    end%for blockIter=1:numBlocks,
    saveSize = (getfield(whos('neunGrayscale'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(neunGrayscaleFileName,'neunGrayscale',scs);
end%if-else numBlocks == 1,
%-----
stage2BestFactorList = bestFactorList;
stage2MedianFactor = medianFactor;
stage2MaxedOutList = maxedOutList;
stage2OutputFileNameList = outputFileNameList;

%----------
outExp = inExp;
outExp.globalParameters.neunBlockFileNameList = stage2OutputFileNameList;
outExp.globalParameters.neunGrayscaleFileName = neunGrayscaleFileName;
outExp.neunBlockChannelSubParameters.stage1BestFactorList = stage1BestFactorList;
outExp.neunBlockChannelSubParameters.stage1MaxedOutList = stage1MaxedOutList;
outExp.neunBlockChannelSubParameters.stage1MedianFactor = stage1MedianFactor;
outExp.neunBlockChannelSubParameters.stage2BestFactorList = stage2BestFactorList;
outExp.neunBlockChannelSubParameters.stage2MaxedOutList = stage2MaxedOutList;
outExp.neunBlockChannelSubParameters.stage2MedianFactor = stage2MedianFactor;
outExp.completionDates.neunBlockChannelSubCompletionDate = datestr(now);
%-----
return


