function outExp = VidaVascBlockChannelSub(inExp)
%last modified 05/08/08 by Phil Tsai

%-----
fileRootName = inExp.globalParameters.fileRootName;
rootDataFolder = inExp.globalParameters.rootDataFolder;
lastRerunStartDateNum = inExp.globalParameters.lastRerunStartDateNum;
processedDataFolder = inExp.globalParameters.processedDataFolder;
intermediateResultsFolder = inExp.globalParameters.intermediateResultsFolder;
cellBlockFileNameList = inExp.globalParameters.cellBlockFileNameList;
vascBlockFileNameList = inExp.globalParameters.vascBlockFileNameList;
cellBlockMaskFileNameList = inExp.globalParameters.cellBlockMaskFileNameList;
blockGrid = inExp.globalParameters.blockGrid;
subVolumeOverlapSize = inExp.globalParameters.subVolumeOverlapSize;
fullStackSize = inExp.globalParameters.fullStackSize;
%-----
vascBlockChannelSubParameters = inExp.vascBlockChannelSubParameters;
maxSubFraction = inExp.vascBlockChannelSubParameters.maxSubFraction;
subFractionStepSize = inExp.vascBlockChannelSubParameters.subFractionStepSize;
channelSubFitPolynomialDegree = inExp.vascBlockChannelSubParameters.channelSubFitPolynomialDegree;
channelSubMaskSubSample = inExp.vascBlockChannelSubParameters.channelSubMaskSubSample;
channelSubNonMaskSubSample = inExp.vascBlockChannelSubParameters.channelSubNonMaskSubSample;
%-----
displayFlag = 0;
%-----
channelSubFactorList = [subFractionStepSize:subFractionStepSize:maxSubFraction];
vascBlockChannelSubFilesFolder = fullfile(processedDataFolder,'vascBlockChannelSubFiles');
[mkdirSuccess,mkdirMessage,mkdirId] = mkdir(vascBlockChannelSubFilesFolder);
disp(['Performing Vasc Block Channel Subtraction']);

numBlocks = length(vascBlockFileNameList);
channelSubFactor = zeros(1,length(vascBlockFileNameList));
for blockIter = 1:numBlocks;
    currentCellBlockFileName = cellBlockFileNameList{blockIter};
    currentVascBlockFileName = vascBlockFileNameList{blockIter};
    currentCellBlockMaskFileName = cellBlockMaskFileNameList{blockIter};
    [pathName,fileName,ext,versn] = fileparts(currentVascBlockFileName);
    newFileName = regexprep(fileName,'vascBlock-','vascBlock-subch-');
    outputBlockFileName = fullfile(vascBlockChannelSubFilesFolder,[newFileName,ext]); 
    %-----
    overwriteFlag = 0;
    fileExistsFlag = 0;
    if exist(outputBlockFileName,'file'),
        fileExistsFlag = 1;
        fileProps = dir(outputBlockFileName);
        fileDateNum = datenum(fileProps.date);
        if fileDateNum < lastRerunStartDateNum,
            overwriteFlag = 1;
        end%if fileDateNum < lastRerunStartDateNum,
    end%if exist(outputBlockFileName,'file'),
    %---
    if and(fileExistsFlag==1,overwriteFlag==0),
        disp(['skipping block ',num2str(blockIter),' - previously completed...']); %---added by Phil 03/02/07
        outputFileNameList{blockIter}=outputBlockFileName;
    else%if and(fileExistsFlag==1,overwriteFlag==0),
        disp(['block ',num2str(blockIter),' of ',num2str(length(vascBlockFileNameList)),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);

        % Perform Channel Subtraction of the Cell signal from
        % the vessel data using the previously computed cell masks.
        %-----
        loadedStruct = load(currentCellBlockFileName);
        loadedVariables = fieldnames(loadedStruct);
        cellBlock = loadedStruct.(loadedVariables{1});
        if isstruct(cellBlock),
            tempPixelList = cellBlock.pixelList;
            tempMaskSize = cellBlock.maskSize;
            cellBlock = false(tempMaskSize);
            cellBlock(tempPixelList) = 1;
            clear tempPixelList tempMaskSize
        end%if isstruct(cellBlock),
        %-----
        loadedStruct = load(currentVascBlockFileName);
        loadedVariables = fieldnames(loadedStruct);
        vascBlock = loadedStruct.(loadedVariables{1});
        if isstruct(vascBlock),
            tempPixelList = vascBlock.pixelList;
            tempMaskSize = vascBlock.maskSize;
            vascBlock = false(tempMaskSize);
            vascBlock(tempPixelList) = 1;
            clear tempPixelList tempMaskSize
        end%if isstruct(vascBlock),
        %-----
        load(currentCellBlockMaskFileName,'cellBlockMask','-mat');
        if isstruct(cellBlockMask),
            tempPixelList = cellBlockMask.pixelList;
            tempMaskSize = cellBlockMask.maskSize;
            cellBlockMask = false(tempMaskSize);
            cellBlockMask(tempPixelList) = 1;
            clear tempPixelList tempMaskSize
        end
        %-----
        if max(cellBlockMask(:))>0,
            numFactors = length(channelSubFactorList);
            maskedModeValue = zeros(1,numFactors);
            notMaskedModeValue = zeros(1,numFactors);
            maskPos = find(cellBlockMask(:));
            nonMaskPos = find(~cellBlockMask(:));
            maskPos = maskPos(1:channelSubMaskSubSample:end);
            nonMaskPos = nonMaskPos(1:channelSubNonMaskSubSample:end);
            %-----
            for iterFactor = 1:numFactors;
                 currentFactor = channelSubFactorList(iterFactor);
                 subtractedBlock = double(vascBlock) - double(currentFactor).*double(cellBlock);
                 subtractedMasked = subtractedBlock(maskPos);
                 subtractedNotMasked = subtractedBlock(nonMaskPos);
                 %Calculate mode of masked regions:
                 rawVals = subtractedMasked;
                 medianVal = median(rawVals);
                 maskedMedianValue(iterFactor) = medianVal;
                 %Calculate mode of non-masked regions:
                 rawVals = subtractedNotMasked;
                 medianVal = median(rawVals);
                 notMaskedMedianValue(iterFactor) = medianVal;
            end%for iterFactor = 1:numFactors;
            diffMedianValue = double(maskedMedianValue) - double(notMaskedMedianValue);

            [p,S] = polyfit(channelSubFactorList,diffMedianValue,channelSubFitPolynomialDegree);
            extendedFactorList = [0,channelSubFactorList];
            f = polyval(p,extendedFactorList);
            closestToZero = find(abs(f) == min(abs(f)));
            bestFactor = extendedFactorList(closestToZero(1));
            
            
            
            
            
%noteToSelf
saturatedThreshTopFraction = 0.995;
saturatedIntensityMultiplier = 1.1;
maxCellBlockVal = max(cellBlock(:));
saturatedThresh = maxCellBlockVal * saturatedThreshTopFraction;
saturatedPixels = find(cellBlock>saturatedThresh);
cellBlock(saturatedThresh) = cellBlock(saturatedThresh) * saturatedIntensityMultiplier;
%noteToSelf            
            
            
            
            vascBlock = max(0,vascBlock-bestFactor.*cellBlock);
% % % % %             vascBlock = max(0,vascBlock-0.43 .* cellBlock);%hardcode
% % % % %             keyboard
            
            
            
            
            vascBlock=uint16(vascBlock);
            %-----
            if displayFlag == 1,
                disp(['Channel Subtraction Factor : ',num2str(bestFactor)]);
                figure(1); plot(extendedFactorList,f);
                pause(0.1);
            end%if displayFlag == 1,
            %-----
        else%if max(cellBlockMask(:)) >0
            bestFactor = 0;
        end%if max(cellBlockMask(:)) >0
        saveSize = (getfield(whos('vascBlock'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(outputBlockFileName,'vascBlock',scs);
        outputFileNameList{blockIter}=outputBlockFileName;
        channelSubFactor(blockIter) = bestFactor;
    end%if and(fileExistsFlag==1,overwriteFlag==0),
end%for blockIter=1:numBlocks
vascBlockChannelSubParameters.channelSubFactor = channelSubFactor;




%Restitch the channel-subtracted grayscale blocks into a single volume
if numBlocks == 1,
    vascGrayscaleFileName = outputFileNameList{1};
else%if numBlocks == 1,
    vascGrayscaleFileName = fullfile(processedDataFolder,[fileRootName,'-VascGrayscale.mat']);
    disp(['Restitching channel subtracted grayscale Blocks']);
    vascGrayscale = zeros(fullStackSize,'uint16');
    overlapOffset = ceil(subVolumeOverlapSize/2);

    minX1 = Inf; minX2 = Inf; minX3 = Inf;
    maxX1 = 0; maxX2 = 0; maxX3 = 0;
    for gridIter = 1:numBlocks;
        blockIterLims=blockGrid{gridIter};
        minX1 = min(minX1,blockIterLims.x1min);
        minX2 = min(minX2,blockIterLims.x2min);
        minX3 = min(minX3,blockIterLims.x3min);
        maxX1 = max(maxX1,blockIterLims.x1max);
        maxX2 = max(maxX2,blockIterLims.x2max);
        maxX3 = max(maxX3,blockIterLims.x3max);
    end
    %---
    for blockIter=1:numBlocks,
        blockIterLims=blockGrid{blockIter};
        load(outputFileNameList{blockIter},'vascBlock','-mat');
        x1a = blockIterLims.x1min;
        x2a = blockIterLims.x2min;
        x3a = blockIterLims.x3min;
        x1b = blockIterLims.x1max;
        x2b = blockIterLims.x2max;
        x3b = blockIterLims.x3max;
        dW1a = 0; dW2a = 0; dW3a = 0;
        dW1b = 0; dW2b = 0; dW3b = 0;
        if x1a > minX1, x1a=x1a + overlapOffset; dW1a = overlapOffset; end
        if x2a > minX2, x2a=x2a + overlapOffset; dW2a = overlapOffset; end
        if x3a > minX3, x3a=x3a + overlapOffset; dW3a = overlapOffset; end
        if x1b < maxX1, x1b=x1b - overlapOffset; dW1b = overlapOffset; end
        if x2b < maxX2, x2b=x2b - overlapOffset; dW2b = overlapOffset; end
        if x3b < maxX3, x3b=x3b - overlapOffset; dW3b = overlapOffset; end
        %---
        x1r=[x1a:x1b];
        x2r=[x2a:x2b];
        x3r=[x3a:x3b];
        vascBlock = vascBlock(1+dW1a:end-dW1b,1+dW2a:end-dW2b,1+dW3a:end-dW3b);
        vascGrayscale(x1r,x2r,x3r)=max(vascGrayscale(x1r,x2r,x3r),vascBlock);
    end%for blockIter=1:numBlocks,
    saveSize = (getfield(whos('vascGrayscale'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vascGrayscaleFileName,'vascGrayscale',scs);
end%if-else numBlocks == 1,

%-----
outExp = inExp;
outExp.globalParameters.vascBlockFileNameList = outputFileNameList;
outExp.globalParameters.vascGrayscaleFileName = vascGrayscaleFileName;
outExp.vascBlockChannelSubParameters = vascBlockChannelSubParameters;
outExp.completionDates.vascBlockChannelSubCompletionDate = datestr(now);

%-----
return


