function vectorProject = RenormalizeRadiusHistogramToAwakeVerA(vectorProject);
%%
awakeHistogramFileName = 'F:\Phil\ForFromPeople\2009 - AwakeStacksFromPat\052909\awakeHist.mat';
load(awakeHistogramFileName);
renormalizeRadiusParameters.renormalizeRadiusFunctionVer = 'RenormalizeRadiusHistogramToAwakeVerA.m';
disp('Renormalizing Radii to Awake Histogram');


vs = vectorProject.vectorizedStructure;
effectiveStrandRadiusList = vs.effectiveStrandRadiusList;
numStrands = numel(vs.effectiveStrandRadiusList);
strands = vs.Strands;
vertices = vs.Vertices;
strandZList = vs.strandMidYXZList(:,3);
minZ = min(strandZList);
strandZList = strandZList-minZ;


strandIndex = [1:numStrands];
meanRad = zeros(1,numStrands);
medianRad = zeros(1,numStrands);
stdRad = zeros(1,numStrands);
meanLength = zeros(1,numStrands);
for kk = 1:numStrands,
    thisStrand = strands(kk);
    iv = thisStrand.StartToEndIndices;
    radList = vertices.AllRadii(iv);
    radList = radList(~isnan(radList));
    meanRad(kk) = mean(radList);
    medianRad(kk) = median(radList);
    stdRad(kk) = std(radList);
    meanLength(kk) = numel(iv);
end%for kk= 1:numStrands,


effectiveStrandRadiusList(isnan(effectiveStrandRadiusList)) = medianRad(isnan(effectiveStrandRadiusList));
efr = effectiveStrandRadiusList' + (meanRad/1000) + (stdRad/10000) + (meanLength/100000) + strandIndex/1e11;
ur = unique(efr);

cutoffDiameter = 7;
renormalizeRadiusParameters.cutoffDiameter = cutoffDiameter;

hx = [0:0.5:15];
zSub = [z1,z2,z3,z4,z5];
zSub = zSub(zSub<=cutoffDiameter);
hd = histc(zSub,hx);
figure(1);bar(hx,hd);
xlabel('Awake in vivo diameters (below 7)');
ylabel('Actual count');


hxfine = linspace(0,15,310);
hdfine = imresize(hd,[1,310],'bilinear');
awakeFine = [];
for kk = 1:310,
    awakeFine = [awakeFine,repmat(hxfine(kk),[1,round(10*hdfine(kk))])];
end
figure(2);bar(hxfine,hdfine)
xlabel('Awake in vivo diameters (below 7)');
ylabel('Resampled count');
figure(3);bar(hx,histc(awakeFine,hx));
xlabel('Awake in vivo diameters (below 7)');
ylabel('Resampled and rebinned count');

%%        

cutoffZ = 200;
renormalizeRadiusParameters.cutoffZ = cutoffZ;


subZFlag = and(strandZList<cutoffZ,efr'<=(cutoffDiameter/2));
subD = efr(subZFlag)*2;
numSubStrands = numel(subD);


% % hD = hist(subD,hx);
% % figure(4);bar(hx,hD);
% % xlabel(['Histology diameters (in top ',num2str(cutoffZ),' microns (below7)']);
% % xlabel('Resampled count');

tempD = efr(strandZList<cutoffZ)*2;
figure(5);bar(hx,hist(tempD,hx));
xlabel(['Histology diameters (in top ',num2str(cutoffZ),' microns)']);
ylabel('Resampled count');


awakeFine2 = imresize(awakeFine,[1,numSubStrands],'bilinear');
awakeSort = awakeFine2;
hd2 = hist([awakeSort],hx);
figure(6);bar(hx,hd2);
xlabel('Awake in vivo diameters (below 7)');
ylabel('Resampled and rebinned count');


histSort = sort(subD);
diffSort = awakeSort-histSort;
figure(7);plot(histSort,diffSort,'r.');
xlabel('Histology diameters');
ylabel('Additive correction to match in-vivo histogram');

percentSort = 100*(awakeSort./histSort -1);
figure(8);plot(histSort,percentSort,'b.');
xlabel('Histology diameters');
ylabel('Percent % correction to match in-vivo histogram');



newDiameterList = zeros(1,numStrands);
for strandIter = 1:numStrands,
    startD = efr(strandIter)*2;
    if startD<=cutoffDiameter,
        q = abs(histSort-startD);
        [minVal,minPos] = min(q);
        newDiameterList(strandIter) = awakeSort(minPos);
    else
        newDiameterList(strandIter) = startD;
    end%if-else
end%for strandIter

hDNew = hist(newDiameterList,hx);
figure(9);bar(hx,hDNew);
xlabel('Adjusted Histology diameters');
ylabel('Count');



vectorProject.renormalizeRadiusParameters = renormalizeRadiusParameters;
vectorProject.vectorizedStructure.adjustedStrandRadiusList = newDiameterList'/2;


