function vectorProject = TrackDownFromPlungingArterioleTips(vectorProject)

%From each plunging tip, follow the major downstream flow along the branch
%that has the most current stopping when the branch is not mostly vertical
%(with the caveat that it ignores short sideways strands)

%Hardcodes:
vertFractionRange = [0 1];
radiusRange = [1 3];
lengthRange = [20 50];
cutoffVerticalFraction = 0.5;
minStrandLength = 20;
% forceNotPlungingList = [6442]
forceNotPlungingList = []

vectorProject.flowAnalysisParameters.trackFromArterioleTipParameters.vertFractionRange = vertFractionRange;
vectorProject.flowAnalysisParameters.trackFromArterioleTipParameters.radiusRange = radiusRange;
vectorProject.flowAnalysisParameters.trackFromArterioleTipParameters.lengthRange = lengthRange;
vectorProject.flowAnalysisParameters.trackFromArterioleTipParameters.cutoffVerticalFraction = cutoffVerticalFraction;
vectorProject.flowAnalysisParameters.trackFromArterioleTipParameters.minStrandLength = minStrandLength;
%---

baseStrandIDList = vectorProject.vectorizedStructure.plungingArterioleTipList;
numBases = numel(baseStrandIDList);
numStrands = numel(vectorProject.vectorizedStructure.Strands);
majorPathList = [];

%Calculate the high flow path for each base Strand
for baseIter = 1:numBases;
    keepGoingFlag = 1;
        thisPath = [];
        thisStrandID = baseStrandIDList(baseIter);
        thisStrand = vectorProject.vectorizedStructure.Strands(thisStrandID);

    while keepGoingFlag == 1,
        thisPath = [thisPath,thisStrandID];
        startVoltage = vectorProject.vectorizedStructure.Vertices.AllVoltages(thisStrand.StartVertexIndex);
        endVoltage = vectorProject.vectorizedStructure.Vertices.AllVoltages(thisStrand.EndVertexIndex);
        %---
        if startVoltage > endVoltage,
            lowerVoltage = endVoltage;
            lowerNeighbors = thisStrand.EndVertexNeighborStrands;
        else
            lowerVoltage = startVoltage;
            lowerNeighbors = thisStrand.StartVertexNeighborStrands;
        end
        lowerNeighbors = lowerNeighbors(lowerNeighbors~=thisStrandID);
        
        %exclude surface vessels as potential tracks
        lowerNeighborLabelList = [vectorProject.vectorizedStructure.Strands(lowerNeighbors).MedianLabel];
        surfaceLabelList = [300:399];
        surfaceNeighborFlagList = ismember(lowerNeighborLabelList,surfaceLabelList);
        lowerNeighbors = lowerNeighbors(~surfaceNeighborFlagList);
        
        
        
        lowerNeighborCurrents = abs([vectorProject.vectorizedStructure.Strands(lowerNeighbors).strandCurrent]);
        lowerNeighborCurrentFraction = lowerNeighborCurrents./sum(lowerNeighborCurrents);
        lowerNeighborCurrentFraction(isnan(lowerNeighborCurrentFraction)) = 0;
        numNeighbors = numel(lowerNeighbors);
        myMetric = [];
        zFraction = [];
        if numNeighbors == 0, keepGoingFlag = 0; break; end
        
        for neighborIter = 1:numNeighbors,
            thisStrandID = lowerNeighbors(neighborIter);
            thisStrand = vectorProject.vectorizedStructure.Strands(thisStrandID);
            currentFraction = lowerNeighborCurrentFraction(neighborIter);
            startVoltage = vectorProject.vectorizedStructure.Vertices.AllVoltages(thisStrand.StartVertexIndex);
            endVoltage = vectorProject.vectorizedStructure.Vertices.AllVoltages(thisStrand.EndVertexIndex);
            startZ = vectorProject.vectorizedStructure.Vertices.AllVerts(thisStrand.StartVertexIndex,3);
            endZ = vectorProject.vectorizedStructure.Vertices.AllVerts(thisStrand.EndVertexIndex,3);
            if startVoltage>endVoltage,
                dz = endZ-startZ;
            else
                dz = startZ-endZ;
            end
            dz = sign(dz) * max(dz,1);%avoid zeros
            strandLength = numel(thisStrand.StartToEndIndices);
            zFraction(neighborIter) = dz/strandLength;
            strandRadius = thisStrand.MedianRadius;
            strandRadius = min(max(radiusRange(1),strandRadius),radiusRange(2));
            strandLength = min(max(lengthRange(1),strandRadius),lengthRange(2));
            normRadius = (strandRadius-radiusRange(1))/(radiusRange(2)-radiusRange(1));
            normLength = (strandLength-lengthRange(1))/(lengthRange(2)-lengthRange(1));
            myMetric(neighborIter) = normRadius+normLength+zFraction(neighborIter)+currentFraction;
            
            if ismember(thisStrandID,forceNotPlungingList), myMetric(neighborIter) = 1e-9; end
                
        end

        
        highMetricIndex = find(myMetric == max(myMetric));
        highMetricIndex = highMetricIndex(1);
        highMetricNeighbor = lowerNeighbors(highMetricIndex);
        highZFraction = zFraction(highMetricIndex);
        thisStrandID = highMetricNeighbor;
        thisStrand = vectorProject.vectorizedStructure.Strands(thisStrandID);
  
        if ismember(thisStrandID,thisPath),
            keepGoingFlag = 0;
        end%if ismember(thisStrandID,thisPath),
        
        %if  neighbors doesn't qualify, then stop
        if highZFraction <0 || (highZFraction < cutoffVerticalFraction && strandLength > minStrandLength),    
            keepGoingFlag = 0;
        end%if zFraction...

    end%while keepGoingFlag == 1,
    majorPathList{baseIter} = thisPath;
    
    
% % % %      disp('%----------%');
% % % %     disp(thisPath)
% % % %     disp('%----------%');
% % % %     disp(vectorProject.vectorizedStructure.strandMidYXZList(thisPath,3)');
% % % %     disp('%----------%');
% % % %        pause

       
end%for baseIter
vectorProject.vectorizedStructure.allPlungingArterioleList = [majorPathList{:}];    
vectorProject.vectorizedStructure.allPlungingArteriolePathList = majorPathList;
    


