function vectorProject = TrackDownFromPlungingVenuleTips(vectorProject);

%From each plunging tip, follow the major downstream flow along the branch
%that has the most current stopping when the branch is not mostly vertical
%(with the caveat that it ignores short sideways strands)

%Hardcodes:
vertFractionRange = [0 1];
radiusRange = [1 3];
lengthRange = [20 50];
cutoffVerticalFraction = 0.5;
minStrandLength = 20;

vectorProject.flowAnalysisParameters.trackFromVenuleTipParameters.vertFractionRange = vertFractionRange;
vectorProject.flowAnalysisParameters.trackFromVenuleTipParameters.radiusRange = radiusRange;
vectorProject.flowAnalysisParameters.trackFromVenuleTipParameters.lengthRange = lengthRange;
vectorProject.flowAnalysisParameters.trackFromVenuleTipParameters.cutoffVerticalFraction = cutoffVerticalFraction;
vectorProject.flowAnalysisParameters.trackFromVenuleTipParameters.minStrandLength = minStrandLength;
%---

baseStrandIDList = vectorProject.vectorizedStructure.plungingVenuleTipList;
numBases = numel(baseStrandIDList);
numStrands = numel(vectorProject.vectorizedStructure.Strands);
majorPathList = [];

%Calculate the high flow path for each base Strand
for baseIter = 1:numBases;
    keepGoingFlag = 1;
        thisPath = [];
        thisStrandID = baseStrandIDList(baseIter);
        thisStrand = vectorProject.vectorizedStructure.Strands(thisStrandID);

    while keepGoingFlag == 1,
        thisPath = [thisPath,thisStrandID];
        startVoltage = vectorProject.vectorizedStructure.Vertices.AllVoltages(thisStrand.StartVertexIndex);
        endVoltage = vectorProject.vectorizedStructure.Vertices.AllVoltages(thisStrand.EndVertexIndex);
        %---
        if startVoltage < endVoltage,
            higherVoltage = endVoltage;
            higherNeighbors = thisStrand.EndVertexNeighborStrands;
        else
            higherVoltage = startVoltage;
            higherNeighbors = thisStrand.StartVertexNeighborStrands;
        end
        higherNeighbors = higherNeighbors(higherNeighbors~=thisStrandID);
        
        %exclude surface vessels as potential tracks
        higherNeighborLabelList = [vectorProject.vectorizedStructure.Strands(higherNeighbors).MedianLabel];
        surfaceLabelList = [300:399];
        surfaceNeighborFlagList = ismember(higherNeighborLabelList,surfaceLabelList);
        higherNeighbors = higherNeighbors(~surfaceNeighborFlagList);
        

        
        higherNeighborCurrents = abs([vectorProject.vectorizedStructure.Strands(higherNeighbors).strandCurrent]);
        higherNeighborCurrentFraction = higherNeighborCurrents./sum(higherNeighborCurrents);
        numNeighbors = numel(higherNeighbors);
        myMetric = [];
        zFraction = [];

        
        if numNeighbors == 0, keepGoingFlag = 0; break; end
        
        for neighborIter = 1:numNeighbors,
            thisStrandID = higherNeighbors(neighborIter);
            thisStrand = vectorProject.vectorizedStructure.Strands(thisStrandID);
            currentFraction = higherNeighborCurrentFraction(neighborIter);
            startVoltage = vectorProject.vectorizedStructure.Vertices.AllVoltages(thisStrand.StartVertexIndex);
            endVoltage = vectorProject.vectorizedStructure.Vertices.AllVoltages(thisStrand.EndVertexIndex);
            startZ = vectorProject.vectorizedStructure.Vertices.AllVerts(thisStrand.StartVertexIndex,3);
            endZ = vectorProject.vectorizedStructure.Vertices.AllVerts(thisStrand.EndVertexIndex,3);
            if startVoltage<endVoltage,
                dz = endZ-startZ;
            else
                dz = startZ-endZ;
            end
            strandLength = numel(thisStrand.StartToEndIndices);
            zFraction(neighborIter) = dz/strandLength;
            strandRadius = thisStrand.MedianRadius;
            strandRadius = min(max(radiusRange(1),strandRadius),radiusRange(2));
            strandLength = min(max(lengthRange(1),strandRadius),lengthRange(2));
            normRadius = (strandRadius-radiusRange(1))/(radiusRange(2)-radiusRange(1));
            normLength = (strandLength-lengthRange(1))/(lengthRange(2)-lengthRange(1));
            myMetric(neighborIter) = normRadius+normLength+zFraction(neighborIter)+currentFraction;
            if isnan(myMetric(neighborIter)), myMetric(neighborIter) = 0; end
        end
        
        highMetricIndex = find(myMetric == max(myMetric));
        highMetricIndex = highMetricIndex(1);
        highMetricNeighbor = higherNeighbors(highMetricIndex);
        highZFraction = zFraction(highMetricIndex);
        thisStrandID = highMetricNeighbor;
        thisStrand = vectorProject.vectorizedStructure.Strands(thisStrandID);
       
        
        
        if ismember(thisStrandID,thisPath),
            keepGoingFlag = 0;
        end%if ismember(thisStrandID,thisPath),
        
        if isempty(highZFraction),
            highZFraction = -999;
        end
        
        %if  neighbors doesn't qualify, then stop
        if highZFraction <0 || (highZFraction < cutoffVerticalFraction && strandLength > minStrandLength),    
            keepGoingFlag = 0;
        end%if zFraction...

    end%while keepGoingFlag == 1,
    majorPathList{baseIter} = thisPath;
    
end%for baseIter
vectorProject.vectorizedStructure.allPlungingVenuleList = [majorPathList{:}];    
vectorProject.vectorizedStructure.allPlungingVenulePathList = majorPathList;
    

