function vectorProject = VectorizeLargestSubgraph(varargin);

if nargin>0,
    vectorProject = varargin{1};
else
    disp('Loading vectorProject');
    [fileName,pathName] = uigetfile('*-VectorProject.mat','Select a vectorProject file');
    cd(pathName)
    load(fileName,'vectorProject');
end

if ~iscell(vectorProject.vectorizedStructure.Vertices.StrandIndices),    
    vectorProject = ConvertToCellStorage(vectorProject);
end%if ~iscell(vectorProject.vectorizedStructure.Vertices.StrandIndices), 
    
    
vectorProjectRootName = vectorProject.vectorProjectRootName;
vectorProjectRootName = [vectorProjectRootName,'-LargestSubgraph'];
vectorProject.vectorProjectRootName = vectorProjectRootName;
vectorProjectFileName = [vectorProjectRootName,'-VectorProject.mat'];
vectorizedStructure = vectorProject.vectorizedStructure;
Strands = vectorizedStructure.Strands;
Vertices = vectorizedStructure.Vertices;



%Find the largest connected subgraph
disp('Finding the largest subgraph');
subgraphLabels = ParseSubgraphs(vectorizedStructure);
numSubgraphs = max(subgraphLabels);
h = hist(subgraphLabels,[1:numSubgraphs]);
[majorSubgraphSize,majorSubgraphLabel] = max(h);
majorSubgraphStrandList = find(subgraphLabels == majorSubgraphLabel);
removableStrandList = find(subgraphLabels ~= majorSubgraphLabel);


   
%Convert from VectorProject format to SE2 format    
maskSize = vectorProject.maskSize;
strands = vectorProject.vectorizedStructure.Strands;
allVerts = double(vectorProject.vectorizedStructure.Vertices.AllVerts);
allRadii = double(vectorProject.vectorizedStructure.Vertices.AllRadii);
allLabels = double(vectorProject.vectorizedStructure.Vertices.AllLabels);
allConfidences = double(vectorProject.vectorizedStructure.Vertices.AllConfidences);
% allNotes = vectorProject.vectorizedStructure.Vertices.AllNotes;
% strandIndices = vectorProject.vectorizedStructure.Vertices.StrandIndices;
junctionPointsTemp = vectorProject.vectorizedStructure.Vertices.JunctionPoints;
freeEndPointsTemp = vectorProject.vectorizedStructure.Vertices.FreeEndPoints;
cpMatrix = [allVerts,freeEndPointsTemp,junctionPointsTemp,ones(size(junctionPointsTemp)),allLabels];
% junctionPoints = allVerts(junctionPointsTemp==1,:);
% freeEndPoints = allVerts(freeEndPointsTemp==1,:);

%-----
disp('Removing all but the largest subgraph');
%Inactivate all points not on the largest subgraph
%First remove all the interior points on the removable strands
inactivatedVertices = [];
for strandIter = removableStrandList
    currentStrand = strands(strandIter);
    startVertexIndex = currentStrand.StartVertexIndex;
    endVertexIndex = currentStrand.EndVertexIndex;
    startVertexNeighborStrands = currentStrand.StartVertexNeighborStrands;
    startVertexNeighborStrands = startVertexNeighborStrands(startVertexNeighborStrands~=strandIter);
    endVertexNeighborStrands = currentStrand.EndVertexNeighborStrands;
    endVertexNeighborStrands = endVertexNeighborStrands(endVertexNeighborStrands~=strandIter);
    interiorVertices = currentStrand.StartToEndIndices(2:end-1);
    %if one of the ends is free a free endpoint, remove that point too
    if isempty(startVertexNeighborStrands),
        freeEndPointsTemp(startVertexIndex) = 0;
        interiorVertices = [interiorVertices,startVertexIndex];
    else
        %check if all attached strands are on the removal list
        if min(ismember(startVertexNeighborStrands,removableStrandList)) == 1,
            interiorVertices = [interiorVertices,startVertexIndex];
        end%if min(ismember(startVertexNeighborStrands,removableStrandList)) == 1
    end%if-else isempty(startVertexNeighborStrands),
    %---
    if isempty(endVertexNeighborStrands),
        interiorVertices = [interiorVertices,endVertexIndex];
    else
        %check if all attached strands are on the removal list
        if min(ismember(endVertexNeighborStrands,removableStrandList)) == 1,
            interiorVertices = [interiorVertices,endVertexIndex];
            freeEndPointsTemp(endVertexIndex) = 0;
        end%if min(ismember(endVertexNeighborStrands,removableStrandList)) == 1
    end%if-else isempty(endVertexNeighborStrands),
    cpMatrix(interiorVertices,6) = 0;
    inactivatedVertices = [inactivatedVertices,interiorVertices];
end
%---
disp('Trimming NonSurface strands with free ends');
%If desired, trim all UNLABELED strands with free endpoints
StrandIndices = Vertices.StrandIndices;
numStrandsRemoved = 0;
keepGoingFlag = 1;
while keepGoingFlag,
    numStrandsRemovedThisRound = 0;
    freeEndPointIndexList = find(freeEndPointsTemp');
    for pointIter = freeEndPointIndexList
        currentFreeEndPointIndex = pointIter;
        currentStrandID = StrandIndices{currentFreeEndPointIndex};
        currentStrand = Strands(currentStrandID);
        interiorVertices = currentStrand.StartToEndIndices(2:end-1);
        endcapVertices = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
        attachedEndVertex = endcapVertices(endcapVertices~=currentFreeEndPointIndex);
        majorityLabel = median(allLabels(interiorVertices));
        if ~ismember(majorityLabel,[330:339,340:349]), %do not trim surface arteries or veins
            %inactivate the strand points
            cpMatrix(interiorVertices,6) = 0;
            cpMatrix(currentFreeEndPointIndex,6) = 0;
            inactivedVertices = [inactivatedVertices,interiorVertices,currentFreeEndPointIndex];
            attachedStrands = StrandIndices{attachedEndVertex};
            reducedStrands = attachedStrands(attachedStrands~=currentStrandID);
            StrandIndices{attachedEndVertex} = reducedStrands;
            freeEndPointsTemp(currentFreeEndPointIndex) = 0;
            if numel(reducedStrands) == 1,
                %this junction has been turned into a freeEndPoint
                freeEndPointsTemp(attachedEndVertex) = 1;
            end%if numel(reducedStrands) == 1,
            numStrandsRemovedThisRound = numStrandsRemovedThisRound + 1;
        end%if ismember(majorityLabel,[110,111]), %only trim unlabeled strands
    end%for pointIter = 1:freeEndPointIndexList
    %---
    if numStrandsRemovedThisRound == 0, keepGoingFlag = 0; end
    numStrandsRemoved = numStrandsRemoved + numStrandsRemovedThisRound
end%while keepGoingFlag


clear junctionPointsTemp freeEndPointsTemp
inactivatedLabels = allLabels(inactivatedVertices);


%-----
globalDataStruct.origVectorProject = vectorProject;
globalDataStruct.maskSize = maskSize;
globalDataStruct.fileRootName = vectorProjectRootName;
% globalDataStruct.originalFileName = loadFileName;
% globalDataStruct.loadFileName = loadFileName;
% globalDataStruct.grayscaleFileName = grayscaleFileName;
globalDataStruct.cpMatrix = cpMatrix;
% globalDataStruct.strands = strands;
globalDataStruct.allVerts = allVerts;
globalDataStruct.allRadii = allRadii;
globalDataStruct.allLabels = allLabels;
globalDataStruct.allConfidences = allConfidences;
globalDataStruct.allNotes = [];
% globalDataStruct.junctionPoints = junctionPoints;
% globalDataStruct.freeEndPoints = freeEndPoints;
% globalDataStruct.strandIndices = strandIndices;
clear maskSize strands allVerts allRadii allLabels allConfidences allNotes cpMatrix






%Restrandify
vectorProject = RestrandifySE2(globalDataStruct);
save(vectorProjectFileName,'vectorProject');

   

