function displayInMIJ(varargin)
%Display via Matlab-FIJI API
%
%
%displayInMIJ(IM) where IM is a 2D or 3D matrix to MIJ for dispaly. Warning: duplicates memory
%displayInMIJ('IM') where IM is a string containing the name of a 2D or 3D variable in the workspace
%displayInMIJ(...,[x11 x12 x21 x22 ...]) will crop the dispaly to the values specified in a 1 by 6 vector.
%displayInMIJ(...,[x1 x2 x3], [x1size,x2size,x3size]) will display a ROI of size x1size x x2size x x3size around x1,x2,x3
%
%Pablo
%This code uses the MIJ API developed by Daniel Saje (http://bigwww.epfl.ch/sage/mij/)

%make sure MIJ is running

%Parse input
if nargin<1
    error('Function needs at least one argument.');
end

if ischar(varargin{1})
    mode = 'inbase';
    imName = varargin{1};
else
    mode = 'direct';
end

cropDim = [];
if nargin==2
    if ~isnumeric(varargin{2})
        error('Second argument must be numeric');
    end
    cropDim = varargin{2};
elseif nargin == 3
    if ~(isnumeric(varargin{2}) && isnumeric(varargin{3}))
        error('Second and third argument must be numeric');
    end
    ctr = varargin{2};
    dims =varargin{3};
    dims = round(dims/2);
    cropDim = [ctr(1)-dims(1) ctr(1)+dims(1) ...
        ctr(2)-dims(2) ctr(2)+dims(2) ...
        ctr(3)-dims(3) ctr(3)+dims(3) ];

end

%ensure API is up
try
    MIJ.version;
catch
    Miji;
end



%Validate cropSize

switch mode
    case 'inbase'
        cmd = sprintf('maskSize = size(%s);',imName);
        cmd = sprintf('%s;assignin(''caller'',''maskSize'',maskSize)',cmd);
        evalin('base',cmd)
    case 'direct'
        disp('Not implemented yet');
        return
end

    
    



    %%%%%%%%%%%%%%%%%%%%%%
    
    
    ctr = round(ctr./2)
    ctr(3) = ctr(3)*2;
    cropDim = round(cropDim./2)
    cropDim(5:6) = cropDim(5:6)*2;
%     maskSize = round(maskSize./2)
    
    %%%%%%%%%%%%%%%%%%%%%%
    
    
    
    
    

tmp = cropDim([2 4 6]);
idx = maskSize < tmp;
tmp(idx)= maskSize(idx);
cropDim([2 4 6]) = tmp;


minDim = [1 1 1];
tmp = cropDim([1 3 5]);
idx = minDim  > tmp;
tmp(idx) = minDim(idx);
cropDim([1 3 5]) = tmp;

%permute x and y to get same orientation in Matlab and IJ
switch mode
    case 'inbase'
        if isempty(cropDim)
            cmd = sprintf('MIJ.createImage(%s)',imName);
        else
%             cmd = sprintf('MIJ.createImage(permute(%s(%d:%d,%d:%d,%d:%d),[1 2 3]));',imName,cropDim); %block
                        cmd = sprintf('MIJ.createImage(%s(%d:%d,%d:%d,%d:%d));',imName,cropDim); %block
%           cmd = [cmd sprintf('MIJ.createImage(%s(%d:%d,%d:%d,%d));',imName,[cropDim(1:4) ctr(3)])]; % plane
        end
         disp(cmd);
        evalin('base',cmd);
    case 'direct'
        disp('Not implemented yet');
        return
end



