function output_txt = FV_GiveStrandInfo(obj,event_obj)
% Display the position of the data cursor
% obj          Currently not used (empty)
% event_obj    Handle to event object
% output_txt   Data cursor text string (string or cell array of strings).



pos = get(event_obj,'Position');
output_txt = {['X: ',num2str(pos(1),4)],...
    ['Y: ',num2str(pos(2),4)]};

% If there is a Z-coordinate in the position, display it as well
if length(pos) > 2
    output_txt{end+1} = ['Z: ',num2str(pos(3),4)];
end

global gds
centerOnClickFlag = gds.centerOnClickFlag;

strandData = get(gco,'UserData');
strandID = strandData.strandID;
medianLabel = strandData.medianLabel;
medianRadius = strandData.medianRadius;
medianVoltage = strandData.medianVoltage;
voltageDrop = strandData.voltageDrop;
strandLength = strandData.strandLength;
current = strandData.current;
closestBase = strandData.closestBase;
bloodOx = strandData.meanBloodOx;

myColor = get(gco,'MarkerEdgeColor');
set(gco,'MarkerEdgeColor',double(myColor)*0.9);


mytext = ['StrandID: ',num2str(strandID),' ,  '];
mytext = [mytext, 'label: ',num2str(medianLabel),' ,  '];
mytext = [mytext, 'length: ',num2str(strandLength),' ,  '];
mytext = [mytext, 'Rmed: ',num2str(medianRadius,'%0.3f'),' ,  '];
mytext = [mytext, 'Vmed: ',num2str(medianVoltage,'%0.3f'),' ,  '];
mytext = [mytext, 'dV: ',num2str(voltageDrop,'%0.3f'),' ,  '];
mytext = [mytext, 'I: ',num2str(current,'%0.3e'),' ,  '];
mytext = [mytext, 'closest: ',num2str(round(closestBase))];

mytext = [mytext, '  bloodOx: ',num2str(bloodOx,'%0.3f')];

title(mytext);
set(gcf,'Name',mytext);

if centerOnClickFlag == 1,
    gds.viewCenter = [pos(1),pos(2),-1*pos(3)];
    camtarget([pos(1),pos(2),pos(3)]);
end
gds.selectedStrandData = strandData;    


return