function [B]=ChamferDT(B,rp,allconncandINDs)

% B=ChamferDT(A,rp) returns distance transform of 3-D binary matrix A where background voxels have
% value of -10 and the Chamfer distance transform is based on a single seed rp with [3,4,5] metric.

%Copyright 2005 Mahnaz Maddah
%Minor compatibility editions, 2006 J.Kaufhold

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.

Q=rp;
B=double(B);
infin=100000;
B(find(B))=infin;
B(find(~B))=-10;
for index=1:size(Q,1)
    B(Q(index,1),Q(index,2),Q(index,3))=0;
end
metric=[3,4,5];
BS=[];
while(~isempty(Q))
    for index=1:size(Q,1)
        i=Q(index,1);j=Q(index,2);k=Q(index,3);
        DTp=B(i,j,k);
        %for all neighbors:-------------------------------------------
        for i1=i-1:i+1
            for j1=j-1:j+1
                for k1=k-1:k+1
                    if (i1>=1) & (i1<=size(B,1)) & (j1>=1) & (j1<=size(B,2)) & (k1>=1) & (k1<=size(B,3)) % bounds check
                        if B(i1,j1,k1)>=0
                            prox=sum(abs([i1,j1,k1]-[i,j,k]));%determine the proximity form (6- 18- or 26 neighbors.
                            if prox	%exclude the center point.
                                DTr1=B(i1,j1,k1);
                                DTr=min(DTr1,DTp+metric(prox));
                                if DTr~=DTr1
                                    B(i1,j1,k1)=DTr;
                                    BS=[BS;i1,j1,k1];
                                end
                            end
                        end
                    end
                end
            end
        end
        %-------------------------------------------------------------
    end
    Q=BS;
    BS=[];
    if any(and(B(allconncandINDs)~=infin , B(allconncandINDs)~=-10));
        Q = [];
    end%if any(B(allconncandINDs)~=infin),
end
B(find(B==infin))=-10;
