function [ptconn2,pt2,BacktrackRejectMask] = ConnectedCenterlinePointThreshRelax(E,B,EPStrandMask,EPMask,pt1,candpts,Thresholds,x,y,z,minvvol,morphfiltsz)

%Copyright 2008 J.Kaufhold

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.




islandsbyblock = 0;
dispon=0;
% compute mask to prevent backtracking
[DEPM]=bwdist(EPMask);
[DEPSM]=bwdist(EPStrandMask);
BacktrackRejectMask=DEPM<DEPSM;
clear DEPM DEPSM


EPMask = imdilate(EPMask,true(3,3,3));


% get point locations in local coordinate systems
lpt1=[pt1(1)-min(y(:))+1,pt1(2)-min(x(:))+1,pt1(3)-min(z(:))+1];
cpts=[candpts(:,1)-min(y(:))+1,candpts(:,2)-min(x(:))+1,candpts(:,3)-min(z(:))+1];

BBcandpts=(candpts(:,1)>min(y(:))+1) & (candpts(:,1)<max(y(:))) & (candpts(:,2)>min(x(:))+1) & (candpts(:,2)<max(x(:))) & (candpts(:,3)>min(z(:))+1) & (candpts(:,3)<max(z(:)));
% compute candidate point locations to connect to pt1
lcandxyz=round(cpts(BBcandpts,:));
lcandINDs=sub2ind(size(B),lcandxyz(:,1),lcandxyz(:,2),lcandxyz(:,3));
    
% Get the indices of noncandidate points (those rejected by BacktrackRejectMask and not in the endpoint mask
nonbtcandINDs = BacktrackRejectMask(lcandINDs) & ~EPMask(lcandINDs);

if islandsbyblock
    % find out which binary regions have no centerline points and do not border BB edge (these are islands)
    Lislands=bwlabeln(B,6);
    IslandCandidates=setdiff(setdiff(unique(Lislands(:)),unique(Lislands(lcandINDs))),0);
    iLUT=zeros(1,max(Lislands(:)+1));
    iLUT(IslandCandidates+1)=1;
    % reorder labels to islands only
    iLUT=cumsum(iLUT).*iLUT;
    Lislands=reshape(iLUT(Lislands+1),size(Lislands));
    % also ensure they're not rejected by the backtracking mask
    NonBTIslandCandidates = setdiff(unique(Lislands(BacktrackRejectMask)),0);
    iLUT=zeros(1,max(Lislands(:)+1));
    iLUT(NonBTIslandCandidates+1)=1;
    % reorder labels to islands only
    iLUT=cumsum(iLUT).*iLUT;
    Lislands=reshape(iLUT(Lislands+1),size(Lislands));
    % Mask out islands to keep (nonbtislands) and then compute centroids and global coordinate system locations
    s=regionprops(Lislands,'centroid');
    % each row is a centroid in the local coordinate system
    IslandCents=reshape([s(:).Centroid],3,length(s))';
    Islandpts=[IslandCents(:,2)+min(y(:))-1,IslandCents(:,1)+min(x(:))-1,IslandCents(:,3)+min(z(:))-1];
end

if dispon
    figure;clf;hold on;
    patch(isosurface(x,y,z,B, .5), 'FaceColor', 'blue', 'EdgeColor', 'none','facealpha',.1);
    plot3(candpts(BBcandpts,2),candpts(BBcandpts,1),candpts(BBcandpts,3),'r.');
    plot3(pt1(2),pt1(1),pt1(3),'ko');
    if islandsbyblock
        patch(isosurface(x,y,z,Lislands, .5), 'FaceColor', 'green', 'EdgeColor', 'none','facealpha',.1);
        plot3(Islandpts(:,2),Islandpts(:,1),Islandpts(:,3),'g*','markersize',15);
    end
    axis([min(x(:)),max(x(:)),min(y(:)),max(y(:)),min(z(:)),max(z(:))]);
    title('connected mask (black)');
end

allcandsxyz=round(candpts(BBcandpts,:));
gcsconncands=allcandsxyz(nonbtcandINDs,:);
gcsexcludecands=allcandsxyz(~nonbtcandINDs,:);

if islandsbyblock
    % get island centerline points indexed into set of candidate points
    islconncandINDs=sub2ind(size(B),round(IslandCents(:,2)),round(IslandCents(:,1)),round(IslandCents(:,3)));
    Lisltmp = false(size(B));
    Lisltmp(islconncandINDs)=true;
end

if dispon
    figure;clf;hold on;
    patch(isosurface(x,y,z,EPStrandMask, .5), 'FaceColor', 'green', 'EdgeColor', 'none','facealpha',.1);
    patch(isosurface(x,y,z,EPMask, .5), 'FaceColor', 'red', 'EdgeColor', 'none','facealpha',.1);
    patch(isosurface(x,y,z,BacktrackRejectMask, .5), 'FaceColor', 'yellow', 'EdgeColor', 'none','facealpha',.1);
    plot3(pt1(2),pt1(1),pt1(3),'ro','markersize',15);
    plot3(candpts(BBcandpts,2),candpts(BBcandpts,1),candpts(BBcandpts,3),'r.');
    plot3(allcandsxyz(:,2),allcandsxyz(:,1),allcandsxyz(:,3),'r.');
    plot3(gcsconncands(:,2),gcsconncands(:,1),gcsconncands(:,3),'r*');
    plot3(gcsexcludecands(:,2),gcsexcludecands(:,1),gcsexcludecands(:,3),'ko');
    if islandsbyblock
        patch(isosurface(x,y,z,Lisltmp, .5), 'FaceColor', 'magenta', 'EdgeColor', 'none','facealpha',.1);
        plot3(Islandpts(:,2),Islandpts(:,1),Islandpts(:,3),'g*','markersize',15);
        plot3(Islandpts(:,2),Islandpts(:,1),Islandpts(:,3),'r.','markersize',15);
    end
    axis([min(x(:)),max(x(:)),min(y(:)),max(y(:)),min(z(:)),max(z(:))]);
    title('connected mask (black)');
end

%% Get mask that connects both endpoints by stepwise reducing threshold
% initialize gap as unconnected


connected=false;
% initialize threshold relaxation termination criterion as fals
finishedrelaxing=false;
% initialize threshold at highest value
TIND=length(Thresholds.Enhanced);


if islandsbyblock
    allconncandINDs=[lcandINDs(nonbtcandINDs);islconncandINDs];
    allgcsconncands=[gcsconncands;round(Islandpts)];
    % newislandflag = [zeros(size(gcsconncands,1),1);ones(size(Islandpts,1),1)];
else
    allconncandINDs=lcandINDs(nonbtcandINDs);
    allgcsconncands=gcsconncands;
end

close all;
while ~connected && ~finishedrelaxing
    % Start at highest threshold and relax until lpt1 and lpt2 are connected
    L1=ThresholdEnhancedVesselMasksPrime(E,Thresholds.Enhanced(TIND),minvvol,morphfiltsz);

    L1=bwlabeln(L1 & BacktrackRejectMask,6);
    % Get binary component label of endpoint that's searching for connection...
    Lept=L1(lpt1(1),lpt1(2),lpt1(3));
    if Lept ~= 0    % if the label of the endpoint to connect is in the mask, isolate that connected component
        Lsearch = (L1==Lept);  % Where Lsearch is 1, a candidate endpoint can be found
    else            % otherwise
        Lsearch = zeros(size(L1));
    end

    % if any points are in Lsearch they are connected at that threshold
    if any(Lsearch(allconncandINDs))
        connected=true; % if the 2 points share the same contiguous label in L0, mark them connected
    end

    if TIND>=2
        TIND=TIND-1; % lower threshold
    else
        disp('ConnectedCenterlinePointThreshRelax: reached last threshold without connecting');
        finishedrelaxing = true;
    end

    if dispon
        figure;clf;hold on;
        patch(isosurface(x,y,z,B, .5), 'FaceColor', 'blue', 'EdgeColor', 'none','facealpha',.1);
        patch(isosurface(x,y,z,Lsearch, .5), 'FaceColor', 'black', 'EdgeColor', 'none','facealpha',.1);
        patch(isosurface(x,y,z,EPStrandMask, .5), 'FaceColor', 'green', 'EdgeColor', 'none','facealpha',.1);
        patch(isosurface(x,y,z,EPMask, .5), 'FaceColor', 'red', 'EdgeColor', 'none','facealpha',.1);
        patch(isosurface(x,y,z,BacktrackRejectMask, .5), 'FaceColor', 'yellow', 'EdgeColor', 'none','facealpha',.1);
        plot3(gcsconncands(:,2),gcsconncands(:,1),gcsconncands(:,3),'r*');
        plot3(gcsexcludecands(:,2),gcsexcludecands(:,1),gcsexcludecands(:,3),'ko');
        if islandsbyblock
            patch(isosurface(x,y,z,Lisltmp, .5), 'FaceColor', 'magenta', 'EdgeColor', 'none','facealpha',.1);
            plot3(Islandpts(:,2),Islandpts(:,1),Islandpts(:,3),'g*','markersize',15);
            plot3(Islandpts(:,2),Islandpts(:,1),Islandpts(:,3),'r.','markersize',15);
        end
        axis([min(x(:)),max(x(:)),min(y(:)),max(y(:)),min(z(:)),max(z(:))]);
        view(-14,12);
        axis equal;
        % title('connected mask (black)');
    end
end


if connected  % IF THERE IS AT LEAST ONE CANDIDATE POINT TO CONNECT

% % % % % % %     %% FIND THE CLOSEST CANDIDATE ENDPOINT TO CONNECT TO VIA PATHLENGTH DISTANCE
% % % % % % %     % Lsearch is a binary mask connecting endpoint of interest to candidate
% % % % % % %     % endpoints...
% % % % % % %     [PathDist]=ChamferDT(Lsearch,[lpt1(1),lpt1(2),lpt1(3)]);
% % % % % % %     candptPathDists=PathDist(allconncandINDs);
% % % % % % % 

    [PathDist] = ChamferDTPrime(Lsearch,[lpt1(1),lpt1(2),lpt1(3)],allconncandINDs);
    candptPathDists=PathDist(allconncandINDs);

    
    % ChamferDT maps out of mask points to -10... This is a problem for min, so assign them largest candidate distance
    candptPathDists(candptPathDists==-10)=max(candptPathDists)+1;
    [minPathDist,closestconnptind]=min(candptPathDists);
    pt2=allgcsconncands(closestconnptind,:);
    % get point closest to pt2 in list of candpts
    [mindist,ptconn2]=min(sqrt(sum([candpts(:,1)-pt2(1),candpts(:,2)-pt2(2),candpts(:,3)-pt2(3)].^2,2)));

    
    
    
    
    
    if dispon
        figure;clf;hold on;
        patch(isosurface(x,y,z,B, .5), 'FaceColor', 'blue', 'EdgeColor', 'none','facealpha',.1);
        patch(isosurface(x,y,z,EPStrandMask, .5), 'FaceColor', 'green', 'EdgeColor', 'none','facealpha',.1);
        patch(isosurface(x,y,z,EPMask, .5), 'FaceColor', 'red', 'EdgeColor', 'none','facealpha',.1);
        patch(isosurface(x,y,z,BacktrackRejectMask, .5), 'FaceColor', 'yellow', 'EdgeColor', 'none','facealpha',.1);
        patch(isosurface(x,y,z,Lsearch, .5), 'FaceColor', 'black', 'EdgeColor', 'none','facealpha',.1);
        plot3(allcandsxyz(:,2),allcandsxyz(:,1),allcandsxyz(:,3),'r.');
        plot3(gcsconncands(:,2),gcsconncands(:,1),gcsconncands(:,3),'r*');
        plot3(pt1(2),pt1(1),pt1(3),'ro','markersize',15);
        plot3(pt2(2),pt2(1),pt2(3),'mo','markersize',15);
        axis([min(x(:)),max(x(:)),min(y(:)),max(y(:)),min(z(:)),max(z(:))]);
        grid on;
        title('connected mask (black)');
    end
else                                               % IF NO CANDIDATE POINTS TO CONNECT TO
    ptconn2 = [];       % set the connection point index to empty
    pt2 = [];           % set the connection point xyz coordinate to empty
end
