function Islands = GetIslandsFromBlock(B,candpts,BBxyz)

%Copyright 2008 J.Kaufhold

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.

dispon=0;
RemoveIslandsOnBlockPerimeter=1;

vsz = size(B);

inBB = (candpts(:,2)>=BBxyz.x1min) & (candpts(:,2)<=BBxyz.x1max) & (candpts(:,1)>=BBxyz.x2min) & (candpts(:,1)<=BBxyz.x2max) & (candpts(:,3)>=BBxyz.x3min) & (candpts(:,3)<=BBxyz.x3max);
localcpts = round(candpts(inBB,:));
localcpts = [localcpts(:,1)-BBxyz.x2min+1,localcpts(:,2)-BBxyz.x1min+1,localcpts(:,3)-BBxyz.x3min+1];
% if dispon
%     % check that coordinate systems of points from vectorization (in red) match with mask
%     figure;
%     imagesc(log(sum(B,3)+1));
%     hold on;
%     plot(localcpts(:,1),localcpts(:,2),'r.');
% end
lcandINDs = sub2ind(vsz,localcpts(:,2),localcpts(:,1),localcpts(:,3));

% find out which binary regions have no centerline points and
Lislands=bwlabeln(B,6);
IslandCandidates=setdiff(setdiff(unique(Lislands(:)),unique(Lislands(lcandINDs))),0);
if RemoveIslandsOnBlockPerimeter
    % AND do not border BB edge (these are islands)
    for iISLCAND = 1:numel(IslandCandidates)
        lmap = single(Lislands==IslandCandidates(iISLCAND));
        % Find which bounding boxes touch borders of block
        BBtmp = regionprops(lmap,'PixelIdxList');
        [bb2,bb1,bb3]=ind2sub([vsz(2),vsz(1),vsz(3)],BBtmp.PixelIdxList);
        if (min(bb1)==1) || (max(bb1)==vsz(1)) || (min(bb2)==1) || (max(bb2)==vsz(2)) || (min(bb3)==1) || (max(bb3)==vsz(3))
            IslandCandidates(iISLCAND) = 0;
        end
    end
    IslandCandidates = IslandCandidates(IslandCandidates>0);
end

iLUT=zeros(1,max(Lislands(:)+1));
iLUT(IslandCandidates+1)=1;
% reorder labels to islands only
iLUT=cumsum(iLUT).*iLUT;
Lislands=reshape(iLUT(Lislands+1),vsz);
% Compute centroids and lcs locations
s=regionprops(Lislands,'centroid');
% each row is a centroid in the local coordinate system
IslandCents=round(reshape([s(:).Centroid],3,length(s))');

% Create binary mask of island centroids to return
Islands = zeros(vsz,'uint8');
islINDs = sub2ind(vsz,IslandCents(:,2),IslandCents(:,1),IslandCents(:,3));
Islands(islINDs) = 1;

if dispon
    figure(1);clf;hold on;
    %patch(isosurface(B, .5), 'FaceColor', 'blue', 'EdgeColor', 'none','facealpha',.1);
    patch(isosurface(Lislands, .5), 'FaceColor', 'green', 'EdgeColor', 'none','facealpha',.1);
    patch(isosurface(Islands, .5), 'FaceColor', 'black', 'EdgeColor', 'none','facealpha',.1);
    plot3(localcpts(:,1),localcpts(:,2),localcpts(:,3),'r.');
    plot3(IslandCents(:,1),IslandCents(:,2),IslandCents(:,3),'kx');
    axis equal;axis tight;grid on;
    title('connected mask (black)');
end
