function [LocalDist,Thresholds,FineThresholds] = GetLocalIntensityDistribution2(Ve,Vr,B)

% Ve is enhanced volume
% Vr is raw volume
% B is binary mask

%Copyright 2008 J.Kaufhold

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.

dispon=0;

uselog = true;

if uselog
    De=log(Ve+1);                            % Compute log-domain statistics of the enhanced volume
else
    De=Ve;                                   % Compute linear domain statistics of the enhanced volume
end

keyboard


Ierange=linspace(min(De(:)),max(De(:)),100); % Get an evenly spaced range of intensities in the log domain
veI=De(B);                                   % Get the intensities inside the vessel masks
bgeI=De(~B);                                 % Get the intensities outside the vessel masks
vedist=histc(veI,Ierange)/length(veI);       % Get the histogram of vessel intensities
bgedist=histc(bgeI,Ierange)/length(bgeI);    % Get the histogram of background intensities



% In current implementation the raw mask and its statistics are ignored
if uselog
    Dr=log(Vr-min(Vr(:))+1);                 % Compute log-domain statistics of the raw volume
else
    Dr=Vr-min(Vr(:));                        % Compute linear domain statistics of the raw volume
end
Irrange=linspace(min(Dr(:)),max(Dr(:)),100); % Get an evenly spaced range of intensities in the log domain
vrI=Dr(B);                                   % Get the intensities inside the vessel masks
bgrI=Dr(~B);                                 % Get the intensities outside the vessel masks
vrdist=histc(vrI,Irrange)/length(vrI);       % Get the histogram of vessel intensities
bgrdist=histc(bgrI,Irrange)/length(bgrI);    % Get the histogram of background intensities

if dispon
    figure;
    plot(Ierange,bgedist);
    hold on;
    plot(Ierange,vedist,'r');
    
    legend('Enhanced BG','Enhanced Vessel');
end

% compute mean and standard deviations of vessels and background
% for enhanced volume
EnhancedStats.logBG.meanfit=mean(bgeI(:));
EnhancedStats.logBG.stdfit=std(bgeI(:));
EnhancedStats.logBG.pdf=bgedist;
EnhancedStats.logVessels.meanfit=mean(veI(:));
EnhancedStats.logVessels.stdfit=std(veI(:));
EnhancedStats.logVessels.pdf=vedist;
% for raw volume
RawStats.logBG.meanfit=mean(bgrI(:));
RawStats.logBG.stdfit=std(bgrI(:));
RawStats.logBG.pdf=bgrdist;
RawStats.logVessels.meanfit=mean(vrI(:));
RawStats.logVessels.stdfit=std(vrI(:));
RawStats.logVessels.pdf=vrdist;
% Store in local distributions
LocalDist.EnhancedStats=EnhancedStats;
LocalDist.RawStats=EnhancedStats;

% compute mutual z-score of ML separator, assuming gaussian mixture with equal priors
zMLEnhanced=(EnhancedStats.logVessels.meanfit-EnhancedStats.logBG.meanfit)/(EnhancedStats.logVessels.stdfit+EnhancedStats.logBG.stdfit);
zMLRaw=(RawStats.logVessels.meanfit-RawStats.logBG.meanfit)/(RawStats.logVessels.stdfit+RawStats.logBG.stdfit);

% compute Threshold list from muBG-3sigma (very loose) to muBG+6sigma (very tight) using a coarse range
zrange=-3:6;

if uselog
    Thresholds.Enhanced=exp(EnhancedStats.logBG.meanfit+zrange*EnhancedStats.logBG.stdfit)-1;
    Thresholds.Raw=exp(RawStats.logBG.meanfit+zrange*RawStats.logBG.stdfit)+min(Vr(:))-1;
    Thresholds.Raw(1) = 0;
    Thresholds.Enhanced(1) = 0;
else
    Thresholds.Enhanced = EnhancedStats.logBG.meanfit + zrange*EnhancedStats.logBG.stdfit;
    Thresholds.Raw = RawStats.logBG.meanfit + zrange*RawStats.logBG.stdfit + min(Vr(:));
    Thresholds.Raw(1) = 0;
    Thresholds.Enhanced(1) = 0;
end
Thresholds.zrange=zrange;
Thresholds.zMLEnhanced=zMLEnhanced;
Thresholds.zMLRaw=zMLRaw;

% compute Threshold list from muBG-3sigma (very loose) to muBG+6sigma (very tight) using 5x finer spacing
zrange=linspace(-3,6,30);
if uselog
    FineThresholds.Enhanced=exp(EnhancedStats.logBG.meanfit+zrange*EnhancedStats.logBG.stdfit)-1;
    FineThresholds.Raw=exp(RawStats.logBG.meanfit+zrange*RawStats.logBG.stdfit)+min(Vr(:))-1;
    FineThresholds.Enhanced(1) = 0;
    FineThresholds.Raw(1) = 0;
else
    FineThresholds.Enhanced = EnhancedStats.logBG.meanfit + zrange*EnhancedStats.logBG.stdfit;
    FineThresholds.Raw = RawStats.logBG.meanfit + zrange*RawStats.logBG.stdfit + min(Vr(:));
    FineThresholds.Enhanced(1) = 0;
    FineThresholds.Raw(1) = 0;
end
FineThresholds.zrange=zrange;
FineThresholds.zMLEnhanced=zMLEnhanced;
FineThresholds.zMLRaw=zMLRaw;
