function [EndpointStrandMask,EndpointMask] = GetSingleEndpointStrandMask2(B,x,y,z,s1yxz,ep1yxz)

% B     : the binary mask of vessels
% x,y,z : where the binary mask, B, is located in absolute coordinates
% s1xyz : the locations of centerline points on strand 1
% ep1xyz: the location of the endpoint on strand 1

% EndpointMask       : a binary mask of 1s where B=1 and closer to the endpoint than other points on the strand
% EndpointStrandMask : a binary mask of 1s where B=1 and closer to other points on the strand than the endpoint

%Copyright 2008 J.Kaufhold

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.

dispon=0;
Bsz = size(B);


% Round the strand's centerline points to points on the voxel lattice
s1subs=round(s1yxz);
% Translate global coordinate system indexing to local coordinate system
s1suboffs=[s1subs(:,1)-min(y(:))+1,s1subs(:,2)-min(x(:))+1,s1subs(:,3)-min(z(:))+1];
% Round the endpoint to a point on the voxel lattice
ep1subs=round(ep1yxz);
% Translate global coordinate system indexing to local coordinate system
ep1suboffs=[ep1subs(:,1)-min(y(:))+1,ep1subs(:,2)-min(x(:))+1,ep1subs(:,3)-min(z(:))+1];

% Create a binary mask with a 1 at the endpoint location
EndpointMask = zeros(Bsz,'uint8');
EndpointMask(ep1suboffs(1,1),ep1suboffs(1,2),ep1suboffs(1,3)) = 1;

% Create a binary mask of 1s at all the strand centerlines except the endpoint, itself
EndpointStrandMask = zeros(Bsz,'uint8');
for iSTRAND = 1:size(s1suboffs,1)
    if ( s1suboffs(iSTRAND,1) >= 1 ) && ( s1suboffs(iSTRAND,1) <= Bsz(1) ) && ( s1suboffs(iSTRAND,2) >= 1 ) && ( s1suboffs(iSTRAND,2) <= Bsz(2) ) && ( s1suboffs(iSTRAND,3) >= 1 ) && ( s1suboffs(iSTRAND,3) <= Bsz(3) )
        EndpointStrandMask(s1suboffs(iSTRAND,1),s1suboffs(iSTRAND,2),s1suboffs(iSTRAND,3)) = 1;
    end
end
EndpointStrandMask(ep1suboffs(1,1),ep1suboffs(1,2),ep1suboffs(1,3)) = 0;


if dispon
    figure(1);clf;hold on;
    patch(isosurface(x,y,z,B, .5), 'FaceColor', 'blue', 'EdgeColor', 'none','facealpha',.1);
    patch(isosurface(x,y,z,EndpointStrandMask, .5), 'FaceColor', 'green', 'EdgeColor', 'none','facealpha',.4);
    patch(isosurface(x,y,z,EndpointMask, .5), 'FaceColor', 'red', 'EdgeColor', 'none','facealpha',.4);
    plot3(s1subs(:,2),s1subs(:,1),s1subs(:,3),'k.-','linewidth',2,'markersize',15);
    plot3(ep1subs(:,2),ep1subs(:,1),ep1subs(:,3),'ko','linewidth',2,'markersize',15);
end
