function [strandSet LUT]= amira2mat_parseFile(fileName)

%Pablo Blinder - 30Sept2007

% This file is part of ThresholdRelaxationAlgorithmV0.1
% 
%     ThresholdRelaxationAlgorithmV0.1 is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ThresholdRelaxationAlgorithmV0.1 is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ThresholdRelaxationAlgorithmV0.1.  If not, see <http://www.gnu.org/licenses/>.

%% load data in file
fprintf('*** Loading file ...')
t0 = clock;
% dataInFile = textread(fileName,'','commentstyle','shell');
fid = fopen(fileName,'r');
dataInFile = textscan(fid,'%f\t%f\t%f\t%f\t%f','commentstyle','#');
dataInFile = cell2mat(dataInFile);
fclose (fid);
fprintf(' loaded in %f sec \n',etime(clock,t0))
%Amira's first index is zero => make it 1
dataInFile(:,1) = dataInFile(:,1) + 1;

%% Parse
%to avoid performing multiple find commands, tag with 1 the first element of each line

t0 = clock;
tic;
idx2StrandsInFile = [0 ;  find(diff(dataInFile(:,1))) ; size(dataInFile,1)];
nStrands = numel(idx2StrandsInFile) - 1;
fprintf('*** Parsing  ');
strandSetInfo = struct('lineId',[],'ptr2coord',[]);
strandSetInfo(nStrands).lineId = nStrands;
nc = 1;
incrementSize = 10000;
LUT = zeros(incrementSize,2);
startCounter = 0;
stopCounter = 0;
for ki = 1 : nStrands
    strandSetInfo(ki).lineId = ki;
    strandSetInfo(ki).ptr2coord = (idx2StrandsInFile(ki) + 1 : idx2StrandsInFile(ki + 1))';
    numPoints = numel(strandSetInfo(ki).ptr2coord);
    stopCounter = stopCounter + numPoints;
    if stopCounter > size(LUT,1),
        LUT = [LUT;zeros(incrementSize,2)];
    end%if counter > size(LUT,1),
    LUT(startCounter+1:stopCounter,:) = [strandSetInfo(ki).ptr2coord repmat(ki,numPoints,1)];
    startCounter = startCounter + numPoints;
end % cycling line indices
LUT = LUT(1:stopCounter,:);

% Build strandSet structure
strandSet = struct('strandData',dataInFile,'strandSetInfo',...
    strandSetInfo,'ptr2endpoints',idx2StrandsInFile,'ctrPoint2StrandLUT',LUT);

fprintf(' %d strands took %f sec \n',nStrands,etime(clock,t0))