function outputSubGraphYXZ=DisconnectSubgraph(inputSubGraphYXZ,minDist)
%modified by Phil Tsai on 09/21/07

%inputSubGraphYXZ is an 3xN array of  [y,x,z] coordinates for the N points in the connected subgraph
%minDist is the minimum connectivity distance, below which merging/elimination should be performed
%CAG stands for Cell Adjacency Graph


% % % try
    %Generate a Euclidean distance matrix between all subgraph points
    euclidDistMatrix=DistMatrix(inputSubGraphYXZ);
    %Find the entries in the subgraph that are further apart than minDist
    candidateCAG=tril(euclidDistMatrix>minDist);
    if sum(candidateCAG(:))==0
        % if there are no candidates, just average locations
        % this occurs if every point is within the minimum distance of
        % every other point in the subgraph
        outputSubGraphYXZ=round(mean(inputSubGraphYXZ,1));
        return
    end%if sum(candidateCAG(:))==0
    [psi,psj]=find(candidateCAG); %indices of points that are >minDist from at least one point in the subgraph
    disallowedpairs=~candidateCAG.*(~candidateCAG'); %logical adjacency matrix of points <minDist from EVERY other point
    numKeeperCandidates=length(psi); %number of potential "keeper" nodes (a node = a pairs of points)
    for iNN=numKeeperCandidates:-1:1
        %generate all possible combinations of keeping iNN of numKeeperCandidates (e.g., keeping 4 of 6)
        %   starting with keeping the most nodes (pairs)
        testNodeCombos=nchoosek(1:numKeeperCandidates,iNN);
        for iTEST=1:size(testNodeCombos,1)%try each combination of nodes (kept pairs), one by one
            clear nodesKept
            nodesKept=[psi(testNodeCombos(iTEST,:)),psj(testNodeCombos(iTEST,:))];
            pointsKept=unique(nodesKept(:)); %if a kept node includes a point put that point on the kept points list
            listlinks=nchoosek(pointsKept,2); %make a list of all possible pairs of kept points
            %now see if any of the disallowed pairs are included in the list of possible pairs kept
            ijl=sub2ind(size(disallowedpairs),listlinks(:,1),listlinks(:,2));
            if ~any(disallowedpairs(ijl))
                keepcentroidlist=pointsKept;
                break %break out of the for iTEST = 1:... loop
            end%if ~any(disallowedpairs(ijl))
        end%for iTEST=1:size(testNodeCombos,1)
        if ~any(disallowedpairs(ijl))
            break %break out of the for iNN=numKeeperCandidates... loop
        end%if ~any(disallowedpairs(ijl))
    end%for iNN=numKeeperCandidates:-1:1
    outputSubGraphYXZ=inputSubGraphYXZ(keepcentroidlist,:);

% % % catch %only happens if some error occurs in the above code
% % %     try
% % %         disp('error disconnecting subgraph with nodes too close');
% % %         disp('reverting to greedy centroid elimination');
% % %         CAG=(euclidDistMatrix<minDist)-eye(size(euclidDistMatrix));
% % %         while sum(CAG(:))>0
% % %             removefrominputSubGraphYXZ=find(max(sum(CAG))==sum(CAG));
% % %             inputSubGraphYXZ(removefrominputSubGraphYXZ(1),:)=[];
% % %             euclidDistMatrix=DistMatrix(inputSubGraphYXZ);
% % %             CAG=(euclidDistMatrix<minDist)-eye(size(euclidDistMatrix));
% % %         end
% % %         outputSubGraphYXZ=inputSubGraphYXZ;
% % %     catch
% % %         disp('error disconnecting subgraph and error in greedy elimination');
% % %         keyboard
% % %     end
% % % end
