function outputSubGraphYXZ=DisconnectSubgraph(inputSubGraphYXZ,minDist)
%modified by Phil Tsai on 09/21/07

%inputSubGraphYXZ is an 3xN array of  [y,x,z] coordinates for the N points in the connected subgraph
%minDist is the minimum connectivity distance, below which merging/elimination should be performed
%CAG stands for Cell Adjacency Graph

mergeDist2 = minDist.^2;
currentGraph = inputSubGraphYXZ;


%Remove centroids one at a time until all point are disconnected
%Rank remaining subgraphs based on number of too-near neighbors
numPoints = size(currentGraph,1);
numTooCloseNeighbors = zeros(1,numPoints);
for pointIter = 1:numPoints,
    offsetList = currentGraph - repmat(currentGraph(pointIter,:),numPoints,1);
    tempD2=sum(offsetList.^2,2);
    tooCloseNeighbors=tempD2<=mergeDist2;
    numTooCloseNeighbors(pointIter) = sum(tooCloseNeighbors)-1;
end

keepGoing = 1;
while keepGoing == 1,
    
    maxNumNeighbors = max(numTooCloseNeighbors);
    removalCandidates = find(numTooCloseNeighbors == maxNumNeighbors);
    numCandidates = size(removalCandidates,2);
    overallMinOffset = zeros(1,numCandidates);
    if numCandidates>1,
        for candidateIter = 1:numCandidates,
            candidateIndex = removalCandidates(candidateIter);
            allOtherIndices = [1:candidateIndex-1,candidateIndex+1:numPoints];
            allOtherGraph = currentGraph(allOtherIndices,:);
            otherCandidatesList = [1:candidateIter-1,candidateIter+1:numCandidates];
            numOtherCandidates = numCandidates-1;
            minOffset = zeros(1,numOtherCandidates);
            for otherIter = 1:numOtherCandidates,
                currentOther = otherCandidatesList(otherIter);
                currentOtherCoords = currentGraph(currentOther,:);
                otherOffsetList = allOtherGraph - repmat(currentOtherCoords,numPoints-1,1);
                tempOtherD2 = sum(otherOffsetList.^2,2);
                tempOtherD2Exclusive = tempOtherD2(tempOtherD2>0);
                minOffset(otherIter) = min(tempOtherD2Exclusive);
            end
            overallMinOffset(candidateIter) = min(minOffset);
        end
        %Remove the candidate that has the highest rank (number of too-near neighbors)
        %and whos removal results in other similar-rank candidates having the least close neighbor.
        [minVal,minIndex] = min(overallMinOffset);
        firstMin = minIndex(1);
        removalIndex = removalCandidates(firstMin);
    else%if numCandidates>1,
        removalIndex = removalCandidates(1);
    end%if numCandidates>1,
    survivalIndices = [1:removalIndex-1,removalIndex+1:numPoints];
    currentGraph = currentGraph(survivalIndices,:);
    numPoints = size(currentGraph,1);
    numTooCloseNeighbors = zeros(1,numPoints);
    for pointIter = 1:numPoints,
        offsetList = currentGraph - repmat(currentGraph(pointIter,:),numPoints,1);
        tempD2=sum(offsetList.^2,2);
        tooCloseNeighbors=tempD2<=mergeDist2;
        numTooCloseNeighbors(pointIter) = sum(tooCloseNeighbors)-1;
    end

    if max(numTooCloseNeighbors) == 0,
        keepGoing = 0;
    end

    myFrame = zeros(40,40);
    for k = 1:numPoints,
        myFrame(currentGraph(k,1),currentGraph(k,2)) = 1;
    end
%     figure(1);imagesc(myFrame);
%     pause


end%while keepGoing == 1,



outputSubGraphYXZ=currentGraph;
