function blockingParameters = SubBlockData(blockingParameters)
%This function sub-blocks a large data file saves the blocks to disk
%The required blockingParameter fields are:
%   blockList : list of subBlock ranges generated by DetermineBlockingLimits.m
%           required blockList fields:
%               startPos : [y x z] start position of each block
%               stopPos  : [y x z] stop position of each block
%   fullDataSize = [y x z] size of the full data set
%   fullDataFileName = full path and file name of the full data .mat file
%   fullDataVariableName = variable name under which the full data is stored
%Additional fields include:
%   fullDataStorageType : either "struct" or "mask", default is mask
%   blockFolder : full path and file name to store blocks (defaults to current dir)
%   blockFileNameRoot : file root name for the block files (defaults to "imageVolume")
%   blockVariableName : variable name under which the block data is stored (defaults to "imageBlock")
%   blockStorageType : either "struct" or "mask", default is mask
%   blockNameZeroPad : string length for block # and coord limits in filename, default=5
%-----
blockList = blockingParameters.blockList;
fullDataSize = blockingParameters.fullDataSize;
fullDataFileName = blockingParameters.fullDataFileName;
fullDataVariableName = blockingParameters.fullDataVariableName;
%---
if isfield(blockingParameters,'fullDataStorageType'),
    fullDataStorageType = blockingParameters.fullDataStorageType;
else%if isfield(blockingParameters,'fullDataStorageType');
    fullDataStorageType = 'mask';
end%if-else isfield(blockingParameters,'fullDataStorageType'),
%---
if isfield(blockingParameters,'blockFolder'),
    blockFolder = blockingParameters.blockFolder;
else%if isfield(blockingParameters,'blockFolder');
    blockFolder = cd;
end%if-else isfield(blockingParameters,'blockFolder'),
%---
if isfield(blockingParameters,'blockFileNameRoot'),
    blockFileNameRoot = blockingParameters.blockFileNameRoot;
else%if isfield(blockingParameters,'blockFileNameRoot');
    blockFileNameRoot = 'imageVolume';
end%if-else isfield(blockingParameters,'blockFileNameRoot'),
%---
if isfield(blockingParameters,'blockVariableName'),
    blockVariableName = blockingParameters.blockVariableName;
else%if isfield(blockingParameters,'blockVariableName');
    blockVariableName = 'imageBlock';
end%if-else isfield(blockingParameters,'blockVariableName'),
%---
if isfield(blockingParameters,'blockStorageType'),
    blockStorageType = blockingParameters.blockStorageType;
else%if isfield(blockingParameters,'blockStorageType');
    blockStorageType = 'mask';
end%if-else isfield(blockingParameters,'blockStorageType'),
%---
if isfield(blockingParameters,'blockNameZeroPad'),
    blockNameZeroPad = blockingParameters.blockNameZeroPad;
else%if isfield(blockingParameters,'blockNameZeroPad');
    blockNameZeroPad = 5;
end%if-else isfield(blockingParameters,'blockNameZeroPad'),
%-----
%-----
switch blockNameZeroPad
    case 1, numString = '%01d';
    case 2, numString = '%02d';
    case 3, numString = '%03d';
    case 4, numString = '%04d';
    case 5, numString = '%05d';
    case 6, numString = '%06d';
    case 7, numString = '%07d';
    otherwise, numString = '%05d';
end%switch blockNameZeroPad      
%-----
numBlocks = numel(blockList);
%-----
load(fullDataFileName,fullDataVariableName);
for blockIter = 1:numBlocks,
    blockDetails = blockList(blockIter);
    blockSize = blockDetails.blockSize;
    p1 = blockDetails.startPos;
    p2 = blockDetails.stopPos;
    blockFileName = [blockFileNameRoot,'-Block',num2str(blockIter,numString) ];
    nameExtender = ['_Y',num2str(p1(1),numString),'-',num2str(p2(1),numString), ...
        '_X',num2str(p1(2),numString),'-',num2str(p2(2),numString), ...
        '_Z',num2str(p1(3),numString),'-',num2str(p2(3),numString)];
    extendedBlockFileName = [blockFileName,'-',nameExtender,'.mat'];
    blockPathFileName = fullfile(blockFolder,extendedBlockFileName);
    blockList(blockIter).blockFileName = blockPathFileName;
    if ~exist(blockPathFileName,'file'),
        switch fullDataStorageType
            case {'struct','Struct','STRUCT','structure','Structure','STRUCTURE'},
                myCommand = [pixelList,' = ',fullDataVariableName,'.pixelList;'];
                eval(myCommand);
                [cc1,cc2,cc3] = ind2sub(fullDataSize,pixelList);
                ff1 = and(cc1>=p1(1),cc1<=p2(1));
                ff2 = and(cc2>=p1(2),cc2<=p2(2));
                ff3 = and(cc3>=p1(3),cc3<=p2(3));
                ffAll = and(and(ff1,ff2),ff3);
                subCC1 = cc1(ffAll);
                subCC2 = cc2(ffAll);
                subCC3 = cc3(ffAll);
                subPixelList = sub2ind(blockSize,subCC1,subCC2,subCC3);
                %---
                switch blockStorageType,
                    case{'struct','Struct','STRUCT','structure','Structure','STRUCTURE'}
                        %Store subblock data in struct
                        myCommand = [blockVariableName,' = [];'];
                        eval(myCommand);
                        myCommand = [blockVariableName,'.maskSize = blockSize;'];
                        eval(myCommand);
                        myCommand = [blockVariableName,'.pixelList = subPixelList'];
                        eval(myCommand);
                    otherwise%all other options including "mask" for blockStorageType
                        %make a mask out of the sub block pixel list
                        myCommand = [blockVariableName,' = false(blockSize);'];
                        eval(myCommand);
                        myCommand = [blockVariableName,'(subPixelList) = 1;'];
                        eval(myCommand);
                end%switch blockStorageType
                saveSize = (getfield(whos(blockVariableName),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
                save(blockPathFileName,blockVariableName,scs);
            otherwise %all other options, including "mask" for fullDataStorageType
                    myCommand = [blockVariableName,' = ',fullDataVariableName,'(p1(1):p2(1),p1(2):p2(2),p1(3):p2(3));'];
                    eval(myCommand);
                    switch blockStorageType,
                        case{'struct','Struct','STRUCT','structure','Structure','STRUCTURE'}
                            %Convert mask to pixel list struct
                            myCommand = ['pixelList = find(',blockVariableName,');'];
                            eval(myCommand);
                            myCommand = [blockVariableName,' = [];'];
                            eval(myCommand);
                            myCommand = [blockVariableName,'.maskSize = blockSize;'];
                            eval(myCommand);
                            myCommand = [blockVariableName,'.pixelList = pixelList'];
                            eval(myCommand);
                        otherwise%all other options including "mask" for blockStorageType
                            %do nothing: leave mask as mask
                    end%switch blockStorageType
                    saveSize = (getfield(whos(blockVariableName),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
                    save(blockPathFileName,blockVariableName,scs);
        end%switch fullDataStorageType
    end%if ~exist(blockPathFileName,'file'),
end%for blockIter = 1:numBlocks,
%-----
blockingParameters.blockList = blockList;
    

    
 
    
    
    


