function [output_filename] = dividefilter2(passed_path_filename1,passed_path_filename2);

%This function divides the values of a Biorad .pic or custom .big file  (or
%.tif file)
%by a single-frame filter file that ranges in value from 0 to 255, and has the
%same image size as the Biorad .pic file (or custom .big file)to be normalized.
%Typically, this is a filter file that shows the nonuniformity of the image
%at the same scan size, and has been smoothed, and offset-subtracted
%Input arguments are the full path of the main file, and the filter file
%If no input variables are given, a user input window displays.
%Output is saved to a file with suffix "-divfilt", and and the full 
%path and filename of the file is returned as a string
%
%written by Phil Tsai 01/25/05
%last updated 08/16/05

nin = nargin;
nout = nargout;
%if only one argument is input, we assume it is the name of the first file.
if nin==0, %no arguments passed => open user input box
    [filename1,pathname1] = uigetfile('*.pic;*.big;*.tif','Choose main image file');
    input_filename1 = [pathname1,filename1];
    cd(pathname1);
else %input arguments were passed
    [junk,filename_size] = size(passed_path_filename1);
    endpath = max(findstr(passed_path_filename1,filesep)); %location of last \ in path
    if isempty(endpath) %if only the filename alone was entered
        pathname1 = cd;
        input_filename1 = [pathname1,filesep,passed_path_filename1];
    else %pathname included in filename
        input_filename1 = passed_path_filename1;
    end %if isemtpy(endpath)
end %if nin==0;

[junk,filename_size] = size(input_filename1);
myext = input_filename1(filename_size-2:filename_size);
myinfo = imfinfo(input_filename1);
dnum_pics1 = double(length(myinfo));
dx_size1 = double(myinfo(1).Height);
dy_size1 = double(myinfo(1).Width);
saveas_suffix = '-divfilt.tif';
myext = 'tif';
myfiletype = '*.tif';

%if two arguments are input, we assume that the second input is the name of
%the second file
if nin<2,
    [filename2,pathname2] = uigetfile(myfiletype,'Choose a secondary file to subtract from the main file');
	input_filename2 = [pathname2,filename2];
else
    [junk,filename_size] = size(passed_path_filename2);
    endpath = max(findstr(passed_path_filename2,filesep)); %location of last \ in path
    if isempty(endpath) %if only the filename alone was entered
        pathname2 = cd;
        input_filename2 = [pathname2,filesep,passed_path_filename2];
    else %pathname included in filename
        input_filename2 = passed_path_filename2;
    end %if isemtpy(endpath)
end

%Create Outputfilename
temp = size(input_filename1);
temp2 = temp(2)-4;
temp3 = input_filename1(1,1:temp2);
temp4 = [temp3,saveas_suffix];
output_filename=temp4;

loadparameters.input_filename = input_filename1;
loadparameters.waitbar_yn = 'no';
mypics1 = loadmovie(loadparameters);
loadparameters.input_filename = input_filename2;
mypics2 = loadmovie(loadparameters);
mymax = max(max(double(mypics2)));
mypics3 = double(mypics2)./mymax;

for k = 1:dnum_pics1,
    oldframe = double(mypics1(:,:,k));
    newframe = uint16(oldframe./mypics3);
    mypics1(:,:,k) = newframe;
end
maketiff(mypics1,output_filename);

return

