function outputParameters = fit_2_gaussians(inputParameters)

%fit n gaussians
%first fit single gaussians one_by_one to maximal peaks
%  considering only a 2 half-stdev's from the peak.
%   the subtract the fitted gaussian, forcing positive
%next, analyze the remaining curve for the next gaussian
%these gaussians are used as seeds for the iterative fitting
%routine MULTI_FIT.m

xRange = inputParameters.xRange;
yData = inputParameters.yData;

%Reserve first Bin for penalties;
yData(1) = 0;

%gaussian = A1.*exp(-((xRange-xo1).^2) / (2*(sig1.^2))) +...;
%           A2.*exp(-((xRange-xo2).^2) / (2*(sig2.^2)));
%beta0 = [A1, xo1, sig1; A2, xo2, sig2];
beta0 = [1, 0, 0.5,1 1 0.5];


[beta1,r,j] = nlinfit(xRange,yData,@twoGaussianFunction,beta0);
A1 = beta1(1);
xo1 = beta1(2);
sig1 = beta1(3);
A2 = beta1(4);
xo2 = beta1(5);
sig2 = beta1(6);
yFit1 = abs(A1).*exp(-((xRange-xo1).^2) / (2*(sig1.^2)));
yFit2 = abs(A2).*exp(-((xRange-xo2).^2) / (2*(sig2.^2)));

outputParameters = inputParameters;
outputParameters.yFit1 = yFit1;
outputParameters.yFit2 = yFit2;

return







function g = twoGaussianFunction(mycoeff,xRange)


g = zeros(size(xRange));
%First Gaussian
A1 = abs(mycoeff(1));
xo1 = mycoeff(2);
sig1 = abs(mycoeff(3));
term1 = A1.*exp(-(xRange-xo1).^2 / (2*(sig1.^2)));
g = g + term1;
%Second Gaussian
A2 = abs(mycoeff(4));
xo2 = mycoeff(5);
sig2 = abs(mycoeff(6));
term2 = A2.*exp(-((xRange-xo2).^2) / (2*(sig2.^2)));
g = g + term2;

% % figure(9);plot(xRange,term1,'b-');hold on
% % figure(9);plot(xRange,term2,'r-');hold off

%Add Penalty Functions

gSum = sum(g(:));
% penaltyWeight = [1 1 0.1 1] .* gSum .* 0.01;
% penaltyWeight = [2 2 0.43 0.43] .* gSum .* 0.005;
penaltyWeight = [2 2 0.43 0.43] .* gSum .* 0.01;

%Force centers of g1 & g2 to be close to 0,1
penalty1 = abs(xo1-0)*penaltyWeight(1);
penalty2 = abs(xo2-1)*penaltyWeight(2);


%Force peak heights to be similar scale
penalty3 = abs(log(A1)-log(A2)) * penaltyWeight(3);

%Force std to be similar
penalty4 = abs(log(abs(sig1))-log(abs(sig2))) * penaltyWeight(4);

g(1) = g(1) + penalty1 + penalty2 + penalty3 + penalty4;

return


