function [output_filename] = subtractoffset(passed_path_filename1,offset_level);

%This function subtracts a manually entered offset value from each pixel in
% a Biorad .pic file one frame at a time.  
%Input argument is a the full pathname and filename string of the desired
%file; and the offset level. If no input variable is given, a user input window displays.
%Output is saved to a file with suffix "-off", and and the full path and filename of the file
%is returned as a string

%
% Written by Phil Tsai 01/24/05
% Last Updated 12/02/06


nin = nargin;
nout = nargout;

%if only one argument is input, we assume it is the name of the first file.
if nin==0, %no arguments passed => open user input box
    [filename1,pathname1] = uigetfile('*.tif;*.pic;*.big','Choose a .pic or .big file');
    input_filename1 = [pathname1,filename1];
    cd(pathname1);
else %input arguments were passed
    [junk,filename_size] = size(passed_path_filename1);
    endpath = max(findstr(passed_path_filename1,filesep)); %location of last \ in path
    if isempty(endpath) %if only the filename alone was entered
        pathname1 = cd;
        input_filename1 = [pathname1,filesep,passed_path_filename1];
    else %pathname included in filename
        input_filename1 = passed_path_filename1;
    end %if isemtpy(endpath)
end %if nin==0;


[junk,filename_size] = size(input_filename1);
myext = input_filename1(filename_size-2:filename_size);

myinfo = imfinfo(input_filename1);
dnum_pics1 = double(length(myinfo));
dx_size1 = double(myinfo(1).Height);
dy_size1 = double(myinfo(1).Width);
myext = 'tif';


%Dialog box to ask for offset level if not given as an input argument

if nin < 2,    
	prompt={'Enter offset level (0=autoseek; not yet implemented)'};
	def={'0'};
	dlgTitle='Offset subtraction';
	lineNo=1;
	answer=inputdlg(prompt,dlgTitle,lineNo,def,'on');
	myoffset = str2double(answer(1));
else
    myoffset = offset_level;
end

    
if myoffset==0,
    %insert code here to read through entire file and find minimum  
end


%Create Outputfilename
%saveas_suffix = ['-off',num2str(myoffset),'.',myext];
saveas_suffix = ['-off.',myext];
temp = size(input_filename1);
temp2 = temp(2)-4;
temp3 = input_filename1(1,1:temp2);
temp4 = [temp3,saveas_suffix];
output_filename=temp4;

loadparameters.input_filename = input_filename1;
loadparameters.waitbar_yn = 'no';

mypics = loadmovie(loadparameters);
mypics = mypics - myoffset;
maketiff(mypics,output_filename,'nowb');


return
