%This code starts with a vectorProject that has been produced from the StrandEditor2
%It assumes that surface veins and surface arteries have been manually labeled. 

%%
%Stage 1: Load a vectorProject

flowAnalysisParameters = [];

%Choose a vectorProject that has been already been through the StrandEditor.
%if the vectorProject file name contains '-LargestSubgraph', then nonattached segments and
%nonsurface dangling ends have already been removed.  If not, run the
%"VectorizeLargestSubgraph" code which produces this result and stores it in a file with
%-LargestSubgraph' prefix.

cleanupFlag = 1;

[fileName,pathName] = uigetfile({'*VectorProject.mat'},'Select a vectorProject file');
if pathName~=0,
    cd(pathName)
    %-----
    loadFileName = [fileName];
    disp('Loading VectorProject...');
    load(loadFileName,'vectorProject');
    
        
    %Fix error with single and uint16 storage
    %Added 07/16/2012
    vertices = vectorProject.vectorizedStructure.Vertices;
    vertices.AllVerts = double(vertices.AllVerts);
    vertices.AllRadii = double(vertices.AllRadii);
    vertices.AllLabels = double(vertices.AllLabels);
    vertices.AllConfidences = double(vertices.AllConfidences);
    vectorProject.vectorizedStructure.Vertices = vertices;
    
    
    %Check for 'LargestSubgraph' prefix;
    if isempty(strfind(fileName,'LargestSubgraph'))
        %LargestSubgraph not yet calculated
        disp('Extracting Largest Subgraph...');
        vectorProject = VectorizeLargestSubgraph(vectorProject);
    end%if isempty(strfind(fileName,'-LargestSubgraph'))
    %-----
    %Check for cell-type storage in the vectorized structure.
    %Conver to non-cell storage for speed of loading
    if iscell(vectorProject.vectorizedStructure.Vertices.StrandIndices),
        vectorProject = ConvertToNonCellStorage(vectorProject);
    end%if ~iscell(vectorProject.vectorizedStructure.Vertices.StrandIndices),
    %-----
    %Update vectorProject Name, Folder, and History
    vectorProjectRootName = vectorProject.vectorProjectRootName;
    vectorProjectRootName = [vectorProjectRootName,'-Flow'];
    vectorProject.vectorProjectRootName = vectorProjectRootName;
    vectorProjectFileName = [vectorProjectRootName,'-VectorProject.mat'];
    if ~isfield(vectorProject,'vectorProjectFolderHistory'),
        vectorProject.vectorProjectFolderHistory = [];
    end%if ~isfield(...
    vectorProject.vectorProjectFolderHistory{numel(vectorProject.vectorProjectFolderHistory)+1} = vectorProject.vectorProjectFolder;
    vectorProject.vectorProjectFolder = pathName;
    flowAnalysisParameters.progressDates.initialized = datestr(now);
    vectorProject.flowAnalysisParameters = flowAnalysisParameters;
    %-----
    %Remove GapConnectionStruct for speed of loading
    vectorProject = rmfield(vectorProject,'GapConnectionStruct');
end%if pathName~=0;

%-----
if cleanupFlag,
    clear fileName pathName loadFileName vectorProjectChosenFlag correspondingFlowProjectFileName 
    clear associatedFlowProjectFolder shortName
end%if cleanupFlag,



%%
%Stage 2 : Calculate true strand length (accounting for diagonal connections being longer)
disp('Calculating true strand length...');
%---
numStrands = numel(vectorProject.vectorizedStructure.Strands);
strandLengthList = zeros(numStrands,1);
strandNumPointsList = zeros(numStrands,1);
for strandIter = 1:numStrands,
    strandID = strandIter;
    thisStrand = vectorProject.vectorizedStructure.Strands(strandID);
    stei = thisStrand.StartToEndIndices;
    av = vectorProject.vectorizedStructure.Vertices.AllVerts(stei,:);
    numPoints = numel(stei);
    pointSep = zeros(100,1);
    for pointIter = 1:numPoints-1;
        pointDist = av(pointIter,:) - av(pointIter+1,:);
        pointSep(pointIter) = sqrt(sum(pointDist.^2));
    end%for pointIter
    strandLengthList(strandID) = sum(pointSep)+1;
    strandNumPointsList(strandID) = numPoints;
end%for strandIter
vectorProject.vectorizedStructure.strandLengthList = strandLengthList;
vectorProject.vectorizedStructure.strandNumPointsList = strandNumPointsList;
%---
if cleanupFlag,
    clear strandID thisStrand iv av numPoints pointSep pointDist pointIter
    clear stei numStrands strandIter strandLengthList strandNumPointsList
end%if cleanupFlag,


%%
%Stage 3 : Take the mean of the (50% - 90%) range of the sorted radius
%  list as the effective Strand Radius.
disp('Calculating effective Strand Radii...');
%---
numStrands = numel(vectorProject.vectorizedStructure.Strands);
effectiveStrandRadiusList = zeros(numStrands,1);
for strandIter = 1:numStrands,
    stei = vectorProject.vectorizedStructure.Strands(strandIter).StartToEndIndices;
    rr = vectorProject.vectorizedStructure.Vertices.AllRadii(stei);
    s = sort(rr);
    cutPoint1 = floor(numel(s)* 0.50);
    cutPoint2 = ceil(numel(s)*0.90);
    cutRR = s(cutPoint1:cutPoint2);
    effectiveStrandRadiusList(strandIter) = mean(cutRR);    
    if mean(cutRR) == 0,
        svn = vectorProject.vectorizedStructure.Strands(strandIter).StartVertexNeighborStrands;
        evn = vectorProject.vectorizedStructure.Strands(strandIter).EndVertexNeighborStrands;
        neighorStrandList = [svn, evn];
        stei = [vectorProject.vectorizedStructure.Strands(neighorStrandList).StartToEndIndices];
        neighborRadiusList = vectorProject.vectorizedStructure.Vertices.AllRadii(stei);
        neighborRadiusList = neighborRadiusList(neighborRadiusList>0);
        if median(neighborRadiusList) == 0,
            disp('Radius Error');
            keyboard
        end% if median(neighborRadiusList) == 0,
        effectiveStrandRadiusList(strandIter) = median(neighborRadiusList);  
    end%if mean(cutRR) == 0,
end%for strandIter = 1:numStrands,
vectorProject.vectorizedStructure.effectiveStrandRadiusList = effectiveStrandRadiusList;
%---
if cleanupFlag,
    clear stei rr s cutPoint1 cutPoint2 cutRR effectiveStrandRadiusList
    clear stei numStrands strandIter strandLengthList strandNumPointsList
end%if cleanupFlag,
%%
effectiveStrandRadiusList = vectorProject.vectorizedStructure.effectiveStrandRadiusList;
effectiveStrandRadiusList = effectiveStrandRadiusList+0.5;
vectorProject.vectorizedStructure.effectiveStrandRadiusList = effectiveStrandRadiusList;

allRadii = vectorProject.vectorizedStructure.Vertices.AllRadii;
allRadii = allRadii+0.5;
vectorProject.vectorizedStructure.Vertices.AllRadii = allRadii;



%%
%Stage 4 : Calculate the midpoints, limits, median radius, label for each strand
disp('Calculating Misc Strand Stats...');
%---
numStrands = numel(vectorProject.vectorizedStructure.Strands);
Vertices = vectorProject.vectorizedStructure.Vertices;
Strands = vectorProject.vectorizedStructure.Strands;
strandEndToEndSeparationList = zeros(numStrands,1);%end-to-end straight line separation of each strand
for strandIter = 1:numStrands,
    stei = Strands(strandIter).StartToEndIndices;
    cc = Vertices.AllVerts(stei,:);
    yxzLim(:,1) = min(cc,[],1)';
    yxzLim(:,2) = max(cc,[],1)';
    strandMidYXZList(strandIter,:) = mean(yxzLim,2);
    Strands(strandIter).YXZLimits = yxzLim;
    Strands(strandIter).MedianRadius = median(Vertices.AllRadii(Strands(strandIter).InteriorVertices,:));
    Strands(strandIter).MedianLabel = mode(Vertices.AllLabels(Strands(strandIter).InteriorVertices,:));
    %---
    p1 = Vertices.AllVerts(stei(1),:);
    p2 = Vertices.AllVerts(stei(end),:);
    dp = p1-p2;
    strandEndToEndSeparationList(strandIter) = sqrt(sum(dp.^2));
        
        
end%for strandIter = 1:numStrands,
vectorProject.vectorizedStructure.Strands = Strands;
vectorProject.vectorizedStructure.Vertices = Vertices;
vectorProject.vectorizedStructure.strandMidYXZList = strandMidYXZList;
vectorProject.vectorizedStructure.strandEndToEndSeparationList = strandEndToEndSeparationList;
%---
if cleanupFlag,
    clear yxzLim strandMidYXZList numStrands Strands Vertices cc
    clear stei p1 p2 dp strandEndToEndSeparationList strandIter
end%if cleanupFlag,


%%
%Stage 5: Calculate the unperturbed flow surface vessel endpoints are held at fixed
%           voltages (100 and 50) using effect Strand Radii & true lengths

%Calculate the unperturbed flow on the trimmed vectorizedStructure
if isfield(vectorProject.vectorizedStructure.Vertices,'AllVoltages'),
    disp('Unperturbed Flow already calculated.');
else%if isfield(flowProject.vectorizedStructure.Vertices,'AllVoltages'),
    disp('Calculating unperturbed flow...');
    vectorProject = CalculateUnperturbedFlow003(vectorProject);
    vectorProject.flowAnalysisParameters.progressDates.unperturbedFlow = datestr(now);
end%if-else isfield(flowProject.vectorizedStructure.Vertices,'AllVoltages'),


%%
%Stage 6: Find "basic set" of plunging arterioles and venules
disp('Finding Plunging Arteriole and Venules...');
%-----
%Plunging Arterioles starting points are the points of connection between
%a surface supply strand and a plunging supply strand
%surface supply labels = [330:339]
%plunging supply labels = [430:439]
label1List = [330:339]; %surface arterioles
label2List = [1:329,340:999]; %plunging arterioles are all other labels
[foundList1,foundList2] = LocateConnectingStrands(vectorProject.vectorizedStructure,label1List,label2List);
vectorProject.vectorizedStructure.plungingArterioleTipList = foundList2;
%---
baseStrandIDList = foundList2;
saveFileName = [vectorProjectRootName,'-PlungingArterioleTipStrandList.mat'];
save(saveFileName,'baseStrandIDList');

%Plunging Venules starting points are the points of connection between
%a surface draining strand and a plunging draining strand
%surface draining labels = [340:349]
%plunging draining labels = [440:449]
label1List = [340:349]; %surface venules
label2List = [0:339,350:999]; %plunging venules are all other labels
[foundList1,foundList2] = LocateConnectingStrands(vectorProject.vectorizedStructure,label1List,label2List);
vectorProject.vectorizedStructure.plungingVenuleTipList = foundList2;
%---
baseStrandIDList = foundList2;
saveFileName = [vectorProjectRootName,'-PlungingVenuleTipStrandList.mat'];
save(saveFileName,'baseStrandIDList');

vectorProject = TrackDownFromPlungingArterioleTips(vectorProject);
baseStrandIDList = vectorProject.vectorizedStructure.allPlungingArterioleList;
saveFileName = [vectorProjectRootName,'-AllPlungingArterioleStrandList.mat'];
save(saveFileName,'baseStrandIDList');

vectorProject = TrackDownFromPlungingVenuleTips(vectorProject);
baseStrandIDList = vectorProject.vectorizedStructure.allPlungingVenuleList;
saveFileName = [vectorProjectRootName,'-AllPlungingVenuleStrandList.mat'];
save(saveFileName,'baseStrandIDList');


%---
if cleanupFlag,
    clear label1List label2List foundList1 foundList2 baseStrandIDList saveFileName
end%if cleanupFlag,

%%
% % % % %  vectorProject.circuitAnalysisParameters.outputMaskSizeDivisor = 3;%Hardcode
% % % % % % Create Masks for visualization
% % % % % if ~isfield(vectorProject.flowAnalysisParameters.progressDates,'unperturbedMasks'),
% % % % %     disp(['Generating VoltageMask : ',datestr(now)]);
% % % % %     GenerateCircuitVoltageMask3(flowProject);
% % % % %     disp(['Generating RGBVoltageMask : ',datestr(now)]);
% % % % %     GenerateCircuitRGBVoltageMask3(flowProject);
% % % % %     %---
% % % % %     disp(['Generating CurrentMask : ',datestr(now)]);
% % % % %     GenerateCircuitCurrentMask3(flowProject);
% % % % %     disp(['Generating RGBCurrentMask : ',datestr(now)]);
% % % % %     GenerateCircuitRGBCurrentMask3(flowProject);
% % % % %     %---\
% % % % %     vectorProject.flowAnalysisParameters.progressDates.unperturbedMasks = datestr(now);
% % % % %     disp('Saving flowProject...');
% % % % %     save(flowProjectFileName,'flowProject');
% % % % % else%if ~isfield(vectorProject.flowAnalysisParameters.progressDates,'unperturbedMasks'),
% % % % %     disp('Unperturbed flow masks previously made.');
% % % % % end%if-else ~isfield(vectorProject.flowAnalysisParameters.progressDates,'unperturbedMasks'),




%%
disp('Saving -Flow-VectorProject');
save(vectorProjectFileName,'vectorProject');


