%HARDCODE:

flowAnalysisParameters = [];



%Choose a vectorProject or flowProject file
%if a vectorProject file is selected, and a flowProject file already exists,
%display warning and exit

%if the vectorProject file name contains '-LargestSubgraph', then nonattached segments and
%nonsurface dangling ends have already been removed.  If not, run the
%"VectorizeLargestSubgraph" code which produces this result and stores it in a file with
%-LargestSubgraph' prefix.

cleanupFlag = 1;

if ~exist('flowProject','var');
    [fileName,pathName] = uigetfile({'*FlowProject.mat;*VectorProject.mat'},'Select a vectorProject or flowProject file');
    if pathName~=0,
        cd(pathName)
        vectorProjectChosenFlag = ~isempty(findstr('VectorProject',fileName));
        if vectorProjectChosenFlag == 1,
             correspondingFlowProjectFileName = regexprep(fileName,'Vector','Flow');
             correspondingFlowProjectFileName = correspondingFlowProjectFileName(1:end-4);
% % %             %Check for corresponding flowProject
% % %             if exist(correspondingFlowProjectFileName,'file') > 0,
% % %                 disp('The corresponding Flow Project has already been started.');
% % %                 keyboard
% % %             end%if exist(correspondingFlowProjectFileName,'file') > 0,
            %-----
            loadFileName = [fileName];
            disp('Loading VectorProject...');
            load(loadFileName,'vectorProject');
            %Check for 'LargestSubgraph' prefix;
            if isempty(strfind(fileName,'-LargestSubgraph'))
                %LargestSubgraph not yet calculated
                disp('Extracting Largest Subgraph...');
                vectorProject = VectorizeLargestSubgraph(vectorProject);
            end%if isempty(strfind(fileName,'-LargestSubgraph'))
            %-----
            if iscell(vectorProject.vectorizedStructure.Vertices.StrandIndices),
                vectorProject = ConvertToNonCellStorage(vectorProject);
            end%if ~iscell(vectorProject.vectorizedStructure.Vertices.StrandIndices),
            %-----
            flowProject.flowProjectRootName = vectorProject.vectorProjectRootName;
            flowProject.vectorProjectFolder = vectorProject.vectorProjectFolder;
            flowProject.flowProjectFileName = correspondingFlowProjectFileName;
            flowProject.sourceVectorProjectFileName = loadFileName;
            flowProject.maskSize = vectorProject.maskSize;
            flowProject.voxelSizeInMicrons = vectorProject.voxelSizeInMicrons;
            flowProject.vectorizedStructure = vectorProject.vectorizedStructure;
            flowProject.cellCentroidAndLabelList = vectorProject.cellCentroidAndLabelList;
            flowProject.volumeFileNames = vectorProject.volumeFileNames;
            flowProject.progressDates.initialized = datestr(now);
            %-----
            clear vectorProject
            
            flowProjectFileName = [correspondingFlowProjectFileName,num2str(ceil(rand(1)*1e6),'%06d'),'.mat'];
            [fileName,pathName] = uiputfile(flowProjectFileName,'Save FlowProject file as...');
            cd(pathName)
            flowProjectFileName = [fileName];
            flowProject.flowProjectFileName = flowProjectFileName;
            shortName = flowProjectFileName(findstr(flowProjectFileName,'FlowProject'):end-4);
            associatedFlowProjectFolder = [shortName,'_Files'];
            flowProject.associatedFlowProjectFolder = associatedFlowProjectFolder;
            mkdir(flowProject.vectorProjectFolder,flowProject.associatedFlowProjectFolder)
            disp('Saving flowProject...');
            save(flowProjectFileName,'flowProject');
        else%if vectorProjectChosenFlag == 1,
            %Load the flowProject
            loadFileName = [fileName];
            load(loadFileName,'flowProject');
            flowProjectFileName = loadFileName;
        end%if-else vectorProjectChosenFlag == 1,
    end%if pathName~=0;
else%if ~exist('flowProject','var');
    disp('Working with flowProject in current memory');
    flowProjectFileName = flowProject.flowProjectFileName;
end%if-else ~exist('flowProject','var');
%-----
if cleanupFlag,
    clear fileName pathName loadFileName vectorProjectChosenFlag correspondingFlowProjectFileName 
    clear associatedFlowProjectFolder shortName 
end%if cleanupFlag,

%----------
%----------
%----------
%----------

%Calculate the unperturbed flow on the trimmed vectorizedStructure
if isfield(flowProject.vectorizedStructure.Vertices,'AllVoltages'),
    disp('Unperturbed Flow already calculated.');
else%if isfield(flowProject.vectorizedStructure.Vertices,'AllVoltages'),
    disp('Calculating unperturbed flow');
    flowProject = CalculateUnperturbedFlow002(flowProject);
    flowProject.progressDates.unperturbedFlow = datestr(now);
    disp('Saving flowProject...');
    save(flowProjectFileName,'flowProject');
end%if-else isfield(flowProject.vectorizedStructure.Vertices,'AllVoltages'),


flowProject.circuitAnalysisParameters.outputMaskSizeDivisor = 3;%Hardcode


% % % % % % Create Masks for visualization
% % % % % if ~isfield(flowProject.progressDates,'unperturbedMasks'),
% % % % %     disp(['Generating VoltageMask : ',datestr(now)]);
% % % % %     GenerateCircuitVoltageMask3(flowProject);
% % % % %     disp(['Generating RGBVoltageMask : ',datestr(now)]);
% % % % %     GenerateCircuitRGBVoltageMask3(flowProject);
% % % % %     %---
% % % % %     disp(['Generating CurrentMask : ',datestr(now)]);
% % % % %     GenerateCircuitCurrentMask3(flowProject);
% % % % %     disp(['Generating RGBCurrentMask : ',datestr(now)]);
% % % % %     GenerateCircuitRGBCurrentMask3(flowProject);
% % % % %     %---\
% % % % %     flowProject.progressDates.unperturbedMasks = datestr(now);
% % % % %     disp('Saving flowProject...');
% % % % %     save(flowProjectFileName,'flowProject');
% % % % % else%if ~isfield(flowProject.progressDates,'unperturbedMasks'),
% % % % %     disp('Unperturbed flow masks previously made.');
% % % % % end%if-else ~isfield(flowProject.progressDates,'unperturbedMasks'),


%Find "basic set" of plunging arterioles tops
if ~isfield(flowProject.progressDates,'plungingArterioleTops'),
    disp('Finding Plunging Arteriole Top Strands...');
    flowProject = LocatePlungingArterioles(flowProject);
    disp('Saving flowProject...');
    flowProject.progressDates.plungingArterioleTops = datestr(now);
    save(flowProjectFileName,'flowProject');
end






