%HARDCODE:

flowAnalysisParameters = [];



%Choose a vectorProject or flowProject file
%if a vectorProject file is selected, and a flowProject file already exists,
%display warning and exit

%if the vectorProject file name contains '-LargestSubgraph', then nonattached segments and
%nonsurface dangling ends have already been removed.  If not, run the
%"VectorizeLargestSubgraph" code which produces this result and stores it in a file with
%-LargestSubgraph' prefix.

cleanupFlag = 1;

if ~exist('flowProject','var');
    [fileName,pathName] = uigetfile({'*FlowProject.mat;*VectorProject.mat'},'Select a vectorProject or flowProject file');
    if pathName~=0,
        cd(pathName)
        vectorProjectChosenFlag = ~isempty(findstr('VectorProject',fileName));
        if vectorProjectChosenFlag == 1,
             correspondingFlowProjectFileName = regexprep(fileName,'Vector','Flow');
             correspondingFlowProjectFileName = correspondingFlowProjectFileName(1:end-4);
% % %             %Check for corresponding flowProject
% % %             if exist(correspondingFlowProjectFileName,'file') > 0,
% % %                 disp('The corresponding Flow Project has already been started.');
% % %                 keyboard
% % %             end%if exist(correspondingFlowProjectFileName,'file') > 0,
            %-----
            loadFileName = [fileName];
            disp('Loading VectorProject...');
            load(loadFileName,'vectorProject');
            %Check for 'LargestSubgraph' prefix;
            if isempty(strfind(fileName,'-LargestSubgraph'))
                %LargestSubgraph not yet calculated
                disp('Extracting Largest Subgraph...');
                vectorProject = VectorizeLargestSubgraph(vectorProject);
            end%if isempty(strfind(fileName,'-LargestSubgraph'))
            %-----
            if iscell(vectorProject.vectorizedStructure.Vertices.StrandIndices),
                vectorProject = ConvertToNonCellStorage(vectorProject);
            end%if ~iscell(vectorProject.vectorizedStructure.Vertices.StrandIndices),
            %-----
            flowProject.flowProjectRootName = vectorProject.vectorProjectRootName;
            flowProject.vectorProjectFolder = vectorProject.vectorProjectFolder;
            flowProject.flowProjectFileName = correspondingFlowProjectFileName;
            flowProject.sourceVectorProjectFileName = loadFileName;
            flowProject.maskSize = vectorProject.maskSize;
            flowProject.voxelSizeInMicrons = vectorProject.voxelSizeInMicrons;
            flowProject.vectorizedStructure = vectorProject.vectorizedStructure;
            flowProject.cellCentroidAndLabelList = vectorProject.cellCentroidAndLabelList;
            flowProject.volumeFileNames = vectorProject.volumeFileNames;
            flowProject.progressDates.initialized = datestr(now);
            %-----
            clear vectorProject
            
            flowProjectFileName = [correspondingFlowProjectFileName,num2str(ceil(rand(1)*1e6),'%06d'),'.mat'];
            [fileName,pathName] = uiputfile(flowProjectFileName,'Save FlowProject file as...');
            cd(pathName)
            flowProjectFileName = [fileName];
            flowProject.flowProjectFileName = flowProjectFileName;
            shortName = flowProjectFileName(findstr(flowProjectFileName,'FlowProject'):end-4);
            associatedFlowProjectFolder = [shortName,'_Files'];
            flowProject.associatedFlowProjectFolder = associatedFlowProjectFolder;
            mkdir(flowProject.vectorProjectFolder,flowProject.associatedFlowProjectFolder)
            disp('Saving flowProject...');
            save(flowProjectFileName,'flowProject');
        else%if vectorProjectChosenFlag == 1,
            %Load the flowProject
            loadFileName = [fileName];
            load(loadFileName,'flowProject');
            flowProjectFileName = loadFileName;
        end%if-else vectorProjectChosenFlag == 1,
    end%if pathName~=0;
else%if ~exist('flowProject','var');
    disp('Working with flowProject in current memory');
    flowProjectFileName = [flowProject.flowProjectFileName(1:end-4),'-adj.mat'];
end%if-else ~exist('flowProject','var');
%-----
if cleanupFlag,
    clear fileName pathName loadFileName vectorProjectChosenFlag correspondingFlowProjectFileName 
    clear associatedFlowProjectFolder shortName 
end%if cleanupFlag,

%----------
%----------
%----------
%----------

flowProject.vectorizedStructure.Vertices


% % % 
% % % % % ppp = [30 114 123 225 338 350 420 439 503 525 796 820 839 1116 1133 1153 1585 1950 2063 2994 3987 6375 6407];
% % % ppp = [30 114 123 225 338 350 420 503 525 796 820 839 1116 1133 1153 1585 1950 2063 2994 3987 6375 6407];
% % % for strandIter = 1:numel(ppp),
% % %     stei = flowProject.vectorizedStructure.Strands(ppp(strandIter)).StartToEndIndices;
% % %     flowProject.vectorizedStructure.Vertices.AllRadii(stei) = 0.1;
% % % end
% % % flowProject.vectorizedStructure.Vertices = rmfield( flowProject.vectorizedStructure.Vertices,'AllVoltages');
% % % 


%Calculate the unperturbed flow on the trimmed vectorizedStructure
if isfield(flowProject.vectorizedStructure.Vertices,'AllVoltages'),
    disp('Unperturbed Flow already calculated.');
    originalVoltages = flowProject.vectorizedStructure.Vertices.AllVoltages;
    flowProject.vectorizedStructure.Vertices.OriginalVoltages = originalVoltages;
    flowProject.vectorizedStructure.Vertices = rmfield( flowProject.vectorizedStructure.Vertices,'AllVoltages');
    clear originalVoltages;
else%if isfield(flowProject.vectorizedStructure.Vertices,'AllVoltages'),
    disp('Calculating unperturbed flow');
    flowProject = CalculateUnperturbedFlow002(flowProject);
    flowProject.progressDates.unperturbedFlow = datestr(now);
    originalVoltages = flowProject.vectorizedStructure.Vertices.AllVoltages;
    flowProject.vectorizedStructure.Vertices.OriginalVoltages = originalVoltages;
    disp('Saving flowProject...');
    save(flowProjectFileName,'flowProject');
    clear originalVoltages
end%if-else isfield(flowProject.vectorizedStructure.Vertices,'AllVoltages'),


flowProject.circuitAnalysisParameters.outputMaskSizeDivisor = 3;%Hardcode



%Find "basic set" of plunging arterioles tops
if ~isfield(flowProject.progressDates,'plungingArterioleTops'),
    disp('Finding Plunging Arteriole Top Strands...');
    flowProject = LocatePlungingArterioles(flowProject);
    disp('Saving flowProject...');
    flowProject.progressDates.plungingArterioleTops = datestr(now);
    save(flowProjectFileName,'flowProject');
end

%%

gds = flowProject.vectorizedStructure;


%First, calculate true strand length (accounting for diagonal connections being longer)
numStrands = numel(gds.Strands);
strandLengthList = zeros(numStrands,1);
strandNumPointsList = zeros(numStrands,1);
disp('Calculating true strand lengths');
for strandIter = 1:numStrands,
    strandID = strandIter;
    thisStrand = gds.Strands(strandID);
    stei = thisStrand.StartToEndIndices;
    av = gds.Vertices.AllVerts(stei,:);
    numPoints = numel(stei);
    pointSep = zeros(100,1);
    for pointIter = 1:numPoints-1;
        pointDist = av(pointIter,:) - av(pointIter+1,:);
        pointSep(pointIter) = sqrt(sum(pointDist.^2));
    end%for pointIter
    strandLengthList(strandID) = sum(pointSep)+1;
    strandNumPointsList(strandID) = numPoints;
end%for strandIter
clear strandID thisStrand iv av numPoints pointSep pointDist pointIter
gds.strandLengthList = strandLengthList;
gds.strandNumPointsList = strandNumPointsList;

%%

%Take the mean of the (50% - 90%) range of the sorted radius list as the effective Strand
%Radius.

gds.Vertices.OriginalRadii = gds.Vertices.AllRadii;

effectiveStrandRadiusList = zeros(numStrands,1);
originalVoltageDropList = zeros(numStrands,1);
numStrands = numel(gds.Strands);
disp('Calculating original voltage drops and effective strand radii and replacing');
for strandIter = 1:numStrands,
    stei = gds.Strands(strandIter).StartToEndIndices;
    rr = gds.Vertices.AllRadii(stei);
    vv = gds.Vertices.OriginalVoltages(stei);
    medianVoltage = median(vv);
    dv = abs(max(vv)-min(vv));
    s = sort(rr);
    cutPoint1 = floor(numel(s)* 0.50);
    cutPoint2 = ceil(numel(s)*0.90);
    cutRR = s(cutPoint1:cutPoint2);
    effectiveStrandRadiusList(strandIter) = mean(cutRR);    
    originalVoltageDropList(strandIter) = dv;
    gds.Vertices.AllRadii(stei) = effectiveStrandRadiusList(strandIter);
    %---
    gds.Strands(strandIter).OriginalConductance = gds.Strands(strandIter).strandConductance;
    gds.Strands(strandIter).OriginalCurrent = gds.Strands(strandIter).strandCurrent;
    gds.Strands(strandIter).MedianVoltage = medianVoltage;
end
gds.effectiveStrandRadiusList = effectiveStrandRadiusList;
gds.originalStrandRadiusList = effectiveStrandRadiusList;
gds.originalVoltageDropList = originalVoltageDropList;

clear stei rr s cutPoint1 cutPoint2 cutRR vv dv originalVoltageDropList medianVoltage;


%%


flowProject.vectorizedStructure = gds;

 disp('Calculating unperturbed flow AGAIN');
    flowProject = CalculateUnperturbedFlow002(flowProject);
    flowProject.progressDates.unperturbedFlow = datestr(now);
    originalVoltages = flowProject.vectorizedStructure.Vertices.AllVoltages;
    flowProject.vectorizedStructure.Vertices.OriginalVoltages = originalVoltages;
    disp('Saving flowProject...');
    save(flowProjectFileName,'flowProject');
    clear originalVoltages


gds = flowProject.vectorizedStructure;
    
    gds.Vertices.OriginalRadii = gds.Vertices.AllRadii;

originalVoltageDropList = zeros(numStrands,1);
numStrands = numel(gds.Strands);
for strandIter = 1:numStrands,
    stei = gds.Strands(strandIter).StartToEndIndices;
    rr = gds.Vertices.AllRadii(stei);
    vv = gds.Vertices.OriginalVoltages(stei);
    medianVoltage = median(vv);
    dv = abs(max(vv)-min(vv));
    originalVoltageDropList(strandIter) = dv;
    %---
    gds.Strands(strandIter).OriginalConductance = gds.Strands(strandIter).strandConductance;
    gds.Strands(strandIter).OriginalCurrent = gds.Strands(strandIter).strandCurrent;
    gds.Strands(strandIter).MedianVoltage = medianVoltage;
end
gds.effectiveStrandRadiusList = effectiveStrandRadiusList;
gds.originalStrandRadiusList = effectiveStrandRadiusList;
gds.originalVoltageDropList = originalVoltageDropList;
    
    
    
    
%%
%Dilate a single plunging arteriole

dilationFraction = 1.1;
plungingVesselChoice = 8;

disp('Dilating single plunging arteriole');
choiceVesselID = flowProject.plungingArterioleTopStrandList(plungingVesselChoice)
choiceVerts = gds.Strands(choiceVesselID).StartToEndIndices;
startRadius = gds.effectiveStrandRadiusList(choiceVesselID);
dilatedRadius = startRadius.*dilationFraction;
gds.Vertices.AllRadii(choiceVerts) = dilatedRadius;
gds.effectiveStrandRadiusList(choiceVesselID) = dilatedRadius;
gds.plungingVesselChoice = plungingVesselChoice;
gds.plungingVesselID = choiceVesselID;
gds.dilationFraction = dilationFraction;

clear startRadius dilatedRadius choiceVerts choiceVesselID dilationFraction plungingVesselChoice



%%
%Rerun Flow Calculation

flowProject.vectorizedStructure = gds;

disp('Calculating perturbed flow - Initialization Round');
flowProject = CalculateUnperturbedFlow002(flowProject);
flowProject.progressDates.perturbedFlow = datestr(now);
disp('Saving flowProject...');
save(flowProjectFileName,'flowProject');

gds = flowProject.vectorizedStructure;

plungingCurrentRatio = gds.Strands(gds.plungingVesselID).strandCurrent ./ gds.Strands(gds.plungingVesselID).OriginalCurrent;
disp(['OneChangePlungingCurrentRatio = ',num2str(plungingCurrentRatio)]);


%%
%Adjust the radii at each vessel according to change in pressures

disp('Adjust the radii at each vessel according to change in pressures');
for strandIter = 1:numStrands,
    stei = gds.Strands(strandIter).StartToEndIndices;
    vv = gds.Vertices.AllVoltages(stei);
    dv = abs(max(vv)-min(vv));
    ov = gds.originalVoltageDropList(strandIter);
    origRadius = gds.originalStrandRadiusList(strandIter);
    voltageRatio = dv/ov;
    voltageRatioList(strandIter) = voltageRatio;
    passiveDilationRange = 0.2;
    
    
    

% %     x = [0.01:0.1:10];
% %     y = 1 + (((0.5.^(1./(x.^8)))*2 - 1) * passiveDilationRange)
% %     figure(1);plot(x,y,'r.');

    changeFraction = 1 + (((0.5.^(1/(voltageRatio.^7)))*2-1) * passiveDilationRange);
    
    adjRadius = origRadius.*changeFraction ^0.25;
    
% % %     if ~ismember(strandIter,subList),
% % %         adjRadius = origRadius;
% % %     end
    
    effectiveStrandRadiusList(strandIter) = adjRadius;    
    gds.Vertices.AllRadii(stei) = adjRadius;
end
gds.effectiveStrandRadiusList = effectiveStrandRadiusList;
gds.passiveDilationRange = passiveDilationRange;
clear stei vv dv ov origRadius voltageRatio passiveDilationRange changeFraction adjRadius effectiveStrandRadius;

%force plunging tip to original dilation
gds.effectiveStrandRadiusList(gds.plungingVesselID) = gds.originalStrandRadiusList(gds.plungingVesselID).*gds.dilationFraction;
plungingTipVerts = gds.Strands(gds.plungingVesselID).StartToEndIndices;
startRadius = median(gds.Vertices.OriginalRadii(plungingTipVerts));
dilatedRadius = startRadius.*gds.dilationFraction;
gds.Vertices.AllRadii(plungingTipVerts) = dilatedRadius;
clear plungingTipVerts startRadius dilatedRadius

%Rerun Flow Calculation

flowProject.vectorizedStructure = gds;

disp('Calculating perturbed flow - Iteration Round');
flowProject = CalculateUnperturbedFlow002(flowProject);
flowProject.progressDates.perturbedFlow = datestr(now);
disp('Saving flowProject...');
save(flowProjectFileName,'flowProject');
gds = flowProject.vectorizedStructure;


plungingCurrentRatio = gds.Strands(gds.plungingVesselID).strandCurrent ./ gds.Strands(gds.plungingVesselID).OriginalCurrent;
disp(['AllowBalloonPlungingCurrentRatio = ',num2str(plungingCurrentRatio)]);


