function vectorProject = CalculateSingleSourceFlow(vectorProject)


    %Stage 2: Run Circuit Analysis on the original structure.
    %Stage 2a: Set up parameters
    if ~isfield(vectorProject,'circuitAnalysisParameters'),
        vectorProject.circuitAnalysisParameters = [];
        vectorProject = SetupCircuitParameters(vectorProject);
    else%if ~isfield(vectorProject,'circuitAnalysisParameters'),\
        vectorProject.singleInjectionAnalysisParameters = vectorProject.circuitAnalysisParameters;
    end%if ~isfield(vectorProject,'circuitAnalysisParameters'),


   
        %Stage 2b: Set up node structure;
            vectorProject = FindCircuitNodes(vectorProject);


    %Stage 2c: Calculate strand conductances;
        vectorProject = CalculateCircuitConductances(vectorProject);



clotStrandID = vectorProject.circuitAnalysisParameters.clotStrandID;
if clotStrandID(1) ~= 0,
    for k = 1:numel(clotStrandID),
        disp(['Clotting Strand #',num2str(clotStrandID(k))])       
        vectorProject.circuitAnalysisParameters.strandConductanceList(clotStrandID(k)) = vectorProject.circuitAnalysisParameters.noFlowConductance;
        vectorProject.circuitAnalysisParameters.strandRadiusList(clotStrandID(k)) = 0.1;%Hardcode
    end%for k...
end%if clotStrandID(1) ~= 0,
vectorProject.circuitAnalysisParameters.originalConductanceList = vectorProject.circuitAnalysisParameters.strandConductanceList;


%Stage 2d: Calculate Gmatrix,
    vectorProject = CalculateCircuitGMatrix(vectorProject);
vectorProject.circuitAnalysisParameters.roundCounter = 0;    



    



%%

function vectorProject = SetupCircuitParameters(vectorProject)
    disp('Setting up circuit parameters');
    circuitAnalysisParameters = vectorProject.circuitAnalysisParameters;
    %-----
    circuitAnalysisParameters.onlyFreeEndPointsAreFixedNodesFlag = 1;%Hardcode
    circuitAnalysisParameters.fixedALabelList = [];%Hardcode : labels to be considerred voltages voltages
    circuitAnalysisParameters.fixedBLabelList = [330:349,340:349];%Hardcode : labels to be considered ground
    circuitAnalysisParameters.fixedAVoltage = 100;%Hardcode: potential at voltage voltage connection
    circuitAnalysisParameters.fixedBVoltage = 50;%Hardcode: potential at ground connection
    circuitAnalysisParameters.conductanceMultiplier = 1e9;%Hardcode
    circuitAnalysisParameters.variableCleanupFlag = 0;
    %---
    circuitAnalysisParameters.radiusPower = 4;%Hardcode
    circuitAnalysisParameters.minRadius = 0.25;%Hardcode
    circuitAnalysisParameters.minStrandLength = 4;%Hardcode
    circuitAnalysisParameters.hematocrit = 0.45;%Hardcode
    circuitAnalysisParameters.radiusAddition = 0;%Hardcode (this shifts the conductance curve to account for the smaller radii in mice)
    %%%%%
    circuitAnalysisParameters.minimumCurrent = 0;%Hardcode for 0 stalls
    circuitAnalysisParameters.noFlowConductance = 1e-1;%Hardcode
    circuitAnalysisParameters.clotStrandID = [0];%Hardcode, set to zero for no clot
    %-----
    vectorProject.singleInjectionAnalysisParameters = circuitAnalysisParameters;
return



%%
function vectorProject = FindCircuitNodes(vectorProject)
    disp(['Finding Circuit Nodes : ',datestr(now)]);
    vectorizedStructure = vectorProject.vectorizedStructure;
    singleInjectionAnalysisParameters = vectorProject.singleInjectionAnalysisParameters;
    singleInjectionStrandID = singleInjectionAnalysisParameters.singleInjectionStrandID;
    maskSize= vectorizedStructure.maskSize;
    Vertices = vectorizedStructure.Vertices;
    Strands = vectorizedStructure.Strands;
    numStrands = numel(Strands);

    freeEndPointFlagList = Vertices.FreeEndPoints;
    junctionPointFlagList = Vertices.JunctionPoints;
    AllVerts = Vertices.AllVerts;
    AllRadii = Vertices.AllRadii;
    AllLabels = Vertices.AllLabels;
    StrandIndices = Vertices.StrandIndices;
    
    onlyFreeEndPointsAreFixedNodesFlag = singleInjectionAnalysisParameters.onlyFreeEndPointsAreFixedNodesFlag;
    fixedALabelList = singleInjectionAnalysisParameters.fixedALabelList;
    fixedBLabelList = singleInjectionAnalysisParameters.fixedBLabelList;
    fixedAVoltage = singleInjectionAnalysisParameters.fixedAVoltage;
    fixedBVoltage = singleInjectionAnalysisParameters.fixedBVoltage;
    conductanceMultiplier = singleInjectionAnalysisParameters.conductanceMultiplier;
    variableCleanupFlag = singleInjectionAnalysisParameters.variableCleanupFlag;
    %---
    radiusPower = singleInjectionAnalysisParameters.radiusPower;
    minRadius = singleInjectionAnalysisParameters.minRadius;
    minStrandLength = singleInjectionAnalysisParameters.minStrandLength;
    hematocrit = singleInjectionAnalysisParameters.hematocrit;
    radiusAddition = singleInjectionAnalysisParameters.radiusAddition;
 
    %-----
    %Find the singleInjection strand, set voltage to VoltageA
    %Set upstream conductances to ~infinity
    injectionStrand = Strands(singleInjectionStrandID);
    if injectionStrand.StartVoltage > injectionStrand.EndVoltage,
        sourceNodeVertexIndex = injectionStrand.StartVertexIndex;
        highConductanceStrandList = injectionStrand.StartVertexNeighborStrands;
    else%if injectionStrand.StartVoltage > injectionStrand.EndVoltage,
        sourceNodeVertexIndex = injectionStrand.EndVertexIndex;
        highConductanceStrandList = injectionStrand.EndVertexNeighborStrands;
    end%if-else injectionStrand.StartVoltage > injectionStrand.EndVoltage,
    highConductanceStrandList = highConductanceStrandList(highConductanceStrandList~=singleInjectionStrandID)
    
    
    %Floating nodes are junctions or freeEndPoints that are not connected to a fixed voltage
    %Fixed nodes are junctions or freeEndPoints that ARE held at a fixed voltage
    %Allow multiple fixed voltages: fixedVoltageA, fixedVoltageB, etc.

    %First find all the nodes (fixed or floating)
    allNodeFlagList = or(freeEndPointFlagList,junctionPointFlagList);% a node is a junction or a free end point
    allNodeIndexList = find(allNodeFlagList);
    freeEndPointIndexList = find(freeEndPointFlagList);
    junctionPointIndexList = find(junctionPointFlagList);
    numNodes = sum(allNodeFlagList);
    numFreeEndPoints = sum(freeEndPointFlagList);
    numJunctionPoints = sum(junctionPointFlagList);
    vertexToAllNodeLUT = zeros(size(AllRadii));
    for nodeIter = 1:numNodes,
        currentNodeIndex = allNodeIndexList(nodeIter);
        vertexToAllNodeLUT(currentNodeIndex) = nodeIter;
    end

    %Go to each free end point and force its label to be the majority label of its strand
    for fepIter = freeEndPointIndexList',
        oldLabel = AllLabels(fepIter);
        fepStrand = StrandIndices(fepIter,:);
        fepStrand = fepStrand(fepStrand~=0);
        fepStrandVerts = Strands(fepStrand).InteriorVertices;
        majorityLabel = median(AllLabels(fepStrandVerts));
        AllLabels(fepIter) = majorityLabel;
    end


    %Now make a list of : nodes that are at fixedVolatageA, fixedVoltageB, etc
    %   the remainder nodes are floating nodes
    %   Note: junction points do not have an unambiguous label - only strands do.
    %       therefore, a junction point will be considered a fixedVoltage node if
    %       any one of its attached strands has the appropriate label
    %If onlyFreeEndPointsAreFixedNodesFlag is set on, then all junction points are floating
    if onlyFreeEndPointsAreFixedNodesFlag == 1,
        freeEndPointLabelList = AllLabels(freeEndPointIndexList);
% %         fixedANodeIndexList = freeEndPointIndexList(ismember(freeEndPointLabelList,fixedALabelList));
        fixedANodeIndexList = sourceNodeVertexIndex;
        fixedBNodeIndexList = freeEndPointIndexList(ismember(freeEndPointLabelList,fixedBLabelList));
        floatingNodeFlagList = allNodeFlagList;
        floatingNodeFlagList(fixedANodeIndexList) = 0;
        floatingNodeFlagList(fixedBNodeIndexList) = 0;
        floatingNodeIndexList = find(floatingNodeFlagList);
        numFloatingNodes = numel(floatingNodeIndexList);
        numFixedANodes = numel(fixedANodeIndexList);
        numFixedBNodes = numel(fixedBNodeIndexList);
        for nodeIter = 1:numFloatingNodes,
            currentNodeIndex = floatingNodeIndexList(nodeIter);
            vertexToFloatingNodeLUT(currentNodeIndex) = nodeIter;
        end
        %---
        for nodeIter = 1:numFixedANodes,
            currentNodeIndex = fixedANodeIndexList(nodeIter);
            vertexToFixedANodeLUT(currentNodeIndex) = nodeIter;
        end
        %---
        for nodeIter = 1:numFixedBNodes,
            currentNodeIndex = fixedBNodeIndexList(nodeIter);
            vertexToFixedBNodeLUT(currentNodeIndex) = nodeIter;
        end

    else%if onlyFreeEndPointsAreFixedNodesFlag == 1,
        %NOT YET CODED
        %NOT YET CODED
        %NOT YET CODED
        %NOT YET CODED
        %NOT YET CODED
        %NOT YET CODED
        %NOT YET CODED
        keyboard
    end%if onlyFreeEndPointsAreFixedNodesFlag == 1,
    
    singleInjectionAnalysisParameters.allNodeFlagList = allNodeFlagList;
    singleInjectionAnalysisParameters.allNodeIndexList = allNodeIndexList;
    singleInjectionAnalysisParameters.freeEndPointIndexList = freeEndPointIndexList;
    singleInjectionAnalysisParameters.junctionPointIndexList = junctionPointIndexList;
    singleInjectionAnalysisParameters.numNodes = numNodes;
    singleInjectionAnalysisParameters.numFreeEndPoints = numFreeEndPoints;
    singleInjectionAnalysisParameters.numJunctionPoints = numJunctionPoints;
    singleInjectionAnalysisParameters.vertexToAllNodeLUT = vertexToAllNodeLUT;
    singleInjectionAnalysisParameters.AllLabels = AllLabels;
    singleInjectionAnalysisParameters.freeEndPointLabelList = freeEndPointLabelList;
    singleInjectionAnalysisParameters.fixedANodeIndexList = fixedANodeIndexList;
    singleInjectionAnalysisParameters.fixedBNodeIndexList = fixedBNodeIndexList;
    singleInjectionAnalysisParameters.floatingNodeFlagList = floatingNodeFlagList;
    singleInjectionAnalysisParameters.floatingNodeIndexList = floatingNodeIndexList;
    singleInjectionAnalysisParameters.numFloatingNodes = numFloatingNodes;
    singleInjectionAnalysisParameters.numFixedANodes = numFixedANodes;
    singleInjectionAnalysisParameters.numFixedBNodes = numFixedBNodes;
    singleInjectionAnalysisParameters.vertexToFloatingNodeLUT = vertexToFloatingNodeLUT;
    singleInjectionAnalysisParameters.vertexToFixedANodeLUT = vertexToFixedANodeLUT;
    singleInjectionAnalysisParameters.vertexToFixedBNodeLUT = vertexToFixedBNodeLUT;
    singleInjectionAnalysisParameters.sourceNodeVertexIndex = sourceNodeVertexIndex;
    singleInjectionAnalysisParameters.highConductanceStrandList = highConductanceStrandList;
  
    vectorProject.singleInjectionAnalysisParameters = singleInjectionAnalysisParameters;
    
    
return

%%
function vectorProject = CalculateCircuitConductances(vectorProject)
    disp(['Calculating Conductances : ',datestr(now)]);
    vectorizedStructure = vectorProject.vectorizedStructure;
    singleInjectionAnalysisParameters = vectorProject.singleInjectionAnalysisParameters;
    effectiveStrandRadiusList = vectorizedStructure.effectiveStrandRadiusList;
    strandLengthList = vectorizedStructure.strandLengthList;
    maskSize= vectorizedStructure.maskSize;
    Vertices = vectorizedStructure.Vertices;
    Strands = vectorizedStructure.Strands;
    numStrands = numel(Strands);

    freeEndPointFlagList = Vertices.FreeEndPoints;
    junctionPointFlagList = Vertices.JunctionPoints;
    AllVerts = Vertices.AllVerts;
    AllRadii = Vertices.AllRadii;
    AllLabels = Vertices.AllLabels;
    StrandIndices = Vertices.StrandIndices;
    
    onlyFreeEndPointsAreFixedNodesFlag = singleInjectionAnalysisParameters.onlyFreeEndPointsAreFixedNodesFlag;
    fixedALabelList = singleInjectionAnalysisParameters.fixedALabelList;
    fixedBLabelList = singleInjectionAnalysisParameters.fixedBLabelList;
    fixedAVoltage = singleInjectionAnalysisParameters.fixedAVoltage;
    fixedBVoltage = singleInjectionAnalysisParameters.fixedBVoltage;
    conductanceMultiplier = singleInjectionAnalysisParameters.conductanceMultiplier;
    variableCleanupFlag = singleInjectionAnalysisParameters.variableCleanupFlag;
    %---
    radiusPower = singleInjectionAnalysisParameters.radiusPower;
    minRadius = singleInjectionAnalysisParameters.minRadius;
    minStrandLength = singleInjectionAnalysisParameters.minStrandLength;
    hematocrit = singleInjectionAnalysisParameters.hematocrit;
    radiusAddition = singleInjectionAnalysisParameters.radiusAddition;
    %---
    allNodeFlagList = singleInjectionAnalysisParameters.allNodeFlagList;
    allNodeIndexList = singleInjectionAnalysisParameters.allNodeIndexList;
    freeEndPointIndexList = singleInjectionAnalysisParameters.freeEndPointIndexList;
    junctionPointIndexList = singleInjectionAnalysisParameters.junctionPointIndexList;
    numNodes = singleInjectionAnalysisParameters.numNodes;
    numFreeEndPoints = singleInjectionAnalysisParameters.numFreeEndPoints;
    numJunctionPoints = singleInjectionAnalysisParameters.numJunctionPoints;
    vertexToAllNodeLUT = singleInjectionAnalysisParameters.vertexToAllNodeLUT;
    AllLabels = singleInjectionAnalysisParameters.AllLabels;
    freeEndPointLabelList = singleInjectionAnalysisParameters.freeEndPointLabelList;
    fixedANodeIndexList = singleInjectionAnalysisParameters.fixedANodeIndexList;
    fixedBNodeIndexList = singleInjectionAnalysisParameters.fixedBNodeIndexList;
    floatingNodeFlagList = singleInjectionAnalysisParameters.floatingNodeFlagList;
    floatingNodeIndexList = singleInjectionAnalysisParameters.floatingNodeIndexList;
    numFloatingNodes = singleInjectionAnalysisParameters.numFloatingNodes;
    numFixedANodes = singleInjectionAnalysisParameters.numFixedANodes;
    numFixedBNodes = singleInjectionAnalysisParameters.numFixedBNodes;
    vertexToFloatingNodeLUT = singleInjectionAnalysisParameters.vertexToFloatingNodeLUT;
    vertexToFixedANodeLUT = singleInjectionAnalysisParameters.vertexToFixedANodeLUT;
    vertexToFixedBNodeLUT = singleInjectionAnalysisParameters.vertexToFixedBNodeLUT;
    %-----
    

    for kk = 1:numStrands,
        thisStrand = Strands(kk);
        iv = thisStrand.InteriorVertices;
        thisStrandRadius = AllRadii(iv);
        strandRadiusList(1,kk) = median(thisStrandRadius(thisStrandRadius~=0));
    end
    
    
    strandConductanceList = [];
    for strandIter = 1:numStrands,
        currentStrandID = strandIter;
        currentStrand = Strands(currentStrandID);
        strandLength = strandLengthList(currentStrandID);
        strandLength = max(strandLength,minStrandLength);
        
        thisStrandRadius = effectiveStrandRadiusList(currentStrandID);
        thisStrandRadius = max(thisStrandRadius,minRadius);
        %HARDCODE
        D = (thisStrandRadius+radiusAddition)*2;%HARDCODE
D = max(0.5,D);

        %HARDCODE
        alpha = 4./(1+exp(-0.593*(D-6.74)));    
        viscosity = 1 + ((exp(hematocrit*alpha)-1) .* (110*exp(-1.424*D) + 3 - 3.45*exp(-0.035*D)) ./ (exp(0.45*alpha)-1));
        strandLength = max(minStrandLength,strandLength);
        strandConductanceList(currentStrandID) = conductanceMultiplier * (D/2).^radiusPower / (viscosity.*strandLength);
        conductanceFormula = 'D = (thisStrandRadius+radiusAddition)*2 ; strandConductance = conductanceMultiplier*((D/2)^rp)/(v*strandLength)  ;  1 + ((exp(hematocrit*alpha)-1) .* (110*exp(-1.424*D) + 3 - 3.45*exp(-0.035*D)) ./ (exp(0.45*alpha)-1)) ;  alpha = 4./(1+exp(-0.593*(D-6.74)))  ;    D = thisStrandRadius*2';
    end%for strandIter = 1:numStrands,
    
    highConductance = max(strandConductanceList)/1e9;
    
    strandConductanceList(singleInjectionAnalysisParameters.highConductanceStrandList) = highConductance;
  
    
    singleInjectionAnalysisParameters.strandConductanceList = strandConductanceList;
    singleInjectionAnalysisParameters.conductanceFormula = conductanceFormula;
    singleInjectionAnalysisParameters.strandRadiusList = strandRadiusList;

    vectorProject.singleInjectionAnalysisParameters = singleInjectionAnalysisParameters;
    
    
return


%%
function vectorProject = CalculateCircuitGMatrix(vectorProject)
    disp(['Populating GMatrix : ',datestr(now)]);
    vectorizedStructure = vectorProject.vectorizedStructure;
    singleInjectionAnalysisParameters = vectorProject.singleInjectionAnalysisParameters;
    maskSize= vectorizedStructure.maskSize;
    Vertices = vectorizedStructure.Vertices;
    Strands = vectorizedStructure.Strands;
    numStrands = numel(Strands);

    freeEndPointFlagList = Vertices.FreeEndPoints;
    junctionPointFlagList = Vertices.JunctionPoints;
    AllVerts = Vertices.AllVerts;
    AllRadii = Vertices.AllRadii;
    AllLabels = Vertices.AllLabels;
    StrandIndices = Vertices.StrandIndices;
    
    onlyFreeEndPointsAreFixedNodesFlag = singleInjectionAnalysisParameters.onlyFreeEndPointsAreFixedNodesFlag;
    fixedALabelList = singleInjectionAnalysisParameters.fixedALabelList;
    fixedBLabelList = singleInjectionAnalysisParameters.fixedBLabelList;
    fixedAVoltage = singleInjectionAnalysisParameters.fixedAVoltage;
    fixedBVoltage = singleInjectionAnalysisParameters.fixedBVoltage;
    conductanceMultiplier = singleInjectionAnalysisParameters.conductanceMultiplier;
    variableCleanupFlag = singleInjectionAnalysisParameters.variableCleanupFlag;
    %---
    radiusPower = singleInjectionAnalysisParameters.radiusPower;
    minRadius = singleInjectionAnalysisParameters.minRadius;
    minStrandLength = singleInjectionAnalysisParameters.minStrandLength;
    hematocrit = singleInjectionAnalysisParameters.hematocrit;
    radiusAddition = singleInjectionAnalysisParameters.radiusAddition;
    %---
    allNodeFlagList = singleInjectionAnalysisParameters.allNodeFlagList;
    allNodeIndexList = singleInjectionAnalysisParameters.allNodeIndexList;
    freeEndPointIndexList = singleInjectionAnalysisParameters.freeEndPointIndexList;
    junctionPointIndexList = singleInjectionAnalysisParameters.junctionPointIndexList;
    numNodes = singleInjectionAnalysisParameters.numNodes;
    numFreeEndPoints = singleInjectionAnalysisParameters.numFreeEndPoints;
    numJunctionPoints = singleInjectionAnalysisParameters.numJunctionPoints;
    vertexToAllNodeLUT = singleInjectionAnalysisParameters.vertexToAllNodeLUT;
    AllLabels = singleInjectionAnalysisParameters.AllLabels;
    freeEndPointLabelList = singleInjectionAnalysisParameters.freeEndPointLabelList;
    fixedANodeIndexList = singleInjectionAnalysisParameters.fixedANodeIndexList;
    fixedBNodeIndexList = singleInjectionAnalysisParameters.fixedBNodeIndexList;
    floatingNodeFlagList = singleInjectionAnalysisParameters.floatingNodeFlagList;
    floatingNodeIndexList = singleInjectionAnalysisParameters.floatingNodeIndexList;
    numFloatingNodes = singleInjectionAnalysisParameters.numFloatingNodes;
    numFixedANodes = singleInjectionAnalysisParameters.numFixedANodes;
    numFixedBNodes = singleInjectionAnalysisParameters.numFixedBNodes;
    vertexToFloatingNodeLUT = singleInjectionAnalysisParameters.vertexToFloatingNodeLUT;
    vertexToFixedANodeLUT = singleInjectionAnalysisParameters.vertexToFixedANodeLUT;
    vertexToFixedBNodeLUT = singleInjectionAnalysisParameters.vertexToFixedBNodeLUT;
    %---
    strandConductanceList = singleInjectionAnalysisParameters.strandConductanceList;
    strandRadiusList = singleInjectionAnalysisParameters.strandRadiusList;
    %-----
    

    vectorProject.singleInjectionAnalysisParameters = singleInjectionAnalysisParameters;

%Populate the conductance matrix
%GMatrix(ij) = -1*g(ij) where g(ij) is the conductance between node i and node j
%GMatrix(ii) = sum[g(ij)] across a row
%Equation: V = Gmatrix * ForcingCurrent
GMatrix = sparse(zeros(numFloatingNodes,numFloatingNodes));
for floatingNodeIter = 1:numFloatingNodes,
    currentNodeIndex = floatingNodeIndexList(floatingNodeIter);
    strandsAttached = StrandIndices(currentNodeIndex,:);
    strandsAttached = strandsAttached(strandsAttached~=0);
    numStrandsAttached = numel(strandsAttached);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentStrand = Strands(currentStrandID);
        strandConductance = strandConductanceList(currentStrandID);
        attachedNodeIndexList = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
        otherNodeIndex = attachedNodeIndexList(attachedNodeIndexList~=currentNodeIndex);
        if ismember(otherNodeIndex,floatingNodeIndexList),
            otherFloatingNodeIndex = vertexToFloatingNodeLUT(otherNodeIndex);
            GMatrix(floatingNodeIter,otherFloatingNodeIndex) =  -1 *  strandConductance;
            GMatrix(otherFloatingNodeIndex,floatingNodeIter) =  -1 *  strandConductance;
        end%if ismember(otherNodeIndex,floatingNodeIndexList),
    end%for strandIter = 1:numStrandsAttached,
end%for floatingNodeIter = 1:numFreeNodes,

            
for floatingNodeIter = 1:numFloatingNodes,
    GMatrix(floatingNodeIter,floatingNodeIter) = -1 * sum(GMatrix(floatingNodeIter,:));
end


forcingVector = zeros(numFloatingNodes,1);
%---
for fixedNodeIter = 1:numFixedANodes,
    currentNodeIndex = fixedANodeIndexList(fixedNodeIter);
    strandsAttached = StrandIndices(currentNodeIndex,:);
    strandsAttached = strandsAttached(strandsAttached~=0);
    numStrandsAttached = numel(strandsAttached);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentStrand = Strands(currentStrandID);
        strandConductance = strandConductanceList(currentStrandID);
        attachedNodeIndexList = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
        otherNodeIndex = attachedNodeIndexList(attachedNodeIndexList~=currentNodeIndex);
        if ismember(otherNodeIndex,floatingNodeIndexList),
            floatingNodeIndex = vertexToFloatingNodeLUT(otherNodeIndex);
            GMatrix(floatingNodeIndex,floatingNodeIndex) = GMatrix(floatingNodeIndex,floatingNodeIndex) + strandConductance;
            forcingVector(floatingNodeIndex) = forcingVector(floatingNodeIndex) + fixedAVoltage.*strandConductance;
        end%ismember(otherNodeIndex,floatingNodeIndexList),
    end%for strandIter = 1:numStrandsAttached,
end%for fixedNodeIter = 1:numFixedANodes,

%---
for fixedNodeIter = 1:numFixedBNodes,
    currentNodeIndex = fixedBNodeIndexList(fixedNodeIter);
    strandsAttached = StrandIndices(currentNodeIndex,:);
    strandsAttached = strandsAttached(strandsAttached~=0);
    numStrandsAttached = numel(strandsAttached);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentStrand = Strands(currentStrandID);
        strandConductance = strandConductanceList(currentStrandID);
        attachedNodeIndexList = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
        otherNodeIndex = attachedNodeIndexList(attachedNodeIndexList~=currentNodeIndex);
        if ismember(otherNodeIndex,floatingNodeIndexList),
            floatingNodeIndex = vertexToFloatingNodeLUT(otherNodeIndex);
            GMatrix(floatingNodeIndex,floatingNodeIndex) = GMatrix(floatingNodeIndex,floatingNodeIndex) + strandConductance;
            forcingVector(floatingNodeIndex) = forcingVector(floatingNodeIndex) + fixedBVoltage.*strandConductance;
        end%ismember(otherNodeIndex,floatingNodeIndexList),
    end%for strandIter = 1:numStrandsAttached,
end%for fixedNodeIter = 1:numFixedANodes,
  


disp(['Calculating Voltage Vector : ',datestr(now)]);
voltageVector = GMatrix\forcingVector;

figure(101);plot(voltageVector)
xlabel('Floating Node ID');
ylabel('Floating Node Voltage');
% % figure(2);plot(forcingVector)
% % figure(3);plot(GMatrix*voltageVector)

%Write results back to VectorizedStructure format
%For each strand, assign a current
%For each vertex, assign a voltage
disp(['Converting to Vectorized Structure Format']);
disp(['Assigning Node Voltages : ',datestr(now)]);
allNodeVoltageList = zeros(size(allNodeIndexList));
%Go node-by-node, and assign a voltage to each node
for nodeIter = 1:numNodes,
    thisNodeIndex = allNodeIndexList(nodeIter);
    %determine if this node is a floating node
    if ismember(thisNodeIndex,floatingNodeIndexList),
        %if so, the voltage is taken from the voltageVector
        thisFloatingNodeIndex = vertexToFloatingNodeLUT(thisNodeIndex);
        allNodeVoltageList(nodeIter) = voltageVector(thisFloatingNodeIndex);
    end%if ismember(thisNodeIndex,floatingNodeIndexList),
    %---
    %determine if this node is a fixed(A) node
    if ismember(thisNodeIndex,fixedANodeIndexList),
        %if so, assign it to the fixedVoltage(A)
        thisFixedNodeIndex = vertexToFixedANodeLUT(thisNodeIndex);
        allNodeVoltageList(nodeIter) = fixedAVoltage;
    end%if ismember(thisNodeIndex,fixedANodeIndexList),
    %---
    %determine if this node is a fixed(B) node
    if ismember(thisNodeIndex,fixedBNodeIndexList),
        %if so, assign it to the fixedVoltage(B)
        thisFixedNodeIndex = vertexToFixedBNodeLUT(thisNodeIndex);
        allNodeVoltageList(nodeIter) = fixedBVoltage;
    end%if ismember(thisNodeIndex,fixedBNodeIndexList),
end

disp(['Assigning Strand Currents and Vertex Voltages: ',datestr(now)]);
%Go strand-by-strand and assign currents
%   note that positive current means flow is from start-to-end index along strand
AllVoltages = zeros(size(AllRadii));
for strandIter = 1:numStrands,
    thisStrand = Strands(strandIter);
    thisStrandConductance = strandConductanceList(strandIter);
    node1VertexID = thisStrand.StartVertexIndex;
    node1Index = vertexToAllNodeLUT(node1VertexID);
    node1Voltage = allNodeVoltageList(node1Index);
    node2VertexID = thisStrand.EndVertexIndex;
    node2Index = vertexToAllNodeLUT(node2VertexID);
    node2Voltage = allNodeVoltageList(node2Index);
    deltaVoltage = node1Voltage-node2Voltage;
    thisStrandCurrent = deltaVoltage * thisStrandConductance;
    Strands(strandIter).strandConductance = thisStrandConductance;
    Strands(strandIter).strandCurrent = thisStrandCurrent;
    strandCurrentList(strandIter) = thisStrandCurrent;
    
    Strands(strandIter).MedianVoltage = (node1Voltage+node2Voltage)/2;
    Strands(strandIter).StartVoltage = node1Voltage;
    Strands(strandIter).EndVoltage = node2Voltage;
    Strands(strandIter).VoltageDrop = deltaVoltage;
    
    %---
    stei = thisStrand.StartToEndIndices;
    thisVoltageList = linspace(node1Voltage,node2Voltage,numel(stei));
    AllVoltages(stei) = thisVoltageList;
end
Vertices.AllVoltages = AllVoltages;
figure(112);semilogy(strandRadiusList,abs(strandCurrentList),'r.');xlabel('radius');ylabel('strandCurrent');

singleInjectionAnalysisParameters.strandCurrentList = strandCurrentList;
%-----
vectorizedStructure.Vertices = Vertices;
vectorizedStructure.Strands = Strands;
vectorProject.vectorizedStructure = vectorizedStructure;
vectorProject.singleInjectionAnalysisParameters = singleInjectionAnalysisParameters;

return

    


