function vectorProject = CalculateUnperturbed_AdjRadiusAV(vectorProject)

%Stage 1:  Load the original vectorProject.


    %Stage 2: Run Circuit Analysis on the original structure.
    %Stage 2a: Set up parameters
    if ~isfield(vectorProject,'circuitAnalysisParameters'),
        vectorProject.circuitAnalysisParameters = [];
    end%if ~isfield(vectorProject,'circuitAnalysisParameters'),
        vectorProject = SetupCircuitParameters(vectorProject);

        
if ~isfield(vectorProject.circuitAnalysisParameters,'vertexToFixedBNodeLUT'),        
    %Stage 2b: Set up node structure;
        vectorProject = FindCircuitNodes(vectorProject);
end%if ~isfield(vectorProject.circuitAnalysisParameters,'vertexToFixedBNodeLUT'), 
        


    %Stage 2c: Calculate strand conductances;
        vectorProject = CalculateCircuitConductances(vectorProject);



clotStrandID = vectorProject.circuitAnalysisParameters.clotStrandID;
if clotStrandID(1) ~= 0,
    for k = 1:numel(clotStrandID),
        disp(['Clotting Strand #',num2str(clotStrandID(k))])       
        vectorProject.circuitAnalysisParameters.strandConductanceList(clotStrandID(k)) = vectorProject.circuitAnalysisParameters.noFlowConductance;
        vectorProject.circuitAnalysisParameters.strandRadiusList(clotStrandID(k)) = 0.1;%Hardcode
    end%for k...
end%if clotStrandID(1) ~= 0,
vectorProject.circuitAnalysisParameters.originalConductanceList = vectorProject.circuitAnalysisParameters.strandConductanceList;


%Stage 2d: Calculate Gmatrix,
    vectorProject = CalculateCircuitGMatrix(vectorProject);
vectorProject.circuitAnalysisParameters.roundCounter = 0;    


% % 
% % %Stage 4: Make currentStoppage Mask
% % disp(['Generating Current Stoppage masks : ',datestr(now)]);
% % GenerateSingleRoundCurrentStoppageMaskRGB(vectorProject);
% % 

    



%%

function vectorProject = SetupCircuitParameters(vectorProject)
    disp('Setting up circuit parameters');
    circuitAnalysisParameters = vectorProject.circuitAnalysisParameters;
    %-----
    circuitAnalysisParameters.onlyFreeEndPointsAreFixedNodesFlag = 1;%Hardcode
    circuitAnalysisParameters.fixedALabelList = [330:339];%Hardcode : labels to be considerred voltages voltages
    circuitAnalysisParameters.fixedBLabelList = [340:349];%Hardcode : labels to be considered ground
    circuitAnalysisParameters.fixedAVoltage = 100;%Hardcode: potential at voltage voltage connection
    circuitAnalysisParameters.fixedBVoltage = 50;%Hardcode: potential at ground connection
    circuitAnalysisParameters.conductanceMultiplier = 1e9;%Hardcode
    circuitAnalysisParameters.variableCleanupFlag = 0;
    %---
    circuitAnalysisParameters.radiusPower = 4;%Hardcode
    circuitAnalysisParameters.minRadius = 0.25;%Hardcode
    circuitAnalysisParameters.minStrandLength = 4;%Hardcode
    circuitAnalysisParameters.hematocrit = 0.45;%Hardcode
    circuitAnalysisParameters.radiusAddition = 0;%Hardcode (this shifts the conductance curve to account for the smaller radii in mice)
    %%%%%
    circuitAnalysisParameters.minimumCurrent = 0;%Hardcode for 0 stalls
    circuitAnalysisParameters.noFlowConductance = 1e-1;%Hardcode
    circuitAnalysisParameters.clotStrandID = [0];%Hardcode, set to zero for no clot
    %-----
    vectorProject.circuitAnalysisParameters = circuitAnalysisParameters;
return



%%
function vectorProject = FindCircuitNodes(vectorProject)
    disp(['Finding Circuit Nodes : ',datestr(now)]);
    vectorizedStructure = vectorProject.vectorizedStructure;
    circuitAnalysisParameters = vectorProject.circuitAnalysisParameters;
    maskSize= vectorizedStructure.maskSize;
    Vertices = vectorizedStructure.Vertices;
    Strands = vectorizedStructure.Strands;
    numStrands = numel(Strands);

    freeEndPointFlagList = Vertices.FreeEndPoints;
    junctionPointFlagList = Vertices.JunctionPoints;
    AllVerts = Vertices.AllVerts;
    AllRadii = Vertices.AllRadii;
    AllLabels = Vertices.AllLabels;
    StrandIndices = Vertices.StrandIndices;
    
    onlyFreeEndPointsAreFixedNodesFlag = circuitAnalysisParameters.onlyFreeEndPointsAreFixedNodesFlag;
    fixedALabelList = circuitAnalysisParameters.fixedALabelList;
    fixedBLabelList = circuitAnalysisParameters.fixedBLabelList;
    fixedAVoltage = circuitAnalysisParameters.fixedAVoltage;
    fixedBVoltage = circuitAnalysisParameters.fixedBVoltage;
    conductanceMultiplier = circuitAnalysisParameters.conductanceMultiplier;
    variableCleanupFlag = circuitAnalysisParameters.variableCleanupFlag;
    %---
    radiusPower = circuitAnalysisParameters.radiusPower;
    minRadius = circuitAnalysisParameters.minRadius;
    minStrandLength = circuitAnalysisParameters.minStrandLength;
    hematocrit = circuitAnalysisParameters.hematocrit;
    radiusAddition = circuitAnalysisParameters.radiusAddition;
 
        
    circuitAnalysisParameters.stalledMatrix = zeros(numStrands,100);%Hardcode
    

    
    %Floating nodes are junctions or freeEndPoints that are not connected to a fixed voltage
    %Fixed nodes are junctions or freeEndPoints that ARE held at a fixed voltage
    %Allow multiple fixed voltages: fixedVoltageA, fixedVoltageB, etc.

    %First find all the nodes (fixed or floating)
    allNodeFlagList = or(freeEndPointFlagList,junctionPointFlagList);% a node is a junction or a free end point
    allNodeIndexList = find(allNodeFlagList);
    freeEndPointIndexList = find(freeEndPointFlagList);
    junctionPointIndexList = find(junctionPointFlagList);
    numNodes = sum(allNodeFlagList);
    numFreeEndPoints = sum(freeEndPointFlagList);
    numJunctionPoints = sum(junctionPointFlagList);
    vertexToAllNodeLUT = zeros(size(AllRadii));
    for nodeIter = 1:numNodes,
        currentNodeIndex = allNodeIndexList(nodeIter);
        vertexToAllNodeLUT(currentNodeIndex) = nodeIter;
    end

    %Go to each free end point and force its label to be the majority label of its strand
    for fepIter = freeEndPointIndexList',
        oldLabel = AllLabels(fepIter);
        fepStrand = StrandIndices(fepIter,:);
        fepStrand = fepStrand(fepStrand~=0);
        fepStrandVerts = Strands(fepStrand).InteriorVertices;
        majorityLabel = median(AllLabels(fepStrandVerts));
        AllLabels(fepIter) = majorityLabel;
    end


    %Now make a list of : nodes that are at fixedVolatageA, fixedVoltageB, etc
    %   the remainder nodes are floating nodes
    %   Note: junction points do not have an unambiguous label - only strands do.
    %       therefore, a junction point will be considered a fixedVoltage node if
    %       any one of its attached strands has the appropriate label
    %If onlyFreeEndPointsAreFixedNodesFlag is set on, then all junction points are floating
    if onlyFreeEndPointsAreFixedNodesFlag == 1,
        freeEndPointLabelList = AllLabels(freeEndPointIndexList);
        fixedANodeIndexList = freeEndPointIndexList(ismember(freeEndPointLabelList,fixedALabelList));
        fixedBNodeIndexList = freeEndPointIndexList(ismember(freeEndPointLabelList,fixedBLabelList));
        floatingNodeFlagList = allNodeFlagList;
        floatingNodeFlagList(fixedANodeIndexList) = 0;
        floatingNodeFlagList(fixedBNodeIndexList) = 0;
        floatingNodeIndexList = find(floatingNodeFlagList);
        numFloatingNodes = numel(floatingNodeIndexList);
        numFixedANodes = numel(fixedANodeIndexList);
        numFixedBNodes = numel(fixedBNodeIndexList);
        for nodeIter = 1:numFloatingNodes,
            currentNodeIndex = floatingNodeIndexList(nodeIter);
            vertexToFloatingNodeLUT(currentNodeIndex) = nodeIter;
        end
        %---
        for nodeIter = 1:numFixedANodes,
            currentNodeIndex = fixedANodeIndexList(nodeIter);
            vertexToFixedANodeLUT(currentNodeIndex) = nodeIter;
        end
        %---
        for nodeIter = 1:numFixedBNodes,
            currentNodeIndex = fixedBNodeIndexList(nodeIter);
            vertexToFixedBNodeLUT(currentNodeIndex) = nodeIter;
        end

    else%if onlyFreeEndPointsAreFixedNodesFlag == 1,
        %NOT YET CODED
        %NOT YET CODED
        %NOT YET CODED
        %NOT YET CODED
        %NOT YET CODED
        %NOT YET CODED
        %NOT YET CODED
        keyboard
    end%if onlyFreeEndPointsAreFixedNodesFlag == 1,
    
    circuitAnalysisParameters.allNodeFlagList = allNodeFlagList;
    circuitAnalysisParameters.allNodeIndexList = allNodeIndexList;
    circuitAnalysisParameters.freeEndPointIndexList = freeEndPointIndexList;
    circuitAnalysisParameters.junctionPointIndexList = junctionPointIndexList;
    circuitAnalysisParameters.numNodes = numNodes;
    circuitAnalysisParameters.numFreeEndPoints = numFreeEndPoints;
    circuitAnalysisParameters.numJunctionPoints = numJunctionPoints;
    circuitAnalysisParameters.vertexToAllNodeLUT = vertexToAllNodeLUT;
    circuitAnalysisParameters.AllLabels = AllLabels;
    circuitAnalysisParameters.freeEndPointLabelList = freeEndPointLabelList;
    circuitAnalysisParameters.fixedANodeIndexList = fixedANodeIndexList;
    circuitAnalysisParameters.fixedBNodeIndexList = fixedBNodeIndexList;
    circuitAnalysisParameters.floatingNodeFlagList = floatingNodeFlagList;
    circuitAnalysisParameters.floatingNodeIndexList = floatingNodeIndexList;
    circuitAnalysisParameters.numFloatingNodes = numFloatingNodes;
    circuitAnalysisParameters.numFixedANodes = numFixedANodes;
    circuitAnalysisParameters.numFixedBNodes = numFixedBNodes;
    circuitAnalysisParameters.vertexToFloatingNodeLUT = vertexToFloatingNodeLUT;
    circuitAnalysisParameters.vertexToFixedANodeLUT = vertexToFixedANodeLUT;
    circuitAnalysisParameters.vertexToFixedBNodeLUT = vertexToFixedBNodeLUT;
    
    vectorProject.circuitAnalysisParameters = circuitAnalysisParameters;
    
    
return

%%
function vectorProject = CalculateCircuitConductances(vectorProject)
    disp(['Calculating Conductances : ',datestr(now)]);
    vectorizedStructure = vectorProject.vectorizedStructure;
    circuitAnalysisParameters = vectorProject.circuitAnalysisParameters;
    maskSize= vectorizedStructure.maskSize;
    Vertices = vectorizedStructure.Vertices;
    Strands = vectorizedStructure.Strands;
    numStrands = numel(Strands);

    freeEndPointFlagList = Vertices.FreeEndPoints;
    junctionPointFlagList = Vertices.JunctionPoints;
    AllVerts = Vertices.AllVerts;
    AllRadii = Vertices.AllRadii;
    AllLabels = Vertices.AllLabels;
    StrandIndices = Vertices.StrandIndices;
    
    onlyFreeEndPointsAreFixedNodesFlag = circuitAnalysisParameters.onlyFreeEndPointsAreFixedNodesFlag;
    fixedALabelList = circuitAnalysisParameters.fixedALabelList;
    fixedBLabelList = circuitAnalysisParameters.fixedBLabelList;
    fixedAVoltage = circuitAnalysisParameters.fixedAVoltage;
    fixedBVoltage = circuitAnalysisParameters.fixedBVoltage;
    conductanceMultiplier = circuitAnalysisParameters.conductanceMultiplier;
    variableCleanupFlag = circuitAnalysisParameters.variableCleanupFlag;
    %---
    radiusPower = circuitAnalysisParameters.radiusPower;
    minRadius = circuitAnalysisParameters.minRadius;
    minStrandLength = circuitAnalysisParameters.minStrandLength;
    hematocrit = circuitAnalysisParameters.hematocrit;
    radiusAddition = circuitAnalysisParameters.radiusAddition;
    %---
    allNodeFlagList = circuitAnalysisParameters.allNodeFlagList;
    allNodeIndexList = circuitAnalysisParameters.allNodeIndexList;
    freeEndPointIndexList = circuitAnalysisParameters.freeEndPointIndexList;
    junctionPointIndexList = circuitAnalysisParameters.junctionPointIndexList;
    numNodes = circuitAnalysisParameters.numNodes;
    numFreeEndPoints = circuitAnalysisParameters.numFreeEndPoints;
    numJunctionPoints = circuitAnalysisParameters.numJunctionPoints;
    vertexToAllNodeLUT = circuitAnalysisParameters.vertexToAllNodeLUT;
    AllLabels = circuitAnalysisParameters.AllLabels;
    freeEndPointLabelList = circuitAnalysisParameters.freeEndPointLabelList;
    fixedANodeIndexList = circuitAnalysisParameters.fixedANodeIndexList;
    fixedBNodeIndexList = circuitAnalysisParameters.fixedBNodeIndexList;
    floatingNodeFlagList = circuitAnalysisParameters.floatingNodeFlagList;
    floatingNodeIndexList = circuitAnalysisParameters.floatingNodeIndexList;
    numFloatingNodes = circuitAnalysisParameters.numFloatingNodes;
    numFixedANodes = circuitAnalysisParameters.numFixedANodes;
    numFixedBNodes = circuitAnalysisParameters.numFixedBNodes;
    vertexToFloatingNodeLUT = circuitAnalysisParameters.vertexToFloatingNodeLUT;
    vertexToFixedANodeLUT = circuitAnalysisParameters.vertexToFixedANodeLUT;
    vertexToFixedBNodeLUT = circuitAnalysisParameters.vertexToFixedBNodeLUT;
    %-----
    

    for kk = 1:numStrands,
        thisStrand = Strands(kk);
        iv = thisStrand.InteriorVertices;
        thisStrandRadius = AllRadii(iv);
        strandRadiusList(1,kk) = median(thisStrandRadius(thisStrandRadius~=0));
    end
    
    
    strandConductanceList = [];
    for strandIter = 1:numStrands,
        currentStrandID = strandIter;
        currentStrand = Strands(currentStrandID);
        InteriorVertices = currentStrand.InteriorVertices;
        majorityLabel = median(AllLabels(InteriorVertices));
        strandLength = numel(InteriorVertices);
            strandLength = max(strandLength,minStrandLength);
        thisRadiusList = AllRadii(InteriorVertices);
            thisRadiusList = thisRadiusList(thisRadiusList~=0);
            thisRadiusList = max(thisRadiusList,minRadius);
        thisStrandRadius = median(thisRadiusList);

        
        %HARDCODE
        D = (thisStrandRadius+radiusAddition)*2;%HARDCODE
D = max(0.5,D);
D = ((thisStrandRadius+1).^0.91)*2; %HARDCODE - RADIUS ADJUSTMENT SPECIFICALLY FOR datasetAV HISTOGRAM
        %HARDCODE

        alpha = 4./(1+exp(-0.593*(D-6.74)));    
        viscosity = 1 + ((exp(hematocrit*alpha)-1) .* (110*exp(-1.424*D) + 3 - 3.45*exp(-0.035*D)) ./ (exp(0.45*alpha)-1));
        strandLength = max(minStrandLength,strandLength);
        strandConductanceList(currentStrandID) = conductanceMultiplier * (D/2).^radiusPower / (viscosity.*strandLength);
        conductanceFormula = 'D = (thisStrandRadius+radiusAddition)*2 ; strandConductance = conductanceMultiplier*((D/2)^rp)/(v*strandLength)  ;  1 + ((exp(hematocrit*alpha)-1) .* (110*exp(-1.424*D) + 3 - 3.45*exp(-0.035*D)) ./ (exp(0.45*alpha)-1)) ;  alpha = 4./(1+exp(-0.593*(D-6.74)))  ;    D = thisStrandRadius*2';
    end%for strandIter = 1:numStrands,
    
    circuitAnalysisParameters.strandConductanceList = strandConductanceList;
    circuitAnalysisParameters.conductanceFormula = conductanceFormula;
    circuitAnalysisParameters.strandRadiusList = strandRadiusList;

    vectorProject.circuitAnalysisParameters = circuitAnalysisParameters;
    
    
return


%%
function vectorProject = CalculateCircuitGMatrix(vectorProject)
    disp(['Populating GMatrix : ',datestr(now)]);
    vectorizedStructure = vectorProject.vectorizedStructure;
    circuitAnalysisParameters = vectorProject.circuitAnalysisParameters;
    maskSize= vectorizedStructure.maskSize;
    Vertices = vectorizedStructure.Vertices;
    Strands = vectorizedStructure.Strands;
    numStrands = numel(Strands);

    freeEndPointFlagList = Vertices.FreeEndPoints;
    junctionPointFlagList = Vertices.JunctionPoints;
    AllVerts = Vertices.AllVerts;
    AllRadii = Vertices.AllRadii;
    AllLabels = Vertices.AllLabels;
    StrandIndices = Vertices.StrandIndices;
    
    onlyFreeEndPointsAreFixedNodesFlag = circuitAnalysisParameters.onlyFreeEndPointsAreFixedNodesFlag;
    fixedALabelList = circuitAnalysisParameters.fixedALabelList;
    fixedBLabelList = circuitAnalysisParameters.fixedBLabelList;
    fixedAVoltage = circuitAnalysisParameters.fixedAVoltage;
    fixedBVoltage = circuitAnalysisParameters.fixedBVoltage;
    conductanceMultiplier = circuitAnalysisParameters.conductanceMultiplier;
    variableCleanupFlag = circuitAnalysisParameters.variableCleanupFlag;
    %---
    radiusPower = circuitAnalysisParameters.radiusPower;
    minRadius = circuitAnalysisParameters.minRadius;
    minStrandLength = circuitAnalysisParameters.minStrandLength;
    hematocrit = circuitAnalysisParameters.hematocrit;
    radiusAddition = circuitAnalysisParameters.radiusAddition;
    %---
    allNodeFlagList = circuitAnalysisParameters.allNodeFlagList;
    allNodeIndexList = circuitAnalysisParameters.allNodeIndexList;
    freeEndPointIndexList = circuitAnalysisParameters.freeEndPointIndexList;
    junctionPointIndexList = circuitAnalysisParameters.junctionPointIndexList;
    numNodes = circuitAnalysisParameters.numNodes;
    numFreeEndPoints = circuitAnalysisParameters.numFreeEndPoints;
    numJunctionPoints = circuitAnalysisParameters.numJunctionPoints;
    vertexToAllNodeLUT = circuitAnalysisParameters.vertexToAllNodeLUT;
    AllLabels = circuitAnalysisParameters.AllLabels;
    freeEndPointLabelList = circuitAnalysisParameters.freeEndPointLabelList;
    fixedANodeIndexList = circuitAnalysisParameters.fixedANodeIndexList;
    fixedBNodeIndexList = circuitAnalysisParameters.fixedBNodeIndexList;
    floatingNodeFlagList = circuitAnalysisParameters.floatingNodeFlagList;
    floatingNodeIndexList = circuitAnalysisParameters.floatingNodeIndexList;
    numFloatingNodes = circuitAnalysisParameters.numFloatingNodes;
    numFixedANodes = circuitAnalysisParameters.numFixedANodes;
    numFixedBNodes = circuitAnalysisParameters.numFixedBNodes;
    vertexToFloatingNodeLUT = circuitAnalysisParameters.vertexToFloatingNodeLUT;
    vertexToFixedANodeLUT = circuitAnalysisParameters.vertexToFixedANodeLUT;
    vertexToFixedBNodeLUT = circuitAnalysisParameters.vertexToFixedBNodeLUT;
    %---
    strandConductanceList = circuitAnalysisParameters.strandConductanceList;
    strandRadiusList = circuitAnalysisParameters.strandRadiusList;
    %-----
    

    vectorProject.circuitAnalysisParameters = circuitAnalysisParameters;

%Populate the conductance matrix
%GMatrix(ij) = -1*g(ij) where g(ij) is the conductance between node i and node j
%GMatrix(ii) = sum[g(ij)] across a row
%Equation: V = Gmatrix * ForcingCurrent
GMatrix = sparse(zeros(numFloatingNodes,numFloatingNodes));
for floatingNodeIter = 1:numFloatingNodes,
    currentNodeIndex = floatingNodeIndexList(floatingNodeIter);
    strandsAttached = StrandIndices(currentNodeIndex,:);
    strandsAttached = strandsAttached(strandsAttached~=0);
    numStrandsAttached = numel(strandsAttached);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentStrand = Strands(currentStrandID);
        strandConductance = strandConductanceList(currentStrandID);
        attachedNodeIndexList = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
        otherNodeIndex = attachedNodeIndexList(attachedNodeIndexList~=currentNodeIndex);
        if ismember(otherNodeIndex,floatingNodeIndexList),
            otherFloatingNodeIndex = vertexToFloatingNodeLUT(otherNodeIndex);
            GMatrix(floatingNodeIter,otherFloatingNodeIndex) =  -1 *  strandConductance;
            GMatrix(otherFloatingNodeIndex,floatingNodeIter) =  -1 *  strandConductance;
        end%if ismember(otherNodeIndex,floatingNodeIndexList),
    end%for strandIter = 1:numStrandsAttached,
end%for floatingNodeIter = 1:numFreeNodes,

            
for floatingNodeIter = 1:numFloatingNodes,
    GMatrix(floatingNodeIter,floatingNodeIter) = -1 * sum(GMatrix(floatingNodeIter,:));
end


forcingVector = zeros(numFloatingNodes,1);
%---
for fixedNodeIter = 1:numFixedANodes,
    currentNodeIndex = fixedANodeIndexList(fixedNodeIter);
    strandsAttached = StrandIndices(currentNodeIndex,:);
    strandsAttached = strandsAttached(strandsAttached~=0);
    numStrandsAttached = numel(strandsAttached);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentStrand = Strands(currentStrandID);
        strandConductance = strandConductanceList(currentStrandID);
        attachedNodeIndexList = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
        otherNodeIndex = attachedNodeIndexList(attachedNodeIndexList~=currentNodeIndex);
        if ismember(otherNodeIndex,floatingNodeIndexList),
            floatingNodeIndex = vertexToFloatingNodeLUT(otherNodeIndex);
            GMatrix(floatingNodeIndex,floatingNodeIndex) = GMatrix(floatingNodeIndex,floatingNodeIndex) + strandConductance;
            forcingVector(floatingNodeIndex) = forcingVector(floatingNodeIndex) + fixedAVoltage.*strandConductance;
        end%ismember(otherNodeIndex,floatingNodeIndexList),
    end%for strandIter = 1:numStrandsAttached,
end%for fixedNodeIter = 1:numFixedANodes,

%---
for fixedNodeIter = 1:numFixedBNodes,
    currentNodeIndex = fixedBNodeIndexList(fixedNodeIter);
    strandsAttached = StrandIndices(currentNodeIndex,:);
    strandsAttached = strandsAttached(strandsAttached~=0);
    numStrandsAttached = numel(strandsAttached);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentStrand = Strands(currentStrandID);
        strandConductance = strandConductanceList(currentStrandID);
        attachedNodeIndexList = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
        otherNodeIndex = attachedNodeIndexList(attachedNodeIndexList~=currentNodeIndex);
        if ismember(otherNodeIndex,floatingNodeIndexList),
            floatingNodeIndex = vertexToFloatingNodeLUT(otherNodeIndex);
            GMatrix(floatingNodeIndex,floatingNodeIndex) = GMatrix(floatingNodeIndex,floatingNodeIndex) + strandConductance;
            forcingVector(floatingNodeIndex) = forcingVector(floatingNodeIndex) + fixedBVoltage.*strandConductance;
        end%ismember(otherNodeIndex,floatingNodeIndexList),
    end%for strandIter = 1:numStrandsAttached,
end%for fixedNodeIter = 1:numFixedANodes,
  


disp(['Calculating Voltage Vector : ',datestr(now)]);
voltageVector = GMatrix\forcingVector;

figure(101);plot(voltageVector)
% % figure(2);plot(forcingVector)
% % figure(3);plot(GMatrix*voltageVector)

%Write results back to VectorizedStructure format
%For each strand, assign a current
%For each vertex, assign a voltage
disp(['Converting to Vectorized Structure Format']);
disp(['Assigning Node Voltages : ',datestr(now)]);
allNodeVoltageList = zeros(size(allNodeIndexList));
%Go node-by-node, and assign a voltage to each node
for nodeIter = 1:numNodes,
    thisNodeIndex = allNodeIndexList(nodeIter);
    %determine if this node is a floating node
    if ismember(thisNodeIndex,floatingNodeIndexList),
        %if so, the voltage is taken from the voltageVector
        thisFloatingNodeIndex = vertexToFloatingNodeLUT(thisNodeIndex);
        allNodeVoltageList(nodeIter) = voltageVector(thisFloatingNodeIndex);
    end%if ismember(thisNodeIndex,floatingNodeIndexList),
    %---
    %determine if this node is a fixed(A) node
    if ismember(thisNodeIndex,fixedANodeIndexList),
        %if so, assign it to the fixedVoltage(A)
        thisFixedNodeIndex = vertexToFixedANodeLUT(thisNodeIndex);
        allNodeVoltageList(nodeIter) = fixedAVoltage;
    end%if ismember(thisNodeIndex,fixedANodeIndexList),
    %---
    %determine if this node is a fixed(B) node
    if ismember(thisNodeIndex,fixedBNodeIndexList),
        %if so, assign it to the fixedVoltage(B)
        thisFixedNodeIndex = vertexToFixedBNodeLUT(thisNodeIndex);
        allNodeVoltageList(nodeIter) = fixedBVoltage;
    end%if ismember(thisNodeIndex,fixedBNodeIndexList),
end

disp(['Assigning Strand Currents and Vertex Voltages: ',datestr(now)]);
%Go strand-by-strand and assign currents
%   note that positive current means flow is from start-to-end index along strand
AllVoltages = zeros(size(AllRadii));
for strandIter = 1:numStrands,
    thisStrand = Strands(strandIter);
    thisStrandConductance = strandConductanceList(strandIter);
    node1VertexID = thisStrand.StartVertexIndex;
    node1Index = vertexToAllNodeLUT(node1VertexID);
    node1Voltage = allNodeVoltageList(node1Index);
    node2VertexID = thisStrand.EndVertexIndex;
    node2Index = vertexToAllNodeLUT(node2VertexID);
    node2Voltage = allNodeVoltageList(node2Index);
    deltaVoltage = node1Voltage-node2Voltage;
    thisStrandCurrent = deltaVoltage * thisStrandConductance;
    Strands(strandIter).strandConductance = thisStrandConductance;
    Strands(strandIter).strandCurrent = thisStrandCurrent;
    strandCurrentList(strandIter) = thisStrandCurrent;
    %---
    stei = thisStrand.StartToEndIndices;
    thisVoltageList = linspace(node1Voltage,node2Voltage,numel(stei));
    AllVoltages(stei) = thisVoltageList;
end
Vertices.AllVoltages = AllVoltages;
figure(112);semilogy(strandRadiusList,abs(strandCurrentList),'r.');xlabel('radius');ylabel('strandCurrent');

circuitAnalysisParameters.strandCurrentList = strandCurrentList;
%-----
vectorizedStructure.Vertices = Vertices;
vectorizedStructure.Strands = Strands;
vectorProject.vectorizedStructure = vectorizedStructure;
vectorProject.circuitAnalysisParameters = circuitAnalysisParameters;

return

    
%%
function vectorProject = StallCircuitStrands(vectorProject)
    disp(['Finding low flow vessels : ',datestr(now)]);
    vectorizedStructure = vectorProject.vectorizedStructure;
    circuitAnalysisParameters = vectorProject.circuitAnalysisParameters;
    Vertices = vectorizedStructure.Vertices;
    Strands = vectorizedStructure.Strands;
    numStrands = numel(Strands);
    strandCurrentList = circuitAnalysisParameters.strandCurrentList;
    originalConductanceList = circuitAnalysisParameters.originalConductanceList;
    strandConductanceList = circuitAnalysisParameters.strandConductanceList;
    stalledMatrix = circuitAnalysisParameters.stalledMatrix;
    roundCounter = circuitAnalysisParameters.roundCounter;
    minimumCurrent = circuitAnalysisParameters.minimumCurrent;
    noFlowConductance = circuitAnalysisParameters.noFlowConductance;


    stalledStrands = find(abs(strandCurrentList)<minimumCurrent);
    numStalledStrands = numel(stalledStrands)

    strandConductanceList(stalledStrands) = noFlowConductance;
    roundCounter = roundCounter+1;
    stalledMatrix(stalledStrands,roundCounter) = 1;
    figure(98);imagesc(stalledMatrix)


    circuitAnalysisParameters.stalledStrands = stalledStrands;
    circuitAnalysisParameters.stalledMatrix = stalledMatrix;
    circuitAnalysisParameters.roundCounter = roundCounter;
    circuitAnalysisParameters.strandConductanceList = strandConductanceList;
    circuitAnalysisParameters.minimumCurrent = minimumCurrent;
    circuitAnalysisParameters.noFlowConductance = noFlowConductance;
    vectorProject.circuitAnalysisParameters = circuitAnalysisParameters;
    
return

%%
function vectorProject = CheckCircuitStalls(vectorProject)
    disp(['Rechecking Stalls : ',datestr(now)]);
    vectorizedStructure = vectorProject.vectorizedStructure;
    circuitAnalysisParameters = vectorProject.circuitAnalysisParameters;
    Vertices = vectorizedStructure.Vertices;
    Strands = vectorizedStructure.Strands;
    numStrands = numel(Strands);
    strandCurrentList = circuitAnalysisParameters.strandCurrentList;
    originalConductanceList = circuitAnalysisParameters.originalConductanceList;
    strandConductanceList = circuitAnalysisParameters.strandConductanceList;
    stalledStrands = circuitAnalysisParameters.stalledStrands;
    stalledMatrix = circuitAnalysisParameters.stalledMatrix;
    roundCounter = circuitAnalysisParameters.roundCounter;
    AllVoltages =  Vertices.AllVoltages;
 
    %Now go to each "stalled" strand and set it to what it would still be  based on new
    %voltages and original conductances;
    numStalledStrands = numel(stalledStrands);
    for kk = 1:numStalledStrands,
        thisStrandID = stalledStrands(kk);
        thisStrand = Strands(thisStrandID);
        voltage1 = AllVoltages(thisStrand.StartVertexIndex);
        voltage2 = AllVoltages(thisStrand.EndVertexIndex);
        dv = abs(voltage1-voltage2);
        newStrandCurrent = originalConductanceList(thisStrandID).*dv;
        Strands(thisStrandID).strandCurrent = newStrandCurrent;
        strandCurrentList(thisStrandID) = newStrandCurrent;
    end

    
    vectorProject.vectorizedStructure.Strands = Strands;
    circuitAnalysisParameters.strandCurrentList = strandCurrentList;
    vectorProject.circuitAnalysisParameters = circuitAnalysisParameters;
return

%%
function vectorProject = GenerateSingleRoundCurrentStoppageMaskRGB(vectorProject)

vectorizedStructure = vectorProject.vectorizedStructure;
vectorProjectRootName = vectorProject.vectorProjectRootName;
maskSize = vectorizedStructure.maskSize;
Vertices = vectorizedStructure.Vertices;
Strands = vectorizedStructure.Strands;
AllVerts = Vertices.AllVerts;
strandCurrentList = abs([Strands.strandCurrent]);
circuitAnalysisParameters = vectorProject.circuitAnalysisParameters;
minimumCurrent = circuitAnalysisParameters.minimumCurrent;


keyboard

belowMinimumCurrentList = find(strandCurrentList<minimumCurrent); disp([numel(belowMinimumCurrentList),numel(strandCurrentList)]);


aboveMinimumCurrentList = find(strandCurrentList>=minimumCurrent);




outputMaskSizeDivisor = 3;%Hardcode


%-----
myMask = zeros(ceil(maskSize/outputMaskSizeDivisor),'uint8');
numStrands = numel(belowMinimumCurrentList);
for strandIter = 1:numStrands,
    thisStrand = Strands(belowMinimumCurrentList(strandIter));
    stei = thisStrand.StartToEndIndices;
    numPoints = numel(stei);
    for pointIter = 1:numPoints,
        coords = ceil(AllVerts(stei(pointIter),:)/outputMaskSizeDivisor);
        myMask(coords(1),coords(2),coords(3)) = 255;
    end%for pointIter = 1:numPoints,
end%for strandIter = 1:numStrands,

mask1 = permute(myMask,[3,1,2]);


%----------------------------
%----------------------------
%----------------------------
%----------------------------
%----------------------------


%-----
myMask = zeros(ceil(maskSize/outputMaskSizeDivisor),'uint8');
numStrands = numel(aboveMinimumCurrentList);
for strandIter = 1:numStrands,
    thisStrand = Strands(aboveMinimumCurrentList(strandIter));
    stei = thisStrand.StartToEndIndices;
    numPoints = numel(stei);
    for pointIter = 1:numPoints,
        coords = ceil(AllVerts(stei(pointIter),:)/outputMaskSizeDivisor);
        myMask(coords(1),coords(2),coords(3)) = 255;
    end%for pointIter = 1:numPoints,
end%for strandIter = 1:numStrands,

mask2 = permute(myMask,[3,1,2]);

%----------------------------
%----------------------------
%----------------------------
%----------------------------
%----------------------------


maxFrame1 = max(mask1(:,:,100:150),[],3);
maxFrame2 = max(mask2(:,:,100:150),[],3);
maxRGB = zeros(size(mask1,1),size(mask1,2),3,'uint8');
maxRGB(:,:,1) = maxFrame1;
maxRGB(:,:,2) = maxFrame2;
figure;imagesc(maxRGB);



tic

fileName = 'SRCurrentStopMaskRGBDiv3.tif';
currentFrame = zeros(size(mask1,1),size(mask1,2),3,'uint8');
currentFrame(:,:,1) = squeeze(mask1(:,:,1));
currentFrame(:,:,2) = squeeze(mask2(:,:,1));
imwrite(uint8(currentFrame),fileName,'tiff','Compression','none');

for frameIter = 2:size(mask1,3);
    currentFrame(:,:,1) = squeeze(mask1(:,:,frameIter));
    currentFrame(:,:,2) = squeeze(mask2(:,:,frameIter));
    imwrite(uint8(currentFrame),fileName,'tiff','WriteMode','append','Compression','none');
end

toc
clear mask1 mask2 mask3 myMask
    




