% function ConvertToNodeAndAnalyzeCircuit_script3e
%This code performs a circuit analysis on a labeled, trimmed vessel set
%Load a globalDataStruct
%
%Note:  no connectivity changes can have occured since the latest restrandify round
%Changes to labels are okay.
%
%Labels: 100=Unlabeled; 200=Temp ; 300=Surface ; 400=Plunging ; 500=Micro ;
%         10=Unlabeled ; 20=Temp ; 30=Supply ; 40=Draining ;  50=Connecting ;
%Therefore: SurfaceArtery=330; SurfaceVein = 340;



%In this version, only free endpoints of surface vessels are given fixed voltages
%The four rectangular edges are allowed different fixed voltage levels
%i.e., left edge artery endpoints = 50V, right edge artery endpoints = 25V




% % load('F:\Phil\AOH_Datasets\pbDB\pbDBVector\VectorProjectFolder\pbDB-LargestSubgraph-VectorProject.mat')

onlyFreeEndPointsAreFixedNodesFlag = 1;%Hardcode
fixedALabelList = [330:339];%Hardcode : labels to be considerred voltages voltages
fixedBLabelList = [340:349];%Hardcode : labels to be considered ground
fixedAVoltage = 100;%Hardcode: potential at voltage voltage connection
fixedBVoltage = 50;%Hardcode: potential at ground connection
conductanceMultiplier = 1e9;%Hardcode
variableCleanupFlag = 0;
%---
radiusPower = 4;%Hardcode
minRadius = 0.25;%Hardcode
minStrandLength = 4;%Hardcode
hematocrit = 0.45;%Hardcode
    

vectorizedStructure = vectorProject.vectorizedStructure;
maskSize= vectorizedStructure.maskSize;
Vertices = vectorizedStructure.Vertices;
Strands = vectorizedStructure.Strands;
numStrands = numel(Strands);




freeEndPointFlagList = Vertices.FreeEndPoints;
junctionPointFlagList = Vertices.JunctionPoints;
AllVerts = Vertices.AllVerts;
AllRadii = Vertices.AllRadii;
AllLabels = Vertices.AllLabels;
StrandIndices = Vertices.StrandIndices;


%Floating nodes are junctions or freeEndPoints that are not connected to a fixed voltage
%Fixed nodes are junctions or freeEndPoints that ARE held at a fixed voltage
%Allow multiple fixed voltages: fixedVoltageA, fixedVoltageB, etc.

%First find all the nodes (fixed or floating)
disp(['Finding Nodes : ',datestr(now)]);
allNodeFlagList = or(freeEndPointFlagList,junctionPointFlagList);% a node is a junction or a free end point
allNodeIndexList = find(allNodeFlagList);
freeEndPointIndexList = find(freeEndPointFlagList);
junctionPointIndexList = find(junctionPointFlagList);
numNodes = sum(allNodeFlagList);
numFreeEndPoints = sum(freeEndPointFlagList);
numJunctionPoints = sum(junctionPointFlagList);
vertexToAllNodeLUT = zeros(size(AllRadii));
for nodeIter = 1:numNodes,
    currentNodeIndex = allNodeIndexList(nodeIter);
    vertexToAllNodeLUT(currentNodeIndex) = nodeIter;
end

%Go to each free end point and force its label to be the majority label of its strand
for fepIter = freeEndPointIndexList',
    oldLabel = AllLabels(fepIter);
    fepStrand = StrandIndices{fepIter};
    fepStrandVerts = Strands(fepStrand).InteriorVertices;
    majorityLabel = median(AllLabels(fepStrandVerts));
    AllLabels(fepIter) = majorityLabel;
end


%Now make a list of : nodes that are at fixedVolatageA, fixedVoltageB, etc
%   the remainder nodes are floating nodes
%   Note: junction points do not have an unambiguous label - only strands do.
%       therefore, a junction point will be considered a fixedVoltage node if
%       any one of its attached strands has the appropriate label
%If onlyFreeEndPointsAreFixedNodesFlag is set on, then all junction points are floating
if onlyFreeEndPointsAreFixedNodesFlag == 1,
    freeEndPointLabelList = AllLabels(freeEndPointIndexList);
    fixedANodeIndexList = freeEndPointIndexList(ismember(freeEndPointLabelList,fixedALabelList));
    fixedBNodeIndexList = freeEndPointIndexList(ismember(freeEndPointLabelList,fixedBLabelList));
    floatingNodeFlagList = allNodeFlagList;
    floatingNodeFlagList(fixedANodeIndexList) = 0;
    floatingNodeFlagList(fixedBNodeIndexList) = 0;
    floatingNodeIndexList = find(floatingNodeFlagList);
    numFloatingNodes = numel(floatingNodeIndexList);
    numFixedANodes = numel(fixedANodeIndexList);
    numFixedBNodes = numel(fixedBNodeIndexList);
    for nodeIter = 1:numFloatingNodes,
        currentNodeIndex = floatingNodeIndexList(nodeIter);
        vertexToFloatingNodeLUT(currentNodeIndex) = nodeIter;
    end
    %---
    for nodeIter = 1:numFixedANodes,
        currentNodeIndex = fixedANodeIndexList(nodeIter);
        vertexToFixedANodeLUT(currentNodeIndex) = nodeIter;
    end
    %---
    for nodeIter = 1:numFixedBNodes,
        currentNodeIndex = fixedBNodeIndexList(nodeIter);
        vertexToFixedBNodeLUT(currentNodeIndex) = nodeIter;
    end

else%if onlyFreeEndPointsAreFixedNodesFlag == 1,
    %NOT YET CODED
    %NOT YET CODED
    %NOT YET CODED
    %NOT YET CODED
    %NOT YET CODED
    %NOT YET CODED
    %NOT YET CODED
    keyboard
end%if onlyFreeEndPointsAreFixedNodesFlag == 1,


disp(['Calculating Strand Conductances : ',datestr(now)]);
strandConductanceList = [];
for strandIter = 1:numStrands,
    currentStrandID = strandIter;
    currentStrand = Strands(currentStrandID);
    InteriorVertices = currentStrand.InteriorVertices;
    majorityLabel = median(AllLabels(InteriorVertices));
    strandLength = numel(InteriorVertices);
        strandLength = max(strandLength,minStrandLength);
    thisRadiusList = AllRadii(InteriorVertices);
        thisRadiusList = max(thisRadiusList,minRadius);
    thisStrandRadius = median(thisRadiusList);
    %Add a random amount to the strandlength to avoid discretation and 
    %potential matrix singularity???
    D = thisStrandRadius*2;  
    
%HARDCODE
D = (thisStrandRadius+2)*2;%HARDCODE
%HARDCODE

    
    
    alpha = 4./(1+exp(-0.593*(D-6.74)));    
    viscosity = 1 + ((exp(hematocrit*alpha)-1) .* (110*exp(-1.424*D) + 3 - 3.45*exp(-0.035*D)) ./ (exp(0.45*alpha)-1));
    strandLength = max(minStrandLength,strandLength);
%     strandLength = max(minStrandLength,strandLength) + rand(1);
    strandConductanceList(currentStrandID) = conductanceMultiplier * (D/2).^radiusPower / (viscosity.*strandLength);
    conductanceFormula = 'D = (thisStrandRadius+2)*2 ; strandConductance = conductanceMultiplier*((D/2)^rp)/(v*strandLength)  ;  1 + ((exp(hematocrit*alpha)-1) .* (110*exp(-1.424*D) + 3 - 3.45*exp(-0.035*D)) ./ (exp(0.45*alpha)-1)) ;  alpha = 4./(1+exp(-0.593*(D-6.74)))  ;    D = thisStrandRadius*2';
end%for strandIter = 1:numStrands,





disp(['Populating GMatrix : ',datestr(now)]);
%Populate the conductance matrix
%GMatrix(ij) = -1*g(ij) where g(ij) is the conductance between node i and node j
%GMatrix(ii) = sum[g(ij)] across a row
%Equation: V = Gmatrix * ForcingCurrent
GMatrix = sparse(zeros(numFloatingNodes,numFloatingNodes));
for floatingNodeIter = 1:numFloatingNodes,
    currentNodeIndex = floatingNodeIndexList(floatingNodeIter);
    strandsAttached = StrandIndices{currentNodeIndex};
    numStrandsAttached = numel(strandsAttached);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentStrand = Strands(currentStrandID);
        strandConductance = strandConductanceList(currentStrandID);
        attachedNodeIndexList = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
        otherNodeIndex = attachedNodeIndexList(attachedNodeIndexList~=currentNodeIndex);
        if ismember(otherNodeIndex,floatingNodeIndexList),
            otherFloatingNodeIndex = vertexToFloatingNodeLUT(otherNodeIndex);
            GMatrix(floatingNodeIter,otherFloatingNodeIndex) =  -1 *  strandConductance;
            GMatrix(otherFloatingNodeIndex,floatingNodeIter) =  -1 *  strandConductance;
        end%if ismember(otherNodeIndex,floatingNodeIndexList),
    end%for strandIter = 1:numStrandsAttached,
end%for floatingNodeIter = 1:numFreeNodes,

            
for floatingNodeIter = 1:numFloatingNodes,
    GMatrix(floatingNodeIter,floatingNodeIter) = -1 * sum(GMatrix(floatingNodeIter,:));
end


forcingVector = zeros(numFloatingNodes,1);
%---
for fixedNodeIter = 1:numFixedANodes,
    currentNodeIndex = fixedANodeIndexList(fixedNodeIter);
    strandsAttached = StrandIndices{currentNodeIndex};
    numStrandsAttached = numel(strandsAttached);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentStrand = Strands(currentStrandID);
        strandConductance = strandConductanceList(currentStrandID);
        attachedNodeIndexList = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
        otherNodeIndex = attachedNodeIndexList(attachedNodeIndexList~=currentNodeIndex);
        if ismember(otherNodeIndex,floatingNodeIndexList),
            floatingNodeIndex = vertexToFloatingNodeLUT(otherNodeIndex);
            GMatrix(floatingNodeIndex,floatingNodeIndex) = GMatrix(floatingNodeIndex,floatingNodeIndex) + strandConductance;
            forcingVector(floatingNodeIndex) = forcingVector(floatingNodeIndex) + fixedAVoltage.*strandConductance;
        end%ismember(otherNodeIndex,floatingNodeIndexList),
    end%for strandIter = 1:numStrandsAttached,
end%for fixedNodeIter = 1:numFixedANodes,

%---
for fixedNodeIter = 1:numFixedBNodes,
    currentNodeIndex = fixedBNodeIndexList(fixedNodeIter);
    strandsAttached = StrandIndices{currentNodeIndex};
    numStrandsAttached = numel(strandsAttached);
    for strandIter = 1:numStrandsAttached,
        currentStrandID = strandsAttached(strandIter);
        currentStrand = Strands(currentStrandID);
        strandConductance = strandConductanceList(currentStrandID);
        attachedNodeIndexList = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
        otherNodeIndex = attachedNodeIndexList(attachedNodeIndexList~=currentNodeIndex);
        if ismember(otherNodeIndex,floatingNodeIndexList),
            floatingNodeIndex = vertexToFloatingNodeLUT(otherNodeIndex);
            GMatrix(floatingNodeIndex,floatingNodeIndex) = GMatrix(floatingNodeIndex,floatingNodeIndex) + strandConductance;
            forcingVector(floatingNodeIndex) = forcingVector(floatingNodeIndex) + fixedBVoltage.*strandConductance;
        end%ismember(otherNodeIndex,floatingNodeIndexList),
    end%for strandIter = 1:numStrandsAttached,
end%for fixedNodeIter = 1:numFixedANodes,
  
    
% % % disp(['Inverting GMatrix : ',datestr(now)]);
% % % tic
% % % GInv = inv(GMatrix);
% % % toc
% % % voltageVector = GInv*forcingVector;

disp(['Calculating Voltage Vector : ',datestr(now)]);
tic
voltageVector = GMatrix\forcingVector;toc
toc

figure(1);plot(voltageVector)
figure(2);plot(forcingVector)
figure(3);plot(GMatrix*voltageVector)

%Write results back to VectorizedStructure format
%For each strand, assign a current
%For each vertex, assign a voltage
disp(['Converting to Vectorized Structure Format']);
disp(['Assigning Node Voltages : ',datestr(now)]);
allNodeVoltageList = zeros(size(allNodeIndexList));
%Go node-by-node, and assign a voltage to each node
for nodeIter = 1:numNodes,
    thisNodeIndex = allNodeIndexList(nodeIter);
    %determine if this node is a floating node
    if ismember(thisNodeIndex,floatingNodeIndexList),
        %if so, the voltage is taken from the voltageVector
        thisFloatingNodeIndex = vertexToFloatingNodeLUT(thisNodeIndex);
        allNodeVoltageList(nodeIter) = voltageVector(thisFloatingNodeIndex);
    end%if ismember(thisNodeIndex,floatingNodeIndexList),
    %---
    %determine if this node is a fixed(A) node
    if ismember(thisNodeIndex,fixedANodeIndexList),
        %if so, assign it to the fixedVoltage(A)
        thisFixedNodeIndex = vertexToFixedANodeLUT(thisNodeIndex);
        allNodeVoltageList(nodeIter) = fixedAVoltage;
    end%if ismember(thisNodeIndex,fixedANodeIndexList),
    %---
    %determine if this node is a fixed(B) node
    if ismember(thisNodeIndex,fixedBNodeIndexList),
        %if so, assign it to the fixedVoltage(B)
        thisFixedNodeIndex = vertexToFixedBNodeLUT(thisNodeIndex);
        allNodeVoltageList(nodeIter) = fixedBVoltage;
    end%if ismember(thisNodeIndex,fixedBNodeIndexList),
end

disp(['Assigning Strand Currents and Vertex Voltages: ',datestr(now)]);
%Go strand-by-strand and assign currents
%   note that positive current means flow is from start-to-end index along strand
AllVoltages = zeros(size(AllRadii));
for strandIter = 1:numStrands,
    thisStrand = Strands(strandIter);
    thisStrandConductance = strandConductanceList(strandIter);
    node1VertexID = thisStrand.StartVertexIndex;
    node1Index = vertexToAllNodeLUT(node1VertexID);
    node1Voltage = allNodeVoltageList(node1Index);
    node2VertexID = thisStrand.EndVertexIndex;
    node2Index = vertexToAllNodeLUT(node2VertexID);
    node2Voltage = allNodeVoltageList(node2Index);
    deltaVoltage = node1Voltage-node2Voltage;
    thisStrandCurrent = deltaVoltage * thisStrandConductance;
    Strands(strandIter).strandConductance = thisStrandConductance;
    Strands(strandIter).strandCurrent = thisStrandCurrent;
    %---
    stei = thisStrand.StartToEndIndices;
    thisVoltageList = linspace(node1Voltage,node2Voltage,numel(stei));
    AllVoltages(stei) = thisVoltageList;
end
Vertices.AllVoltages = AllVoltages;

circuitAnalysisParameters = [];
circuitAnalysisParameters.fixedALabelList = fixedALabelList;
circuitAnalysisParameters.fixedBLabelList = fixedBLabelList;
circuitAnalysisParameters.fixedAVoltage = fixedAVoltage;
circuitAnalysisParameters.fixedBVoltage = fixedBVoltage;
circuitAnalysisParameters.onlyFreeEndPointsAreFixedNodesFlag = onlyFreeEndPointsAreFixedNodesFlag;
circuitAnalysisParameters.radiusPower = radiusPower;
circuitAnalysisParameters.minRadius = minRadius;
circuitAnalysisParameters.minStrandLength = minStrandLength;
circuitAnalysisParameters.conductanceMultiplier = conductanceMultiplier;
circuitAnalysisParameters.hematocrit = hematocrit;
circuitAnalysisParameters.conductanceFormula = conductanceFormula;
%-----
vectorizedStructure.Vertices = Vertices;
vectorizedStructure.Strands = Strands;
vectorProject.vectorizedStructure = vectorizedStructure;
vectorProject.circuitAnalysisParameters = circuitAnalysisParameters;


vectorProject.circuitAnalysisParameters.outputMaskSizeDivisor = 3;



save('AnalysisResults.mat','vectorProject');



%Stage 8) Create Masks for visualization
disp(['Stage 8 : Create masks for visulization',datestr(now)]);
disp(['Stage 8a : VoltageMask - ',datestr(now)]);
GenerateCircuitVoltageMask2(vectorProject);
disp(['Stage 8b : RGBVoltageMask - ',datestr(now)]);
GenerateCircuitRGBVoltageMask(vectorProject);
%---
disp(['Stage 8c : CurrentMask - ',datestr(now)]);
GenerateCircuitCurrentMask2(vectorProject);
disp(['Stage 8d : RGBCurrentMask - ',datestr(now)]);
GenerateCircuitRGBCurrentMask(vectorProject);















