function vectorProject = GenerateCircuitRGBCurrentMask3(flowProject)

vectorizedStructure = flowProject.vectorizedStructure;
flowProjectRootName = flowProject.flowProjectRootName;
vectorProjectFolder = flowProject.vectorProjectFolder;
maskSize = vectorizedStructure.maskSize;
Vertices = vectorizedStructure.Vertices;
Strands = vectorizedStructure.Strands;
AllVerts = Vertices.AllVerts;
strandCurrentList = abs([Strands.strandCurrent]);
outputMaskSizeDivisor = flowProject.circuitAnalysisParameters.outputMaskSizeDivisor;


logFlag = 1;%Hardcode
colorScheme = 'RedToBlue';
colorScheme = 'GreenToWhite';


if logFlag == 1,
    strandCurrentList = log(strandCurrentList+1);
end%if logFlag == 1,



maxCurrent = max(strandCurrentList);
minCurrent = min(strandCurrentList);
currentRange = maxCurrent-minCurrent;
minOutputValue = 56;
maxOutputValue = 255;
outputRange = maxOutputValue - minOutputValue;
normCurrents = (((strandCurrentList-minCurrent)/currentRange)*outputRange)+minOutputValue;



medianCurrent = median(normCurrents);
stdCurrent = std(normCurrents);
cutCurrents = normCurrents(and(normCurrents>medianCurrent-3*stdCurrent,normCurrents<medianCurrent+3*stdCurrent));
medianCurrent = median(cutCurrents);
stdCurrent = std(cutCurrents);

%renormalize
minVal = medianCurrent-3*stdCurrent;
maxVal = medianCurrent+3*stdCurrent;
valRange = maxVal-minVal;
normCurrents = max(0,normCurrents-minVal);
normCurrents = min(255,(normCurrents/valRange)*255);

switch(lower(colorScheme));
    case{'redtoblue'},
        colorRange = 0.7; %0.5 = no red/blue overlap,1 = full overlap,
        multiplier = 1/colorRange;
        offset = 255/multiplier;
        subtractor = (255-offset)*multiplier;
        redCurrents = uint8((double(normCurrents)*multiplier)-subtractor);
        blueCurrents = uint8((double(255-normCurrents)*multiplier)-subtractor);
        greenCurrents = uint8(255-(redCurrents+blueCurrents));
    case{'greentowhite'},
        raisePower = 0.5;
        greenCurrents = ones(size(normCurrents))*255;
        redCurrents = (((255-normCurrents)/255).^raisePower)*255;
        blueCurrents = (((255-normCurrents)/255).^raisePower)*255;
end%switch(lower(colorScheme));
%-----
numVertices = numel(strandCurrentList);
redMask = zeros(ceil(maskSize/outputMaskSizeDivisor),'uint8');
blueMask = zeros(ceil(maskSize/outputMaskSizeDivisor),'uint8');
greenMask = zeros(ceil(maskSize/outputMaskSizeDivisor),'uint8');
numStrands = numel(strandCurrentList);
for strandIter = 1:numStrands,
    thisStrand = Strands(strandIter);
    stei = thisStrand.StartToEndIndices;
    numPoints = numel(stei);
    thisCurrent = normCurrents(strandIter);
    for pointIter = 1:numPoints,
        coords = ceil(AllVerts(stei(pointIter),:)/outputMaskSizeDivisor);
        redMask(coords(1),coords(2),coords(3)) = redCurrents(strandIter);
        blueMask(coords(1),coords(2),coords(3)) = blueCurrents(strandIter);
        greenMask(coords(1),coords(2),coords(3)) = greenCurrents(strandIter);
    end%for pointIter = 1:numPoints,
end%for strandIter = 1:numStrands,




if ~exist(fullfile(flowProject.vectorProjectFolder,flowProject.associatedFlowProjectFolder)),
    mkdir(flowProject.vectorProjectFolder,flowProject.associatedFlowProjectFolder)
end%if ~exist...

saveFileName = [flowProjectRootName,'-CurrentMaskRGBDiv',num2str(outputMaskSizeDivisor),'.tif'];
savePathFileName = fullfile(fullfile(vectorProjectFolder,flowProject.associatedFlowProjectFolder),saveFileName);

currentFrame = zeros(size(redMask,1),size(redMask,2),3,'uint8');
currentFrame(:,:,1) = redMask(:,:,1);
currentFrame(:,:,2) = greenMask(:,:,1);
currentFrame(:,:,3) = blueMask(:,:,1);
imwrite(uint8(currentFrame),savePathFileName,'tiff','Compression','none');

for frameIter = 2:size(redMask,3);
    currentFrame(:,:,1) = redMask(:,:,frameIter);
    currentFrame(:,:,2) = greenMask(:,:,frameIter);
    currentFrame(:,:,3) = blueMask(:,:,frameIter);
    imwrite(uint8(currentFrame),savePathFileName,'tiff','WriteMode','append','Compression','none');
end





