function vectorProject = GenerateCircuitRGBVoltageMask(vectorProject)

vectorizedStructure = vectorProject.vectorizedStructure;
vectorProjectRootName = vectorProject.vectorProjectRootName;
maskSize = vectorizedStructure.maskSize;
Vertices = vectorizedStructure.Vertices;
Strands = vectorizedStructure.Strands;
AllVerts = Vertices.AllVerts;
AllVoltages = Vertices.AllVoltages;

outputMaskSizeDivisor = 3;%Hardcode
logFlag = 1;%Hardcode

if logFlag == 1,
    AllVoltages = log(AllVoltages+1);
end%if logFlag == 1,



normVoltages = AllVoltages;
maxVoltage = max(AllVoltages);
minVoltage = min(AllVoltages);
voltageRange = maxVoltage-minVoltage;
minOutputValue = 56;
maxOutputValue = 255;
outputRange = maxOutputValue - minOutputValue;
normVoltages = (((AllVoltages-minVoltage)/voltageRange)*outputRange)+minOutputValue;

medianVoltage = median(normVoltages);
stdVoltage = std(normVoltages);
cutVoltages = normVoltages(and(normVoltages>medianVoltage-3*stdVoltage,normVoltages<medianVoltage+3*stdVoltage));
medianVoltage = median(cutVoltages);
stdVoltage = std(cutVoltages);

%renormalize
minVal = medianVoltage-3*stdVoltage;
maxVal = medianVoltage+3*stdVoltage;
valRange = maxVal-minVal;
normVoltages = max(0,normVoltages-minVal);
normVoltages = min(255,(normVoltages/valRange)*255);

colorRange = 0.7; %0.5 = no red/blue overlap,1 = full overlap,
multiplier = 1/colorRange;
offset = 255/multiplier;
subtractor = (255-offset)*multiplier;

% % % offset = 128;
% % % multiplier = 255/offset
% % % subtractor = (255-offset)*multiplier

redVoltages = uint8((double(normVoltages)*multiplier)-subtractor);
blueVoltages = uint8((double(255-normVoltages)*multiplier)-subtractor);
greenVoltages = uint8(255-(redVoltages+blueVoltages));
%-----
numVertices = numel(AllVoltages);
redMask = zeros(ceil(maskSize/outputMaskSizeDivisor),'uint8');
blueMask = zeros(ceil(maskSize/outputMaskSizeDivisor),'uint8');
greenMask = zeros(ceil(maskSize/outputMaskSizeDivisor),'uint8');
for vertexIter = 1:numVertices,
    coords = ceil(AllVerts(vertexIter,:)/outputMaskSizeDivisor);
    thisVoltage = normVoltages(vertexIter);
    redMask(coords(1),coords(2),coords(3)) = redVoltages(vertexIter);
    blueMask(coords(1),coords(2),coords(3)) = blueVoltages(vertexIter);
    greenMask(coords(1),coords(2),coords(3)) = greenVoltages(vertexIter);
    
end


fileName = [vectorProjectRootName,'-VoltageMaskRGBDiv',num2str(outputMaskSizeDivisor),'.tif'];
currentFrame = zeros(size(redMask,1),size(redMask,2),3,'uint8');
currentFrame(:,:,1) = redMask(:,:,1);
currentFrame(:,:,2) = greenMask(:,:,1);
currentFrame(:,:,3) = blueMask(:,:,1);
imwrite(uint8(currentFrame),fileName,'tiff','Compression','none');

for frameIter = 2:size(redMask,3);
    currentFrame(:,:,1) = redMask(:,:,frameIter);
    currentFrame(:,:,2) = greenMask(:,:,frameIter);
    currentFrame(:,:,3) = blueMask(:,:,frameIter);
    imwrite(uint8(currentFrame),fileName,'tiff','WriteMode','append','Compression','none');
end


