function flowProject = LocatePlungingArterioles(flowProject)

vectorizedStructure = flowProject.vectorizedStructure;
vectorProjectFolder = flowProject.vectorProjectFolder; 
if isfield(flowProject,'flowProjectRootName'),
    flowProjectRootName = flowProject.flowProjectRootName;
    associatedFlowProjectFolder = flowProject.associatedFlowProjectFolder;
else%if isfield...
    flowProjectRootName = flowProject.vectorProjectRootName;
    associatedFlowProjectFolder = '';
end%if-else isfield...

%Plunging Arterioles starting points are the points of connection between
%a surface supply strand and a plunging supply strand
%surface supply labels = [330:339]
%plunging supply labels = [430:439]
label1List = [330:339];
label2List = [430:439];
[foundList1,foundList2] = LocateConnectingStrands(vectorizedStructure,label1List,label2List);

if isfield(flowProject,'vectorizedStructure'),
    flowProject.vectorizedStructure.plungingArterioleTopStrandList = foundList2;
else%if isfield(flowProject,'vectorizedStructure'),
    flowProject.plungingArterioleTopStrandList = foundList2;
end%if-else isfield(flowProject,'vectorizedStructure'),
baseStrandIDList = foundList2;
saveFileName = [flowProjectRootName,'-PlungingArterioleTopStrandList.mat'];
savePathFileName = fullfile(fullfile(vectorProjectFolder,associatedFlowProjectFolder),saveFileName);
save(savePathFileName,'baseStrandIDList');





function [foundList1,foundList2] = LocateConnectingStrands(vectorizedStructure,label1List,label2List);


maskSize = vectorizedStructure.maskSize;
Vertices = vectorizedStructure.Vertices;
Strands = vectorizedStructure.Strands;
AllVerts = Vertices.AllVerts;
AllLabels = Vertices.AllLabels;
StrandIndices = Vertices.StrandIndices;
foundList1 = [];
foundList2 = [];

if iscell(StrandIndices),
    disp('StrandIndices is in wrong format');
    keyboard
end%if iscell(StrandIndices),

numPoints = numel(AllLabels);
label1PointList = find(ismember(AllLabels,label1List));
label1StrandList = unique([StrandIndices(label1PointList,1)]);
numStrands = numel(label1StrandList);
for strandIter = 1:numStrands,
    thisStrandID = label1StrandList(strandIter);
    thisStrand = Strands(thisStrandID);
    thisMedLabel = mode(AllLabels(thisStrand.InteriorVertices));
    if ismember(thisMedLabel,label1List),
        neighborStrandList = [thisStrand.StartVertexNeighborStrands,thisStrand.EndVertexNeighborStrands];
        neighborStrandList = unique(neighborStrandList(neighborStrandList~=thisStrandID));
        for neighborIter = 1:numel(neighborStrandList),
            thisNeighborID = neighborStrandList(neighborIter);
            thisNeighborStrand = Strands(thisNeighborID);
            neighborMedLabel = mode(AllLabels(thisNeighborStrand.InteriorVertices));
            if ismember(neighborMedLabel,label2List),
                foundList1 = [foundList1,thisStrandID];
                foundList2 = [foundList2,thisNeighborID];
            end%if ismember(neighborMedLabel,label2List),
        end%for neighborIter...
    end%if ismember(...
end%for strandIter...

foundList1 = unique(foundList1);
foundList2 = unique(foundList2);






