
%-----------------------------------------------------------------------
function adjStruct = GetStrandAdjacencyMatrix(strandIndices)
% % 
% % if ~iscell(strandIndices),
% %     %Convert to cell storage
% %     disp('Converting to Cell Storage...');
% %     numVertices = size(strandIndices,1);
% %     siCell{numVertices} = 0;
% %     pause(0.1);
% %     for vertIter = 1:numVertices,
% %         thisSI = strandIndices(vertIter,:);
% %         zeroList = find(thisSI == 0);
% %         firstZero = zeroList(1);
% %         siCell{vertIter} = thisSI(1:firstZero-1);
% %     end
% %     strandIndices = siCell;
% %     disp('Conversion complete');
% % end%if ~iscell

if iscell(strandIndices),
    % Get all vertices of junctions
    JunctionVerts = cellfun(@length,strandIndices)>1;
    % Get all strand indices incident on all junctions
    ConnectedStrands = strandIndices(JunctionVerts);
    % Compute the highest connectivity of any junction vertex
    MaxStrandConnectivity = max(cellfun(@length,ConnectedStrands));
else
    % Get all vertices of junctions
    JunctionVerts = sum(strandIndices>0,2)>1;
    % Get all strand indices incident on all junctions
    ConnectedStrands = strandIndices(JunctionVerts,:);
    % Compute the highest connectivity of any junction vertex
    MaxStrandConnectivity = max(sum(strandIndices>0,2));
end

% Preallocate the LUT for indexing pairs of strands
AdjacencyLUT = cell(1,MaxStrandConnectivity);
for iAL = 1:MaxStrandConnectivity; AdjacencyLUT{iAL} = getAllPairs(iAL); end

% Preallocate the pairs adjacency matrix
AMpairs = cell(1,numel(ConnectedStrands));
% for iCS = 1:numel(ConnectedStrands)
for iCS = 1:length(ConnectedStrands)
% %     try
% %         CStmp = ConnectedStrands{iCS};
% %     catch
        CStmp = ConnectedStrands(iCS,:);
        CStmp = CStmp(CStmp~=0);
% %     end%try-catch
    %avoid self connections
    if numel(unique(CStmp))>=2
        ipairs = AdjacencyLUT{numel(CStmp)};
        AMpairs{iCS} = unique(reshape(CStmp(ipairs(:)),size(ipairs)),'rows');
    end
end


% Create sparse indexing matrix from cell array of strand index pairs
Sij = cell2mat(AMpairs');
% Make it symmetric
SAM = sparse([Sij(:,1);Sij(:,2)],[Sij(:,2);Sij(:,1)],ones(2*size(Sij,1),1));
SAM(SAM>1) = 1; % ensure all ones

subglabels = zeros(1,size(SAM,1));
SAM = spones(SAM)+speye(size(SAM,1));
[p,q,r] = dmperm(SAM);
for iP = 1:length(r)-1; subglabels(p(r(iP):r(iP+1)-1))=iP; end
nsubgs=max(subglabels(:));

adjStruct.SAM = SAM;
adjStruct.numSubgraphs = nsubgs;
adjStruct.subgraphLabels = subglabels;
adjStruct.strandIndices = strandIndices;




%-----------------------------------------------------------------------
function indexpairs = getAllPairs(npairs)

indexpairs = zeros((npairs^2 - npairs)/2, 2);
icounter = 1;
for i = 1:npairs - 1
    for j = i+1:npairs
        indexpairs(icounter,1:2) = [i, j];
        icounter = icounter + 1;
    end
end


