%Calculate Tissue Oxygenation based on Time of Flight of blood

%Model:
%time per (um) pixel  =  r^2 / strandCurrent
%assume blood oxygenation drops off linearly with time
%   this assumes an exposure to a roughly constant gradient of oxygenation
%amount of oxygen delivered to tissue by a pixel of vessel is then
%   the difference between the blood oxygenation of that pixel and its
%   immediately upstream vessel pixel.
%assume oxygenation from a point drops off 1/r2 from source pixel (weak tissue consumption)
%   or add an exponential drop off with r from source pixel (added tissue consumption)


%Method:
%Start from artery tips (near surface).  Assume some surface oxygenation.
%For each strand, calculate the time per pixel (r2/I)
%Assign a time_of_flight to each vertex
%Check off that strand as calculated
%Pick a downstream strand - continue until no downstream strands
%If there was only one choice of downstream branch, 


%Instructions:
%Load an FV file with artery tips labeled.

numIterations = 5;
alpha = 0.4e5;%1.6e5 %this sets the exponential drop of (plasma) blood ox with transit time
diffAlpha = 1/40; %this sets the exponential drop of (tissue) blood ox with distance from vessel.
radiusAdjust = 0.7

numStrands = numel(gds.Strands);
numVertices = numel(gds.Vertices.AllRadii);
numBaseStrands = numel(gds.baseStrandIDList);
newStartTimeList = zeros(1,numStrands)+inf;

for kk = 1:numStrands,
    meanY(kk) = mean(gds.Strands(kk).YXZLimits(1,:));
    meanX(kk) = mean(gds.Strands(kk).YXZLimits(2,:));
    meanZ(kk) = mean(gds.Strands(kk).YXZLimits(3,:));
end
    



for recalcIter = 1:numIterations,


    tofStruct = []; %timeOfFlight dataStruct
    tofStruct(numStrands).currentList = [];
    tofStruct(numStrands).startTimeList = [];


    baseStrandIDList = gds.baseStrandIDList;
    maxBranchSearch = 50;%Hardcode
    stopVoltage = 50.1;%Hardcode;
    minimumFluxFraction = 1/30000;%Hardcode

    downstreamPathList = [];
    downstreamBranchList = zeros(numStrands,numel(baseStrandIDList)) + Inf;
    transitTimeList = zeros(numStrands,1);

    for baseStrandIter = 1:numBaseStrands;
        endTimeList = zeros(numStrands,1);
        baseStrandID = baseStrandIDList(baseStrandIter);
        baseStrand = gds.Strands(baseStrandID);
        bsiv = baseStrand.InteriorVertices;
        baseStrandCoordList = gds.Vertices.AllVerts(bsiv,:);
        strandNodes = [baseStrand.StartVertexIndex,baseStrand.EndVertexIndex];
        nodeVoltages = gds.Vertices.AllVoltages(strandNodes);
        [highVoltage,highVoltagePos] = max(nodeVoltages);
        [lowVoltage,lowVoltagePos] = min(nodeVoltages);
        highNode = strandNodes(highVoltagePos);
        lowNode = strandNodes(strandNodes~=highNode);

        pathActiveFlagList = false(1,100000);
        pathActiveFlagList(1) = 1;
        numActivePath = sum(pathActiveFlagList);
        strandPathList = {};
        nodePathList = {};
        fractionPathList = {};
        strandPathList{1} = baseStrandID;
        nodePathList{1} = [highNode lowNode];
        fractionPathList{1} = 1;
        strandFlowList = zeros(numStrands,1);
        strandFlowList(baseStrandID) = 1;
        strandVoltageList = zeros(numStrands,1);
        strandVoltageList(1) = mean([highVoltage,lowVoltage]);
        pathCounter = 1;
        keepGoingFlag = 1;
        branchCounter = 0;
        %---
        baseStrandCurrent = abs(baseStrand.strandCurrent);
        strandRadius = baseStrand.MedianRadius + radiusAdjust;
        tofStruct(baseStrandID).currentList = baseStrandCurrent;
        tofStruct(baseStrandID).startTimeList = 0;
        strandLength = numel(gds.Strands(baseStrandID).StartToEndIndices);
    % %     timeOfFlight = strandLength*(strandRadius.^2/baseStrandCurrent);
    
    baseStrandVelocity = baseStrandCurrent./(strandRadius.^2);
%     timeOfFlight = strandLength/baseStrandCurrent;
    timeOfFlight = strandLength/baseStrandVelocity;


        transitTimeList(baseStrandID) = timeOfFlight;
        endTimeList(baseStrandID) = timeOfFlight;

        while (keepGoingFlag == 1),
            myCount = 0;
            branchCounter = branchCounter+1;
            for pathIter = find(pathActiveFlagList == 1);
                currentPathID = pathIter;
                currentStrandPath = strandPathList{pathIter};
                currentNodePath = nodePathList{pathIter};
                currentFractionPath = fractionPathList{pathIter};
                currentStrandID = currentStrandPath(end);
                currentStrand = gds.Strands(currentStrandID);
                %-----
                strandNodes = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
                nodeVoltages = gds.Vertices.AllVoltages(strandNodes);
                [highVoltage,highVoltagePos] = max(nodeVoltages);
                [lowVoltage,lowVoltagePos] = min(nodeVoltages);
                highNode = strandNodes(highVoltagePos);
                lowNode = strandNodes(strandNodes~=highNode);
                lowAttachedStrands = gds.Vertices.StrandIndices(lowNode,:);
                lowAttachedStrands = lowAttachedStrands(lowAttachedStrands~=0);
                lowAttachedStrands = lowAttachedStrands(lowAttachedStrands~=currentStrandID);
                numDaughterPaths = numel(lowAttachedStrands);
                strandVoltageList(currentStrandID) = mean([highVoltage,lowVoltage]);
                stopTracingFlag = 0;
                if numDaughterPaths == 0, stopTracingFlag = 1; end
                if lowVoltage < stopVoltage, stopTracingFlag = 1; end
                if currentFractionPath(end) < minimumFluxFraction, stopTracingFlag = 1; end
                if stopTracingFlag == 1,
                    pathActiveFlagList(currentPathID) = 0;
                else%if stopTracingFlag == 1,
                    motherCurrent = abs(currentStrand.strandCurrent);
                    validDaughterCounter = 0;
                    validDaughterStrandList = [];
                    validDaughterCurrentList = zeros(1,numDaughterPaths);
                    validDaughterPathIndexList = [];
                    for daughterIter = 1:numDaughterPaths,
                        %Check to see if the daughter strand leads to a lower voltage
                        candidateDaughterID = lowAttachedStrands(daughterIter);
                        candidateDaughterStrand = gds.Strands(candidateDaughterID);
                        candidateDaughterNodes = [candidateDaughterStrand.StartVertexIndex,candidateDaughterStrand.EndVertexIndex];
                        candidateDaughterOtherNode = candidateDaughterNodes(candidateDaughterNodes~=lowNode);
                        candidateDaughterOtherVoltage = gds.Vertices.AllVoltages(candidateDaughterOtherNode);
                        if candidateDaughterOtherVoltage < lowVoltage,
                            %if so, add the strand to a path
                            myCount = myCount + 1;
                            validDaughterCounter = validDaughterCounter + 1;
                            validDaughterStrandList(validDaughterCounter) = candidateDaughterID;
                            validDaughterCurrentList(validDaughterCounter) = abs(candidateDaughterStrand.strandCurrent);
                            if validDaughterCounter == 1,
                                %if this is the first valid daughter, add to current path
                                strandPathList{currentPathID} = [currentStrandPath, lowAttachedStrands(daughterIter)];
                                nodePathList{currentPathID} = [currentNodePath, candidateDaughterOtherNode];
                                validDaughterPathIndexList(1) = currentPathID;
                            else%if validDaughterCounter == 1,
                                %if not the first daughter, duplicate path up to this point, and
                                %make new path
                                pathCounter = pathCounter + 1;
                                strandPathList{pathCounter} = [currentStrandPath, lowAttachedStrands(daughterIter)];
                                nodePathList{pathCounter} = [currentNodePath, candidateDaughterOtherNode];
                                fractionPathList{pathCounter} = currentFractionPath;
                                pathActiveFlagList(pathCounter) = 1;  
                                validDaughterPathIndexList(validDaughterCounter) = pathCounter;
                            end%if-else validDaughterCounter == 1,
                        end%if candidateDaughterOtherVoltage < lowVoltage,
                    end%for daughterIter = 1:numDaughterPaths,
                    clear daughterIter
                    %-----
                    daughterSumCurrent = sum(validDaughterCurrentList);
                    fractionalDaughterCurrent = validDaughterCurrentList./daughterSumCurrent;
                    for validDaughterIter = 1:validDaughterCounter;
                        validDaughterSegmentID = validDaughterStrandList(validDaughterIter);
                        validDaughterFraction = currentFractionPath(end).*fractionalDaughterCurrent(validDaughterIter);
                        fractionPathList{validDaughterPathIndexList(validDaughterIter)} = [currentFractionPath , validDaughterFraction];
                        strandFlowList(validDaughterSegmentID) = strandFlowList(validDaughterSegmentID) + validDaughterFraction;


                        strandRadius = gds.Strands(validDaughterSegmentID).MedianRadius + radiusAdjust;
                        currentList = [tofStruct(validDaughterSegmentID).currentList, baseStrandCurrent .* validDaughterFraction];
                        tofStruct(validDaughterSegmentID).currentList = currentList;
                        thisDaughterCurrent = abs(gds.Strands(validDaughterSegmentID).strandCurrent);
                        thisMotherEndTime = endTimeList(currentStrandID);
                        strandLength = numel(gds.Strands(validDaughterSegmentID).StartToEndIndices);
    %                     timeOfFlight = strandLength*(strandRadius.^2)/thisDaughterCurrent;
    thisDaughterVelocity = thisDaughterCurrent./(strandRadius.^2);
%     timeOfFlight = strandLength/thisDaughterCurrent;
    timeOfFlight = strandLength/thisDaughterVelocity;


    if thisMotherEndTime > newStartTimeList(validDaughterSegmentID),
        thisMotherEndTime = newStartTimeList(validDaughterSegmentID);
    end

                        thisDaughterEndTime = thisMotherEndTime + timeOfFlight;
                        transitTimeList(validDaughterSegmentID) = timeOfFlight;
                        startTimeList = [tofStruct(validDaughterSegmentID).startTimeList, thisMotherEndTime];
                        tofStruct(validDaughterSegmentID).startTimeList = startTimeList;
                        endTimeList(validDaughterSegmentID) = thisDaughterEndTime;



                    end%for validDaughterIter = 1:validDaughterCounter;
                end%if-else stopTracingFlag == 1,
            end%for pathIter = find(pathActiveFlagList == 1);
            if sum(pathActiveFlagList) == 0, keepGoingFlag = 0;end
            if myCount == 0, keepGoingFlag = 0;end
            if branchCounter >= maxBranchSearch, keepGoingFlag = 0; end
        end%while (keepGoingFlag == 1),
        downstreamPathList{baseStrandIter} = strandPathList;
        disp(baseStrandIter)
    end%for baseStrandIter = 1:numBaseStrands;

    for baseStrandIter = 1:numBaseStrands;
        thisPathSet = downstreamPathList{baseStrandIter};
        for pathIter = 1:numel(thisPathSet),
            thisPath = thisPathSet{pathIter};
            thisPath = thisPath(2:end);
            posList = [1:numel(thisPath)]';
            downstreamBranchList(thisPath,baseStrandIter) = min(downstreamBranchList(thisPath,baseStrandIter),posList);
        end%for pathIter = 1:numel(thisPathSet);
    end%for baseStrandIter = ...




    for kk = 1:numStrands,
        startTimeList = tofStruct(kk).startTimeList;
        currentList = tofStruct(kk).currentList;
        fractionCurrent(kk) = abs(sum(currentList)./gds.Strands(kk).strandCurrent);
        bloodOx = exp(-alpha*startTimeList);
        normCurrentList = currentList./sum(currentList);
        meanBloodOx(kk) = sum(bloodOx.*normCurrentList);
        gds.Strands(kk).meanBloodOx = meanBloodOx(kk);
        meanYXZList(kk,1) = mean(gds.Strands(kk).YXZLimits(1,:));
        meanYXZList(kk,2) = mean(gds.Strands(kk).YXZLimits(2,:));
        meanYXZList(kk,3) = mean(gds.Strands(kk).YXZLimits(3,:));    
        strandRadiusList(kk) = gds.Strands(kk).MedianRadius;
        meanVoltage = gds.Strands(kk).MedianVoltage;
    end
    meanVoltage = [gds.Strands.MedianVoltage];
    figure(1);plot(meanVoltage,meanBloodOx,'r.');

    fx = and(meanYXZList(:,2)>300,meanYXZList(:,2)<1100);
    fy = and(meanYXZList(:,1)>300,meanYXZList(:,1)<800);
    fz = and(meanYXZList(:,3)>00,meanYXZList(:,3)<300);
    ff = and(and(fy,fx),fz);
    fr = strandRadiusList'<3; 
    ff = and(ff,fr);
    fv = and(meanVoltage'>60 , meanVoltage'<80);
    ff = and(ff,fv);
    fb = meanBloodOx'>0;
    ff = and(ff,fb);


    subV = meanVoltage(ff);
    subBloodOx = meanBloodOx(ff) * 0.8;
    figure(2);plot(subV,subBloodOx,'g.');


    bloodOxList = [gds.Strands.meanBloodOx];



    for k = 1:numStrands,
        thisPos = repmat(meanYXZList(k,:),[numStrands,1]);
        thisSep = (thisPos-meanYXZList).^2;
        thisSep = sqrt(sum(thisSep,2));
        thisList = [[1:numStrands]',thisSep];
        sortList = sortrows(thisList,2);
        shortList = sortList(1:100,:);
        neighborList = shortList(:,1);
        distList = shortList(:,2);
        subBloodOxList = bloodOxList(neighborList);
        diffusedBOList = subBloodOxList' .* exp(-distList.*diffAlpha);
        newBloodOxList(k) = max(diffusedBOList);
    end
    newStartTimeList = -1*log(newBloodOxList)/alpha;



    fx = and(meanYXZList(:,2)>400,meanYXZList(:,2)<1000);
    fy = and(meanYXZList(:,1)>400,meanYXZList(:,1)<700);
    ff = and(fx,fy);
    figure(recalcIter*10+1);plot(meanZ(ff),bloodOxList(ff),'g.');
    xlabel('Depth');
    ylabel('Relative Blood Ox');
    title(['PreComp Iteration ',num2str(recalcIter)]);
    figure(recalcIter*10+2);plot(meanZ(ff),newBloodOxList(ff),'r.');
    xlabel('Depth');
    ylabel('Relative Blood Ox');
    title(['PostComp Iteration ',num2str(recalcIter)]);
 

    pause(1);

end%for recalcIter...


