function vectorProject = vectorProjectTool_labelStrandsBySubgraph(vectorProject)


doDisplay = 1;

if ~isfield(vectorProject.vectorizedStructure,'StrandAM');
    vectorProject.vectorizedStructure.StrandAM = GetStrandAdjacencyMatrix(vectorProject.vectorizedStructure);
end
[nsubs subglabels] = labelSubGraphs(vectorProject.vectorizedStructure.StrandAM);
[n] = hist(subglabels,1:nsubs);
[largestComp  sgi] = max(n);
vectorProject.vectorizedStructure.isStrandInLargestComp = subglabels==sgi;
vectorProject.vectorizedStructure.numberOfStrandsSubgraphs = nsubs;
vectorProject.vectorizedStructure.strandSubgraphId = subglabels;

if doDisplay
    h2strands = vectorProjectTool_displayStrandGraph(vectorProject);
    map =jet(nsubs);
    Pn(1)={'Color'};
    Pv=num2cell(map(subglabels,:),2);
    set(h2strands,Pn,Pv)
end


%-----------------------------------------------------------------------
function SAM = GetStrandAdjacencyMatrix(vectorizedStructure)
Vertices = vectorizedStructure.Vertices;

% Get all vertices of junctions
JunctionVerts = cellfun(@length,Vertices.StrandIndices)>1;
% Get all strand indices incident on all junctions
ConnectedStrands = Vertices.StrandIndices(JunctionVerts);
% Compute the highest connectivity of any junction vertex
MaxStrandConnectivity = max(cellfun(@length,ConnectedStrands));

% Preallocate the LUT for indexing pairs of strands
AdjacencyLUT = cell(1,MaxStrandConnectivity);
for iAL = 1:MaxStrandConnectivity; AdjacencyLUT{iAL} = getAllPairs(iAL); end

% Preallocate the pairs adjacency matrix
AMpairs = cell(1,numel(ConnectedStrands));
for iCS = 1:numel(ConnectedStrands)
    CStmp = ConnectedStrands{iCS};
    %avoid self connections
    if numel(unique(CStmp))>=2
        ipairs = AdjacencyLUT{numel(CStmp)};

        AMpairs{iCS} = unique(reshape(CStmp(ipairs(:)),size(ipairs)),'rows');
    end
end

% Create sparse indexing matrix from cell array of strand index pairs
Sij = cell2mat(AMpairs');
% Make it symmetric
SAM = sparse([Sij(:,1);Sij(:,2)],[Sij(:,2);Sij(:,1)],ones(2*size(Sij,1),1));
SAM(SAM>1) = 1; % ensure all ones



%-----------------------------------------------------------------------
function indexpairs = getAllPairs(npairs)

indexpairs = zeros((npairs^2 - npairs)/2, 2);
icounter = 1;
for i = 1:npairs - 1
    for j = i+1:npairs
        indexpairs(icounter,1:2) = [i, j];
        icounter = icounter + 1;
    end
end


%-----------------------------------------------------------------------
function [nsubgs subglabels] = labelSubGraphs(Adj)
% Adj          : the symmetric adjacency matrix
% subglabels : an integer label indexed into every point Adj relates
% nsubgs     : the number of subgraphs in Adj
% modified from John Kaufhold by Pablo Blinder

% label
subglabels = zeros(1,size(Adj,1));
Adj = spones(Adj)+speye(size(Adj,1));
[p,q,r] = dmperm(Adj);
for iP = 1:length(r)-1; subglabels(p(r(iP):r(iP+1)-1))=iP; end
nsubgs=max(subglabels(:));


function h2Strand=vectorProjectTool_displayStrandGraph(vectorProject,varargin)
%DISPLAYSTRANDGRAPH - display strands
%
%Usage:
%
%   displayStrandGraph(vectorProject); displays all strands
%   displayStrandGraph(vectorProject,idx); displays strands whos indices are in idx;
%
%Pablo

doSingleColor = 0;
doCreateFigure = 0;

strandList2plot = [];
if nargin>1
    strandList2plot = varargin{1};
end
if isempty(strandList2plot);strandList2plot = 1 : numel(vectorProject.vectorizedStructure.Strands);end
nStrands2plot = numel(strandList2plot);
h2Strand = zeros(nStrands2plot,1);


if doCreateFigure
    color = [.3 .3 .3];
    figure('color',color);
    set(gca,'color',color,'xcolor','w','ycolor','w','zcolor','w');
end
hold on
% plot3(Vertices.AllVerts(:,1),Vertices.AllVerts(:,2),Vertices.AllVerts(:,3),'w.','markerSize',3)
for iS2PLOT =  1:nStrands2plot
    iSTRAND = strandList2plot(iS2PLOT);
    ctp = vectorProject.vectorizedStructure.Vertices.AllVerts(vectorProject.vectorizedStructure.Strands(iSTRAND).StartToEndIndices,:);
    radius = median(vectorProject.vectorizedStructure.Vertices.AllRadii(vectorProject.vectorizedStructure.Strands(iSTRAND).StartToEndIndices,:));
    if radius==0;radius=1;end;
    if numel(ctp)>0
        %         h2Strand(iSTRAND) = plot3(ctp(:,1),ctp(:,2),ctp(:,3),'w-','UserData',iSTRAND,'ButtonDownFcn','get(gco,''UserData'')','lineWidth',radius);
        h2Strand(iSTRAND) = plot3(ctp(:,1),ctp(:,2),ctp(:,3),'k.','UserData',iSTRAND,'ButtonDownFcn','get(gco,''UserData'')','markerSize',radius*4);
        
    end
end


view(3)
if ~doSingleColor
    map =jet(nStrands2plot);
    Pn(1)={'Color'};
    Pv=num2cell(map,2);
    set(h2Strand(strandList2plot),Pn,Pv)
end