% function effResistance = CalculatePointToPointResistance(vectorProject,vertex1ID,vertex2ID);


setIDNum = 4;

switch setIDNum,
    case 1, vectorProject = allProject.au;
    case 2, vectorProject = allProject.av;
    case 3, vectorProject = allProject.co;
    case 4, vectorProject = allProject.db;
end

vectorizedStructure = vectorProject.vectorizedStructure;
strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;
rootName = vectorProject.vectorProjectRootName;
findDash = findstr(rootName,'-');
shortName = rootName(1:findDash(1)-1);
circuitAnalysisParameters = vectorProject.circuitAnalysisParameters;
floatingNodeIndexList = circuitAnalysisParameters.floatingNodeIndexList;
allVerts = double(vertices.AllVerts);
strandIndices = vertices.StrandIndices;


cutName = shortName(3:end);
eigenSetFileName = ['F:\Phil\AOH_VectorProjects\',lower(cutName),'MinusSurfaceEigen.mat']

if exist('eigenValues','var') && numel(eigenValues) == numel(floatingNodeIndexList),
    disp('EigenValues already loaded');
else
    disp('Loading EigenValues and EigenVectors');pause(0.1);
    load(eigenSetFileName);
end

numStrands = numel(strands);
numFloatingNodes = numel(floatingNodeIndexList);
maxResistance = 2e-6

%%
%Make a list of plunging arteriole tip nodes
plungingArterioleTipList = vectorizedStructure.plungingArterioleTipList;
plungingVenuleTipList = vectorizedStructure.plungingVenuleTipList;
numArterioleTips = numel(plungingArterioleTipList);
numVenuleTips = numel(plungingVenuleTipList);
aNodeIDList = [];
vNodeIDList = [];

arterioleTipFloatingNodeList = [];
for tipIter = 1:numArterioleTips,
    thisTipID = plungingArterioleTipList(tipIter);
    thisTip = strands(thisTipID);
    if thisTip.StartVoltage > thisTip.EndVoltage,
        thisNodeID = thisTip.StartVertexIndex;
    else%if thisTip.StartVoltage > thisTip.EndVoltage,
        thisNodeID = thisTip.EndVertexIndex;
    end%if thisTip.StartVoltage > thisTip.EndVoltage,
    aNodeIDList(tipIter) = thisNodeID;
    thisFloatingNodeID = find(floatingNodeIndexList==thisNodeID);
    arterioleTipFloatingNodeList(tipIter) = thisFloatingNodeID;
end

venuleTipFloatingNodeList = [];
for tipIter = 1:numVenuleTips,
    thisTipID = plungingVenuleTipList(tipIter);
    thisTip = strands(thisTipID);
    if thisTip.StartVoltage < thisTip.EndVoltage,
        thisNodeID = thisTip.StartVertexIndex;
    else%if thisTip.StartVoltage < thisTip.EndVoltage,
        thisNodeID = thisTip.EndVertexIndex;
    end%if thisTip.StartVoltage < thisTip.EndVoltage,
    vNodeIDList(tipIter) = thisNodeID;
    thisFloatingNodeID = find(floatingNodeIndexList==thisNodeID);
    venuleTipFloatingNodeList(tipIter) = thisFloatingNodeID;
end

%%
effectiveResistanceList = zeros(1,1e6);
sepList = zeros(1,1e6);
aStrandList = zeros(1,1e6);
vStrandList = zeros(1,1e6);
counter = 0;
for arterioleTipIter = 1:numArterioleTips,
    disp(arterioleTipIter);
    aNodeID = arterioleTipFloatingNodeList(arterioleTipIter);
    aPos = allVerts(aNodeIDList(arterioleTipIter),:);
%     figure(1);hold off;
    for venuleTipIter = 1:numVenuleTips,
        vNodeID = venuleTipFloatingNodeList(venuleTipIter);
        vPos = allVerts(vNodeIDList(venuleTipIter),:);
        runSum = 0;
        mSize = size(eigenVectors,1);
        for iIter = 2:mSize;
            eVectorAlpha = eigenVectors(aNodeID,iIter);
            eVectorBeta = eigenVectors(vNodeID,iIter);
            eVal = eigenValues(iIter);
            cutoff = -1;
            if eVal > cutoff,
                runSum = runSum + (((eVectorAlpha-eVectorBeta).^2)/abs(eVal));
            end
        end%for iIter;
        effectiveResistance = runSum;
        effectiveResistance = min(effectiveResistance,maxResistance);
        sepDist = sqrt(sum((aPos(1:2)-vPos(1:2)).^2));
        counter = counter + 1;
        effectiveResistanceList(counter) = effectiveResistance;
        sepList(counter) = sepDist;
        aStrandList(counter) = plungingArterioleTipList(arterioleTipIter);
        vStrandList(counter) = plungingVenuleTipList(venuleTipIter);
        
% %         figure(1);p1 = plot(sepDist,effectiveResistance,'k.');hold on;
% %         set(p1,'MarkerSize',1);
    end%for venuleTipIter
%     figure(1);hold off;pause
end%for arterioleTipIter
effectiveResistanceList = effectiveResistanceList(1:counter);
sepList = sepList(1:counter);
aStrandList = aStrandList(1:counter);
vStrandList = vStrandList(1:counter);



%%



if ~exist('multiResistanceList','var'),
    multiSetIDList = [];
    multiResistanceList = [];
    multiSepDistList = [];
    multiNameList = [];

end%if ~exist...
multiSetIDList = [multiSetIDList,repmat(setIDNum,size(allResList'))];
multiResistanceList = [multiResistanceList;effectiveResistanceList'];
multiSepDistList = [multiSepDistList;sepList'];
multiNameList = [multiNameList,shortName,'-']

%%   

plotString = 'r.';
switch setIDNum,
    case 1, plotString = 'r.';
    case 2, plotString = 'g.';
    case 3, plotString = 'b.';
    case 4, plotString = 'm.';
end%switch
    
figure(1)
figure(1);p1 = plot(sepList,effectiveResistanceList,plotString);hold on
set(p1,'MarkerSize',5);

%Calc running median
sortData = sortrows([sepList',effectiveResistanceList'],1);
halfAveSize = 50;
numPoints = size(sortData,1);
medPathDistList = [];
medResList = [];
for pointIter = halfAveSize+1:numPoints-halfAveSize
    medPathDistList(pointIter-halfAveSize) = median(sortData(pointIter-halfAveSize:pointIter+halfAveSize,1));
    medResList(pointIter-halfAveSize) = median(sortData(pointIter-halfAveSize:pointIter+halfAveSize,2));
end
figure(1);plot(medPathDistList,medResList,'b-');
xlabel('xy distance between arteriole and venule [um]');
ylabel('effective network resistance');
title(['Dataset ',multiNameList]); 

%%



