% function effResistance = CalculatePointToPointResistance(vectorProject,vertex1ID,vertex2ID);


setIDNum = 4;

switch setIDNum,
    case 1, vectorProject = allProject.au;
    case 2, vectorProject = allProject.av;
    case 3, vectorProject = allProject.co;
    case 4, vectorProject = allProject.db;
end

vectorizedStructure = vectorProject.vectorizedStructure;
strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;
rootName = vectorProject.vectorProjectRootName;
findDash = findstr(rootName,'-');
shortName = rootName(1:findDash(1)-1);
circuitAnalysisParameters = vectorProject.circuitAnalysisParameters;
floatingNodeIndexList = circuitAnalysisParameters.floatingNodeIndexList;
allVerts = double(vertices.AllVerts);
strandIndices = vertices.StrandIndices;


cutName = shortName(3:end);
eigenSetFileName = ['F:\Phil\AOH_VectorProjects\',lower(shortName),'MinusSurfaceEigenAdjRadii.mat']

if exist('eigenValues','var') && numel(eigenValues) == numel(floatingNodeIndexList),
    disp('EigenValues already loaded');
else
    disp('Loading EigenValues and EigenVectors');pause(0.1);
    load(eigenSetFileName);
end

numStrands = numel(strands);
numFloatingNodes = numel(floatingNodeIndexList);
maxResistance = 2e-6

%%
%Make a list of plunging arteriole tip nodes
plungingArterioleTipList = vectorizedStructure.plungingArterioleTipList;
plungingVenuleTipList = vectorizedStructure.plungingVenuleTipList;
numArterioleTips = numel(plungingArterioleTipList);
numVenuleTips = numel(plungingVenuleTipList);
aNodeIDList = [];
vNodeIDList = [];

arterioleTipFloatingNodeList = [];
for tipIter = 1:numArterioleTips,
    thisTipID = plungingArterioleTipList(tipIter);
    thisTip = strands(thisTipID);
    if thisTip.StartVoltage > thisTip.EndVoltage,
        thisNodeID = thisTip.EndVertexIndex;
    else%if thisTip.StartVoltage > thisTip.EndVoltage,
        thisNodeID = thisTip.StartVertexIndex;
    end%if thisTip.StartVoltage > thisTip.EndVoltage,
    aNodeIDList(tipIter) = thisNodeID;
    thisFloatingNodeID = find(floatingNodeIndexList==thisNodeID);
    arterioleTipFloatingNodeList(tipIter) = thisFloatingNodeID;
end

venuleTipFloatingNodeList = [];
for tipIter = 1:numVenuleTips,
    thisTipID = plungingVenuleTipList(tipIter);
    thisTip = strands(thisTipID);
    if thisTip.StartVoltage < thisTip.EndVoltage,
        thisNodeID = thisTip.EndVertexIndex;
    else%if thisTip.StartVoltage < thisTip.EndVoltage,
        thisNodeID = thisTip.StartVertexIndex;
    end%if thisTip.StartVoltage < thisTip.EndVoltage,
    vNodeIDList(tipIter) = thisNodeID;
    thisFloatingNodeID = find(floatingNodeIndexList==thisNodeID);
    venuleTipFloatingNodeList(tipIter) = thisFloatingNodeID;
end


%MANUAL EXCEPTION LIST:
if setIDNum == 2,
    %this is a doublet tip - the next strand down is high-resistance - move tip down one
    %branch
    ff = find(plungingArterioleTipList==870);
    arterioleTipFloatingNodeList(ff) = find(floatingNodeIndexList==strands(1214).EndVertexIndex);
    %--
    ff = find(plungingArterioleTipList==871);
    arterioleTipFloatingNodeList(ff) = find(floatingNodeIndexList==strands(1214).EndVertexIndex);    
end
    
if setIDNum == 1,
    %flow is reversed in this venule tip (due to nearby connection to other surface vein)
    ff = find(plungingVenuleTipList==203);
    venuleTipFloatingNodeList(ff) = find(floatingNodeIndexList==strands(203).StartVertexIndex);
end


if setIDNum == 4,
    %this is a spurious venule tip - goes off to a small loop
    ff = find(plungingVenuleTipList==2573);
    venuleTipFloatingNodeList(ff) = venuleTipFloatingNodeList(ff-1);
end


%%
effectiveResistanceList = zeros(1,1e6);
sepList = zeros(1,1e6);
aStrandList = zeros(1,1e6);
vStrandList = zeros(1,1e6);
counter = 0;
for arterioleTipIter = 1:numArterioleTips,
    disp(arterioleTipIter);
    aNodeID = arterioleTipFloatingNodeList(arterioleTipIter);
    aPos = allVerts(aNodeIDList(arterioleTipIter),:);
%     figure(1);hold off;
    for venuleTipIter = 1:numVenuleTips,
        vNodeID = venuleTipFloatingNodeList(venuleTipIter);
        vPos = allVerts(vNodeIDList(venuleTipIter),:);
        runSum = 0;
        mSize = size(eigenVectors,1);
        for iIter = 2:mSize;
            eVectorAlpha = eigenVectors(aNodeID,iIter);
            eVectorBeta = eigenVectors(vNodeID,iIter);
            eVal = eigenValues(iIter);
            cutoff = -1;
            if eVal > cutoff,
                runSum = runSum + (((eVectorAlpha-eVectorBeta).^2)/abs(eVal));
            end
        end%for iIter;
        effectiveResistance = runSum;
        effectiveResistance = min(effectiveResistance,maxResistance);
        sepDist = sqrt(sum((aPos(1:2)-vPos(1:2)).^2));
        counter = counter + 1;
        effectiveResistanceList(counter) = effectiveResistance;
        sepList(counter) = sepDist;
        aStrandList(counter) = plungingArterioleTipList(arterioleTipIter);
        vStrandList(counter) = plungingVenuleTipList(venuleTipIter);
        
% %         figure(1);p1 = plot(sepDist,effectiveResistance,'k.');hold on;
% %         set(p1,'MarkerSize',1);
    end%for venuleTipIter
%     figure(1);hold off;pause
end%for arterioleTipIter
effectiveResistanceList = effectiveResistanceList(1:counter);
sepList = sepList(1:counter);
aStrandList = aStrandList(1:counter);
vStrandList = vStrandList(1:counter);



%%



if ~exist('multiResistanceList','var'),
    multiSetIDList = [];
    multiResistanceList = [];
    multiSepDistList = [];
    multiNameList = [];
    multiMarkerSize = 5;
    multiAStrandList = [];
    multiVStrandList = [];

end%if ~exist...
multiSetIDList = [multiSetIDList;repmat(setIDNum,size(effectiveResistanceList'))];
multiResistanceList = [multiResistanceList;effectiveResistanceList'];
multiSepDistList = [multiSepDistList;sepList'];
multiAStrandList = [multiAStrandList;aStrandList'];
multiVStrandList = [multiVStrandList;vStrandList'];
multiNameList = [multiNameList,shortName,'-']

%%   

plotString = 'ro';
switch setIDNum,
    case 1, plotString = 'ro';
    case 2, plotString = 'go';
    case 3, plotString = 'bo';
    case 4, plotString = 'yo';
end%switch
    
figure(1)
figure(1);p1 = plot(sepList,effectiveResistanceList,plotString);hold on
set(p1,'MarkerSize',multiMarkerSize);
multiMarkerSize = max(1,multiMarkerSize-1)

%Calc running median
sortData = sortrows([sepList',effectiveResistanceList'],1);
halfAveSize = 50;
numPoints = size(sortData,1);
medPathDistList = [];
medResList = [];
for pointIter = halfAveSize+1:numPoints-halfAveSize
    medPathDistList(pointIter-halfAveSize) = median(sortData(pointIter-halfAveSize:pointIter+halfAveSize,1));
    medResList(pointIter-halfAveSize) = median(sortData(pointIter-halfAveSize:pointIter+halfAveSize,2));
end
figure(1);plot(medPathDistList,medResList,'b-');
xlabel('xy distance between arteriole and venule [um]');
ylabel('effective network resistance');
title(['Dataset ',multiNameList]); 

%%


largeResistance = 1e-7;
ff = find(effectiveResistanceList>largeResistance);
largeAList = aStrandList(ff);
largeVList = vStrandList(ff);
uAList = unique(largeAList);
uVList = unique(largeVList);

numRep = [];
for uAIter = 1:numel(uAList),
    uA = uAList(uAIter);
    numRep(uAIter) = sum(ismember(largeAList,uA));
end
disp(numRep);
temp = find(numRep>median(numRep));
suspectAList = uAList(temp)

numRep = [];
for uVIter = 1:numel(uVList),
    uV = uVList(uVIter);
    numRep(uVIter) = sum(ismember(largeVList,uV));
end
disp(numRep);
temp = find(numRep>median(numRep));
suspectVList = uVList(temp)


%%

figData.multiResistanceList = multiResistanceList;
figData.multiSepDistList = multiSepDistList;
figData.multiAStrandList = multiAStrandList;
figData.multiVStrandList = multiVStrandList;
figData.multiSetIDList = multiSetIDList;
figData.note1 = 'effectiveNetworkResistance from Arteriole Tip to Venule Tip versus separation';
figData.note2 = '4 datasets: AU,AV,CO,DB ;';
figData.note3 = 'radii adjusted to Awake histogram with AAA_RenormalizeRadiusHistogramToAwake.m';
figData.note4 = 'Eigenvalues calculated with AAA_CalculateEigenvaluesForSurfaceStrippedDatasetAdjRadii.m';
figData.note4 = 'Network resistance calculated with AAA_CalculateArteryToVeinEffectiveResistance_AdjRadii.m';

multiMarkerSize = 5;
for setIter = 1:4,

    plotString = 'ro';
    switch setIter,
        case 1, plotString = 'ro';
        case 2, plotString = 'go';
        case 3, plotString = 'bo';
        case 4, plotString = 'yo';
    end%switch

    setFlagList = figData.multiSetIDList == setIter;
    effectiveResistanceList = figData.multiResistanceList(setFlagList);
    sepList = figData.multiSepDistList(setFlagList);
    figure(2);p1 = plot(sepList,effectiveResistanceList,plotString);hold on
    set(p1,'MarkerSize',multiMarkerSize);
    multiMarkerSize = max(1,multiMarkerSize-1)
end

%Calc running median
sortData = sortrows([figData.multiSepDistList,figData.multiResistanceList],1);
halfAveSize = 100;
numPoints = size(sortData,1);
medPathDistList = [];
medResList = [];
for pointIter = halfAveSize+1:numPoints-halfAveSize
    medPathDistList(pointIter-halfAveSize) = median(sortData(pointIter-halfAveSize:pointIter+halfAveSize,1));
    medResList(pointIter-halfAveSize) = median(sortData(pointIter-halfAveSize:pointIter+halfAveSize,2));
end
figure(2);p2 = plot(medPathDistList,medResList,'k-');
set(p2,'LineWidth',5);
xlabel('xy distance between arteriole and venule [um]');
ylabel('effective network resistance');
title(['4 Datasets compiled']); 
