%Load a FlowProject that has been run through CalculateAndStoreEffectiveStrandRadius.m
%Data is stored in the gds structure.

% % % %First, calculate true strand length (accounting for diagonal connections being longer)
% % % numStrands = numel(gds.Strands);
% % % strandLengthList = zeros(numStrands,1);
% % % strandNumPointsList = zeros(numStrands,1);
% % % for strandIter = 1:numStrands,
% % %     strandID = strandIter;
% % %     thisStrand = gds.Strands(strandID);
% % %     iv = thisStrand.InteriorVertices;
% % %     av = gds.Vertices.AllVerts(iv,:);
% % %     numPoints = numel(iv);
% % %     pointSep = zeros(100,1);
% % %     for pointIter = 1:numPoints-1;
% % %         pointDist = av(pointIter,:) - av(pointIter+1,:);
% % %         pointSep(pointIter) = sqrt(sum(pointDist.^2));
% % %     end%for pointIter
% % %     strandLengthList(strandID) = sum(pointSep)+1;
% % %     strandNumPointsList(strandID) = numPoints;
% % % end%for strandIter
% % % clear strandID thisStrand iv av numPoints pointSep pointDist pointIter

%%

%Consider only microvessels of diameter less than chosen value
%Use adjustedStrandRadiusList (adjustedStrandRadiusList = adjusted radii distribution to match in_vivo distribution)

setIDNum = 4;

switch setIDNum,
    case 1, vectorProject = allProject.au;
    case 2, vectorProject = allProject.av;
    case 3, vectorProject = allProject.co;
    case 4, vectorProject = allProject.db;
end

rootName = vectorProject.vectorProjectRootName;
findDash = findstr(rootName,'-');
shortName = rootName(1:findDash(1)-1);

cutoffDiameter = 7;
xyMargin = 100;
zMax = 1200;

gds = vectorProject.vectorizedStructure;

surfDepth = min(gds.strandMidYXZList(:,3));
gds.strandMidYXZList(:,3) = gds.strandMidYXZList(:,3) - surfDepth;
ccl = vectorProject.cellCentroidAndLabelList;

depthDim = 3;

ccl(:,depthDim) = ccl(:,depthDim)-surfDepth;
neuronZList = ccl(ccl(:,4)==1,depthDim);

strandConductanceList = [gds.Strands(:).strandConductance];
strandCurrentList = [gds.Strands(:).strandCurrent];


if ~isfield(gds,'adjustedStrandRadiusList')
    disp('No effectiveStrandRadiusList - use adjusted radii and choose a proper strand radius criteria');
    disp('No effectiveStrandRadiusList - use adjusted radii and choose a proper strand radius criteria');
    disp('No effectiveStrandRadiusList - use adjusted radii and choose a proper strand radius criteria');
    disp('PAUSED');
    pause
    pause
end

if ~isfield(gds,'strandEndSeparationList'),
    strands = gds.Strands;
    vertices = gds.Vertices;
    numStrands = numel(strands);
    strandEndSeparationList = [];
    for strandIter = 1:numStrands;
        svi = strands(strandIter).StartVertexIndex;
        evi = strands(strandIter).EndVertexIndex;
        coordsDelta = vertices.AllVerts(evi,:) - vertices.AllVerts(svi,:);
        sepDist = sqrt(sum(coordsDelta.^2));
        strandEndSeparationList(strandIter,1) = sepDist;
    end%for strandIter..
    gds.strandEndSeparationList = strandEndSeparationList;
end
        


adjustedDiameterList = gds.adjustedStrandRadiusList * 2;
x = [0:0.5:30];
h = hist(adjustedDiameterList,x);
figure(1);bar(x,h);
xlabel('Strand Diameter (um)');
ylabel('Count');
title(vectorProject.vectorProjectRootName)


microVesselFlagList = adjustedDiameterList <= cutoffDiameter;
maskSize = gds.maskSize;
yFlagList = and(gds.strandMidYXZList(:,1)>xyMargin , gds.strandMidYXZList(:,1)<maskSize(1)-xyMargin);
xFlagList = and(gds.strandMidYXZList(:,2)>xyMargin , gds.strandMidYXZList(:,2)<maskSize(2)-xyMargin);
zFlagList = gds.strandMidYXZList(:,3)<zMax;
microVesselFlagList = and(and(and(yFlagList,xFlagList),zFlagList),microVesselFlagList);


microVesselRadiusList = gds.adjustedStrandRadiusList(microVesselFlagList);
microVesselLengthList = gds.strandLengthList(microVesselFlagList);
microVesselZList = gds.strandMidYXZList(microVesselFlagList,3);
microVesselEndSepList = gds.strandEndSeparationList(microVesselFlagList);
microVesselTortuosityList = (microVesselLengthList+1)./(microVesselEndSepList+1); 
microVesselConductanceList = strandConductanceList(microVesselFlagList);
microVesselCurrentList = abs(strandCurrentList(microVesselFlagList));

%Find lowest 10% of currents
sortCurrent = sort(microVesselCurrentList,'ascend');
threshCurrent = sortCurrent(ceil(numel(sortCurrent)*0.10));
lowCurrentFlagList = abs(strandCurrentList) < threshCurrent;
lowMicroCurrentFlagList = and(microVesselFlagList,lowCurrentFlagList');
lowMicroCurrentZList = gds.strandMidYXZList(lowMicroCurrentFlagList,3);


if ~exist('multiRadiusList','var'),
    multiSetIDList = [];
    multiRadiusList = [];
    multiLengthList = [];
    multiZList = [];
    multiTortuosityList = [];
    multiConductanceList = [];
    multiCurrentList = [];
    multiNameList = [];
    multiLowCurrentZList = [];
    multiNeuronZList = [];
end%if ~exist...

setIDList = ones(size(microVesselRadiusList))*setIDNum;
multiSetIDList = [multiSetIDList;setIDList];
multiRadiusList = [multiRadiusList;microVesselRadiusList];
multiLengthList = [multiLengthList;microVesselLengthList];
multiZList = [multiZList;microVesselZList];
multiTortuosityList = [multiTortuosityList;microVesselTortuosityList];
multiConductanceList = [multiConductanceList;microVesselConductanceList'];
multiCurrentList = [multiCurrentList;microVesselCurrentList'];
multiNameList = [multiNameList,shortName,'-']
multiLowCurrentZList = [multiLowCurrentZList;lowMicroCurrentZList];
multiNeuronZList = [multiNeuronZList;neuronZList];

%Sort the data by the Zposition of the strand
allData = [microVesselZList,microVesselRadiusList,microVesselLengthList,microVesselTortuosityList,microVesselConductanceList',microVesselCurrentList',setIDList];
sortData = sortrows(allData,1);


%Filter the data
filterSize = 500;
skipSize = 50;
% % filtRadiusList = imfilter(sortData(:,2),ones(filterSize,1),'symmetric')./filterSize;
% % filtLengthList = imfilter(sortData(:,3),ones(filterSize,1),'symmetric')./filterSize;

filtRadiusList = medfilt2(sortData(:,2),[filterSize,1],'symmetric');
filtLengthList = medfilt2(sortData(:,3),[filterSize,1],'symmetric');
filtTortuosityList = medfilt2(sortData(:,4),[filterSize,1],'symmetric');
filtConductanceList = medfilt2(sortData(:,5),[filterSize,1],'symmetric');
filtCurrentList = medfilt2(sortData(:,6),[filterSize,1],'symmetric');


figure(7);subplot(3,2,1);plot(sortData(1:skipSize:end,1),sortData(1:skipSize:end,2)*2,'r.');
hold on; plot(sortData(:,1),filtRadiusList*2,'k-'); hold off;
xlabel('Microvessel Strand Depth (um)');
ylabel('Effective Microvessel Strand Diameter');
title(vectorProject.vectorProjectRootName)

figure(7);subplot(3,2,2);plot(sortData(1:skipSize:end,1),sortData(1:skipSize:end,3),'c.');
hold on; plot(sortData(:,1),filtLengthList,'k-'); hold off;
set(gca,'YLim',[0 70]);
xlabel('Microvessel Strand Depth (um)');
ylabel('Microvessel Strand Length');
title(vectorProject.vectorProjectRootName)

figure(7);subplot(3,2,3);plot(sortData(1:skipSize:end,1),sortData(1:skipSize:end,4),'g.');
hold on; plot(sortData(:,1),filtTortuosityList,'k-'); hold off;
set(gca,'YLim',[1 1.5]);
xlabel('Microvessel Strand Depth (um)');
ylabel('Microvessel Tortuosity');
title(vectorProject.vectorProjectRootName)




zRange = [0:100:ceil(max(sortData(:,1)))];
hz = histc(microVesselZList,zRange);
figure(7);subplot(3,2,4);;bar(zRange,hz);
set(gca,'XLim',[0 max(zRange)]);
xlabel('Microvessel Strand Depth (um)');
ylabel('Count');
title(vectorProject.vectorProjectRootName)

zRange2 = [0:10:ceil(max(sortData(:,1)))];
hNeuron = histc(neuronZList,zRange2);
figure(8);plot(zRange2,hNeuron);


% % figure(7);subplot(3,2,5);plot(sortData(1:skipSize:end,1),sortData(1:skipSize:end,5),'g.');
% % hold on; plot(sortData(:,1),filtConductanceList,'k-'); hold off;
% % set(gca,'YLim',[1 1e9]);
% % xlabel('Microvessel Strand Depth (um)');
% % ylabel('Microvessel Conductance');
% % title(vectorProject.vectorProjectRootName)

figure(7);subplot(3,2,5);plot(sortData(1:skipSize:end,1),sortData(1:skipSize:end,6),'g.');
hold on; plot(sortData(:,1),filtCurrentList,'k-'); hold off;
set(gca,'YLim',[1 1e8]);
xlabel('Microvessel Strand Depth (um)');
ylabel('Microvessel Current');
title(vectorProject.vectorProjectRootName)


zRange = [0:100:ceil(max(sortData(:,1)))];
hz = histc(lowMicroCurrentZList,zRange);
figure(7);subplot(3,2,6);bar(zRange,hz);
set(gca,'XLim',[0 1200]);
xlabel('Low Current Strand Depth (um)');
ylabel('Count');
title(vectorProject.vectorProjectRootName)
%%




%Sort the data by the Zposition of the strand
allData = [multiZList,multiRadiusList,multiLengthList,multiTortuosityList,multiConductanceList,multiCurrentList];
sortData = sortrows(allData,1);


%Filter the data
filterSize = 500;
skipSize = 20;
% % filtRadiusList = imfilter(sortData(:,2),ones(filterSize,1),'symmetric')./filterSize;
% % filtLengthList = imfilter(sortData(:,3),ones(filterSize,1),'symmetric')./filterSize;

filtRadiusList = medfilt2(sortData(:,2),[filterSize,1],'symmetric');
filtLengthList = medfilt2(sortData(:,3),[filterSize,1],'symmetric');
filtTortuosityList = medfilt2(sortData(:,4),[filterSize,1],'symmetric');
filtConductanceList = medfilt2(sortData(:,5),[filterSize,1],'symmetric');
filtCurrentList = medfilt2(sortData(:,6),[filterSize,1],'symmetric');

% % filtConductanceList = imfilter(sortData(:,5),ones(filterSize,1),'symmetric')./filterSize;
% % filtCurrentList = imfilter(sortData(:,6),ones(filterSize,1),'symmetric')./filterSize;


figure(17);subplot(3,2,1);plot(sortData(1:skipSize:end,1),sortData(1:skipSize:end,2)*2,'r.');
hold on; plot(sortData(:,1),filtRadiusList*2,'k-'); hold off;
set(gca,'Xlim',[0 1200]);
xlabel('Microvessel Strand Depth (um)');
ylabel('Effective Microvessel Strand Diameter');
title(multiNameList)

figure(17);subplot(3,2,2);plot(sortData(1:skipSize:end,1),sortData(1:skipSize:end,3),'c.');
hold on; plot(sortData(:,1),filtLengthList,'k-'); hold off;
set(gca,'YLim',[0 100]);
set(gca,'Xlim',[0 1200]);
xlabel('Microvessel Strand Depth (um)');
ylabel('Microvessel Strand Length');
title(multiNameList)

figure(17);subplot(3,2,3);plot(sortData(1:skipSize:end,1),sortData(1:skipSize:end,4),'g.');
hold on; plot(sortData(:,1),filtTortuosityList,'k-'); hold off;
set(gca,'YLim',[1 1.5]);
set(gca,'Xlim',[0 1200]);
xlabel('Microvessel Strand Depth (um)');
ylabel('Microvessel Tortuosity');
title(multiNameList)


zRange = [0:100:ceil(max(sortData(:,1)))];
hz = histc(multiZList,zRange);
figure(17);subplot(3,2,4);;bar(zRange,hz);
set(gca,'XLim',[0 max(zRange)]);
set(gca,'Xlim',[0 1200]);
xlabel('Microvessel Strand Depth (um)');
ylabel('Count');
title(multiNameList)


zRange2 = [0:10:ceil(max(sortData(:,1)))];
hNeuron = histc(multiNeuronZList,zRange2);
figure(18);plot(zRange2,hNeuron);
ylabel('Neuron Count');
xlabel('Cortical Depth (um)');
title(multiNameList)



% % figure(7);subplot(3,2,5);plot(sortData(1:skipSize:end,1),sortData(1:skipSize:end,5),'g.');
% % hold on; plot(sortData(:,1),filtConductanceList,'k-'); hold off;
% % set(gca,'YLim',[1 1e10]);
% % set(gca,'Xlim',[0 1200]);
% % xlabel('Microvessel Strand Depth (um)');
% % ylabel('Microvessel Conductance');
% % title(multiNameList)

figure(17);subplot(3,2,5);plot(sortData(1:skipSize:end,1),sortData(1:skipSize:end,6),'g.');
hold on; plot(sortData(:,1),filtCurrentList,'k-'); hold off;
set(gca,'YLim',[1 1e9]);
set(gca,'Xlim',[0 1200]);
xlabel('Microvessel Strand Depth (um)');
ylabel('Microvessel Current');
title(multiNameList)



zRange = [0:100:ceil(max(sortData(:,1)))];
hz = histc(multiLowCurrentZList,zRange);
hz = hz./sum(hz);
figure(17);subplot(3,2,6);bar(zRange,hz);
set(gca,'XLim',[0 1200]);
xlabel('Low Current Strand Depth (um)');
ylabel('Count');
title(multiNameList)
%%


% % %Histogram a shallow, mid, and deep region
% % shallowZFlagList = and(microVesselZList>250, microVesselZList<450); 
% % midZFlagList = and(microVesselZList>500, microVesselZList<700);
% % deepZFlagList = and(microVesselZList>1000, microVesselZList<1200); 
% % 
% % xr = [0:0.5:cutoffDiameter];
% % xl = [0:500];
% % hrShallow = hist(microVesselRadiusList(shallowZFlagList)*2,xr);
% % hlShallow = hist(microVesselLengthList(shallowZFlagList),xl);
% % figure(4);bar(xr,hrShallow);
% % xlabel('ShallowZ Microvessel Diameter (um)');
% % figure(5);bar(xl,hlShallow);
% % xlabel('ShallowZ Microvessel Length (um)');
% % %--
% % hrMid = hist(microVesselRadiusList(midZFlagList)*2,xr);
% % hlMid = hist(microVesselLengthList(midZFlagList),xl);
% % figure(6);bar(xr,hrMid);
% % xlabel('MidZ Microvessel Diameter (um)');
% % figure(7);bar(xl,hlMid);
% % xlabel('MidZ Microvessel Length (um)');
% % %--
% % hrDeep = hist(microVesselRadiusList(deepZFlagList)*2,xr);
% % hlDeep = hist(microVesselLengthList(deepZFlagList),xl);
% % figure(8);bar(xr,hrDeep);
% % xlabel('DeepZ Microvessel Diameter (um)');
% % figure(9);bar(xl,hlDeep);
% % xlabel('DeepZ Microvessel Length (um)');


%%

% % % %Plot total vascular volume versus depth
% % % vascVol = zeros(2000,1);
% % % microVascVol = zeros(2000,1);
% % % for zIter = 1:2000;
% % %     fz = gds.Vertices.AllVerts(:,3) == zIter;
% % %     rList = gds.Vertices.AllRadii(fz);
% % %     microRList = rList(rList<=(cutoffDiameter/2));
% % %     aList = pi*rList.^2;
% % %     microAList = pi*microRList.^2;
% % %     vascVol(zIter) = sum(aList);
% % %     microVascVol(zIter) = sum(microAList);
% % % end
% % % filterSize = 25;
% % % fractionVV = vascVol./(maskSize(1)*maskSize(2));
% % % filtVV = imfilter(fractionVV,ones(filterSize,1),'symmetric')./filterSize;
% % % figure(10);plot(filtVV);
% % % 
% % % fractionMicroVV = microVascVol./(maskSize(1)*maskSize(2));
% % % filtMicroVV = imfilter(fractionMicroVV,ones(filterSize,1),'symmetric')./filterSize;
% % % figure(20);plot(filtMicroVV);


%%
z = sortData(:,1);
r = sortData(:,2);

nSize = 300;

zMean = [];
rMean = [];
rStd = [];
rStdMean = [];
for kk = 1:floor(numel(z)/nSize)-1,
    ii = (kk-1)*nSize+1;
    jj = ii+nSize;
    zSub = z(ii:jj);
    rSub = r(ii:jj);
    zMean(kk) = mean(zSub);
    rMean(kk) = mean(rSub);
    rStd(kk) = std(rSub);
    rStdMean(kk) = std(rSub)./sqrt(numel(rSub));
end

figure(1);plot(zMean,rMean,'k-');hold on
plot(zMean,rMean+rStd,'b-');
plot(zMean,rMean-rStd,'b-');
plot(zMean,rMean+rStdMean,'r-');
plot(zMean,rMean-rStdMean,'r-');hold off

    
%%

multiZRList = [multiZList,multiRadiusList,multiSetIDList];
sortedZRList = sortrows(multiZRList,1);
sr = sortedZRList(:,2);
nr = floor(numel(sr)/10);
q1 = sr(1:nr);
q2 = sr(1*nr+1:2*nr);
q3 = sr(2*nr+1:3*nr);
q4 = sr(3*nr+1:4*nr);
q5 = sr(4*nr+1:5*nr);
q6 = sr(5*nr+1:6*nr);
q7 = sr(6*nr+1:7*nr);
q8 = sr(7*nr+1:8*nr);
q9 = sr(8*nr+1:9*nr);

disp([mean(q4),std(q4)./sqrt(numel(q4))]);
disp([mean(q9),std(q9)./sqrt(numel(q9))]);


%--
figData.rList = sortedZRList(:,2);
figData.zList = sortedZRList(:,1);
figData.setIDList = sortedZRList(:,3);
figData.note1 = 'microvessel radii versus depth of strand';
figData.note2 = '4 datasets: AU,AV,CO,DB ; sorted by Z';
figData.note3 = 'radii adjusted to Awake histogram with AAA_RenormalizeRadiusHistogramToAwake.m';
figData.note4 = 'vessel stats calculated with AA_CalculateDistributionOfMicrovesselLengths';
figData.cutoffDiameter = cutoffDiameter;
figData.xyMargin = xyMargin;
figData.zMax = zMax;

