% function effResistance = CalculatePointToPointResistance(vectorProject,vertex1ID,vertex2ID);


vectorProject = allProject.db;
vectorizedStructure = vectorProject.vectorizedStructure;
strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;
rootName = vectorProject.vectorProjectRootName;
findDash = findstr(rootName,'-');
shortName = rootName(1:findDash(1)-1);
circuitAnalysisParameters = vectorProject.circuitAnalysisParameters;
floatingNodeIndexList = circuitAnalysisParameters.floatingNodeIndexList;
allVerts = vertices.AllVerts;
strandIndices = vertices.StrandIndices;


cutName = shortName(3:end);
eigenSetFileName = ['F:\Phil\AOH_VectorProjects\',lower(cutName),'Eigen.mat'];

if exist('eigenValues','var') && numel(eigenValues) == numel(floatingNodeIndexList),
    disp('EigenValues already loaded');
else
    disp('Loading EigenValues and EigenVectors');pause(0.1);
    load(eigenSetFileName);
end

numStrands = numel(strands);
numFloatingNodes = numel(floatingNodeIndexList);


%%
%For each strand, calculate the ratio of effective network resistance to strand resistance

strandResistanceList = zeros(numStrands,1);
strandNetworkResistanceList = zeros(numStrands,1);
tic
disp('Calculating effective network resistance across each strand');pause(0.01);
for strandIter = 1:numStrands,
    if mod(strandIter,1000) == 100, 
        timeSpent = toc;
        ratioLeft = (numStrands-strandIter)/strandIter;
        timeLeft = timeSpent*ratioLeft;
        if timeLeft <99,
            disp([num2str(timeLeft,'%0.0f'),' seconds remaining']);pause(0.01);
        else%if timeLeft <99,
            disp([num2str(timeLeft/60,'%0.2f'),' minutes remaining']);pause(0.01);
        end%if timeLeft <99,
    end%if mod(strandIter,100) == 0, 
    strandID = strandIter;
    vertex1ID = strands(strandID).StartVertexIndex;
    vertex2ID = strands(strandID).EndVertexIndex;
    strandResistance = 1/strands(strandID).strandConductance;
    floatingNode1ID = find(floatingNodeIndexList==vertex1ID);
    floatingNode2ID = find(floatingNodeIndexList==vertex2ID);
    strandResistanceList(strandIter) = strandResistance;
    if isempty(floatingNode1ID) || isempty(floatingNode2ID),
        strandNetworkResistanceList(strandIter) = strandResistance;
    else%if isempty(floatingNode1ID) || isempty(floatingNode2ID),
    
    runSum = 0;
    mSize = size(eigenVectors,1);
    for iIter = 2:mSize;
        eVectorAlpha = eigenVectors(floatingNode1ID,iIter);
        eVectorBeta = eigenVectors(floatingNode2ID,iIter);
        eVal = eigenValues(iIter);
        cutoff = -1;
        if eVal > cutoff,
            runSum = runSum + (((eVectorAlpha-eVectorBeta).^2)/abs(eVal));
        end
    end%for iIter;
    effectiveResistance = runSum;
    strandNetworkResistanceList(strandIter) = effectiveResistance;
    end%if-else isempty(floatingNode1ID) || isempty(floatingNode2ID),
end%for strandIter = 1:numStrands,
disp('Done.');pause(0.1);
strandNetworkResistanceRatioList = strandResistanceList./strandNetworkResistanceList;
%Large ratio implies more parallelization; ratio of 1 implies no parallelization
vectorProject.vectorizedStructure.strandNetworkResistanceList = strandNetworkResistanceList;


strandResistanceList = 1./[strands(:).strandConductance]';
strandNetworkResistanceList = vectorProject.vectorizedStructure.strandNetworkResistanceList;
strandNetworkResistanceRatioList = strandResistanceList./strandNetworkResistanceList;
figure(1);plot(eigenVectors(:,1),'r.');
figure(2);plot(log10(eigenValues),'g.');
figure(3);plot(log10(strandNetworkResistanceRatioList),'b.');


%%
numVertices = size(allVerts,1);
vertNetworkResistanceRatioList = strandNetworkResistanceRatioList(strandIndices(:,1));


y1 = 600;y2 = maskSize(1)-y1;
x1 = 000;x2 = maskSize(2)-x1;%x2 = 400;
z1 = 0;z2 = 1600;


f1 = and(allVerts(:,1)>y1,allVerts(:,1)<y2);
f2 = and(allVerts(:,2)>x1,allVerts(:,2)<x2);
f3 = and(allVerts(:,3)>z1,allVerts(:,3)<z2);
ff = and(and(f1,f2),f3);

logVertList = log10(vertNetworkResistanceRatioList);
logRatioRange = [-999,[0:0.1:1.5],999];
numColors = numel(logRatioRange);
myColorMap = cool;
figure(1);
p0 = plot3(y1,x1,-z1,'w.');hold on
p0 = plot3(y2,x2,-z2,'w.');hold on
for colorIter = 1:numColors-1,
    thisColor = myColorMap(ceil(64*colorIter/numColors),:);
    range1 = logRatioRange(colorIter);
    range2 = logRatioRange(colorIter+1);
    fr = and(logVertList>=range1 , logVertList<range2);
    ffr = and(ff,fr);
    cc = allVerts(ffr,:);
    p1 = plot3(cc(:,1),cc(:,2),-1*cc(:,3),'k.');
    set(p1,'Color',thisColor);
    
end%for colorIter = 1:numColors-1,
colormap(cool);
colorbar
axis equal

barrels = vectorProject.barrelBoundaryParameters.barrels;
for barrelIter = 1:numel(barrels);
    cb = barrels{barrelIter};
    fb1 = and(cb(:,1)>y1,cb(:,1)<y2);
    fb2 = and(cb(:,2)>x1,cb(:,2)<x2);
    fb3 = and(cb(:,3)>z1,cb(:,3)<z2);
    fbf = and(and(fb1,fb2),fb3);
    cb = cb(fbf,:);
    pb = plot3(cb(:,1),cb(:,2),-1*cb(:,3),'k.');
end%for barrelIter,
    

hold off

    
%%
%Plot median z versus networkResistanceRatio
figure(2);hold off
figure(4);hold off
strandZList = vectorizedStructure.strandMidYXZList(:,3);
plot(strandZList,log10(strandNetworkResistanceRatioList),'r.');
zRangeList = [0:100:1200];
for zIter = 1:numel(zRangeList-1);
    z1 = zRangeList(zIter);
    z2 = zRangeList(zIter+1);
    fz = find(and(strandZList>=z1,strandZList<z2));
    subRatioList = strandNetworkResistanceRatioList(fz);
    subRatioList = (subRatioList(subRatioList<inf));
    logRatioRange = [0:0.05:2];
    hr = hist(log10(subRatioList+0.001),logRatioRange);
    figure(3);bar(logRatioRange,hr);
    cs = cumsum(hr)./sum(hr);
    figure(4);plot(cs,'k-');hold on
    pause
    m = median(subRatioList)
end
figure(4);hold off
    


%%
%Plot networkResistanceRatio vs minBranchFromPlungingArteriole
%%%Stage 2: Find the offshoots of plunging arterioles
flowProject.maskSize = vectorProject.vectorizedStructure.maskSize;
flowProject.Strands = vectorProject.vectorizedStructure.Strands;
flowProject.Vertices = vectorProject.vectorizedStructure.Vertices;
startList = vectorProject.vectorizedStructure.allPlungingArterioleList;
flowProject.baseStrandIDList = startList;
flowProject.maxBranchSearch = 6;
flowProject.stopVoltage = 50.1;%Hardcode;
flowProject.minimumFluxFraction = 1/(2^18);%Hardcode

disp('Calculating downstream paths...');
tic
flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
arterioleOffshootList = find(min(flowProject.downstreamBranchList,[],2)==1);
numArterioleOffshoots = numel(arterioleOffshootList);
strandBranchValueList = min(flowProject.downstreamBranchList,[],2);
strandBranchValueList = min(strandBranchValueList,flowProject.maxBranchSearch+1);
toc
vectorProject.vectorizedStructure.strandBranchValueList = strandBranchValueList;



strandBranchValueList = vectorProject.vectorizedStructure.strandBranchValueList;

figure(2);hold off
snrrList = strandNetworkResistanceRatioList;
snrrList(snrrList==inf)= 1;
plot(strandBranchValueList+rand(size(strandBranchValueList)),log10(snrrList),'m.');


%---
%%
%%
figure(5);hold off
figure(6);hold off
figure(7);hold off
figure(8);hold off

%Plot network resistance from a node to every other node in a z-range
midZ = 700;

allSlope = [];
for midZ = 106:20:1400; disp(midZ);

    
    
plusMinusZ = 5;
strandZList = vectorizedStructure.strandMidYXZList(:,3);
thisZStrandList = find(and(strandZList>=midZ-plusMinusZ,strandZList<=midZ+plusMinusZ));
arterioleTipList = thisZStrandList(1:3:end);
venuleTipList = thisZStrandList(2:3:end);

% arterioleTipList = vectorizedStructure.plungingArterioleTipList;
% venuleTipList = vectorizedStructure.plungingVenuleTipList;


numArterioleTips = numel(arterioleTipList);
numVenuleTips = numel(venuleTipList);


            arterioleTipList(1) = arterioleTipList(ceil(numel(arterioleTipList)/2));


myColorMap = cool;
for aIter = 1:1,
% for aIter = 1:numArterioleTips,
%     myColor = myColorMap(mod(aIter*7,64)+1,:);
    myColor = myColorMap(ceil(4*midZ/100),:);
    aID = arterioleTipList(aIter);
    acc = vectorizedStructure.strandMidYXZList(aID,:);
    sa = strands(aID);
    if sa.strandCurrent<0,
        vertex1ID = sa.StartVertexIndex;
    else%if sa.strandCurrent<0,
        vertex1ID = sa.EndVertexIndex;
    end%if-else sa.strandCurrent<0,
    %---
    resList = [];
    xyDistList = [];
    for vIter = 1:numVenuleTips,
        vID = venuleTipList(vIter);
        vcc = vectorizedStructure.strandMidYXZList(vID,:);
        ccDiff = acc-vcc;
        xyDist = sqrt(sum(ccDiff(1:2).^2));
        %---
        sv = strands(vID);
        if sv.strandCurrent>0,
            vertex2ID = sv.StartVertexIndex;
        else%if sv.strandCurrent<0,
            vertex2ID = sv.EndVertexIndex;
        end%if-else sv.strandCurrent<0,
        %---
        floatingNode1ID = find(floatingNodeIndexList==vertex1ID);
        floatingNode2ID = find(floatingNodeIndexList==vertex2ID);
        runSum = 0;
        mSize = size(eigenVectors,1);
        for iIter = 2:mSize;
            eVectorAlpha = eigenVectors(floatingNode1ID,iIter);
            eVectorBeta = eigenVectors(floatingNode2ID,iIter);
            eVal = eigenValues(iIter);
            cutoff = -1;
            if eVal > cutoff,
                runSum = runSum + (((eVectorAlpha-eVectorBeta).^2)/abs(eVal));
            end
        end%for iIter;
        effectiveResistance = runSum;
if isempty(effectiveResistance),effectiveResistance = inf;end
effectiveResistance = min(effectiveResistance,4e-7);
resList(vIter) = effectiveResistance;
xyDistList(vIter) = xyDist;
        figure(5);p1 = plot(xyDist,effectiveResistance,'r.');hold on
        set(p1,'Color',myColor);
    end%for vIter...
    sortList = sortrows([xyDistList',resList'],1);
    xx = sortList(:,1);
    yy = sortList(:,2);
    p1 = polyfit(xx,yy,1);
    myslope = p1(1);
    allSlope = [allSlope,myslope];
    
    py1 = polyval(p1,xx);
    figure(5);plot(xx,py1,'k-');
    p2 = polyfit(xx,yy,2);
    py2 = polyval(p2,xx);
    figure(5);plot(xx,py2,'k--');hold off
    
    figure(6);pp1 = plot(xx,py1,'r-');hold on
    figure(7);pp2 = plot(xx,py2,'r-');hold on
    figure(8);pp3 = plot(midZ,myslope,'ro');hold on;
    
    set(pp1,'Color',myColor);
    set(pp2,'Color',myColor);
    set(pp3,'Color',myColor);
    

  
end%for aIter ...

end%for midZ


figure(5);hold off
figure(6);hold off        
figure(7);hold off        
figure(8);hold off
%%

figure(1);hold off
figure(2);hold off
% % figure(7);hold off
% % figure(8);hold off

%Plot network resistance from a node as you track outward
midZ = 500;%400
plusMinusZ = 200;%5

allSlope = [];
allUsedStrandList = [];
actualStrandList = [];
allPathDistList = [];
allXYDistList = []
allResList = [];
% % for midZ = 105:10:1400; disp(midZ);

    
y1 = 300;y2 = maskSize(1)-y1;
x1 = 300;x2 = maskSize(2)-x1;

strandZList = vectorizedStructure.strandMidYXZList(:,3);
strandYList = vectorizedStructure.strandMidYXZList(:,1);
strandXList = vectorizedStructure.strandMidYXZList(:,2);

zFlagList = and(strandZList>=midZ-plusMinusZ,strandZList<=midZ+plusMinusZ);
yFlagList = and(strandYList>=y1,strandYList<=y2);
xFlagList = and(strandXList>=x1,strandXList<=x2);

thisZStrandList = find(and(and(zFlagList,yFlagList),xFlagList));

numStart = numel(thisZStrandList);

skipVal = 33;

for startNum = 1:skipVal:numStart,

    startingStrand = thisZStrandList(startNum);
    attachedStrandList1 = [strands([vectorizedStructure.allPlungingArterioleList,vectorizedStructure.allPlungingVenuleList]).StartVertexNeighborStrands];
    attachedStrandList2 = [strands([vectorizedStructure.allPlungingArterioleList,vectorizedStructure.allPlungingVenuleList]).EndVertexNeighborStrands];
    usedStrandList = unique([attachedStrandList1,attachedStrandList2,startingStrand]);


    vertex1ID = strands(startingStrand).StartVertexIndex;
    vertex2ID = strands(startingStrand).EndVertexIndex;
    nextNeighborList = strands(startingStrand).EndVertexNeighborStrands;
    nextNeighborList = nextNeighborList(nextNeighborList~=startingStrand);
    nextNeighborID = nextNeighborList(1);
    sep = vertices.AllVerts(vertex1ID,:) - vertices.AllVerts(vertex2ID,:);
    sepDist = sqrt(sum(sep.^2));


    % for aIter = 1:1,
    % for aIter = 1:numArterioleTips,
    %     myColor = myColorMap(mod(aIter*7,64)+1,:);
    thisStrandID = startingStrand;  
    resList = [];
    xyDistList = [];
    pathDistList = 0;
        for vIter = 1:30,
            floatingNode1ID = find(floatingNodeIndexList==vertex1ID);
            floatingNode2ID = find(floatingNodeIndexList==vertex2ID);
            runSum = 0;
            mSize = size(eigenVectors,1);
            for iIter = 2:mSize;
                eVectorAlpha = eigenVectors(floatingNode1ID,iIter);
                eVectorBeta = eigenVectors(floatingNode2ID,iIter);
                eVal = eigenValues(iIter);
                cutoff = -1;
                if eVal > cutoff,
                    runSum = runSum + (((eVectorAlpha-eVectorBeta).^2)/abs(eVal));
                end
            end%for iIter;
            effectiveResistance = runSum;
            if isempty(effectiveResistance),effectiveResistance = inf;end

            effectiveResistance = min(effectiveResistance,4e-7);
            resList(vIter) = effectiveResistance;
            xyDistList(vIter) = sepDist;
            pathDistList(vIter) = pathDistList(end) + vectorizedStructure.strandLengthList(thisStrandID);

            %----
            %Find next node
            previousStrandID = thisStrandID;
            thisStrandID = nextNeighborID;
            usedStrandList = [usedStrandList,thisStrandID];
            if ismember(previousStrandID,strands(thisStrandID).EndVertexNeighborStrands),
                vertex2ID = strands(thisStrandID).StartVertexIndex;
                nextNeighborList = strands(thisStrandID).StartVertexNeighborStrands;
            else%if ismember(previousStrandID,
                vertex2ID = strands(thisStrandID).EndVertexIndex;
                nextNeighborList = strands(thisStrandID).EndVertexNeighborStrands;
            end%if ismember(previousStrandID,

            nextNeighborList = nextNeighborList(~ismember(nextNeighborList,usedStrandList));
            if isempty(nextNeighborList),break;end
            nextNeighborID = nextNeighborList(1);
            sep = vertices.AllVerts(vertex1ID,:) - vertices.AllVerts(vertex2ID,:);
            sepDist = sqrt(sum(sep.^2));

            actualStrandList = [actualStrandList,thisStrandID];
        end%for vIter...

    figure(1);plot(xyDistList,resList,'r.');hold on
    xlabel('radial distance [um]'),ylabel('network resistance');
    figure(2);plot(pathDistList,resList,'g.');hold on
    xlabel('pathlength distance [um]'),ylabel('network resistance');
    allUsedStrandList = unique([allUsedStrandList,usedStrandList]);
    allPathDistList = [allPathDistList,pathDistList];
    allXYDistList = [allXYDistList,xyDistList];
    allResList = [allResList,resList];
    
    
end%for startNum
sss = [strands(actualStrandList).strandConductance];
xxx = [-10:0.1:0];
h = hist(log10(1./sss),xxx);
figure(998);plot(xxx,h);
xlabel('log10(strandResistance)')
ylabel('occurences');
meanResistance = 1/mean(sss)
[maxVal,maxPos] = max(h);
modeStrandResistance = 10^(xxx(maxPos));


%Calc running median
sortData = sortrows([allPathDistList',allResList'],1);
halfAveSize = 50;
numPoints = size(sortData,1);
medPathDistList = [];
medResList = [];
for pointIter = halfAveSize+1:numPoints-halfAveSize
    medPathDistList(pointIter-halfAveSize) = median(sortData(pointIter-halfAveSize:pointIter+halfAveSize,1));
    medResList(pointIter-halfAveSize) = median(sortData(pointIter-halfAveSize:pointIter+halfAveSize,2));
end
figure(12);plot(allPathDistList,allResList,'g.');hold on
figure(12);plot(medPathDistList,medResList,'k-');
figure(12);plot([0,2500],[modeStrandResistance, modeStrandResistance],'b-');hold off
xlabel('path distance along microvessel network');
ylabel('effective network resistance');
%---
sortData = sortrows([allXYDistList',allResList'],1);
halfAveSize = 50;
numPoints = size(sortData,1);
medXYDistList = [];
medResList = [];
for pointIter = halfAveSize+1:numPoints-halfAveSize
    medXYDistList(pointIter-halfAveSize) = median(sortData(pointIter-halfAveSize:pointIter+halfAveSize,1));
    medResList(pointIter-halfAveSize) = median(sortData(pointIter-halfAveSize:pointIter+halfAveSize,2));
end
figure(11);plot(allXYDistList,allResList,'r.');hold on
figure(11);plot(medXYDistList,medResList,'k-');
figure(11);plot([0,700],[modeStrandResistance, modeStrandResistance],'b-');hold off
xlabel('radial distance');
ylabel('effective network resistance');




%%

 myColor = myColorMap(ceil(4*midZ/100),:);
    aID = arterioleTipList(aIter);
    acc = vectorizedStructure.strandMidYXZList(aID,:);
    sa = strands(aID);
    if sa.strandCurrent<0,
        vertex1ID = sa.StartVertexIndex;
    else%if sa.strandCurrent<0,
        vertex1ID = sa.EndVertexIndex;
    end%if-else sa.strandCurrent<0,
    %---
    resList = [];
    xyDistList = [];
    for vIter = 1:numVenuleTips,
        vID = venuleTipList(vIter);
        vcc = vectorizedStructure.strandMidYXZList(vID,:);
        ccDiff = acc-vcc;
        xyDist = sqrt(sum(ccDiff(1:2).^2));
        %---
        sv = strands(vID);
        if sv.strandCurrent>0,
            vertex2ID = sv.StartVertexIndex;
        else%if sv.strandCurrent<0,
            vertex2ID = sv.EndVertexIndex;
        end%if-else sv.strandCurrent<0,
        %---
        
        
        
        figure(5);p1 = plot(xyDist,effectiveResistance,'r.');hold on
        set(p1,'Color',myColor);
    end%for vIter...
    sortList = sortrows([xyDistList',resList'],1);
    xx = sortList(:,1);
    yy = sortList(:,2);
    p1 = polyfit(xx,yy,1);
    myslope = p1(1);
    allSlope = [allSlope,myslope];
    
    py1 = polyval(p1,xx);
    figure(5);plot(xx,py1,'k-');
    p2 = polyfit(xx,yy,2);
    py2 = polyval(p2,xx);
    figure(5);plot(xx,py2,'k--');hold off
    
    figure(6);pp1 = plot(xx,py1,'r-');hold on
    figure(7);pp2 = plot(xx,py2,'r-');hold on
    figure(8);pp3 = plot(midZ,myslope,'ro');hold on;
    
    set(pp1,'Color',myColor);
    set(pp2,'Color',myColor);
    set(pp3,'Color',myColor);
    

  
end%for aIter ...

end%for midZ


figure(5);hold off
figure(6);hold off        
figure(7);hold off        
figure(8);hold off


%%
%Plot network resistance from arteriole tips to venule tips
arterioleTipList = vectorizedStructure.plungingArterioleTipList;
venuleTipList = vectorizedStructure.plungingVenuleTipList;
numArterioleTips = numel(arterioleTipList);
numVenuleTips = numel(venuleTipList);
figure(5);hold off
myColorMap = cool;
for aIter = 1:numArterioleTips,
    myColor = myColorMap(mod(aIter*7,64)+1,:);
    aID = arterioleTipList(aIter);
    acc = vectorizedStructure.strandMidYXZList(aID,:);
    sa = strands(aID);
    if sa.strandCurrent<0,
        vertex1ID = sa.StartVertexIndex;
    else%if sa.strandCurrent<0,
        vertex1ID = sa.EndVertexIndex;
    end%if-else sa.strandCurrent<0,
    %---
    for vIter = 1:numVenuleTips,
        vID = venuleTipList(vIter);
        vcc = vectorizedStructure.strandMidYXZList(vID,:);
        ccDiff = acc-vcc;
        xyDist = sqrt(sum(ccDiff(1:2).^2));
        %---
        sv = strands(vID);
        if sv.strandCurrent>0,
            vertex2ID = sv.StartVertexIndex;
        else%if sv.strandCurrent<0,
            vertex2ID = sv.EndVertexIndex;
        end%if-else sv.strandCurrent<0,
        %---
        floatingNode1ID = find(floatingNodeIndexList==vertex1ID);
        floatingNode2ID = find(floatingNodeIndexList==vertex2ID);
        runSum = 0;
        mSize = size(eigenVectors,1);
        for iIter = 2:mSize;
            eVectorAlpha = eigenVectors(floatingNode1ID,iIter);
            eVectorBeta = eigenVectors(floatingNode2ID,iIter);
            eVal = eigenValues(iIter);
            cutoff = -1;
            if eVal > cutoff,
                runSum = runSum + (((eVectorAlpha-eVectorBeta).^2)/abs(eVal));
            end
        end%for iIter;
        effectiveResistance = runSum;
effectiveResistance = min(effectiveResistance,4e-7);
        figure(5);p1 = plot(xyDist,effectiveResistance,'r.');hold on
        set(p1,'Color',myColor);
    end%for vIter...
    pause
    hold off
end%for aIter ...
figure(5);hold off
        



%%
%Display the eigenvectors
for eVectorChoice = 1:100,
% eVectorChoice = 3;
numTopComponents = 1000;
eVector = eigenVectors(:,end+1-eVectorChoice);
sortEV = sort(abs(eVector),'descend');
cutoff = sortEV(numTopComponents);
topNodeList = find(abs(eVector)>cutoff);
topNodeValList = eVector(topNodeList);

topNodeVertexList = floatingNodeIndexList(topNodeList);

activeVertList = zeros(size(allVerts,1),1);
% activeVertList([strands(topStrandList).InteriorVertices]) = 1;

activeVertList(topNodeVertexList) = 1;


y1 = 000;y2 = maskSize(1)-y1;
x1 = 000;x2 = maskSize(2)-x1;%x2 = 400;
z1 = 500;z2 = 800;


f1 = and(allVerts(:,1)>y1,allVerts(:,1)<y2);
f2 = and(allVerts(:,2)>x1,allVerts(:,2)<x2);
f3 = and(allVerts(:,3)>z1,allVerts(:,3)<z2);
ff = and(and(f1,f2),f3);
ffa = and(ff,activeVertList);
cc = allVerts(ffa,:);

figure(1);%hold off
p0 = plot3(y1,x1,-z1,'w.');hold on
p0 = plot3(y2,x2,-z2,'w.');hold on
p1 = plot3(cc(:,1),cc(:,2),-1*cc(:,3),'r.');

barrels = vectorProject.barrelBoundaryParameters.barrels;
for barrelIter = 1:numel(barrels);
    cb = barrels{barrelIter};
    fb1 = and(cb(:,1)>y1,cb(:,1)<y2);
    fb2 = and(cb(:,2)>x1,cb(:,2)<x2);
    fb3 = and(cb(:,3)>z1,cb(:,3)<z2);
    fbf = and(and(fb1,fb2),fb3);
    cb = cb(fbf,:);
    pb = plot3(cb(:,1),cb(:,2),-1*cb(:,3),'k.');
end%for barrelIter,
    
hold off
axis equal
campos([1236 720 8737]);

pause;
end
%%
%Plot the network resistance across each strand of the penetrating vessels
%(appropriate for datasets with plunging vessels stripped (set to very high resistance)
%Plot versus depth of strand


allPlungingArterioleList = [vectorizedStructure.allPlungingVenuleList,vectorizedStructure.allPlungingArterioleList];
strandID = allPlungingArterioleList(211);

figure(7);hold off;
for plungeIter = 1:numel(allPlungingArterioleList),
    
    strandID = allPlungingArterioleList(plungeIter);
    strandZ = vectorProject.vectorizedStructure.strandMidYXZList(strandID,3);
    vertex1ID = strands(strandID).StartVertexIndex;
    vertex2ID = strands(strandID).EndVertexIndex;
    strandResistance = 1/strands(strandID).strandConductance;
    floatingNode1ID = find(floatingNodeIndexList==vertex1ID);
    floatingNode2ID = find(floatingNodeIndexList==vertex2ID);
    % % %     strandResistanceList(strandIter) = strandResistance;
    % % %     if isempty(floatingNode1ID) || isempty(floatingNode2ID),
    % % %         strandNetworkResistanceList(strandIter) = strandResistance;
    % % %     else%if isempty(floatingNode1ID) || isempty(floatingNode2ID),

    runSum = 0;
    mSize = size(eigenVectors,1);
    for iIter = 2:mSize;
        eVectorAlpha = eigenVectors(floatingNode1ID,iIter);
        eVectorBeta = eigenVectors(floatingNode2ID,iIter);
        eVal = eigenValues(iIter);
        cutoff = -1;
        if eVal > cutoff,
            runSum = runSum + (((eVectorAlpha-eVectorBeta).^2)/abs(eVal));
        end
    end%for iIter;
    effectiveResistance = runSum;
    figure(7);plot(strandZ,log(effectiveResistance),'r.');hold on;
    
    
end%for plungeIter,

figure(8);plot(vectorProject.vectorizedStructure.strandMidYXZList(:,3),log(1./[vectorProject.vectorizedStructure.Strands(:).strandConductance]),'b.');
figure(9);plot(vectorProject.vectorizedStructure.strandMidYXZList(:,3),log(strandNetworkResistanceList),'g.');
figure(10);plot(vectorProject.vectorizedStructure.strandMidYXZList(:,3),log(strandNetworkResistanceRatioList),'m.');    
sortData = sortrows([vectorProject.vectorizedStructure.strandMidYXZList(:,3),strandNetworkResistanceRatioList],1);
sortData(sortData==inf) = 0;
f = find(sortData(:,2)>1);
sortData = sortData(f,:);
q = medfilt2(sortData,[1000,1],'symmetric');
figure(11);plot(q(:,1),log(q(:,2)),'m-');    
figure(11);plot(sortData(:,2),'r.');    

    
%%
%Plot the network resistance across each penetrating arteriole outlet to the nearest
%penetrating vein outlet;

%Find the offshoots of plunging arterioles
flowProject.maskSize = vectorProject.vectorizedStructure.maskSize;
flowProject.Strands = vectorProject.vectorizedStructure.Strands;
flowProject.Vertices = vectorProject.vectorizedStructure.Vertices;
startList = vectorProject.vectorizedStructure.allPlungingArterioleList;
flowProject.baseStrandIDList = startList;
flowProject.maxBranchSearch = 1;
flowProject.stopVoltage = 50.1;%Hardcode;
flowProject.minimumFluxFraction = 1/(2^18);%Hardcode
disp('Calculating downstream paths...');
flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
arterioleOffshootList = find(min(flowProject.downstreamBranchList,[],2)==1);
numArterioleOffshoots = numel(arterioleOffshootList);
%Find the offshoots of plunging venules
startList = vectorProject.vectorizedStructure.allPlungingVenuleList;
flowProject.baseStrandIDList = startList;
flowProject.maxBranchSearch = 1;
flowProject.stopVoltage = 99;%Hardcode;
flowProject.minimumFluxFraction = 1/(2^18);%Hardcode
disp('Calculating upstream paths...');
flowProject = CalculateUpstreamPathsFromVectorProject(flowProject);
venuleOffshootList = find(min(flowProject.upstreamBranchList,[],2)==1);
numVenuleOffshoots = numel(venuleOffshootList);
%%
figure(7);hold off

vcc = vectorProject.vectorizedStructure.strandMidYXZList(venuleOffshootList,:);

for aIter = 1:numArterioleOffshoots,
    aID = arterioleOffshootList(aIter);
    acc = repmat(vectorProject.vectorizedStructure.strandMidYXZList(aID,:),[size(vcc,1),1]);
    delta = sum((acc-vcc).^2,2);
    sortData = sortrows([delta,venuleOffshootList],1);
    sortVenuleOffshootList = sortData(:,2);
    saz = acc(1,3); 
    sa = strands(aID);
    if sa.strandCurrent<0,
        vertex1ID = sa.StartVertexIndex;
    else%if sa.strandCurrent<0,
        vertex1ID = sa.EndVertexIndex;
    end%if-else sa.strandCurrent<0,
    
    for vIter = 1:10,%numVenuleOffshoots,
        vID = sortVenuleOffshootList(vIter);
        sv = strands(vID);
        if sv.strandCurrent>0,
            vertex2ID = sv.StartVertexIndex;
        else%if sa.strandCurrent>0,
            vertex2ID = sv.EndVertexIndex;
        end%if-else sa.strandCurrent>0,

        floatingNode1ID = find(floatingNodeIndexList==vertex1ID);
        floatingNode2ID = find(floatingNodeIndexList==vertex2ID);

        runSum = 0;
        mSize = size(eigenVectors,1);
        for iIter = 2:mSize;
            eVectorAlpha = eigenVectors(floatingNode1ID,iIter);
            eVectorBeta = eigenVectors(floatingNode2ID,iIter);
            eVal = eigenValues(iIter);
            cutoff = -1;
            if eVal > cutoff,
                runSum = runSum + (((eVectorAlpha-eVectorBeta).^2)/abs(eVal));
            end
        end%for iIter;
        effectiveResistance = runSum;
    figure(7);plot(saz,log(effectiveResistance),'r.');hold on;
    end%for vIter
end%for aIter

%%    
%Plot network resistance from arteriole offshoot as you further away
figure(7);hold off

vcc = vectorProject.vectorizedStructure.strandMidYXZList(venuleOffshootList,:);

for aIter = 1:numArterioleOffshoots,
    aID = arterioleOffshootList(aIter);
    
aID = ceil(numStrands*rand(1));   
    
    acc = repmat(vectorProject.vectorizedStructure.strandMidYXZList(aID,:),[size(vcc,1),1]);
    delta = sum((acc-vcc).^2,2);
    sortData = sortrows([delta,venuleOffshootList],1);
    sortVenuleOffshootList = sortData(:,2);
    startCoords = acc(1,:); 
    sa = strands(aID);
    if sa.strandCurrent>0,
        vertex1ID = sa.StartVertexIndex;
    else%if sa.strandCurrent>0,
        vertex1ID = sa.EndVertexIndex;
    end%if-else sa.strandCurrent>0,
    
    thisStrandID = aID;  

    for vIter = 1:30,%numVenuleOffshoots,

thisStrandID = ceil(numStrands*rand(1));   

        
        sc = strands(thisStrandID);
        if sc.strandCurrent>0,
            vertex2ID = sc.EndVertexIndex;
            downstreamNeighborList = sc.EndVertexNeighborStrands; 
        else%if sc.strandCurrent<0,
            vertex2ID = sc.StartVertexIndex;
            downstreamNeighborList = sc.StartVertexNeighborStrands;
        end%if-else sc.strandCurrent<0,
        downstreamNeighborList = downstreamNeighborList(~ismember(downstreamNeighborList,thisStrandID));
        if isempty(downstreamNeighborList),
            break%break out of vIter loop
        end%if isempty(downstreamNeighborList),
        [minV,minVPos] = min([strands(downstreamNeighborList).MedianVoltage]);
        thisCoords = vectorProject.vectorizedStructure.strandMidYXZList(thisStrandID,:);
        floatingNode1ID = find(floatingNodeIndexList==vertex1ID);
        floatingNode2ID = find(floatingNodeIndexList==vertex2ID);

        runSum = 0;
        mSize = size(eigenVectors,1);
        for iIter = 2:mSize;
            eVectorAlpha = eigenVectors(floatingNode1ID,iIter);
            eVectorBeta = eigenVectors(floatingNode2ID,iIter);
            eVal = eigenValues(iIter);
            cutoff = -1;
            if eVal > cutoff,
                runSum = runSum + (((eVectorAlpha-eVectorBeta).^2)/abs(eVal));
            end
        end%for iIter;
        
        if vIter == 1; baseResistance = 1; runSum; end;
        effectiveResistance = runSum;
        sep = sqrt(sum((startCoords-thisCoords).^2,2));
        thisStrandID = downstreamNeighborList(minVPos);
%         disp(strands(thisStrandID).MedianVoltage)
       
    figure(7);plot(sep,log(effectiveResistance/baseResistance),'r.');hold on;

    end%for vIter
    pause;
    hold off;
end%for aIter

        
