%Open a ...-Flow-VectorProject.

%%
vectorizedStructure = vectorProject.vectorizedStructure;
maskSize = vectorizedStructure.maskSize;
strands = vectorizedStructure.Strands;
vertices = vectorizedStructure.Vertices;
minZ = min(vectorizedStructure.strandMidYXZList(:,3));
zRange = [0:100:2000];
flowProject.maskSize = vectorProject.vectorizedStructure.maskSize;
flowProject.Strands = vectorProject.vectorizedStructure.Strands;
flowProject.Vertices = vectorProject.vectorizedStructure.Vertices;
%---
vectorProjectRootName = vectorProject.vectorProjectRootName;
dataSetName = vectorProjectRootName;
findDash = findstr('-',dataSetName);
if ~isempty(findDash),
    dataSetName = dataSetName(1:findDash(1)-1);
end

%%
%Find the arteriole outlets
flowProject.baseStrandIDList = vectorizedStructure.allPlungingArterioleList;
flowProject.maxBranchSearch = 3;
flowProject.stopVoltage = 50.1;%Hardcode;
flowProject.minimumFluxFraction = 1/(2^18);%Hardcode
%---
arterioleProject = CalculateDownstreamPathsFromVectorProject(flowProject);
%---
dsbl = arterioleProject.downstreamBranchList;
tempOutletFlagMatrix = (dsbl==1);
tempOutletStrandFlagList = max(tempOutletFlagMatrix,[],2);
tempOutletStrandList = find(tempOutletStrandFlagList);
arterioleOutletList = tempOutletStrandList(~ismember(tempOutletStrandList,arterioleProject.baseStrandIDList));
%---
%---
%Find the venule outlets
flowProject.baseStrandIDList = vectorizedStructure.allPlungingVenuleList;
flowProject.maxBranchSearch = 3;
flowProject.stopVoltage = 99;%Hardcode;
flowProject.minimumFluxFraction = 1/(2^18);%Hardcode
%---
venuleProject = CalculateUpstreamPathsFromVectorProject(flowProject);
%---
dsbl = venuleProject.upstreamBranchList;
tempOutletFlagMatrix = (dsbl==1);
tempOutletStrandFlagList = max(tempOutletFlagMatrix,[],2);
tempOutletStrandList = find(tempOutletStrandFlagList);
venuleOutletList = tempOutletStrandList(~ismember(tempOutletStrandList,venuleProject.baseStrandIDList));
%%
%Associate each outlet with a path (penetrating vessel)
arterioleOutletPathIDList = [];
allPlungingArteriolePathList = vectorizedStructure.allPlungingArteriolePathList;
arterioleNumPaths = numel(allPlungingArteriolePathList);
arterioleNumOutlets = numel(arterioleOutletList);
for outletIter = 1:arterioleNumOutlets
    neighbors = [strands(arterioleOutletList(outletIter)).StartVertexNeighborStrands,strands(arterioleOutletList(outletIter)).EndVertexNeighborStrands];
    for pathIter = 1:arterioleNumPaths,
        thisPath = allPlungingArteriolePathList{pathIter};
        if sum(ismember(neighbors,thisPath))>0,
            arterioleOutletPathIDList(outletIter) = pathIter;
        end%if sum...
    end%for pathIter
end% for outletIter
%---
venuleOutletPathIDList = [];
allPlungingVenulePathList = vectorizedStructure.allPlungingVenulePathList;
venuleNumPaths = numel(allPlungingVenulePathList);
venuleNumOutlets = numel(venuleOutletList);
for outletIter = 1:venuleNumOutlets
    neighbors = [strands(venuleOutletList(outletIter)).StartVertexNeighborStrands,strands(venuleOutletList(outletIter)).EndVertexNeighborStrands];
    for pathIter = 1:venuleNumPaths,
        thisPath = allPlungingVenulePathList{pathIter};
        if sum(ismember(neighbors,thisPath))>0,
            venuleOutletPathIDList(outletIter) = pathIter;
        end%if sum...
    end%for pathIter
end% for outletIter

%%
arterioleOutletZList = vectorProject.vectorizedStructure.strandMidYXZList(arterioleOutletList,3)-minZ;
arterioleOutletHist = hist(arterioleOutletZList,zRange);
arterioleOutletCumSum = cumsum(arterioleOutletHist);
arterioleOutletNormHist = arterioleOutletHist./sum(arterioleOutletHist);
venuleOutletZList = vectorProject.vectorizedStructure.strandMidYXZList(venuleOutletList,3)-minZ;
venuleOutletHist = hist(venuleOutletZList,zRange);
venuleOutletCumSum = cumsum(venuleOutletHist);
venuleOutletNormHist = venuleOutletHist./sum(venuleOutletHist);
%%

figure(81);b81v = bar(zRange,venuleOutletHist);hold on
figure(81);b81a = bar(zRange,arterioleOutletHist);hold off
xlabel('Depth from surface [um]');
ylabel('Number of plunging outlets');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
set(b81v,'FaceColor','blue','BarWidth',0.8);
set(b81a,'FaceColor','red','BarWidth',0.4);
%---
figure(82);p82a = plot(zRange,arterioleOutletCumSum./max(arterioleOutletCumSum),'r-');hold on
figure(82);p82v = plot(zRange,venuleOutletCumSum./max(venuleOutletCumSum),'b-');hold off
xlabel('Depth from surface [um]');
ylabel('Cumulative Fraction of outlets');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
%---
figure(83);b83v = bar(zRange,venuleOutletNormHist);hold on
figure(83);b83a = bar(zRange,arterioleOutletNormHist);hold off
xlabel('Depth from surface [um]');
ylabel('Number of plunging outlets');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
set(b83v,'FaceColor','blue','BarWidth',0.8);
set(b83a,'FaceColor','red','BarWidth',0.4);

%%

normZList = vectorProject.vectorizedStructure.strandMidYXZList(:,3);
normZList = normZList - min(normZList);
strandCurrentList = abs([vectorProject.vectorizedStructure.Strands(:).strandCurrent]);
myArterioleOutletZList = normZList(arterioleOutletList);
myArterioleOutletCurrentList = strandCurrentList(arterioleOutletList);

figure(99);plot(myArterioleOutletZList,myArterioleOutletCurrentList,'ro');hold on



%%
%Calculate the maxiumum depth of each penetrating vessel
%As well as the position of the first and second outlet
arterioleMaxPathZ = [];
arterioleTopPathZ = [];
arterioleSecondPathZ = [];
for pathIter = 1:arterioleNumPaths,
    f1 = find(arterioleOutletPathIDList == pathIter);
    thisZList = arterioleOutletZList(f1);
    if isempty(thisZList),thisZList = -99;end
    sortZList = sort(thisZList);
    arterioleMaxPathZ(pathIter) = max(thisZList);
    arterioleTopPathZ(pathIter) = min(thisZList);
    if numel(sortZList) < 2,
        arterioleSecondPathZ(pathIter) = -99;
    else
        arterioleSecondPathZ(pathIter) = sortZList(2);
        figure(84);p84a = plot(arterioleMaxPathZ(pathIter),max(diff(sortZList)),'rs');hold on;
    end
    figure(85);p85a = plot(arterioleMaxPathZ(pathIter),sortZList,'ro');hold on
end
%---
figure(84)
xlabel('Max Z depth of plunging vessel [um]');
ylabel('Median spacing between outlets [um]');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
figure(85);hold off
xlabel('Max Z depth of plunging vessel [um]');
ylabel('Z-depth of all outlets [um]');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
%-----
%-----
venuleMaxPathZ = [];
venuleTopPathZ = [];
venuleSecondPathZ = [];
for pathIter = 1:venuleNumPaths,
    f1 = find(venuleOutletPathIDList == pathIter);
    thisZList = venuleOutletZList(f1);
    if isempty(thisZList),thisZList = -99;end
    sortZList = sort(thisZList);
    venuleMaxPathZ(pathIter) = max(thisZList);
    venuleTopPathZ(pathIter) = min(thisZList);
    if numel(sortZList) < 2,
        venuleSecondPathZ(pathIter) = -99;
    else
        venuleSecondPathZ(pathIter) = sortZList(2);
        figure(84);p84v = plot(venuleMaxPathZ(pathIter),max(diff(sortZList)),'bs');hold on;
    end
    figure(86);p86v = plot(venuleMaxPathZ(pathIter),sortZList,'bo');hold on
end

figure(84);hold off
xlabel('Max Z depth of plunging vessel [um]');
ylabel('Median spacing between outlets [um]');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
figure(86);hold off
xlabel('Max Z depth of plunging vessel [um]');
ylabel('Z-depth of all outlets [um]');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);

%%
if ~exist('megaHzA','var');
    megaHzA = zeros(size(zRange));
    megaHzV = zeros(size(zRange));
    megaHSubZA = zeros(size(zRange));
    megaHSubZV = zeros(size(zRange));
    megaNameList = [];
end%if ~exist

hzA = hist(arterioleMaxPathZ,zRange);
hzV = hist(venuleMaxPathZ,zRange);
megaHzA = megaHzA+hzA;
megaHzV = megaHzV+hzV;
hzA = hzA./sum(hzA);
hzV = hzV./sum(hzV);
figure(71);b1 = bar(zRange,hzV);hold on
figure(71);b2 = bar(zRange,hzA);hold off
set(b1,'FaceColor','blue','BarWidth',0.75);
set(b2,'FaceColor','red','BarWidth',0.4);
xlabel('Max Plunging Depth of Vessel');
ylabel('Fraction of Plunging Vessels');


%Consider the deepest 5 of each

[sortV,vi] = sort(venuleMaxPathZ,'ascend');
[sortA,ai] = sort(arterioleMaxPathZ,'ascend');
subA = [allPlungingArteriolePathList{ai(end-4:end)}];
subV = [allPlungingVenulePathList{vi(end-4:end)}];
subAInd = unique([strands(subA).StartVertexIndex,strands(subA).EndVertexIndex]);
subVInd = unique([strands(subV).StartVertexIndex,strands(subV).EndVertexIndex]);
subAZ = allVerts(subAInd,3);
subVZ = allVerts(subVInd,3);
hSubZA = hist(subAZ,zRange);
hSubZV = hist(subVZ,zRange);
megaHSubZA = megaHSubZA+hSubZA;
megaHSubZV = megaHSubZV+hSubZV;
megaNameList = [megaNameList,shortName,'_'];
hSubZA = hSubZA./sum(hSubZA);
hSubZV = hSubZV./sum(hSubZV);
figure(72);b1 = bar(zRange,hSubZV);hold on
figure(72);b2 = bar(zRange,hSubZA);hold off
set(b1,'FaceColor','blue','BarWidth',0.75);
set(b2,'FaceColor','red','BarWidth',0.4);
xlabel('Depth of Plunging Vessel Outlet [um]');
ylabel('Fraction of Outlets');
title('5 deepest plunging vessels');

%%

hzA = megaHzA./sum(megaHzA);
hzV = megaHzV./sum(megaHzV);
figure(75);b1 = bar(zRange,hzV);hold on
figure(75);b2 = bar(zRange,hzA);hold off
set(b1,'FaceColor','blue','BarWidth',0.75);
set(b2,'FaceColor','red','BarWidth',0.4);
xlabel('Max Plunging Depth of Vessel');
ylabel('Fraction of Plunging Vessels');
title(megaNameList);


hSubZA = megaHSubZA./sum(megaHSubZA);
hSubZV = megaHSubZV./sum(megaHSubZV);
figure(76);b1 = bar(zRange,hSubZV);hold on
figure(76);b2 = bar(zRange,hSubZA);hold off
set(b1,'FaceColor','blue','BarWidth',0.75);
set(b2,'FaceColor','red','BarWidth',0.4);
xlabel('Depth of Plunging Vessel Outlet [um]');
ylabel('Fraction of Outlets');
title('5 deepest plunging vessels');

%%

figure(87);p87v = plot(venuleMaxPathZ,venuleTopPathZ,'b.');hold on;
figure(87);p87a = plot(arterioleMaxPathZ,arterioleTopPathZ,'r.');hold off;
xlabel('Max Z depth of plunging vessel [um]');
ylabel('Z-depth of first outlet [um]');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
set(gca,'yLim',[-100,max(arterioleMaxPathZ)]);
set(gca,'xLim',[-100,max(arterioleMaxPathZ)]);
%---
figure(88);p88v = plot(venuleMaxPathZ,venuleSecondPathZ,'bo');hold on;
figure(88);p88a = plot(arterioleMaxPathZ,arterioleSecondPathZ,'ro');hold off;
xlabel('Max Z depth of plunging vessel [um]');
ylabel('Z-depth of second outlet [um]');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
set(gca,'yLim',[-100,max(arterioleMaxPathZ)]);
set(gca,'xLim',[-100,max(arterioleMaxPathZ)]);

%%
outletStructure = [];
outletStructure.dataSetName = dataSetName;
outletStructure.vectorProjectRootName = vectorProjectRootName;
outletStructure.zRange = zRange;
outletStructure.arterioleMaxPathZ = arterioleMaxPathZ;
outletStructure.arterioleTopPathZ = arterioleTopPathZ;
outletStructure.arterioleSecondPathZ = arterioleSecondPathZ;
outletStructure.arterioleOutletCumSum = arterioleOutletCumSum;
outletStructure.arterioleOutletNormHist = arterioleOutletNormHist;
outletStructure.arterioleOutletHist = arterioleOutletHist;
outletStructure.arterioleOutletPathIDList = arterioleOutletPathIDList;
outletStructure.arterioleOutletZList = arterioleOutletZList;

outletStructure.venuleMaxPathZ = venuleMaxPathZ;
outletStructure.venuleTopPathZ = venuleTopPathZ;
outletStructure.venuleSecondPathZ = venuleSecondPathZ;
outletStructure.venuleOutletCumSum = venuleOutletCumSum;
outletStructure.venuleOutletNormHist = venuleOutletNormHist;
outletStructure.venuleOutletHist = venuleOutletHist;
outletStructure.venuleOutletPathIDList = venuleOutletPathIDList;
outletStructure.venuleOutletZList = venuleOutletZList;
save([dataSetName,'-outletStructure.mat'],'outletStructure');




%%

% % clear allOutletStructure;
% % load('F:\Phil\AOH_VectorProjects\pbAU\VectorProjectFolder\pbAU-outletStructure.mat');
% % allOutletStructure(1) = outletStructure;
% % load('F:\Phil\AOH_VectorProjects\pbAV\VectorProjectFolder\pbAV-outletStructure.mat');
% % allOutletStructure(2) = outletStructure;
% % load('F:\Phil\AOH_VectorProjects\pbCO\VectorProjectFolder\pbCO-outletStructure.mat');
% % allOutletStructure(3) = outletStructure;
% % load('F:\Phil\AOH_VectorProjects\pbDB\VectorProjectFolder\pbDB-outletStructure.mat');
% % allOutletStructure(4) = outletStructure;

numDatasets = 4;
zRange = allOutletStructure(1).zRange;
arterioleMaxPathZ = [];
arterioleTopPathZ = [];
arterioleSecondPathZ = [];
arterioleOutletPathIDList = [];
arterioleOutletZList = [];
arterioleOutletCumSum = (allOutletStructure(1).arterioleOutletCumSum)*0;
arterioleOutletNormHist = (allOutletStructure(1).arterioleOutletNormHist)*0;
arterioleOutletHist = (allOutletStructure(1).arterioleOutletHist)*0;
venuleMaxPathZ = [];
venuleTopPathZ = [];
venuleSecondPathZ = [];
venuleOutletPathIDList = [];
venuleOutletZList = [];
venuleOutletCumSum = (allOutletStructure(1).venuleOutletCumSum)*0;
venuleOutletNormHist = (allOutletStructure(1).venuleOutletNormHist)*0;
venuleOutletHist = (allOutletStructure(1).venuleOutletHist)*0;
vPathCounter = 0;
aPathCounter = 0;
for setIter = 1:numDatasets,
    outletStructure = allOutletStructure(setIter);
    arterioleMaxPathZ = [arterioleMaxPathZ,outletStructure.arterioleMaxPathZ];
    arterioleTopPathZ = [arterioleTopPathZ,outletStructure.arterioleTopPathZ];
    arterioleSecondPathZ = [arterioleSecondPathZ,outletStructure.arterioleSecondPathZ];
    arterioleOutletNormHist = arterioleOutletNormHist + outletStructure.arterioleOutletNormHist;
    arterioleOutletHist = arterioleOutletHist + outletStructure.arterioleOutletHist;
    arterioleOutletZList = [arterioleOutletZList, outletStructure.arterioleOutletZList'];
    venuleMaxPathZ = [venuleMaxPathZ,outletStructure.venuleMaxPathZ];
    venuleTopPathZ = [venuleTopPathZ,outletStructure.venuleTopPathZ];
    venuleSecondPathZ = [venuleSecondPathZ,outletStructure.venuleSecondPathZ];
    venuleOutletNormHist = venuleOutletNormHist + outletStructure.venuleOutletNormHist;
    venuleOutletHist = venuleOutletHist + outletStructure.venuleOutletHist;
    venuleOutletZList = [venuleOutletZList, outletStructure.venuleOutletZList'];
    arterioleOutletCumSum = arterioleOutletCumSum + outletStructure.arterioleOutletCumSum;
    venuleOutletCumSum = venuleOutletCumSum + outletStructure.venuleOutletCumSum;
    arterioleOutletPathIDList = [arterioleOutletPathIDList,outletStructure.arterioleOutletPathIDList+aPathCounter];
    venuleOutletPathIDList = [venuleOutletPathIDList,outletStructure.venuleOutletPathIDList+vPathCounter];
    arterioleTempNumPath = numel(outletStructure.arterioleTopPathZ);
    venuleTempNumPath = numel(outletStructure.venuleTopPathZ);
    aPathCounter = aPathCounter+arterioleTempNumPath;
    vPathCounter = vPathCounter+venuleTempNumPath;
end

%---
dataSetName = 'All Datasets';
%----
figure(81);b81v = bar(zRange,venuleOutletHist);hold on
figure(81);b81a = bar(zRange,arterioleOutletHist);hold off
xlabel('Depth from surface [um]');
ylabel('Number of plunging outlets');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
set(b81v,'FaceColor','blue','BarWidth',0.8);
set(b81a,'FaceColor','red','BarWidth',0.4);
%---
figure(82);p82a = plot(zRange,arterioleOutletCumSum./max(arterioleOutletCumSum),'r-');hold on
figure(82);p82v = plot(zRange,venuleOutletCumSum./max(venuleOutletCumSum),'b-');hold off
xlabel('Depth from surface [um]');
ylabel('Cumulative Fraction of outlets');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
%---
figure(83);b83v = bar(zRange,venuleOutletNormHist);hold on
figure(83);b83a = bar(zRange,arterioleOutletNormHist);hold off
xlabel('Depth from surface [um]');
ylabel('Number of plunging outlets');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
set(b83v,'FaceColor','blue','BarWidth',0.8);
set(b83a,'FaceColor','red','BarWidth',0.4);
%---
%%
arterioleNumPaths = aPathCounter;
for pathIter = 1:arterioleNumPaths,
    f1 = find(arterioleOutletPathIDList == pathIter);
    thisZList = arterioleOutletZList(f1);
    if isempty(thisZList),thisZList = -99;end
    sortZList = sort(thisZList);
    if numel(sortZList) > 1,
        figure(84);p84a = plot(arterioleMaxPathZ(pathIter),max(diff(sortZList)),'rs');hold on;
    end
    figure(85);p85a = plot(arterioleMaxPathZ(pathIter),sortZList,'ro');hold on
end
%---
figure(84)
xlabel('Max Z depth of plunging vessel [um]');
ylabel('Median spacing between outlets [um]');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
figure(85);hold off
xlabel('Max Z depth of plunging vessel [um]');
ylabel('Z-depth of all outlets [um]');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
%-----
%-----
venuleNumPaths = vPathCounter;
for pathIter = 1:venuleNumPaths,
    f1 = find(venuleOutletPathIDList == pathIter);
    thisZList = venuleOutletZList(f1);
    if isempty(thisZList),thisZList = -99;end
    sortZList = sort(thisZList);
    if numel(sortZList) > 1,
        figure(84);p84v = plot(venuleMaxPathZ(pathIter),max(diff(sortZList)),'bs');hold on;
    end
    figure(86);p86v = plot(venuleMaxPathZ(pathIter),sortZList,'bo');hold on
end
figure(84);hold off
xlabel('Max Z depth of plunging vessel [um]');
ylabel('Median spacing between outlets [um]');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
figure(86);hold off
xlabel('Max Z depth of plunging vessel [um]');
ylabel('Z-depth of all outlets [um]');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
%%--
figure(87);p87v = plot(venuleMaxPathZ,venuleTopPathZ,'b.');hold on;
figure(87);p87a = plot(arterioleMaxPathZ,arterioleTopPathZ,'r.');hold off;
xlabel('Max Z depth of plunging vessel [um]');
ylabel('Z-depth of first outlet [um]');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
set(gca,'yLim',[-100,max(arterioleMaxPathZ)]);
set(gca,'xLim',[-100,max(arterioleMaxPathZ)]);
%---
figure(88);p88v = plot(venuleMaxPathZ,venuleSecondPathZ,'bo');hold on;
figure(88);p88a = plot(arterioleMaxPathZ,arterioleSecondPathZ,'ro');hold off;
xlabel('Max Z depth of plunging vessel [um]');
ylabel('Z-depth of second outlet [um]');
title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
set(gca,'yLim',[-100,max(arterioleMaxPathZ)]);
set(gca,'xLim',[-100,max(arterioleMaxPathZ)]);












%%
% % cutOffDepth = 800;%Hardcode
% % 
% % deepPathList = find(maxPathZ>cutOffDepth);
% % shallowPathList = find(maxPathZ<=cutOffDepth);
% % 
% % deepPathOutletList = find(ismember(arterioleOutletPathIDList , deepPathList));
% % shallowPathOutletList = find(ismember(arterioleOutletPathIDList , shallowPathList));
% % 
% % deepPathOutletZ =arterioleOutletZList(deepPathOutletList);
% % shallowPathOutletZ =arterioleOutletZList(shallowPathOutletList);
% % 
% % zRange = [0:50:1500];
% % deepHist = hist(deepPathOutletZ,zRange);
% % shallowHist = hist(shallowPathOutletZ,zRange);
% % 
% % figure(1);bar(zRange,deepHist);
% % figure(2);bar(zRange,shallowHist);
% % 

