



%%



scriptVersion = 'AAA_DisplayPenetratingVesselsScript1x.m'


vectorProject = allProject.db;
strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;
rootName = vectorProject.vectorProjectRootName;
findDash = findstr(rootName,'-');
shortName = rootName(1:findDash(1)-1);
allVerts = vertices.AllVerts;

strandMidYXZList = vectorProject.vectorizedStructure.strandMidYXZList;

strandLabelList = [strands(:).MedianLabel];
surfList = find(and(strandLabelList>=300,strandLabelList<=399)); 
zSurf = strandMidYXZList(surfList,3);
medianZSurf = median(zSurf);
minZSurf = min(zSurf);

strandMidYXZList(:,3) = max(strandMidYXZList(:,3)-minZSurf,0);
allPlungingVenulePathList = vectorProject.vectorizedStructure.allPlungingVenulePathList;
allPlungingArteriolePathList = vectorProject.vectorizedStructure.allPlungingArteriolePathList;

%Eliminate "plunging" vessels that have only one strand
pathLength = [];
for kk = 1:numel(allPlungingVenulePathList);
    pathLength(kk) = numel(allPlungingVenulePathList{kk});
end
ffv = find(pathLength>2);
allPlungingVenulePathList = allPlungingVenulePathList(ffv);
%---
pathLength = [];
for kk = 1:numel(allPlungingArteriolePathList);
    pathLength(kk) = numel(allPlungingArteriolePathList{kk});
end
ffa = find(pathLength>2);
allPlungingArteriolePathList = allPlungingArteriolePathList(ffa);
%---



flowAnalysisParameters = [];
cleanupFlag = 1;




%Find plunging vessel outlets
numPlungingArterioles = numel(allPlungingArteriolePathList);
arterioleOutletZList = [];
arterioleMaxZList = [];
allArterioleOutletZList = [];
for plungingIter = 1:numPlungingArterioles,
%     disp(plungingIter)
    thisPath = allPlungingArteriolePathList{plungingIter};
    outletVertInd = unique([strands(thisPath(2:end)).StartVertexIndex,strands(thisPath(2:end)).EndVertexIndex]);
    outletVertZ = allVerts(outletVertInd,3)-minZSurf;
    arterioleMaxZList(plungingIter) = max(outletVertZ);
    arterioleOutletZList{plungingIter} = outletVertZ';
    allArterioleOutletZList = [allArterioleOutletZList,outletVertZ'];
end%for plungingIter..

numPlungingVenules = numel(allPlungingVenulePathList);
venuleOutletZList = [];
venuleMaxZList = [];
allVenuleOutletZList = [];
for plungingIter = 1:numPlungingVenules,
%     disp(plungingIter)
    thisPath = allPlungingVenulePathList{plungingIter};
    outletVertInd = unique([strands(thisPath(2:end)).StartVertexIndex,strands(thisPath(2:end)).EndVertexIndex]);
    outletVertZ = allVerts(outletVertInd,3)-minZSurf;
    venuleMaxZList(plungingIter) = max(outletVertZ);
    venuleOutletZList{plungingIter} = outletVertZ';
    allVenuleOutletZList = [allVenuleOutletZList,outletVertZ'];
end%for plungingIter..



zRange = [0:100:2000];
arterioleOutletHist = hist(allArterioleOutletZList,zRange);
arterioleOutletCumSum = cumsum(arterioleOutletHist);
arterioleOutletNormHist = arterioleOutletHist./sum(arterioleOutletHist);
venuleOutletHist = hist(allVenuleOutletZList,zRange);
venuleOutletCumSum = cumsum(venuleOutletHist);
venuleOutletNormHist = venuleOutletHist./sum(venuleOutletHist);

figure(70);subplot(3,1,1);b81v = bar(zRange,venuleOutletNormHist);hold on
figure(70);subplot(3,1,1);b81a = bar(zRange,arterioleOutletNormHist);hold off
xlabel('Depth from surface [um]');
ylabel('Number of plunging outlets');
title([shortName,' :  Red = Arteriole , Blue = Venule']);
set(b81v,'FaceColor','blue','BarWidth',0.8);
set(b81a,'FaceColor','red','BarWidth',0.4);
%---



if ~exist('megaHZA','var');
    megaHZA = zeros(size(zRange));
    megaHZV = zeros(size(zRange));
    megaHMaxZA = zeros(size(zRange));
    megaHMaxZV = zeros(size(zRange));
    
    megaHSubZA = zeros(size(zRange));
    megaHSubZV = zeros(size(zRange));
    megaHSubAOutZ = zeros(size(zRange));
    megaHSubVOutZ = zeros(size(zRange));
    megaNameList = [];
end%if ~exist

megaNameList = [megaNameList,shortName,'_']
megaHZA = megaHZA+arterioleOutletHist;
megaHZV = megaHZV+venuleOutletHist;

    
hMaxZA = hist(arterioleMaxZList,zRange);
hMaxZV = hist(venuleMaxZList,zRange);
megaHMaxZA = megaHMaxZA+hMaxZA;
megaHMaxZV = megaHMaxZV+hMaxZV;
normHMaxZA = hMaxZA./sum(hMaxZA);
normHMaxZV = hMaxZV./sum(hMaxZV);
megaNormHMaxZA = megaHMaxZA./sum(megaHMaxZA);
megaNormHMaxZV = megaHMaxZV./sum(megaHMaxZV);
figure(70);subplot(3,1,2);b1 = bar(zRange,normHMaxZV);hold on
figure(70);subplot(3,1,2);b2 = bar(zRange,normHMaxZA);hold off
set(b1,'FaceColor','blue','BarWidth',0.75);
set(b2,'FaceColor','red','BarWidth',0.4);
xlabel('Max Plunging Depth of Vessel');
ylabel('Fraction of Plunging Vessels');
title([shortName,' :  Red = Arteriole , Blue = Venule']);








%Consider the deepest 3 of each

[sortV,vi] = sort(venuleMaxZList,'ascend');
[sortA,ai] = sort(arterioleMaxZList,'ascend');
subAOutZ = [arterioleOutletZList{ai(end-2:end)}];
subVOutZ = [venuleOutletZList{vi(end-2:end)}];
hSubAOutZ = hist(subAOutZ,zRange);
hSubVOutZ = hist(subVOutZ,zRange);
megaHSubAOutZ = megaHSubAOutZ + hSubAOutZ;
megaHSubVOutZ = megaHSubVOutZ + hSubVOutZ;
normHSubAOutZ = hSubAOutZ./sum(hSubAOutZ);
normHSubVOutZ = hSubVOutZ./sum(hSubVOutZ);
megaNormHSubAOutZ = megaHSubAOutZ./sum(megaHSubAOutZ);
megaNormHSubVOutZ = megaHSubVOutZ./sum(megaHSubVOutZ);
figure(70);subplot(3,1,3);b1 = bar(zRange,normHSubVOutZ);hold on
figure(70);subplot(3,1,3);b2 = bar(zRange,normHSubAOutZ);hold off
set(b1,'FaceColor','blue','BarWidth',0.75);
set(b2,'FaceColor','red','BarWidth',0.4);
xlabel('Depth of Plunging Vessel Outlet [um]');
ylabel('Fraction of Outlets');
title([shortName,' :  3 deepest plunging vessels']);



megaNormHZA = megaHZA./sum(megaHZA);
megaNormHZV = megaHZV./sum(megaHZV);

figure(80);subplot(3,1,1);b81v = bar(zRange,megaNormHZV);hold on
figure(80);subplot(3,1,1);b81a = bar(zRange,megaNormHZA);hold off
xlabel('Depth from surface [um]');
ylabel('Number of plunging outlets');
title([megaNameList,' :  Red = Arteriole , Blue = Venule']);
set(b81v,'FaceColor','blue','BarWidth',0.8);
set(b81a,'FaceColor','red','BarWidth',0.4);
%---
figure(80);subplot(3,1,2);b1 = bar(zRange,megaNormHMaxZV);hold on
figure(80);subplot(3,1,2);b2 = bar(zRange,megaNormHMaxZA);hold off
set(b1,'FaceColor','blue','BarWidth',0.75);
set(b2,'FaceColor','red','BarWidth',0.4);
xlabel('Max Plunging Depth of Vessel');
ylabel('Fraction of Plunging Vessels');
title([megaNameList,' :  Red = Arteriole , Blue = Venule']);
%---
figure(80);subplot(3,1,3);b1 = bar(zRange,megaNormHSubVOutZ);hold on
figure(80);subplot(3,1,3);b2 = bar(zRange,megaNormHSubAOutZ);hold off
set(b1,'FaceColor','blue','BarWidth',0.75);
set(b2,'FaceColor','red','BarWidth',0.4);
xlabel('Depth of Plunging Vessel Outlet [um]');
ylabel('Fraction of Outlets');
title([megaNameList,' :  3 deepest plunging vessels']);




%%
% % 
% % hzA = megaHzA./sum(megaHzA);
% % hzV = megaHzV./sum(megaHzV);
% % figure(75);b1 = bar(zRange,hzV);hold on
% % figure(75);b2 = bar(zRange,hzA);hold off
% % set(b1,'FaceColor','blue','BarWidth',0.75);
% % set(b2,'FaceColor','red','BarWidth',0.4);
% % xlabel('Max Plunging Depth of Vessel');
% % ylabel('Fraction of Plunging Vessels');
% % title(megaNameList);
% % 
% % 
% % hSubZA = megaHSubZA./sum(megaHSubZA);
% % hSubZV = megaHSubZV./sum(megaHSubZV);
% % figure(76);b1 = bar(zRange,hSubZV);hold on
% % figure(76);b2 = bar(zRange,hSubZA);hold off
% % set(b1,'FaceColor','blue','BarWidth',0.75);
% % set(b2,'FaceColor','red','BarWidth',0.4);
% % xlabel('Depth of Plunging Vessel Outlet [um]');
% % ylabel('Fraction of Outlets');
% % title('5 deepest plunging vessels');



%%
%%
% % % % % % % % 
% % % % % % % % 
% % % % % % % % figure(81);b81v = bar(zRange,venuleOutletHist);hold on
% % % % % % % % figure(81);b81a = bar(zRange,arterioleOutletHist);hold off
% % % % % % % % xlabel('Depth from surface [um]');
% % % % % % % % ylabel('Number of plunging outlets');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % set(b81v,'FaceColor','blue','BarWidth',0.8);
% % % % % % % % set(b81a,'FaceColor','red','BarWidth',0.4);
% % % % % % % % %---
% % % % % % % % figure(82);p82a = plot(zRange,arterioleOutletCumSum./max(arterioleOutletCumSum),'r-');hold on
% % % % % % % % figure(82);p82v = plot(zRange,venuleOutletCumSum./max(venuleOutletCumSum),'b-');hold off
% % % % % % % % xlabel('Depth from surface [um]');
% % % % % % % % ylabel('Cumulative Fraction of outlets');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % %---
% % % % % % % % figure(83);b83v = bar(zRange,venuleOutletNormHist);hold on
% % % % % % % % figure(83);b83a = bar(zRange,arterioleOutletNormHist);hold off
% % % % % % % % xlabel('Depth from surface [um]');
% % % % % % % % ylabel('Number of plunging outlets');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % set(b83v,'FaceColor','blue','BarWidth',0.8);
% % % % % % % % set(b83a,'FaceColor','red','BarWidth',0.4);
% % % % % % % % 
% % % % % % % % %%
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % %%
% % % % % % % % normZList = vectorProject.vectorizedStructure.strandMidYXZList(:,3);
% % % % % % % % normZList = normZList - min(normZList);
% % % % % % % % strandCurrentList = abs([vectorProject.vectorizedStructure.Strands(:).strandCurrent]);
% % % % % % % % myArterioleOutletZList = normZList(arterioleOutletList);
% % % % % % % % myArterioleOutletCurrentList = strandCurrentList(arterioleOutletList);
% % % % % % % % 
% % % % % % % % figure(99);plot(myArterioleOutletZList,myArterioleOutletCurrentList,'ro');hold on
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % %%
% % % % % % % % %Calculate the maxiumum depth of each penetrating vessel
% % % % % % % % %As well as the position of the first and second outlet
% % % % % % % % arterioleMaxPathZ = [];
% % % % % % % % arterioleTopPathZ = [];
% % % % % % % % arterioleSecondPathZ = [];
% % % % % % % % for pathIter = 1:arterioleNumPaths,
% % % % % % % %     f1 = find(arterioleOutletPathIDList == pathIter);
% % % % % % % %     thisZList = arterioleOutletZList(f1);
% % % % % % % %     if isempty(thisZList),thisZList = -99;end
% % % % % % % %     sortZList = sort(thisZList);
% % % % % % % %     arterioleMaxPathZ(pathIter) = max(thisZList);
% % % % % % % %     arterioleTopPathZ(pathIter) = min(thisZList);
% % % % % % % %     if numel(sortZList) < 2,
% % % % % % % %         arterioleSecondPathZ(pathIter) = -99;
% % % % % % % %     else
% % % % % % % %         arterioleSecondPathZ(pathIter) = sortZList(2);
% % % % % % % %         figure(84);p84a = plot(arterioleMaxPathZ(pathIter),max(diff(sortZList)),'rs');hold on;
% % % % % % % %     end
% % % % % % % %     figure(85);p85a = plot(arterioleMaxPathZ(pathIter),sortZList,'ro');hold on
% % % % % % % % end
% % % % % % % % %---
% % % % % % % % figure(84)
% % % % % % % % xlabel('Max Z depth of plunging vessel [um]');
% % % % % % % % ylabel('Median spacing between outlets [um]');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % figure(85);hold off
% % % % % % % % xlabel('Max Z depth of plunging vessel [um]');
% % % % % % % % ylabel('Z-depth of all outlets [um]');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % %-----
% % % % % % % % %-----
% % % % % % % % venuleMaxPathZ = [];
% % % % % % % % venuleTopPathZ = [];
% % % % % % % % venuleSecondPathZ = [];
% % % % % % % % for pathIter = 1:venuleNumPaths,
% % % % % % % %     f1 = find(venuleOutletPathIDList == pathIter);
% % % % % % % %     thisZList = venuleOutletZList(f1);
% % % % % % % %     if isempty(thisZList),thisZList = -99;end
% % % % % % % %     sortZList = sort(thisZList);
% % % % % % % %     venuleMaxPathZ(pathIter) = max(thisZList);
% % % % % % % %     venuleTopPathZ(pathIter) = min(thisZList);
% % % % % % % %     if numel(sortZList) < 2,
% % % % % % % %         venuleSecondPathZ(pathIter) = -99;
% % % % % % % %     else
% % % % % % % %         venuleSecondPathZ(pathIter) = sortZList(2);
% % % % % % % %         figure(84);p84v = plot(venuleMaxPathZ(pathIter),max(diff(sortZList)),'bs');hold on;
% % % % % % % %     end
% % % % % % % %     figure(86);p86v = plot(venuleMaxPathZ(pathIter),sortZList,'bo');hold on
% % % % % % % % end
% % % % % % % % 
% % % % % % % % figure(84);hold off
% % % % % % % % xlabel('Max Z depth of plunging vessel [um]');
% % % % % % % % ylabel('Median spacing between outlets [um]');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % figure(86);hold off
% % % % % % % % xlabel('Max Z depth of plunging vessel [um]');
% % % % % % % % ylabel('Z-depth of all outlets [um]');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % 
% % % % % % % % %%
% % % % % % % % if ~exist('megaHzA','var');
% % % % % % % %     megaHzA = zeros(size(zRange));
% % % % % % % %     megaHzV = zeros(size(zRange));
% % % % % % % %     megaHSubZA = zeros(size(zRange));
% % % % % % % %     megaHSubZV = zeros(size(zRange));
% % % % % % % %     megaNameList = [];
% % % % % % % % end%if ~exist
% % % % % % % % 
% % % % % % % % hzA = hist(arterioleMaxPathZ,zRange);
% % % % % % % % hzV = hist(venuleMaxPathZ,zRange);
% % % % % % % % megaHzA = megaHzA+hzA;
% % % % % % % % megaHzV = megaHzV+hzV;
% % % % % % % % hzA = hzA./sum(hzA);
% % % % % % % % hzV = hzV./sum(hzV);
% % % % % % % % figure(71);b1 = bar(zRange,hzV);hold on
% % % % % % % % figure(71);b2 = bar(zRange,hzA);hold off
% % % % % % % % set(b1,'FaceColor','blue','BarWidth',0.75);
% % % % % % % % set(b2,'FaceColor','red','BarWidth',0.4);
% % % % % % % % xlabel('Max Plunging Depth of Vessel');
% % % % % % % % ylabel('Fraction of Plunging Vessels');
% % % % % % % % 
% % % % % % % % 
% % % % % % % % %Consider the deepest 5 of each
% % % % % % % % 
% % % % % % % % [sortV,vi] = sort(venuleMaxPathZ,'ascend');
% % % % % % % % [sortA,ai] = sort(arterioleMaxPathZ,'ascend');
% % % % % % % % subA = [allPlungingArteriolePathList{ai(end-4:end)}];
% % % % % % % % subV = [allPlungingVenulePathList{vi(end-4:end)}];
% % % % % % % % subAInd = unique([strands(subA).StartVertexIndex,strands(subA).EndVertexIndex]);
% % % % % % % % subVInd = unique([strands(subV).StartVertexIndex,strands(subV).EndVertexIndex]);
% % % % % % % % subAZ = allVerts(subAInd,3);
% % % % % % % % subVZ = allVerts(subVInd,3);
% % % % % % % % hSubZA = hist(subAZ,zRange);
% % % % % % % % hSubZV = hist(subVZ,zRange);
% % % % % % % % megaHSubZA = megaHSubZA+hSubZA;
% % % % % % % % megaHSubZV = megaHSubZV+hSubZV;
% % % % % % % % megaNameList = [megaNameList,shortName,'_'];
% % % % % % % % hSubZA = hSubZA./sum(hSubZA);
% % % % % % % % hSubZV = hSubZV./sum(hSubZV);
% % % % % % % % figure(72);b1 = bar(zRange,hSubZV);hold on
% % % % % % % % figure(72);b2 = bar(zRange,hSubZA);hold off
% % % % % % % % set(b1,'FaceColor','blue','BarWidth',0.75);
% % % % % % % % set(b2,'FaceColor','red','BarWidth',0.4);
% % % % % % % % xlabel('Depth of Plunging Vessel Outlet [um]');
% % % % % % % % ylabel('Fraction of Outlets');
% % % % % % % % title('5 deepest plunging vessels');
% % % % % % % % 
% % % % % % % % %%
% % % % % % % % 
% % % % % % % % hzA = megaHzA./sum(megaHzA);
% % % % % % % % hzV = megaHzV./sum(megaHzV);
% % % % % % % % figure(75);b1 = bar(zRange,hzV);hold on
% % % % % % % % figure(75);b2 = bar(zRange,hzA);hold off
% % % % % % % % set(b1,'FaceColor','blue','BarWidth',0.75);
% % % % % % % % set(b2,'FaceColor','red','BarWidth',0.4);
% % % % % % % % xlabel('Max Plunging Depth of Vessel');
% % % % % % % % ylabel('Fraction of Plunging Vessels');
% % % % % % % % title(megaNameList);
% % % % % % % % 
% % % % % % % % 
% % % % % % % % hSubZA = megaHSubZA./sum(megaHSubZA);
% % % % % % % % hSubZV = megaHSubZV./sum(megaHSubZV);
% % % % % % % % figure(76);b1 = bar(zRange,hSubZV);hold on
% % % % % % % % figure(76);b2 = bar(zRange,hSubZA);hold off
% % % % % % % % set(b1,'FaceColor','blue','BarWidth',0.75);
% % % % % % % % set(b2,'FaceColor','red','BarWidth',0.4);
% % % % % % % % xlabel('Depth of Plunging Vessel Outlet [um]');
% % % % % % % % ylabel('Fraction of Outlets');
% % % % % % % % title('5 deepest plunging vessels');
% % % % % % % % 
% % % % % % % % %%
% % % % % % % % 
% % % % % % % % figure(87);p87v = plot(venuleMaxPathZ,venuleTopPathZ,'b.');hold on;
% % % % % % % % figure(87);p87a = plot(arterioleMaxPathZ,arterioleTopPathZ,'r.');hold off;
% % % % % % % % xlabel('Max Z depth of plunging vessel [um]');
% % % % % % % % ylabel('Z-depth of first outlet [um]');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % set(gca,'yLim',[-100,max(arterioleMaxPathZ)]);
% % % % % % % % set(gca,'xLim',[-100,max(arterioleMaxPathZ)]);
% % % % % % % % %---
% % % % % % % % figure(88);p88v = plot(venuleMaxPathZ,venuleSecondPathZ,'bo');hold on;
% % % % % % % % figure(88);p88a = plot(arterioleMaxPathZ,arterioleSecondPathZ,'ro');hold off;
% % % % % % % % xlabel('Max Z depth of plunging vessel [um]');
% % % % % % % % ylabel('Z-depth of second outlet [um]');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % set(gca,'yLim',[-100,max(arterioleMaxPathZ)]);
% % % % % % % % set(gca,'xLim',[-100,max(arterioleMaxPathZ)]);
% % % % % % % % 
% % % % % % % % %%
% % % % % % % % outletStructure = [];
% % % % % % % % outletStructure.dataSetName = dataSetName;
% % % % % % % % outletStructure.vectorProjectRootName = vectorProjectRootName;
% % % % % % % % outletStructure.zRange = zRange;
% % % % % % % % outletStructure.arterioleMaxPathZ = arterioleMaxPathZ;
% % % % % % % % outletStructure.arterioleTopPathZ = arterioleTopPathZ;
% % % % % % % % outletStructure.arterioleSecondPathZ = arterioleSecondPathZ;
% % % % % % % % outletStructure.arterioleOutletCumSum = arterioleOutletCumSum;
% % % % % % % % outletStructure.arterioleOutletNormHist = arterioleOutletNormHist;
% % % % % % % % outletStructure.arterioleOutletHist = arterioleOutletHist;
% % % % % % % % outletStructure.arterioleOutletPathIDList = arterioleOutletPathIDList;
% % % % % % % % outletStructure.arterioleOutletZList = arterioleOutletZList;
% % % % % % % % 
% % % % % % % % outletStructure.venuleMaxPathZ = venuleMaxPathZ;
% % % % % % % % outletStructure.venuleTopPathZ = venuleTopPathZ;
% % % % % % % % outletStructure.venuleSecondPathZ = venuleSecondPathZ;
% % % % % % % % outletStructure.venuleOutletCumSum = venuleOutletCumSum;
% % % % % % % % outletStructure.venuleOutletNormHist = venuleOutletNormHist;
% % % % % % % % outletStructure.venuleOutletHist = venuleOutletHist;
% % % % % % % % outletStructure.venuleOutletPathIDList = venuleOutletPathIDList;
% % % % % % % % outletStructure.venuleOutletZList = venuleOutletZList;
% % % % % % % % save([dataSetName,'-outletStructure.mat'],'outletStructure');
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % %%
% % % % % % % % 
% % % % % % % % % % clear allOutletStructure;
% % % % % % % % % % load('F:\Phil\AOH_VectorProjects\pbAU\VectorProjectFolder\pbAU-outletStructure.mat');
% % % % % % % % % % allOutletStructure(1) = outletStructure;
% % % % % % % % % % load('F:\Phil\AOH_VectorProjects\pbAV\VectorProjectFolder\pbAV-outletStructure.mat');
% % % % % % % % % % allOutletStructure(2) = outletStructure;
% % % % % % % % % % load('F:\Phil\AOH_VectorProjects\pbCO\VectorProjectFolder\pbCO-outletStructure.mat');
% % % % % % % % % % allOutletStructure(3) = outletStructure;
% % % % % % % % % % load('F:\Phil\AOH_VectorProjects\pbDB\VectorProjectFolder\pbDB-outletStructure.mat');
% % % % % % % % % % allOutletStructure(4) = outletStructure;
% % % % % % % % 
% % % % % % % % numDatasets = 4;
% % % % % % % % zRange = allOutletStructure(1).zRange;
% % % % % % % % arterioleMaxPathZ = [];
% % % % % % % % arterioleTopPathZ = [];
% % % % % % % % arterioleSecondPathZ = [];
% % % % % % % % arterioleOutletPathIDList = [];
% % % % % % % % arterioleOutletZList = [];
% % % % % % % % arterioleOutletCumSum = (allOutletStructure(1).arterioleOutletCumSum)*0;
% % % % % % % % arterioleOutletNormHist = (allOutletStructure(1).arterioleOutletNormHist)*0;
% % % % % % % % arterioleOutletHist = (allOutletStructure(1).arterioleOutletHist)*0;
% % % % % % % % venuleMaxPathZ = [];
% % % % % % % % venuleTopPathZ = [];
% % % % % % % % venuleSecondPathZ = [];
% % % % % % % % venuleOutletPathIDList = [];
% % % % % % % % venuleOutletZList = [];
% % % % % % % % venuleOutletCumSum = (allOutletStructure(1).venuleOutletCumSum)*0;
% % % % % % % % venuleOutletNormHist = (allOutletStructure(1).venuleOutletNormHist)*0;
% % % % % % % % venuleOutletHist = (allOutletStructure(1).venuleOutletHist)*0;
% % % % % % % % vPathCounter = 0;
% % % % % % % % aPathCounter = 0;
% % % % % % % % for setIter = 1:numDatasets,
% % % % % % % %     outletStructure = allOutletStructure(setIter);
% % % % % % % %     arterioleMaxPathZ = [arterioleMaxPathZ,outletStructure.arterioleMaxPathZ];
% % % % % % % %     arterioleTopPathZ = [arterioleTopPathZ,outletStructure.arterioleTopPathZ];
% % % % % % % %     arterioleSecondPathZ = [arterioleSecondPathZ,outletStructure.arterioleSecondPathZ];
% % % % % % % %     arterioleOutletNormHist = arterioleOutletNormHist + outletStructure.arterioleOutletNormHist;
% % % % % % % %     arterioleOutletHist = arterioleOutletHist + outletStructure.arterioleOutletHist;
% % % % % % % %     arterioleOutletZList = [arterioleOutletZList, outletStructure.arterioleOutletZList'];
% % % % % % % %     venuleMaxPathZ = [venuleMaxPathZ,outletStructure.venuleMaxPathZ];
% % % % % % % %     venuleTopPathZ = [venuleTopPathZ,outletStructure.venuleTopPathZ];
% % % % % % % %     venuleSecondPathZ = [venuleSecondPathZ,outletStructure.venuleSecondPathZ];
% % % % % % % %     venuleOutletNormHist = venuleOutletNormHist + outletStructure.venuleOutletNormHist;
% % % % % % % %     venuleOutletHist = venuleOutletHist + outletStructure.venuleOutletHist;
% % % % % % % %     venuleOutletZList = [venuleOutletZList, outletStructure.venuleOutletZList'];
% % % % % % % %     arterioleOutletCumSum = arterioleOutletCumSum + outletStructure.arterioleOutletCumSum;
% % % % % % % %     venuleOutletCumSum = venuleOutletCumSum + outletStructure.venuleOutletCumSum;
% % % % % % % %     arterioleOutletPathIDList = [arterioleOutletPathIDList,outletStructure.arterioleOutletPathIDList+aPathCounter];
% % % % % % % %     venuleOutletPathIDList = [venuleOutletPathIDList,outletStructure.venuleOutletPathIDList+vPathCounter];
% % % % % % % %     arterioleTempNumPath = numel(outletStructure.arterioleTopPathZ);
% % % % % % % %     venuleTempNumPath = numel(outletStructure.venuleTopPathZ);
% % % % % % % %     aPathCounter = aPathCounter+arterioleTempNumPath;
% % % % % % % %     vPathCounter = vPathCounter+venuleTempNumPath;
% % % % % % % % end
% % % % % % % % 
% % % % % % % % %---
% % % % % % % % dataSetName = 'All Datasets';
% % % % % % % % %----
% % % % % % % % figure(81);b81v = bar(zRange,venuleOutletHist);hold on
% % % % % % % % figure(81);b81a = bar(zRange,arterioleOutletHist);hold off
% % % % % % % % xlabel('Depth from surface [um]');
% % % % % % % % ylabel('Number of plunging outlets');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % set(b81v,'FaceColor','blue','BarWidth',0.8);
% % % % % % % % set(b81a,'FaceColor','red','BarWidth',0.4);
% % % % % % % % %---
% % % % % % % % figure(82);p82a = plot(zRange,arterioleOutletCumSum./max(arterioleOutletCumSum),'r-');hold on
% % % % % % % % figure(82);p82v = plot(zRange,venuleOutletCumSum./max(venuleOutletCumSum),'b-');hold off
% % % % % % % % xlabel('Depth from surface [um]');
% % % % % % % % ylabel('Cumulative Fraction of outlets');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % %---
% % % % % % % % figure(83);b83v = bar(zRange,venuleOutletNormHist);hold on
% % % % % % % % figure(83);b83a = bar(zRange,arterioleOutletNormHist);hold off
% % % % % % % % xlabel('Depth from surface [um]');
% % % % % % % % ylabel('Number of plunging outlets');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % set(b83v,'FaceColor','blue','BarWidth',0.8);
% % % % % % % % set(b83a,'FaceColor','red','BarWidth',0.4);
% % % % % % % % %---
% % % % % % % % %%
% % % % % % % % arterioleNumPaths = aPathCounter;
% % % % % % % % for pathIter = 1:arterioleNumPaths,
% % % % % % % %     f1 = find(arterioleOutletPathIDList == pathIter);
% % % % % % % %     thisZList = arterioleOutletZList(f1);
% % % % % % % %     if isempty(thisZList),thisZList = -99;end
% % % % % % % %     sortZList = sort(thisZList);
% % % % % % % %     if numel(sortZList) > 1,
% % % % % % % %         figure(84);p84a = plot(arterioleMaxPathZ(pathIter),max(diff(sortZList)),'rs');hold on;
% % % % % % % %     end
% % % % % % % %     figure(85);p85a = plot(arterioleMaxPathZ(pathIter),sortZList,'ro');hold on
% % % % % % % % end
% % % % % % % % %---
% % % % % % % % figure(84)
% % % % % % % % xlabel('Max Z depth of plunging vessel [um]');
% % % % % % % % ylabel('Median spacing between outlets [um]');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % figure(85);hold off
% % % % % % % % xlabel('Max Z depth of plunging vessel [um]');
% % % % % % % % ylabel('Z-depth of all outlets [um]');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % %-----
% % % % % % % % %-----
% % % % % % % % venuleNumPaths = vPathCounter;
% % % % % % % % for pathIter = 1:venuleNumPaths,
% % % % % % % %     f1 = find(venuleOutletPathIDList == pathIter);
% % % % % % % %     thisZList = venuleOutletZList(f1);
% % % % % % % %     if isempty(thisZList),thisZList = -99;end
% % % % % % % %     sortZList = sort(thisZList);
% % % % % % % %     if numel(sortZList) > 1,
% % % % % % % %         figure(84);p84v = plot(venuleMaxPathZ(pathIter),max(diff(sortZList)),'bs');hold on;
% % % % % % % %     end
% % % % % % % %     figure(86);p86v = plot(venuleMaxPathZ(pathIter),sortZList,'bo');hold on
% % % % % % % % end
% % % % % % % % figure(84);hold off
% % % % % % % % xlabel('Max Z depth of plunging vessel [um]');
% % % % % % % % ylabel('Median spacing between outlets [um]');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % figure(86);hold off
% % % % % % % % xlabel('Max Z depth of plunging vessel [um]');
% % % % % % % % ylabel('Z-depth of all outlets [um]');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % %%--
% % % % % % % % figure(87);p87v = plot(venuleMaxPathZ,venuleTopPathZ,'b.');hold on;
% % % % % % % % figure(87);p87a = plot(arterioleMaxPathZ,arterioleTopPathZ,'r.');hold off;
% % % % % % % % xlabel('Max Z depth of plunging vessel [um]');
% % % % % % % % ylabel('Z-depth of first outlet [um]');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % set(gca,'yLim',[-100,max(arterioleMaxPathZ)]);
% % % % % % % % set(gca,'xLim',[-100,max(arterioleMaxPathZ)]);
% % % % % % % % %---
% % % % % % % % figure(88);p88v = plot(venuleMaxPathZ,venuleSecondPathZ,'bo');hold on;
% % % % % % % % figure(88);p88a = plot(arterioleMaxPathZ,arterioleSecondPathZ,'ro');hold off;
% % % % % % % % xlabel('Max Z depth of plunging vessel [um]');
% % % % % % % % ylabel('Z-depth of second outlet [um]');
% % % % % % % % title([dataSetName,' :  Red = Arteriole , Blue = Venule']);
% % % % % % % % set(gca,'yLim',[-100,max(arterioleMaxPathZ)]);
% % % % % % % % set(gca,'xLim',[-100,max(arterioleMaxPathZ)]);
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % 
% % % % % % % % %%
% % % % % % % % % % cutOffDepth = 800;%Hardcode
% % % % % % % % % % 
% % % % % % % % % % deepPathList = find(maxPathZ>cutOffDepth);
% % % % % % % % % % shallowPathList = find(maxPathZ<=cutOffDepth);
% % % % % % % % % % 
% % % % % % % % % % deepPathOutletList = find(ismember(arterioleOutletPathIDList , deepPathList));
% % % % % % % % % % shallowPathOutletList = find(ismember(arterioleOutletPathIDList , shallowPathList));
% % % % % % % % % % 
% % % % % % % % % % deepPathOutletZ =arterioleOutletZList(deepPathOutletList);
% % % % % % % % % % shallowPathOutletZ =arterioleOutletZList(shallowPathOutletList);
% % % % % % % % % % 
% % % % % % % % % % zRange = [0:50:1500];
% % % % % % % % % % deepHist = hist(deepPathOutletZ,zRange);
% % % % % % % % % % shallowHist = hist(shallowPathOutletZ,zRange);
% % % % % % % % % % 
% % % % % % % % % % figure(1);bar(zRange,deepHist);
% % % % % % % % % % figure(2);bar(zRange,shallowHist);
% % % % % % % % % % 
% % % % % % % % 
