



%%



scriptVersion = 'AAA_DisplayPenetratingVesselsScript1x.m'


vectorProject = allProject.db;
strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;
rootName = vectorProject.vectorProjectRootName;
findDash = findstr(rootName,'-');
shortName = rootName(1:findDash(1)-1);
allVerts = vertices.AllVerts;

strandMidYXZList = vectorProject.vectorizedStructure.strandMidYXZList;

strandLabelList = [strands(:).MedianLabel];
surfList = find(and(strandLabelList>=300,strandLabelList<=399)); 
zSurf = strandMidYXZList(surfList,3);
medianZSurf = median(zSurf);
minZSurf = min(zSurf);

strandMidYXZList(:,3) = max(strandMidYXZList(:,3)-minZSurf,0);
allPlungingVenulePathList = vectorProject.vectorizedStructure.allPlungingVenulePathList;
allPlungingArteriolePathList = vectorProject.vectorizedStructure.allPlungingArteriolePathList;

%Eliminate "plunging" vessels that have only one strand
pathLength = [];
for kk = 1:numel(allPlungingVenulePathList);
    pathLength(kk) = numel(allPlungingVenulePathList{kk});
end
ffv = find(pathLength>2);
allPlungingVenulePathList = allPlungingVenulePathList(ffv);
%---
pathLength = [];
for kk = 1:numel(allPlungingArteriolePathList);
    pathLength(kk) = numel(allPlungingArteriolePathList{kk});
end
ffa = find(pathLength>2);
allPlungingArteriolePathList = allPlungingArteriolePathList(ffa);
%---



flowAnalysisParameters = [];
cleanupFlag = 1;




%Find plunging vessel outlets
numPlungingArterioles = numel(allPlungingArteriolePathList);
arterioleOutletZList = [];
arterioleMaxZList = [];
allArterioleOutletZList = [];
for plungingIter = 1:numPlungingArterioles,
%     disp(plungingIter)
    thisPath = allPlungingArteriolePathList{plungingIter};
    outletVertInd = unique([strands(thisPath(2:end)).StartVertexIndex,strands(thisPath(2:end)).EndVertexIndex]);
    outletVertZ = allVerts(outletVertInd,3)-minZSurf;
    arterioleMaxZList(plungingIter) = max(outletVertZ);
    arterioleOutletZList{plungingIter} = outletVertZ';
    allArterioleOutletZList = [allArterioleOutletZList,outletVertZ'];
end%for plungingIter..

numPlungingVenules = numel(allPlungingVenulePathList);
venuleOutletZList = [];
venuleMaxZList = [];
allVenuleOutletZList = [];
for plungingIter = 1:numPlungingVenules,
%     disp(plungingIter)
    thisPath = allPlungingVenulePathList{plungingIter};
    outletVertInd = unique([strands(thisPath(2:end)).StartVertexIndex,strands(thisPath(2:end)).EndVertexIndex]);
    outletVertZ = allVerts(outletVertInd,3)-minZSurf;
    venuleMaxZList(plungingIter) = max(outletVertZ);
    venuleOutletZList{plungingIter} = outletVertZ';
    allVenuleOutletZList = [allVenuleOutletZList,outletVertZ'];
end%for plungingIter..



zRange = [0:100:2000];
arterioleOutletHist = hist(allArterioleOutletZList,zRange);
arterioleOutletCumSum = cumsum(arterioleOutletHist);
arterioleOutletNormHist = arterioleOutletHist./sum(arterioleOutletHist);
venuleOutletHist = hist(allVenuleOutletZList,zRange);
venuleOutletCumSum = cumsum(venuleOutletHist);
venuleOutletNormHist = venuleOutletHist./sum(venuleOutletHist);

figure(70);subplot(3,1,1);b81v = bar(zRange,venuleOutletNormHist);hold on
figure(70);subplot(3,1,1);b81a = bar(zRange,arterioleOutletNormHist);hold off
xlabel('Depth from surface [um]');
ylabel('Number of plunging outlets');
title([shortName,' :  Red = Arteriole , Blue = Venule']);
set(b81v,'FaceColor','blue','BarWidth',0.8);
set(b81a,'FaceColor','red','BarWidth',0.4);
%---



if ~exist('megaHZA','var');
    megaHZA = zeros(size(zRange));
    megaHZV = zeros(size(zRange));
    megaHMaxZA = zeros(size(zRange));
    megaHMaxZV = zeros(size(zRange));
    
    megaHSubZA = zeros(size(zRange));
    megaHSubZV = zeros(size(zRange));
    megaHSubAOutZ = zeros(size(zRange));
    megaHSubVOutZ = zeros(size(zRange));
    megaArterioleOutletZList = [];
    megaVenuleOutletZList = [];
    megaNameList = [];
end%if ~exist

megaNameList = [megaNameList,shortName,'-']
megaHZA = megaHZA+arterioleOutletHist;
megaHZV = megaHZV+venuleOutletHist;
megaArterioleOutletZList = [megaArterioleOutletZList,allArterioleOutletZList];
megaVenuleOutletZList = [megaVenuleOutletZList,allVenuleOutletZList];
    
hMaxZA = hist(arterioleMaxZList,zRange);
hMaxZV = hist(venuleMaxZList,zRange);
megaHMaxZA = megaHMaxZA+hMaxZA;
megaHMaxZV = megaHMaxZV+hMaxZV;
normHMaxZA = hMaxZA./sum(hMaxZA);
normHMaxZV = hMaxZV./sum(hMaxZV);
megaNormHMaxZA = megaHMaxZA./sum(megaHMaxZA);
megaNormHMaxZV = megaHMaxZV./sum(megaHMaxZV);
figure(70);subplot(3,1,2);b1 = bar(zRange,normHMaxZV);hold on
figure(70);subplot(3,1,2);b2 = bar(zRange,normHMaxZA);hold off
set(b1,'FaceColor','blue','BarWidth',0.75);
set(b2,'FaceColor','red','BarWidth',0.4);
xlabel('Max Plunging Depth of Vessel');
ylabel('Fraction of Plunging Vessels');
title([shortName,' :  Red = Arteriole , Blue = Venule']);








%Consider the deepest 3 of each

[sortV,vi] = sort(venuleMaxZList,'ascend');
[sortA,ai] = sort(arterioleMaxZList,'ascend');
subAOutZ = [arterioleOutletZList{ai(end-2:end)}];
subVOutZ = [venuleOutletZList{vi(end-2:end)}];
hSubAOutZ = hist(subAOutZ,zRange);
hSubVOutZ = hist(subVOutZ,zRange);
megaHSubAOutZ = megaHSubAOutZ + hSubAOutZ;
megaHSubVOutZ = megaHSubVOutZ + hSubVOutZ;
normHSubAOutZ = hSubAOutZ./sum(hSubAOutZ);
normHSubVOutZ = hSubVOutZ./sum(hSubVOutZ);
megaNormHSubAOutZ = megaHSubAOutZ./sum(megaHSubAOutZ);
megaNormHSubVOutZ = megaHSubVOutZ./sum(megaHSubVOutZ);
figure(70);subplot(3,1,3);b1 = bar(zRange,normHSubVOutZ);hold on
figure(70);subplot(3,1,3);b2 = bar(zRange,normHSubAOutZ);hold off
set(b1,'FaceColor','blue','BarWidth',0.75);
set(b2,'FaceColor','red','BarWidth',0.4);
xlabel('Depth of Plunging Vessel Outlet [um]');
ylabel('Fraction of Outlets');
title([shortName,' :  3 deepest plunging vessels']);



megaNormHZA = megaHZA./sum(megaHZA);
megaNormHZV = megaHZV./sum(megaHZV);

figure(80);subplot(3,1,1);b81v = bar(zRange,megaNormHZV);hold on
figure(80);subplot(3,1,1);b81a = bar(zRange,megaNormHZA);hold off
xlabel('Depth from surface [um]');
ylabel('Number of plunging outlets');
title([megaNameList,' :  Red = Arteriole , Blue = Venule']);
set(b81v,'FaceColor','blue','BarWidth',0.8);
set(b81a,'FaceColor','red','BarWidth',0.4);
%---
figure(80);subplot(3,1,2);b1 = bar(zRange,megaNormHMaxZV);hold on
figure(80);subplot(3,1,2);b2 = bar(zRange,megaNormHMaxZA);hold off
set(b1,'FaceColor','blue','BarWidth',0.75);
set(b2,'FaceColor','red','BarWidth',0.4);
xlabel('Max Plunging Depth of Vessel');
ylabel('Fraction of Plunging Vessels');
title([megaNameList,' :  Red = Arteriole , Blue = Venule']);
%---
figure(80);subplot(3,1,3);b1 = bar(zRange,megaNormHSubVOutZ);hold on
figure(80);subplot(3,1,3);b2 = bar(zRange,megaNormHSubAOutZ);hold off
set(b1,'FaceColor','blue','BarWidth',0.75);
set(b2,'FaceColor','red','BarWidth',0.4);
xlabel('Depth of Plunging Vessel Outlet [um]');
ylabel('Fraction of Outlets');
title([megaNameList,' :  3 deepest plunging vessels']);

%%

figData.arterioleOutletZList = megaArterioleOutletZList;
figData.venuleOutletZList = megaVenuleOutletZList;
figData.note1 = 'plungingVesselOutletVersusDepth';
figData.note2 = '4 datasets: AU,AV,CO,DB ;';
figData.note4 = 'vessel stats calculated with AA_CalculateZDensityOfPenetratingVesselOutlets4.m';


zRange = [0:100:2000];
arterioleOutletHist = hist(figData.arterioleOutletZList,zRange);
venuleOutletHist = hist(figData.venuleOutletZList,zRange);

normArterioleHist = arterioleOutletHist./sum(arterioleOutletHist);
normVenuleHist = venuleOutletHist./sum(venuleOutletHist);



figure(8);b81v = bar(zRange,normVenuleHist);hold on
figure(8);b81a = bar(zRange,normArterioleHist);hold off
xlabel('Depth from surface [um]');
ylabel('Fraction of plunging outlets');
title([megaNameList,' :  Red = Arteriole , Blue = Venule']);
set(b81v,'FaceColor','blue','BarWidth',0.8);
set(b81a,'FaceColor','red','BarWidth',0.4);
