
% % cd('F:\Phil\AOH_VectorProjects')
% % load('F:\Phil\AOH_VectorProjects\allProject002b.mat')


vectorProject = allProject.av;
vectorizedStructure = vectorProject.vectorizedStructure;
strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;
rootName = vectorProject.vectorProjectRootName;
findDash = findstr(rootName,'-');
shortName = rootName(1:findDash(1)-1);
circuitAnalysisParameters = vectorProject.circuitAnalysisParameters;
floatingNodeIndexList = circuitAnalysisParameters.floatingNodeIndexList;
allVerts = vertices.AllVerts;
allVoltages = vertices.AllVoltages;
strandIndices = full(vertices.StrandIndices);
plungingVenuleTipList = vectorizedStructure.plungingVenuleTipList;
plungingVenuleTipList = vectorizedStructure.plungingVenuleTipList;
allPlungingArterioleList = vectorizedStructure.allPlungingArterioleList; 
allPlungingVenuleList = vectorizedStructure.allPlungingVenuleList; 
allPlungingVenulePathList = vectorizedStructure.allPlungingVenulePathList;
allPlungingVenulePathList = vectorizedStructure.allPlungingVenulePathList;
barrels = vectorProject.barrelBoundaryParameters.barrels;
numTips = numel(plungingVenuleTipList);
numBarrels = numel(barrels);
numStrands = numel(strands);


%%
%Generate a color for each tip
myColorMap = zeros(64,3);
% % myColorMap(1,:) = [0.5 0.8 0.1];
myColorMap(1,:) = [0 0 0];
for kk = 2:64,
    myColorMap(kk,:) = myColorMap(kk-1,:) + [0.37 0.13 0.19];
end;

myColorMap = mod(myColorMap,1);
myColorMap(end,:) = [1,1,1];


%%

allBarrelMask = false(maskSize(1),maskSize(2));
barrelDensity = [];
numBarrels = numel(barrels);
minZ1 = inf;
maxZ2 = 0;

for barrelChoice = 1:numBarrels,

    thisBarrel = barrels{barrelChoice};
    flatBarrel = thisBarrel(thisBarrel(:,3)==thisBarrel(end,3),:);
    barrelMask = false(maskSize(1),maskSize(2));
    flatBarrelInd = sub2ind(size(barrelMask),flatBarrel(:,1),flatBarrel(:,2));
    barrelMask(flatBarrelInd) = 1;
    figure(1);imagesc(barrelMask);colormap(gray)
    barrelMask = imclose(barrelMask,circle2d(7));
    figure(1);imagesc(barrelMask);colormap(gray);
    barrelMask = imfill(barrelMask,'holes');
    figure(1);imagesc(barrelMask);colormap(gray);
    barrelCOM = mean(flatBarrel);
    allBarrelMask = max(allBarrelMask,barrelMask);
end%for barrelChoice...
allBarrelOutlineMask = allBarrelMask - imerode(allBarrelMask,circle2d(11)>0);
[bcc1,bcc2] = ind2sub([maskSize(1),maskSize(2)],find(allBarrelOutlineMask));

figure(1);imagesc(allBarrelOutlineMask);

%%
%Generate a barrel strand list that contains the strands that lie within each barrel
barrelStrandList = [];
allIndList = sub2ind(maskSize,allVerts(:,1),allVerts(:,2),allVerts(:,3));
for barrelIter = 1:numBarrels,
    thisBarrel = barrels{barrelIter};
    minZ = min(thisBarrel(:,3));
    maxZ = max(thisBarrel(:,3));
    flatBarrel = thisBarrel(thisBarrel(:,3)==thisBarrel(end,3),:);
    barrelMask = false(maskSize(1),maskSize(2));
    flatBarrelInd = sub2ind(size(barrelMask),flatBarrel(:,1),flatBarrel(:,2));
    barrelMask(flatBarrelInd) = 1;
    figure(1);imagesc(barrelMask);colormap(gray)
    barrelMask = imclose(barrelMask,circle2d(7));
    figure(1);imagesc(barrelMask);colormap(gray);
    barrelMask = imfill(barrelMask,'holes');
    figure(1);imagesc(barrelMask);colormap(gray);
    internalPoints = find(barrelMask);
    numInternalPoints = numel(internalPoints);
    allInternalPoints = zeros(1e6,1);
    myCounter = 0;
    disp([minZ,maxZ]);
    for zIter = minZ:maxZ,
        allInternalPoints(1+myCounter:numInternalPoints+myCounter) = internalPoints +(maskSize(1)*maskSize(2)*(zIter-1));
        myCounter = myCounter+numInternalPoints;
    end;
    allInternalPoints = allInternalPoints(allInternalPoints~=0);
    internalVerts = find(ismember(allIndList,allInternalPoints));
    tempList = unique(strandIndices(internalVerts,:));
    internalStrandList = tempList(tempList~=0);
    barrelStrandList{barrelIter} = internalStrandList;
end


%%
%Find all plunging arteriole outlet strands
steiList = [strands(allPlungingArterioleList).StartToEndIndices];
attachedStrands = unique(strandIndices(steiList,:));
attachedStrands = attachedStrands(~ismember(attachedStrands,[allPlungingArterioleList,0]));
attachedStrandLabels = [strands(attachedStrands).MedianLabel];
attachedStrands = attachedStrands(~ismember(attachedStrandLabels,[300:399]));
arterioleOutlets = attachedStrands;

%%




%%
%Propagate downstream from a barrelStrand
%Adapted from the DyeTracer_ver01b code

barrelIter = 6;
dyeSourceStrandList = arterioleOutlets;


minimumFluxFraction = 1/3000;
stopVoltage = 50.5;
divFactor = 3;
%-----
displayFlag = 1;
overlayDisplayFlag = 0;
pauseFlag = 0;
normalizeToStartStrandCurrentFlag = 1; %multiply the flow by the startStrand current


% % pathColorStorageList = zeros(size(AllVerts));
% % myColorSpace = GenerateColorSpace(numStartStrands);

maxFlowFractionStorageList = zeros(size(strands));
associatedStartStorageList = zeros(size(strands));
%-----               
numStartStrands = numel(dyeSourceStrandList);
plotCounter = 0;
for startStrandIter = 1:numStartStrands;
    allDaughterStrandList = [];
    startStrandID = dyeSourceStrandList(startStrandIter);
    disp([startStrandIter, startStrandID]);


    startStrand = strands(startStrandID);
    ssiv = startStrand.InteriorVertices;
    startStrandCoordList = allVerts(ssiv,:);
    strandNodes = [startStrand.StartVertexIndex,startStrand.EndVertexIndex];
    nodeVoltages = allVoltages(strandNodes);
    [highVoltage,highVoltagePos] = max(nodeVoltages);
    [lowVoltage,lowVoltagePos] = min(nodeVoltages);
    highNode = strandNodes(highVoltagePos);
    lowNode = strandNodes(strandNodes~=highNode);

    pathActiveFlagList = false(1,100000);
    pathActiveFlagList(1) = 1;
    numActivePath = sum(pathActiveFlagList);
    strandPathList = {};
    nodePathList = {};
    fractionPathList = {};
    strandPathList{1} = startStrandID;
    nodePathList{1} = [highNode lowNode];
    fractionPathList{1} = 1;
    strandFlowList = zeros(numStrands,1);
    strandFlowList(startStrandID) = 1;
    strandVoltageList = zeros(numStrands,1);
    strandVoltageList(1) = mean([highVoltage,lowVoltage]);
    pathCounter = 1;
    keepGoingFlag = 1;
    myCamPos = [0 0 0];
    myCamUp = [-1 0 0];
    myCamVA = 6.6086;

    while (keepGoingFlag == 1),
        myCount = 0;
        for pathIter = find(pathActiveFlagList == 1);
             
            currentPathID = pathIter;
            currentStrandPath = strandPathList{pathIter};
            currentNodePath = nodePathList{pathIter};
            currentFractionPath = fractionPathList{pathIter};
            currentStrandID = currentStrandPath(end);
            currentStrand = strands(currentStrandID);
            %-----
            strandNodes = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
            nodeVoltages = allVoltages(strandNodes);
            [highVoltage,highVoltagePos] = max(nodeVoltages);
            [lowVoltage,lowVoltagePos] = min(nodeVoltages);
            highNode = strandNodes(highVoltagePos);
            lowNode = strandNodes(strandNodes~=highNode);
            lowAttachedStrands = strandIndices(lowNode,:);
            lowAttachedStrands = lowAttachedStrands(lowAttachedStrands~=0);
            lowAttachedStrands = lowAttachedStrands(lowAttachedStrands~=currentStrandID);
            numDaughterPaths = numel(lowAttachedStrands);
            strandVoltageList(currentStrandID) = mean([highVoltage,lowVoltage]);
            stopTracingFlag = 0;
            if numDaughterPaths == 0, stopTracingFlag = 1; end
            if lowVoltage < stopVoltage, stopTracingFlag = 1; end
            if currentFractionPath(end) < minimumFluxFraction, stopTracingFlag = 1; end
            if stopTracingFlag == 1,
                pathActiveFlagList(currentPathID) = 0;
            else%if stopTracingFlag == 1,
                motherCurrent = abs(currentStrand.strandCurrent);
                validDaughterCounter = 0;
                validDaughterStrandList = [];
                validDaughterCurrentList = zeros(1,numDaughterPaths);
                validDaughterPathIndexList = [];
                for daughterIter = 1:numDaughterPaths,
                    %Check to see if the daughter strand leads to a lower voltage
                    candidateDaughterID = lowAttachedStrands(daughterIter);
                    candidateDaughterStrand = strands(candidateDaughterID);
                    candidateDaughterNodes = [candidateDaughterStrand.StartVertexIndex,candidateDaughterStrand.EndVertexIndex];
                    candidateDaughterOtherNode = candidateDaughterNodes(candidateDaughterNodes~=lowNode);
                    candidateDaughterOtherVoltage = allVoltages(candidateDaughterOtherNode);
                    if candidateDaughterOtherVoltage < lowVoltage,
                        %if so, add the strand to a path
                        myCount = myCount + 1;
                        validDaughterCounter = validDaughterCounter + 1;
                        validDaughterStrandList(validDaughterCounter) = candidateDaughterID;
                        validDaughterCurrentList(validDaughterCounter) = abs(candidateDaughterStrand.strandCurrent);
                        if validDaughterCounter == 1,
                            %if this is the first valid daughter, add to current path
                            strandPathList{currentPathID} = [currentStrandPath, lowAttachedStrands(daughterIter)];
                            nodePathList{currentPathID} = [currentNodePath, candidateDaughterOtherNode];
                            validDaughterPathIndexList(1) = currentPathID;
                        else%if validDaughterCounter == 1,
                            %if not the first daughter, duplicate path up to this point, and
                            %make new path
                            pathCounter = pathCounter + 1;
                            strandPathList{pathCounter} = [currentStrandPath, lowAttachedStrands(daughterIter)];
                            nodePathList{pathCounter} = [currentNodePath, candidateDaughterOtherNode];
                            fractionPathList{pathCounter} = currentFractionPath;
                            pathActiveFlagList(pathCounter) = 1;  
                            validDaughterPathIndexList(validDaughterCounter) = pathCounter;
                        end%if-else validDaughterCounter == 1,
                    end%if candidateDaughterOtherVoltage < lowVoltage,
                end%for daughterIter = 1:numDaughterPaths,
                allDaughterStrandList = unique([allDaughterStrandList, validDaughterStrandList]);
                clear daughterIter
                %-----
                daughterSumCurrent = sum(validDaughterCurrentList);
                fractionalDaughterCurrent = validDaughterCurrentList./daughterSumCurrent;
                for validDaughterIter = 1:validDaughterCounter;
                    validDaughterSegmentID = validDaughterStrandList(validDaughterIter);
                    validDaughterFraction = currentFractionPath(end).*fractionalDaughterCurrent(validDaughterIter);
                    fractionPathList{validDaughterPathIndexList(validDaughterIter)} = [currentFractionPath , validDaughterFraction];
                    strandFlowList(validDaughterSegmentID) = strandFlowList(validDaughterSegmentID) + validDaughterFraction;
                end%for validDaughterIter = 1:validDaughterCounter;
            end%if-else stopTracingFlag == 1,
        end%for pathIter = find(pathActiveFlagList == 1);
        if sum(pathActiveFlagList) == 0, keepGoingFlag = 0;end
        if myCount == 0, keepGoingFlag = 0;end
    end%while (keepGoingFlag == 1),
    %-----
    z1 = 400; z2 = 550;
    allPointsList = [strands(allDaughterStrandList).InteriorVertices];
    cc = allVerts(allPointsList,:);
    ccf = find(and(cc(:,3)>z1, cc(:,3)<z2));
    cc = cc(ccf,:);
    if size(cc,1) >300,
        plotCounter =plotCounter +1;
        figure(98);subplot(11,14,plotCounter);
        p1 = plot(cc(:,1),cc(:,2),'b.');hold on;
        set(p1,'MarkerSize',1);
        p2 = plot(bcc1,bcc2,'k.');hold off
        set(p2,'MarkerSize',1);
        set(gca,'XLim',[0,1100],'YLim',[0,1100])
        pause(0.5);
    end%if size(cc,1)>50
    
end%for startStrandIter = 1:...



%%
