
% % cd('F:\Phil\AOH_VectorProjects')
% % load('F:\Phil\AOH_VectorProjects\allProject002b.mat')


vectorProject = allProject.co;
vectorizedStructure = vectorProject.vectorizedStructure;
strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;
rootName = vectorProject.vectorProjectRootName;
findDash = findstr(rootName,'-');
shortName = rootName(1:findDash(1)-1);
circuitAnalysisParameters = vectorProject.circuitAnalysisParameters;
floatingNodeIndexList = circuitAnalysisParameters.floatingNodeIndexList;
allVerts = vertices.AllVerts;
allVoltages = vertices.AllVoltages;
strandIndices = full(vertices.StrandIndices);
plungingArterioleTipList = vectorizedStructure.plungingArterioleTipList;
allPlungingArterioleList = vectorizedStructure.allPlungingArterioleList; 
allPlungingArteriolePathList = vectorizedStructure.allPlungingArteriolePathList;
barrels = vectorProject.barrelBoundaryParameters.barrels;
numTips = numel(plungingArterioleTipList);
numBarrels = numel(barrels);
numStrands = numel(strands);


%%
%Generate a color for each tip
myColorMap = zeros(64,3);
% % myColorMap(1,:) = [0.5 0.8 0.1];
myColorMap(1,:) = [0 0 0];
for kk = 2:64,
    myColorMap(kk,:) = myColorMap(kk-1,:) + [0.37 0.13 0.19];
end;

myColorMap = mod(myColorMap,1);
myColorMap(end,:) = [1,1,1];


%%

allBarrelMask = false(maskSize(1),maskSize(2));
barrelDensity = [];
numBarrels = numel(barrels);
minZ1 = inf;
maxZ2 = 0;

for barrelChoice = 1:numBarrels,

    thisBarrel = barrels{barrelChoice};
    flatBarrel = thisBarrel(thisBarrel(:,3)==thisBarrel(end,3),:);
    barrelMask = false(maskSize(1),maskSize(2));
    flatBarrelInd = sub2ind(size(barrelMask),flatBarrel(:,1),flatBarrel(:,2));
    barrelMask(flatBarrelInd) = 1;
    figure(1);imagesc(barrelMask);colormap(gray)
    barrelMask = imclose(barrelMask,circle2d(7));
    figure(1);imagesc(barrelMask);colormap(gray);
    barrelMask = imfill(barrelMask,'holes');
    figure(1);imagesc(barrelMask);colormap(gray);
    barrelCOM = mean(flatBarrel);
    allBarrelMask = max(allBarrelMask,barrelMask);

end%for barrelChoice...

figure(1);imagesc(allBarrelMask);



%%
%Make a surface tesselation that only considers those plunging vessels that reach >400 um
plungingMustGoPast = 400;
tipCoordList = [];
flatMask = false(maskSize(1),maskSize(2));
tipCounter = 0;
tipUsedList = [];
for tipIter = 1:numTips,
    thisTipID = plungingArterioleTipList(tipIter);
    thisPlungingStrandList = allPlungingArteriolePathList{tipIter};
    allStrandZ = allVerts([strands(thisPlungingStrandList).StartToEndIndices],3);
    maxZ = max(allStrandZ);
    if maxZ > plungingMustGoPast,
        tipCounter = tipCounter +1;
        thisStrand = strands(thisTipID);
        iv = thisStrand.InteriorVertices;
        if thisStrand.strandCurrent < 0,
            iv = fliplr(iv);
        end%%if thisStrand.strandCurrent < 0,
        cc = allVerts(iv,:);
        figure(101);subplot(1,2,1);p1a = plot(cc(:,1),-1*cc(:,3),'r.');axis equal
        figure(101);subplot(1,2,2);p1b = plot(cc(:,2),-1*cc(:,3),'r.');axis equal
        title(['Tip #',num2str(tipIter),',  StrandID #',num2str(thisTipID)]);
    %     pause
        vertSep = 5;
        thresholdSlope  = 1;
        numVerts = numel(iv);
        if numVerts>vertSep,
            subCC2 = cc(vertSep+1:end,:);
            subCC1 = cc(1:end-vertSep,:);
            diffCC = subCC2-subCC1;
            drList = sqrt(diffCC(:,1).^2 + diffCC(:,2).^2);
            dzList = diffCC(:,3);
            subSlope = abs(dzList./drList);
            highSlope = find(subSlope>thresholdSlope);
            if isempty(highSlope),
                highInd = numVerts;
                disp([tipIter]);
            else%if isempty(highSlope),
                highInd = highSlope(1) + ceil(vertSep/2);
            end%if isempty(highSlope),
        else%if numVerts>vertSep,
            highInd = numVerts;
        end%if numVerts>vertSep
        tipCoord = cc(highInd,:);
        tipUsedList(tipCounter) = tipIter;
        tipCoordList(tipCounter,:) = tipCoord;
        figure(101);subplot(1,2,1);hold on;p1a = plot(tipCoord(:,1),-1*tipCoord(:,3),'g.');axis equal;hold off
        set(gca,'XLim',[tipCoord(:,1)-20,tipCoord(:,1)+20],'YLim',-1*[tipCoord(:,3)+100,tipCoord(:,3)-100])
        figure(101);subplot(1,2,2);hold on;p1b = plot(tipCoord(:,2),-1*tipCoord(:,3),'g.');axis equal;hold off
        set(gca,'XLim',[tipCoord(:,2)-20,tipCoord(:,2)+20],'YLim',-1*[tipCoord(:,3)+100,tipCoord(:,3)-100])
       pause(0.1);%pause
        %---
        flatMask(tipCoord(1),tipCoord(2)) = 1;
    end%if maxZ >...
end%for tipIter

bw = bwdist(flatMask);
tempWatershed = watershed(bw);
numTipsUsed = numel(tipUsedList);
deepArteryAtSurfaceTess = zeros(size(tempWatershed));
for tipIter = 1:numel(tipUsedList);
    tempValue = tempWatershed(tipCoordList(tipIter,1),tipCoordList(tipIter,2));
    trueValue = tipUsedList(tipIter);
    deepArteryAtSurfaceTess(find(tempWatershed==tempValue))=trueValue;
end%for tipIter
deepArteryAtSurfaceTess(1,1) = 0;%normalize colors
deepArteryAtSurfaceTess(1,2) = numTips;%normalize colors
figure(21);colormap(myColorMap);
figure(21);imagesc(deepArteryAtSurfaceTess);
title(['Surface Tesselation of Arteries that plunge to at least ',num2str(plungingMustGoPast),' microns']);


deepArteryAtSurfaceTessBorder = zeros(size(flatMask));
for tipIter = 1:numel(tipUsedList);
    trueValue = tipUsedList(tipIter);
    tempImage = deepArteryAtSurfaceTess==trueValue;
    tempImage2 = imerode(tempImage,ones(11,11)>0);
    borderImage = tempImage-tempImage2;
    figure(99);imagesc(borderImage);pause(0.1);
    deepArteryAtSurfaceTessBorder = deepArteryAtSurfaceTessBorder +borderImage*trueValue;
end
deepArteryAtSurfaceTessBorder(1,1) = 0;%normalize colors
deepArteryAtSurfaceTessBorder(1,2) = numTips;%normalize colors
figure(31);imagesc(deepArteryAtSurfaceTessBorder);colormap(myColorMap);
title(['Surface Tesselation of Arteries that plunge to at least ',num2str(plungingMustGoPast),' microns']);



%%
%%
%Make a deep tesselation that only considers those plunging vessels that reach the barrel
%field depth
deepCCList = [];
medBarrelZList = [];
deepMask = false(maskSize(1),maskSize(2));
for barrelIter = 1:numBarrels;
    thisBarrel = barrels{barrelIter};
    medBarrelZList(barrelIter) = median(thisBarrel(:,3));
end
barrelLayerZ = ceil(median(medBarrelZList));
%-----
tipCoordList = [];
flatMask = false(maskSize(1),maskSize(2));
tipCounter = 0;
tipUsedList = [];
for tipIter = 1:numTips,
    thisTipID = plungingArterioleTipList(tipIter);
    thisPlungingStrandList = allPlungingArteriolePathList{tipIter};
    allStrandCoords = allVerts([strands(thisPlungingStrandList).StartToEndIndices],:);
    allStrandZ = allStrandCoords(:,3);
    maxZ = max(allStrandZ);
    if maxZ > barrelLayerZ,
        tipCounter = tipCounter +1;
        plungingTipID = tipIter;
        barrelLayerIndex = find(allStrandZ == barrelLayerZ);
        cc = allStrandCoords(barrelLayerIndex(1),:);
        tipUsedList(tipCounter) = tipIter;
        tipCoordList(tipCounter,:) = cc;
        %---
        flatMask(cc(1),cc(2)) = 1;
    end%if maxZ >...
end%for tipIter

bw = bwdist(flatMask);
tempWatershed = watershed(bw);
numTipsUsed = numel(tipUsedList);
deepArteryAtBarrelTess = zeros(size(tempWatershed));
for tipIter = 1:numel(tipUsedList);
    tempValue = tempWatershed(tipCoordList(tipIter,1),tipCoordList(tipIter,2));
    trueValue = tipUsedList(tipIter);
    f = find(tempWatershed==tempValue);
    disp([tipIter, tempValue,numel(f)]);
    deepArteryAtBarrelTess(find(tempWatershed==tempValue))=trueValue;
end%for tipIter
deepArteryAtBarrelTess(1,1) = 0;%normalize colors
deepArteryAtBarrelTess(1,2) = numTips;%normalize colors
figure(22);colormap(myColorMap);
figure(22);imagesc(deepArteryAtBarrelTess);
title(['Tesselation of Arteries that plunge to Barrel Depth at ',num2str(barrelLayerZ),' microns']);

% % %put a ball at the actual vessel
% % for tipIter = 1:numel(tipUsedList);
% %     tempFrame = false(maskSize(1),maskSize(2));
% %     tempFrame(tipCoordList(tipIter,1),tipCoordList(tipIter,2)) = 1;
% %     tempFrame = imdilate(tempFrame,circle2d(11)>0);
% %     deepArteryAtBarrelTess(tempFrame>0) = 0;
% % end%for tipIter
% % deepArteryAtBarrelTess(1,1) = 0;%normalize colors
% % deepArteryAtBarrelTess(1,2) = numTips;%normalize colors
% % figure(22);colormap(myColorMap);
% % figure(22);imagesc(deepArteryAtBarrelTess);
% % title(['Tesselation of Arteries that plunge to Barrel Depth at ',num2str(barrelLayerZ),' microns']);


deepArteryAtBarrelTessBorder = zeros(size(flatMask));
for tipIter = 1:numel(tipUsedList);
    trueValue = tipUsedList(tipIter);
    tempImage = deepArteryAtBarrelTess==trueValue;
    tempImage2 = imerode(tempImage,ones(11,11)>0);
    borderImage = tempImage-tempImage2;
    disp([tipIter,trueValue]);
    figure(99);imagesc(borderImage);pause(0.1);
    deepArteryAtBarrelTessBorder = deepArteryAtBarrelTessBorder +borderImage*trueValue;
end
deepArteryAtBarrelTessBorder(1,1) = 0;%normalize colors
deepArteryAtBarrelTessBorder(1,2) = numTips;%normalize colors
figure(32);imagesc(deepArteryAtBarrelTessBorder);colormap(myColorMap);
title(['Tesselation of Arteries that plunge to Barrel Depth at ',num2str(barrelLayerZ),' microns']);

%%
%
%
%
%
%
%NOW SWITCH TO CALCULATE THE DOMINANT DOMAINS
%
%
%
%

%%
%Adapted from the DyeTracer_ver01b code

dyeSourceStrandList = plungingArterioleTipList;
fileTag = '_InjectAtPlungingTips';
minimumFluxFraction = 1/3000;
stopVoltage = 50.5;
divFactor = 3;
%-----
displayFlag = 1;
overlayDisplayFlag = 0;
pauseFlag = 0;
normalizeToStartStrandCurrentFlag = 1; %multiply the flow by the startStrand current


% % pathColorStorageList = zeros(size(AllVerts));
% % myColorSpace = GenerateColorSpace(numStartStrands);

maxFlowFractionStorageList = zeros(size(strands));
associatedStartStorageList = zeros(size(strands));
%-----               
numStartStrands = numel(dyeSourceStrandList);
for startStrandIter = 1:numStartStrands;
    startStrandID = dyeSourceStrandList(startStrandIter);
    disp([startStrandIter, startStrandID]);


    startStrand = strands(startStrandID);
    ssiv = startStrand.InteriorVertices;
    startStrandCoordList = allVerts(ssiv,:);
    strandNodes = [startStrand.StartVertexIndex,startStrand.EndVertexIndex];
    nodeVoltages = allVoltages(strandNodes);
    [highVoltage,highVoltagePos] = max(nodeVoltages);
    [lowVoltage,lowVoltagePos] = min(nodeVoltages);
    highNode = strandNodes(highVoltagePos);
    lowNode = strandNodes(strandNodes~=highNode);

    pathActiveFlagList = false(1,100000);
    pathActiveFlagList(1) = 1;
    numActivePath = sum(pathActiveFlagList);
    strandPathList = {};
    nodePathList = {};
    fractionPathList = {};
    strandPathList{1} = startStrandID;
    nodePathList{1} = [highNode lowNode];
    fractionPathList{1} = 1;
    strandFlowList = zeros(numStrands,1);
    strandFlowList(startStrandID) = 1;
    strandVoltageList = zeros(numStrands,1);
    strandVoltageList(1) = mean([highVoltage,lowVoltage]);
    pathCounter = 1;
    keepGoingFlag = 1;
    myCamPos = [0 0 0];
    myCamUp = [-1 0 0];
    myCamVA = 6.6086;

    while (keepGoingFlag == 1),
        myCount = 0;
        for pathIter = find(pathActiveFlagList == 1);
             
            currentPathID = pathIter;
            currentStrandPath = strandPathList{pathIter};
            currentNodePath = nodePathList{pathIter};
            currentFractionPath = fractionPathList{pathIter};
            currentStrandID = currentStrandPath(end);
            currentStrand = strands(currentStrandID);
            %-----
            strandNodes = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
            nodeVoltages = allVoltages(strandNodes);
            [highVoltage,highVoltagePos] = max(nodeVoltages);
            [lowVoltage,lowVoltagePos] = min(nodeVoltages);
            highNode = strandNodes(highVoltagePos);
            lowNode = strandNodes(strandNodes~=highNode);
            lowAttachedStrands = strandIndices(lowNode,:);
            lowAttachedStrands = lowAttachedStrands(lowAttachedStrands~=0);
            lowAttachedStrands = lowAttachedStrands(lowAttachedStrands~=currentStrandID);
            numDaughterPaths = numel(lowAttachedStrands);
            strandVoltageList(currentStrandID) = mean([highVoltage,lowVoltage]);
            stopTracingFlag = 0;
            if numDaughterPaths == 0, stopTracingFlag = 1; end
            if lowVoltage < stopVoltage, stopTracingFlag = 1; end
            if currentFractionPath(end) < minimumFluxFraction, stopTracingFlag = 1; end
            if stopTracingFlag == 1,
                pathActiveFlagList(currentPathID) = 0;
            else%if stopTracingFlag == 1,
                motherCurrent = abs(currentStrand.strandCurrent);
                validDaughterCounter = 0;
                validDaughterStrandList = [];
                validDaughterCurrentList = zeros(1,numDaughterPaths);
                validDaughterPathIndexList = [];
                for daughterIter = 1:numDaughterPaths,
                    %Check to see if the daughter strand leads to a lower voltage
                    candidateDaughterID = lowAttachedStrands(daughterIter);
                    candidateDaughterStrand = strands(candidateDaughterID);
                    candidateDaughterNodes = [candidateDaughterStrand.StartVertexIndex,candidateDaughterStrand.EndVertexIndex];
                    candidateDaughterOtherNode = candidateDaughterNodes(candidateDaughterNodes~=lowNode);
                    candidateDaughterOtherVoltage = allVoltages(candidateDaughterOtherNode);
                    if candidateDaughterOtherVoltage < lowVoltage,
                        %if so, add the strand to a path
                        myCount = myCount + 1;
                        validDaughterCounter = validDaughterCounter + 1;
                        validDaughterStrandList(validDaughterCounter) = candidateDaughterID;
                        validDaughterCurrentList(validDaughterCounter) = abs(candidateDaughterStrand.strandCurrent);
                        if validDaughterCounter == 1,
                            %if this is the first valid daughter, add to current path
                            strandPathList{currentPathID} = [currentStrandPath, lowAttachedStrands(daughterIter)];
                            nodePathList{currentPathID} = [currentNodePath, candidateDaughterOtherNode];
                            validDaughterPathIndexList(1) = currentPathID;
                        else%if validDaughterCounter == 1,
                            %if not the first daughter, duplicate path up to this point, and
                            %make new path
                            pathCounter = pathCounter + 1;
                            strandPathList{pathCounter} = [currentStrandPath, lowAttachedStrands(daughterIter)];
                            nodePathList{pathCounter} = [currentNodePath, candidateDaughterOtherNode];
                            fractionPathList{pathCounter} = currentFractionPath;
                            pathActiveFlagList(pathCounter) = 1;  
                            validDaughterPathIndexList(validDaughterCounter) = pathCounter;
                        end%if-else validDaughterCounter == 1,
                    end%if candidateDaughterOtherVoltage < lowVoltage,
                end%for daughterIter = 1:numDaughterPaths,
                clear daughterIter
                %-----
                daughterSumCurrent = sum(validDaughterCurrentList);
                fractionalDaughterCurrent = validDaughterCurrentList./daughterSumCurrent;
                for validDaughterIter = 1:validDaughterCounter;
                    validDaughterSegmentID = validDaughterStrandList(validDaughterIter);
                    validDaughterFraction = currentFractionPath(end).*fractionalDaughterCurrent(validDaughterIter);
                    fractionPathList{validDaughterPathIndexList(validDaughterIter)} = [currentFractionPath , validDaughterFraction];
                    strandFlowList(validDaughterSegmentID) = strandFlowList(validDaughterSegmentID) + validDaughterFraction;
                end%for validDaughterIter = 1:validDaughterCounter;
            end%if-else stopTracingFlag == 1,
        end%for pathIter = find(pathActiveFlagList == 1);
        if sum(pathActiveFlagList) == 0, keepGoingFlag = 0;end
        if myCount == 0, keepGoingFlag = 0;end
    end%while (keepGoingFlag == 1),
    %-----
    if displayFlag == 1,
        %Generate a plot
%         myColorVector = myColorSpace(startStrandIter,:);
%         myR = [0:255]'*myColorVector(1);
%         myG = [0:255]'*myColorVector(2);
%         myB = [0:255]'*myColorVector(3);
%         myColorMap = [myR,myG,myB]./256;
        %-----
        divMaskSize = ceil(maskSize./divFactor);
        activeStrandList = find(strandFlowList>0);
        numActiveStrands = numel(activeStrandList);
        ff = figure(10);
        myCamPos = campos;
        myCamUp = camup;
        myCamVA = camva;
        cameratoolbar
        for strandIter = 1:numActiveStrands 
            thisStrandID = activeStrandList(strandIter);
            thisStrand = strands(thisStrandID);
            pathIndexList = thisStrand.InteriorVertices;
            pathCoordList = ceil(allVerts(pathIndexList,:)/divFactor);
            maskIndexList = sub2ind(divMaskSize,pathCoordList(:,1),pathCoordList(:,2),pathCoordList(:,3));
            p3 = plot3(pathCoordList(:,1),pathCoordList(:,2),-1*pathCoordList(:,3),'r.');hold on
            thisStrandFlow = ceil(strandFlowList(thisStrandID)*255);
            thisStrandVoltage = ceil(255*(strandVoltageList(thisStrandID)-49)./51);
            set(p3,'MarkerSize',thisStrandFlow/20);
        %     set(p3,'Color',myColorMap(thisStrandVoltage,:))
            set(p3,'Color',myColorMap(end,:))
        end%for pathIter = 1:numPaths,
        ssccc = ceil(startStrandCoordList./divFactor);
        p4 = plot3(ssccc(:,1),ssccc(:,2),-1*ssccc(:,3),'ko');
        set(p4,'MarkerSize',7);
        xlabel('Axis 1');
        ylabel('Axis 2');
        zlabel('Axis 3');
        plot3(0,0,0,'b.');
        plot3(ceil(1500/divFactor),ceil(1500/divFactor),ceil(-1300./divFactor),'b.');
        campos(myCamPos)
        camup(myCamUp);
        camva(myCamVA);
        if overlayDisplayFlag ~= 1, hold off; end
        % saveas(ff,['Figure_StartStrand_',num2str(startStrandID,'%05d'),'.tif']);
% %         fp = figure(11);
% %         pp4 = plot3(ssccc(:,1),ssccc(:,2),-1*ssccc(:,3),'ko');hold on
% %         plot3(ceil(1500/divFactor),ceil(1500/divFactor),ceil(-1300./divFactor),'b.');
        if pauseFlag ==1, pause, else, pause(0.1);end
    end%if displayFlag == 1,
    %-----
    %-----
    %Generate an dominant flow map 
    activeStrandList = find(strandFlowList>0);
    numActiveStrands = numel(activeStrandList);

    for strandIter = 1:numActiveStrands 
        thisStrandID = activeStrandList(strandIter);
        thisStrand = strands(thisStrandID);
        thisStrandFlow = (strandFlowList(thisStrandID));
        if normalizeToStartStrandCurrentFlag == 1,
            thisStrandFlow = abs(startStrand.strandCurrent) * thisStrandFlow;
        end%if normalizeToStartStrandCurrentFlag == 1,
        storedFlowFraction = maxFlowFractionStorageList(thisStrandID);
        if thisStrandFlow > storedFlowFraction,
            maxFlowFractionStorageList(thisStrandID) = thisStrandFlow;
            associatedStartStorageList(thisStrandID) = startStrandIter;
        end%if thisStrandFlow > storedFlowFraction,
    end%for strandIter = 1:numActiveStrands 
%     disp(numel(find(associatedStartStorageList>0)))
end%for startStrandIter = 1:...




%%
%Generate a color for each tip
myColorMap = zeros(64,3);
% % myColorMap(1,:) = [0.5 0.8 0.1];
myColorMap(1,:) = [0 0 0];
for kk = 2:64,
    myColorMap(kk,:) = myColorMap(kk-1,:) + [0.13 0.23 0.37];
end;

myColorMap = mod(myColorMap,1);
myColorMap(end,:) = [1,1,1];


divMaskSize = ceil(maskSize./divFactor);
redMask = zeros(divMaskSize,'uint8');
greenMask = zeros(divMaskSize,'uint8');
blueMask = zeros(divMaskSize,'uint8');
bwMask = zeros(divMaskSize,'uint8');

for strandIter = 1:numStrands,
    thisStrandID = strandIter;
    thisStrand = strands(thisStrandID);
    colorPointer = associatedStartStorageList(thisStrandID);
    thisStrandFlow = maxFlowFractionStorageList(thisStrandID);
    thisStrandCurrent = thisStrand.strandCurrent;
    thisStrandRatio(strandIter) = abs(thisStrandFlow/thisStrandCurrent);
% %     if mod(strandIter,1000) == 0, disp([thisStrandFlow,thisStrandCurrent]);end
    if colorPointer > 0,
        if thisStrandRatio(strandIter) >= 0,
            thisColor = myColorMap(colorPointer,:);
            thisStrandCoords = ceil(allVerts(thisStrand.StartToEndIndices,:)./divFactor);
            ccc = thisStrandCoords;
            thisStrandIndices = sub2ind(divMaskSize,ccc(:,1),ccc(:,2),ccc(:,3));
            redMask(thisStrandIndices) = thisColor(1);
            greenMask(thisStrandIndices) = thisColor(2);
            blueMask(thisStrandIndices) = thisColor(3);
            bwMask(thisStrandIndices) = colorPointer;
        end%if thisStrandRatio >0.5,
    end%if colorPointer > 0,
end

barrelZDivPlane = ceil(barrelLayerZ / divFactor);
deltaZ = 75;%microns
deltaZDiv = ceil(deltaZ/divFactor);
maxProjDomains = max(bwMask(:,:,barrelZDivPlane-deltaZDiv : barrelZDivPlane+deltaZDiv),[],3);


% % %%for DB only
% % maxProjDomains(maxProjDomains==25)=30;



maxProjDomains(1,1) = 0;
maxProjDomains(1,2) = numTips;
figure(23);colormap(myColorMap);
figure(23);imagesc(maxProjDomains);
title(['Tesselation of Arteries that plunge to Barrel Depth at ',num2str(barrelLayerZ),' microns']);

barrelOutlineMask = double((allBarrelMask - imerode(allBarrelMask,circle2d(11)>0)))*numTips;

maxProjDomainsBig = double(imresize(maxProjDomains,3,'nearest'));
maxProjDomainsBig = maxProjDomainsBig(1:maskSize(1),1:maskSize(2));

deepArteryAtSurfaceTessPlusDomain = max(deepArteryAtSurfaceTessBorder,maxProjDomainsBig);
deepArteryAtSurfaceTessPlusDomain = max(deepArteryAtSurfaceTessPlusDomain,barrelOutlineMask);

figure(31);imagesc(deepArteryAtSurfaceTessPlusDomain); colormap(myColorMap);
title(['Surface Tesselation of Arteries that plunge to at least ',num2str(plungingMustGoPast),' microns']);


deepArteryAtBarrelTessPlusDomain = max(deepArteryAtBarrelTessBorder,maxProjDomainsBig);
deepArteryAtBarrelTessPlusDomain = max(deepArteryAtBarrelTessPlusDomain,barrelOutlineMask);
figure(32);imagesc(deepArteryAtBarrelTessPlusDomain); colormap(myColorMap);
title(['Tesselation of Arteries that plunge to Barrel Depth at ',num2str(barrelLayerZ),' microns']);


%%
% % % maketiff(bwMask,['bwMask2',fileTag,'.tif']);
% % % 
% % % 
% % % fileName = ['DominantFlowRGB2',fileTag,'.tif'];
% % % currentFrame = zeros(size(redMask,1),size(redMask,2),3,'uint8');
% % % currentFrame(:,:,1) = redMask(:,:,1);
% % % currentFrame(:,:,2) = greenMask(:,:,1);
% % % currentFrame(:,:,3) = blueMask(:,:,1);
% % % imwrite(uint8(currentFrame),fileName,'tiff','Compression','none');
% % % 
% % % for frameIter = 2:size(redMask,3);
% % %     currentFrame(:,:,1) = redMask(:,:,frameIter);
% % %     currentFrame(:,:,2) = greenMask(:,:,frameIter);
% % %     currentFrame(:,:,3) = blueMask(:,:,frameIter);
% % %     imwrite(uint8(currentFrame),fileName,'tiff','WriteMode','append','Compression','none');
% % % end

%%
% % allPlungingStrandPath = gds.allPlungingArteriolePathList;
% % 
% % for startStrandIter = 1:numStartStrands;
% %     ccPlunging = AllVerts([Strands(gds.allPlungingArterioleList).StartToEndIndices],:);
% %     figure(21);hold off
% %     plot3(ccPlunging(:,1),ccPlunging(:,2),-1*ccPlunging(:,3),'r.');hold on
% % 
% % 
% %     
% %     dominantFlowStrands = find(associatedStartStorageList==startStrandIter);
% %     ccDom = AllVerts([Strands(dominantFlowStrands).StartToEndIndices],:);
% %     plot3(ccDom(:,1),ccDom(:,2),-1*ccDom(:,3),'g.');hold on
% %     
% %     figure(21);
% %     plot3(ccPlunging(:,1),ccPlunging(:,2),-1*ccPlunging(:,3),'r.');hold on
% % 
% %     ccStart = AllVerts([Strands([allPlungingStrandPath{startStrandIter}]).StartToEndIndices],:);
% %     plot3(ccStart(:,1),ccStart(:,2),-1*ccStart(:,3),'b.');hold on
% %  
% %     
% %     pause
% %     hold on
% %     
% % end

    


    

















