%This script takes all the plunging artery tops and clots them one-by-one
%and compares the flow downstream to the non-clot flow.
%In this version, the main line of the plunging arteriole is identified
%and branch counting starts OFF of that line.

%Instructions:
%Load allProject

scriptVersion = 'ClotPlungingArterioleTipsScript4a.m';
%this version uses the adjustedRadius.
%this version plots median and median+/std


vectorProject = allProject.av;

 vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
           

strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;
rootName = vectorProject.vectorProjectRootName;
findDash = findstr(rootName,'-');
shortName = rootName(1:findDash(1)-1);

strandMidYXZList = vectorProject.vectorizedStructure.strandMidYXZList;

strandLabelList = [strands(:).MedianLabel];
surfList = find(and(strandLabelList>=300,strandLabelList<=399)); 
zSurf = strandMidYXZList(surfList,3);
medianZSurf = median(zSurf);
minZSurf = min(zSurf);

strandMidYXZList(:,3) = max(strandMidYXZList(:,3)-minZSurf,0);
allPlungingVenuleList = vectorProject.vectorizedStructure.allPlungingVenuleList;
allPlungingArterioleList = vectorProject.vectorizedStructure.allPlungingArterioleList;

flowAnalysisParameters = [];
cleanupFlag = 1;





%%Stage 2: Run the unperturbed flow analysis:
unperturbedFileName = ['F:\Phil\AOH_VectorProjects\OcclusionFiles_AdjRadius\',shortName,'-Unperturbed.mat'];
if exist(unperturbedFileName,'file'),
    load(unperturbedFileName,'origCurrentList');
    disp('UnPerturbed Flow Calculation loaded');
else%if exist...
    unperturbedProject = vectorProject;
    unperturbedProject.circuitAnalysisParameters.minRadius = 0;
    unperturbedProject = CalculateUnperturbedFlow003(unperturbedProject);
    origCurrentList = [unperturbedProject.vectorizedStructure.Strands(:).strandCurrent];
    save(unperturbedFileName,'origCurrentList');
    disp('UnPerturbed Flow Calculation completed');
end%if exist...

clear tipList allPlungingArteriolePathList numTips



%%
%%Stage 3: Select a vessel segment to occlude
tipList = vectorProject.vectorizedStructure.plungingArterioleTipList;
allPlungingArteriolePathList = vectorProject.vectorizedStructure.allPlungingArteriolePathList;
numTips = numel(tipList);


zEndMustBeGreater = 200;
zEndMustBeLess = 9999;

subList = [];
for kk = 1:numel(allPlungingArteriolePathList);
    thisPath = allPlungingArteriolePathList{kk};
    if numel(thisPath)>0,
        thisZ = strandMidYXZList(thisPath,3);
          if thisZ(end)>zEndMustBeGreater && thisZ(end)<zEndMustBeLess,
            subList = [subList,kk];
        end%if thisPath(2)...
    end%if numel...
end%for kk
tipList = tipList(subList);
allPlungingArteriolePathList = allPlungingArteriolePathList(subList);
numTips = numel(tipList);


%Set limits on analysis field of view
y1 = 0; y2 = inf;
x1 = 0; x2 = inf;
z1 = 0; z2 = inf;

y1 = 100; y2 = maskSize(1)-y1;
x1 = 100; x2 = maskSize(2)-x1;
z1 = 000; z2 = 500;


analyzeDownstreamBranchesOnlyFlag = 1;
considerOnlyBranchesInFOVFlag = 1;
ignoreLowStartingCurrentStrandsFlag = 1;
excludeVeinsAndSurfaceFlag = 1;
stallLowFlowVesselFlag = 0;
stopTrackingOnceOutsideFlag = 1;
forceOutletClotFlag = 0;

stallNumFraction = 0.10;
maxBranch = 15;
numPlungingStrandToOcclude = 1;
outletLimit = 2; %consider at most, this number of outlets per occlusion arteriole
samplingLimit = 9999; 

f1 = and(strandMidYXZList(:,1)>y1,strandMidYXZList(:,1)<y2);
f2 = and(strandMidYXZList(:,2)>x1,strandMidYXZList(:,2)<x2);
f3 = and(strandMidYXZList(:,3)>z1,strandMidYXZList(:,3)<z2);
insideStrandList = find(and(and(f1,f2),f3));
outsideStrandList = find(~(and(and(f1,f2),f3)));
insideStrandCurrentList = abs(origCurrentList(insideStrandList));
sortList = sort(insideStrandCurrentList,'ascend');
thresholdCurrentForStall = sortList(ceil(stallNumFraction *numel(sortList)));


allTipMedianCurrentRatioList = [];
allRatioList =[]; allRatioList{maxBranch} = [];
allPreCurrentList =[]; allPreCurrentList{maxBranch} = [];
allPostCurrentList =[]; allPostCurrentList{maxBranch} = [];

f1 = figure(1); set(f1,'DoubleBuffer','on');clf
f2 = figure(2); set(f2,'DoubleBuffer','on');clf

tipCounter = 0;
allTipMedianCurrentRatioList = [];
allRatioList =[]; allRatioList{maxBranch} = [];
allPreCurrentList =[]; allPreCurrentList{maxBranch} = [];
allPostCurrentList =[]; allPostCurrentList{maxBranch} = [];
crossingPointList = [];
medianOffShootZList = [];
for tipIter = 1:1:numTips,
   
    disp(['TIP ITER ',num2str(tipIter)]);
    occlusionPath = allPlungingArteriolePathList{tipIter};
    myNumOcclude= min(numel(occlusionPath),numPlungingStrandToOcclude);
    occlusionStrandID =  occlusionPath(1:myNumOcclude)
    

    
    origRadius = vectorProject.vectorizedStructure.effectiveStrandRadiusList(occlusionStrandID);
    pathList = allPlungingArteriolePathList{tipIter};
    pathCoords = strandMidYXZList(pathList,:);
    %disp(pathCoords);
    maxPathCoords = max(pathCoords,[],1);
    minPathCoords = min(pathCoords,[],1);
    insideFlag = 1;
    
    if minPathCoords(1)<y1, insideFlag = 0;end
    if minPathCoords(2)<x1, insideFlag = 0;end
    if maxPathCoords(1)>y2, insideFlag = 0;end
    if maxPathCoords(2)>x2, insideFlag = 0;end
    
    %Consider branches off the clotted plunging arteriole line
    flowProject.maskSize = maskSize;
    flowProject.Strands = strands;
    flowProject.Vertices = vertices;
    flowProject.baseStrandIDList = pathList;
    flowProject.maxBranchSearch = 2;
    flowProject.stopVoltage = 50.1;%Hardcode;
    flowProject.minimumFluxFraction = 1/(2^18);%Hardcode
    disp('Finding arteriole outlets paths...');
    flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);    
    arterioleOffshootList = find(min(flowProject.downstreamBranchList,[],2)==1);
    numArterioleOffshoots = numel(arterioleOffshootList);
    downstreamBranchList = flowProject.downstreamBranchList;
    arterioleOffshootList = arterioleOffshootList(~ismember(arterioleOffshootList,vectorProject.vectorizedStructure.allPlungingArterioleList));
    offshootZ = strandMidYXZList(arterioleOffshootList,3);
    medianOffshootZ = median(offshootZ);
    arterioleOffshootList = arterioleOffshootList(and(offshootZ>z1,offshootZ<z2));
    if ~isempty(arterioleOffshootList),
        arterioleOffshootList = arterioleOffshootList(1:min(numel(arterioleOffshootList),outletLimit))
    end%if~isempty
    
    if forceOutletClotFlag == 1,
        occlusionStrandID = [occlusionStrandID,arterioleOffshootList'];
    end%if forceOutletClotFlag == 1,
    
    occlusionString = num2str(occlusionStrandID);
    occlusionString = regexprep(occlusionString,'     ','_');
    occlusionString = regexprep(occlusionString,'    ','_');
    occlusionString = regexprep(occlusionString,'   ','_');
    occlusionString = regexprep(occlusionString,'  ','_');
    occlusionString = regexprep(occlusionString,' ','_');
    occlusionString = [shortName,'-Clot-',occlusionString,'.mat'];
    occlusionFilePath = 'F:\Phil\AOH_VectorProjects\OcclusionFiles_AdjRadius';
    occlusionFileName = fullfile(occlusionFilePath,occlusionString);
    
    
    if isempty(arterioleOffshootList), insideFlag = 0; end
        if insideFlag == 1,
            perturbedProject = vectorProject;
            perturbedProject.circuitAnalysisParameters.minRadius = 0;
            perturbedProject.vectorizedStructure.effectiveStrandRadiusList(occlusionStrandID) = 0;
            
            if exist(occlusionFileName,'file'),
                disp('Perturbed Flow loading from file');pause(0.1);
                load(occlusionFileName,'perturbedCurrentList');
                %---
                clear downstreamBranchList
                q = load(occlusionFileName,'downstreamBranchList');
                if isfield(q,'downstreamBranchList'),
                    downstreamBranchList = q.downstreamBranchList;
                else,
                    flowProject.baseStrandIDList = arterioleOffshootList;
                    flowProject.maxBranchSearch = maxBranch;
                    disp('Calculating downstream paths');
                    tic
                    flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
                    toc
                    downstreamBranchList = flowProject.downstreamBranchList;
                    save(occlusionFileName,'perturbedCurrentList','downstreamBranchList');
                end%if isempty(q);
            else%if exist(occlusionFileName...
                perturbedProject = CalculateUnperturbedFlow003(perturbedProject);
                perturbedCurrentList = [perturbedProject.vectorizedStructure.Strands(:).strandCurrent];
                flowProject.baseStrandIDList = arterioleOffshootList;
                flowProject.maxBranchSearch = maxBranch;
                disp('Calculating downstream paths');pause(0.1);
                flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
                downstreamBranchList = flowProject.downstreamBranchList;
                save(occlusionFileName,'perturbedCurrentList','downstreamBranchList');
                disp('Perturbed Flow Calculation completed');pause(0.1);
            end%if-else exist(occlusionFileName...

                
            if stallLowFlowVesselFlag == 1,
                perturbedCurrentList(abs(perturbedCurrentList)<thresholdCurrentForStall) = 0;
            end%if stallLowFlowVesselFlag == 1,
            
           
            lowCurrentStrandList = find(abs(origCurrentList) < thresholdCurrentForStall);
            currentRatioList = abs(perturbedCurrentList./origCurrentList);


            switch analyzeDownstreamBranchesOnlyFlag,
                case 1 %analyze only downstream branches
% % %                     flowProject.baseStrandIDList = arterioleOffshootList;
% % %                     flowProject.maxBranchSearch = maxBranch;
% % %                     disp('Calculating downstream paths');
% % %                     flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
% % %                     downstreamBranchList = flowProject.downstreamBranchList;
                    offshootRatio = currentRatioList(arterioleOffshootList);
                    allRatioList{1} = offshootRatio;
                    allPreCurrentList{1} = origCurrentList(arterioleOffshootList);
                    allPostCurrentList{1} = perturbedCurrentList(arterioleOffshootList);
                    
                    branchClassList = [1:maxBranch];
                    medianCurrentRatioList = [];
                    medianCurrentRatioList(1) = median(offshootRatio);
                    for branchIter = 2:maxBranch,
                        findBranch = find(sum(downstreamBranchList==(branchIter-1),2)>0);
                        findBranch = findBranch(~ismember(findBranch,pathList));
                        if considerOnlyBranchesInFOVFlag == 1,
                            branchCoordList = strandMidYXZList(findBranch,:);
                            branchListZ = branchCoordList(:,3);
                            f1 = and(branchCoordList(:,1)>y1,branchCoordList(:,1)<y2);
                            f2 = and(branchCoordList(:,2)>x1,branchCoordList(:,2)<x2);
                            f3 = and(branchCoordList(:,3)>z1,branchCoordList(:,3)<z2);
                            ff = and(and(f1,f2),f3);
                            findBranch = findBranch(ff);
                        end%if considerOnly...
                        crl = currentRatioList(findBranch);
                        if ignoreLowStartingCurrentStrandsFlag == 1,
                            crl = crl(~ismember(crl,lowCurrentStrandList));
                        end%if ignoreLow...
                        allRatioList{branchIter} = [allRatioList{branchIter},crl];
                        allPreCurrentList{branchIter} = [allPreCurrentList{branchIter},origCurrentList(findBranch)];
                        allPostCurrentList{branchIter} = [allPostCurrentList{branchIter},perturbedCurrentList(findBranch)];

                        figure(1);p1 = plot(repmat(branchIter,size(crl)),crl,'bo');hold on
                        figure(2);p2 = plot(repmat(branchIter,size(crl)),crl,'bo');hold on
                        figure(3);p2 = plot(repmat(branchIter,size(branchListZ)),branchListZ,'go');
                        medianCurrentRatioList(branchIter) = median(crl);
                    end%for branchIter;
                    figure(1);plot(branchClassList,medianCurrentRatioList,'r-');
                    set(gca,'YLim',[0,2]);
                    figure(2);plot(branchClassList,medianCurrentRatioList,'r-');hold off
                    set(gca,'YLim',[0,2]);
                    
                case 0 %analyze all branches regardless of flow direction
% % % %                     flowProject.baseStrandIDList = arterioleOffshootList;
% % % %                     flowProject.maxBranchSearch = maxBranch;
% % % %                     flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
% % % %                     downstreamBranchList = flowProject.downstreamBranchList;
                    offshootRatio = currentRatioList(arterioleOffshootList);
                    allRatioList{1} = [allRatioList{1},offshootRatio];
                    allPreCurrentList{1} = origCurrentList(arterioleOffshootList);
                    allPostCurrentList{1} = perturbedCurrentList(arterioleOffshootList);
                    
                    figure(1);p1 = plot(repmat(1,size(offshootRatio)),offshootRatio,'bo');hold on
                    figure(2);p2 = plot(repmat(1,size(offshootRatio)),offshootRatio,'bo');hold on
%                       
                    
                    alreadyUsedList = [pathList,arterioleOffshootList'];
                    if stopTrackingOnceOutsideFlag == 1,
                         alreadyUsedList = [alreadyUsedList,outsideStrandList'];
                    end%if stopTracking...
                    if excludeVeinsAndSurfaceFlag == 1,
                        alreadyUsedList = [alreadyUsedList,allPlungingVenuleList,surfList,allPlungingArterioleList];
                    end%if excludeVeinsAndSurfaceFlag == 1,
                    nextStrandList = [strands(arterioleOffshootList).StartVertexNeighborStrands,strands(arterioleOffshootList).EndVertexNeighborStrands];
                    nextStrandList = nextStrandList(~ismember(nextStrandList,alreadyUsedList));
                    branchClassList = [1:maxBranch];
                    medianCurrentRatioList = [];
                    medianCurrentRatioList(1) = median(offshootRatio);
                    for branchIter = 2:maxBranch,
                        findBranch = nextStrandList;
                        alreadyUsedList = [alreadyUsedList,findBranch];
                        if considerOnlyBranchesInFOVFlag == 1,
                            branchCoordList = strandMidYXZList(findBranch,:);
                            branchListZ = branchCoordList(:,3);
                            f1 = and(branchCoordList(:,1)>y1,branchCoordList(:,1)<y2);
                            f2 = and(branchCoordList(:,2)>x1,branchCoordList(:,2)<x2);
                            f3 = and(branchCoordList(:,3)>z1,branchCoordList(:,3)<z2);
                            ff = and(and(f1,f2),f3);
                            findBranch = findBranch(ff);
                        end%if considerOnly...
                        crl = currentRatioList(findBranch);
                        
% % %                          if sum(crl>5)>0,
% % %                             disp('Big Ratio!');
% % %                             pause
% % %                             pause
% % %                         end%if sum..
                        
                        
                        if ignoreLowStartingCurrentStrandsFlag == 1,
                            crl = crl(~ismember(findBranch,lowCurrentStrandList));
                        end%if ignoreLow...
         
                        
                        allRatioList{branchIter} = [allRatioList{branchIter},crl];
                        allPreCurrentList{branchIter} = [allPreCurrentList{branchIter},origCurrentList(findBranch)];
                        allPostCurrentList{branchIter} = [allPostCurrentList{branchIter},perturbedCurrentList(findBranch)];
                        
                        qq = numel(crl);qqq = min(3,qq);
                        crl2 = [crl(1:qqq),crl([2:100:end])];
                        figure(1);p1 = plot(repmat(branchIter,size(crl2)),crl2,'bo');hold on
                        figure(2);p2 = plot(repmat(branchIter,size(crl2)),crl2,'bo');hold on
%                         figure(3);p2 = plot(repmat(branchIter,size(branchListZ)),branchListZ,'go');hold off
                        nonNanCRL = crl(~isnan(crl));
                        medianCurrentRatioList(branchIter) = median(nonNanCRL);
                        %---
                        nextStrandList = [strands(nextStrandList).StartVertexNeighborStrands,strands(nextStrandList).EndVertexNeighborStrands];
                        nextStrandList = nextStrandList(~ismember(nextStrandList,alreadyUsedList));
                    
                        
                    end%for branchIter;
                    figure(1);plot(branchClassList,medianCurrentRatioList,'r-');
                    set(gca,'YLim',[0,2]);
                    figure(2);plot(branchClassList,medianCurrentRatioList,'r-');hold off
                    set(gca,'YLim',[0,2]);
                    
            end%switch-case
            tipCounter = tipCounter +1;
            allTipMedianCurrentRatioList(tipCounter,:) = medianCurrentRatioList;
            
            
            above75 = find(medianCurrentRatioList>0.75);
            if ~isempty(above75),
                if above75(1)>1,
                    q2 = above75(1);
                    q1 = above75(1)-1;
                    q3 = q1+((0.75-medianCurrentRatioList(q1))/(medianCurrentRatioList(q2)-medianCurrentRatioList(q1)));
                    crossingPointList(tipCounter) =  q3;
                else%if above75(1)>1
                    crossingPointList(tipCounter) = 1;
                end%if above75(1)>1
            else%if isempty(above75),
                crossingPointList(tipCounter) = maxBranch
            end%if isempty
            medianOffShootZList(tipCounter) = medianOffshootZ;
            
            figure(9);plot(medianOffShootZList,crossingPointList,'ro');
            
            pause(0.1);


    
    
    

    

     
                    
        
        


        %---

        
        
  

    end%if insideFlag == 1,
end%for tipIter..

if ~exist('megaRatioList','var'),
    megaRatioList{maxBranch} = [];
    megaNameList = [];
end%if ~exist...

for kk = 1:maxBranch,
    q = allTipMedianCurrentRatioList(:,kk);
    q2 = q(~isnan(q));
    q2m = median(q2);
    q(isnan(q)) = q2m;
    allTipMedianCurrentRatioList(:,kk) = q;
end
medianOfMedianCurrentRatioList = median(allTipMedianCurrentRatioList,1);
figure(2);plot(branchClassList,medianOfMedianCurrentRatioList);

fullMedianList = [];
for branchIter = 1:maxBranch,
    q = allRatioList{branchIter};
    q2 = q(~isnan(q));
    q2m = median(q2);
    q(isnan(q)) = q2m;
    fullMedianList(branchIter) = median(q);
    megaRatioList{branchIter} = [megaRatioList{branchIter},q];
    megaRatioMedian(branchIter) = median(megaRatioList{branchIter});
    sList = sort(megaRatioList{branchIter});
    megaS1(branchIter) = sList(ceil(numel(sList)*0.25));
    megaS2(branchIter) = sList(ceil(numel(sList)*0.75));
    numChanged = 1;
    ratioList = megaRatioList{branchIter};
    while numChanged~=0,
        q1 = numel(ratioList);
        s = std(ratioList);
        f = find(ratioList<mean(ratioList)+4*s);
        ratioList = ratioList(f);
        q2 = numel(ratioList);
        numChanged = q1-q2;
    end%while
    megaRatioMedian(branchIter) = median(ratioList);
% %     megaS1(branchIter) = median(ratioList) + std(ratioList);
% %     megaS2(branchIter) = median(ratioList) - std(ratioList);
    megaS1(branchIter) = median(ratioList) + std(ratioList)./sqrt(numel(ratioList));
    megaS2(branchIter) = median(ratioList) - std(ratioList)./sqrt(numel(ratioList));
    
end%branchIter
megaNameList = [megaNameList,shortName,'_']
figure(3);plot(branchClassList,fullMedianList,'g-');

b(1) = mean(fullMedianList(1:2));   
b(2) = mean(fullMedianList(3:4));   
b(3) = mean(fullMedianList(5:6));   
b(4) = mean(fullMedianList(7:10));   
b(5) = mean(fullMedianList(11:15));
figure(4);plot(b,'ro');

figure(7);plot(megaRatioMedian,'mo');hold on
figure(7);plot(megaS1,'c*');hold on
figure(7);plot(megaS2,'c*');hold on

figure(7);p1 = plot([1.5,1.5],[0,0.10],'k-');hold on;set(p1,'LineWidth',7);
figure(7);p1 = plot([3.5,3.5],[0,0.25],'k-');hold on;set(p1,'LineWidth',7);
figure(7);p1 = plot([5.5,5.5],[0.35,1],'k-');hold on;set(p1,'LineWidth',7);
figure(7);p1 = plot([8.5,8.5],[0.15,1.15],'k-');hold on;set(p1,'LineWidth',7);
figure(7);p1 = plot([11.5,11.5],[0.55,1.2],'k-');hold on;set(p1,'LineWidth',7);
figure(7);plot([1.5,3.5,5.5,8.5,11.5],[0.03 0.08 0.55 0.80 0.85],'wo');hold off






clotResultStruct = [];

clotResultStruct.scriptVersion = scriptVersion;
clotResultStruct.analyzeDownstreamBranchesOnlyFlag = analyzeDownstreamBranchesOnlyFlag;
clotResultStruct.considerOnlyBranchesInFOVFlag = considerOnlyBranchesInFOVFlag;
clotResultStruct.ignoreLowStartingCurrentStrandsFlag = ignoreLowStartingCurrentStrandsFlag;
clotResultStruct.excludeVeinsAndSurfaceFlag = excludeVeinsAndSurfaceFlag;
clotResultStruct.stallLowFlowVesselFlag = stallLowFlowVesselFlag;
clotResultStruct.stopTrackingOnceOutsideFlag = stopTrackingOnceOutsideFlag;
clotResultStruct.forceOutletClotFlag = forceOutletClotFlag;

clotResultStruct.x1 = x1;
clotResultStruct.y1 = y1;
clotResultStruct.z1 = z1;
clotResultStruct.x2 = x2;
clotResultStruct.y2 = y2;
clotResultStruct.z2 = z2;
clotResultStruct.zEndMustBeGreater = zEndMustBeGreater;

clotResultStruct.stallNumFraction = stallNumFraction;
clotResultStruct.maxBranch = maxBranch;
clotResultStruct.numPlungingStrandToOcclude = numPlungingStrandToOcclude;
clotResultStruct.outletLimit = outletLimit; %consider at most, this number of outlets per occlusion arteriole
clotResultStruct.samplingLimit = samplingLimit; 
clotResultStruct.megaRatioList = megaRatioList;
clotResultStruct.megaNameList = megaNameList;
clotResultStruct.megaRatioMedian = megaRatioMedian;
clotResultStruct.megaS1 = megaS1;
clotResultStruct.megaS2 = megaS2;

%save clotResult000.mat clotResultStruct
%clear mega*

%%

scriptVersion = clotResultStruct.scriptVersion;
analyzeDownstreamBranchesOnlyFlag = clotResultStruct.analyzeDownstreamBranchesOnlyFlag;
considerOnlyBranchesInFOVFlag = clotResultStruct.considerOnlyBranchesInFOVFlag;
ignoreLowStartingCurrentStrandsFlag = clotResultStruct.ignoreLowStartingCurrentStrandsFlag;
excludeVeinsAndSurfaceFlag = clotResultStruct.excludeVeinsAndSurfaceFlag;
stallLowFlowVesselFlag = clotResultStruct.stallLowFlowVesselFlag;
stopTrackingOnceOutsideFlag = clotResultStruct.stopTrackingOnceOutsideFlag;
forceOutletClotFlag = clotResultStruct.forceOutletClotFlag;

x1 = clotResultStruct.x1;
y1 = clotResultStruct.y1;
z1 = clotResultStruct.z1;
x2 = clotResultStruct.x2;
y2 = clotResultStruct.y2;
z2 = clotResultStruct.z2;
zEndMustBeGreater = clotResultStruct.zEndMustBeGreater;

stallNumFraction = clotResultStruct.stallNumFraction;
maxBranch = clotResultStruct.maxBranch;
numPlungingStrandToOcclude = clotResultStruct.numPlungingStrandToOcclude;
outletLimit = clotResultStruct.outletLimit; %consider at most, this number of outlets per occlusion arteriole
samplingLimit = clotResultStruct.samplingLimit; 
megaRatioList = clotResultStruct.megaRatioList;
megaNameList = clotResultStruct.megaNameList;
megaRatioMedian = clotResultStruct.megaRatioMedian;
megaS1 = clotResultStruct.megaS1;
megaS2 = clotResultStruct.megaS2;



