function vectorProject = AssignBarrelBoundariesFunction(vectorProject);

vectorizedStructure = vectorProject.vectorizedStructure;
maskSize = vectorizedStructure.maskSize;
strands = vectorizedStructure.Strands;
vertices = vectorizedStructure.Vertices;
numVerts = numel(vertices.AllRadii);
numStrands = numel(strands);
allVerts = double(vertices.AllVerts);

barrelBoundaryParameters = vectorProject.barrelBoundaryParameters;

barrels = barrelBoundaryParameters.barrels;
divRatio = barrelBoundaryParameters.divRatio;
closeCircleDiameter = barrelBoundaryParameters.closeCircleDiameter;
erodeCircleDiameter = barrelBoundaryParameters.erodeCircleDiameter;
dilateCircleDiameter = barrelBoundaryParameters.dilateCircleDiameter;
extendBarrelZRange = barrelBoundaryParameters.extendBarrelZRange;
extendFullRangeFlag = barrelBoundaryParameters.extendBarrelZAcrossFullFieldFlag;

%%

numBarrels = numel(barrels)
allBarrelCoords = [];
for barrelIter = 1:numBarrels;
    allBarrelCoords = [allBarrelCoords;barrels{barrelIter}];
end

globalMinZ = min(allBarrelCoords(:,3))- extendBarrelZRange;
globalMaxZ = max(allBarrelCoords(:,3))+ extendBarrelZRange;
globalZRange = globalMaxZ-globalMinZ+1;


subMaskSize = ceil(maskSize/divRatio);
subMaskSize(3) = ceil(globalZRange/divRatio);
%%
allVertsExpanded = [allVerts,[1:numVerts]'];
cutVerts = allVertsExpanded(allVerts(:,3)>=globalMinZ,1:4);
cutVerts = cutVerts(cutVerts(:,3)<=globalMaxZ,1:4);
cutVertIndList = sub2ind(maskSize,cutVerts(:,1),cutVerts(:,2),cutVerts(:,3));
subVerts = cutVerts;
subVerts(:,3) = cutVerts(:,3) - globalMinZ + 1;
subVerts(:,1:3) = ceil(double(subVerts(:,1:3))/3);
subVertIndList = sub2ind(subMaskSize,subVerts(:,1),subVerts(:,2),subVerts(:,3));
%---

%%
vertBarrelValueList = zeros(numVerts,1,'int8');

subMask = zeros(subMaskSize,'int8')-128;
barrelIter = 1;
for barrelIter = 1:numBarrels;
    tic
    disp(barrelIter);
    thisBarrelCoords = barrels{barrelIter};
    minZ = min(thisBarrelCoords(:,3)) - extendBarrelZRange;
    maxZ = max(thisBarrelCoords(:,3)) + extendBarrelZRange;
    
    
    if extendFullRangeFlag == 1,
        minZ = globalMinZ;disp('Extending each barrel across all Z of barrel field');
        maxZ = globalMaxZ;
    end%if extendFullRangeFlag == 1,
    
    
    
    
    indOffset = (minZ-1)*maskSize(1)*maskSize(2);
    frameCoords = thisBarrelCoords(find(thisBarrelCoords(:,3) == min(thisBarrelCoords(:,3))),1:2);
    thisFrame = false(maskSize(1),maskSize(2));
    frameIndList = sub2ind(size(thisFrame),frameCoords(:,1),frameCoords(:,2));
    thisFrame(frameIndList) = 1;
    closeFrame = imclose(thisFrame,circle2d(closeCircleDiameter));
    fillFrame = imfill(closeFrame,'holes');
    erodeFrame = imerode(fillFrame,circle2d(erodeCircleDiameter));
    dilateFrame = imdilate(fillFrame,circle2d(erodeCircleDiameter));
    outsideFrame = imdilate(fillFrame,circle2d(dilateCircleDiameter));
    outsideFrame = (outsideFrame-dilateFrame)>0;
    edgeFrame = (dilateFrame-erodeFrame)>0;
    insideFrame = erodeFrame;
    valueFrame = int8(zeros(maskSize(1),maskSize(2)))-128;
    valueFrame(insideFrame>0) = barrelIter;
    valueFrame(edgeFrame>0) = -1*barrelIter;
    valueFrame(outsideFrame>0) = -127;
    subValueFrame = imresize(valueFrame,[subMaskSize(1),subMaskSize(2)],'nearest');
    insideFrameIndList = find(insideFrame);
    edgeFrameIndList = find(edgeFrame);
    outsideFrameIndList = find(outsideFrame);
    zStart = minZ-globalMinZ+1;
    zStop = maxZ-globalMinZ+1;
    insideIndList = [];
    edgeIndList = [];
    outsideIndList = [];
    for zIter = minZ:maxZ,
        cutZ = zIter-globalMinZ+1;
        subZ = ceil(cutZ/divRatio);
%         indOffset = (zIter-1)*maskSize(1)*maskSize(2);
%         insideIndList = [insideIndList; insideFrameIndList+indOffset];
%         edgeIndList = [edgeIndList; edgeFrameIndList+indOffset];
        %---
        subMask(:,:,subZ) = max(subMask(:,:,subZ),subValueFrame);
        %---
    end%for zIter = ...
    toc
end%for barrelIter = ...
%---
for zIter = 1:subMaskSize(3),
    origFrame = subMask(:,:,zIter);
    thisFrame = origFrame~=-128;
    fillFrame = imfill(thisFrame,'holes');
    addFrame = (fillFrame-thisFrame)>0;
    joinedFrame = origFrame;
    joinedFrame(find(addFrame)) = -127;
    subMask(:,:,zIter) = joinedFrame;
end%for zIter;
vertBarrelValueList(subVerts(:,4)) = subMask(subVertIndList);
   

viewstack(and(subMask>-126,subMask<0));
viewstack(subMask==-127);


%%
%%
bufferZone = 000;
y1 = bufferZone+1; x1 = bufferZone+1;
y2 = maskSize(1)-bufferZone;
x2 = maskSize(2)-bufferZone;
subBufferZone = ceil(bufferZone./divRatio);
y1sub = subBufferZone+1; x1sub = subBufferZone+1;
y2sub = subMaskSize(1)-subBufferZone;
x2sub = subMaskSize(2)-subBufferZone;


croppedSubMask = subMask;
croppedSubMask(1:y1sub,:,:) = -128;
croppedSubMask(y2sub:end,:,:) = -128;
croppedSubMask(:,1:x1sub,:) = -128;
croppedSubMask(:,x2sub:end,:) = -128;
f1 = or(allVerts(:,1)<y1,allVerts(:,1)>y2);
f2 = or(allVerts(:,2)<x1,allVerts(:,2)>x2);
fAll = or(f1,f2);
croppedVertBarrelValueList = vertBarrelValueList;
croppedVertBarrelValueList(fAll) = -128;


numOutsideVoxels = sum(croppedSubMask(:)==-127);
numEdgeVoxels = sum(and(croppedSubMask(:)>-101,croppedSubMask(:)<0));
numInsideVoxels = sum(croppedSubMask(:)>0);
numNonCountVoxels = sum(croppedSubMask(:)==-128);


outsideVertIndexList = find(croppedVertBarrelValueList==-127);
edgeVertIndexList = find(and(croppedVertBarrelValueList>-126,croppedVertBarrelValueList<0));
insideVertIndexList = find(croppedVertBarrelValueList>0);

outsideVertDensity = numel(outsideVertIndexList)./numOutsideVoxels;
edgeVertDensity = numel(edgeVertIndexList)./numEdgeVoxels;
insideVertDensity = numel(insideVertIndexList)./numInsideVoxels;
disp('All Vessels:');
disp([1, edgeVertDensity./insideVertDensity,outsideVertDensity./insideVertDensity]);
%---


ov = allVerts(outsideVertIndexList,:);
iv = allVerts(insideVertIndexList,:);
ev = allVerts(edgeVertIndexList,:);

figure(11);pov = plot3(ov(:,1),ov(:,2),-1*ov(:,3),'c.');hold on
figure(11);piv = plot3(iv(:,1),iv(:,2),-1*iv(:,3),'k.');hold on
figure(11);pev = plot3(ev(:,1),ev(:,2),-1*ev(:,3),'m.');hold off


%%


vectorProject.vertBarrelValueList = vertBarrelValueList;
vectorProject.barrelValueSubMask = subMask;
vectorProject.barrelBoundaryParameters.lastRunDate = datestr(now);
% % vectorProject.barrelBoundaryParameters.cutVertIndexList = cutVertIndList; 
% % vectorProject.barrelBoundaryParameters.subVertIndexList = subVertIndList;

