%load allProject03c.mat

vp = allProject.db;

velocityDivisor = 2e7;
maxDepth = 800;
smoothPlotFlag = 1;
fluxDivisor = 1

vs = vp.vectorizedStructure;
strandIndices = vs.Vertices.StrandIndices;
strands = vs.Strands;
numStrands = numel(strands);



fullStrandCurrentList = abs([strands(:).strandCurrent])';
fullStrandRadiusList = vs.adjustedStrandRadiusList;

if smoothPlotFlag == 1,
    meanStrandRadiusList = zeros(numStrands,1);
    for kk = 1:numStrands,
        meanStrandRadiusList(kk) = mean(vs.Vertices.AllRadii(strands(kk).StartToEndIndices));
    end
    fullStrandRadiusList = (vs.adjustedStrandRadiusList*1 + meanStrandRadiusList)/2;
end%if smoothPlotFlag == 1,

fullStrandZList = vs.strandMidYXZList(:,3);
allFlagList = true(size(fullStrandZList));
noFlagList = false(size(fullStrandZList));

%---

shallowZFlagList = fullStrandZList < maxDepth;
capFlagList = fullStrandRadiusList <= 3.5;

shallowCapFlagList = and(shallowZFlagList,capFlagList);
shallowCapCurrentList = fullStrandCurrentList(shallowCapFlagList);
shallowCapRadiusList = fullStrandRadiusList(shallowCapFlagList);
shallowCapZList = fullStrandZList(shallowCapFlagList);
shallowCapAreaList = pi*shallowCapRadiusList.^2;
shallowCapVelocityList = shallowCapCurrentList./shallowCapAreaList;
shallowCapCalibratedVelocityList = shallowCapVelocityList/velocityDivisor;

figure(1);
loglog(2*shallowCapRadiusList,shallowCapCalibratedVelocityList,'k.');hold on
set(gca,'XLim',[1,100],'YLim',[0.001,50]);


%--

allArterioleFlagList = noFlagList;
allArterioleFlagList(vs.allPlungingArterioleList) = 1;
allArterioleCurrentList = fullStrandCurrentList(allArterioleFlagList);
allArterioleRadiusList = fullStrandRadiusList(allArterioleFlagList);
allArterioleZList = fullStrandZList(allArterioleFlagList);
allArterioleAreaList = pi*allArterioleRadiusList.^2;
allArterioleVelocityList = allArterioleCurrentList./allArterioleAreaList;
allArterioleCalibratedVelocityList = allArterioleVelocityList/velocityDivisor;

% figure(1);hold on;
% loglog(2*allArterioleRadiusList,allArterioleCalibratedVelocityList,'r.');
% set(gca,'XLim',[1,100],'YLim',[0.001,50]);hold off
% 

%--

allVenuleFlagList = noFlagList;
allVenuleFlagList(vs.allPlungingVenuleList) = 1;
allVenuleCurrentList = fullStrandCurrentList(allVenuleFlagList);
allVenuleRadiusList = fullStrandRadiusList(allVenuleFlagList);
allVenuleZList = fullStrandZList(allVenuleFlagList);
allVenuleAreaList = pi*allVenuleRadiusList.^2;
allVenuleVelocityList = allVenuleCurrentList./allVenuleAreaList;
allVenuleCalibratedVelocityList = allVenuleVelocityList/velocityDivisor;

% figure(1);hold on;
% loglog(2*allVenuleRadiusList,allVenuleCalibratedVelocityList,'b.');
% set(gca,'XLim',[1,100],'YLim',[0.001,50]);hold off



%--

arterioleTipFlagList = noFlagList;
arterioleTipFlagList(vs.plungingArterioleTipList) = 1;
arterioleTipCurrentList = fullStrandCurrentList(arterioleTipFlagList);
arterioleTipRadiusList = fullStrandRadiusList(arterioleTipFlagList);
arterioleTipZList = fullStrandZList(arterioleTipFlagList);
arterioleTipAreaList = pi*arterioleTipRadiusList.^2;
arterioleTipVelocityList = arterioleTipCurrentList./arterioleTipAreaList;
arterioleTipCalibratedVelocityList = arterioleTipVelocityList/velocityDivisor;%mm/s
arterioleTipFluxList = (arterioleTipCalibratedVelocityList*1000).*(pi*arterioleTipRadiusList.^2)/1e6; %nL/s
figure(2);semilogy(arterioleTipFluxList,'r.');


figure(1);hold on;
loglog(2*arterioleTipRadiusList,arterioleTipCalibratedVelocityList,'r.');
set(gca,'XLim',[1,100],'YLim',[0.001,50]);hold off


%--

venuleTipFlagList = noFlagList;
venuleTipFlagList(vs.plungingVenuleTipList) = 1;
venuleTipCurrentList = fullStrandCurrentList(venuleTipFlagList);
venuleTipRadiusList = fullStrandRadiusList(venuleTipFlagList);
venuleTipZList = fullStrandZList(venuleTipFlagList);
venuleTipAreaList = pi*venuleTipRadiusList.^2;
venuleTipVelocityList = venuleTipCurrentList./venuleTipAreaList;
venuleTipCalibratedVelocityList = venuleTipVelocityList/velocityDivisor;

figure(1);hold on;
loglog(2*venuleTipRadiusList,venuleTipCalibratedVelocityList,'b.');
set(gca,'XLim',[1,100],'YLim',[0.001,50]);hold off

%--

shallowArterioleFlagList = noFlagList;
shallowArterioleFlagList(vs.allPlungingArterioleList) = 1;
shallowArterioleFlagList = and(shallowArterioleFlagList,shallowZFlagList);
shallowArterioleCurrentList = fullStrandCurrentList(shallowArterioleFlagList);
shallowArterioleRadiusList = fullStrandRadiusList(shallowArterioleFlagList);
shallowArterioleZList = fullStrandZList(shallowArterioleFlagList);
shallowArterioleAreaList = pi*shallowArterioleRadiusList.^2;
shallowArterioleVelocityList = shallowArterioleCurrentList./shallowArterioleAreaList;
shallowArterioleCalibratedVelocityList = shallowArterioleVelocityList/velocityDivisor;

figure(1);hold on;
loglog(2*shallowArterioleRadiusList,shallowArterioleCalibratedVelocityList,'r.');
set(gca,'XLim',[1,100],'YLim',[0.001,50]);hold off


%--

shallowVenuleFlagList = noFlagList;
shallowVenuleFlagList(vs.allPlungingVenuleList) = 1;
shallowVenuleFlagList = and(shallowVenuleFlagList,shallowZFlagList);
shallowVenuleCurrentList = fullStrandCurrentList(shallowVenuleFlagList);
shallowVenuleRadiusList = fullStrandRadiusList(shallowVenuleFlagList);
shallowVenuleZList = fullStrandZList(shallowVenuleFlagList);
shallowVenuleAreaList = pi*shallowVenuleRadiusList.^2;
shallowVenuleVelocityList = shallowVenuleCurrentList./shallowVenuleAreaList;
shallowVenuleCalibratedVelocityList = shallowVenuleVelocityList/velocityDivisor;

figure(1);hold on;
loglog(2*shallowVenuleRadiusList,shallowVenuleCalibratedVelocityList,'b.');
set(gca,'XLim',[1,100],'YLim',[0.001,50]);hold off



%%

scl = [strands(:).strandCurrent];
srl = vs.adjustedStrandRadiusList;
szl = vs.strandMidYXZList(:,3);

figure(1);loglog(2*shallowCapRadiusList,shallowCapCurrentList,'r.');
allA = vs.allPlungingArterioleList;
allV = vs.allPlungingVenuleList;
vTips = vs.plungingVenuleTipList;

allStrandList = 1:numel(strands);
capStrandList = allStrandList(~ismember(allStrandList,[allA,allV]));
capSCL = scl(capStrandList);
capSRL = srl(capStrandList);
capZList = szl(capStrandList);

shallowFlag = capZList<300;
shallowCapSCL = capSCL(shallowFlag)/fluxDivisor;
shallowCapSRL = capSRL(shallowFlag);

figure(1);loglog(2*shallowCapSRL,shallowCapSCL,'k.');

sub7Flag = shallowCapSRL<=3.5;
sub7SCL = shallowCapSCL(sub7Flag);

x = [0:0.01:4];
h = hist(sub7SCL,x);
figure(2);plot(x(1:end-1),h(1:end-1))

aTips = vs.plungingArterioleTipList;
aTipR = vs.adjustedStrandRadiusList(aTips);
aTipv = 1000* scl(aTips)./fluxDivisor;%in microns/sec

aTipArea = pi*aTipR.^2;%in microns-squared

aFlux = aTipv.*aTipR'/1e6;
figure(3);plot(aFlux,'r.');


vSCL = scl(vTips)/fluxDivisor;
vSRL = srl(vTips);
figure(1);hold on;loglog(2*vSRL,vSCL,'b.');

aSCL = scl(aTips)/fluxDivisor;
aSRL = srl(aTips);
figure(1);hold on;loglog(2*aSRL,aSCL,'r.');hold off

