%Load a FlowProject
%Data is stored in the gds structure.

%First, calculate true strand length (accounting for diagonal connections being longer)
numStrands = numel(gds.Strands);
strandLengthList = zeros(numStrands,1);
strandNumPointsList = zeros(numStrands,1);
for strandIter = 1:numStrands,
    strandID = strandIter;
    thisStrand = gds.Strands(strandID);
    stei = thisStrand.StartToEndIndices;
    av = gds.Vertices.AllVerts(stei,:);
    numPoints = numel(stei);
    pointSep = zeros(100,1);
    for pointIter = 1:numPoints-1;
        pointDist = av(pointIter,:) - av(pointIter+1,:);
        pointSep(pointIter) = sqrt(sum(pointDist.^2));
    end%for pointIter
    strandLengthList(strandID) = sum(pointSep)+1;
    strandNumPointsList(strandID) = numPoints;
end%for strandIter
clear strandID thisStrand iv av numPoints pointSep pointDist pointIter
gds.strandLengthList = strandLengthList;
gds.strandNumPointsList = strandNumPointsList;


%%

%Take the mean of the (50% - 90%) range of the sorted radius list as the effective Strand
%Radius.
effectiveStrandRadiusList = zeros(numStrands,1);
numStrands = numel(gds.Strands);
for strandIter = 1:numStrands,
    stei = gds.Strands(strandIter).StartToEndIndices;
    rr = gds.Vertices.AllRadii(stei);
    s = sort(rr);
    cutPoint1 = floor(numel(s)* 0.50);
    cutPoint2 = ceil(numel(s)*0.90);
    cutRR = s(cutPoint1:cutPoint2);
    effectiveStrandRadiusList(strandIter) = mean(cutRR);    
end
clear stei rr s cutPoint1 cutPoint2 cutRR
gds.effectiveStrandRadiusList = effectiveStrandRadiusList;

%%
%Calculate the midpoint Y,X,Z positions of each strand
numStrands = numel(gds.Strands);
strandMidYXZList = zeros(numStrands,3);
for strandIter = 1:numStrands,
    yxz = gds.Strands(strandIter).YXZLimits;
    strandMidYXZList(strandIter,:) = mean(yxz,2);  
end
clear yxz
gds.strandMidYXZList = strandMidYXZList;



%%

%Calculate the end-to-end straight-line separation of each strand
strandEndSeparationList = zeros(numStrands,1);
numStrands = numel(gds.Strands);
for strandIter = 1:numStrands,
    stei = gds.Strands(strandIter).StartToEndIndices;
    p1 = gds.Vertices.AllVerts(stei(1),:);
    p2 = gds.Vertices.AllVerts(stei(end),:);
    dp = p1-p2;
    strandEndSeparationList(strandIter) = sqrt(sum(dp.^2));
end
clear stei p1 p2 dp
gds.strandEndSeparationList = strandEndSeparationList;


%%
saveFileName = ['FVFile_',datestr(now,'yyyy_mm_dd-HHMMSS'),'.mat'];
[fileName,pathName] = uiputfile(saveFileName,'Save FV as...');
cd(pathName)
saveFileName = [fileName];
save(saveFileName,'gds');


 





