function vectorProject = CalculateBarrelVersusBloodOxFunction(vectorProject)
%%
vectorizedStructure = vectorProject.vectorizedStructure;
vectorProjectRootName = vectorProject.vectorProjectRootName;
dashPos = findstr(vectorProjectRootName,'-');
vectorProjectShortName = vectorProjectRootName(1:dashPos-1);
maskSize = vectorizedStructure.maskSize;
strands = vectorizedStructure.Strands;
vertices = vectorizedStructure.Vertices;
numVerts = numel(vertices.AllRadii);
numStrands = numel(strands);
allVerts = vertices.AllVerts;
barrelValueSubMask = vectorProject.barrelValueSubMask;
subMaskSize = size(barrelValueSubMask);
divRatio = vectorProject.barrelBoundaryParameters.divRatio;
vertBarrelValueList = vectorProject.vertBarrelValueList;
vertNormMeanBloodOxList = vectorProject.vectorizedStructure.vertNormMeanBloodOxList;



vertNormMeanBloodOxList = vectorProject.vectorizedStructure.vertMeanBloodOxList;

% % % vertNormMeanBloodOxList = vertices.AllVoltages;
% % % vertNormMeanBloodOxList = vertNormMeanBloodOxList - min(vertNormMeanBloodOxList);
% % % vertNormMeanBloodOxList = vertNormMeanBloodOxList./max(vertNormMeanBloodOxList);
% % % 


barrelVersusBloodOxParameters = vectorProject.barrelVersusBloodOxParameters;
bufferZone = barrelVersusBloodOxParameters.bufferZone;
bloodOxCutoffList = barrelVersusBloodOxParameters.bloodOxCutoffList;




%---
y1 = bufferZone+1; x1 = bufferZone+1;
y2 = maskSize(1)-bufferZone;
x2 = maskSize(2)-bufferZone;
subBufferZone = ceil(bufferZone./divRatio);
y1sub = subBufferZone+1; x1sub = subBufferZone+1;
y2sub = subMaskSize(1)-subBufferZone;
x2sub = subMaskSize(2)-subBufferZone;
%---
%barrelValueSubMask: subsampled version of barrelMask (int8)
%   Outside of barrel field = -128
%   Outside of barrels = -127
%   Edge of barrel = -1*(barrelID)
%   Inside of barrel = +1*(barrelID)
croppedSubMask = barrelValueSubMask;
croppedSubMask(1:y1sub,:,:) = -128;
croppedSubMask(y2sub:end,:,:) = -128;
croppedSubMask(:,1:x1sub,:) = -128;
croppedSubMask(:,x2sub:end,:) = -128;
f1 = or(allVerts(:,1)<y1,allVerts(:,1)>y2);
f2 = or(allVerts(:,2)<x1,allVerts(:,2)>x2);
fAll = or(f1,f2);
croppedVertBarrelValueList = vertBarrelValueList;
croppedVertBarrelValueList(fAll) = -128;
%---
%Calculate number of voxels in each region of submask:
numOutsideVoxels = sum(croppedSubMask(:)==-127);
numEdgeVoxels = sum(and(croppedSubMask(:)>-101,croppedSubMask(:)<0));
numInsideVoxels = sum(croppedSubMask(:)>0);
numNonCountVoxels = sum(croppedSubMask(:)==-128);
%---
%Find indices of vertices (allVerts = (nx3) list) that fall within each region
outsideVertIndexList = find(croppedVertBarrelValueList==-127);
edgeVertIndexList = find(and(croppedVertBarrelValueList>-126,croppedVertBarrelValueList<0));
insideVertIndexList = find(croppedVertBarrelValueList>0);
%---
%Calculate the density of vessel points in each region
outsideVertDensity = numel(outsideVertIndexList)./numOutsideVoxels;
edgeVertDensity = numel(edgeVertIndexList)./numEdgeVoxels;
insideVertDensity = numel(insideVertIndexList)./numInsideVoxels;
disp('Vessel density relative to inside density: (inside, edge, outside)');
disp([1, edgeVertDensity./insideVertDensity,outsideVertDensity./insideVertDensity]);
%---
%Display the vertices that fall within each region
ov = allVerts(outsideVertIndexList,:);
iv = allVerts(insideVertIndexList,:);
ev = allVerts(edgeVertIndexList,:);
figure(11);pov = plot3(ov(:,1),ov(:,2),-1*ov(:,3),'c.');hold on
figure(11);piv = plot3(iv(:,1),iv(:,2),-1*iv(:,3),'k.');hold on
figure(11);pev = plot3(ev(:,1),ev(:,2),-1*ev(:,3),'m.');hold off
%---
%Histogram the blood ox on the inside, edge, and outside
oxRange = [0:0.01:1];
insideOxList = vertNormMeanBloodOxList(insideVertIndexList);
edgeOxList = vertNormMeanBloodOxList(edgeVertIndexList);
outsideOxList = vertNormMeanBloodOxList(outsideVertIndexList);
insideHist = hist(insideOxList,oxRange);
edgeHist = hist(edgeOxList,oxRange);
outsideHist = hist(outsideOxList,oxRange);
figure(91);b91 = bar(oxRange,insideHist);
figure(92);b92 = bar(oxRange,edgeHist);
figure(93);b93 = bar(oxRange,outsideHist);
disp('median');
disp([median(insideOxList),median(edgeOxList),median(outsideOxList)])
disp('mean');
disp([mean(insideOxList),mean(edgeOxList),mean(outsideOxList)])


figure(99);plot(outsideOxList,'r.');


%---
%Look in slices of blood-ox value, and determine how many are in-versus-out
selectedOutsideDensity = [];
selectedEdgeDensity = [];
selectedInsideDensity = [];
numVertsInside = [];
numVertsEdge = [];
numVertsOutside = [];


sortOx = sort(vertNormMeanBloodOxList,'ascend');
c1 = sortOx(ceil(0.9*numVerts));
c2 = sortOx(ceil(0.8*numVerts));
c3 = sortOx(ceil(0.7*numVerts));
c4 = sortOx(ceil(0.6*numVerts));
c5 = sortOx(ceil(0.5*numVerts));
c6 = sortOx(ceil(0.4*numVerts));
c7 = sortOx(ceil(0.3*numVerts));
c8 = sortOx(ceil(0.2*numVerts));
c9 = sortOx(ceil(0.1*numVerts));
bloodOxCutoffList = [1, c1, c2, c3, c4, c5, c6, c7, c8, c9, 0]; 



for oxIter = 1:numel(bloodOxCutoffList)-1,
    highOx = bloodOxCutoffList(oxIter);
    lowOx = bloodOxCutoffList(oxIter+1);
    thisSetVertList = find(and(vertNormMeanBloodOxList>lowOx,vertNormMeanBloodOxList<highOx));
    selectedBVL = croppedVertBarrelValueList(thisSetVertList);
    selectedOutsideIndexList = find(selectedBVL==-127);
    selectedEdgeIndexList = find(and(selectedBVL>-126,selectedBVL<0));
    selectedInsideIndexList = find(selectedBVL>0);
    %---
    numVertsInside(oxIter) = numel(selectedInsideIndexList);
    numVertsEdge(oxIter) = numel(selectedEdgeIndexList);
    numVertsOutside(oxIter) = numel(selectedOutsideIndexList);
    %---
    selectedOutsideDensity(oxIter) = numel(selectedOutsideIndexList)./numOutsideVoxels;
    selectedEdgeDensity(oxIter) = numel(selectedEdgeIndexList)./numEdgeVoxels;
    selectedInsideDensity(oxIter) = numel(selectedInsideIndexList)./numInsideVoxels;
    %---
end%for oxIter

selectedAllDensity = selectedOutsideDensity + selectedEdgeDensity + selectedInsideDensity;
% % figure(3);plot(selectedInsideDensity,'r-');hold on
% % plot(selectedEdgeDensity,'g-');
% % plot(selectedOutsideDensity,'b-');hold off
%---
xRange = bloodOxCutoffList(1:end-1);
        xRange = [1:10]
figure(4);plot(xRange,selectedInsideDensity./selectedAllDensity,'m-');hold on
figure(4);plot(xRange,selectedEdgeDensity./selectedAllDensity,'y-');hold on
figure(4);plot(xRange,selectedOutsideDensity./selectedAllDensity,'c-');hold on
figure(4);plot(xRange,selectedInsideDensity./selectedOutsideDensity,'k-');hold off
xlabel('NormalizedBloodOx [1(0)= mean of plunging arteries(veins)');
ylabel('Fraction of vessel in each region');
title([vectorProjectShortName,' - pink/yellow/cyan = inside/edge/outside']);

%---
% % figure(5);plot(xRange,selectedInsideDensity./insideVertDensity,'r-');hold on
% % figure(5);plot(xRange,selectedEdgeDensity./edgeVertDensity,'g-');hold on
% % figure(5);plot(xRange,selectedOutsideDensity./outsideVertDensity,'b-');hold off
%---
figure(6);plot(xRange,numVertsInside,'r-');hold on
figure(6);plot(xRange,numVertsEdge,'g-');hold on
figure(6);plot(xRange,numVertsOutside,'b-');hold off
xlabel('NormalizedBloodOx [1(0)= mean of plunging arteries(veins)');
ylabel('Number of vessel vertices at each blood ox in each region');
title([vectorProjectShortName,' - red/green/blue = inside/edge/outside']);

%%
%optional code:
%Find the source of the different oxygenation blood:
strandIndices = vertices.StrandIndices;
allSourceStrandList = [];
allSourceStrandZ = [];
strandByStrandSourceList = zeros(numStrands,1);
strandByStrandSourceZ = zeros(numStrands,1);
figure(111);hold off;clf

modVertNormMeanBloodOxList = vertNormMeanBloodOxList*0-99;
countVertIndexList = [insideVertIndexList;edgeVertIndexList;outsideVertIndexList];
modVertNormMeanBloodOxList(countVertIndexList) = vertNormMeanBloodOxList(countVertIndexList);


for oxIter = 1:numel(bloodOxCutoffList)-1,
    highOx = bloodOxCutoffList(oxIter);
    lowOx = bloodOxCutoffList(oxIter+1);
    thisSetVertList = find(and(modVertNormMeanBloodOxList>lowOx,modVertNormMeanBloodOxList<highOx));
    thisSetStrandList = unique(strandIndices(thisSetVertList));
    numSubStrands = numel(thisSetStrandList);
    sourceStrandList = [];
    for strandIter = 1:numSubStrands,
        startStrandID = thisSetStrandList(strandIter);
        thisStrandID = startStrandID;
        thisStrand = strands(startStrandID);
        keepGoing = 1;
        counter = 0;
        while keepGoing == 1,
            counter = counter+1;
            %follow upstream
            if thisStrand.StartVoltage>thisStrand.EndVoltage,
                neighborList = thisStrand.StartVertexNeighborStrands;
            else
                neighborList = thisStrand.EndVertexNeighborStrands;
            end
            neighborList = neighborList(neighborList~=thisStrandID);
            currentList = abs([strands(neighborList).MedianVoltage]);
            [maxVal,maxPos] = max(currentList);
            upStrandID = neighborList(maxPos(1));
            if ismember(upStrandID,vectorProject.vectorizedStructure.allPlungingArterioleList),
                keepGoing = 0;
            end
            if counter>100, disp('timed out'),keepGoing = 0;end
            thisStrandID = upStrandID;
            thisStrand = strands(upStrandID);
        end%while keepGoing..
        sourceStrandList(strandIter) = upStrandID;
        sourceStrandZ(strandIter) = vectorProject.vectorizedStructure.strandMidYXZList(upStrandID,3);
        strandByStrandSourceList(startStrandID) = upStrandID;
        strandByStrandSourceZ(startStrandID) = vectorProject.vectorizedStructure.strandMidYXZList(upStrandID,3);    
    end%for strandIter...
allSourceStrandList{oxIter} = sourceStrandList;
allSourceStrandZ{oxIter} = sourceStrandZ;
meanSourceStrandZ(oxIter) = mean(sourceStrandZ);
figure(111);plot(xRange(oxIter),sourceStrandZ,'g.');hold on
end%for oxIter


figure(81);plot(xRange,meanSourceStrandZ,'g.');

insideSourceZ = strandByStrandSourceZ(strandIndices(outsideVertIndexList,1));
edgeSourceZ = strandByStrandSourceZ(strandIndices(edgeVertIndexList,1));
outsideSourceZ = strandByStrandSourceZ(strandIndices(insideVertIndexList,1));

vertByVertSourceZ = zeros(length(allVerts),1);
for vertIter = 1:numVerts,
    vertSourceZ = strandByStrandSourceZ(strandIndices(vertIter));
    vertByVertSourceZ(vertIter) = vertSourceZ;
% %     if or(vertSourceZ <200, vertSourceZ>650),
% %         vertNormMeanBloodOxList(vertIter) = -99;
% %     end
end
    

insideVertStuff = [insideVertIndexList,vertByVertSourceZ(insideVertIndexList)];
edgeVertStuff = [edgeVertIndexList,vertByVertSourceZ(edgeVertIndexList)];
outsideVertStuff = [outsideVertIndexList,vertByVertSourceZ(outsideVertIndexList)];

deepCutoff = 700;
inDeep = insideVertStuff(insideVertStuff(:,2)>deepCutoff,:);
edgeDeep = edgeVertStuff(edgeVertStuff(:,2)>deepCutoff,:);
outDeep = outsideVertStuff(outsideVertStuff(:,2)>deepCutoff,:);

numInDeep = length(inDeep);
numEdgeDeep = length(edgeDeep);
numOutDeep = length(outDeep);
countDeep = numInDeep+numEdgeDeep+numOutDeep;

countVoxels = numOutsideVoxels+numEdgeVoxels+numInsideVoxels;
volumeRatio = [numInsideVoxels,numEdgeVoxels,numOutsideVoxels]/countVoxels;
deepRatio = [numInDeep,numEdgeDeep,numOutDeep]/countDeep;
distributionRatio = deepRatio./volumeRatio


%%
barrelVersusBloodOxStruct.xRange = xRange;
barrelVersusBloodOxStruct.selectedInsideDensity = selectedInsideDensity;
barrelVersusBloodOxStruct.selectedEdgeDensity = selectedEdgeDensity;
barrelVersusBloodOxStruct.selectedOutsideDensity = selectedOutsideDensity;
barrelVersusBloodOxStruct.selectedAllDensity = selectedAllDensity;
barrelVersusBloodOxStruct.insideVertDensity = insideVertDensity;
barrelVersusBloodOxStruct.edgeVertDensity = edgeVertDensity;
barrelVersusBloodOxStruct.outsideVertDensity = outsideVertDensity;
barrelVersusBloodOxStruct.numVertsInside = numVertsInside;
barrelVersusBloodOxStruct.numVertsEdge = numVertsEdge;
barrelVersusBloodOxStruct.numVertsOutside = numVertsOutside;



vectorProject.barrelVersusBloodOxStruct = barrelVersusBloodOxStruct;
vectorProject.barrelVersusBloodOxParameters.lastRunDate = datestr(now);

