% function vectorProject = CalculateBarrelVersusPathStats(vectorProject)
%%
vectorizedStructure = vectorProject.vectorizedStructure;
vectorProjectRootName = vectorProject.vectorProjectRootName;
dashPos = findstr(vectorProjectRootName,'-');
vectorProjectShortName = vectorProjectRootName(1:dashPos-1);
maskSize = vectorizedStructure.maskSize;
strands = vectorizedStructure.Strands;
vertices = vectorizedStructure.Vertices;
strandIndices = vertices.StrandIndices;
numVerts = numel(vertices.AllRadii);
numStrands = numel(strands);
allVerts = vertices.AllVerts;
barrelValueSubMask = vectorProject.barrelValueSubMask;
subMaskSize = size(barrelValueSubMask);
divRatio = vectorProject.barrelBoundaryParameters.divRatio;
vertBarrelValueList = vectorProject.vertBarrelValueList;
vertNormMeanBloodOxList = vectorProject.vectorizedStructure.vertNormMeanBloodOxList;
vertMeanBloodOxList = vectorProject.vectorizedStructure.vertMeanBloodOxList;


barrelVersusBloodOxParameters = vectorProject.barrelVersusBloodOxParameters;
bufferZone = barrelVersusBloodOxParameters.bufferZone;
bloodOxCutoffList = barrelVersusBloodOxParameters.bloodOxCutoffList;


%%
%Find the vertices that are in the various regions of the barrel field
bufferZone = 200;disp('HARDCODE OVERRIDE');
%---
y1 = bufferZone+1; x1 = bufferZone+1;
y2 = maskSize(1)-bufferZone;
x2 = maskSize(2)-bufferZone;
subBufferZone = ceil(bufferZone./divRatio);
y1sub = subBufferZone+1; x1sub = subBufferZone+1;
y2sub = subMaskSize(1)-subBufferZone;
x2sub = subMaskSize(2)-subBufferZone;




% %             barrelValueSubMask(barrelValueSubMask==-128)=-127;
% %             vertBarrelValueList(vertBarrelValueList==-128)=-127;





%---
%barrelValueSubMask: subsampled version of barrelMask (int8)
%   Outside of barrel field = -128
%   Outside of barrels = -127
%   Edge of barrel = -1*(barrelID)
%   Inside of barrel = +1*(barrelID)
croppedSubMask = barrelValueSubMask;
croppedSubMask(1:y1sub,:,:) = -128;
croppedSubMask(y2sub:end,:,:) = -128;
croppedSubMask(:,1:x1sub,:) = -128;
croppedSubMask(:,x2sub:end,:) = -128;
f1 = or(allVerts(:,1)<y1,allVerts(:,1)>y2);
f2 = or(allVerts(:,2)<x1,allVerts(:,2)>x2);
fAll = or(f1,f2);
croppedVertBarrelValueList = vertBarrelValueList;
croppedVertBarrelValueList(fAll) = -128;
%---
%Calculate number of voxels in each region of submask:
numOutsideVoxels = sum(croppedSubMask(:)==-127);
numEdgeVoxels = sum(and(croppedSubMask(:)>-101,croppedSubMask(:)<0));
numInsideVoxels = sum(croppedSubMask(:)>0);
numNonCountVoxels = sum(croppedSubMask(:)==-128);
numCountVoxels = numOutsideVoxels+numEdgeVoxels+numInsideVoxels;
%---
%Find indices of vertices (allVerts = (nx3) list) that fall within each region
outsideVertIndexList = find(croppedVertBarrelValueList==-127);
edgeVertIndexList = find(and(croppedVertBarrelValueList>-126,croppedVertBarrelValueList<0));
insideVertIndexList = find(croppedVertBarrelValueList>0);
activeVertIndexList = [outsideVertIndexList;edgeVertIndexList;insideVertIndexList];
%---
%Calculate the density of vessel points in each region
outsideVertDensity = numel(outsideVertIndexList)./numOutsideVoxels;
edgeVertDensity = numel(edgeVertIndexList)./numEdgeVoxels;
insideVertDensity = numel(insideVertIndexList)./numInsideVoxels;
disp('Vessel density relative to inside density: (inside, edge, outside)');
disp([1, edgeVertDensity./insideVertDensity,outsideVertDensity./insideVertDensity]);
%---
%Display the vertices that fall within each region
ov = allVerts(outsideVertIndexList,:);
iv = allVerts(insideVertIndexList,:);
ev = allVerts(edgeVertIndexList,:);
figure(11);pov = plot3(ov(:,1),ov(:,2),-1*ov(:,3),'c.');hold on
figure(11);piv = plot3(iv(:,1),iv(:,2),-1*iv(:,3),'k.');hold on
figure(11);pev = plot3(ev(:,1),ev(:,2),-1*ev(:,3),'m.');hold off
%---
%%
%Find the strands that are in the barrelfield
strandByStrandSourceList = zeros(numStrands,1);

modVertNormMeanBloodOxList = vertNormMeanBloodOxList*0-99;
countVertIndexList = [insideVertIndexList;edgeVertIndexList;outsideVertIndexList];
modVertNormMeanBloodOxList(countVertIndexList) = vertNormMeanBloodOxList(countVertIndexList);
startStrandList = unique(strandIndices(activeVertIndexList,1));
numStartStrands = numel(startStrandList);
strandByStrandSourceZ = zeros(numStrands,1);
sourceStrandList = [];
sourceStrandZList = [];
allUpstreamPathList = [];
allDownstreamPathList = [];
for strandIter = 1:numStartStrands,
    startStrandID = startStrandList(strandIter);
    thisStrandID = startStrandID;
    thisStrand = strands(startStrandID);
    %Track upstream
    keepGoing = 1;
    upstreamPath = [];
    counter = 0;
    while keepGoing == 1,
        counter = counter+1;
        %follow upstream
        if thisStrand.StartVoltage>thisStrand.EndVoltage,
            neighborList = thisStrand.StartVertexNeighborStrands;
        else
            neighborList = thisStrand.EndVertexNeighborStrands;
        end
        neighborList = neighborList(neighborList~=thisStrandID);
        if isempty(neighborList),
            keepGoing = 0;
        else %if isemtpy...
            medianVoltageList = abs([strands(neighborList).MedianVoltage]);
            [maxVal,maxPos] = max(medianVoltageList);
            upStrandID = neighborList(maxPos(1));
            upstreamPath = [upstreamPath,upStrandID];
            if ismember(upStrandID,vectorProject.vectorizedStructure.allPlungingArterioleList),
                keepGoing = 0;
            end
            if counter>100, disp('timed out'),keepGoing = 0;end
            thisStrandID = upStrandID;
            thisStrand = strands(upStrandID);
        end %if isempty(neighborList)
    end%while keepGoing..
    allUpstreamPathList{strandIter} = upstreamPath;
    sourceStrandList(strandIter) = upStrandID;
    sourceStrandZ(strandIter) = vectorProject.vectorizedStructure.strandMidYXZList(upStrandID,3);
    strandByStrandSourceList(upstreamPath) = upStrandID;
    %---
    thisStrandID = startStrandID;
    thisStrand = strands(startStrandID);
    %Track downstream
    keepGoing = 1;
    downstreamPath = [];
    counter = 0;
    while keepGoing == 1,
        counter = counter+1;
        %follow downstream
        if thisStrand.StartVoltage<thisStrand.EndVoltage,
            neighborList = thisStrand.StartVertexNeighborStrands;
        else
            neighborList = thisStrand.EndVertexNeighborStrands;
        end
        neighborList = neighborList(neighborList~=thisStrandID);
        if isempty(neighborList),
            keepGoing = 0;
        else %if isempty(neighborList),
            medianVoltageList = abs([strands(neighborList).MedianVoltage]);
            [minVal,minPos] = min(medianVoltageList);
            downStrandID = neighborList(minPos(1));
            downstreamPath = [downstreamPath,downStrandID];
            if ismember(downStrandID,vectorProject.vectorizedStructure.allPlungingVenuleList),
                keepGoing = 0;
            end
            if counter>100, disp('timed out'),keepGoing = 0;end
            thisStrandID = downStrandID;
            thisStrand = strands(downStrandID);
        end%if isempty(neighborList);
    end%while keepGoing..
    allDownstreamPathList{strandIter} = downstreamPath;
    sourceStrandZList{strandIter} = sourceStrandZ;
    strandByStrandSourceZ(startStrandID) = upStrandID;
end%for strandIter...

vertByVertSourceZ = zeros(length(allVerts),1);
for vertIter = 1:numVerts,
    vertSourceZ = strandByStrandSourceZ(strandIndices(vertIter));
    vertByVertSourceZ(vertIter) = vertSourceZ;
end
    

%%
allAdjValueVertPath = zeros(numStartStrands,3000);
allBloodOxVertPath = zeros(numStartStrands,3000);
fractionPathList = zeros(numStartStrands,1);
vertFractionPathList = zeros(length(allVerts),1);
for strandIter = 1:numStartStrands,
    startStrandID = startStrandList(strandIter);
    upStreamPath = allUpstreamPathList{strandIter};
    upStreamVertPath = [strands(upStreamPath).InteriorVertices];
    downStreamPath = allDownstreamPathList{strandIter};
    downStreamVertPath = [strands(downStreamPath).InteriorVertices];
    fullStrandPath = [fliplr(upStreamPath),startStrandID,downStreamPath];
    fullVertPath = [strands(fullStrandPath).InteriorVertices];
    fullVertStrandPath = strandIndices(fullVertPath);
    fullVertCurrentPath = abs([strands(fullVertStrandPath).strandCurrent]);
    fullVertTimePath = cumsum(1./fullVertCurrentPath);
    bloodOxPath = vertMeanBloodOxList(fullVertPath);
    barrelValueVertPath = vertBarrelValueList(fullVertPath);
    adjValueVertPath = barrelValueVertPath;
    adjValueVertPath(barrelValueVertPath == -128) = 50; %inside barrel planes, but outside barrel field
    adjValueVertPath(barrelValueVertPath == -127) = 100; %inside barrel field, but outside barrels
    adjValueVertPath(adjValueVertPath <0) = -1; %inside barrel edge
    adjValueVertPath(barrelValueVertPath >0) = 127; %inside barrel edge
%     figure(1);plot(adjValueVertPath); pause
    allAdjValueVertPath(strandIter,1:length(adjValueVertPath)) = adjValueVertPath;
    allBloodOxVertPath(strandIter,1:length(bloodOxPath)) = bloodOxPath;
    fractionPathList(strandIter) = numel(upStreamVertPath)./numel(fullVertPath);
    
    
    
            
% %         fractionPathList(strandIter) = fullVertTimePath(numel(upStreamVertPath)+1)./fullVertTimePath(end);
    
            fractionPathList(strandIter) = numel(downStreamVertPath);

    vertFractionPathList(strands(startStrandID).StartToEndIndices) = fractionPathList(strandIter);

    
            

end


x = [1:50:2000];

% % 
% % 
% % vertFractionPathList = vertices.AllRadii;
% % x = [0:0.5:10];




activeVertFractionPathList = vertFractionPathList(activeVertIndexList);
activeVertBarrelValueList = double(vertBarrelValueList(activeVertIndexList));
activeVertBloodOxList = vertMeanBloodOxList(activeVertIndexList);
rr1 = activeVertFractionPathList + rand(size(activeVertFractionPathList))*0.001;
rr2 = activeVertBarrelValueList + rand(size(activeVertBarrelValueList))*5;
rr3 = activeVertBloodOxList + rand(size(activeVertBloodOxList))*0.001;
figure(99);plot(rr1,rr2,'r.');
figure(77);plot(rr1,rr3,'g.');xlabel('fraction path'),ylabel('bloodOx');



%%
%Consider those vessels that fall into the (0.8) fraction path (seem to cluster in barrels?)
selectActiveList = find(and(activeVertFractionPathList>0.75,activeVertFractionPathList<0.85));
selectVertIndexList = activeVertIndexList(selectActiveList);
selectStrands = unique(strandIndices(selectVertIndexList));

selectVoltages = [strands(selectStrands).MedianVoltage];
selectBloodOx = [strands(selectStrands).meanBloodOx];
figure(21);plot(selectBloodOx,'r.');ylabel('BloodOx of the 0.8 fraction pathlength');
    %consider the subgroup with high blood ox
    highSelectStrands = selectStrands(find(and(selectBloodOx>0.00,selectBloodOx<0.9)));
    highSelectVerts = [strands(highSelectStrands).InteriorVertices];
    activeHighSelectVerts =  highSelectVerts(ismember(highSelectVerts,activeVertIndexList));
    activeHighSelectBV = vertBarrelValueList(activeHighSelectVerts);
    numInside = sum(activeHighSelectBV>0)./numInsideVoxels;
    numEdge = sum(and(activeHighSelectBV>-100,activeHighSelectBV<0))./numEdgeVoxels;
    numOutside = sum(activeHighSelectBV==-127)./numOutsideVoxels;
    totalNum = (numInside+numEdge+numOutside);
    myRatio = [numInside,numEdge,numOutside]./totalNum
    

    
%%
insideFractionList = activeVertFractionPathList(activeVertBarrelValueList>0);
edgeFractionList = activeVertFractionPathList(and(activeVertBarrelValueList>-100,activeVertBarrelValueList<0));
outsideFractionList = activeVertFractionPathList(activeVertBarrelValueList==-127);



hIn = hist(insideFractionList,x);nIn = sum(hIn);hIn = hIn./sum(hIn);
hEdge = hist(edgeFractionList,x);nEdge = sum(hEdge);hEdge = hEdge./sum(hEdge);
hOut = hist(outsideFractionList,x);nOut = sum(hOut);hOut = hOut./sum(hOut);

switch lower(vectorProjectShortName),
    case {'pbau'},figure(291);
    case {'pbav'},figure(292);
    case {'pbco'},figure(293);
    case {'pbdb'},figure(295);
end%switch
b1 = bar(x,hOut);set(b1,'FaceColor','blue','BarWidth',0.8);hold on
b2 = bar(x,hEdge);set(b2,'FaceColor','green','BarWidth',0.5);hold on
b3 = bar(x,hIn);set(b3,'FaceColor','red','BarWidth',0.25);hold off
xlabel('fractional pathlength');ylabel('fraction of total in region');
title(vectorProjectShortName)


zz = 5;
for kk = 1+zz:numel(x)-zz,
    mm1(kk) = mean(hOut(kk-zz:kk+zz));
    ss1(kk) = std(hOut(kk-zz:kk+zz))./sqrt(nOut);
    mm2(kk) = mean(hEdge(kk-zz:kk+zz));
    ss2(kk) = std(hEdge(kk-zz:kk+zz))./sqrt(nEdge);
    mm3(kk) = mean(hIn(kk-zz:kk+zz));
    ss3(kk) = std(hIn(kk-zz:kk+zz))./sqrt(nIn);
end
    
figure(92);plot(mm1,'b-');hold on
figure(92);plot(mm1+3*ss1,'b--');hold on;plot(mm1-3*ss1,'b--');
figure(92);plot(mm2,'g-');
figure(92);plot(mm2+3*ss2,'g--');hold on;plot(mm2-3*ss2,'g--');
figure(92);plot(mm3,'r-');
figure(92);plot(mm3+3*ss3,'r--');hold on;plot(mm3-3*ss3,'r--');hold off




activeBloodOxList = [vertMeanBloodOxList(activeVertIndexList),activeVertIndexList,double(vertBarrelValueList(activeVertIndexList))];
sortList = sortrows(activeBloodOxList,1);
sortCutoff = ceil(0.1*length(sortList));
subSortList = sortList(1:sortCutoff,:);
subSortStrands = unique(strandIndices(subSortList(:,2)));
subStartIndexList = find(ismember(startStrandList,subSortStrands));
subAdjValueVertPath = allAdjValueVertPath(subStartIndexList,:);
% % figure(1);imagesc(allAdjValueVertPath);
% % figure(2);imagesc(subAdjValueVertPath);


barrels = vectorProject.barrelBoundaryParameters.barrels;
allBarrelCoords = cat(1,barrels{:});
subBarrelCoords = allBarrelCoords(1:5:end,:);
whereDidItStart = median(allAdjValueVertPath(:,1:100),2);

lowOxVerts = allVerts(subSortList(:,2),:);
% % figure(4);plot3(subBarrelCoords(:,1),subBarrelCoords(:,2),subBarrelCoords(:,3),'k.');hold on;
% % figure(4);plot3(lowOxVerts(:,1),lowOxVerts(:,2),lowOxVerts(:,3),'r.');hold off



barrelLowOxSortList = subSortList(subSortList(:,3)>0,:);
lowOxBarrelVerts = allVerts(barrelLowOxSortList(:,2),:);
% % figure(5);plot3(subBarrelCoords(:,1),subBarrelCoords(:,2),subBarrelCoords(:,3),'k.');hold on;
% % figure(5);plot3(lowOxBarrelVerts(:,1),lowOxBarrelVerts(:,2),lowOxBarrelVerts(:,3),'b.');hold off

% % figure(6);plot(allBloodOxVertPath(:,1))
% % for k = 1:1000,plot(allBloodOxVertPath(k,1:1000),'r.');pause;end



activeVertConvergenceList = strandConvergenceList(strandIndices(activeVertIndexList));

insideConvergenceList = activeVertConvergenceList(activeVertBarrelValueList>0);
edgeConvergenceList = activeVertConvergenceList(and(activeVertBarrelValueList>-100,activeVertBarrelValueList<0));
outsideConvergenceList = activeVertConvergenceList(activeVertBarrelValueList==-127);

disp(['Convergence']);
disp([mean(insideConvergenceList),mean(edgeConvergenceList),mean(outsideConvergenceList)]);




% % ahIn = [ahIn;hIn];
% % ahEdge = [ahEdge;hEdge];
% % ahOut = [ahOut;hOut];


%%













% % % % % % % 
% % % % % % % 
% % % % % % % %%
% % % % % % % figure(81);plot(xRange,meanSourceStrandZ,'g.');
% % % % % % % 
% % % % % % % insideSourceZ = strandByStrandSourceZ(strandIndices(outsideVertIndexList,1));
% % % % % % % edgeSourceZ = strandByStrandSourceZ(strandIndices(edgeVertIndexList,1));
% % % % % % % outsideSourceZ = strandByStrandSourceZ(strandIndices(insideVertIndexList,1));
% % % % % % % 
% % % % % % % vertByVertSourceZ = zeros(length(allVerts),1);
% % % % % % % for vertIter = 1:numVerts,
% % % % % % %     vertSourceZ = strandByStrandSourceZ(strandIndices(vertIter));
% % % % % % %     vertByVertSourceZ(vertIter) = vertSourceZ;
% % % % % % %     
% % % % % % % % %     if or(vertSourceZ <200, vertSourceZ>650),
% % % % % % % % %         vertNormMeanBloodOxList(vertIter) = -99;
% % % % % % % % %     end
% % % % % % % end
% % % % % % %     
% % % % % % % 
% % % % % % % insideVertStuff = [insideVertIndexList,vertByVertSourceZ(insideVertIndexList)];
% % % % % % % edgeVertStuff = [edgeVertIndexList,vertByVertSourceZ(edgeVertIndexList)];
% % % % % % % outsideVertStuff = [outsideVertIndexList,vertByVertSourceZ(outsideVertIndexList)];
% % % % % % % 
% % % % % % % deepCutoff = 700;
% % % % % % % inDeep = insideVertStuff(insideVertStuff(:,2)>deepCutoff,:);
% % % % % % % edgeDeep = edgeVertStuff(edgeVertStuff(:,2)>deepCutoff,:);
% % % % % % % outDeep = outsideVertStuff(outsideVertStuff(:,2)>deepCutoff,:);
% % % % % % % 
% % % % % % % numInDeep = length(inDeep);
% % % % % % % numEdgeDeep = length(edgeDeep);
% % % % % % % numOutDeep = length(outDeep);
% % % % % % % countDeep = numInDeep+numEdgeDeep+numOutDeep;
% % % % % % % 
% % % % % % % countVoxels = numOutsideVoxels+numEdgeVoxels+numInsideVoxels;
% % % % % % % volumeRatio = [numInsideVoxels,numEdgeVoxels,numOutsideVoxels]/countVoxels;
% % % % % % % deepRatio = [numInDeep,numEdgeDeep,numOutDeep]/countDeep;
% % % % % % % distributionRatio = deepRatio./volumeRatio
% % % % % % % 
% % % % % % % 
% % % % % % % 
% % % % % % % 
% % % % % % % 
% % % % % % % %Histogram the blood ox on the inside, edge, and outside
% % % % % % % oxRange = [0:0.01:1];
% % % % % % % insideOxList = vertNormMeanBloodOxList(insideVertIndexList);
% % % % % % % edgeOxList = vertNormMeanBloodOxList(edgeVertIndexList);
% % % % % % % outsideOxList = vertNormMeanBloodOxList(outsideVertIndexList);
% % % % % % % insideHist = hist(insideOxList,oxRange);
% % % % % % % edgeHist = hist(edgeOxList,oxRange);
% % % % % % % outsideHist = hist(outsideOxList,oxRange);
% % % % % % % figure(91);b91 = bar(oxRange,insideHist);
% % % % % % % figure(92);b92 = bar(oxRange,edgeHist);
% % % % % % % figure(93);b93 = bar(oxRange,outsideHist);
% % % % % % % disp('median');
% % % % % % % disp([median(insideOxList),median(edgeOxList),median(outsideOxList)])
% % % % % % % disp('mean');
% % % % % % % disp([mean(insideOxList),mean(edgeOxList),mean(outsideOxList)])
% % % % % % % 
% % % % % % % 
% % % % % % % figure(99);plot(outsideOxList,'r.');
% % % % % % % 
% % % % % % % 
% % % % % % % %---
% % % % % % % %Look in slices of blood-ox value, and determine how many are in-versus-out
% % % % % % % selectedOutsideDensity = [];
% % % % % % % selectedEdgeDensity = [];
% % % % % % % selectedInsideDensity = [];
% % % % % % % numVertsInside = [];
% % % % % % % numVertsEdge = [];
% % % % % % % numVertsOutside = [];
% % % % % % % 
% % % % % % % 
% % % % % % % sortOx = sort(vertNormMeanBloodOxList,'ascend');
% % % % % % % c1 = sortOx(ceil(0.9*numVerts));
% % % % % % % c2 = sortOx(ceil(0.8*numVerts));
% % % % % % % c3 = sortOx(ceil(0.7*numVerts));
% % % % % % % c4 = sortOx(ceil(0.6*numVerts));
% % % % % % % c5 = sortOx(ceil(0.5*numVerts));
% % % % % % % c6 = sortOx(ceil(0.4*numVerts));
% % % % % % % c7 = sortOx(ceil(0.3*numVerts));
% % % % % % % c8 = sortOx(ceil(0.2*numVerts));
% % % % % % % c9 = sortOx(ceil(0.1*numVerts));
% % % % % % % bloodOxCutoffList = [1, c1, c2, c3, c4, c5, c6, c7, c8, c9, 0]; 
% % % % % % % 
% % % % % % % 
% % % % % % % 
% % % % % % % for oxIter = 1:numel(bloodOxCutoffList)-1,
% % % % % % %     highOx = bloodOxCutoffList(oxIter);
% % % % % % %     lowOx = bloodOxCutoffList(oxIter+1);
% % % % % % %     thisSetVertList = find(and(vertNormMeanBloodOxList>lowOx,vertNormMeanBloodOxList<highOx));
% % % % % % %     selectedBVL = croppedVertBarrelValueList(thisSetVertList);
% % % % % % %     selectedOutsideIndexList = find(selectedBVL==-127);
% % % % % % %     selectedEdgeIndexList = find(and(selectedBVL>-126,selectedBVL<0));
% % % % % % %     selectedInsideIndexList = find(selectedBVL>0);
% % % % % % %     %---
% % % % % % %     numVertsInside(oxIter) = numel(selectedInsideIndexList);
% % % % % % %     numVertsEdge(oxIter) = numel(selectedEdgeIndexList);
% % % % % % %     numVertsOutside(oxIter) = numel(selectedOutsideIndexList);
% % % % % % %     %---
% % % % % % %     selectedOutsideDensity(oxIter) = numel(selectedOutsideIndexList)./numOutsideVoxels;
% % % % % % %     selectedEdgeDensity(oxIter) = numel(selectedEdgeIndexList)./numEdgeVoxels;
% % % % % % %     selectedInsideDensity(oxIter) = numel(selectedInsideIndexList)./numInsideVoxels;
% % % % % % %     %---
% % % % % % % end%for oxIter
% % % % % % % 
% % % % % % % selectedAllDensity = selectedOutsideDensity + selectedEdgeDensity + selectedInsideDensity;
% % % % % % % % % figure(3);plot(selectedInsideDensity,'r-');hold on
% % % % % % % % % plot(selectedEdgeDensity,'g-');
% % % % % % % % % plot(selectedOutsideDensity,'b-');hold off
% % % % % % % %---
% % % % % % % xRange = bloodOxCutoffList(1:end-1);
% % % % % % % figure(4);plot(xRange,selectedInsideDensity./selectedAllDensity,'m-');hold on
% % % % % % % figure(4);plot(xRange,selectedEdgeDensity./selectedAllDensity,'y-');hold on
% % % % % % % figure(4);plot(xRange,selectedOutsideDensity./selectedAllDensity,'c-');hold on
% % % % % % % figure(4);plot(xRange,selectedInsideDensity./selectedOutsideDensity,'k-');hold off
% % % % % % % xlabel('NormalizedBloodOx [1(0)= mean of plunging arteries(veins)');
% % % % % % % ylabel('Fraction of vessel in each region');
% % % % % % % title([vectorProjectShortName,' - pink/yellow/cyan = inside/edge/outside']);
% % % % % % % 
% % % % % % % %---
% % % % % % % % % figure(5);plot(xRange,selectedInsideDensity./insideVertDensity,'r-');hold on
% % % % % % % % % figure(5);plot(xRange,selectedEdgeDensity./edgeVertDensity,'g-');hold on
% % % % % % % % % figure(5);plot(xRange,selectedOutsideDensity./outsideVertDensity,'b-');hold off
% % % % % % % %---
% % % % % % % figure(6);plot(xRange,numVertsInside,'r-');hold on
% % % % % % % figure(6);plot(xRange,numVertsEdge,'g-');hold on
% % % % % % % figure(6);plot(xRange,numVertsOutside,'b-');hold off
% % % % % % % xlabel('NormalizedBloodOx [1(0)= mean of plunging arteries(veins)');
% % % % % % % ylabel('Number of vessel vertices at each blood ox in each region');
% % % % % % % title([vectorProjectShortName,' - red/green/blue = inside/edge/outside']);
% % % % % % % 
% % % % % % % %%
% % % % % % % %optional code:
% % % % % % % %Find the source of the different oxygenation blood:
% % % % % % % 
% % % % % % % %%
% % % % % % % barrelVersusBloodOxStruct.xRange = xRange;
% % % % % % % barrelVersusBloodOxStruct.selectedInsideDensity = selectedInsideDensity;
% % % % % % % barrelVersusBloodOxStruct.selectedEdgeDensity = selectedEdgeDensity;
% % % % % % % barrelVersusBloodOxStruct.selectedOutsideDensity = selectedOutsideDensity;
% % % % % % % barrelVersusBloodOxStruct.selectedAllDensity = selectedAllDensity;
% % % % % % % barrelVersusBloodOxStruct.insideVertDensity = insideVertDensity;
% % % % % % % barrelVersusBloodOxStruct.edgeVertDensity = edgeVertDensity;
% % % % % % % barrelVersusBloodOxStruct.outsideVertDensity = outsideVertDensity;
% % % % % % % barrelVersusBloodOxStruct.numVertsInside = numVertsInside;
% % % % % % % barrelVersusBloodOxStruct.numVertsEdge = numVertsEdge;
% % % % % % % barrelVersusBloodOxStruct.numVertsOutside = numVertsOutside;
% % % % % % % 
% % % % % % % 
% % % % % % % 
% % % % % % % vectorProject.barrelVersusBloodOxStruct = barrelVersusBloodOxStruct;
% % % % % % % vectorProject.barrelVersusBloodOxParameters.lastRunDate = datestr(now);
% % % % % % % 
