% function vectorProject = CalculateBloodOxDropVersusDistanceToArterioleFunction(vectorProject)
%%
vectorizedStructure = vectorProject.vectorizedStructure;
vectorProjectRootName = vectorProject.vectorProjectRootName;
dashPos = findstr(vectorProjectRootName,'-');
vectorProjectShortName = vectorProjectRootName(1:dashPos-1);
maskSize = vectorizedStructure.maskSize;
strands = vectorizedStructure.Strands;
vertices = vectorizedStructure.Vertices;
strandIndices = vertices.StrandIndices;
numVerts = numel(vertices.AllRadii);
numStrands = numel(strands);
allVerts = vertices.AllVerts;
barrelValueSubMask = vectorProject.barrelValueSubMask;
subMaskSize = size(barrelValueSubMask);
divRatio = vectorProject.barrelBoundaryParameters.divRatio;
vertBarrelValueList = vectorProject.vertBarrelValueList;
vertNormMeanBloodOxList = vectorProject.vectorizedStructure.vertNormMeanBloodOxList;
vertMeanBloodOxList = vectorProject.vectorizedStructure.vertMeanBloodOxList;
strandMidYXZList = vectorizedStructure.strandMidYXZList;
strandMeanBloodOxList = vectorizedStructure.strandMeanBloodOxList;

barrelVersusBloodOxParameters = vectorProject.barrelVersusBloodOxParameters;
bufferZone = barrelVersusBloodOxParameters.bufferZone;
bloodOxCutoffList = barrelVersusBloodOxParameters.bloodOxCutoffList;




%%%Stage 2: Find the offshoots of plunging arterioles
flowProject.maskSize = vectorProject.vectorizedStructure.maskSize;
flowProject.Strands = vectorProject.vectorizedStructure.Strands;
flowProject.Vertices = vectorProject.vectorizedStructure.Vertices;
startList = vectorProject.vectorizedStructure.allPlungingArterioleList;
flowProject.baseStrandIDList = startList;
flowProject.maxBranchSearch = 3;
flowProject.stopVoltage = 50.1;%Hardcode;
flowProject.minimumFluxFraction = 1/(2^18);%Hardcode

disp('Calculating downstream paths...');
flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
arterioleOffshootList = find(min(flowProject.downstreamBranchList,[],2)==1);
numArterioleOffshoots = numel(arterioleOffshootList);


%---

flowProject.maskSize = vectorProject.vectorizedStructure.maskSize;
flowProject.Strands = vectorProject.vectorizedStructure.Strands;
flowProject.Vertices = vectorProject.vectorizedStructure.Vertices;
startList = vectorProject.vectorizedStructure.allPlungingVenuleList;
flowProject.baseStrandIDList = startList;
flowProject.maxBranchSearch = 3;
flowProject.stopVoltage = 99;%Hardcode;
flowProject.minimumFluxFraction = 1/(2^18);%Hardcode

disp('Calculating upstream paths...');
flowProject = CalculateUpstreamPathsFromVectorProject(flowProject);
venuleOffshootList = find(min(flowProject.upstreamBranchList,[],2)==1);
numVenuleOffshoots = numel(venuleOffshootList);
%---
%%

baseList = vectorizedStructure.allPlungingArterioleList;
% baseList = vectorizedStructure.allPlungingVenuleList;
% baseList = arterioleOffshootList;
% baseList = venuleOffshootList;
% % allPath = vectorizedStructure.allPlungingVenulePathList;



baseZList = strandMidYXZList(baseList,3);


z1 = 50;
z2 = 350;

f1 = find(and(baseZList>z1,baseZList<z2));
baseStrandList = baseList(f1);
subStrandList = find(and(strandMidYXZList(:,3)>z1,strandMidYXZList(:,3)<z2));
subVertList = find(and(allVerts(:,3)>z1,allVerts(:,3)<z2));
subVertCoordList = allVerts(subVertList,:);
subOxList = vertMeanBloodOxList(subVertList);
baseIter = 22;

    thisBaseStrandID = baseStrandList(baseIter);
    baseVerts = allVerts(strands(thisBaseStrandID).InteriorVertices,:);
    midZ = ceil(median(baseVerts(:,3)));
    fz = find(baseVerts(:,3) == midZ);
    thisBaseStrandY = baseVerts(fz(1),1);
    thisBaseStrandX = baseVerts(fz(1),2);

    baseSep =  sqrt((allBaseStrandY-thisBaseStrandY).^2 + (allBaseStrandX-thisBaseStrandX).^2);
    sortBaseSep = sort(baseSep,'ascend');
    closestBaseSep = sortBaseSep(2);
    
    sampleRate = 21
    halfSample = ceil(sampleRate/2);
    littleFrame = ones(sampleRate,sampleRate);
    [myY,myX] = ind2sub(size(littleFrame),find(littleFrame));
    myY = (myY-halfSample)*10;
    myX = (myX-halfSample)*10;
    distList = sqrt(myY.^2 + myX.^2);
    
    yCoordList = thisBaseStrandY + myY;
    xCoordList = thisBaseStrandX + myX;
    tissueOxList = [];
    for coordIter = 1:numel(yCoordList),
        thisY = yCoordList(coordIter);
        thisX = xCoordList(coordIter);
        vertSep = sqrt((subVertCoordList(:,1)-thisY).^2 + (subVertCoordList(:,2)-thisX).^2 + (subVertCoordList(:,3)-midZ).^2);
        closeVertIndList = find(vertSep<100);
        if isempty(closeVertIndList),
            tissueOxList(coordIter) = 0;
        else%if isempty
            closeSep = vertSep(closeVertIndList);
            closeOx = subOxList(closeVertIndList);
   tissueAlpha = 2e-2;
%             tissueOx = closeOx .* exp(-tissueAlpha * closeSep.^2);
            tissueOx = closeOx .* exp(-tissueAlpha * closeSep);
            maxTissueOx = max(tissueOx);
            tissueOxList(coordIter) = maxTissueOx;
        end% if isempty...
    end
    figure(1);plot(distList,tissueOxList,'r.')
    tt = reshape(tissueOxList,[sampleRate,sampleRate]);
    figure(2);imagesc(tt);
        
%%
        
    for jj = 1:numel(allPath),
        thisPath = allPath{jj};
        if ismember(thisBaseStrandID,thisPath),
            myPath = thisPath;
        end%if ismember
    end%for jj
    otherBaseList = allPlungingArterioleList(~ismember(allPlungingArterioleList,myPath));
   
    
    allBaseStrandY = strandMidYXZList(otherBaseList,1);
    allBaseStrandX = strandMidYXZList(otherBaseList,2);
    baseSep =  sqrt((allBaseStrandY-thisBaseStrandY).^2 + (allBaseStrandX-thisBaseStrandX).^2);
    sortBaseSep = sort(baseSep,'ascend');
    closestBaseSep = sortBaseSep(2);
   
    
    if closestBaseSep>100,


        subStrandY = strandMidYXZList(subStrandList,1);
        subStrandX = strandMidYXZList(subStrandList,2);
        subStrandBloodOxList = strandMeanBloodOxList(subStrandList);
        subVertY = allVerts(subVertList,1);
        subVertX = allVerts(subVertList,2);
        subVertDistList = sqrt((subVertY-thisBaseStrandY).^2 + (subVertX-thisBaseStrandX).^2);
        subVertOxList = strandMeanBloodOxList(strandIndices(subVertList));

        subStrandDistList = sqrt((subStrandY-thisBaseStrandY).^2 + (subStrandX-thisBaseStrandX).^2);
        allDistList = [allDistList;subStrandDistList];
        allOxList = [allOxList;subStrandBloodOxList];



        figure(1);p1 = plot(subStrandDistList,subStrandBloodOxList,'r.');
        figure(2);p2 = plot(allDistList,allOxList,'g.');set(gca,'XLim',[0 400]);
%         disp([thisBaseStrandY,thisBaseStrandX]);
        figure(3);p3 = plot(subVertDistList,subVertOxList,'b.');

        pause(0.01);  %pause
    else
        disp('hi')
    
    end %if closestBaseSep,
    
end%for kk

filtSize = 50;
allData = [allDistList,allOxList];
sortList = sortrows(allData,1);
filtList = imfilter(sortList,ones(filtSize,1),'symmetric')./filtSize;
figure(3);p3 = plot(filtList(:,1),filtList(:,2),'m.');set(gca,'XLim',[0 400]);



