function vectorProject =  CalculateMeanBloodOxForAllVesselsFunction(vectorProject);
%Calculate Tissue Oxygenation based on Time of Flight of blood

%Model:
%time per (um) pixel  =  r^2 / strandCurrent
%assume blood oxygenation drops off linearly with time
%   this assumes an exposure to a roughly constant gradient of oxygenation
%amount of oxygen delivered to tissue by a pixel of vessel is then
%   the difference between the blood oxygenation of that pixel and its
%   immediately upstream vessel pixel.
%assume oxygenation from a point drops off 1/r2 from source pixel (weak tissue consumption)
%   or add an exponential drop off with r from source pixel (added tissue consumption)


%Method:
%Start from artery tips (near surface).  Assume some surface oxygenation.
%For each strand, calculate the time per pixel (r2/I)
%Assign a time_of_flight to each vertex
%Check off that strand as calculated
%Pick a downstream strand - continue until no downstream strands
%If there was only one choice of downstream branch, 


%Instructions:
%Load an FV file with artery tips labeled.

% wacko

strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
strandLengthList = vectorProject.vectorizedStructure.strandLengthList;
baseStrandIDList = vectorProject.vectorizedStructure.plungingArterioleTipList;
effectiveStrandRadiusList = vectorProject.vectorizedStructure.effectiveStrandRadiusList;
bloodOxParameters = vectorProject.bloodOxParameters;




numBaseStrands = numel(baseStrandIDList);
numStrands = numel(strands);
numVertices = numel(vertices.AllRadii);
tofStruct = []; %timeOfFlight dataStruct
tofStruct(numStrands).currentList = [];
tofStruct(numStrands).startTimeList = [];


maxBranchSearch = bloodOxParameters.maxBranchSearch;
stopVoltage = bloodOxParameters.stopVoltage;
minimumFluxFraction = bloodOxParameters.minimumFluxFraction;

downstreamPathList = [];
downstreamBranchList = zeros(numStrands,numel(baseStrandIDList)) + Inf;
transitTimeList = zeros(numStrands,1);

for baseStrandIter = 1:numBaseStrands;
    endTimeList = zeros(numStrands,1);
    baseStrandID = baseStrandIDList(baseStrandIter);
    baseStrand = strands(baseStrandID);
    bsiv = baseStrand.InteriorVertices;
    baseStrandCoordList = vertices.AllVerts(bsiv,:);
    strandNodes = [baseStrand.StartVertexIndex,baseStrand.EndVertexIndex];
    nodeVoltages = vertices.AllVoltages(strandNodes);
    [highVoltage,highVoltagePos] = max(nodeVoltages);
    [lowVoltage,lowVoltagePos] = min(nodeVoltages);
    highNode = strandNodes(highVoltagePos);
    lowNode = strandNodes(strandNodes~=highNode);

    pathActiveFlagList = false(1,100000);
    pathActiveFlagList(1) = 1;
    numActivePath = sum(pathActiveFlagList);
    strandPathList = {};
    nodePathList = {};
    fractionPathList = {};
    strandPathList{1} = baseStrandID;
    nodePathList{1} = [highNode lowNode];
    fractionPathList{1} = 1;
    strandFlowList = zeros(numStrands,1);
    strandFlowList(baseStrandID) = 1;
    strandVoltageList = zeros(numStrands,1);
    strandVoltageList(1) = mean([highVoltage,lowVoltage]);
    pathCounter = 1;
    keepGoingFlag = 1;
    branchCounter = 0;
    %---
    baseStrandCurrent = abs(baseStrand.strandCurrent);
    strandRadius = effectiveStrandRadiusList(baseStrandID);
    tofStruct(baseStrandID).currentList = baseStrandCurrent;
    tofStruct(baseStrandID).startTimeList = 0;
    strandLength = strandLengthList(baseStrandID);
    timeOfFlight = strandLength*(strandRadius.^2/baseStrandCurrent);
% % timeOfFlight = strandLength/baseStrandCurrent;


    transitTimeList(baseStrandID) = timeOfFlight;
    endTimeList(baseStrandID) = timeOfFlight;

    while (keepGoingFlag == 1),
        myCount = 0;
        branchCounter = branchCounter+1;
        for pathIter = find(pathActiveFlagList == 1);
            currentPathID = pathIter;
            currentStrandPath = strandPathList{pathIter};
            currentNodePath = nodePathList{pathIter};
            currentFractionPath = fractionPathList{pathIter};
            currentStrandID = currentStrandPath(end);
            currentStrand = strands(currentStrandID);
            %-----
            strandNodes = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
            nodeVoltages = vertices.AllVoltages(strandNodes);
            [highVoltage,highVoltagePos] = max(nodeVoltages);
            [lowVoltage,lowVoltagePos] = min(nodeVoltages);
            highNode = strandNodes(highVoltagePos);
            lowNode = strandNodes(strandNodes~=highNode);
            lowAttachedStrands = vertices.StrandIndices(lowNode,:);
            lowAttachedStrands = lowAttachedStrands(lowAttachedStrands~=0);
            lowAttachedStrands = lowAttachedStrands(lowAttachedStrands~=currentStrandID);
            numDaughterPaths = numel(lowAttachedStrands);
            strandVoltageList(currentStrandID) = mean([highVoltage,lowVoltage]);
            stopTracingFlag = 0;
            if numDaughterPaths == 0, stopTracingFlag = 1; end
            if lowVoltage < stopVoltage, stopTracingFlag = 1; end
            if currentFractionPath(end) < minimumFluxFraction, stopTracingFlag = 1; end
            if stopTracingFlag == 1,
                pathActiveFlagList(currentPathID) = 0;
            else%if stopTracingFlag == 1,
                motherCurrent = abs(currentStrand.strandCurrent);
                validDaughterCounter = 0;
                validDaughterStrandList = [];
                validDaughterCurrentList = zeros(1,numDaughterPaths);
                validDaughterPathIndexList = [];
                for daughterIter = 1:numDaughterPaths,
                    %Check to see if the daughter strand leads to a lower voltage
                    candidateDaughterID = lowAttachedStrands(daughterIter);
                    candidateDaughterStrand = strands(candidateDaughterID);
                    candidateDaughterNodes = [candidateDaughterStrand.StartVertexIndex,candidateDaughterStrand.EndVertexIndex];
                    candidateDaughterOtherNode = candidateDaughterNodes(candidateDaughterNodes~=lowNode);
                    candidateDaughterOtherVoltage = vertices.AllVoltages(candidateDaughterOtherNode);
                    if candidateDaughterOtherVoltage < lowVoltage,
                        %if so, add the strand to a path
                        myCount = myCount + 1;
                        validDaughterCounter = validDaughterCounter + 1;
                        validDaughterStrandList(validDaughterCounter) = candidateDaughterID;
                        validDaughterCurrentList(validDaughterCounter) = abs(candidateDaughterStrand.strandCurrent);
                        if validDaughterCounter == 1,
                            %if this is the first valid daughter, add to current path
                            strandPathList{currentPathID} = [currentStrandPath, lowAttachedStrands(daughterIter)];
                            nodePathList{currentPathID} = [currentNodePath, candidateDaughterOtherNode];
                            validDaughterPathIndexList(1) = currentPathID;
                        else%if validDaughterCounter == 1,
                            %if not the first daughter, duplicate path up to this point, and
                            %make new path
                            pathCounter = pathCounter + 1;
                            strandPathList{pathCounter} = [currentStrandPath, lowAttachedStrands(daughterIter)];
                            nodePathList{pathCounter} = [currentNodePath, candidateDaughterOtherNode];
                            fractionPathList{pathCounter} = currentFractionPath;
                            pathActiveFlagList(pathCounter) = 1;  
                            validDaughterPathIndexList(validDaughterCounter) = pathCounter;
                        end%if-else validDaughterCounter == 1,
                    end%if candidateDaughterOtherVoltage < lowVoltage,
                end%for daughterIter = 1:numDaughterPaths,
                clear daughterIter
                %-----
                daughterSumCurrent = sum(validDaughterCurrentList);
                fractionalDaughterCurrent = validDaughterCurrentList./daughterSumCurrent;
                for validDaughterIter = 1:validDaughterCounter;
                    validDaughterSegmentID = validDaughterStrandList(validDaughterIter);
                    validDaughterFraction = currentFractionPath(end).*fractionalDaughterCurrent(validDaughterIter);
                    fractionPathList{validDaughterPathIndexList(validDaughterIter)} = [currentFractionPath , validDaughterFraction];
                    strandFlowList(validDaughterSegmentID) = strandFlowList(validDaughterSegmentID) + validDaughterFraction;
                    
                    
                    strandRadius = effectiveStrandRadiusList(validDaughterSegmentID);
                    currentList = [tofStruct(validDaughterSegmentID).currentList, baseStrandCurrent .* validDaughterFraction];
                    tofStruct(validDaughterSegmentID).currentList = currentList;
                    thisDaughterCurrent = abs(strands(validDaughterSegmentID).strandCurrent);
                    thisMotherEndTime = endTimeList(currentStrandID);
                    strandLength = strandLengthList(validDaughterSegmentID);
                    timeOfFlight = strandLength*(strandRadius.^2)/thisDaughterCurrent;

                    
                    velocityMultiplier = 1e-6;
                    assumedDensity = 1e-2; %rbc per um3 (assuming 1x7x7 cell, and 50% hematocrit)
                    %strand velocity in um/sec (expect ~500 - 1000um/s for small vessels)
                    strandVelocity = velocityMultiplier * thisDaughterCurrent./((pi*strandRadius^2)*assumedDensity);
                    timeOfFlight = strandLength./strandVelocity;
                    strandVelocityList(validDaughterSegmentID) = strandVelocity;

                                        
                    thisDaughterEndTime = thisMotherEndTime + timeOfFlight;
                    transitTimeList(validDaughterSegmentID) = timeOfFlight;
                    startTimeList = [tofStruct(validDaughterSegmentID).startTimeList, thisMotherEndTime];
                    tofStruct(validDaughterSegmentID).startTimeList = startTimeList;
                    endTimeList(validDaughterSegmentID) = thisDaughterEndTime;
                    

                    
                end%for validDaughterIter = 1:validDaughterCounter;
            end%if-else stopTracingFlag == 1,
        end%for pathIter = find(pathActiveFlagList == 1);
        if sum(pathActiveFlagList) == 0, keepGoingFlag = 0;end
        if myCount == 0, keepGoingFlag = 0;end
        if branchCounter >= maxBranchSearch, keepGoingFlag = 0; end
    end%while (keepGoingFlag == 1),
    downstreamPathList{baseStrandIter} = strandPathList;
    disp(baseStrandIter)
end%for baseStrandIter = 1:numBaseStrands;

for baseStrandIter = 1:numBaseStrands;
    thisPathSet = downstreamPathList{baseStrandIter};
    for pathIter = 1:numel(thisPathSet),
        thisPath = thisPathSet{pathIter};
        thisPath = thisPath(2:end);
        posList = [1:numel(thisPath)]';
        downstreamBranchList(thisPath,baseStrandIter) = min(downstreamBranchList(thisPath,baseStrandIter),posList);
    end%for pathIter = 1:numel(thisPathSet);
end%for baseStrandIter = ...

figure(41);plot(strandVelocityList,'g.');
xlabel('Strand ID #');ylabel('estimated RBC velocity [um/s]');


%%


alpha = bloodOxParameters.decayAlpha;
meanBloodOx = zeros(1,numStrands);
for kk = 1:numStrands,
    startTimeList = tofStruct(kk).startTimeList;
    currentList = tofStruct(kk).currentList;
% %     if isempty(startTimeList),startTimeList = 0;end
% %     q(kk) = min(startTimeList);
    fractionCurrent(kk) = abs(sum(currentList)./strands(kk).strandCurrent);
    bloodOx = exp(-alpha*startTimeList);
    normCurrentList = currentList./sum(currentList);
    meanBloodOx(kk) = sum(bloodOx.*normCurrentList);
    strands(kk).meanBloodOx = meanBloodOx(kk);
    meanYXZList(kk,1) = mean(strands(kk).YXZLimits(1,:));
    meanYXZList(kk,2) = mean(strands(kk).YXZLimits(2,:));
    meanYXZList(kk,3) = mean(strands(kk).YXZLimits(3,:));    
    meanVoltage = strands(kk).MedianVoltage;
    meanZ(kk) = meanYXZList(kk,3);
end


meanVoltage = [strands.MedianVoltage];
figure(1);plot(meanVoltage,meanBloodOx,'r.');
xlabel('mean Voltage');ylabel('meanBloodOx');
figure(2);plot(meanZ,meanBloodOx,'r.');
xlabel('mean Z depth');ylabel('meanBloodOx');

strandIndices = vertices.StrandIndices;
allPlungingArterioleList = vectorProject.vectorizedStructure.allPlungingArterioleList;
allPlungingVenuleList = vectorProject.vectorizedStructure.allPlungingVenuleList;
arterioleMeanBloodOx = mean(meanBloodOx(allPlungingArterioleList));
venuleMeanBloodOx = mean(meanBloodOx(allPlungingVenuleList));

vertNormMeanBloodOxList = zeros(size(vertices.AllRadii));
vertMeanBloodOxList = zeros(size(vertices.AllRadii));
normMeanBloodOxList = (meanBloodOx-venuleMeanBloodOx)./(arterioleMeanBloodOx-venuleMeanBloodOx);
numVerts = numel(vertices.AllRadii);
for vertIter = 1:numVerts,
    vertNormMeanBloodOxList(vertIter) = normMeanBloodOxList(strandIndices(vertIter,1));
    vertMeanBloodOxList(vertIter) = meanBloodOx(strandIndices(vertIter,1));
end

vectorProject.vectorizedStructure.strandMeanBloodOxList = meanBloodOx';
vectorProject.vectorizedStructure.strandNormMeanBloodOxList = normMeanBloodOxList';
vectorProject.vectorizedStructure.vertMeanBloodOxList = vertMeanBloodOxList;
vectorProject.vectorizedStructure.vertNormMeanBloodOxList = vertNormMeanBloodOxList;
vectorProject.bloodOxParameters.lastRunDate = datestr(now);

vectorProject.vectorizedStructure.Strands = strands;





