%Stage 1: Load a -flow-vectorProject that has a blood ox calculated for each strand
%   (for example, using CalculateMeanTimeOfFlightForAllVessels.m);

strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;

%%

%Try again, but looking at strand centers
yxOffset = 200;
y1 = yxOffset; y2 = maskSize(1)-yxOffset;
x1 = yxOffset; x2 = maskSize(2)-yxOffset;
z1 = 200; z2 = 900;

threshRadius = 75;
% % % 
% % % yRandList = ceil((y2-y1)*rand(numRand,1)+y1);
% % % xRandList = ceil((x2-x1)*rand(numRand,1)+x1);
% % % zRandList = ceil((z2-z1)*rand(numRand,1)+z1);

allVerts = vertices.AllVerts;
strandMidYXZList = vectorProject.vectorizedStructure.strandMidYXZList;
numStrands = size(strandMidYXZList,1);
numPoints = size(allVerts,1);
strandVoltageList = [strands(:).MedianVoltage]';
strandLengthList = vectorProject.vectorizedStructure.strandLengthList;
strandBloodOxList = [strands(:).meanBloodOx]';
strandDensityList = zeros(numStrands,1);s
fieldBloodOxList = zeros(numStrands,1);
fieldVoltageList = zeros(numStrands,1);
for strandIter = 1:1:numStrands;
    if and(strandMidYXZList(strandIter,1) > y1, strandMidYXZList(strandIter,1) <y2),
        if and(strandMidYXZList(strandIter,2) > x1, strandMidYXZList(strandIter,2) <x2),
            if and(strandMidYXZList(strandIter,3) > z1, strandMidYXZList(strandIter,3) <z2),


                pointSepList = allVerts - repmat(strandMidYXZList(strandIter,:),[numPoints,1]);
                pointDistList = sqrt(sum(pointSepList.^2,2));
                ff = find(pointDistList < threshRadius);
                strandContributorList = vertices.StrandIndices(ff,1);
                bloodOxList = strandBloodOxList(strandContributorList);
                voltageList = strandVoltageList(strandContributorList);
                subBloodOxList = bloodOxList(bloodOxList>0);
                subVoltageList = voltageList;
                fieldBloodOxList(strandIter) = mean(subBloodOxList);
                fieldVoltageList(strandIter) = mean(subVoltageList);
                strandDensityList(strandIter) = numel(ff) / (1.33*pi*threshRadius^3);
                if mod(strandIter,10) == 0,
                    figure(9);plot(strandDensityList,fieldBloodOxList,'r.');
                    xlabel('Strand Length Density'); ylabel('Local Field Blood Ox');
                    figure(10);plot(strandDensityList,fieldVoltageList,'b.');
                    xlabel('Strand Length Density'); ylabel('Local Field Circuit Voltage');
                    pause(0.1)
                end%if mod...
                
                
            end%if and..
        end%if and..
    end%if and..
end
disp('Completed field Blood ox calculation');


%%
strandDensityList2 = strandDensityList(strandDensityList>0);
fieldBloodOxList2 = fieldBloodOxList(strandDensityList>0);
fieldVoltageList2 = fieldVoltageList(strandDensityList>0);

allDataList = [strandDensityList2,fieldBloodOxList2,fieldVoltageList2];

sortDataList = sortrows(allDataList,1);
filtSize = 50;
filtDataList = imfilter(sortDataList,ones(filtSize,1),'replicate')./filtSize;

figure(9);p9 = plot(strandDensityList2,fieldBloodOxList2,'r.');hold on;
p9b = plot(filtDataList(:,1),filtDataList(:,2),'k-');hold off;
xlabel('Strand Length Density'); ylabel('Local Field Blood Ox');
set(p9,'MarkerSize',1);

figure(10);p10 = plot(strandDensityList2,fieldVoltageList2,'b.');hold on;
p10b = plot(filtDataList(:,1),filtDataList(:,3),'k-');hold off;
xlabel('Strand Length Density'); ylabel('Local Field Circuit Voltage');
set(p10,'MarkerSize',1);

%%
%Plot the veins as blue circles
aList = vectorProject.vectorizedStructure.allPlungingArterioleList;
vList = vectorProject.vectorizedStructure.allPlungingVenuleList;

subSampleRate = 3;
allVerts = vertices.AllVerts;
aIndices = [strands(aList).StartToEndIndices];
vIndices = [strands(vList).StartToEndIndices];
aPoints = allVerts(aIndices(1:subSampleRate:end),:);
vPoints = allVerts(vIndices(1:subSampleRate:end),:);

f21 = figure(21);hold off;
set(f21,'DoubleBuffer','on');
pa = plot3(aPoints(:,1),aPoints(:,2),-1 * aPoints(:,3),'r.');hold on
set(pa,'MarkerSize',2);
pv = plot3(vPoints(:,1),vPoints(:,2),-1 * vPoints(:,3),'b.');
set(pv,'MarkerSize',2);
pause
myCamva = camva;
myCampos = campos;
myCamup = camup;
myCamtarget = camtarget;

for threshBloodOx = 1 : -0.01 : 0 
    figure(21);
    meanBloodOxList = [strands(:).meanBloodOx];
    bStrands = find(meanBloodOxList > threshBloodOx);
    bIndices = [strands(bStrands).StartToEndIndices];
    bPoints = allVerts(bIndices,:);
    pb = plot3(bPoints(:,1),bPoints(:,2),-1 * bPoints(:,3),'g.');
    set(pb,'MarkerSize',1);
    camva(myCamva);campos(myCampos);camup(myCamup);camtarget(myCamtarget);
    pause
    myCamva = camva;myCampos = campos;myCamup = camup;myCamtarget = camtarget;

    
end

%%
aList = vectorProject.vectorizedStructure.allPlungingArterioleList;
vList = vectorProject.vectorizedStructure.allPlungingVenuleList;

subSampleRate = 3;
allVerts = vertices.AllVerts;
aIndices = [strands(aList).StartToEndIndices];
vIndices = [strands(vList).StartToEndIndices];
aPoints = allVerts(aIndices(1:subSampleRate:end),:);
vPoints = allVerts(vIndices(1:subSampleRate:end),:);

[avIndices] = unique(cat(2,aIndices,vIndices));
zaIndices = find(and(allVerts(:,3)>0,allVerts(:,3)<200));
zbIndices = find(and(allVerts(:,3)>200,allVerts(:,3)<400));
zcIndices = find(and(allVerts(:,3)>400,allVerts(:,3)<600));
zdIndices = find(and(allVerts(:,3)>600,allVerts(:,3)<800));
zeIndices = find(and(allVerts(:,3)>800,allVerts(:,3)<1000));
zfIndices = find(and(allVerts(:,3)>1000,allVerts(:,3)<1600));
origAVSum = numel(avIndices);
origASum = numel(aIndices);
origVSum = numel(vIndices);
origZASum = numel(zaIndices);
origZBSum = numel(zbIndices);
origZCSum = numel(zcIndices);
origZDSum = numel(zdIndices);
origZESum = numel(zeIndices);
origZFSum = numel(zfIndices);

meanBloodOxList = [strands(:).meanBloodOx];
bloodOxThreshList = [1:-0.01:0];
currentAVFraction = [];
currentAFraction = [];
currentVFraction = [];
currentZAFraction = [];
currentZBFraction = [];
currentZCFraction = [];
currentZDFraction = [];
currentZEFraction = [];
currentZFFraction = [];

%Graph what percentage of each layer (as well as the plunging vessel subsets) 
%   are at a blood ox of x% or higher
for threshIter = 1:numel(bloodOxThreshList);
    threshBloodOx = bloodOxThreshList(threshIter);
    bStrands = find(meanBloodOxList > threshBloodOx);
    bIndices = [strands(bStrands).StartToEndIndices];
    aIndices =  aIndices(~ismember(aIndices,bIndices));
    vIndices =  vIndices(~ismember(vIndices,bIndices));
    avIndices =  avIndices(~ismember(avIndices,bIndices));
    zaIndices =  zaIndices(~ismember(zaIndices,bIndices));
    zbIndices =  zbIndices(~ismember(zbIndices,bIndices));
    zcIndices =  zcIndices(~ismember(zcIndices,bIndices));
    zdIndices =  zdIndices(~ismember(zdIndices,bIndices));
    zeIndices =  zeIndices(~ismember(zeIndices,bIndices));
    zfIndices =  zfIndices(~ismember(zfIndices,bIndices));
    currentAVFraction(threshIter) = numel(avIndices)./origAVSum;
    currentAFraction(threshIter) = numel(aIndices)./origASum;
    currentVFraction(threshIter) = numel(vIndices)./origVSum;
    currentZAFraction(threshIter) = numel(zaIndices)./origZASum;
    currentZBFraction(threshIter) = numel(zbIndices)./origZBSum;
    currentZCFraction(threshIter) = numel(zcIndices)./origZCSum;
    currentZDFraction(threshIter) = numel(zdIndices)./origZDSum;
    currentZEFraction(threshIter) = numel(zeIndices)./origZESum;
    currentZFFraction(threshIter) = numel(zfIndices)./origZFSum;
% %     figure(31);plot(currentAVFraction,'k-');hold on
    figure(31);plot(currentAFraction,'r-');hold on
    figure(31);plot(currentVFraction,'b-');
    figure(31);plot(currentZAFraction,'g-');
    figure(31);plot(currentZBFraction,'m-');
    figure(31);plot(currentZCFraction,'c-');
    figure(31);plot(currentZDFraction,'y-');
    figure(31);plot(currentZEFraction,'k--');
    figure(31);plot(currentZFFraction,'g--');hold off
    xlabel('%Percent deoxy (crudely)');
    ylabel('fraction of vessel subclass at this (or higher) blood ox');
    
    
    aPoints = allVerts(aIndices,:);
    f32 = figure(32);hold off;
    set(f32,'DoubleBuffer','on');
    pa = plot3(aPoints(:,1),aPoints(:,2),-1 * aPoints(:,3),'r.');hold on
    vPoints = allVerts(vIndices,:);
    pv = plot3(vPoints(:,1),vPoints(:,2),-1 * vPoints(:,3),'b.');hold off
    camva(myCamva);campos(myCampos);camup(myCamup);camtarget(myCamtarget);
    pause
    myCamva = camva;myCampos = campos;myCamup = camup;myCamtarget = camtarget;
    
    
end
    









