%Calculate Tissue Oxygenation based on Time of Flight of blood

%Model:
%time per (um) pixel  =  r^2 / strandCurrent
%assume blood oxygenation drops off linearly with time
%   this assumes an exposure to a roughly constant gradient of oxygenation
%amount of oxygen delivered to tissue by a pixel of vessel is then
%   the difference between the blood oxygenation of that pixel and its
%   immediately upstream vessel pixel.
%assume oxygenation from a point drops off 1/r2 from source pixel (weak tissue consumption)
%   or add an exponential drop off with r from source pixel (added tissue consumption)


%Method:
%Start from artery tips (near surface).  Assume some surface oxygenation.
%For each strand, calculate the time per pixel (r2/I)
%Assign a time_of_flight to each vertex
%Check off that strand as calculated
%Pick a downstream strand - continue until no downstream strands
%If there was only one choice of downstream branch, 


%Instructions:
%Load an FV file with artery tips labeled.

% wacko

Strands = gds.Strands;
Vertices = gds.Vertices;
strandLengthList = gds.strandLengthList;
baseStrandIDList = gds.plungingArterioleTipList;



baseStrandIDList = [baseStrandIDList,820];
baseStrandIDList = [baseStrandIDList];




numBaseStrands = numel(baseStrandIDList);
numStrands = numel(Strands);
numVertices = numel(Vertices.AllRadii);
tofStruct = []; %timeOfFlight dataStruct
tofStruct(numStrands).currentList = [];
tofStruct(numStrands).startTimeList = [];


maxBranchSearch = 50;%Hardcode
stopVoltage = 50.1;%Hardcode;
minimumFluxFraction = 1/30000;%Hardcode

downstreamPathList = [];
downstreamBranchList = zeros(numStrands,numel(baseStrandIDList)) + Inf;
transitTimeList = zeros(numStrands,1);

for baseStrandIter = 1:numBaseStrands;
    endTimeList = zeros(numStrands,1);
    baseStrandID = baseStrandIDList(baseStrandIter);
    baseStrand = Strands(baseStrandID);
    bsiv = baseStrand.InteriorVertices;
    baseStrandCoordList = Vertices.AllVerts(bsiv,:);
    strandNodes = [baseStrand.StartVertexIndex,baseStrand.EndVertexIndex];
    nodeVoltages = Vertices.AllVoltages(strandNodes);
    [highVoltage,highVoltagePos] = max(nodeVoltages);
    [lowVoltage,lowVoltagePos] = min(nodeVoltages);
    highNode = strandNodes(highVoltagePos);
    lowNode = strandNodes(strandNodes~=highNode);

    pathActiveFlagList = false(1,100000);
    pathActiveFlagList(1) = 1;
    numActivePath = sum(pathActiveFlagList);
    strandPathList = {};
    nodePathList = {};
    fractionPathList = {};
    strandPathList{1} = baseStrandID;
    nodePathList{1} = [highNode lowNode];
    fractionPathList{1} = 1;
    strandFlowList = zeros(numStrands,1);
    strandFlowList(baseStrandID) = 1;
    strandVoltageList = zeros(numStrands,1);
    strandVoltageList(1) = mean([highVoltage,lowVoltage]);
    pathCounter = 1;
    keepGoingFlag = 1;
    branchCounter = 0;
    %---
    baseStrandCurrent = abs(baseStrand.strandCurrent);
    strandRadius = baseStrand.MedianRadius;
    tofStruct(baseStrandID).currentList = baseStrandCurrent;
    tofStruct(baseStrandID).startTimeList = 0;
    strandLength = strandLengthList(baseStrandID);
% %     timeOfFlight = strandLength*(strandRadius.^2/baseStrandCurrent);
timeOfFlight = strandLength/baseStrandCurrent;


    transitTimeList(baseStrandID) = timeOfFlight;
    endTimeList(baseStrandID) = timeOfFlight;

    while (keepGoingFlag == 1),
        myCount = 0;
        branchCounter = branchCounter+1;
        for pathIter = find(pathActiveFlagList == 1);
            currentPathID = pathIter;
            currentStrandPath = strandPathList{pathIter};
            currentNodePath = nodePathList{pathIter};
            currentFractionPath = fractionPathList{pathIter};
            currentStrandID = currentStrandPath(end);
            currentStrand = Strands(currentStrandID);
            %-----
            strandNodes = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
            nodeVoltages = Vertices.AllVoltages(strandNodes);
            [highVoltage,highVoltagePos] = max(nodeVoltages);
            [lowVoltage,lowVoltagePos] = min(nodeVoltages);
            highNode = strandNodes(highVoltagePos);
            lowNode = strandNodes(strandNodes~=highNode);
            lowAttachedStrands = Vertices.StrandIndices(lowNode,:);
            lowAttachedStrands = lowAttachedStrands(lowAttachedStrands~=0);
            lowAttachedStrands = lowAttachedStrands(lowAttachedStrands~=currentStrandID);
            numDaughterPaths = numel(lowAttachedStrands);
            strandVoltageList(currentStrandID) = mean([highVoltage,lowVoltage]);
            stopTracingFlag = 0;
            if numDaughterPaths == 0, stopTracingFlag = 1; end
            if lowVoltage < stopVoltage, stopTracingFlag = 1; end
            if currentFractionPath(end) < minimumFluxFraction, stopTracingFlag = 1; end
            if stopTracingFlag == 1,
                pathActiveFlagList(currentPathID) = 0;
            else%if stopTracingFlag == 1,
                motherCurrent = abs(currentStrand.strandCurrent);
                validDaughterCounter = 0;
                validDaughterStrandList = [];
                validDaughterCurrentList = zeros(1,numDaughterPaths);
                validDaughterPathIndexList = [];
                for daughterIter = 1:numDaughterPaths,
                    %Check to see if the daughter strand leads to a lower voltage
                    candidateDaughterID = lowAttachedStrands(daughterIter);
                    candidateDaughterStrand = Strands(candidateDaughterID);
                    candidateDaughterNodes = [candidateDaughterStrand.StartVertexIndex,candidateDaughterStrand.EndVertexIndex];
                    candidateDaughterOtherNode = candidateDaughterNodes(candidateDaughterNodes~=lowNode);
                    candidateDaughterOtherVoltage = Vertices.AllVoltages(candidateDaughterOtherNode);
                    if candidateDaughterOtherVoltage < lowVoltage,
                        %if so, add the strand to a path
                        myCount = myCount + 1;
                        validDaughterCounter = validDaughterCounter + 1;
                        validDaughterStrandList(validDaughterCounter) = candidateDaughterID;
                        validDaughterCurrentList(validDaughterCounter) = abs(candidateDaughterStrand.strandCurrent);
                        if validDaughterCounter == 1,
                            %if this is the first valid daughter, add to current path
                            strandPathList{currentPathID} = [currentStrandPath, lowAttachedStrands(daughterIter)];
                            nodePathList{currentPathID} = [currentNodePath, candidateDaughterOtherNode];
                            validDaughterPathIndexList(1) = currentPathID;
                        else%if validDaughterCounter == 1,
                            %if not the first daughter, duplicate path up to this point, and
                            %make new path
                            pathCounter = pathCounter + 1;
                            strandPathList{pathCounter} = [currentStrandPath, lowAttachedStrands(daughterIter)];
                            nodePathList{pathCounter} = [currentNodePath, candidateDaughterOtherNode];
                            fractionPathList{pathCounter} = currentFractionPath;
                            pathActiveFlagList(pathCounter) = 1;  
                            validDaughterPathIndexList(validDaughterCounter) = pathCounter;
                        end%if-else validDaughterCounter == 1,
                    end%if candidateDaughterOtherVoltage < lowVoltage,
                end%for daughterIter = 1:numDaughterPaths,
                clear daughterIter
                %-----
                daughterSumCurrent = sum(validDaughterCurrentList);
                fractionalDaughterCurrent = validDaughterCurrentList./daughterSumCurrent;
                for validDaughterIter = 1:validDaughterCounter;
                    validDaughterSegmentID = validDaughterStrandList(validDaughterIter);
                    validDaughterFraction = currentFractionPath(end).*fractionalDaughterCurrent(validDaughterIter);
                    fractionPathList{validDaughterPathIndexList(validDaughterIter)} = [currentFractionPath , validDaughterFraction];
                    strandFlowList(validDaughterSegmentID) = strandFlowList(validDaughterSegmentID) + validDaughterFraction;
                    
                    
                    strandRadius = Strands(validDaughterSegmentID).MedianRadius;
                    currentList = [tofStruct(validDaughterSegmentID).currentList, baseStrandCurrent .* validDaughterFraction];
                    tofStruct(validDaughterSegmentID).currentList = currentList;
                    thisDaughterCurrent = abs(Strands(validDaughterSegmentID).strandCurrent);
                    thisMotherEndTime = endTimeList(currentStrandID);
                    strandLength = strandLengthList(validDaughterSegmentID);
%                     timeOfFlight = strandLength*(strandRadius.^2)/thisDaughterCurrent;
timeOfFlight = strandLength/thisDaughterCurrent;
                    
                    
                    thisDaughterEndTime = thisMotherEndTime + timeOfFlight;
                    transitTimeList(validDaughterSegmentID) = timeOfFlight;
                    startTimeList = [tofStruct(validDaughterSegmentID).startTimeList, thisMotherEndTime];
                    tofStruct(validDaughterSegmentID).startTimeList = startTimeList;
                    endTimeList(validDaughterSegmentID) = thisDaughterEndTime;
                    

                    
                end%for validDaughterIter = 1:validDaughterCounter;
            end%if-else stopTracingFlag == 1,
        end%for pathIter = find(pathActiveFlagList == 1);
        if sum(pathActiveFlagList) == 0, keepGoingFlag = 0;end
        if myCount == 0, keepGoingFlag = 0;end
        if branchCounter >= maxBranchSearch, keepGoingFlag = 0; end
    end%while (keepGoingFlag == 1),
    downstreamPathList{baseStrandIter} = strandPathList;
    disp(baseStrandIter)
end%for baseStrandIter = 1:numBaseStrands;

for baseStrandIter = 1:numBaseStrands;
    thisPathSet = downstreamPathList{baseStrandIter};
    for pathIter = 1:numel(thisPathSet),
        thisPath = thisPathSet{pathIter};
        thisPath = thisPath(2:end);
        posList = [1:numel(thisPath)]';
        downstreamBranchList(thisPath,baseStrandIter) = min(downstreamBranchList(thisPath,baseStrandIter),posList);
    end%for pathIter = 1:numel(thisPathSet);
end%for baseStrandIter = ...


%%

alpha = 7e4;
alpha = 1e3;
alpha = 7e4

for kk = 1:numStrands,
    startTimeList = tofStruct(kk).startTimeList;
    currentList = tofStruct(kk).currentList;
    fractionCurrent(kk) = abs(sum(currentList)./Strands(kk).strandCurrent);
    bloodOx = exp(-alpha*startTimeList);
    normCurrentList = currentList./sum(currentList);
    meanBloodOx(kk) = sum(bloodOx.*normCurrentList);
    Strands(kk).meanBloodOx = meanBloodOx(kk);
    meanYXZList(kk,1) = mean(Strands(kk).YXZLimits(1,:));
    meanYXZList(kk,2) = mean(Strands(kk).YXZLimits(2,:));
    meanYXZList(kk,3) = mean(Strands(kk).YXZLimits(3,:));    
    strandRadiusList(kk) = Strands(kk).MedianRadius;
    meanVoltage = Strands(kk).MedianVoltage;
    meanZ(kk) = meanYXZList(kk,3);
end
meanVoltage = [Strands.MedianVoltage];
figure(1);plot(meanVoltage,meanBloodOx,'r.');
figure(2);plot(meanZ,meanBloodOx,'r.');


fx = and(meanYXZList(:,2)>200,meanYXZList(:,2)<800);
fy = and(meanYXZList(:,1)>200,meanYXZList(:,1)<900);
fz = and(meanYXZList(:,3)>00,meanYXZList(:,3)<900);
ff = and(and(fy,fx),fz);
fr = strandRadiusList'<3; 
ff = and(ff,fr);
fv = and(meanVoltage'>60 , meanVoltage'<80);
ff = and(ff,fv);
fb = meanBloodOx'>0;
ff = and(ff,fb);


subV = meanVoltage(ff);
subZ = meanZ(ff);
subBloodOx = meanBloodOx(ff) * 0.8;
figure(11);plot(subV,subBloodOx,'g.');
figure(12);plot(subZ,subBloodOx,'g.');




%%

z1 = 600;
z2 = z1+150;

lowOx = 0.15;
highOx = 0.9;

ff = and(Vertices.AllVerts(:,3) > z1,Vertices.AllVerts(:,3)<z2);
vertList = find(ff);
numVerts = numel(vertList);

redFrame = zeros(gds.maskSize(1),gds.maskSize(2));
blueFrame = zeros(gds.maskSize(1),gds.maskSize(2));
greenFrame = zeros(gds.maskSize(1),gds.maskSize(2));
for vertIter = 1:numVerts,
    vertID = vertList(vertIter);
    vy = Vertices.AllVerts(vertID,1);
    vx = Vertices.AllVerts(vertID,2);
    assocStrand = Vertices.StrandIndices(vertID,1);
    assocBloodOx = meanBloodOx(assocStrand);
% % %     if assocBloodOx ==0, disp(assocStrand);disp(Strands(assocStrand).YXZLimits);disp(' ');end
    if assocBloodOx < lowOx,
        blueFrame(vy,vx) = 1;
    end%if assocBloodOx < lowOx,
    %---
    if assocBloodOx > highOx,
        redFrame(vy,vx) = 1;
    end%if assocBloodOx < lowOx,
    %---
    if assocBloodOx < highOx && assocBloodOx > lowOx,
        greenFrame(vy,vx) = 1;
    end%if assocBloodOx < highOx && assocBloodOx > lowOx,
end

RGB = cat(3,cat(3,redFrame,greenFrame),blueFrame);
figure(99);imagesc(RGB);























