%Calculate Stream Convergence (are branches diverging or converging)
%This code looks at all branches and nodes
%It counts branch number starting from the baseStrandsList and flowing downstream
%by current along all downstream paths
%The result is stored in gds.Strands.convergenceValue
%Additionally, gds.nodeConvergenceList contains a list of the [y x z c]
%   where yxz are the coordinates of the node, and c is its +1/-1
%   divergence/convergence value


%Load an FV file
dxy = 200;

yLim1 = dxy;
yLim2 = 1150-dxy;
xLim1 = 100;
xLim2 = 1450-dxy;
zLim1 = 000;
zLim2 = 1400;


numStrands = numel(gds.Strands);
numVertices = numel(gds.Vertices.AllRadii);
convergenceValueList = zeros(1,numStrands);
branchNumberList = zeros(1,numStrands);
meanYXZ = zeros(numStrands,3);
nodeConvergenceValueList = zeros(1,numVertices)+inf;
nodeBranchNumberList = zeros(1,numVertices)+inf;
nodeConvergenceList = zeros(numVertices,4)+inf;

for strandIter = 1:numStrands,
    thisStrandID = strandIter;
    thisStrand = gds.Strands(thisStrandID);
    startVoltage = gds.Vertices.AllVoltages(thisStrand.StartVertexIndex);
    endVoltage = gds.Vertices.AllVoltages(thisStrand.EndVertexIndex);
    thisCurrent = thisStrand.strandCurrent;
    if startVoltage > endVoltage,
        lowerVoltage = endVoltage;
        lowerNeighbors = thisStrand.EndVertexNeighborStrands;
        lowerVertex = thisStrand.EndVertexIndex;
    else
        lowerVoltage = startVoltage;
        lowerNeighbors = thisStrand.StartVertexNeighborStrands;
        lowerVertex = thisStrand.StartVertexIndex;
        thisCurrent = thisCurrent*-1;
    end
    lowerNeighbors = lowerNeighbors(lowerNeighbors~=thisStrandID);
    if numel(lowerNeighbors)>0,
        neighborVoltages = [gds.Strands(lowerNeighbors).MedianVoltage];
        neighborCurrents = [gds.Strands(lowerNeighbors).strandCurrent];
        for neighborIter = 1:numel(lowerNeighbors);
            nStrandID = lowerNeighbors(neighborIter);
            nStrand = gds.Strands(nStrandID);
            startVoltage = gds.Vertices.AllVoltages(nStrand.StartVertexIndex);
            endVoltage = gds.Vertices.AllVoltages(nStrand.EndVertexIndex);
            if endVoltage == lowerVoltage,
                neighborCurrents(neighborIter) = neighborCurrents(neighborIter) * -1;
            end%if endVoltage == lowerVoltage
        end% for neighborIter...
    
        diffCurrent = (thisCurrent - sum(neighborCurrents));
%         if abs(diffCurrent) > 1e0, keyboard; end

    end%if numel...
    
    if numel(lowerNeighbors>0),
        neighborVoltages = [gds.Strands(lowerNeighbors).MedianVoltage];
        convergenceValue = sum(double(neighborVoltages<lowerVoltage)*2-1)-1;
        convergenceValue = max(-1,min(1,convergenceValue));
        convergenceValueList(thisStrandID) = convergenceValue;
        nodeConvergenceValueList(lowerVertex) = convergenceValue;
        meanYXZ(thisStrandID,1) = mean(thisStrand.YXZLimits(1,:));
        meanYXZ(thisStrandID,2) = mean(thisStrand.YXZLimits(2,:));
        meanYXZ(thisStrandID,3) = mean(thisStrand.YXZLimits(3,:));
        nodeConvergenceList(lowerVertex,1:3) = gds.Vertices.AllVerts(lowerVertex,:);
        nodeConvergenceList(lowerVertex,4) = convergenceValue;
    end
    %---
    dbl = gds.downstreamBranchList(thisStrandID,:);
    branchNumber = min(min(dbl),999);
    branchNumberList(thisStrandID) = branchNumber;
    nodeBranchNumberList(lowerVertex) = branchNumber;   
end%for strandIter;
activeNodeIndex = find(~isinf(nodeConvergenceList(:,4)));
nodeConvergenceList = nodeConvergenceList(activeNodeIndex,:);
gds.nodeConvergenceList = nodeConvergenceList;


maxBranchVal = max(branchNumberList(branchNumberList<999));

fY = and(meanYXZ(:,1)>yLim1,meanYXZ(:,1)<yLim2);
fX = and(meanYXZ(:,2)>zLim1,meanYXZ(:,2)<xLim2);
fZ = and(meanYXZ(:,3)>zLim1,meanYXZ(:,3)<zLim2);
fYXZ = and(and(fY,fX),fZ);
subStrandList = find(fYXZ);
subBranchNumberList = branchNumberList(fYXZ);
subConvergenceValueList = convergenceValueList(fYXZ);
subStrands = [gds.Strands(fYXZ)];

av = gds.Vertices.AllVerts; 
fY = and(av(:,1)>yLim1,av(:,1)<yLim2);
fX = and(av(:,2)>xLim1,av(:,2)<xLim2);
fZ = and(av(:,3)>zLim1,av(:,3)<zLim2);
fYXZ = and(and(fY,fX),fZ);
subNodeConvergenceValueList = nodeConvergenceValueList(fYXZ);
subNodeBranchNumberList = nodeBranchNumberList(fYXZ);




for branchIter = 1:maxBranchVal,
    ff1 = find(subBranchNumberList==branchIter);
    meanConvergenceList(branchIter) = mean(subConvergenceValueList(ff1));
    meanVoltageList(branchIter) = mean([subStrands(ff1).MedianVoltage]);
end

figure(1);plot(meanConvergenceList,'b-');
figure(2);plot(meanVoltageList,'r-');
figure(3);plot(subBranchNumberList+rand(size(subBranchNumberList))-0.5,[subStrands.MedianVoltage],'r.');
figure(4);plot(subBranchNumberList+rand(size(subBranchNumberList))-0.5,subConvergenceValueList+rand(size(subBranchNumberList))-0.5,'b.');

    

f1 = find(subNodeConvergenceValueList~=inf);
subNodeConvergenceValueList = subNodeConvergenceValueList(f1);
subNodeBranchNumberList = subNodeBranchNumberList(f1);


meanNodeConvergenceList = [];
for branchIter = 1:maxBranchVal,
    ff1 = find(subNodeBranchNumberList==branchIter);
    meanNodeConvergenceList(branchIter) = mean(subNodeConvergenceValueList(ff1));
end


figure(5);plot(meanNodeConvergenceList,'b-');
figure(6);plot(subNodeBranchNumberList+rand(size(subNodeBranchNumberList))-0.5,subNodeConvergenceValueList+rand(size(subNodeConvergenceValueList))-0.5,'b.');

for k = 1:numel(gds.Strands),
    gds.Strands(k).convergenceValue = convergenceValueList(k);
end


