%This script takes all the plunging artery tops and clots them one-by-one
%and compares the flow downstream to the non-clot flow.
%In this version, the main line of the plunging arteriole is identified
%and branch counting starts OFF of that line.

%Instructions:
%Load allProject

scriptVersion = 'ClotPlungingArterioleTipAndDisplayStallsScript1x.m';




vectorProject = allProject.db;

shortName = 'pbDBminus1p0',% pbDBplus0p5 pbDBplus1p0 pbDBminus0p5 pbDBminus1p0 pbDB
radiusChange = -1;

tipIter = 2;%26 12 14 18
showBlue = 0;
lowFlowFraction = 0.5;
figNum = 1;









strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;
rootName = vectorProject.vectorProjectRootName;
findDash = findstr(rootName,'-');
allVerts = vertices.AllVerts;
strandIndices1 = vertices.StrandIndices(:,1);
strandMidYXZList = vectorProject.vectorizedStructure.strandMidYXZList;

strandLabelList = [strands(:).MedianLabel];
surfList = find(and(strandLabelList>=300,strandLabelList<=399)); 
zSurf = strandMidYXZList(surfList,3);
medianZSurf = median(zSurf);
minZSurf = min(zSurf);

strandMidYXZList(:,3) = max(strandMidYXZList(:,3)-minZSurf,0);
allPlungingVenuleList = vectorProject.vectorizedStructure.allPlungingVenuleList;
allPlungingArterioleList = vectorProject.vectorizedStructure.allPlungingArterioleList;

flowAnalysisParameters = [];
cleanupFlag = 1;





%%Stage 2: Run the unperturbed flow analysis:
unperturbedFileName = ['F:\Phil\AOH_VectorProjects\OcclusionFiles\',shortName,'-Unperturbed.mat'];
if exist(unperturbedFileName,'file'),
    load(unperturbedFileName,'origCurrentList');
    disp('UnPerturbed Flow Calculation loaded');
else%if exist...
    unperturbedProject = vectorProject;
    unperturbedProject.circuitAnalysisParameters.minRadius = 0;
    unperturbedProject.vectorizedStructure.effectiveStrandRadiusList = max(unperturbedProject.vectorizedStructure.effectiveStrandRadiusList ...
    +radiusChange ,0.25);
    unperturbedProject = CalculateUnperturbedFlow003(unperturbedProject);
    origCurrentList = [unperturbedProject.vectorizedStructure.Strands(:).strandCurrent];
    save(unperturbedFileName,'origCurrentList');
    disp('UnPerturbed Flow Calculation completed');
end%if exist...

clear tipList allPlungingArteriolePathList numTips



%%Stage 3: Select a vessel segment to occlude
tipList = vectorProject.vectorizedStructure.plungingArterioleTipList;
allPlungingArteriolePathList = vectorProject.vectorizedStructure.allPlungingArteriolePathList;
numTips = numel(tipList);


zEndMustBeGreater = 300;


subList = [];
for kk = 1:numel(allPlungingArteriolePathList);
    thisPath = allPlungingArteriolePathList{kk};
    if numel(thisPath)>0,
        thisZ = strandMidYXZList(thisPath,3);
          if thisZ(end)>zEndMustBeGreater
            subList = [subList,kk];
        end%if thisPath(2)...
    end%if numel...
end%for kk
tipList = tipList(subList);
allPlungingArteriolePathList = allPlungingArteriolePathList(subList);
numTips = numel(tipList);


%Set limits on analysis field of view
y1 = 0; y2 = inf;
x1 = 0; x2 = inf;
z1 = 0; z2 = inf;

y1 = 100; y2 = maskSize(1)-y1;
x1 = 100; x2 = maskSize(2)-x1;
z1 = 000; z2 = 1200;


analyzeDownstreamBranchesOnlyFlag = 0;
considerOnlyBranchesInFOVFlag = 1;
ignoreLowStartingCurrentStrandsFlag = 0;
excludeVeinsAndSurfaceFlag = 1;
stallLowFlowVesselFlag = 0;
stopTrackingOnceOutsideFlag = 1;
forceOutletClotFlag = 0;

stallNumFraction = 0.1;
maxBranch = 15;
numPlungingStrandToOcclude = 1;
outletLimit = 999; %consider at most, this number of outlets per occlusion arteriole
samplingLimit = 9999; 

f1 = and(strandMidYXZList(:,1)>y1,strandMidYXZList(:,1)<y2);
f2 = and(strandMidYXZList(:,2)>x1,strandMidYXZList(:,2)<x2);
f3 = and(strandMidYXZList(:,3)>z1,strandMidYXZList(:,3)<z2);
insideStrandList = find(and(and(f1,f2),f3));
outsideStrandList = find(~(and(and(f1,f2),f3)));
insideStrandCurrentList = abs(origCurrentList(insideStrandList));
sortList = sort(insideStrandCurrentList,'ascend');
thresholdCurrentForStall = sortList(ceil(stallNumFraction *numel(sortList)));


allTipMedianCurrentRatioList = [];
allRatioList =[]; allRatioList{maxBranch} = [];
allPreCurrentList =[]; allPreCurrentList{maxBranch} = [];
allPostCurrentList =[]; allPostCurrentList{maxBranch} = [];

tipCounter = 0;
allTipMedianCurrentRatioList = [];
allRatioList =[]; allRatioList{maxBranch} = [];
allPreCurrentList =[]; allPreCurrentList{maxBranch} = [];
allPostCurrentList =[]; allPostCurrentList{maxBranch} = [];
crossingPointList = [];
medianOffShootZList = [];
% for tipIter = 1:1:numTips,









    disp(['TIP ITER ',num2str(tipIter)]);
    occlusionPath = allPlungingArteriolePathList{tipIter};
    myNumOcclude= min(numel(occlusionPath),numPlungingStrandToOcclude);
    occlusionStrandID =  occlusionPath(1:myNumOcclude)
    

    
    origRadius = vectorProject.vectorizedStructure.effectiveStrandRadiusList(occlusionStrandID);
    pathList = allPlungingArteriolePathList{tipIter};
    pathCoords = strandMidYXZList(pathList,:)
    maxPathCoords = max(pathCoords,[],1);
    minPathCoords = min(pathCoords,[],1);
    insideFlag = 1;
    
    if minPathCoords(1)<y1, insideFlag = 0;end
    if minPathCoords(2)<x1, insideFlag = 0;end
    if maxPathCoords(1)>y2, insideFlag = 0;end
    if maxPathCoords(2)>x2, insideFlag = 0;end
    
insideFlag 

dy = 120; dx = 1200;
my1 = median(pathCoords(:,1));
mx1 = median(pathCoords(:,2));
yy1 = my1-dy; yy2 = my1+dy;
xx1 = mx1-dx; xx2 = mx1+dx;
zz1 = 0; zz2 = inf;


vertOrigCurrentList = origCurrentList(strandIndices1)';
plungingVesselFlagList = zeros(size(allVerts,1),1);
plungingVesselFlagList([strands(pathList).StartToEndIndices]) = 1;
allVertsSuper = [allVerts,abs(vertOrigCurrentList),plungingVesselFlagList];

f1 = and(allVertsSuper(:,1)>yy1,allVertsSuper(:,1)<yy2);
f2 = and(allVertsSuper(:,2)>xx1,allVertsSuper(:,2)<xx2);
f3 = and(allVertsSuper(:,3)>zz1,allVertsSuper(:,3)<zz2);
f4 = allVertsSuper(:,4)<thresholdCurrentForStall;
ff1 = and(and(f1,f2),f3);
ff2 = and(ff1,f4);
ff3 = and(ff1,allVertsSuper(:,5)==1);
av1 = allVerts(ff1,:);
av2 = allVerts(ff2,:);
pv1 = allVerts(ff3,:);

figure(figNum);p1 = plot3(av1(:,1),av1(:,2),-1*av1(:,3),'g.');hold on;set(p1,'MarkerSize',1);
p3 = plot3(pv1(:,1),pv1(:,2),-1*pv1(:,3),'k.');hold on;
p2 = plot3(av2(:,1),av2(:,2),-1*av2(:,3),'r.');hold off;set(p2,'MarkerSize',1);
axis equal
camva(7.8260);camtarget([596.5000  730.0000 -717.0000]);
campos([-9382 -230 716])
% campos([1065 -8760 -1365]);
    
    
    %Consider branches off the clotted plunging arteriole line
    flowProject.maskSize = maskSize;
    flowProject.Strands = strands;
    flowProject.Vertices = vertices;
    flowProject.baseStrandIDList = pathList;
    flowProject.maxBranchSearch = 2;
    flowProject.stopVoltage = 50.1;%Hardcode;
    flowProject.minimumFluxFraction = 1/(2^18);%Hardcode
    disp('Finding arteriole outlets paths...');
    flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
    arterioleOffshootList = find(min(flowProject.downstreamBranchList,[],2)==1);
    numArterioleOffshoots = numel(arterioleOffshootList);
    downstreamBranchList = flowProject.downstreamBranchList;
    arterioleOffshootList = arterioleOffshootList(~ismember(arterioleOffshootList,vectorProject.vectorizedStructure.allPlungingArterioleList));
    offshootZ = strandMidYXZList(arterioleOffshootList,3);
    medianOffshootZ = median(offshootZ);
    arterioleOffshootList = arterioleOffshootList(and(offshootZ>z1,offshootZ<z2));
    if ~isempty(arterioleOffshootList),
        arterioleOffshootList = arterioleOffshootList(1:min(numel(arterioleOffshootList),outletLimit));
    end%if~isempty
    
    if forceOutletClotFlag == 1,
        occlusionStrandID = [occlusionStrandID,arterioleOffshootList'];
    end%if forceOutletClotFlag == 1,
    
    occlusionString = num2str(occlusionStrandID);
    occlusionString = regexprep(occlusionString,'     ','_');
    occlusionString = regexprep(occlusionString,'    ','_');
    occlusionString = regexprep(occlusionString,'   ','_');
    occlusionString = regexprep(occlusionString,'  ','_');
    occlusionString = regexprep(occlusionString,' ','_');
    occlusionString = [shortName,'-Clot-',occlusionString,'.mat'];
    occlusionFilePath = 'F:\Phil\AOH_VectorProjects\OcclusionFiles';
    occlusionFileName = fullfile(occlusionFilePath,occlusionString);
    
    
    perturbedProject = vectorProject;
    perturbedProject.circuitAnalysisParameters.minRadius = 0;
    perturbedProject.vectorizedStructure.effectiveStrandRadiusList(occlusionStrandID) = 0;

    if exist(occlusionFileName,'file'),
        load(occlusionFileName,'perturbedCurrentList');
        disp('Perturbed Flow loaded from file');
    else%if exist(occlusionFileName...
        perturbedProject.vectorizedStructure.effectiveStrandRadiusList = max(perturbedProject.vectorizedStructure.effectiveStrandRadiusList ...
        +radiusChange ,0.25);
        perturbedProject = CalculateUnperturbedFlow003(perturbedProject);
        perturbedCurrentList = [perturbedProject.vectorizedStructure.Strands(:).strandCurrent];
        save(occlusionFileName,'perturbedCurrentList');
        disp('Perturbed Flow Calculation completed');
    end%if-else exist(occlusionFileName...



    vertPerturbedCurrentList = perturbedCurrentList(strandIndices1)';
    plungingVesselFlagList = zeros(size(allVerts,1),1);
    plungingVesselFlagList([strands(pathList).StartToEndIndices]) = 1;
    allVertsSuper = [allVerts,abs(vertPerturbedCurrentList),plungingVesselFlagList];

    f1 = and(allVertsSuper(:,1)>yy1,allVertsSuper(:,1)<yy2);
    f2 = and(allVertsSuper(:,2)>xx1,allVertsSuper(:,2)<xx2);
    f3 = and(allVertsSuper(:,3)>zz1,allVertsSuper(:,3)<zz2);
    f4 = allVertsSuper(:,4)<thresholdCurrentForStall;
    ff1 = and(and(f1,f2),f3);
    ff2 = and(ff1,f4);
    ff3 = and(ff1,allVertsSuper(:,5)==1);
    ff4 = and(ff1,abs(vertPerturbedCurrentList./vertOrigCurrentList)<lowFlowFraction);

    av1 = allVerts(ff1,:);
    av2 = allVerts(ff2,:);
    pv1 = allVerts(ff3,:);
    sv1 = allVerts(ff4,:);

    figure(figNum+1);p1 = plot3(av1(:,1),av1(:,2),-1*av1(:,3),'g.');hold on;set(p1,'MarkerSize',1);
    p3 = plot3(pv1(:,1),pv1(:,2),-1*pv1(:,3),'k.');hold on;
    if showBlue,
        p4 = plot3(sv1(:,1),sv1(:,2),-1*sv1(:,3),'b.');hold on;set(p4,'MarkerSize',1);
    end
    p2 = plot3(av2(:,1),av2(:,2),-1*av2(:,3),'r.');hold off;set(p2,'MarkerSize',1);
    axis equal
    camva(7.8260);camtarget([596.5000  730.0000 -717.0000]);
    campos([-9382 -230 716])
%     campos([1065 -8760 -1365]);

            
