%This script takes all the plunging artery tops and clots them one-by-one
%and compares the flow downstream to the non-clot flow.
%In this version, all downstream branches are equal: no "plunging arteriole" line
%   is identified

%Instructions:
%CD to the VectorProjectFolder
%Load the latest FV file


thresholdCurrentForStall = 0;
maxBranchNum = 15;

%Calculate Unperturbed Flow
disp('Calculating UnPerturbed flow');
clotProject = [];
clotProject.circuitAnalysisParameters = gds.circuitAnalysisParameters;
clotProject.vectorizedStructure.maskSize = gds.maskSize;
clotProject.vectorizedStructure.Vertices = gds.Vertices;
clotProject.vectorizedStructure.Strands = gds.Strands;
%Force Calculation of strand conductances one time:
clotProject.circuitAnalysisParameters = rmfield(clotProject.circuitAnalysisParameters,'strandConductanceList');
    
    clotProject = CalculateFlowWithClot(clotProject);
    
gds.Vertices = clotProject.vectorizedStructure.Vertices;
gds.Strands = clotProject.vectorizedStructure.Strands;
gds.clotAnalysisParameters = clotProject.circuitAnalysisParameters;
gds.circuitAnalysisParameters = clotProject.circuitAnalysisParameters;
%---
for strandIter = 1:numel(gds.Strands),
    cc = gds.Vertices.AllVerts(gds.Strands(strandIter).StartToEndIndices,:);
    yxzLim(:,1) = min(cc,[],1)';
    yxzLim(:,2) = max(cc,[],1)';
    gds.Strands(strandIter).YXZLimits = yxzLim;
    gds.Strands(strandIter).MedianRadius = median(gds.Vertices.AllRadii(gds.Strands(strandIter).InteriorVertices,:));
    gds.Strands(strandIter).MedianLabel = mode(gds.Vertices.AllLabels(gds.Strands(strandIter).InteriorVertices,:));
    gds.Strands(strandIter).MedianVoltage = mode(gds.Vertices.AllVoltages(gds.Strands(strandIter).InteriorVertices,:));
    gds.Strands(strandIter).VoltageDrop = max(gds.Vertices.AllVoltages(gds.Strands(strandIter).InteriorVertices,:)) - ...
            min(gds.Vertices.AllVoltages(gds.Strands(strandIter).InteriorVertices,:));
end%for strandIter = 1:numel(gds.Strands),
disp('DONE');


unperturbedGDS = gds;



for kk = 1:maxBranchNum
    runningMedianRatioList{kk} = [];
    runningSelectedStrandList = [];
end

strandsToClotList = gds.baseStrandIDList;
numStrandsToClot = numel(strandsToClotList);

for clotIter = 1:numStrandsToClot,
    gds = unperturbedGDS;
    clotStrandID = strandsToClotList(clotIter);
    
    fprintf('Clotting Strand %0.0f of %0.0f \n',clotIter,numStrandsToClot)
    clotProject = [];
    clotProject.circuitAnalysisParameters = gds.circuitAnalysisParameters;
    clotProject.vectorizedStructure.maskSize = gds.maskSize;
    clotProject.vectorizedStructure.Vertices = gds.Vertices;
    clotProject.vectorizedStructure.Strands = gds.Strands;
    clotProject.circuitAnalysisParameters.clotStrandID = clotStrandID;
    
        clotProject = CalculateFlowWithClot(clotProject);

    gds.Vertices = clotProject.vectorizedStructure.Vertices;
    gds.Strands = clotProject.vectorizedStructure.Strands;
    gds.clotAnalysisParameters = clotProject.circuitAnalysisParameters;
    %---
    for strandIter = 1:numel(gds.Strands),
        cc = gds.Vertices.AllVerts(gds.Strands(strandIter).StartToEndIndices,:);
        yxzLim(:,1) = min(cc,[],1)';
        yxzLim(:,2) = max(cc,[],1)';
        gds.Strands(strandIter).YXZLimits = yxzLim;
        gds.Strands(strandIter).MedianRadius = median(gds.Vertices.AllRadii(gds.Strands(strandIter).InteriorVertices,:));
        gds.Strands(strandIter).MedianLabel = mode(gds.Vertices.AllLabels(gds.Strands(strandIter).InteriorVertices,:));
        gds.Strands(strandIter).MedianVoltage = mode(gds.Vertices.AllVoltages(gds.Strands(strandIter).InteriorVertices,:));
        gds.Strands(strandIter).VoltageDrop = max(gds.Vertices.AllVoltages(gds.Strands(strandIter).InteriorVertices,:)) - ...
                min(gds.Vertices.AllVoltages(gds.Strands(strandIter).InteriorVertices,:));
    end%for strandIter = 1:numel(gds.Strands),
    disp('DONE');

    
    %Locate 1st downstream branch from clot
    selectedStrandID = clotStrandID;
    selectedStrand = unperturbedGDS.Strands(selectedStrandID);
    startAttachedStrands = [gds.Vertices.StrandIndices(selectedStrand.StartVertexIndex,:)];
    startAttachedStrands = startAttachedStrands(startAttachedStrands~=0);
    startAttachedStrands = startAttachedStrands(startAttachedStrands~=selectedStrandID);
    endAttachedStrands = [gds.Vertices.StrandIndices(selectedStrand.EndVertexIndex,:)];
    endAttachedStrands = endAttachedStrands(endAttachedStrands~=0);
    endAttachedStrands = endAttachedStrands(endAttachedStrands~=selectedStrandID);
    startMedianVoltage = median([unperturbedGDS.Strands(startAttachedStrands).MedianVoltage]);
    endMedianVoltage = median([unperturbedGDS.Strands(endAttachedStrands).MedianVoltage]);
    if startMedianVoltage > endMedianVoltage,
        nextRoundStrands = endAttachedStrands;
        trackedStrands = [selectedStrandID,endAttachedStrands];
    else%if startMedianVoltage > endMedianVoltage,
        nextRoundStrands = startAttachedStrands;
        trackedStrands = [selectedStrandID,startAttachedStrands];
    end%if-else startMedianVoltage > endMedianVoltage,

    currentRatioCell = {};
    branchIter = 1;
    clotCurrent = gds.clotAnalysisParameters.strandCurrentList(selectedStrandID);
    origCurrent = unperturbedGDS.clotAnalysisParameters.strandCurrentList(selectedStrandID);
    currentRatio = 0;%by Default for the clotted Strand
    currentRatioCell{1} = currentRatio;


    %Track Down Along the branches
    for branchIter = 2:maxBranchNum,
        seedStrands = nextRoundStrands;

        nextRoundStrands = [];
        currentRatioList = [];
        for strandIter = 1:numel(seedStrands),
            thisStrandID = seedStrands(strandIter);
            thisStrand = gds.Strands(thisStrandID);
            if ~ismember(thisStrandID,gds.clotAnalysisParameters.clotStrandID),
                clotCurrent = abs(gds.clotAnalysisParameters.strandCurrentList(thisStrandID));
                origCurrent = abs(unperturbedGDS.clotAnalysisParameters.strandCurrentList(thisStrandID));
                if abs(clotCurrent) < thresholdCurrentForStall, clotCurrent = 0; end
                currentRatio = clotCurrent/origCurrent;
                currentRatioList = [currentRatioList,currentRatio];
                %Find the next branch down
                allAttachedStrands = [gds.Vertices.StrandIndices(thisStrand.StartVertexIndex,:),gds.Vertices.StrandIndices(thisStrand.EndVertexIndex,:)];
                nextStrands = allAttachedStrands(~ismember(allAttachedStrands,trackedStrands));
                nextStrands = nextStrands(nextStrands~=0);
                trackedStrands = [trackedStrands,nextStrands];
                nextRoundStrands = [nextRoundStrands,nextStrands];
            end%if ~ismember(thisStrandID,gds.clotAnalysisParameters.clotStrandID),
        end%for strandIter = 1:numel(seedStrands),
        currentRatioCell{branchIter} = currentRatioList;
    end %for branchIter = 2:maxBranchNum,       

    medianRatioList = [];
    for kk = 1:numel(currentRatioCell);
        q = currentRatioCell{kk}; 
        q = q(~isnan(q));
        medianRatioList(kk) = median(q);
        runningMedianRatioList{kk} = [runningMedianRatioList{kk},median(q)];
    end

    howFarDown = min(gds.downstreamBranchList(selectedStrandID,:));

    figure(1);plot(medianRatioList,'r.');
    runningSelectedStrandList = [runningSelectedStrandList,selectedStrandID];

    figure(2);plot(0,0);hold on
    for kk = 1:numel(runningMedianRatioList);
        q = runningMedianRatioList{kk};
        q = q(~isnan(q));
        for jj = 1:numel(q);
            plot(kk,(q(jj)),'bo');
        end%forjj
    end 
    hold off
    pause(1);
end%for clotIter


