%This script takes all the plunging artery tops and clots them one-by-one
%and compares the flow downstream to the non-clot flow.
%In this version, the main line of the plunging arteriole is identified
%and branch counting starts OFF of that line.

%Instructions:
%Load allProject

vectorProject = allProject.co;
strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;
flowAnalysisParameters = [];
cleanupFlag = 1;

thresholdCurrentForStall = 0;
maxBranchNum = 15;



%%
%%Stage 2: Run the unperturbed flow analysis:
unperturbedProject = vectorProject;
unperturbedProject.circuitAnalysisParameters.minRadius = 0;
unperturbedProject = CalculateUnperturbedFlow003(unperturbedProject);
disp('UnPerturbed Flow Calculation completed');



%%
%%Stage 3: Select a vessel segment to occlude
tipList = vectorProject.vectorizedStructure.plungingArterioleTipList;
allPlungingArteriolePathList = vectorProject.vectorizedStructure.allPlungingArteriolePathList;
numTips = numel(tipList);
stallMatrix = [];
stallIDMatrix = [];

%%
f1 = figure(1);
set(f1,'DoubleBuffer','on');
clf
y1 = 0; y2 = inf;
x1 = 0; x2 = inf;
z1 = 0; z2 = inf;

y1 = 200; y2 = maskSize(1)-y1;
x1 = 200; x2 = maskSize(2)-x1;
z1 = 100; z2 = 300;

% zo = 700; dz = 100; z1 = zo-dz;z2 = zo+dz;
maxBranch = 15;

allTipMedianCurrentRatioList = [];
allRatioList =[]; allRatioList{maxBranch} = [];
allPreCurrentList =[]; allPreCurrentList{maxBranch} = [];
allPostCurrentList =[]; allPostCurrentList{maxBranch} = [];

% % f1 = figure(1);
% % set(f1,'DoubleBuffer','on');
% % hold off

tipCounter = 0;
allTipMedianCurrentRatioList = [];
allRatioList =[]; allRatioList{maxBranch} = [];
allPreCurrentList =[]; allPreCurrentList{maxBranch} = [];
allPostCurrentList =[]; allPostCurrentList{maxBranch} = [];
for tipIter = 1:numTips,

     
    
f1 = figure(1);
set(f1,'DoubleBuffer','on');
clf
    
    
    disp(['TIP ITER ',num2str(tipIter)]);
    occlusionStrandID = tipList(tipIter)
    pathList = allPlungingArteriolePathList{tipIter};
    pathCoords = vectorProject.vectorizedStructure.strandMidYXZList(pathList,:)
    maxPathCoords = max(pathCoords);
    minPathCoords = min(pathCoords);
    insideFlag = 1;
    
    if minPathCoords(1)<y1, insideFlag = 0;end
    if minPathCoords(2)<x1, insideFlag = 0;end
    if maxPathCoords(1)>y2, insideFlag = 0;end
    if maxPathCoords(2)>x2, insideFlag = 0;end
    
    %Consider branches off the clotted arteriole line
    flowProject.maskSize = unperturbedProject.vectorizedStructure.maskSize;
    flowProject.Strands = unperturbedProject.vectorizedStructure.Strands;
    flowProject.Vertices = unperturbedProject.vectorizedStructure.Vertices;
    startList = pathList;
    flowProject.baseStrandIDList = startList;
    flowProject.maxBranchSearch = maxBranch;
    flowProject.stopVoltage = 50.1;%Hardcode;
    flowProject.minimumFluxFraction = 1/(2^18);%Hardcode
        disp('Calculating downstream paths...');
    flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
    arterioleOffshootList = find(min(flowProject.downstreamBranchList,[],2)==1);
    numArterioleOffshoots = numel(arterioleOffshootList);
    downstreamBranchList = flowProject.downstreamBranchList;

    arterioleOffshootList = arterioleOffshootList(~ismember(arterioleOffshootList,vectorProject.vectorizedStructure.allPlungingArterioleList))
    offshootZ = vectorProject.vectorizedStructure.strandMidYXZList(arterioleOffshootList,3)
    arterioleOffshootList = arterioleOffshootList(and(offshootZ>z1,offshootZ<z2))

    if isempty(arterioleOffshootList), insideFlag = 0; end
    

    if insideFlag == 1,

        origRadius = vectorProject.vectorizedStructure.effectiveStrandRadiusList(occlusionStrandID)

        %Stage 4: Calculate the perturbed flow surface vessel endpoints are held at fixed
        %           voltages (100 and 50) using effect Strand Radii & true lengths

        perturbedProject = vectorProject;
        perturbedProject.circuitAnalysisParameters.minRadius = 0;
        perturbedProject.vectorizedStructure.effectiveStrandRadiusList(occlusionStrandID) = 0;
        perturbedProject = CalculateUnperturbedFlow003(perturbedProject);
        disp('Perturbed Flow Calculation completed');



        %Compare the two results
        origCurrentList = [unperturbedProject.vectorizedStructure.Strands(:).strandCurrent];
        perturbedCurrentList = [perturbedProject.vectorizedStructure.Strands(:).strandCurrent];
        
% % %         perturbedCurrentList(abs(perturbedCurrentList)<thresholdCurrentForStall) = 0;
        lowCurrentStrandList = find(origCurrentList < thresholdCurrentForStall);

        currentRatioList = abs(perturbedCurrentList./origCurrentList);



        %---
        flowProject.baseStrandIDList = arterioleOffshootList;
        flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
        downstreamBranchList = flowProject.downstreamBranchList;
        
        
        
        
        
        branchClassList = [];
        medianCurrentRatioList = [];
        for branchIter = 1:maxBranch,
            branchClass = branchIter;
            branchClassList(branchIter) = branchClass;
            findBranch = find(sum(downstreamBranchList==branchClass,2)>0);
            findBranch = findBranch(~ismember(findBranch,startList));
            crl = currentRatioList(findBranch);
            
% % crl = crl(~ismember(crl,lowCurrentStrandList));
            
            allRatioList{branchClass} = [allRatioList{branchClass},crl];
            allPreCurrentList{branchClass} = [allPreCurrentList{branchClass},origCurrentList(findBranch)];
            allPostCurrentList{branchClass} = [allPostCurrentList{branchClass},perturbedCurrentList(findBranch)];
            
            figure(1);p1 = plot(repmat(branchClass,size(crl)),crl,'bo');hold on
            medianCurrentRatioList(branchClass) = median(crl);
        end%for branchIter;
        figure(1);plot(branchClassList,medianCurrentRatioList,'r-');
        set(gca,'YLim',[0,2]);
        tipCounter = tipCounter +1;
        allTipMedianCurrentRatioList(tipCounter,:) = medianCurrentRatioList;
        pause(0.1);
    end%if insideFlag == 1,
end%for tipIter..
medianOfMedianCurrentRatioList = median(allTipMedianCurrentRatioList,1);
figure(2);plot(branchClassList,medianOfMedianCurrentRatioList);

fullMedianList = [];
for branchIter = 1:maxBranch,
    fullMedianList(branchIter) = median(allRatioList{branchIter});
end%branchIter
figure(3);plot(branchClassList,fullMedianList,'g-');

    
    
%%
    


    
   
    
    
 
    for arterioleLineIter = 3:5,
        thisStrand = gds.Strands(arterioleLineIDList(arterioleLineIter-1));
        startAttachedStrands = gds.Vertices.StrandIndices(thisStrand.StartVertexIndex,:);
        endAttachedStrands = gds.Vertices.StrandIndices(thisStrand.EndVertexIndex,:);
        if ismember(arterioleLineIDList(arterioleLineIter-2),startAttachedStrands),
            nextStrands = endAttachedStrands;
        else
            nextStrands = startAttachedStrands;
        end%if ismember
        nextStrands = nextStrands(nextStrands~=0);
        nextStrands = nextStrands(nextStrands~=arterioleLineIDList(arterioleLineIter-1));
        nextRadii = strandRadiusList(nextStrands);
        bigRadiusIndex = find(nextRadii==max(nextRadii));
        otherIndicesFlag = true(size(nextRadii));
        otherIndicesFlag(bigRadiusIndex(1)) = 0;
        nextRoundStrands = [nextRoundStrands,nextStrands(otherIndicesFlag)];
        arterioleLineIDList(arterioleLineIter) = nextStrands(bigRadiusIndex(1));
    end

    trackedStrands = arterioleLineIDList;
    
    
    currentRatioCell = {};
% %     branchIter = 1;
% %     clotCurrent = gds.clotAnalysisParameters.strandCurrentList(selectedStrandID);
% %     origCurrent = unperturbedGDS.clotAnalysisParameters.strandCurrentList(selectedStrandID);
% %     currentRatio = 0;%by Default for the clotted Strand
    currentRatioCell{1} = 0;


    %Track Down Along the branches
    for branchIter = 1:maxBranchNum,
        seedStrands = nextRoundStrands;

        nextRoundStrands = [];
        currentRatioList = [];
        for strandIter = 1:numel(seedStrands),
            thisStrandID = seedStrands(strandIter);
            thisStrand = gds.Strands(thisStrandID);
            if ~ismember(thisStrandID,gds.clotAnalysisParameters.clotStrandID),
                clotCurrent = abs(gds.clotAnalysisParameters.strandCurrentList(thisStrandID));
                origCurrent = abs(unperturbedGDS.clotAnalysisParameters.strandCurrentList(thisStrandID));
                if abs(clotCurrent) < thresholdCurrentForStall, clotCurrent = 0; end
                currentRatio = clotCurrent/origCurrent;
                currentRatioList = [currentRatioList,currentRatio];
                %Find the next branch down
                allAttachedStrands = [gds.Vertices.StrandIndices(thisStrand.StartVertexIndex,:),gds.Vertices.StrandIndices(thisStrand.EndVertexIndex,:)];
                nextStrands = allAttachedStrands(~ismember(allAttachedStrands,trackedStrands));
                nextStrands = nextStrands(nextStrands~=0);
                trackedStrands = [trackedStrands,nextStrands];
                nextRoundStrands = [nextRoundStrands,nextStrands];
            end%if ~ismember(thisStrandID,gds.clotAnalysisParameters.clotStrandID),
        end%for strandIter = 1:numel(seedStrands),
        currentRatioCell{branchIter} = currentRatioList;
    end %for branchIter = 2:maxBranchNum,       

    medianRatioList = [];
    for kk = 1:numel(currentRatioCell);
        q = currentRatioCell{kk}; 
        q = q(~isnan(q));
        medianRatioList(kk) = median(q);
        runningMedianRatioList{kk} = [runningMedianRatioList{kk},median(q)];
    end

    howFarDown = min(gds.downstreamBranchList(selectedStrandID,:));

    figure(1);plot(medianRatioList,'r.');
    runningSelectedStrandList = [runningSelectedStrandList,selectedStrandID];

    figure(2);plot(0,0);hold on
    for kk = 1:numel(runningMedianRatioList);
        q = runningMedianRatioList{kk};
        q = q(~isnan(q));
        for jj = 1:numel(q);
            plot(kk,(q(jj)),'bo');
        end%forjj
    end 
    hold off
    pause(1);
end%for clotIter    
    
    
    
    
    
    
%%
    
    
    
    
    
    
    %Plot decreased flow as as shades of blue
    ratioList = [0.5 0.4 0.3 0.2 0.1];
    colorMultiplier = 2;
    for ratioIter = 1:numel(ratioList),
        thisRatio = ratioList(ratioIter);
        thisColor = [thisRatio*colorMultiplier, thisRatio*colorMultiplier, 1];
        thisColor = min(1,thisColor);
        activeStrands = find(currentRatioList<thisRatio);
        activeIndices = [strands(activeStrands).StartToEndIndices];
        av = vertices.AllVerts;
        activeCoords = [vertices.AllVerts(activeIndices,:)];
        av = av(activeIndices,:);
        activeCoords = av(and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2)),:);
        p1 = plot3(activeCoords(:,1),activeCoords(:,2),-1*activeCoords(:,3),'b.');
        set(p1,'MarkerSize',1,'Color',thisColor);
        hold on
    end

    %Plot decreased flow as as shades of red
    ratioList = [1.5 1.6 1.7 1.8 1.9 2];
    colorMultiplier = 0.5;
    maxRatio = ratioList(end);
    ratioRange = ratioList(end)-ratioList(1);
    for ratioIter = 1:numel(ratioList),
        thisRatio = ratioList(ratioIter);
        thisColor = [1, (maxRatio-thisRatio)/ratioRange, (maxRatio-thisRatio)/ratioRange];
        thisColor = min(1,thisColor);
        activeStrands = find(currentRatioList>thisRatio);
        activeIndices = [strands(activeStrands).StartToEndIndices];
        av = vertices.AllVerts;
        activeCoords = [vertices.AllVerts(activeIndices,:)];
        av = av(activeIndices,:);
        activeCoords = av(and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2)),:);
        p1 = plot3(activeCoords(:,1),activeCoords(:,2),-1*activeCoords(:,3),'b.');
        set(p1,'MarkerSize',1,'Color',thisColor);
        hold on
    end


    occulsionIndices = [strands(occlusionStrandID).StartToEndIndices];
    occulsionCoords = [vertices.AllVerts(occulsionIndices,:)];
    av = vertices.AllVerts;
    av = av(occulsionIndices,:);
    occulsionCoords = av(and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2)),:);
    p1 = plot3(occulsionCoords(:,1),occulsionCoords(:,2),-1*occulsionCoords(:,3),'g.');
    set(p1,'MarkerSize',2);
    hold off


    %%Stage 7: Occlude stalled vessels

    stallRatioList = [0.25:0.05:0.75];%
    stopCountingNumStalls = 15000;%Hardcode
    numStallRatio = numel(stallRatioList);
    numRepeats = 50;
    numStalls = zeros(numStallRatio,numRepeats)-1;
    stallIDList = [];
    stopFlag = 0;
    for stallRatioIter = 1:numel(stallRatioList);
        stallRatio = stallRatioList(stallRatioIter);
        disp(['STALL RATIO : ',num2str(stallRatio)]);
        for repeatIter = 1:numRepeats,
            disp(['REPEAT # ',num2str(repeatIter)]);
            origCurrentList = [unperturbedProject.vectorizedStructure.Strands(:).strandCurrent];
            perturbedCurrentList = [perturbedProject.vectorizedStructure.Strands(:).strandCurrent];
            currentRatioList = abs(perturbedCurrentList./origCurrentList);
            stalledVesselList = find(currentRatioList<stallRatio);
            numStalls(stallRatioIter,repeatIter) = numel(stalledVesselList);
            stallIDList{stallRatioIter,repeatIter} = stalledVesselList;
            
            perturbedProject.vectorizedStructure.effectiveStrandRadiusList(stalledVesselList) = 0;
            perturbedProject = CalculateUnperturbedFlow003(perturbedProject);
            disp('Perturbed Flow Calculation completed');

            %Stage 7b: compare the two results
            origCurrentList = [unperturbedProject.vectorizedStructure.Strands(:).strandCurrent];
            perturbedCurrentList = [perturbedProject.vectorizedStructure.Strands(:).strandCurrent];
            currentRatioList = abs(perturbedCurrentList./origCurrentList);
            f1 = figure(1);
            set(f1,'DoubleBuffer','on');
            clf
            y1 = 0; y2 = inf;
            x1 = 0; x2 = inf;
            z1 = 0; z2 = inf;
            % zo = 700; dz = 100; z1 = zo-dz;z2 = zo+dz;
            %Plot decreased flow as as shades of blue
            ratioList = [0.5 0.4 0.3 0.2 0.1];
            colorMultiplier = 2;
            for ratioIter = 1:numel(ratioList),
                thisRatio = ratioList(ratioIter);
                thisColor = [thisRatio*colorMultiplier, thisRatio*colorMultiplier, 1];
                thisColor = min(1,thisColor);
                activeStrands = find(currentRatioList<thisRatio);
                activeIndices = [strands(activeStrands).StartToEndIndices];
                av = vertices.AllVerts;
                activeCoords = [vertices.AllVerts(activeIndices,:)];
                av = av(activeIndices,:);
                activeCoords = av(and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2)),:);
                p1 = plot3(activeCoords(:,1),activeCoords(:,2),-1*activeCoords(:,3),'b.');
                set(p1,'MarkerSize',1,'Color',thisColor);
                hold on
            end
            %Plot decreased flow as as shades of red
            ratioList = [1.5 1.6 1.7 1.8 1.9 2];
            colorMultiplier = 0.5;
            maxRatio = ratioList(end);
            ratioRange = ratioList(end)-ratioList(1);
            for colorRatioIter = 1:numel(ratioList),
                thisRatio = ratioList(colorRatioIter);
                thisColor = [1, (maxRatio-thisRatio)/ratioRange, (maxRatio-thisRatio)/ratioRange];
                thisColor = min(1,thisColor);
                activeStrands = find(currentRatioList>thisRatio);
                activeIndices = [strands(activeStrands).StartToEndIndices];
                av = vertices.AllVerts;
                activeCoords = [vertices.AllVerts(activeIndices,:)];
                av = av(activeIndices,:);
                activeCoords = av(and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2)),:);
                p1 = plot3(activeCoords(:,1),activeCoords(:,2),-1*activeCoords(:,3),'b.');
                set(p1,'MarkerSize',1,'Color',thisColor);
                hold on
            end%for colorRatioIter
            %---
            occulsionIndices = [strands(occlusionStrandID).StartToEndIndices];
            occulsionCoords = [vertices.AllVerts(occulsionIndices,:)];
            av = vertices.AllVerts;
            av = av(occulsionIndices,:);
            occulsionCoords = av(and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2)),:);
            p1 = plot3(occulsionCoords(:,1),occulsionCoords(:,2),-1*occulsionCoords(:,3),'g.');
            set(p1,'MarkerSize',2);
            hold off

            if repeatIter >5,
                if numStalls(stallRatioIter,repeatIter) <= min(numStalls(stallRatioIter,repeatIter-3:repeatIter-1)),
                    if numStalls(stallRatioIter,repeatIter) > stopCountingNumStalls, stopFlag = 1; end
                    break%break out of repeatIter loop
                end%if numStalls...
            end%if repeatIter...
            figure(99);plot(numStalls');
        end%for repeatIter = ...
        if stopFlag == 1,
            break%break out of stallRatioIter loop
        end%if stopFlag == 1,
    end%for stallRatioIter...
    stallMatrix{tipIter} = numStalls;
    stallIDMatrix{tipIter} = stallIDList;

end%for tipIter = 

myParameters.generatedBy = 'IterateDamageFromSingleOcclusion_Auto.m';
myParameters.circuitAnalysisParameters = unperturbedProject.circuitAnalysisParameters;
myParameters.stallRatioList = stallRatioList;
myParameters.stopCountingNumStalls = stopCountingNumStalls;
myParameters.numRepeats = numRepeats;









%%

%Calculate Unperturbed Flow
disp('Calculating UnPerturbed flow');
clotProject = [];
clotProject.circuitAnalysisParameters = gds.circuitAnalysisParameters;
clotProject.vectorizedStructure.maskSize = gds.maskSize;
clotProject.vectorizedStructure.Vertices = gds.Vertices;
clotProject.vectorizedStructure.Strands = gds.Strands;
%Force Calculation of strand conductances one time:
clotProject.circuitAnalysisParameters = rmfield(clotProject.circuitAnalysisParameters,'strandConductanceList');
    
    clotProject = CalculateFlowWithClot(clotProject);
    
gds.Vertices = clotProject.vectorizedStructure.Vertices;
gds.Strands = clotProject.vectorizedStructure.Strands;
gds.clotAnalysisParameters = clotProject.circuitAnalysisParameters;
gds.circuitAnalysisParameters = clotProject.circuitAnalysisParameters;
%---
for strandIter = 1:numel(gds.Strands),
    cc = gds.Vertices.AllVerts(gds.Strands(strandIter).StartToEndIndices,:);
    yxzLim(:,1) = min(cc,[],1)';
    yxzLim(:,2) = max(cc,[],1)';
    gds.Strands(strandIter).YXZLimits = yxzLim;
    gds.Strands(strandIter).MedianRadius = median(gds.Vertices.AllRadii(gds.Strands(strandIter).InteriorVertices,:));
    gds.Strands(strandIter).MedianLabel = mode(gds.Vertices.AllLabels(gds.Strands(strandIter).InteriorVertices,:));
    gds.Strands(strandIter).MedianVoltage = mode(gds.Vertices.AllVoltages(gds.Strands(strandIter).InteriorVertices,:));
    gds.Strands(strandIter).VoltageDrop = max(gds.Vertices.AllVoltages(gds.Strands(strandIter).InteriorVertices,:)) - ...
            min(gds.Vertices.AllVoltages(gds.Strands(strandIter).InteriorVertices,:));
end%for strandIter = 1:numel(gds.Strands),
disp('DONE');


unperturbedGDS = gds;
strandRadiusList = [unperturbedGDS.Strands(:).MedianRadius];




for kk = 1:maxBranchNum
    runningMedianRatioList{kk} = [];
    runningSelectedStrandList = [];
end

strandsToClotList = gds.baseStrandIDList;
numStrandsToClot = numel(strandsToClotList);

for clotIter = 1:numStrandsToClot,
    gds = unperturbedGDS;
    clotStrandID = strandsToClotList(clotIter);
    
    fprintf('Clotting Strand %0.0f of %0.0f \n',clotIter,numStrandsToClot)
    clotProject = [];
    clotProject.circuitAnalysisParameters = gds.circuitAnalysisParameters;
    clotProject.vectorizedStructure.maskSize = gds.maskSize;
    clotProject.vectorizedStructure.Vertices = gds.Vertices;
    clotProject.vectorizedStructure.Strands = gds.Strands;
    clotProject.circuitAnalysisParameters.clotStrandID = clotStrandID;
    
        clotProject = CalculateFlowWithClot(clotProject);

    gds.Vertices = clotProject.vectorizedStructure.Vertices;
    gds.Strands = clotProject.vectorizedStructure.Strands;
    gds.clotAnalysisParameters = clotProject.circuitAnalysisParameters;
    %---
    for strandIter = 1:numel(gds.Strands),
        cc = gds.Vertices.AllVerts(gds.Strands(strandIter).StartToEndIndices,:);
        yxzLim(:,1) = min(cc,[],1)';
        yxzLim(:,2) = max(cc,[],1)';
        gds.Strands(strandIter).YXZLimits = yxzLim;
        gds.Strands(strandIter).MedianRadius = median(gds.Vertices.AllRadii(gds.Strands(strandIter).InteriorVertices,:));
        gds.Strands(strandIter).MedianLabel = mode(gds.Vertices.AllLabels(gds.Strands(strandIter).InteriorVertices,:));
        gds.Strands(strandIter).MedianVoltage = mode(gds.Vertices.AllVoltages(gds.Strands(strandIter).InteriorVertices,:));
        gds.Strands(strandIter).VoltageDrop = max(gds.Vertices.AllVoltages(gds.Strands(strandIter).InteriorVertices,:)) - ...
                min(gds.Vertices.AllVoltages(gds.Strands(strandIter).InteriorVertices,:));
    end%for strandIter = 1:numel(gds.Strands),
    disp('DONE');
    
    %ID the main arteriole line
    arterioleLineIDList = clotStrandID;
    selectedStrandID = clotStrandID;
    selectedStrand = unperturbedGDS.Strands(selectedStrandID);
    startAttachedStrands = [gds.Vertices.StrandIndices(selectedStrand.StartVertexIndex,:)];
    startAttachedStrands = startAttachedStrands(startAttachedStrands~=0);
    startAttachedStrands = startAttachedStrands(startAttachedStrands~=selectedStrandID);
    endAttachedStrands = [gds.Vertices.StrandIndices(selectedStrand.EndVertexIndex,:)];
    endAttachedStrands = endAttachedStrands(endAttachedStrands~=0);
    endAttachedStrands = endAttachedStrands(endAttachedStrands~=selectedStrandID);
    startMedianVoltage = median([unperturbedGDS.Strands(startAttachedStrands).MedianVoltage]);
    endMedianVoltage = median([unperturbedGDS.Strands(endAttachedStrands).MedianVoltage]);
    if startMedianVoltage > endMedianVoltage,
        nextStrands = endAttachedStrands;
    else%if startMedianVoltage > endMedianVoltage,
        nextStrands = startAttachedStrands;
    end%if-else startMedianVoltage > endMedianVoltage,
    nextRoundStrands = [];

    nextRadii = strandRadiusList(nextStrands);
    bigRadiusIndex = find(nextRadii==max(nextRadii));
    otherIndicesFlag = true(size(nextRadii));
    otherIndicesFlag(bigRadiusIndex(1)) = 0;
    nextRoundStrands = [nextRoundStrands,nextStrands(otherIndicesFlag)];
    arterioleLineIDList(2) = nextStrands(bigRadiusIndex(1));
    
    for arterioleLineIter = 3:5,
        thisStrand = gds.Strands(arterioleLineIDList(arterioleLineIter-1));
        startAttachedStrands = gds.Vertices.StrandIndices(thisStrand.StartVertexIndex,:);
        endAttachedStrands = gds.Vertices.StrandIndices(thisStrand.EndVertexIndex,:);
        if ismember(arterioleLineIDList(arterioleLineIter-2),startAttachedStrands),
            nextStrands = endAttachedStrands;
        else
            nextStrands = startAttachedStrands;
        end%if ismember
        nextStrands = nextStrands(nextStrands~=0);
        nextStrands = nextStrands(nextStrands~=arterioleLineIDList(arterioleLineIter-1));
        nextRadii = strandRadiusList(nextStrands);
        bigRadiusIndex = find(nextRadii==max(nextRadii));
        otherIndicesFlag = true(size(nextRadii));
        otherIndicesFlag(bigRadiusIndex(1)) = 0;
        nextRoundStrands = [nextRoundStrands,nextStrands(otherIndicesFlag)];
        arterioleLineIDList(arterioleLineIter) = nextStrands(bigRadiusIndex(1));
    end

    trackedStrands = arterioleLineIDList;
    
    
    currentRatioCell = {};
% %     branchIter = 1;
% %     clotCurrent = gds.clotAnalysisParameters.strandCurrentList(selectedStrandID);
% %     origCurrent = unperturbedGDS.clotAnalysisParameters.strandCurrentList(selectedStrandID);
% %     currentRatio = 0;%by Default for the clotted Strand
    currentRatioCell{1} = 0;


    %Track Down Along the branches
    for branchIter = 1:maxBranchNum,
        seedStrands = nextRoundStrands;

        nextRoundStrands = [];
        currentRatioList = [];
        for strandIter = 1:numel(seedStrands),
            thisStrandID = seedStrands(strandIter);
            thisStrand = gds.Strands(thisStrandID);
            if ~ismember(thisStrandID,gds.clotAnalysisParameters.clotStrandID),
                clotCurrent = abs(gds.clotAnalysisParameters.strandCurrentList(thisStrandID));
                origCurrent = abs(unperturbedGDS.clotAnalysisParameters.strandCurrentList(thisStrandID));
                if abs(clotCurrent) < thresholdCurrentForStall, clotCurrent = 0; end
                currentRatio = clotCurrent/origCurrent;
                currentRatioList = [currentRatioList,currentRatio];
                %Find the next branch down
                allAttachedStrands = [gds.Vertices.StrandIndices(thisStrand.StartVertexIndex,:),gds.Vertices.StrandIndices(thisStrand.EndVertexIndex,:)];
                nextStrands = allAttachedStrands(~ismember(allAttachedStrands,trackedStrands));
                nextStrands = nextStrands(nextStrands~=0);
                trackedStrands = [trackedStrands,nextStrands];
                nextRoundStrands = [nextRoundStrands,nextStrands];
            end%if ~ismember(thisStrandID,gds.clotAnalysisParameters.clotStrandID),
        end%for strandIter = 1:numel(seedStrands),
        currentRatioCell{branchIter} = currentRatioList;
    end %for branchIter = 2:maxBranchNum,       

    medianRatioList = [];
    for kk = 1:numel(currentRatioCell);
        q = currentRatioCell{kk}; 
        q = q(~isnan(q));
        medianRatioList(kk) = median(q);
        runningMedianRatioList{kk} = [runningMedianRatioList{kk},median(q)];
    end

    howFarDown = min(gds.downstreamBranchList(selectedStrandID,:));

    figure(1);plot(medianRatioList,'r.');
    runningSelectedStrandList = [runningSelectedStrandList,selectedStrandID];

    figure(2);plot(0,0);hold on
    for kk = 1:numel(runningMedianRatioList);
        q = runningMedianRatioList{kk};
        q = q(~isnan(q));
        for jj = 1:numel(q);
            plot(kk,(q(jj)),'bo');
        end%forjj
    end 
    hold off
    pause(1);
end%for clotIter


