%Follow each possible downstream flow from a penetrating arteriole offshoot to a
%penetrating venule.  At each branch node, evaluate if the node is one of converging or
%diverging flow.


%Stage 1: Load a -flow-vectorProject

flowAnalysisParameters = [];
perturbedProject = vectorProject;
cleanupFlag = 1;
strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;



%%%Stage 2: Find the offshoots of plunging arterioles
flowProject.maskSize = vectorProject.vectorizedStructure.maskSize;
flowProject.Strands = vectorProject.vectorizedStructure.Strands;
flowProject.Vertices = vectorProject.vectorizedStructure.Vertices;
startList = vectorProject.vectorizedStructure.allPlungingArterioleList;
flowProject.baseStrandIDList = startList;
flowProject.maxBranchSearch = 3;
flowProject.stopVoltage = 50.1;%Hardcode;
flowProject.minimumFluxFraction = 1/(2^18);%Hardcode


disp('Calculating downstream paths...');
tic
flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
toc
offshootList = find(min(flowProject.downstreamBranchList,[],2)==1);
numOffshoots = numel(offshootList);


        

%%
%Pick an offshoot: track it down to the first node that is a converging (versus diverging) branch.

myChoice = 30;
purelyDivergentStrandSet = [];
allDownstreamStrandSet = [];

numFlipsList = [];
pathLengthList = [];
startStrandList = [];
meanDepthList = [];
maxDepthList = [];

allNumFlips = [];
allPathLength = [];
allStartStrand = [];
allMeanDepth = [];
allMaxDepth = [];

minPathValue = zeros(1,numel(strands))+999;
maxCumCDValue = zeros(1,numel(strands))-999;
%%
for myChoice = 1:1:numOffshoots,
    startStrandID = offshootList(myChoice)
    flowProject.baseStrandIDList = startStrandID;
    flowProject.maxBranchSearch = 20;
    
    %force stop when hit plunging venule
    plungingVenuleList = vectorProject.vectorizedStructure.allPlungingVenuleList;
    flowProject.Vertices.AllVoltages(plungingVenuleList) = 50;

    
    flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);

    downstreamPathList = flowProject.downstreamPathList{1};
    numPaths = numel(downstreamPathList);

    numFlips = [];
    pathLength = [];
    startStrand = [];
    meanDepth = [];
    maxDepth = [];
    

    

    
        
 
    for pathIter = 1:numPaths,

        thisPath = downstreamPathList{pathIter};
        pathDepth = vectorProject.vectorizedStructure.strandMidYXZList(thisPath,3);
        numStrands = numel(thisPath);
        cdList = zeros(size(thisPath));
        for strandIter = 1:numStrands-1;
            rootStrandID = thisPath(strandIter);
            childStrandID = thisPath(strandIter+1);
            childStrand = strands(childStrandID);
            minPathValue(rootStrandID) = min(minPathValue(rootStrandID),strandIter);
            svns = childStrand.StartVertexNeighborStrands;
            evns = childStrand.EndVertexNeighborStrands;
            neighborList = [];
            if ismember(rootStrandID,svns),neighborList = svns; end
            if ismember(rootStrandID,evns),neighborList = evns; end
            neighborList = neighborList(neighborList~=rootStrandID);
            neighborList = neighborList(neighborList~=childStrandID);
            childVoltage = childStrand.StartVoltage;
            neighborVoltageList = [strands(neighborList).MedianVoltage];
            neighborUpstreamFlagList = neighborVoltageList > childVoltage;
            if sum(neighborUpstreamFlagList) == 0,
                %all neighbors are downstream, and this is a purely divergent branch
                cdList(strandIter) = -1;%divergent
            else%if sum(neighborUpstreamFlagList) == 0,
                cdList(strandIter) = +1;%convergent
            end%if-else sum(neighborUpstreamFlagList) == 0,
        end%for strandIter...
        maxCumCDValue(thisPath) = max(maxCumCDValue(thisPath),cumsum(cdList));
        cdList = cdList(1:end-1);
        flipList = abs(diff(cdList));
        numFlips(pathIter) = sum(flipList>0);
        pathLength(pathIter) = numel(thisPath);
        startStrand(pathIter) = thisPath(1);
        meanDepth(pathIter) = mean(pathDepth);
        maxDepth(pathIter) = max(pathDepth);
        
        
     end%for pathIter...
 
% %          figure(1);plot(numFlips,'r.');
% %         figure(2);plot(numFlips./pathLength,'b.');
% %         pause(0.1);
     
     
    numFlipsList{myChoice} = numFlips;
    pathLengthList{myChoice} = pathLength;
    startStrandList{myChoice} = startStrand;
    meanDepthList{myChoice} = meanDepth;
    maxDepthList{myChoice} = maxDepth;
    allNumFlips = [allNumFlips,numFlips];
    allPathLength = [allPathLength,pathLength];
    allStartStrand = [allStartStrand,startStrand];
    allMeanDepth = [allMeanDepth,meanDepth];
    allMaxDepth = [allMaxDepth,maxDepth];
    
% %     figure(3);p3 = plot(allMeanDepth,allNumFlips+rand(size(allNumFlips))*2-1,'g.');
% %     set(p3,'MarkerSize',1);
% %     figure(4);p4 = plot(allMeanDepth,(allNumFlips+rand(size(allNumFlips))*2-1)./allPathLength,'m.');
% %     set(p4,'MarkerSize',1);
    
end%for myChoice ...

%%
myData = [allMaxDepth',allNumFlips',allPathLength'];
sortData = sortrows(myData,1);
filtSize = 200;
filtData = imfilter(sortData,ones(filtSize,1),'replicate')./filtSize;
figure(98);plot(filtData(:,1),filtData(:,2),'r.');
figure(99);plot(filtData(:,1),filtData(:,2)./filtData(:,3),'r.');
midz = vectorProject.vectorizedStructure.strandMidYXZList(:,3);

save ConvergenceDivergenceResults.mat

%%

barrelCoords = [];
for k = 1:numel(barrels);
    barrelCoords = [barrelCoords;barrels{k}];
end
sbc = barrelCoords(1:10:end,:);
figure(111);
p111 = plot3(sbc(:,1),sbc(:,2),-1*sbc(:,3),'b.');



%%

strandCurrentList = abs([strands(:).strandCurrent]);


y1 = 0; y2 = inf; x1 = 0; x2 = inf; z1 = 0; z2 = inf;
zo = 570; dz = 30; z1 = zo-dz;z2 = zo+dz;

av = vertices.AllVerts;
numVerts = size(av,1);
subAVFlags = and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2));


    thisColor = [1 0 0];
    activeStrands1 = find(maxCumCDValue<-2);
    activeStrands1 = find(strandCurrentList<3.0e7);
%     activeStrands1 = find(and(minPathValue<=4,minPathValue<=15));
    
    activeIndices1 = [strands(activeStrands1).StartToEndIndices];
    activeFlags1 = zeros(numVerts,1);
    activeFlags1(activeIndices1) = 1;
    activeCoords1 = [vertices.AllVerts(and(activeFlags1,subAVFlags),:)];
    figure(1);p1 = plot3(activeCoords1(:,1),activeCoords1(:,2),-1*activeCoords1(:,3),'b.');hold on
    set(p1,'MarkerSize',1,'Color',thisColor);hold on
    sbc2 = sbc(and(and(and(sbc(:,1)>y1,sbc(:,1)<y2),and(sbc(:,2)>x1,sbc(:,2)<x2)),and(sbc(:,3)>z1,sbc(:,3)<z2)),:);
    s1 = plot3(sbc2(:,1),sbc2(:,2),-1*sbc2(:,3),'m.');hold off
    
%%     

    thisColor = [0 1 0];
    activeStrands1 = find(and(minPathValue>=7,minPathValue<=8));
    
    activeIndices1 = [strands(activeStrands1).StartToEndIndices];
    activeFlags1 = zeros(numVerts,1);
    activeFlags1(activeIndices1) = 1;
    activeCoords1 = [vertices.AllVerts(and(activeFlags1,subAVFlags),:)];
    figure(1);p1 = plot3(activeCoords1(:,1),activeCoords1(:,2),-1*activeCoords1(:,3),'b.');hold on
    set(p1,'MarkerSize',1,'Color',thisColor);hold on
   
    
    
        thisColor = [0 0 1];
    activeStrands1 = find(maxCumCDValue<-1);
    activeStrands1 = find(and(minPathValue>=11,minPathValue<=15));
    
    activeIndices1 = [strands(activeStrands1).StartToEndIndices];
    activeFlags1 = zeros(numVerts,1);
    activeFlags1(activeIndices1) = 1;
    activeCoords1 = [vertices.AllVerts(and(activeFlags1,subAVFlags),:)];
    figure(1);p1 = plot3(activeCoords1(:,1),activeCoords1(:,2),-1*activeCoords1(:,3),'b.');hold on
    set(p1,'MarkerSize',1,'Color',thisColor);hold off
    
    
    
    
    
    
