%Follow each possible downstream flow from a penetrating arteriole offshoot to a
%penetrating venule.  At each branch node, evaluate if the node is one of converging or
%diverging flow.


%Stage 1: Load a -flow-vectorProject

flowAnalysisParameters = [];
perturbedProject = vectorProject;
cleanupFlag = 1;
strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
numStrands = numel(strands);



        

strandConvergenceList = [];
for strandIter = 1:numStrands,
    thisStrandID = strandIter;
    thisStrand = vectorProject.vectorizedStructure.Strands(thisStrandID);
    startVoltage = vectorProject.vectorizedStructure.Vertices.AllVoltages(thisStrand.StartVertexIndex);
    endVoltage = vectorProject.vectorizedStructure.Vertices.AllVoltages(thisStrand.EndVertexIndex);
    if startVoltage > endVoltage,
        lowerVoltage = endVoltage;
        upperVoltage = startVoltage;
        lowerNeighbors = thisStrand.EndVertexNeighborStrands;
        upperNeighbors = thisStrand.StartVertexNeighborStrands;
        lowerNodeID = thisStrand.EndVertexIndex;
        upperNodeID = thisStrand.StartVertexIndex;
    else
        lowerVoltage = startVoltage;
        upperVoltage = endVoltage;
        lowerNeighbors = thisStrand.StartVertexNeighborStrands;
        upperNeighbors = thisStrand.EndVertexNeighborStrands;
        lowerNodeID = thisStrand.StartVertexIndex;
        upperNodeID = thisStrand.EndVertexIndex;
        
    end

    lowerNeighbors = lowerNeighbors(lowerNeighbors~=thisStrandID);
    upperNeighbors = upperNeighbors(upperNeighbors~=thisStrandID);
    
    lowerConvergenceValue = 0;
    upperConvergenceValue = 0;
    
    if numel(lowerNeighbors>0),
        neighborVoltages = [vectorProject.vectorizedStructure.Strands(lowerNeighbors).MedianVoltage];
        numHigherNeighbors = sum(neighborVoltages>lowerVoltage);
        numLowerNeighbors = sum(neighborVoltages<lowerVoltage);
        numInputs = numHigherNeighbors+1;%this strand is an input to the lower node
        numOutputs = numLowerNeighbors;
        lowerConvergenceValue = numOutputs-numInputs;%+1 = divergent, -1 = convergent
    end
    %---
    if numel(upperNeighbors>0),
        neighborVoltages = [vectorProject.vectorizedStructure.Strands(upperNeighbors).MedianVoltage];
        numHigherNeighbors = sum(neighborVoltages>upperVoltage);
        numLowerNeighbors = sum(neighborVoltages<upperVoltage);
        numInputs = numHigherNeighbors;
        numOutputs = numLowerNeighbors+1;% this strand is an output to the upper node
        upperConvergenceValue = numOutputs-numInputs;
    end
    strandConvergenceList(strandIter,1:2) = [upperConvergenceValue, lowerConvergenceValue];
end%for strandIter;

%%

strandFlipList = strandConvergenceList(:,1) - strandConvergenceList(:,2);

 y1 = 0; y2 = inf; x1 = 0; x2 = inf; z1 = 0; z2 = inf;
 zo = 450; dz =50; z1 = zo-dz;z2 = zo+dz;
av = vertices.AllVerts;
numVerts = size(av,1);
subAVFlags = and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2));


showStrands = find(strandFlipList >= 2);
%---
activeStrands = showStrands;
activeIndices = [strands(activeStrands).StartToEndIndices];
activeFlags = false(numVerts,1);
activeFlags(activeIndices) = 1;
activeCoords = [vertices.AllVerts(and(activeFlags,subAVFlags),:)];
figure(3);p2 = plot3(activeCoords(:,1),activeCoords(:,2),-1*activeCoords(:,3),'b.');
hold on; plot3(0,0,0,'w.');
plot3(maskSize(1),maskSize(2),-1*maskSize(3),'w.');hold off;






