vp = allProject.au;

vs = vp.vectorizedStructure;
strands = vs.Strands;
vertices = vs.Vertices;
strandIndices = vs.Vertices.StrandIndices;
maskSize = vp.maskSize;
vertBarrelValueList = vp.vertBarrelValueList;

barrelIter = 4;
barrelVertList = find(vertBarrelValueList == barrelIter);
barrelStrandList = unique(strandIndices(barrelVertList,1));

flowProject = vs;
flowProject.baseStrandIDList = barrelStrandList;
flowProject.maxBranchSearch = 10;
flowProject.stopVoltage = 50.1;
flowProject.minimumFluxFraction = 1/3000;


flowProject.baseStrandIDList = barrelStrandList;
tic
disp('Calculating Downstream branches');
flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
toc
allDownstreamPathList = flowProject.downstreamPathList ;
allDownstreamBranchList = flowProject.downstreamBranchList ;


%%



% % tic
% % disp('Calculating DownstreamFlow');
% % flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
% % toc

% % downstreamPathList = flowProject.downstreamPathList ;
% % downstreamBranchList = flowProject.downstreamBranchList ;

%Now take each strand in the barrel, and track it down, dividing its dye as it goes
allSumStrandFractionList = zeros(numel(strands),1);
for startStrandIter = 1:numel(barrelStrandList);
    disp(['startStrandIter: ',num2str(startStrandIter)]);
    startStrandID = barrelStrandList(startStrandIter);
    startStrand = strands(startStrandID);
    flowProject.baseStrandIDList = startStrandID;
    flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
    downstreamPathList = flowProject.downstreamPathList ;
    downstreamBranchList = flowProject.downstreamBranchList ;
    downstreamPathList = downstreamPathList{1};
    numPaths = numel(downstreamPathList);
    sumStrandFractionList = zeros(numel(strands),1);
    for pathIter = 1:numPaths,
        stillValidPathFlag = 1;
        thisPath = downstreamPathList{pathIter};

        strandFractionList = zeros(numel(strands),1);
        nowFraction = 1;%tracks the fraction of the fluid that is "dyed" at this point in the path
        strandFractionList(startStrandID) = 0;

        for strandIter = 1:numel(thisPath)-1,
           
            thisStrandID = thisPath(strandIter);
            nextStrandID = thisPath(strandIter+1);
            if ismember(nextStrandID,barrelStrandList),
                %this path will already be counted as STARTING from the next strand
                %stop analyzing this path to avoid double counting
                stillValidPathFlag = 0;
            end
            if stillValidPathFlag == 1,
                thisStrand = strands(thisStrandID);
                if thisStrand.StartVoltage > thisStrand.EndVoltage,
                    %downstream node is end node
                    exitNeighbors = thisStrand.EndVertexNeighborStrands;
                    exitVertex = thisStrand.EndVertexIndex;
                else%if thisStrand.StartVoltage > thisStrand.EndVoltage,
                    %downstream node is start node
                    exitNeighbors = thisStrand.StartVertexNeighborStrands;
                    exitVertex = thisStrand.StartVertexIndex;
                end%if-else thisStrand.StartVoltage > thisStrand.EndVoltage,
                exitNeighbors = exitNeighbors(~ismember(exitNeighbors,thisStrandID));
                numExitNeighbors = numel(exitNeighbors);
                flowInNeighborList = [];
                flowOutNeighborList = [];
                for neighborIter = 1:numExitNeighbors,
                    thisNeighborID = exitNeighbors(neighborIter);
                    thisNeighbor = strands(thisNeighborID);
                    if exitVertex == thisNeighbor.StartVertexIndex,
                        %neighor attached through its StartVertex
                        attachedVertexVoltage = thisNeighbor.StartVoltage;
                        otherVertexVoltage = thisNeighbor.EndVoltage;
                    else%if exitVertex ==
                        %neighbor attached through its EndVertex
                        attachedVertexVoltage = thisNeighbor.EndVoltage;
                        otherVertexVoltage = thisNeighbor.StartVoltage;
                    end%if-else exitVertex ==...
                    %---
                    if attachedVertexVoltage > otherVertexVoltage,
                        %this neighbor is a drain
                        flowOutNeighborList = [flowInNeighborList,thisNeighborID];
                    else%if attachedVertexVoltage > otherVertexVoltage,
                        %this neighbor is another source
                        flowInNeighborList = [flowInNeighborList,thisNeighborID];
                    end%if-else attachedVertexVoltage > otherVertexVoltage,
                end%for neighborIter = 1:numExitNeighbors,
                %---
                if numel(flowInNeighborList) == 0,
                    %all flow is outward, 
                    strandFractionList(nextStrandID) = nowFraction;
                else%if numel(flowInNeighborList) = 0,
                    %there is other inflow, this dilutes the dye
                    sumOtherCurrents = sum(abs([strands(flowInNeighborList).strandCurrent]));
                    dyeSourceCurrent = abs(thisStrand.strandCurrent);
                    cutFraction = dyeSourceCurrent./(dyeSourceCurrent+sumOtherCurrents);
                    nowFraction = nowFraction * cutFraction;
                    strandFractionList(nextStrandID) = nowFraction;
                end%if-else numel(flowInNeighborList) = 0,
            end%if stillValidPathFlag == 1
        end%for strandIter = ...
%         sumStrandFractionList = sumStrandFractionList + strandFractionList;
        sumStrandFractionList = max(sumStrandFractionList,strandFractionList);
    end%for pathIter
allSumStrandFractionList = allSumStrandFractionList + sumStrandFractionList;

end%for startStrandIter...        
    


%%

%Visualize the barrel strands, and their downstream flow strands


barrelVertCoords = vertices.AllVerts(barrelVertList,:);
allDownstreamStrandList = find(min(allDownstreamBranchList,[],2)<Inf);

cc1 = [];cc2 = [];cc3 = [];
for strandIter = 1:numel(allDownstreamStrandList)
    thisStrand = strands(allDownstreamStrandList(strandIter));
    stei = thisStrand.StartToEndIndices;
    strandVertCoordList = vertices.AllVerts(stei,:);
    cc1 = [cc1;strandVertCoordList(:,1)];
    cc2 = [cc2;strandVertCoordList(:,2)];
    cc3 = [cc3;strandVertCoordList(:,3)];
end%for strandIter

bvc1 = barrelVertCoords(:,1);
bvc2 = barrelVertCoords(:,2);
bvc3 = barrelVertCoords(:,3);

figure(1);plot3(cc1,cc2,-1*cc3,'b.');hold on
figure(1);plot3(bvc1,bvc2,-1*bvc3,'r.');hold off
set(gca,'XLim',[0,1400],'YLim',[0,1500],'ZLim',[-1000,0]);


%%
%Visualize the barrel strands, and their downstream flow strands with dye dilution


barrelVertCoords = vertices.AllVerts(barrelVertList,:);
allDownstreamStrandList = find(min(allDownstreamBranchList,[],2)<Inf);

minStrandStrength = 0.1;

cc1 = [];cc2 = [];cc3 = [];
for strandIter = 1:numel(allDownstreamStrandList)
    thisStrandID = allDownstreamStrandList(strandIter);
    thisStrand = strands(thisStrandID);
    thisStrandStrength = allSumStrandFractionList(thisStrandID);
    if thisStrandStrength > minStrandStrength,
        stei = thisStrand.StartToEndIndices;
        strandVertCoordList = vertices.AllVerts(stei,:);
        cc1 = [strandVertCoordList(:,1)];
        cc2 = [strandVertCoordList(:,2)];
        cc3 = [strandVertCoordList(:,3)];
        figure(2);p1 = plot3(cc1,cc2,-1*cc3,'b.');hold on
        set(p1,'MarkerSize',(thisStrandStrength+0.0001)*5);
    end%if thisStrandStrength >...
end%for strandIter

bvc1 = barrelVertCoords(:,1);
bvc2 = barrelVertCoords(:,2);
bvc3 = barrelVertCoords(:,3);

figure(2);plot3(bvc1,bvc2,-1*bvc3,'r.');hold off
set(gca,'XLim',[0,1400],'YLim',[0,1500],'ZLim',[-1000,0]);


%%

%%


% % %%
% % %Visualize the barrel strands, and their downstream flow strands
% % topView = zeros(maskSize(1),maskSize(2));
% % barrelVertCoords = vertices.AllVerts(barrelVertList,:);
% % 
% % 
% % downstreamStrandList = find(min(downstreamBranchList,[],2)<Inf);
% % for strandIter = 1:numel(downstreamStrandList)
% %     thisStrand = strands(downstreamStrandList(strandIter));
% %     stei = thisStrand.StartToEndIndices;
% %     strandVertCoordList = vertices.AllVerts(stei,:);
% %     for vertIter = 1:numel(stei);
% %         topView(strandVertCoordList(vertIter,1),strandVertCoordList(vertIter,2)) = 1;
% %     end%for vertIter
% % end%for strandIter
% % 
% % for vertIter = 1:numel(barrelVertList);
% %     topView(barrelVertCoords(vertIter,1),barrelVertCoords(vertIter,2)) = 2;
% % end%for vertIter
% % 
% % 
% % 
% % 
% % figure(1);imagesc(topView)
% %     


