%Load  a -flow-VectorProject.mat file

%%

vectorizedStructure = vectorProject.vectorizedStructure;
maskSize = vectorizedStructure.maskSize;
strands = vectorizedStructure.Strands;
vertices = vectorizedStructure.Vertices;
numVerts = numel(vertices.AllRadii);
numStrands = numel(strands);
allVerts = vertices.AllVerts;

allPlungingArterioleList = vectorizedStructure.allPlungingArterioleList;
allPlungingVenuleList = vectorizedStructure.allPlungingVenuleList;


%%

%%%Stage 2: Find the offshoots of plunging arterioles
flowProject.maskSize = vectorProject.vectorizedStructure.maskSize;
flowProject.Strands = vectorProject.vectorizedStructure.Strands;
flowProject.Vertices = vectorProject.vectorizedStructure.Vertices;
startList = vectorProject.vectorizedStructure.allPlungingArterioleList;
flowProject.baseStrandIDList = startList;
flowProject.maxBranchSearch = 3;
flowProject.stopVoltage = 50.1;%Hardcode;
flowProject.minimumFluxFraction = 1/(2^18);%Hardcode

disp('Calculating downstream paths...');
flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
arterioleOffshootList = find(min(flowProject.downstreamBranchList,[],2)==1);
numArterioleOffshoots = numel(arterioleOffshootList);

%---

flowProject.maskSize = vectorProject.vectorizedStructure.maskSize;
flowProject.Strands = vectorProject.vectorizedStructure.Strands;
flowProject.Vertices = vectorProject.vectorizedStructure.Vertices;
startList = vectorProject.vectorizedStructure.allPlungingVenuleList;
flowProject.baseStrandIDList = startList;
flowProject.maxBranchSearch = 3;
flowProject.stopVoltage = 99;%Hardcode;
flowProject.minimumFluxFraction = 1/(2^18);%Hardcode

disp('Calculating upstream paths...');
flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
venuleOffshootList = find(min(flowProject.downstreamBranchList,[],2)==1);
numVenuleOffshoots = numel(venuleOffshootList);
        
%%
strandIndices = vertices.StrandIndices;

adjStruct = ParseStrandAdjacencyFromStrandIndices(strandIndices);
SAM = adjStruct.SAM;
subgraphLabels = adjStruct.subgraphLabels;
startLabels = subgraphLabels;
outsideStrands = find(subgraphLabels~=1);
outsideVerts = ismember(strandIndices,outsideStrands);
strandIndices(outsideVerts==1) = 0;
nextRoundRemovalStrandList = [];

%Remove surface vessels;
strandMedianLabelList = [strands(:).MedianLabel];
surfaceStrandList = find(ismember(strandMedianLabelList,[300:399]));
surfaceVerts = ismember(strandIndices,surfaceStrandList);
strandIndices(surfaceVerts==1) = 0;
% % adjStruct = ParseStrandAdjacencyFromStrandIndices(strandIndices);
% % SAM = adjStruct.SAM;
% % subgraphLabels = adjStruct.subgraphLabels;


%Remove plunging Arterioles;
plungingArterioleVerts = ismember(strandIndices(:),allPlungingArterioleList);
strandIndices(plungingArterioleVerts) = 0;
attachedStrands = unique([strands(allPlungingArterioleList).StartVertexNeighborStrands,strands(allPlungingArterioleList).EndVertexNeighborStrands]);
nextRoundRemovalStrandList = [nextRoundRemovalStrandList,attachedStrands];
% % adjStruct = ParseStrandAdjacencyFromStrandIndices(strandIndices);
% % SAM = adjStruct.SAM;
% % subgraphLabels = adjStruct.subgraphLabels;



%Remove plunging Venules;
plungingVenuleVerts = ismember(strandIndices(:),allPlungingVenuleList);
strandIndices(plungingVenuleVerts) = 0;
attachedStrands = unique([strands(allPlungingVenuleList).StartVertexNeighborStrands,strands(allPlungingVenuleList).EndVertexNeighborStrands]);
nextRoundRemovalStrandList = [nextRoundRemovalStrandList,attachedStrands];
adjStruct = ParseStrandAdjacencyFromStrandIndices(strandIndices);
SAM = adjStruct.SAM;
subgraphLabels = adjStruct.subgraphLabels;

numSubgraphs = max(subgraphLabels);
subGraphSizeList = [];
for kk = 1:numSubgraphs,
    subGraphSizeList(kk) = numel(find(subgraphLabels==kk));
end%for kk
sortSubgraphSizeList = sort(subGraphSizeList,'descend');
disp(sortSubgraphSizeList(1:10));

%%
barrelCoords = [];
for k = 1:numel(barrels);
    barrelCoords = [barrelCoords;barrels{k}];
end
sbc = barrelCoords(1:10:end,:);
figure(1);hold off
p1 = plot3(sbc(:,1),sbc(:,2),-1*sbc(:,3),'k.');hold on



%%
%Trim next round of strands
thisRoundRemovalStrandList = nextRoundRemovalStrandList;
nextRoundRemovalStrandList = [];

removalVerts = ismember(strandIndices(:),thisRoundRemovalStrandList);
strandIndices(removalVerts) = 0;
attachedStrands = unique([strands(thisRoundRemovalStrandList).StartVertexNeighborStrands,strands(thisRoundRemovalStrandList).EndVertexNeighborStrands]);
nextRoundRemovalStrandList = [nextRoundRemovalStrandList,attachedStrands];
adjStruct = ParseStrandAdjacencyFromStrandIndices(strandIndices);
SAM = adjStruct.SAM;
subgraphLabels = adjStruct.subgraphLabels;

numSubgraphs = max(subgraphLabels);
subGraphSizeList = [];
for kk = 1:numSubgraphs,
    subGraphSizeList(kk) = numel(find(subgraphLabels==kk));
end%for kk
[sortSubgraphSizeList,sortInd] = sort(subGraphSizeList,'descend');
disp(sortSubgraphSizeList(1:10));



y1 = 0; y2 = inf; x1 = 0; x2 = inf; z1 = 0; z2 = inf;
zo = 460; dz = 70; z1 = zo-dz;z2 = zo+dz;

av = vertices.AllVerts;
numVerts = size(av,1);
subAVFlags = and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2));




bigGraphCutoff = find(sortSubgraphSizeList<10);
bigGraphCutoff = bigGraphCutoff(1);
bigGraphStart = find(sortSubgraphSizeList<300);
bigGraphStart = bigGraphStart(1);

markerString = 'k.';
colorList = 'rgbcmyk';
qJet = jet;
colorCounter = 0;
for kk = bigGraphStart:bigGraphCutoff,
%     markerString = [colorList(kk),'.'];
%     colorCounter = kk;
    colorCounter = colorCounter+1;
    myColor = qJet(mod(colorCounter*7,62)+1,:);
    f1 = find(subgraphLabels == sortInd(kk));
    ff = [strands(f1).StartToEndIndices];
    ffFlags = subAVFlags==Inf;
    ffFlags(ff) = 1;
    ff = and(ffFlags,subAVFlags);
    figure(1);p1 = plot3(allVerts(ff,1),allVerts(ff,2),-1*allVerts(ff,3),markerString);hold on
    set(p1,'MarkerEdgeColor',myColor);
end%for kk
% hold off

    


