%From each plunging tip, follow the downstream flow along the branch
%that has the most current

%Load an FV file

yLim1 = 400;
yLim2 = 600;
xLim1 = 400;
xLim2 = 600;
zLim1 = 0;
zLim2 = 600;

lowVoltageCutoff = 55;

baseStrandIDList = gds.baseStrandIDList;
numBases = numel(baseStrandIDList);

numStrands = numel(gds.Strands);
convergenceValueList = zeros(1,numStrands)+inf;
meanYXZ = zeros(numStrands,3);
majorPathList = [];
majorPathLengthList = [];

%Calculate the high flow (or fastest voltage drop) path for each base Strand
for baseIter = 1:numBases;
    keepGoingFlag = 1;
        thisPath = [];
        thisPathLength = [];
        thisStrandID = baseStrandIDList(baseIter);
        thisStrand = gds.Strands(thisStrandID);

    while keepGoingFlag == 1,
        thisPath = [thisPath,thisStrandID];
        thisPathLength = [thisPathLength,numel(thisStrand.StartToEndIndices)];
        startVoltage = gds.Vertices.AllVoltages(thisStrand.StartVertexIndex);
        endVoltage = gds.Vertices.AllVoltages(thisStrand.EndVertexIndex);
        if endVoltage < lowVoltageCutoff,
            keepGoingFlag = 0;
        end
        %---
        if startVoltage > endVoltage,
            lowerVoltage = endVoltage;
            lowerNeighbors = thisStrand.EndVertexNeighborStrands;
        else
            lowerVoltage = startVoltage;
            lowerNeighbors = thisStrand.StartVertexNeighborStrands;
        end
        lowerNeighbors = lowerNeighbors(lowerNeighbors~=thisStrandID);
        
% %         lowerNeighborCurrents = abs([gds.Strands(lowerNeighbors).strandCurrent]);
% %         highFlowIndex = find(lowerNeighborCurrents == max(lowerNeighborCurrents));
% %         highFlowNeighbor = lowerNeighbors(highFlowIndex);
% %         thisStrandID = highFlowNeighbor;
% %         thisStrand = gds.Strands(thisStrandID);
        
        lowerNeighborVoltages = [gds.Strands(lowerNeighbors).MedianVoltage];
        highVDropIndex = find(lowerNeighborVoltages == min(lowerNeighborVoltages));
        highVDropNeighbor = lowerNeighbors(highVDropIndex(1));
        thisStrandID = highVDropNeighbor;
        thisStrand = gds.Strands(thisStrandID);
        
        
    end%while keepGoingFlag == 1,
    majorPathList{baseIter} = thisPath;
    majorPathLengthList{baseIter} = thisPathLength;
end%for baseIter
    


%Calculate the flow convergence along each path
for baseIter = 1:numBases;
    pathConvergence = [];
    majorPath = majorPathList{baseIter};
    numStrands = numel(majorPath);
    for strandIter = 1:numStrands,
        thisStrandID = majorPath(strandIter);
        thisStrand = gds.Strands(thisStrandID);
        startVoltage = gds.Vertices.AllVoltages(thisStrand.StartVertexIndex);
        endVoltage = gds.Vertices.AllVoltages(thisStrand.EndVertexIndex);
        if startVoltage > endVoltage,
            lowerVoltage = endVoltage;
            lowerNeighbors = thisStrand.EndVertexNeighborStrands;
        else
            lowerVoltage = startVoltage;
            lowerNeighbors = thisStrand.StartVertexNeighborStrands;
        end

        lowerNeighbors = lowerNeighbors(lowerNeighbors~=thisStrandID);

        if numel(lowerNeighbors>0),
            neighborVoltages = [gds.Strands(lowerNeighbors).MedianVoltage];
            convergenceValue = sum(double(neighborVoltages<lowerVoltage)*2-1)-1;
            convergenceValue = max(-1,min(1,convergenceValue));
            pathConvergence(strandIter) = convergenceValue;
        end
        %---
    end%for strandIter = 1:numStrands,
    pathConvergenceList{baseIter} = pathConvergence;
end%for baseIter
    

for baseIter = 1:numBases;
    pathConvergence = pathConvergenceList{baseIter};
    pathLength = majorPathLengthList{baseIter};
    cumPathLength = cumsum(pathLength);
    figure(1);plot(cumPathLength,pathConvergence+baseIter/500,'ro');
    hold on
end%for baseIter
hold off


sumConvergence = zeros(1,100);
numEntries = zeros(1,100);
for baseIter = 1:numBases,
    pathConvergence = pathConvergenceList{baseIter};
    pathLength = numel(pathConvergence);
    sumConvergence(1:pathLength) = sumConvergence(1:pathLength)+pathConvergence;
    numEntries(1:pathLength) = numEntries(1:pathLength)+1;
end
meanConvergence = sumConvergence./max(1,numEntries);
figure(2);plot(meanConvergence)




    
    

