%From each plunging tip, follow the major downstream flow along the branch
%that has the most current stopping when the branch is not mostly vertical
%(with the caveat that it ignores short sideways strands)


%Load an FV file

vertFractionRange = [0 1];
radiusRange = [1 3];
lengthRange = [20 50];

cutoffVerticalFraction = 0.5;
minStrandLength = 20;

baseStrandIDList = gds.baseStrandIDList;
numBases = numel(baseStrandIDList);
numStrands = numel(gds.Strands);
majorPathList = [];

%Calculate the high flow path for each base Strand
for baseIter = 1:numBases;
    keepGoingFlag = 1;
        thisPath = [];
        thisStrandID = baseStrandIDList(baseIter);
        thisStrand = gds.Strands(thisStrandID);

    while keepGoingFlag == 1,
        thisPath = [thisPath,thisStrandID];
        startVoltage = gds.Vertices.AllVoltages(thisStrand.StartVertexIndex);
        endVoltage = gds.Vertices.AllVoltages(thisStrand.EndVertexIndex);
        %---
        if startVoltage > endVoltage,
            lowerVoltage = endVoltage;
            lowerNeighbors = thisStrand.EndVertexNeighborStrands;
        else
            lowerVoltage = startVoltage;
            lowerNeighbors = thisStrand.StartVertexNeighborStrands;
        end
        lowerNeighbors = lowerNeighbors(lowerNeighbors~=thisStrandID);
        
        lowerNeighborCurrents = abs([gds.Strands(lowerNeighbors).strandCurrent]);
        lowerNeighborCurrentFraction = lowerNeighborCurrents./sum(lowerNeighborCurrents);
        numNeighbors = numel(lowerNeighbors);
        myMetric = [];
        for neighborIter = 1:numNeighbors,
            thisStrandID = lowerNeighbors(neighborIter);
            thisStrand = gds.Strands(thisStrandID);
            currentFraction = lowerNeighborCurrentFraction(neighborIter);
            startVoltage = gds.Vertices.AllVoltages(thisStrand.StartVertexIndex);
            endVoltage = gds.Vertices.AllVoltages(thisStrand.EndVertexIndex);
            startZ = gds.Vertices.AllVerts(thisStrand.StartVertexIndex,3);
            endZ = gds.Vertices.AllVerts(thisStrand.EndVertexIndex,3);
            if startVoltage>endVoltage,
                dz = endZ-startZ;
            else
                dz = startZ-endZ;
            end
            strandLength = numel(thisStrand.StartToEndIndices);
            zFraction = dz/strandLength;
            strandRadius = thisStrand.MedianRadius;
            strandRadius = min(max(radiusRange(1),strandRadius),radiusRange(2));
            strandLength = min(max(lengthRange(1),strandRadius),lengthRange(2));
            normRadius = (strandRadius-radiusRange(1))/(radiusRange(2)-radiusRange(1));
            normLength = (strandRadius-lengthRange(1))/(lengthRange(2)-lengthRange(1));
            myMetric(neighborIter) = normRadius+normLength+zFraction+currentFraction;
        end
        
        highMetricIndex = find(myMetric == max(myMetric));
        highMetricNeighbor = lowerNeighbors(highMetricIndex);
        thisStrandID = highMetricNeighbor;
        thisStrand = gds.Strands(thisStrandID);
  
        
        %if  neighbors doesn't qualify, then stop
        if zFraction <0 || (zFraction < cutoffVerticalFraction && strandLength > minStrandLength),    
            keepGoingFlag = 0;
        end%if zFraction...

    end%while keepGoingFlag == 1,
    majorPathList{baseIter} = thisPath;
    
end%for baseIter
    

    

