
%Stage 1: Load a -flow-vectorProject

flowAnalysisParameters = [];
perturbedProject = vectorProject;
cleanupFlag = 1;


%%Stage 2: Find the offshoots of plunging arterioles

pathList = vectorProject.vectorizedStructure.allPlungingArteriolePathList;
numPaths = numel(pathList);

dilationFraction = 1.2;
for plungingIter = 2:numPaths

perturbedProject = vectorProject;
dilationPath = pathList{plungingIter}
% dilationPath = dilationPath(1:ceil(numel(dilationPath)/2))


origRadiusList = [vectorProject.vectorizedStructure.effectiveStrandRadiusList(dilationPath)]
perturbedProject.vectorizedStructure.effectiveStrandRadiusList(dilationPath) = origRadiusList * dilationFraction;


%Stage 4: Calculate the perturbed flow surface vessel endpoints are held at fixed
%           voltages (100 and 50) using effect Strand Radii & true lengths


perturbedProject = CalculateUnperturbedFlow003(perturbedProject);
disp('Flow Calculation completed');


%Option 6: tesselate the zone of increased flow
origCurrentList = [vectorProject.vectorizedStructure.Strands(:).strandCurrent];
perturbedCurrentList = [perturbedProject.vectorizedStructure.Strands(:).strandCurrent];
currentRatioList = abs(perturbedCurrentList./origCurrentList);
maskSize = vectorProject.maskSize;
f3 = figure(3); colormap(gray);
set(f3,'DoubleBuffer','on');
clf

thresholdRatio = 1.1;


y1 = 0; y2 = inf;
x1 = 0; x2 = inf;
z1 = 0; z2 = inf;

dz = 100;
zoList = [dz:dz:maskSize-dz];
numZo = numel(zoList);
circleStack = false(maskSize(1),maskSize(2),numZo);
blobStack = false(maskSize(1),maskSize(2),numZo);
for zIter = 1:numZo
    zo = zoList(zIter);
    z1 = zo-dz;z2 = zo+dz;

    activeStrands = find(currentRatioList>thresholdRatio);
    activeIndices = [vectorProject.vectorizedStructure.Strands(activeStrands).StartToEndIndices];
    av = vectorProject.vectorizedStructure.Vertices.AllVerts;
    av = av(activeIndices,:);
    activeCoords = av(and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2)),:);


    myBlock = zeros(maskSize(1),maskSize(2),2*dz+1);
    blockSize = size(myBlock);
    blockCoords = activeCoords;
    blockCoords(:,3) = activeCoords(:,3) - z1 +1;
    blockIndices = sub2ind(blockSize,blockCoords(:,1),blockCoords(:,2),blockCoords(:,3));
    myBlock(blockIndices) = 1;

    myFrame = max(myBlock,[],3);
    filterSize = 39;%39 = 13*3;
    myFilter = double(circle2d(39));
    filtFrame = imfilter(myFrame,myFilter);
    figure(3);imagesc(filtFrame);


    filterCutoff = filterSize;
    threshFrame = filtFrame>filterCutoff;
    figure(3);imagesc(threshFrame);

    closeFrame = imclose(threshFrame,myFilter>0);
    figure(3);imagesc(closeFrame);

    bwFrame = bwlabeln(closeFrame>0);
    myProps = regionprops(bwFrame,'Area','Centroid');
    if isempty(myProps),
        fillFrame = false(size(myFrame));
        circleFrame = false(size(myFrame));
        disp('empty');
    else%if isempty(myProps),
        areaList = [myProps.Area];
        [maxArea,maxPos] = max(areaList);
        singleFrame = bwFrame == maxPos;
        myCentroid = round(myProps(maxPos).Centroid);
        myRadius = sqrt(maxArea/pi);
        figure(3);imagesc(singleFrame);

        fillFrame = imfill(singleFrame,'holes');
        figure(3);imagesc(fillFrame);

        circleFrame = false(size(myFrame));
        circleFrame(myCentroid(2),myCentroid(1)) = 1;
        circleFrame = bwdist(circleFrame);
        circleFrame = circleFrame<myRadius;
        figure(4);imagesc(circleFrame);colormap(gray);
        pause(0.1);
    end%
% % %     mask1Name = ['BlobMask_PlungingVessel',num2str(plungingIter,'%03d'),'_zRange',num2str(z1,'%04d'),'-',num2str(z2,'%04d'),'.tif'];
% % %     mask2Name = ['CircleMask_PlungingVessel',num2str(plungingIter,'%03d'),'_zRange',num2str(z1,'%04d'),'-',num2str(z2,'%04d'),'.tif'];
% % %     maketiff(fillFrame,mask1Name);
% % %     maketiff(circleFrame,mask2Name);
    blobStack(:,:,zIter) = fillFrame;
    circleStack(:,:,zIter) = circleFrame;
end%for zo = ...

mask1Name = ['BlobMask_PlungingVessel',num2str(plungingIter,'%03d'),'.tif'];
mask2Name = ['CircleMask_PlungingVessel',num2str(plungingIter,'%03d'),'.tif'];
maketiff(blobStack,mask1Name);
maketiff(circleStack,mask2Name);


end%for plungingIter...


details.dilationFraction = dilationFraction;
details.thresholdRatio = thresholdRatio;
details.filterSize = filterSize;
details.filterCutoff = filterCutoff;
details.dz = dz;
save('details.mat','details');    

