
%Stage 1: Load a -flow-vectorProject
strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;

flowAnalysisParameters = [];
cleanupFlag = 1;
%%
%%Stage 2: Run the unperturbed flow analysis:
unperturbedProject = vectorProject;
unperturbedProject.circuitAnalysisParameters.minRadius = 0;
unperturbedProject = CalculateUnperturbedFlow003(unperturbedProject);
disp('UnPerturbed Flow Calculation completed');


%%
%%Stage 3: Select a vessel segment to occlude
tipList = vectorProject.vectorizedStructure.plungingVenuleTipList;
numTips = numel(tipList);
stallMatrix = [];
stallIDMatrix = [];

%%
for myChoice = 1:numTips,
    disp(['TIP ITER ',num2str(myChoice)]);
    occlusionStrandID = tipList(myChoice)
    origRadius = vectorProject.vectorizedStructure.effectiveStrandRadiusList(occlusionStrandID)

    %Stage 4: Calculate the perturbed flow surface vessel endpoints are held at fixed
    %           voltages (100 and 50) using effect Strand Radii & true lengths

    perturbedProject = vectorProject;
    perturbedProject.circuitAnalysisParameters.minRadius = 0;
    perturbedProject.vectorizedStructure.effectiveStrandRadiusList(occlusionStrandID) = 0;
    perturbedProject = CalculateUnperturbedFlow003(perturbedProject);
    disp('Perturbed Flow Calculation completed');

    %Compare the two results
    origCurrentList = [unperturbedProject.vectorizedStructure.Strands(:).strandCurrent];
    perturbedCurrentList = [perturbedProject.vectorizedStructure.Strands(:).strandCurrent];
    currentRatioList = abs(perturbedCurrentList./origCurrentList);
    f1 = figure(1);
    set(f1,'DoubleBuffer','on');
    clf
    y1 = 0; y2 = inf;
    x1 = 0; x2 = inf;
    z1 = 0; z2 = inf;
    % zo = 700; dz = 100; z1 = zo-dz;z2 = zo+dz;

    %Plot decreased flow as as shades of blue
    ratioList = [0.5 0.4 0.3 0.2 0.1];
    colorMultiplier = 2;
    for ratioIter = 1:numel(ratioList),
        thisRatio = ratioList(ratioIter);
        thisColor = [thisRatio*colorMultiplier, thisRatio*colorMultiplier, 1];
        thisColor = min(1,thisColor);
        activeStrands = find(currentRatioList<thisRatio);
        activeIndices = [strands(activeStrands).StartToEndIndices];
        av = vertices.AllVerts;
        activeCoords = [vertices.AllVerts(activeIndices,:)];
        av = av(activeIndices,:);
        activeCoords = av(and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2)),:);
        p1 = plot3(activeCoords(:,1),activeCoords(:,2),-1*activeCoords(:,3),'b.');
        set(p1,'MarkerSize',1,'Color',thisColor);
        hold on
    end

    %Plot decreased flow as as shades of red
    ratioList = [1.5 1.6 1.7 1.8 1.9 2];
    colorMultiplier = 0.5;
    maxRatio = ratioList(end);
    ratioRange = ratioList(end)-ratioList(1);
    for ratioIter = 1:numel(ratioList),
        thisRatio = ratioList(ratioIter);
        thisColor = [1, (maxRatio-thisRatio)/ratioRange, (maxRatio-thisRatio)/ratioRange];
        thisColor = min(1,thisColor);
        activeStrands = find(currentRatioList>thisRatio);
        activeIndices = [strands(activeStrands).StartToEndIndices];
        av = vertices.AllVerts;
        activeCoords = [vertices.AllVerts(activeIndices,:)];
        av = av(activeIndices,:);
        activeCoords = av(and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2)),:);
        p1 = plot3(activeCoords(:,1),activeCoords(:,2),-1*activeCoords(:,3),'b.');
        set(p1,'MarkerSize',1,'Color',thisColor);
        hold on
    end


    occulsionIndices = [strands(occlusionStrandID).StartToEndIndices];
    occulsionCoords = [vertices.AllVerts(occulsionIndices,:)];
    av = vertices.AllVerts;
    av = av(occulsionIndices,:);
    occulsionCoords = av(and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2)),:);
    p1 = plot3(occulsionCoords(:,1),occulsionCoords(:,2),-1*occulsionCoords(:,3),'g.');
    set(p1,'MarkerSize',2);
    hold off


    %%Stage 7: Occlude stalled vessels

    stallRatioList = [0.25:0.05:0.75];%
    stopCountingNumStalls = 15000;%Hardcode
    numStallRatio = numel(stallRatioList);
    numRepeats = 50;
    numStalls = zeros(numStallRatio,numRepeats)-1;
    stallIDList = [];
    stopFlag = 0;
    for stallRatioIter = 1:numel(stallRatioList);
        stallRatio = stallRatioList(stallRatioIter);
        disp(['STALL RATIO : ',num2str(stallRatio)]);
        for repeatIter = 1:numRepeats,
            disp(['REPEAT # ',num2str(repeatIter)]);
            origCurrentList = [unperturbedProject.vectorizedStructure.Strands(:).strandCurrent];
            perturbedCurrentList = [perturbedProject.vectorizedStructure.Strands(:).strandCurrent];
            currentRatioList = abs(perturbedCurrentList./origCurrentList);
            stalledVesselList = find(currentRatioList<stallRatio);
            numStalls(stallRatioIter,repeatIter) = numel(stalledVesselList);
            stallIDList{stallRatioIter,repeatIter} = stalledVesselList;
            
            perturbedProject.vectorizedStructure.effectiveStrandRadiusList(stalledVesselList) = 0;
            perturbedProject = CalculateUnperturbedFlow003(perturbedProject);
            disp('Perturbed Flow Calculation completed');

            %Stage 7b: compare the two results
            origCurrentList = [unperturbedProject.vectorizedStructure.Strands(:).strandCurrent];
            perturbedCurrentList = [perturbedProject.vectorizedStructure.Strands(:).strandCurrent];
            currentRatioList = abs(perturbedCurrentList./origCurrentList);
            f1 = figure(1);
            set(f1,'DoubleBuffer','on');
            clf
            y1 = 0; y2 = inf;
            x1 = 0; x2 = inf;
            z1 = 0; z2 = inf;
            % zo = 700; dz = 100; z1 = zo-dz;z2 = zo+dz;
            %Plot decreased flow as as shades of blue
            ratioList = [0.5 0.4 0.3 0.2 0.1];
            colorMultiplier = 2;
            for ratioIter = 1:numel(ratioList),
                thisRatio = ratioList(ratioIter);
                thisColor = [thisRatio*colorMultiplier, thisRatio*colorMultiplier, 1];
                thisColor = min(1,thisColor);
                activeStrands = find(currentRatioList<thisRatio);
                activeIndices = [strands(activeStrands).StartToEndIndices];
                av = vertices.AllVerts;
                activeCoords = [vertices.AllVerts(activeIndices,:)];
                av = av(activeIndices,:);
                activeCoords = av(and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2)),:);
                p1 = plot3(activeCoords(:,1),activeCoords(:,2),-1*activeCoords(:,3),'b.');
                set(p1,'MarkerSize',1,'Color',thisColor);
                hold on
            end
            %Plot decreased flow as as shades of red
            ratioList = [1.5 1.6 1.7 1.8 1.9 2];
            colorMultiplier = 0.5;
            maxRatio = ratioList(end);
            ratioRange = ratioList(end)-ratioList(1);
            for colorRatioIter = 1:numel(ratioList),
                thisRatio = ratioList(colorRatioIter);
                thisColor = [1, (maxRatio-thisRatio)/ratioRange, (maxRatio-thisRatio)/ratioRange];
                thisColor = min(1,thisColor);
                activeStrands = find(currentRatioList>thisRatio);
                activeIndices = [strands(activeStrands).StartToEndIndices];
                av = vertices.AllVerts;
                activeCoords = [vertices.AllVerts(activeIndices,:)];
                av = av(activeIndices,:);
                activeCoords = av(and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2)),:);
                p1 = plot3(activeCoords(:,1),activeCoords(:,2),-1*activeCoords(:,3),'b.');
                set(p1,'MarkerSize',1,'Color',thisColor);
                hold on
            end%for colorRatioIter
            %---
            occulsionIndices = [strands(occlusionStrandID).StartToEndIndices];
            occulsionCoords = [vertices.AllVerts(occulsionIndices,:)];
            av = vertices.AllVerts;
            av = av(occulsionIndices,:);
            occulsionCoords = av(and(and(and(av(:,1)>y1,av(:,1)<y2),and(av(:,2)>x1,av(:,2)<x2)),and(av(:,3)>z1,av(:,3)<z2)),:);
            p1 = plot3(occulsionCoords(:,1),occulsionCoords(:,2),-1*occulsionCoords(:,3),'g.');
            set(p1,'MarkerSize',2);
            hold off

            if repeatIter >5,
                if numStalls(stallRatioIter,repeatIter) <= min(numStalls(stallRatioIter,repeatIter-3:repeatIter-1)),
                    if numStalls(stallRatioIter,repeatIter) > stopCountingNumStalls, stopFlag = 1; end
                    break%break out of repeatIter loop
                end%if numStalls...
            end%if repeatIter...
            figure(99);plot(numStalls');
        end%for repeatIter = ...
        if stopFlag == 1,
            break%break out of stallRatioIter loop
        end%if stopFlag == 1,
    end%for stallRatioIter...
    stallMatrix{myChoice} = numStalls;
    stallIDMatrix{myChoice} = stallIDList;

end%for myChoice = 

myParameters.generatedBy = 'IterateDamageFromSingleOcclusionVenule_Auto.m';
myParameters.circuitAnalysisParameters = unperturbedProject.circuitAnalysisParameters;
myParameters.stallRatioList = stallRatioList;
myParameters.stopCountingNumStalls = stopCountingNumStalls;
myParameters.numRepeats = numRepeats;

save IterateDamageFromSingleOcclusionVenuleResults.mat myParameters stallMatrix stallIDMatrix


%%
volumePerStrand = prod(maskSize)/numel(strands);
allCystDiameter = zeros(numTips,numStallRatio);
explodeVal = zeros(numTips,1);
for myChoice = 2:numTips,
    disp(['Tip # ',num2str(myChoice)]);
    stallCurves = stallMatrix{myChoice};
    allCurveMax = 0;
    for curveIter = 1:size(stallCurves,1),
        thisRatio = stallRatioList(curveIter);
        figure(77);plot(stallCurves(1:curveIter,:)');
        title(num2str(thisRatio));
        if max(stallCurves(curveIter,:)) >stopCountingNumStalls,
            explodeVal(myChoice) = thisRatio;
        end%if max(...)
        maxStalls = max(allCurveMax,max(stallCurves(curveIter,:)));
        allCurveMax = maxStalls;
        figure(78);plot(explodeVal,'g.');
        figure(79);plot(thisRatio,maxStalls,'r.');hold on
        
        cystVol = maxStalls * volumePerStrand;
        cystDiameter = 2*(cystVol*(0.75/pi))^0.333;
        figure(184);plot(thisRatio,cystDiameter,'m.');hold on
        allCystDiameter(myChoice,curveIter) = cystDiameter;
%         pause
        
    end
end
figure(79);hold off
    

for stallRatioIter = 1:numStallRatio,
    cystDiameterList = allCystDiameter(:,stallRatioIter);
    nonZeroList = cystDiameterList(cystDiameterList~=0);
    medianCystDiameter(stallRatioIter) = median(nonZeroList);
end
medianCystDiameter(isnan(medianCystDiameter)) = 0;
figure(184);plot(stallRatioList,medianCystDiameter,'r-');
figure(184);hold off

