%Calculate the xy-distance of each downstream branch to its nearest source
%   where source = plunging arteriole offshoot.
%Plot distribution and mean xy-distance at each branchLevel, across all
%   sources



%Open a ...-Flow-VectorProject.



%%
flowProject.maskSize = vectorProject.vectorizedStructure.maskSize;
flowProject.Strands = vectorProject.vectorizedStructure.Strands;
flowProject.Vertices = vectorProject.vectorizedStructure.Vertices;
flowProject.baseStrandIDList = vectorProject.vectorizedStructure.plungingArterioleTipList;
flowProject.maxBranchSearch = 33;
flowProject.stopVoltage = 50.1;%Hardcode;
flowProject.minimumFluxFraction = 1/(2^18);%Hardcode

flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
disp('Downstream Calculation Completed');



%%
%Calculate the minimum time of flight from an plungingArterioleTip;
%velocity = strandCurrent(Flux) / meanStrandXSectionalArea
%timeOfFlight = strandLength / velocity 

numStrands = numel(flowProject.Strands);
meanYXZList = zeros(numStrands,3);
for strandIter = 1:numStrands,
    thisStrand = flowProject.Strands(strandIter);
    YXZLimits = thisStrand.YXZLimits;
    meanYXZ = mean(YXZLimits,2);
    meanYXZList(strandIter,:) = meanYXZ';
end
timeMultiplier = 1e6;

tofList = zeros(numStrands,1)+inf;    
baseStrandIDList = flowProject.baseStrandIDList;
numBases = numel(baseStrandIDList);
for baseIter = 1:numBases,
    thisBaseStrandID = baseStrandIDList(baseIter);
    thisBaseStrand = flowProject.Strands(baseIter);
    dsbl = flowProject.downstreamBranchList(:,baseIter);
    thisPathList = flowProject.downstreamPathList{baseIter};
    numPaths = numel(thisPathList);
    for pathIter = 1:numPaths,
        thisPath = thisPathList{pathIter};
        currentList = abs([vectorProject.vectorizedStructure.Strands(thisPath).strandCurrent]');
        lengthList = vectorProject.vectorizedStructure.strandLengthList(thisPath);
        radiusList = vectorProject.vectorizedStructure.effectiveStrandRadiusList(thisPath);
        velocityList = currentList./(pi*radiusList.^2);
        timeList = timeMultiplier.*lengthList./velocityList;
        cumTimeList = cumsum(timeList);
        tempTOFList = tofList+inf;
        tempTOFList(thisPath) = cumTimeList;
        tofList = min(tofList,tempTOFList);
    end
    figure(1);plot(min(tofList,1000),'r.');
    pause(0.1)
end%for baseIter = 1:numBases,
disp('Time of Flight Calculation Completed');

    
%%
f1 = figure(1);
set(f1,'DoubleBuffer','on');
clf
flightTimeStep = 20;

activeStrands = find(tofList<inf);
activeIndices = [flowProject.Strands(activeStrands).StartToEndIndices];
av = flowProject.Vertices.AllVerts;
activeCoords = [flowProject.Vertices.AllVerts(activeIndices,:)];
zRange = [0:10:1200];
finalHist = hist(activeCoords(:,3),zRange);


for flightTime = 0:flightTimeStep:1100
    activeStrands = find(tofList<flightTime);
    activeIndices = [flowProject.Strands(activeStrands).StartToEndIndices];
    av = flowProject.Vertices.AllVerts;
    activeCoords = [flowProject.Vertices.AllVerts(activeIndices,:)];
    inactiveStrands = find(tofList>flightTime);
    inactiveIndices = [flowProject.Strands(inactiveStrands).StartToEndIndices];
    iav = flowProject.Vertices.AllVerts;
    iactiveCoords = [flowProject.Vertices.AllVerts(inactiveIndices,:)];
% %     figure(1);p1 = plot3(av(:,1),av(:,2),-1*av(:,3),'y.');hold on
% %     set(p1,'MarkerSize',1);
    figure(1);p1 = plot3(activeCoords(:,1),activeCoords(:,2),-1*activeCoords(:,3),'r.');
    title(num2str(flightTime));
    set(p1,'MarkerSize',1);
    hold off

    figure(2);p2 = plot3(iactiveCoords(:,1),iactiveCoords(:,2),-1*iactiveCoords(:,3),'g.');
    title(num2str(flightTime));
    set(p2,'MarkerSize',1);
    hold off
    
    zHist = hist(activeCoords(:,3),zRange);
    figure(3);plot(zRange,zHist./finalHist,'r-');

    pause
end









