%Load  a -flow-VectorProject.mat file and barrelBoundaryResults file

%%

vectorizedStructure = vectorProject.vectorizedStructure;
maskSize = vectorizedStructure.maskSize;
strands = vectorizedStructure.Strands;
vertices = vectorizedStructure.Vertices;
numVerts = numel(vertices.AllRadii);
numStrands = numel(strands);
allVerts = vertices.AllVerts;

allPlungingArterioleList = vectorizedStructure.allPlungingArterioleList;
allPlungingVenuleList = vectorizedStructure.allPlungingVenuleList;


%%

maxBranchSearch  = 18;

%%%Stage 2: Find the offshoots of plunging arterioles
flowProject.maskSize = vectorProject.vectorizedStructure.maskSize;
flowProject.Strands = vectorProject.vectorizedStructure.Strands;
flowProject.Vertices = vectorProject.vectorizedStructure.Vertices;
startList = vectorProject.vectorizedStructure.allPlungingArterioleList;
flowProject.baseStrandIDList = startList;
flowProject.maxBranchSearch = maxBranchSearch;
flowProject.stopVoltage = 50.1;%Hardcode;
flowProject.minimumFluxFraction = 1/(2^18);%Hardcode

disp('Calculating downstream paths...');
tic
flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
toc

downstreamBranchStrandList = [];
for kk = 1:maxBranchSearch,
    downstreamBranchStrandList{kk} = find(min(flowProject.downstreamBranchList,[],2)==kk);
end%for kk

%%

%%
vectorizedStructure = vectorProject.vectorizedStructure;
maskSize = vectorizedStructure.maskSize;
strands = vectorizedStructure.Strands;
vertices = vectorizedStructure.Vertices;
numVerts = numel(vertices.AllRadii);
numStrands = numel(strands);
allVerts = vertices.AllVerts;
subMaskSize = size(subMask);
%%
bufferZone = 000;
y1 = bufferZone+1; x1 = bufferZone+1;
y2 = maskSize(1)-bufferZone;
x2 = maskSize(2)-bufferZone;
subBufferZone = ceil(bufferZone./divRatio);
y1sub = subBufferZone+1; x1sub = subBufferZone+1;
y2sub = subMaskSize(1)-subBufferZone;
x2sub = subMaskSize(2)-subBufferZone;


croppedSubMask = subMask;
croppedSubMask(1:y1sub,:,:) = -128;
croppedSubMask(y2sub:end,:,:) = -128;
croppedSubMask(:,1:x1sub,:) = -128;
croppedSubMask(:,x2sub:end,:) = -128;
f1 = or(allVerts(:,1)<y1,allVerts(:,1)>y2);
f2 = or(allVerts(:,2)<x1,allVerts(:,2)>x2);
fAll = or(f1,f2);
croppedVertBarrelValueList = vertBarrelValueList;
croppedVertBarrelValueList(fAll) = -128;


numOutsideVoxels = sum(croppedSubMask(:)==-127);
numEdgeVoxels = sum(and(croppedSubMask(:)>-101,croppedSubMask(:)<0));
numInsideVoxels = sum(croppedSubMask(:)>0);
numNonCountVoxels = sum(croppedSubMask(:)==-128);


outsideVertIndexList = find(croppedVertBarrelValueList==-127);
edgeVertIndexList = find(and(croppedVertBarrelValueList>-126,croppedVertBarrelValueList<0));
insideVertIndexList = find(croppedVertBarrelValueList>0);

outsideVertDensity = numel(outsideVertIndexList)./numOutsideVoxels;
edgeVertDensity = numel(edgeVertIndexList)./numEdgeVoxels;
insideVertDensity = numel(insideVertIndexList)./numInsideVoxels;
disp('All Vessels:');
disp([1, edgeVertDensity./insideVertDensity,outsideVertDensity./insideVertDensity]);
%---
%%
ov = allVerts(outsideVertIndexList,:);
iv = allVerts(insideVertIndexList,:);
ev = allVerts(edgeVertIndexList,:);

figure(11);pov = plot3(ov(:,1),ov(:,2),-1*ov(:,3),'c.');hold on
figure(11);piv = plot3(iv(:,1),iv(:,2),-1*iv(:,3),'k.');hold on
figure(11);pev = plot3(ev(:,1),ev(:,2),-1*ev(:,3),'m.');hold off

%%
selectedOutsideDensity = [];
selectedEdgeDensity = [];
selectedInsideDensity = [];
for kk = 1:maxBranchSearch,

    thisStrandList = downstreamBranchStrandList{kk};



    thisSetVertList = [strands(thisStrandList).StartToEndIndices];
    selectedBVL = croppedVertBarrelValueList(thisSetVertList);



    selectedOutsideIndexList = find(selectedBVL==-127);
    selectedEdgeIndexList = find(and(selectedBVL>-126,selectedBVL<0));
    selectedInsideIndexList = find(selectedBVL>0);

    selectedOutsideDensity(kk) = numel(selectedOutsideIndexList)./numOutsideVoxels;
    selectedEdgeDensity(kk) = numel(selectedEdgeIndexList)./numEdgeVoxels;
    selectedInsideDensity(kk) = numel(selectedInsideIndexList)./numInsideVoxels;
    
    disp([1, selectedEdgeDensity(kk)./selectedInsideDensity(kk),selectedOutsideDensity(kk)./selectedInsideDensity(kk)]);

    lv = allVerts(thisSetVertList,:);
    figure(2);plv = plot3(lv(:,1),lv(:,2),-1*lv(:,3),'b.');hold off
end

selectedAllDensity = selectedOutsideDensity + selectedEdgeDensity + selectedInsideDensity;

figure(3);plot(selectedInsideDensity,'r-');hold on
plot(selectedEdgeDensity,'g-');
plot(selectedOutsideDensity,'b-');hold off

figure(4);plot(selectedInsideDensity./selectedAllDensity,'c-');hold on
figure(4);plot(selectedEdgeDensity./selectedAllDensity,'y-');hold on
figure(4);plot(selectedOutsideDensity./selectedAllDensity,'m-');hold off

figure(5);plot(selectedInsideDensity./insideVertDensity,'r-');hold on
figure(5);plot(selectedEdgeDensity./edgeVertDensity,'g-');hold on
figure(5);plot(selectedOutsideDensity./outsideVertDensity,'b-');hold off



save Results-VesselBarrelDistribByDownstreamBranch flowProject downstreamBranchStrandList
