%load the adjustedVectorProjectFile

rootFolder = 'F:\Phil\ForFromPeople\Joyce\CX13';
fileName = 'CX13-adjustedVectorProject.mat';
cd(rootFolder)
load(fileName);

%%
%find coordinates of arterioleVenule Junctions

vectorizedStructure = vectorProject.vectorizedStructure;
vertices = vectorizedStructure.Vertices;
strands = vectorizedStructure.Strands;
allVerts = vertices.AllVerts;
numStrands = numel(strands);
allLabels = vertices.AllLabels;
allRadii = vertices.AllRadii;
allConfidences = vertices.AllConfidences;
maskSize = vectorProject.maskSize;
strandIndices = full(vertices.StrandIndices);

medianLabelList = zeros(numStrands,1);
medianRadiusList = zeros(numStrands,1);
for strandIter = 1:numStrands;
    thisStrandID = strandIter;
    thisStrand = strands(thisStrandID);
    stei = thisStrand.StartToEndIndices;
    labelList = allLabels(stei);
    radiusList = allRadii(stei);
    medianLabelList(strandIter) = median(labelList);
    medianRadiusList(strandIter) = median(radiusList);
end

surfaceArteryList = find(ismember(medianLabelList,[330:339]));
surfaceVeinList = find(ismember(medianLabelList,[340:349]));

badConnectPointList = [];
for arteryIter = 1:numel(surfaceArteryList);
    thisArteryID = surfaceArteryList(arteryIter);
    thisArtery = strands(thisArteryID);
    endVertexNeighbors = thisArtery.EndVertexNeighborStrands;
    endVertexNeighborLabels = medianLabelList(endVertexNeighbors);
    startVertexNeighbors = thisArtery.StartVertexNeighborStrands;
    startVertexNeighborLabels = medianLabelList(startVertexNeighbors);
    badConnect = find(ismember(endVertexNeighborLabels,[340:349]));
    if ~isempty(badConnect),
        badConnectPointList = [badConnectPointList;thisArtery.EndVertexIndex];
    end%if ~isempty(badConnect);
    badConnect = find(ismember(startVertexNeighborLabels,[340:349]));
    if ~isempty(badConnect),
        badConnectPointList = [badConnectPointList;thisArtery.StartVertexIndex];
    end%if ~isempty(badConnect);
end

avJunctionCoords = allVerts(unique(badConnectPointList),:)
numAVJunctions = size(avJunctionCoords,1);


%%
%Generate the lists necessary for restrandify code:
%   pixelList, radiusList, confidenceList, labelList
pixelList = sub2ind(maskSize,allVerts(:,1),allVerts(:,2),allVerts(:,3));
radiusList = vertices.AllRadii;
labelList = vertices.AllLabels;
confidenceList = vertices.AllConfidences;
activeList = ones(size(radiusList));

%%
%For each AVJunction, disconnect and reconnect

deleteSize = 6;
addPixelList = [];
addRadiusList = [];
addLabelList = [];
addConfidenceList = [];
addActiveList = [];
addCounter = 0;

%In CX12 is a tri-junction, remove the artery.  (Verified visually)
avcc = avJunctionCoords(1,:);
thisPixelIndex = sub2ind(maskSize,avcc(1),avcc(2),avcc(3));
thisPixelNum = find(pixelList==thisPixelIndex);
attachedStrandList = strandIndices(thisPixelNum,:);
attachedStrandList = attachedStrandList(attachedStrandList~=0);
attachedStrandLabelList = medianLabelList(attachedStrandList);
isSupplyFlag = ismember(rem(attachedStrandLabelList,100),[30:39]);
isDrainFlag = ismember(rem(attachedStrandLabelList,100),[40:49]);
supplyStrandList = attachedStrandList(isSupplyFlag);
drainStrandList = attachedStrandList(isDrainFlag);
numAttached = numel(attachedStrandList);
numSupply = numel(supplyStrandList);
numDrain = numel(drainStrandList);


%Look at the two source strands (ss)
thisStrandID = supplyStrandList(1);
thisStrand = strands(thisStrandID);
ss1PointList = [];
if thisPixelNum == thisStrand.StartVertexIndex,
    ss1PointList = thisStrand.StartToEndIndices(2:deleteSize+1);
end
if thisPixelNum == thisStrand.EndVertexIndex,
    ss1PointList = fliplr(thisStrand.StartToEndIndices(end-deleteSize:end-1));
end
if isempty(ss1PointList),
    disp('ss1_Error'); pause; pause;
end

%Disconnect by inactivating points
allRemovalPoints = [ss1PointList(1:end-1)];
activeList(allRemovalPoints) = 0;



for avIter = 1:numAVJunctions,
    try
        avcc = avJunctionCoords(avIter,:);
        thisPixelIndex = sub2ind(maskSize,avcc(1),avcc(2),avcc(3));
        thisPixelNum = find(pixelList==thisPixelIndex);
        attachedStrandList = strandIndices(thisPixelNum,:);
        attachedStrandList = attachedStrandList(attachedStrandList~=0);
        attachedStrandLabelList = medianLabelList(attachedStrandList);
        isSupplyFlag = ismember(rem(attachedStrandLabelList,100),[30:39]);
        isDrainFlag = ismember(rem(attachedStrandLabelList,100),[40:49]);
        supplyStrandList = attachedStrandList(isSupplyFlag);
        drainStrandList = attachedStrandList(isDrainFlag);
        numAttached = numel(attachedStrandList);
        numSupply = numel(supplyStrandList);
        numDrain = numel(drainStrandList);
        disp(['Num of attached Strands : ',num2str([numAttached,numSupply,numDrain])]);

        %Verify that no other strands are within 10 pixels:
        blockcc = zeros((((2*deleteSize)+1)^3),3);
        counter = 0;
        for ii = -deleteSize:deleteSize,
            for jj = -deleteSize:deleteSize,
                for kk = -deleteSize:deleteSize,
                    qcc = avcc + [ii,jj,kk];
                    qcc = max(qcc,[0 0 0]);
                    qcc = min(qcc,maskSize);
                    counter = counter+1;
                    blockcc(counter,:) = qcc;
                end%for kk
            end%for jj
        end%for ii
        blockIndexList = sub2ind(maskSize,blockcc(:,1),blockcc(:,2),blockcc(:,3));
        neighborIndexList = find(ismember(pixelList,blockIndexList));
        neighborStrandList = unique(strandIndices(neighborIndexList,:));
        neighborStrandList = neighborStrandList(neighborStrandList~=0);
        matchedSetList = ismember(neighborStrandList,attachedStrandList);
        if ismember(0,matchedSetList),
            disp('Nearby Point!!!!');pause; pause;
        end%if ismember...

        %Look at the two source strands (ss)
        thisStrandID = supplyStrandList(1);
        thisStrand = strands(thisStrandID);
        ss1PointList = [];
        if thisPixelNum == thisStrand.StartVertexIndex,
            ss1PointList = thisStrand.StartToEndIndices(2:deleteSize+1);
        end
        if thisPixelNum == thisStrand.EndVertexIndex,
            ss1PointList = fliplr(thisStrand.StartToEndIndices(end-deleteSize:end-1));
        end
        if isempty(ss1PointList),
            disp('ss1_Error'); pause; pause;
        end
        ss1MedianZ = median(allVerts(ss1PointList,3));
        %---
        thisStrandID = supplyStrandList(2);
        thisStrand = strands(thisStrandID);
        ss2PointList = [];
        if thisPixelNum == thisStrand.StartVertexIndex,
            ss2PointList = thisStrand.StartToEndIndices(2:deleteSize+1);
        end
        if thisPixelNum == thisStrand.EndVertexIndex,
            ss2PointList = fliplr(thisStrand.StartToEndIndices(end-deleteSize:end-1));
        end
        if isempty(ss2PointList),
            disp('ss2_Error'); pause; pause;
        end
        ss2MedianZ = median(allVerts(ss2PointList,3));
        %---
        %Look at the two drain strands (ds)
        thisStrandID = drainStrandList(1);
        thisStrand = strands(thisStrandID);
        ds1PointList = [];
        if thisPixelNum == thisStrand.StartVertexIndex,
            ds1PointList = thisStrand.StartToEndIndices(2:deleteSize+1);
        end
        if thisPixelNum == thisStrand.EndVertexIndex,
            ds1PointList = fliplr(thisStrand.StartToEndIndices(end-deleteSize:end-1));
        end
        if isempty(ds1PointList),
            disp('ds1_Error'); pause; pause;
        end
        ds1MedianZ = median(allVerts(ds1PointList,3));
        %---
        thisStrandID = drainStrandList(2);
        thisStrand = strands(thisStrandID);
        ds2PointList = [];
        if thisPixelNum == thisStrand.StartVertexIndex,
            ds2PointList = thisStrand.StartToEndIndices(2:deleteSize+1);
        end
        if thisPixelNum == thisStrand.EndVertexIndex,
            ds2PointList = fliplr(thisStrand.StartToEndIndices(end-deleteSize:end-1));
        end
        if isempty(ds2PointList),
            disp('ds2_Error'); pause; pause;
        end
        ds2MedianZ = median(allVerts(ds2PointList,3));
        %---
        %---
        supplyMedianZ = mean([ss1MedianZ,ss2MedianZ]);
        supplyMedianR = mean([medianRadiusList(supplyStrandList)]);
        drainMedianZ = mean([ds1MedianZ,ds2MedianZ]);
        drainMedianR = mean([medianRadiusList(drainStrandList)]);

        %Disconnect by inactivating points
        allRemovalPoints = [ss1PointList(1:end-1),ss2PointList(1:end-1),ds1PointList(1:end-1),ds2PointList(1:end-1),thisPixelNum];
        ss1ConnectorPoint = ss1PointList(end);
        ss2ConnectorPoint = ss2PointList(end);
        ds1ConnectorPoint = ds1PointList(end);
        ds2ConnectorPoint = ds2PointList(end);
        ss1ConnectCC = allVerts(ss1ConnectorPoint,:);
        ss2ConnectCC = allVerts(ss2ConnectorPoint,:);
        ds1ConnectCC = allVerts(ds1ConnectorPoint,:);
        ds2ConnectCC = allVerts(ds2ConnectorPoint,:);

        activeList(allRemovalPoints) = 0;

        %Make two points, one 3 above, and one 3 below original junction
        if supplyMedianZ > drainMedianZ,
            supplyJointCC = avcc + [0 0 3];
            drainJointCC = avcc - [0 0 3];
        else%if supplyMedianZ...
            supplyJointCC = avcc - [0 0 3];
            drainJointCC = avcc + [0 0 3];
        end%if supplyMedianZ...

        %Add Lines
        for lineIter = 1:4,
            switch lineIter
                case 1
                    pointA = ss1ConnectCC;
                    pointB = supplyJointCC;
                    addRadius = medianRadiusList(supplyStrandList(1));
                    addLabel = medianLabelList(supplyStrandList(1));
                case 2
                    pointA = ss2ConnectCC;
                    pointB = supplyJointCC;
                    addRadius = medianRadiusList(supplyStrandList(2));
                    addLabel = medianLabelList(supplyStrandList(2));
                case 3
                    pointA = ds1ConnectCC;
                    pointB = drainJointCC;
                    addRadius = medianRadiusList(drainStrandList(1));
                    addLabel = medianLabelList(drainStrandList(1));
                case 4
                    pointA = ds2ConnectCC;
                    pointB = drainJointCC;
                    addRadius = medianRadiusList(drainStrandList(2));
                    addLabel = medianLabelList(drainStrandList(2));
            end%switch-case
            sep = pointA - pointB;
            maxSep = max(abs(sep));
            ySeq = round(linspace(pointA(1),pointB(1),maxSep+1));
            xSeq = round(linspace(pointA(2),pointB(2),maxSep+1));
            zSeq = round(linspace(pointA(3),pointB(3),maxSep+1));
            numAdd = numel(ySeq)
            pixelSeq = sub2ind(maskSize,ySeq,xSeq,zSeq);
            addPixelList(addCounter+1:addCounter+numAdd,1) = pixelSeq;
            addRadiusList(addCounter+1:addCounter+numAdd,1) = addRadius;
            addLabelList(addCounter+1:addCounter+numAdd,1) = addLabel;
            addConfidenceList(addCounter+1:addCounter+numAdd,1) = 1;
            addActiveList(addCounter+1:addCounter+numAdd,1) = 1;
            addCounter = addCounter+numAdd;


        end%for lineIter = 1:4,
    catch
        fprintf('\n avintersectin %d generated error %s',avIter,lasterr);
    end

end%for avIter = 1:numAVJunctions,



fullPixelList = [pixelList;addPixelList];
fullRadiusList = [radiusList;addRadiusList];
fullLabelList = [labelList;addLabelList];
fullConfidenceList = [confidenceList;addConfidenceList];
fullActiveList = [activeList;addActiveList];

newPixelList = fullPixelList(fullActiveList==1);
newRadiusList = fullRadiusList(fullActiveList==1);
newLabelList = fullLabelList(fullActiveList==1);
newConfidenceList = fullConfidenceList(fullActiveList==1);








%%

% % inputDataStruct.cpMatrix = double(inputDataStruct.cpMatrix);

% origVectorProject = inputDataStruct.origVectorProject;
% % cpMatrix = inputDataStruct.cpMatrix;
% % maskSize = inputDataStruct.maskSize;
% % allRadii = inputDataStruct.allRadii;
% % allLabels = inputDataStruct.allLabels;
% % allConfidences = inputDataStruct.allConfidences;
% % allNotes = inputDataStruct.allNotes;

% % % origVectorProjectRootName = inputDataStruct.fileRootName;
% % % origVectorProjectFolder = inputDataStruct.vectorProjectFolder;
% % % strandifyParameters = inputDataStruct.strandifyParameters;
% % % voxelSizeInMicrons = inputDataStruct.voxelSizeInMicrons;

% origVectorProject = inputDataStruct.origVectorProject;


% origVectorProjectRootName = inputDataStruct.fileRootName;
% origVectorProjectFolder = origVectorProject.vectorProjectFolder;
strandifyParameters = vectorProject.strandifyParameters;
% voxelSizeInMicrons = origVectorProject.voxelSizeInMicrons;



%-----
%cpMatrix:
%   cols1-3:coords, col4:endPointFlag, col5:junctionFlag, col6:activeFlag
%   col7:label



%Label Structure
%100's Labels : Topology   : 100=Unlabeled, 200=Temporary, 300=Surface, 400=Plunging, 500=Connecting
%10's Labels  : Physiology :  10=Unlabeled,  20=Temporary,  30=Supply ,  40=Draining,  50=Microvessel
%1's Labels   : WhenAdded  :   1=Original ,   2=TR Added ,   3=Manual



centerlineRadiusStructure = [];
% % activeList = cpMatrix(:,6) == 1;
% % pixelList = sub2ind(maskSize,cpMatrix(activeList,1),cpMatrix(activeList,2),cpMatrix(activeList,3));
% % radiusList = allRadii(activeList);
% % labelList = allLabels(activeList);
% % confidenceList = allConfidences(activeList);
% % % % noteList = allNotes(activeList);

%-----
centerlineRadiusStructure.pixelList = newPixelList;
centerlineRadiusStructure.maskSize = maskSize;
centerlineRadiusStructure.radiusList = newRadiusList;
centerlineRadiusStructure.labelList = newLabelList;
centerlineRadiusStructure.confidenceList = newConfidenceList;
% % centerlineRadiusStructure.noteList = noteList;
%-----
strandifyParameters.voxelSizeInMicrons = vectorProject.voxelSizeInMicrons;
registeredVascGrayscaleFileOnlyName = vectorProject.volumeFileNames.permutedVascGrayscale;
strandifyExp.registeredVascGrayscaleFileName = [vectorProject.vectorProjectFolder,filesep,'PermutedFilesFolder',filesep,registeredVascGrayscaleFileOnlyName];
strandifyExp.strandifyParameters = strandifyParameters;
strandifyExp.centerlineRadiusStructure = centerlineRadiusStructure;
strandifyExp = RestrandifyAndTransferLabels2(strandifyExp);
vectorizedStructure = strandifyExp.vectorizedStructure;
%-----
% vectorProject.note = origVectorProject.note;
% vectorProject.vectorProjectRootName = origVectorProject.vectorProjectRootName;
% vectorProject.vectorProjectFolder = origVectorProject.vectorProjectFolder;
% vectorProject.rootCodeFolder = origVectorProject.rootCodeFolder;
% vectorProject.maskSize = origVectorProject.maskSize;
% vectorProject.voxelSizeInMicrons = origVectorProject.voxelSizeInMicrons;
% vectorProject.progressFlags = origVectorProject.progressFlags;
%     vectorProject.vectorizedStructure = vectorizedStructure;
% vectorProject.cellCentroidAndLabelList = origVectorProject.cellCentroidAndLabelList;
% vectorProject.thresholdRelaxationParameters = origVectorProject.thresholdRelaxationParameters;
% vectorProject.strandifyParameters = origVectorProject.strandifyParameters;
% vectorProject.volumeFileNames = origVectorProject.volumeFileNames;
% % if isfield(origVectorProject,'GapConnectionStruct')
% %     vectorProject.GapConnectionStruct = origVectorProject.GapConnectionStruct;
% % else
% %     vectorProject.GapConnectionStruct = [];
% % end


vectorProject.vectorizedStructure = vectorizedStructure;

%-----
disp([' ';' ';' ']);
disp(['Completed Restrandifying SE2 globalDataStruct : ',datestr(now)]);
disp([' ';' ';' ']);
%----

save('AVRemovedAdjustedVectorProject.mat','vectorProject')







