%This script takes all the plunging artery tops and clots them one-by-one
%and compares the flow downstream to the non-clot flow.
%In this version, the main line of the plunging arteriole is identified
%and branch counting starts OFF of that line.

%Instructions:
%Load allProject

scriptVersion = 'AAB_ClotPlungingArterioleAdd4RadiusScript1a.m';
allProjectName = 'allProject03e_withVelocity';
%this version uses the adjustedRadius.
%this version plots median and median+/std

% 
for conditionNum = [1,4,5,16,17];
%condition 1 = effectiveStrandRadiusList;r
%condition 2 = effectiveStrandRadiusList + 2 (all radii add 2)g
%condition 3 = effectiveStrandRadiusList + 4 (all radii add 4)b
%condition 4 = subtract 1 from capillary radius only c
%condition 5 = add 1 to capillary radius only (min 1)y
%condition 6 = half capillary radius only m
%condition 7 = double capillary radius only k
%condition 8 = tenfold capillary radius only r
%condition 9 = set all capillary radius to 0.5 g
%condition 10 = set all capillary radius to 1 b
%condition 11 = set all capillary radius to 1.5 c 
%condition 12 = subtract 2 from capillary radius only y
%condition 13 = set all capillary radius to median radius m-
%condition 14 = set all radii to 3;k
%condition 15 = randomize cap radii r
%condition 16 = subtract 0.5 from capillary radius only g
%condition 17 = add 0.5 from capillary radius only b
%condition 18 = rat-like y-.
%condition 19 = missing 10% of cap bed
%condition 20 = missing 20% of cap bed
%condition 21 = rat-like plunging vessels.
%
%
vectorProject = allProject.co;

clear mega*


strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;
rootName = vectorProject.vectorProjectRootName;
findDash = findstr(rootName,'-');
shortName = rootName(1:findDash(1)-1);

strandMidYXZList = vectorProject.vectorizedStructure.strandMidYXZList;

strandLabelList = [strands(:).MedianLabel];
surfList = find(and(strandLabelList>=300,strandLabelList<=399)); 
zSurf = strandMidYXZList(surfList,3);
medianZSurf = median(zSurf);
minZSurf = min(zSurf);

strandMidYXZList(:,3) = max(strandMidYXZList(:,3)-minZSurf,0);
allPlungingVenuleList = vectorProject.vectorizedStructure.allPlungingVenuleList;
allPlungingArterioleList = vectorProject.vectorizedStructure.allPlungingArterioleList;

capFlagList = ~ismember(strandLabelList,[330:349,430:449]);
arterioleFlagList = ismember(strandLabelList,[330:339,430:439]);
venuleFlagList = ismember(strandLabelList,[340:349,440:449]);


flowAnalysisParameters = [];
cleanupFlag = 1;



switch conditionNum,
    case 1
        %do nothing, use effectiveStrandRadiusList
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\origAdjustedStrandRadius\';
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        graphTitle = 'orig effectiveStrandRadii';
    case 2
        %add 2 to all radii
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList +2 ;
        graphTitle = 'add 2 radius all';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\Add2AllRadius\';
    case 3
        %add 4 to all radii
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList +4 ;
        graphTitle = 'add 4 radius all';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\Add4AllRadius\';
    case 4
        %subtract 1 to from cap radii only
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = vectorProject.vectorizedStructure.adjustedStrandRadiusList(capFlagList) -1 ;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = max(0.5,vectorProject.vectorizedStructure.effectiveStrandRadiusList);
        graphTitle = 'subtract 1 radius capillary only';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\Sub1CapRadius\';
    case 5
        %add 1 to to cap radii only
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = vectorProject.vectorizedStructure.adjustedStrandRadiusList(capFlagList) + 1 ;
        graphTitle = 'add 1 radius capillary only';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\Add1CapRadius\';
    case 6
        %cut cap radii only by half
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = vectorProject.vectorizedStructure.adjustedStrandRadiusList(capFlagList) / 2 ;
        graphTitle = 'half radius capillary only';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\HalfCapRadius\';
    case 7
        %increase cap radii only by double
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = vectorProject.vectorizedStructure.adjustedStrandRadiusList(capFlagList) * 2 ;
        graphTitle = 'double radius capillary only';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\DoubleCapRadius\';
    case 8
        %increase cap radii only by tenfold
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = vectorProject.vectorizedStructure.adjustedStrandRadiusList(capFlagList) * 10 ;
        graphTitle = 'tenfold radius capillary only';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\TenfoldCapRadius\';
    case 9
        %set all cap radii to 0.5
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = 0.5;
        graphTitle = '0.5 radii capillary only';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\OneCapRadius0p5\';
    case 10
        %set all cap radii to 1
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = 1;
        graphTitle = '1.0 radii capillary only';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\OneCapRadius1p0\';
    case 11
        %set all cap radii to 1.5
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = 1.5;
        graphTitle = '1.5 radii capillary only';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\OneCapRadius1p5\';
     case 12
        %subtract 2 to from cap radii only
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = vectorProject.vectorizedStructure.adjustedStrandRadiusList(capFlagList) -2 ;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = max(1,vectorProject.vectorizedStructure.effectiveStrandRadiusList);
        graphTitle = 'subtract 2 radius capillary only';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\Sub2CapRadius\';
     case 13
        %set all cap to median cap radius
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = median(vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList));
        graphTitle = 'all median cap radius';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\AllMedianCapRadius\';
    case 14
        %set all radii to 3 (including surface and plunging)
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(:) = 3;
        graphTitle = 'all radii = 3';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\AllRadius3\';
    case 15
        %set all radii to 3 (including surface and plunging)
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        crl0 = vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList);%cap radius list
        crl1 = zeros(size(crl0));
        capList = find(capFlagList);
        for capIter = 1:numel(capList),
            capInd = ceil(rand(1)*numel(crl0));
            crl1(capIter) = crl0(capInd);
            crl0 = crl0([1:capInd-1,capInd+1:end]);
        end
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capList) = crl1;
        graphTitle = 'randomizeCapRadii';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\RandomizeCapRadii\';
    case 16
        %subtract 0.5 to from cap radii only
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = vectorProject.vectorizedStructure.adjustedStrandRadiusList(capFlagList) - 0.5 ;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = max(1,vectorProject.vectorizedStructure.effectiveStrandRadiusList);
        graphTitle = 'subtract 0.5 radius capillary only';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\Sub0p5CapRadius\';
    case 17
        %add 0.5 to to cap radii only
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = vectorProject.vectorizedStructure.adjustedStrandRadiusList(capFlagList) + 0.5 ;
        graphTitle = 'add 0.5 radius capillary only';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\Add0p5CapRadius\';
    case 18
        %make vessels rat-like
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = vectorProject.vectorizedStructure.adjustedStrandRadiusList(capFlagList) + 1.5 ;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(arterioleFlagList) = vectorProject.vectorizedStructure.effectiveStrandRadiusList(arterioleFlagList) *2;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(venuleFlagList) = vectorProject.vectorizedStructure.effectiveStrandRadiusList(venuleFlagList) *3;
        graphTitle = 'RatLike1';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\RatLike1\';
    case 19
        %remove 10% of capillaries
        capList = find(capFlagList);
        numCap = numel(capList);
        missingCapList = capList(round(rand(round(numCap*0.10),1)*numCap));
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(missingCapList) = 0.1;
         graphTitle = 'Missing10PercentCap';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\Missing10PercentCap\';
    case 20
        %remove 20% of capillaries
        capList = find(capFlagList);
        numCap = numel(capList);
        missingCapList = capList(round(rand(round(numCap*0.20),1)*numCap));
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(missingCapList) = 0.1;
         graphTitle = 'Missing20PercentCap';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\Missing20PercentCap\';
    case 21
        %make vessels rat-like
        vectorProject.vectorizedStructure.effectiveStrandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;
%         vectorProject.vectorizedStructure.effectiveStrandRadiusList(capFlagList) = vectorProject.vectorizedStructure.adjustedStrandRadiusList(capFlagList) + 1.5 ;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(arterioleFlagList) = vectorProject.vectorizedStructure.effectiveStrandRadiusList(arterioleFlagList) *2;
        vectorProject.vectorizedStructure.effectiveStrandRadiusList(venuleFlagList) = vectorProject.vectorizedStructure.effectiveStrandRadiusList(venuleFlagList) *3;
        graphTitle = 'RatLikePlunging';
        folderName = 'F:\Phil\VectorProjects\OcclusionFiles\RatLikePlunging\';
                         
end


%%Stage 2: Run the unperturbed flow analysis:
unperturbedFileName = [folderName,shortName,'-Unperturbed.mat'];
if exist(unperturbedFileName,'file'),
    load(unperturbedFileName,'origCurrentList');
    disp('UnPerturbed Flow Calculation loaded');
else%if exist...
    unperturbedProject = vectorProject;
    unperturbedProject.circuitAnalysisParameters.minRadius = 0;
    unperturbedProject.circuitAnalysisParameters.hematocrit = 0.45;
    unperturbedProject = CalculateUnperturbedFlow003(unperturbedProject);
    origCurrentList = [unperturbedProject.vectorizedStructure.Strands(:).strandCurrent];
    save(unperturbedFileName,'origCurrentList');
    disp('UnPerturbed Flow Calculation completed');
end%if exist...

clear tipList allPlungingArteriolePathList numTips



%%
%%Stage 3: Select a vessel segment to occlude
tipList = vectorProject.vectorizedStructure.plungingArterioleTipList;
allPlungingArteriolePathList = vectorProject.vectorizedStructure.allPlungingArteriolePathList;
numTips = numel(tipList);


zEndMustBeGreater = 200;
zEndMustBeLess = 9999;

subList = [];
for kk = 1:numel(allPlungingArteriolePathList);
    thisPath = allPlungingArteriolePathList{kk};
    if numel(thisPath)>0,
        thisZ = strandMidYXZList(thisPath,3);
          if thisZ(end)>zEndMustBeGreater && thisZ(end)<zEndMustBeLess,
            subList = [subList,kk];
        end%if thisPath(2)...
    end%if numel...
end%for kk
tipList = tipList(subList);
allPlungingArteriolePathList = allPlungingArteriolePathList(subList);
numTips = numel(tipList);


%Set limits on analysis field of view
y1 = 0; y2 = inf;
x1 = 0; x2 = inf;
z1 = 0; z2 = inf;

y1 = 100; y2 = maskSize(1)-y1;
x1 = 100; x2 = maskSize(2)-x1;
z1 = 000; z2 = 500;


analyzeDownstreamBranchesOnlyFlag = 1;
considerOnlyBranchesInFOVFlag = 1;
ignoreLowStartingCurrentStrandsFlag = 1;
excludeVeinsAndSurfaceFlag = 1;
stallLowFlowVesselFlag = 0;
stopTrackingOnceOutsideFlag = 1;
forceOutletClotFlag = 0;

stallNumFraction = 0.10;
maxBranch = 15;
numPlungingStrandToOcclude = 1;
outletLimit = 2; %consider at most, this number of outlets per occlusion arteriole
samplingLimit = 9999; 

f1 = and(strandMidYXZList(:,1)>y1,strandMidYXZList(:,1)<y2);
f2 = and(strandMidYXZList(:,2)>x1,strandMidYXZList(:,2)<x2);
f3 = and(strandMidYXZList(:,3)>z1,strandMidYXZList(:,3)<z2);
insideStrandList = find(and(and(f1,f2),f3));
outsideStrandList = find(~(and(and(f1,f2),f3)));
insideStrandCurrentList = abs(origCurrentList(insideStrandList));
sortList = sort(insideStrandCurrentList,'ascend');
thresholdCurrentForStall = sortList(ceil(stallNumFraction *numel(sortList)));


allTipMedianCurrentRatioList = [];
allRatioList =[]; allRatioList{maxBranch} = [];
allPreCurrentList =[]; allPreCurrentList{maxBranch} = [];
allPostCurrentList =[]; allPostCurrentList{maxBranch} = [];

f1 = figure(1); set(f1,'DoubleBuffer','on');clf
f2 = figure(2); set(f2,'DoubleBuffer','on');clf

tipCounter = 0;
allTipMedianCurrentRatioList = [];
allRatioList =[]; allRatioList{maxBranch} = [];
allPreCurrentList =[]; allPreCurrentList{maxBranch} = [];
allPostCurrentList =[]; allPostCurrentList{maxBranch} = [];
crossingPointList = [];
medianOffShootZList = [];
for tipIter = 1:1:numTips,
   
    disp(['TIP ITER ',num2str(tipIter)]);
    occlusionPath = allPlungingArteriolePathList{tipIter};
    myNumOcclude= min(numel(occlusionPath),numPlungingStrandToOcclude);
    occlusionStrandID =  occlusionPath(1:myNumOcclude)
    

    
    origRadius = vectorProject.vectorizedStructure.effectiveStrandRadiusList(occlusionStrandID);
    pathList = allPlungingArteriolePathList{tipIter};
    pathCoords = strandMidYXZList(pathList,:);
    %disp(pathCoords);
    maxPathCoords = max(pathCoords,[],1);
    minPathCoords = min(pathCoords,[],1);
    insideFlag = 1;
    
    if minPathCoords(1)<y1, insideFlag = 0;end
    if minPathCoords(2)<x1, insideFlag = 0;end
    if maxPathCoords(1)>y2, insideFlag = 0;end
    if maxPathCoords(2)>x2, insideFlag = 0;end
    
    %Consider branches off the clotted plunging arteriole line
    flowProject.maskSize = maskSize;
    flowProject.Strands = strands;
    flowProject.Vertices = vertices;
    flowProject.baseStrandIDList = pathList;
    flowProject.maxBranchSearch = 2;
    flowProject.stopVoltage = 50.1;%Hardcode;
    flowProject.minimumFluxFraction = 1/(2^18);%Hardcode
    disp('Finding arteriole outlets paths...');
    flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);    
    arterioleOffshootList = find(min(flowProject.downstreamBranchList,[],2)==1);
    numArterioleOffshoots = numel(arterioleOffshootList);
    downstreamBranchList = flowProject.downstreamBranchList;
    arterioleOffshootList = arterioleOffshootList(~ismember(arterioleOffshootList,vectorProject.vectorizedStructure.allPlungingArterioleList));
    offshootZ = strandMidYXZList(arterioleOffshootList,3);
    medianOffshootZ = median(offshootZ);
    arterioleOffshootList = arterioleOffshootList(and(offshootZ>z1,offshootZ<z2));
    if ~isempty(arterioleOffshootList),
        arterioleOffshootList = arterioleOffshootList(1:min(numel(arterioleOffshootList),outletLimit))
    end%if~isempty
    
    if forceOutletClotFlag == 1,
        occlusionStrandID = [occlusionStrandID,arterioleOffshootList'];
    end%if forceOutletClotFlag == 1,
    
    occlusionString = num2str(occlusionStrandID);
    occlusionString = regexprep(occlusionString,'     ','_');
    occlusionString = regexprep(occlusionString,'    ','_');
    occlusionString = regexprep(occlusionString,'   ','_');
    occlusionString = regexprep(occlusionString,'  ','_');
    occlusionString = regexprep(occlusionString,' ','_');
    occlusionString = [shortName,'-Clot-',occlusionString,'.mat'];
    occlusionFilePath = folderName;
    occlusionFileName = fullfile(occlusionFilePath,occlusionString);
    
    
    if isempty(arterioleOffshootList), insideFlag = 0; end
        if insideFlag == 1,
            perturbedProject = vectorProject;
            perturbedProject.circuitAnalysisParameters.minRadius = 0;
            perturbedProject.circuitAnalysisParameters.hematocrit = 0.45;
            perturbedProject.vectorizedStructure.effectiveStrandRadiusList(occlusionStrandID) = 0;
            
            
            if exist(occlusionFileName,'file'),
                disp('Perturbed Flow loading from file');pause(0.1);
                load(occlusionFileName,'perturbedCurrentList');
                %---
                clear downstreamBranchList
                q = load(occlusionFileName,'downstreamBranchList');
                if isfield(q,'downstreamBranchList'),
                    downstreamBranchList = q.downstreamBranchList;
                else,
                    flowProject.baseStrandIDList = arterioleOffshootList;
                    flowProject.maxBranchSearch = maxBranch;
                    disp('Calculating downstream paths');
                    tic
                    flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
                    toc
                    downstreamBranchList = flowProject.downstreamBranchList;
                    save(occlusionFileName,'perturbedCurrentList','downstreamBranchList');
                end%if isempty(q);
            else%if exist(occlusionFileName...
                perturbedProject = CalculateUnperturbedFlow003(perturbedProject);
                perturbedCurrentList = [perturbedProject.vectorizedStructure.Strands(:).strandCurrent];
                flowProject.baseStrandIDList = arterioleOffshootList;
                flowProject.maxBranchSearch = maxBranch;
                disp('Calculating downstream paths');pause(0.1);
                flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
                downstreamBranchList = flowProject.downstreamBranchList;
                save(occlusionFileName,'perturbedCurrentList','downstreamBranchList');
                disp('Perturbed Flow Calculation completed');pause(0.1);
            end%if-else exist(occlusionFileName...

                
            if stallLowFlowVesselFlag == 1,
                perturbedCurrentList(abs(perturbedCurrentList)<thresholdCurrentForStall) = 0;
            end%if stallLowFlowVesselFlag == 1,
            
           
            lowCurrentStrandList = find(abs(origCurrentList) < thresholdCurrentForStall);
            currentRatioList = abs(perturbedCurrentList./origCurrentList);


            switch analyzeDownstreamBranchesOnlyFlag,
                case 1 %analyze only downstream branches
% % %                     flowProject.baseStrandIDList = arterioleOffshootList;
% % %                     flowProject.maxBranchSearch = maxBranch;
% % %                     disp('Calculating downstream paths');
% % %                     flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
% % %                     downstreamBranchList = flowProject.downstreamBranchList;
                    offshootRatio = currentRatioList(arterioleOffshootList);
                    allRatioList{1} = [allRatioList{1},offshootRatio];
                    allPreCurrentList{1} = [allPreCurrentList{1},origCurrentList(arterioleOffshootList)];
                    allPostCurrentList{1} = [allPostCurrentList{1},perturbedCurrentList(arterioleOffshootList)];
                    
                    figure(1);plot(repmat(1,size(offshootRatio)),offshootRatio,'bo');hold on
                    figure(2);plot(repmat(1,size(offshootRatio)),offshootRatio,'bo');hold on
                    
                    
                    
                    branchClassList = [1:maxBranch];
                    medianCurrentRatioList = [];
                    medianCurrentRatioList(1) = median(offshootRatio);
                    for branchIter = 2:maxBranch,
                        findBranch = find(sum(downstreamBranchList==(branchIter-1),2)>0);
                        findBranch = findBranch(~ismember(findBranch,pathList));
                        if considerOnlyBranchesInFOVFlag == 1,
                            branchCoordList = strandMidYXZList(findBranch,:);
                            branchListZ = branchCoordList(:,3);
                            f1 = and(branchCoordList(:,1)>y1,branchCoordList(:,1)<y2);
                            f2 = and(branchCoordList(:,2)>x1,branchCoordList(:,2)<x2);
                            f3 = and(branchCoordList(:,3)>z1,branchCoordList(:,3)<z2);
                            ff = and(and(f1,f2),f3);
                            findBranch = findBranch(ff);
                        end%if considerOnly...
                        crl = currentRatioList(findBranch);
                        if ignoreLowStartingCurrentStrandsFlag == 1,
                            crl = crl(~ismember(crl,lowCurrentStrandList));
                        end%if ignoreLow...
                        allRatioList{branchIter} = [allRatioList{branchIter},crl];
                        allPreCurrentList{branchIter} = [allPreCurrentList{branchIter},origCurrentList(findBranch)];
                        allPostCurrentList{branchIter} = [allPostCurrentList{branchIter},perturbedCurrentList(findBranch)];

                        figure(1);p1 = plot(repmat(branchIter,size(crl)),crl,'bo');hold on
                        figure(2);p2 = plot(repmat(branchIter,size(crl)),crl,'bo');hold on
                        figure(3);p2 = plot(repmat(branchIter,size(branchListZ)),branchListZ,'go');
                        medianCurrentRatioList(branchIter) = median(crl);
                    end%for branchIter;
                    figure(1);plot(branchClassList,medianCurrentRatioList,'r-');
                    set(gca,'YLim',[0,2]);
                    figure(2);plot(branchClassList,medianCurrentRatioList,'r-');hold off
                    set(gca,'YLim',[0,2]);
                    
                case 0 %analyze all branches regardless of flow direction
% % % %                     flowProject.baseStrandIDList = arterioleOffshootList;
% % % %                     flowProject.maxBranchSearch = maxBranch;
% % % %                     flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);
% % % %                     downstreamBranchList = flowProject.downstreamBranchList;
                    offshootRatio = currentRatioList(arterioleOffshootList);
                    allRatioList{1} = [allRatioList{1},offshootRatio];
                    allPreCurrentList{1} = origCurrentList(arterioleOffshootList);
                    allPostCurrentList{1} = perturbedCurrentList(arterioleOffshootList);
                    
                    figure(1);p1 = plot(repmat(1,size(offshootRatio)),offshootRatio,'bo');hold on
                    figure(2);p2 = plot(repmat(1,size(offshootRatio)),offshootRatio,'bo');hold on
%                       
                    
                    alreadyUsedList = [pathList,arterioleOffshootList'];
                    if stopTrackingOnceOutsideFlag == 1,
                         alreadyUsedList = [alreadyUsedList,outsideStrandList'];
                    end%if stopTracking...
                    if excludeVeinsAndSurfaceFlag == 1,
                        alreadyUsedList = [alreadyUsedList,allPlungingVenuleList,surfList,allPlungingArterioleList];
                    end%if excludeVeinsAndSurfaceFlag == 1,
                    nextStrandList = [strands(arterioleOffshootList).StartVertexNeighborStrands,strands(arterioleOffshootList).EndVertexNeighborStrands];
                    nextStrandList = nextStrandList(~ismember(nextStrandList,alreadyUsedList));
                    branchClassList = [1:maxBranch];
                    medianCurrentRatioList = [];
                    medianCurrentRatioList(1) = median(offshootRatio);
                    for branchIter = 2:maxBranch,
                        findBranch = nextStrandList;
                        alreadyUsedList = [alreadyUsedList,findBranch];
                        if considerOnlyBranchesInFOVFlag == 1,
                            branchCoordList = strandMidYXZList(findBranch,:);
                            branchListZ = branchCoordList(:,3);
                            f1 = and(branchCoordList(:,1)>y1,branchCoordList(:,1)<y2);
                            f2 = and(branchCoordList(:,2)>x1,branchCoordList(:,2)<x2);
                            f3 = and(branchCoordList(:,3)>z1,branchCoordList(:,3)<z2);
                            ff = and(and(f1,f2),f3);
                            findBranch = findBranch(ff);
                        end%if considerOnly...
                        crl = currentRatioList(findBranch);
                        
% % %                          if sum(crl>5)>0,
% % %                             disp('Big Ratio!');
% % %                             pause
% % %                             pause
% % %                         end%if sum..
                        
                        
                        if ignoreLowStartingCurrentStrandsFlag == 1,
                            crl = crl(~ismember(findBranch,lowCurrentStrandList));
                        end%if ignoreLow...
         
                        
                        allRatioList{branchIter} = [allRatioList{branchIter},crl];
                        allPreCurrentList{branchIter} = [allPreCurrentList{branchIter},origCurrentList(findBranch)];
                        allPostCurrentList{branchIter} = [allPostCurrentList{branchIter},perturbedCurrentList(findBranch)];
                        
                        qq = numel(crl);qqq = min(3,qq);
                        crl2 = [crl(1:qqq),crl([2:100:end])];
                        figure(1);p1 = plot(repmat(branchIter,size(crl2)),crl2,'bo');hold on
                        figure(2);p2 = plot(repmat(branchIter,size(crl2)),crl2,'bo');hold on
%                         figure(3);p2 = plot(repmat(branchIter,size(branchListZ)),branchListZ,'go');hold off
                        nonNanCRL = crl(~isnan(crl));
                        medianCurrentRatioList(branchIter) = median(nonNanCRL);
                        %---
                        nextStrandList = [strands(nextStrandList).StartVertexNeighborStrands,strands(nextStrandList).EndVertexNeighborStrands];
                        nextStrandList = nextStrandList(~ismember(nextStrandList,alreadyUsedList));
                    
                        
                    end%for branchIter;
                    figure(1);plot(branchClassList,medianCurrentRatioList,'r-');
                    set(gca,'YLim',[0,2]);
                    figure(2);plot(branchClassList,medianCurrentRatioList,'r-');hold off
                    set(gca,'YLim',[0,2]);
                    
                    
                    
            end%switch-case
            tipCounter = tipCounter +1;
            allTipMedianCurrentRatioList(tipCounter,:) = medianCurrentRatioList;
            
            
            above75 = find(medianCurrentRatioList>0.75);
            if ~isempty(above75),
                if above75(1)>1,
                    q2 = above75(1);
                    q1 = above75(1)-1;
                    q3 = q1+((0.75-medianCurrentRatioList(q1))/(medianCurrentRatioList(q2)-medianCurrentRatioList(q1)));
                    crossingPointList(tipCounter) =  q3;
                else%if above75(1)>1
                    crossingPointList(tipCounter) = 1;
                end%if above75(1)>1
            else%if isempty(above75),
                crossingPointList(tipCounter) = maxBranch
            end%if isempty
            medianOffShootZList(tipCounter) = medianOffshootZ;
            figure(9);plot(medianOffShootZList,crossingPointList,'ro');

            figure(1);title(graphTitle);xlabel('% Baseline Flow'); ylabel('Median %baseline flow');
            pause(0.1);


    end%if insideFlag == 1,
end%for tipIter..

if ~exist('megaRatioList','var'),
    megaRatioList{maxBranch} = [];
    megaNameList = [];
end%if ~exist...

for kk = 1:maxBranch,
    q = allTipMedianCurrentRatioList(:,kk);
    q2 = q(~isnan(q));
    q2m = median(q2);
    q(isnan(q)) = q2m;
    allTipMedianCurrentRatioList(:,kk) = q;
end
medianOfMedianCurrentRatioList = median(allTipMedianCurrentRatioList,1);
figure(2);plot(branchClassList,medianOfMedianCurrentRatioList);

fullMedianList = [];
for branchIter = 1:maxBranch,
    q = allRatioList{branchIter};
    q2 = q(~isnan(q));
    q2m = median(q2);
    q(isnan(q)) = q2m;
    fullMedianList(branchIter) = median(q);
    megaRatioList{branchIter} = [megaRatioList{branchIter},q];
    megaRatioMedian(branchIter) = median(megaRatioList{branchIter});
    sList = sort(megaRatioList{branchIter});
    megaS1(branchIter) = sList(ceil(numel(sList)*0.25));
    megaS2(branchIter) = sList(ceil(numel(sList)*0.75));
    numChanged = 1;
    ratioList = megaRatioList{branchIter};
    while numChanged~=0,
        q1 = numel(ratioList);
        s = std(ratioList);
        f = find(ratioList<mean(ratioList)+4*s);
        ratioList = ratioList(f);
        q2 = numel(ratioList);
        numChanged = q1-q2;
    end%while
    megaRatioMedian(branchIter) = median(ratioList);
% %     megaS1(branchIter) = median(ratioList) + std(ratioList);
% %     megaS2(branchIter) = median(ratioList) - std(ratioList);
    megaS1(branchIter) = median(ratioList) + std(ratioList)./sqrt(numel(ratioList));
    megaS2(branchIter) = median(ratioList) - std(ratioList)./sqrt(numel(ratioList));
    
end%branchIter
megaNameList = [megaNameList,shortName,'_']
figure(3);plot(branchClassList,fullMedianList,'g-');
figure(3);title(graphTitle);xlabel('# branches downstream'); ylabel('Median %baseline flow');

b(1) = mean(fullMedianList(1:2));   
b(2) = mean(fullMedianList(3:4));   
b(3) = mean(fullMedianList(5:6));   
b(4) = mean(fullMedianList(7:10));   
b(5) = mean(fullMedianList(11:15));
figure(4);plot(b,'ro');
figure(4);title('Add 2 Radius');xlabel('NozomiPlot Bin#'); ylabel('Median %baseline flow');

figure(7);plot(megaRatioMedian,'mo');hold on
figure(7);plot(megaS1,'c*');hold on
figure(7);plot(megaS2,'c*');hold on

figure(7);p1 = plot([1.5,1.5],[0,0.10],'k-');hold on;set(p1,'LineWidth',7);
figure(7);p1 = plot([3.5,3.5],[0,0.25],'k-');hold on;set(p1,'LineWidth',7);
figure(7);p1 = plot([5.5,5.5],[0.35,1],'k-');hold on;set(p1,'LineWidth',7);
figure(7);p1 = plot([8.5,8.5],[0.15,1.15],'k-');hold on;set(p1,'LineWidth',7);
figure(7);p1 = plot([11.5,11.5],[0.55,1.2],'k-');hold on;set(p1,'LineWidth',7);
figure(7);plot([1.5,3.5,5.5,8.5,11.5],[0.03 0.08 0.55 0.80 0.85],'wo');hold off
figure(7);title(graphTitle);xlabel('# branches downstream'); ylabel('Median %baseline flow');


multiCondition(conditionNum).medianList = megaRatioMedian;
multiCondition(conditionNum).box25 = megaS1;
multiCondition(conditionNum).box75 = megaS2;





clotResultStruct = [];

clotResultStruct.scriptVersion = scriptVersion;
clotResultStruct.allProjectName = allProjectName;
clotResultStruct.analyzeDownstreamBranchesOnlyFlag = analyzeDownstreamBranchesOnlyFlag;
clotResultStruct.considerOnlyBranchesInFOVFlag = considerOnlyBranchesInFOVFlag;
clotResultStruct.ignoreLowStartingCurrentStrandsFlag = ignoreLowStartingCurrentStrandsFlag;
clotResultStruct.excludeVeinsAndSurfaceFlag = excludeVeinsAndSurfaceFlag;
clotResultStruct.stallLowFlowVesselFlag = stallLowFlowVesselFlag;
clotResultStruct.stopTrackingOnceOutsideFlag = stopTrackingOnceOutsideFlag;
clotResultStruct.forceOutletClotFlag = forceOutletClotFlag;

clotResultStruct.x1 = x1;
clotResultStruct.y1 = y1;
clotResultStruct.z1 = z1;
clotResultStruct.x2 = x2;
clotResultStruct.y2 = y2;
clotResultStruct.z2 = z2;
clotResultStruct.zEndMustBeGreater = zEndMustBeGreater;

clotResultStruct.stallNumFraction = stallNumFraction;
clotResultStruct.maxBranch = maxBranch;
clotResultStruct.numPlungingStrandToOcclude = numPlungingStrandToOcclude;
clotResultStruct.outletLimit = outletLimit; %consider at most, this number of outlets per occlusion arteriole
clotResultStruct.samplingLimit = samplingLimit; 
clotResultStruct.megaRatioList = megaRatioList;
clotResultStruct.megaNameList = megaNameList;
clotResultStruct.megaRatioMedian = megaRatioMedian;
clotResultStruct.megaS1 = megaS1;
clotResultStruct.megaS2 = megaS2;


multiCondition(conditionNum).medianList = megaRatioMedian;
multiCondition(conditionNum).box25 = megaS1;
multiCondition(conditionNum).box75 = megaS2;
multiCondition(conditionNum).clotResultStruct = clotResultStruct;

%%
% for conditionNum = [4,16,1,17,5]
% cs = 'rgbcymkrgbcymk';
cs = {'r-','g-','b-','c-','y-','m-','k-','r--','g--','b--','c--','y--','m--','k--','r-.','g-.','b-.','c-.','y-.','m-.','k-.'};
cs{4} = 'r-';
cs{16} = 'g-';
cs{1} = 'b-';
cs{17} = 'c-';
cs{5} = 'm-';
cs{18} = 'k-';


colorString = cs{conditionNum};
figure(3);hold on
plot(multiCondition(conditionNum).medianList,colorString);hold on
%---
% % % p1 = plot([1.5,1.5],[0,0.10],'k-');hold on;set(p1,'LineWidth',7);
% % % p1 = plot([3.5,3.5],[0,0.25],'k-');hold on;set(p1,'LineWidth',7);
% % % p1 = plot([5.5,5.5],[0.35,1],'k-');hold on;set(p1,'LineWidth',7);
% % % p1 = plot([8.5,8.5],[0.15,1.15],'k-');hold on;set(p1,'LineWidth',7);
% % % p1 = plot([11.5,11.5],[0.55,1.2],'k-');hold on;set(p1,'LineWidth',7);
% % % plot([1.5,3.5,5.5,8.5,11.5],[0.03 0.08 0.55 0.80 0.85],'wo')
% % % title(graphTitle);xlabel('# branches downstream'); ylabel('Median %baseline flow');


save multiConditionCO.mat multiCondition

clear mega*


end%for conditionNum =
%%
% % 
% % scriptVersion = clotResultStruct.scriptVersion;
% % analyzeDownstreamBranchesOnlyFlag = clotResultStruct.analyzeDownstreamBranchesOnlyFlag;
% % considerOnlyBranchesInFOVFlag = clotResultStruct.considerOnlyBranchesInFOVFlag;
% % ignoreLowStartingCurrentStrandsFlag = clotResultStruct.ignoreLowStartingCurrentStrandsFlag;
% % excludeVeinsAndSurfaceFlag = clotResultStruct.excludeVeinsAndSurfaceFlag;
% % stallLowFlowVesselFlag = clotResultStruct.stallLowFlowVesselFlag;
% % stopTrackingOnceOutsideFlag = clotResultStruct.stopTrackingOnceOutsideFlag;
% % forceOutletClotFlag = clotResultStruct.forceOutletClotFlag;
% % 
% % x1 = clotResultStruct.x1;
% % y1 = clotResultStruct.y1;
% % z1 = clotResultStruct.z1;
% % x2 = clotResultStruct.x2;
% % y2 = clotResultStruct.y2;
% % z2 = clotResultStruct.z2;
% % zEndMustBeGreater = clotResultStruct.zEndMustBeGreater;
% % 
% % stallNumFraction = clotResultStruct.stallNumFraction;
% % maxBranch = clotResultStruct.maxBranch;
% % numPlungingStrandToOcclude = clotResultStruct.numPlungingStrandToOcclude;
% % outletLimit = clotResultStruct.outletLimit; %consider at most, this number of outlets per occlusion arteriole
% % samplingLimit = clotResultStruct.samplingLimit; 
% % megaRatioList = clotResultStruct.megaRatioList;
% % megaNameList = clotResultStruct.megaNameList;
% % megaRatioMedian = clotResultStruct.megaRatioMedian;
% % megaS1 = clotResultStruct.megaS1;
% % megaS2 = clotResultStruct.megaS2;



