function outputStructure = MultiThreadChildVascOnlyVidaSuite(varargin)

%if input variable is a string, assume it is the name of the .m setup
%function file to run. - the strings "current" and "default" default to
%using the setup function: SuperVidaParamatersCurrent.m
%if input variable is a structure, assume the call is from the GUI
%and the setup parameters are stored in a .mat file
outputStructure = [];
nin = nargin;
if nin == 0,
    commandVariable = 'current';
else%if nin==0
    commandVariable = varargin{1};
end%if nin==0
%---
% % % if isstruct(commandVariable),
% % %     calledFromGuiFlag = 1;
% % %     setupMatFileName = commandVariable.setupMatFileName; 
% % %     parameterSetupFunctionName = 'current';
% % % else%if isstruct(commandVariable)
% % %     calledFromGuiFlag = 0;
% % %     parameterSetupFunctionName = commandVariable;
% % % end%if-else isstruct(commandVariable)
% % % %---

parameterSetupFunctionName = commandVariable;

switch parameterSetupFunctionName
    case {'default','Default','DEFAULT','def','Def','DEF'};
        parameterSetupFunctionName = 'DefaultClassicParameters.m';
    case {'current','CURRENT','Current'}
        parameterSetupFunctionName = 'DefaultClassicParameters.m';
    otherwise
        parameterSetupFunctionName = parameterSetupFunctionName;
end%switch parameterSetupFunctionName
%-----------
%Automatically detect rootCodeFolder and add setupFunctions
rootCodeFolder = fileparts(which('VIDA'));
addpath(fullfile(rootCodeFolder,'SetupFunctionsFolder'))
%-----
    outputProject = VidaAutoDectectVidaProjectRootFolder([]);
vidaProjectRootName = outputProject.vidaProjectRootName;
vidaProjectRootFolder = outputProject.vidaProjectRootFolder;
remoteVidaProjectRootFolder = vidaProjectRootFolder;
%----------
%SetupParameters
defaultOverwriteFilesStateFlag = 0;
vidaProjectStyleFlag = 1;
useTifStorageFlag = 0; % use variable imageStack in Mat file
skipAutoUpgradeFlag = 0;
debugModeAvoidOverwriteFlag = 1; %set to 1 to avoid overwriting all files experiment file is deleted

deleteAfterCompletedFlags.deleteOffsetTif = 1;
deleteAfterCompletedFlags.deleteSubBlockFiles = 1;
deleteAfterCompletedFlags.deleteVascBlockChannelSubFiles = 1;
deleteAfterCompletedFlags.deleteNeunBlockChannelSubFiles = 1;
deleteAfterCompletedFlags.deleteCellSeg = 1;
deleteAfterCompletedFlags.deleteVascSegFiles = 0;
deleteAfterCompletedFlags.deleteCenterlineFiles = 0;
deleteAfterCompletedFlags.deleteVascRadiiFiles = 0;

createTifFlags.createRawMaskTif = 0;
createTifFlags.createEnhMaskTif = 0;
createTifFlags.createCenterlineMaskTif = 0;

%-----

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp([' ';' ';' ']);
disp(['VIDA Script Started at ',datestr(now)]);
disp([' ';' ';' ']);
%--------------------------------------------------------------------------


vidaProject.vidaProjectRootName = vidaProjectRootName;
vidaProject.vidaProjectRootFolder = vidaProjectRootFolder;
vidaProject.rootCodeFolder = rootCodeFolder;
%---
vidaProject.progressFlags.initialized = 1;
vidaProject.progressFlags.sortOriginalTifs = 0;
vidaProject.progressFlags.createUniformityFilter = 0;
vidaProject.progressFlags.processImageStacksCompleted = 0;
vidaProject.progressFlags.multiStackProcessingCompleted = 0;


%Initialize the VidaProject file with folder names and setup Parameters
%This step initializes the vidaProject structure, but does NOT save to disk
vidaProject.setupParameters.vidaProjectRootName = vidaProjectRootName;
vidaProject.setupParameters.vidaProjectRootFolder = vidaProjectRootFolder;
vidaProject.setupParameters.parameterSetupFunctionName = parameterSetupFunctionName;
vidaProject.setupParameters.defaultOverwriteFilesStateFlag = defaultOverwriteFilesStateFlag;
vidaProject.setupParameters.useTifStorageFlag = useTifStorageFlag;
vidaProject.setupParameters.skipAutoUpgradeFlag = skipAutoUpgradeFlag;
vidaProject.setupParameters.defaultOverwriteFilesStateFlag = defaultOverwriteFilesStateFlag;
    vidaProject = VidaSetupVidaProjectFile(vidaProject);
revisedCodeFolders = vidaProject.setupParameters.revisedCodeFolders;
vidaProjectFileName = vidaProject.vidaProjectFileName;

%-----
experimentTemplate = vidaProject;
experimentTemplate.deleteAfterCompletedFlags = deleteAfterCompletedFlags;
experimentTemplate.createTifFlags = createTifFlags;
%-----

% experimentTemplate.globalParameters.storePixelListInExperimentFlag = 1;%noteToSelf
%-----
vidaProject.experimentTemplate = experimentTemplate;


%Add all code Folder to Matlab path
codeFolders = experimentTemplate.codeFolders;
codeFolderList = fieldnames(experimentTemplate.codeFolders);
numCodeFolders = size(codeFolderList,1);
for codeFolderIter = 1:numCodeFolders,
    currentField = codeFolderList{codeFolderIter};
    currentCodePath = experimentTemplate.codeFolders.(currentField);
    addpath(currentCodePath)
end
    



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-----
%STAGE 2: Wait until Parent process begins image stack processing, load updated VidaProject
disp('Pausing to check Parent process status : image stack processing');
imageStackProcessingFlag = 0;
while imageStackProcessingFlag == 0,
    try
        load(vidaProjectFileName,'vidaProject');
        check = vidaProject.multiThreadProgress.imageStackProcessingFlag;
    catch
        disp('Read error - will try again momentarily');
        pause(10);
    end%try-catch
    %---
    if isfield(vidaProject,'multiThreadProgress'),
        if vidaProject.multiThreadProgress.imageStackProcessingFlag > 0,
            imageStackProcessingFlag = 1;
            pause(10);
        end%if vidaProject.multiThreadProgress.imageStackProcessingFlag > 0,
    end%if isfield(vidaProject,'multiThreadProgress'),
end%while imageStackProcessingFlag == 0,        
imageStackFolderNameList = vidaProject.imageStackFolderNameList;
originalVidaProjectRootFolder = vidaProject.vidaProjectRootFolder;


%-----
%STAGE 2b: Modify the necessary local VidaProject parameters to reflect remote filename on another computer
multiThreadParentRunningFlagFileName = fullfile(remoteVidaProjectRootFolder,[vidaProjectRootName,'-MultiThreadParentRunningFlag.mat']);
multiThreadRequestFolder = fullfile(remoteVidaProjectRootFolder,'MultiThreadRequestFiles');
vidaProject.originalVidaProjectRootFolder = originalVidaProjectRootFolder;
vidaProject.remoteVidaProjectRootFolder = remoteVidaProjectRootFolder;
vidaProject.vidaProjectRootFolder = remoteVidaProjectRootFolder;
vidaProject.experimentTemplate.globalParameters.vidaProjectRootFolder = remoteVidaProjectRootFolder;
vidaProject.multiThreadProgress.multiThreadRequestFolder = multiThreadRequestFolder;
%---
pathLength = size(originalVidaProjectRootFolder,2);
numImageStacks = size(vidaProject.imageStackFolderNameList,2);
for stackIter = 1:numImageStacks,
    currentFolderName = vidaProject.imageStackFolderNameList{stackIter};
    vidaProject.imageStackFolderNameList{stackIter} = [remoteVidaProjectRootFolder,currentFolderName(pathLength+1:end)];
end%for stackIter = 1:numImageStacks,
%-----



%-----
%STAGE 4 : Check out an individual image stack, and run processing.
if vidaProject.multiThreadProgress.imageStackProcessingFlag == 2,
    fprintf('\n');
    disp('Processing of individual image stacks previously completed');
    fprintf('\n');
else%if vidaProject.multiThreadProgress.imageStackProcessingFlag == 2,  
    fprintf('\nBegin Processing of individual image stacks\n\n');vidaProject.multiThreadProgress.imageStackProcessingFlag = 1;
        vidaProject = VidaMultiThreadImageStackVascOnlyChild(vidaProject);
    vidaProject.multiThreadProgress.imageStackProcessingFlag = 2;
end%if vidaProject.multiThreadProgress.imageStackProcessingFlag == 2,




%-----
%STAGE 5: Wait until Parent process begins registration, load updated VidaProject
disp('Pausing to check parent process status : adjacent pair registering');
parentRegStartedFlag = 0;
while parentRegStartedFlag == 0,
    try
        load(vidaProjectFileName,'vidaProject');
        check = vidaProject.multiThreadProgress.registeringFlag;
    catch
        disp('Read error - will try again momentarily');
        pause(5);
    end%try-catch
    %---
        if vidaProject.multiThreadProgress.registeringFlag > 0,
            parentRegStartedFlag = 1;
            pause(10);
        end%if vidaProject.multiThreadProgress.registeringFlag > 0,
end%while parentRegStartedFlag == 0,        
imageStackFolderNameList = vidaProject.imageStackFolderNameList;
originalVidaProjectRootFolder = vidaProject.vidaProjectRootFolder;



%-----
%STAGE 5b: Modify the necessary local VidaProject parameters to reflect remote filename on another computer
multiThreadParentRunningFlagFileName = fullfile(remoteVidaProjectRootFolder,[vidaProjectRootName,'-MultiThreadParentRunningFlag.mat']);
multiThreadRequestFolder = fullfile(remoteVidaProjectRootFolder,'MultiThreadRequestFiles');
vidaProject.originalVidaProjectRootFolder = originalVidaProjectRootFolder;
vidaProject.remoteVidaProjectRootFolder = remoteVidaProjectRootFolder;
vidaProject.vidaProjectRootFolder = remoteVidaProjectRootFolder;
vidaProject.experimentTemplate.globalParameters.vidaProjectRootFolder = remoteVidaProjectRootFolder;
vidaProject.multiThreadProgress.multiThreadRequestFolder = multiThreadRequestFolder;
%---
pathLength = size(originalVidaProjectRootFolder,2);
numImageStacks = size(vidaProject.imageStackFolderNameList,2);
for stackIter = 1:numImageStacks,
    currentFolderName = vidaProject.imageStackFolderNameList{stackIter};
    vidaProject.imageStackFolderNameList{stackIter} = [remoteVidaProjectRootFolder,currentFolderName(pathLength+1:end)];
end%for stackIter = 1:numImageStacks,
%-----






%-----
%STAGE 7 : Register multiple stacks into single global coordinate sytem.
disp('Calculating Block Registrations');
if vidaProject.multiThreadProgress.registeringFlag == 2,
    fprintf('\nRegistration of adjacent pairs previously completed\n\n');
else%if vidaProject.multiThreadProgress.registeringFlag == 2,
    fprintf('\nBegin Registering of adjacent pairs\n\n');
    vidaProject.originalVidaProjectRootFolder = originalVidaProjectRootFolder;
    vidaProject.remoteVidaProjectRootFolder = remoteVidaProjectRootFolder;
        vidaProject = VidaRegisterBlocksChildSL(vidaProject);
    vidaProject.multiThreadProgress.registeringFlag = 2;
end%if-else vidaProject.multiThreadProgress.registeringFlag == 2,




registeredPathway = 'CenterlineFromStitchedMask';%Hardcode
%-----
%STAGE 8 : Choose a Multi-stack processing pathway, and run.
switch registeredPathway,
    case{'CenterlineFromStitchedMask'}
        vidaProject = VidaRegisteredProcessCenterlineChildSL(vidaProject);
end%switch multiStackPathway
saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save(vidaProjectFileName,'vidaProject',scs);



%-----
%FINISH
disp(['Vida Child-Process Completed at ',datestr(now)]);
outputStructure = [];





