function outputParameters = VidaNeunAnalyzeSubblock(inputParameters)

%updated 02/19/07 - now get cell area labels with a straight watershed
%   and matching to cell centroids

%Load SubBlocks
disp(['load subblock : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
%---
cellCentroidList = inputParameters.cellCentroidList;
subVolumeOverlapSize = inputParameters.subVolumeOverlapSize;
%-----
blockIter = inputParameters.blockIter;
blockGrid = inputParameters.blockGrid;
cellBlockFileName = inputParameters.cellBlockFileName;
neunBlockFileName = inputParameters.neunBlockFileName;
cellBlockMaskFileName = inputParameters.cellBlockMaskFileName;
vascBlockRawMaskFileName = inputParameters.vascBlockRawMaskFileName; 
neunResultsMatFileName = inputParameters.neunResultsMatFileName; 
fullStackSize = inputParameters.fullStackSize;
voxelSizeInMicrons = inputParameters.voxelSizeInMicrons;
%---
vascMaskFilterSize = inputParameters.vascMaskFilterSize;
mainCellDilation = inputParameters.mainCellDilation;
bkgShellDilation = inputParameters.bkgShellDilation;
otherCellDilation = inputParameters.otherCellDilation;
excludeVascFromCellFlag = inputParameters.excludeVascFromCellFlag; %0 = no, 1 = yes
excludeVascFromBkgFlag = inputParameters.excludeVascFromBkgFlag; %0 = no, 1 = yes
minCellAnalysisVolumeInVoxels = inputParameters.minCellAnalysisVolumeInVoxels;
minBkgAnalysisVolumeInVoxels = inputParameters.minBkgAnalysisVolumeInVoxels;
%-----
%-----
blockPos = blockGrid{blockIter};
yStartPos = blockPos.x1min;
xStartPos = blockPos.x2min;
zStartPos = blockPos.x3min;
yStopPos = blockPos.x1max;
xStopPos = blockPos.x2max;
zStopPos = blockPos.x3max;
%---
%Load the Neun subblock
loadedStruct = load(neunBlockFileName);
loadedVariables = fieldnames(loadedStruct);
if isfield(loadedStruct,'neunBlock'),
    neunBlock = loadedStruct.neunBlock;
else%if isfield(loadedStruct,'neunBlock'),
    neunBlock = loadedStruct.(loadedVariables{1});
end%if-else isfield(loadedStruct,'neunBlock'),
if isstruct(neunBlock),
    tempPixelList = neunBlock.pixelList;
    tempMaskSize = neunBlock.maskSize;
    neunBlock = false(tempMaskSize);
    neunBlock(tempPixelList) = 1;
    clear tempPixelList tempMaskSize
end%if isstruct(neunBlock),
%-----
loadedStruct = load(cellBlockFileName);
loadedVariables = fieldnames(loadedStruct);
cellBlock = loadedStruct.(loadedVariables{1});
if isstruct(cellBlock),
    tempPixelList = cellBlock.pixelList;
    tempMaskSize = cellBlock.maskSize;
    cellBlock = false(tempMaskSize);
    cellBlock(tempPixelList) = 1;
    clear tempPixelList tempMaskSize
end%if isstruct(cellBlock),
%-----
%Load Cell Mask
load(cellBlockMaskFileName,'cellBlockMask');
if isstruct(cellBlockMask),
    currentPixelList = cellBlockMask.pixelList;
    currentMaskSize = cellBlockMask.maskSize;
    cellBlockMask = false(currentMaskSize);
    cellBlockMask(currentPixelList) = 1;
end%if isstruct(cellBlockMask),
% % % %observationally, the CellMask seems to be shifted by one pixel
% % % %Fix this by shifting the CellMask back.
% % % [tempy,tempx,tempz] = size(CellMask);
% % % CellMask(1:tempy-1,1:tempx-1,1:tempz-1) = CellMask(2:tempy,2:tempx,2:tempz);
% % % %find centroids in the subregion only; note that as of 02/19/07, the 
% % % %GetCentroidsFromMask.m code has been changed so that centroids reflect
% % % %the SHIFTED mask (by -1,-1,-1).  The stored mask is the original, but seem
% % % %to be +1,+1,+1 compared to the raw stack images; hence the -1 shift. -Phil
%---

%Load Vasc Mask and dilate it
loadedStruct = load(vascBlockRawMaskFileName);
loadedVariables = fieldnames(loadedStruct);
vascBlockRawMask = loadedStruct.(loadedVariables{1});
if isstruct(vascBlockRawMask),
    tempPixelList = vascBlockRawMask.pixelList;
    tempMaskSize = vascBlockRawMask.maskSize;
    vascBlockRawMask = false(tempMaskSize);
    vascBlockRawMask(tempPixelList) = 1;
    clear tempPixelList tempMaskSize
end%if isstruct(ZZZZZ),
%-----

% % % if isstruct(vascBlockRawMask),
% % %     currentPixelList = vascBlockRawMask.pixelList;
% % %     currentMaskSize = vascBlockRawMask.maskSize;
% % %     vascBlockRawMask = false(currentMaskSize);
% % %     vascBlockRawMask(vascBlockRawMask) = 1;
% % % end%if isstruct(vascBlockCenterlineMask),
% % % %---
mycircle = circle3d(vascMaskFilterSize);
dilatedVascBlockMask = imfilter(vascBlockRawMask,double(mycircle))>0;
clear vascBlockRawMask;

%-----

maxDilate = 0;
maxDilate = max(max(mainCellDilation(2),bkgShellDilation(2)),otherCellDilation(2));

%-----
%Delineate Cell Regions
disp(['Delineate Cell Regions : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
%---
yInside = and(cellCentroidList(:,1)>=yStartPos , cellCentroidList(:,1)<=yStopPos);
xInside = and(cellCentroidList(:,2)>=xStartPos , cellCentroidList(:,2)<=xStopPos);
zInside = and(cellCentroidList(:,3)>=zStartPos , cellCentroidList(:,3)<=zStopPos);
allInside = (yInside .* xInside .* zInside)>0;
subCentroidList = cellCentroidList(allInside,:);
numCellsInSubblock = size(subCentroidList,1);
centroidPosInSubblock = subCentroidList-repmat([yStartPos-1,xStartPos-1,zStartPos-1],[numCellsInSubblock,1]);
centroidPosInSubblock = unique(centroidPosInSubblock,'rows');


%---
if numCellsInSubblock > 0,
    %Make a mask that has a bright dot at each centroid
    centroidDotMask = cellBlockMask.*0;
    centroidDotMask(sub2ind(size(centroidDotMask),centroidPosInSubblock(:,1),centroidPosInSubblock(:,2),centroidPosInSubblock(:,3))) = 1;

    %Determine which cell mask pixels are associated with which centroid
    [D,L] = bwdist(centroidDotMask,'quasi-euclidean');%quasi-euclidean is used for speed
    clear D
    uL = unique(L);
    LUT = zeros(1,max(uL));
    for jLUT=1:length(uL); LUT(uL(jLUT))=jLUT; end

    % % % %The next line is a cluge fix.  Very Rarely, a centroid falls on a dark mask pixel. -Phil 022707
    cellBlockMask(sub2ind(size(cellBlockMask),centroidPosInSubblock(:,1),centroidPosInSubblock(:,2),centroidPosInSubblock(:,3)))=1;
    enumeratedMask = reshape(LUT(L(:)),size(L)).*cellBlockMask;
    %---
    clear maskFilename CellMask new_CellMask HORDFilename NEUNFilename L uL LUT jLUT
    pause(0.1)

    %-----
    disp(['Cell-by-Cell NeuN Evaluation : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
    [ysize,xsize,zsize] = size(enumeratedMask);
    mycircle = double(circle3d(3));
    uniqueNumCells = size(centroidPosInSubblock,1);
    safetyMargin = (maxDilate+2);
    %---
    P = regionprops(enumeratedMask,'PixelIDxList');
    %---    


    
    fullY = fullStackSize(1);
    fullX = fullStackSize(2);
    fullZ = fullStackSize(3);
    subVolumeOverlapHalfSize = floor(subVolumeOverlapSize/2)-2;
    
    yStartPosSafe = blockPos.x1min;
    yStopPosSafe  = blockPos.x1max;
    xStartPosSafe = blockPos.x2min;
    xStopPosSafe  = blockPos.x2max;
    zStartPosSafe = blockPos.x3min;
    zStopPosSafe  = blockPos.x3max;
    
    if blockPos.x1min > 1, yStartPosSafe = yStartPos + subVolumeOverlapHalfSize; end
    if blockPos.x2min > 1, xStartPosSafe = xStartPos + subVolumeOverlapHalfSize; end
    if blockPos.x3min > 1, zStartPosSafe = zStartPos + subVolumeOverlapHalfSize; end
    if blockPos.x1max < fullY, yStopPosSafe = yStopPos - subVolumeOverlapHalfSize; end
    if blockPos.x2max < fullX, xStopPosSafe = xStopPos - subVolumeOverlapHalfSize; end
    if blockPos.x3max < fullZ, zStopPosSafe = zStopPos - subVolumeOverlapHalfSize; end
    
    %Find those centroid that are within the safe region of the block
    yInsideSafe = and(cellCentroidList(:,1)>=yStartPosSafe , cellCentroidList(:,1)<=yStopPosSafe);
    xInsideSafe = and(cellCentroidList(:,2)>=xStartPosSafe , cellCentroidList(:,2)<=xStopPosSafe);
    zInsideSafe = and(cellCentroidList(:,3)>=zStartPosSafe , cellCentroidList(:,3)<=zStopPosSafe);
    allInsideSafe = (yInsideSafe .* xInsideSafe .* zInsideSafe)>0;
    subCentroidSafeList = cellCentroidList(allInsideSafe,:);
    numCellsInSubblockSafe = size(subCentroidSafeList,1);
    centroidPosInSubblockSafe = subCentroidSafeList-repmat([yStartPos-1,xStartPos-1,zStartPos-1],[numCellsInSubblockSafe,1]);
    centroidPosInSubblockSafe = unique(centroidPosInSubblockSafe,'rows');
    %---
    uniqueNumCells = size(centroidPosInSubblockSafe,1);
    blank = zeros(1,uniqueNumCells);
        yCenterPos = blank;
        xCenterPos = blank;
        zCenterPos = blank;
        cell_size = blank;
        meanNeunChannelCenterVal = blank;
        meanNeunChannelBkgVal = blank;

    clear yInside xInside zInside allInside subCentroidList numCellsInSubBlock centroidPosInSubblock

        
    
    neunQuantifyResults.cellCoords = zeros(uniqueNumCells,3);
    neunQuantifyResults.maskSize = zeros(uniqueNumCells,1);
    neunQuantifyResults.meanCell = zeros(uniqueNumCells,1);
    neunQuantifyResults.stdCell = zeros(uniqueNumCells,1);
    neunQuantifyResults.medianCell = zeros(uniqueNumCells,1);
    neunQuantifyResults.meanBkg = zeros(uniqueNumCells,1);
    neunQuantifyResults.stdBkg = zeros(uniqueNumCells,1);
    neunQuantifyResults.medianBkg = zeros(uniqueNumCells,1);
    neunQuantifyResults.meanHORDCell = zeros(uniqueNumCells,1);
    neunQuantifyResults.meanHORDBkg = zeros(uniqueNumCells,1);

     for cellIter = 1:uniqueNumCells;
        cypos = centroidPosInSubblockSafe(cellIter,1);
        cxpos = centroidPosInSubblockSafe(cellIter,2);
        czpos = centroidPosInSubblockSafe(cellIter,3);
        whichCell = enumeratedMask(cypos,cxpos,czpos);
        mycellPos = P(whichCell).PixelIdxList;
        [ypos,xpos,zpos] = ind2sub(size(enumeratedMask),mycellPos);
        x1 = max(min(xpos)-safetyMargin,1);
        x2 = min(max(xpos)+safetyMargin,xsize);
        y1 = max(min(ypos)-safetyMargin,1);
        y2 = min(max(ypos)+safetyMargin,ysize);
        z1 = max(min(zpos)-safetyMargin,1);
        z2 = min(max(zpos)+safetyMargin,zsize);

        try
        subN = neunBlock(y1:y2,x1:x2,z1:z2);
        catch
            keyboard
        end


        subH = cellBlock(y1:y2,x1:x2,z1:z2);
        submask = enumeratedMask(y1:y2,x1:x2,z1:z2);
        vascSubMask = dilatedVascBlockMask(y1:y2,x1:x2,z1:z2);
        blankMask = false(size(submask)); 
        mainCellMask = blankMask;
        otherCellMask = blankMask;

        %Generate the appropriate Masks;
        mainCellMask(submask == whichCell) = 1;
        otherCellMask = (submask>0) - mainCellMask; %mask of all other cells in region

        
        
        
        %Create the appropriate mask for the main cell
        innerSize = mainCellDilation(1);
        innerSizeMinus99Flag = abs(innerSize*voxelSizeInMicrons + 99)<0.1;
        innerMask = mainCellMask;
        if innerSize<0, %erode
            if innerSizeMinus99Flag,
                innerMask = blankMask;
            else%if innerSizeMinus99Flag,
                mycircle  = circle3d( (2*abs(innerSize))+1 );
                innerMask = imerode(innerMask,mycircle);
            end%if innerSizeMinus99Flag,
        end%if innerSize<0, %erode
        if innerSize>0, %dilate
            mycircle  = circle3d( (2*abs(innerSize))+1 );
            innerMask = imdilate(innerMask,mycircle);
        end%if innerSize>0, %dilate     

        %---
        outerSize = mainCellDilation(2);
        outerMask = mainCellMask;
        if outerSize<0, %erode
            mycircle  = circle3d( (2*abs(outerSize))+1 );
            outerMask = imerode(outerMask,mycircle);
            if sum(outerMask)==0, %if eroded to nothing, try less erosion
                outerMask = mainCellMask;
                keepGoing = 1;
                tempMask = outerMask;
                while keepGoing, %stop just before going to zero pixels
                    outerMask = tempMask;
                    tempMask = imerode(outerMask,ones(3,3,3));    
                    if sum(outerMask(:))==0,keepGoing = 0;end
                end%while keepGoing,
            end%if sum(outerMask)==0, 
        end%if outerSize<0, %erode
        if outerSize>0, %dilate
            mycircle  = circle3d( (2*abs(outerSize))+1 );
            outerMask = imdilate(outerMask,mycircle);
        end%if outerSize>0, %dilate      
        %---
        mainCellRegion = (outerMask - innerMask)>0;

        %-----
        %-----
        %Create the appropriate mask for the background shell
        innerSize = bkgShellDilation(1);
        innerSize99Flag = abs(innerSize*voxelSizeInMicrons+99) < 0.1;%
        innerMask = mainCellMask;
        if innerSize<0, %erode
            if innerSizeMinus99Flag
                innerMask = blankMask;
            else%if innerSizeMinus99Flag
                mycircle  = circle3d( (2*abs(innerSize))+1 );
                innerMask = imerode(innerMask,mycircle);
            end%if innerSizeMinus99Flag
        end%if innerSize<0, %erode
        if innerSize>0, %dilate
            mycircle  = circle3d( (2*abs(innerSize))+1 );
            innerMask = imdilate(innerMask,mycircle);
        end%if innerSize>0, %dilate     
        %---
        outerSize = bkgShellDilation(2);
        outerMask = mainCellMask;
        if outerSize>0, %dilate
            if innerSize>0,
                outerMask = innerMask;
                diffSize = outerSize-innerSize;
                mycircle  = circle3d( (2*abs(diffSize))+1 );
                outerMask = imdilate(outerMask,mycircle);
            else%if innerSize>0,
                mycircle  = circle3d( (2*abs(outerSize))+1 );
                outerMask = imdilate(outerMask,mycircle);
            end%if innerSize>0,
        end%if outerSize>0, %dilate      
        %---
        bkgShellRegion = (outerMask - innerMask)>0;

        %-----
        %-----
        %Create the appropriate mask for the other cells
        innerSize = otherCellDilation(1);
        innerMask = otherCellMask;
        if innerSize<0, %erode
            if innerSizeMinus99Flag,
                innerMask = blankMask;
            else%if innerSizeMinus99Flag,
                mycircle  = circle3d( (2*abs(innerSize))+1 );
                innerMask = imerode(innerMask,mycircle);
            end%if innerSizeMinus99Flag,
        end%if innerSize<0, %erode
        if innerSize>0, %dilate
            mycircle  = circle3d( (2*abs(innerSize))+1 );
            innerMask = imdilate(innerMask,mycircle);
        end%if innerSize>0, %dilate     
        %---
        outerSize = otherCellDilation(2);
        outerMask = otherCellMask;
        if outerSize>0, %dilate
              mycircle  = double(circle3d(3));
              for k = 1:outerSize,
                  outerMask = imfilter(outerMask,mycircle)>0;
              end
        end%if outerSize>0, %dilate   
        %---
        otherCellRegion = (outerMask - innerMask)>0;
        %-----
        %-----



        %Exclude the "other cell" regions from the analysis regions
        cellAnalysisRegion = and(mainCellRegion,~otherCellRegion);
        bkgAnalysisRegion = and(bkgShellRegion,~otherCellRegion);

        if excludeVascFromCellFlag == 1,
            cellAnalysisRegion = and(cellAnalysisRegion,~vascSubMask);
        end%if excludeVascFromCell == 1,
        %---
        if excludeVascFromBkgFlag == 1,
            bkgAnalysisRegion = and(bkgAnalysisRegion,~vascSubMask);
        end%if excludeVascFromBkg == 1,
        %-----
        %-----
        %Make sure at least SOME region is chosen
        if sum(cellAnalysisRegion(:))<minCellAnalysisVolumeInVoxels,
            cellAnalysisRegion = mainCellRegion;
        end%if sum(cellAnalysisRegion(:))==0,

        if sum(bkgAnalysisRegion(:))<minBkgAnalysisVolumeInVoxels,
            bkgAnalysisRegion = bkgShellRegion;
        end%if sum(cellAnalysisRegion(:))==0,
        %---
        cellAnalysisVals = subN(cellAnalysisRegion>0);
        bkgAnalysisVals =  subN(bkgAnalysisRegion>0);
        HORDCellAnalysisVals = subH(mainCellRegion>0);
        HORDBkgAnalysisVals = subH(bkgAnalysisRegion>0);
        %---
        neunQuantifyResults.cellCoords(cellIter,:) = [cypos,cxpos,czpos];
        neunQuantifyResults.maskSize(cellIter) = sum(mainCellMask(:));
        neunQuantifyResults.meanCell(cellIter) = mean(cellAnalysisVals);
        neunQuantifyResults.stdCell(cellIter) = std(cellAnalysisVals);
        neunQuantifyResults.medianCell(cellIter) = median(cellAnalysisVals);
        neunQuantifyResults.meanBkg(cellIter) =  mean(bkgAnalysisVals);
        neunQuantifyResults.stdBkg(cellIter) = std(bkgAnalysisVals);
        neunQuantifyResults.medianBkg(cellIter) = median(bkgAnalysisVals);
        neunQuantifyResults.meanHORDCell(cellIter) = mean(HORDCellAnalysisVals);
        neunQuantifyResults.meanHORDBkg(cellIter) = mean(HORDBkgAnalysisVals);
    end%for cellIter = 1:uniqueNumCells;

else%if numCellsInSubblock > 0,
%     mydata = [1;1;1;1;2;1;1];
        neunQuantifyResults.cellCoords = [];
        neunQuantifyResults.maskSize = [];
        neunQuantifyResults.meanCell = [];
        neunQuantifyResults.stdCell = [];
        neunQuantifyResults.medianCell = [];
        neunQuantifyResults.meanBkg = [];
        neunQuantifyResults.stdBkg = [];
        neunQuantifyResults.medianBkg = [];
        neunQuantifyResults.meanHORDCell = [];
        neunQuantifyResults.meanHORDBkg = [];
end%if numCellsInSubblock > 0,
%-----
saveSize = (getfield(whos('neunQuantifyResults'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save(neunResultsMatFileName,'neunQuantifyResults','blockPos',scs);
pause(0.1);


outputParameters = inputParameters;
return


