function outputParameters = VidaNeuronRatioValueClassify(inputParameters)


compiledNeunResults=inputParameters.compiledNeunResults;
intermediateResultsFolder = inputParameters.intermediateResultsFolder;
neuronClassifyFilesFolder = inputParameters.neuronClassifyFilesFolder;
maxNumIterationsNeuronSegregation = inputParameters.maxNumIterationsNeuronSegregation;%10
minRatioRange = inputParameters.minRatioRange;
maxRatioRange = inputParameters.maxRatioRange;
ratioRangeStepSize = inputParameters.ratioRangeStepSize;
evalMetric = inputParameters.evalMetric;
%------
minRatioThresh = inputParameters.minRatioThresh;
maxRatioThresh = inputParameters.maxRatioThresh;
convergenceFraction = inputParameters.convergenceFraction;
ratioNeighborhoodXYInMicrons = inputParameters.ratioNeighborhoodXYInMicrons;
ratioNeighborhoodZInMicrons = inputParameters.ratioNeighborhoodZInMicrons;
deltaZInMicrons = inputParameters.deltaZInMicrons;
minNumCells = inputParameters.minNumCells;
voxelSizeInMicrons = inputParameters.voxelSizeInMicrons;
displayFlag = 0;
%-----
ratioNeighborhoodXY = round(ratioNeighborhoodXYInMicrons ./ voxelSizeInMicrons);
ratioNeighborhoodZ = round(ratioNeighborhoodZInMicrons ./ voxelSizeInMicrons);
deltaZ = round(deltaZInMicrons ./ voxelSizeInMicrons);
%-----
crossingPoint = 0.5; %initialize the crossingPoint (ratioThreshold)
threshList = [crossingPoint]; %ongoing list of local ratio thresholds
cellCoords = compiledNeunResults(1).cellCoords;
numCells = size(cellCoords,1); %total number of cells in dataset
maxCellCoords = max(cellCoords,[],1);
maxZ = maxCellCoords(3);
cellID = [1:numCells]';
emptyBlockFlag = 0;
xRange = [minRatioRange:ratioRangeStepSize:maxRatioRange];
%-----

%Sort cell by Z-position
compiledData = cat(2,cellCoords,cat(2,evalMetric,cellID));
[sortData,sortInd] = sortrows(compiledData,3);
cellCoords = sortData(:,1:3);
evalMetric = sortData(:,4);
clear compiledData sortData


%initialize output matrices
allRatios = ones(numCells,1)-10; %ratio of cell to neighbor neurons
allThresh = ones(numCells,1); %ratio threshold used for each cell
allLabels = ones(numCells,1)-10; %0/1 label for non-neuron/neuron
allDist = ones(numCells,1)+Inf; %track distance of each cell from the cell that defined the cluster.
    %   a cell may be evaluated multiple time if it is the neighbor of multiple
    %   "selected cells".  this distance will be used to keep the ratio & label
    %   for "closest" neighborhood for a cell.

cellID = [1:numCells]';
cellProgress = cellID<0; %initialize all to logical zero, set to 1 when a cell is evaluated.




cycleCounter = 0;
keepGoing = 1;


while keepGoing == 1,
    cycleCounter=cycleCounter+1;
    outOfBoundsCounter = 0;
    useMedianThresholdFlag = 0;
    notAnalyzed = cellID(~cellProgress);
    if ~isempty(notAnalyzed),
        outOfBoundsFlag = 0;
        currentCell = notAnalyzed(1);
        cz = cellCoords(currentCell,3);
        if cz<deltaZ, cz = deltaZ; end
        
        %In case there are sparse cells, expand deltaZ to include at least minNumCells
        tempDeltaZ = deltaZ;
        tempNumCells = 0;

        
        while tempNumCells < minNumCells,
            f1 = cellCoords(:,3) <= cz + tempDeltaZ;
            f2 = cellCoords(:,3) >= cz - tempDeltaZ;
            ff = and(f1,f2);
            tempNumCells = sum(ff);
            tempDeltaZ = tempDeltaZ + 1;
            if tempDeltaZ > maxZ,
                emptyBlockFlag = 1;
                break
            end%if tempDeltaZ > maxZ
        end%while tempNumCells < minNumCells,
        
        currentZDist = abs(cellCoords(:,3)-cz);
        subCellList = cellID(ff);
        subZDist = currentZDist(subCellList);
        numZSliceCells = length(subZDist);
        subCellID = [1:numZSliceCells]';
        neighborCoords = cellCoords(subCellList,:);
        numLocalCellsForThreshEval = numZSliceCells;
        
        
        % initialize Labels to be half Neurons to start
        subCellMetric = evalMetric(subCellList);
        medianMetric = median(subCellMetric);
        Labels = subCellMetric > medianMetric;
        Relabels=Labels;
        % preallocate ratios
        newRatio=zeros(numLocalCellsForThreshEval,1);
        splitPointHistory = zeros(maxNumIterationsNeuronSegregation,1);
        numNeuronsHistory = zeros(maxNumIterationsNeuronSegregation,1);
        numNonNeuronsHistory = zeros(maxNumIterationsNeuronSegregation,1);
        numChangedHistory = zeros(maxNumIterationsNeuronSegregation,1);
        convergenceNumChangedAllowed = ceil(numLocalCellsForThreshEval*convergenceFraction);


        for iterIND=1:maxNumIterationsNeuronSegregation
            mymessage = ['Segregation Iteration #',num2str(iterIND),' : ',datestr(now,'mm/dd/yy HH:MM:SS')];
            disp(mymessage);
            % Go through each cell and update measurements based on hidden labels
            for kCell = 1:numLocalCellsForThreshEval,
                %Take only the closest cells,
                thisCellID = subCellList(kCell);
                thisCellCoords = cellCoords(thisCellID,:);
                ty = thisCellCoords(1);
                tx = thisCellCoords(2);
                tz = thisCellCoords(3);
                flagY = and(neighborCoords(:,1)>ty-ratioNeighborhoodXY , neighborCoords(:,1)<ty+ratioNeighborhoodXY);
                flagX = and(neighborCoords(:,2)>tx-ratioNeighborhoodXY , neighborCoords(:,2)<tx+ratioNeighborhoodXY);
                flagZ = and(neighborCoords(:,3)>tz-ratioNeighborhoodZ , neighborCoords(:,3)<tz+ratioNeighborhoodZ);
                flagNear = and(flagY,and(flagX,flagZ));
                flagSelf = and( and( neighborCoords(:,1) == ty , neighborCoords(:,2) == tx) , neighborCoords(:,3) == tz);
                flagNeighbors = and(flagNear,~flagSelf);
                neighborID = subCellID(flagNeighbors);
                neighborMetric = subCellMetric(neighborID);
                
                neighborLabels = Labels(neighborID);
                % Get local means
                if sum(neighborLabels)>1
                    % Try using mean of only Neun
                    localNeuronOnlyMetric = mean(neighborMetric(neighborLabels));
                else%if sum(neighborLabels)>1
                    % Otherwise try using mean of all neighbors
                    localNeuronOnlyMetric=mean(neighborMetric(:));
                end%if sum(neighborLabels)>1

                currentMetric = evalMetric(thisCellID);
                % compute ratio metric
                if (localNeuronOnlyMetric ~= 0) && (~isnan(localNeuronOnlyMetric))     % make sure not to divide by zero
                    newRatio(kCell)=currentMetric/localNeuronOnlyMetric;
                else % if you do, set the Ratio to be zero (err on not classifiying as Neuron)
                    newRatio(kCell)=currentMetric./medianMetric;
                end%if (localNeuronOnlyMetric(kCell) ~= 0)&& (~isnan(localNeuronOnlyMetric))
 
                if currentMetric == 0,
                    newRatio(kCell) = xRange(1) - 1;
                end%if currentMetric == 0,
                
            end%for kCell=1:numLocalCellsForThreshEval
            

            ratioHistogram=histc(newRatio,xRange);
            
            normHistogram = ratioHistogram'./max(ratioHistogram);
            fitParameters.xRange = xRange;
            fitParameters.yData = normHistogram;
            fitParameters = fit_2_gaussians(fitParameters);
            yFit1 = fitParameters.yFit1;
            yFit2 = fitParameters.yFit2;
            peakX1 = xRange(yFit1==max(yFit1));
            peakX2 = xRange(yFit2==max(yFit2));
            peakX1 = peakX1(1);
            peakX2 = peakX2(1);
            leftPeakX = min(peakX1,peakX2);
            rightPeakX = max(peakX1,peakX2);
            

            
            xPosLeft = find(abs(xRange-leftPeakX) == min(abs(xRange-leftPeakX)));
            xPosRight = find(abs(xRange-rightPeakX) == min(abs(xRange-rightPeakX)));

            xRangeMiddle = xRange(xPosLeft:xPosRight);
            yDiff = abs(yFit1-yFit2);
            yDiffMiddle = yDiff(xPosLeft:xPosRight);
            yDiffMinPos = find(yDiffMiddle == min(yDiffMiddle));
            tempCrossingPoint = xRangeMiddle(yDiffMinPos(1));
            outOfBoundsFlag = 0;
            if tempCrossingPoint==xRangeMiddle(1), outOfBoundsFlag =1; end;
            if tempCrossingPoint==xRangeMiddle(end), outOfBoundsFlag =1; end;
            if tempCrossingPoint<minRatioThresh, outOfBoundsFlag =1; end;   
            if tempCrossingPoint>maxRatioThresh, outOfBoundsFlag =1; end;   
            
            if displayFlag == 1,
                figure(1); plot(xRange,normHistogram,'b-'); hold on
                figure(1); plot(xRange,yFit1,'r-');
                figure(1); plot(xRange,yFit2,'r-');
                figure(1); plot(xRange,yFit1+yFit2,'k-');hold off
            end%if displayFlag == 1,
            
            
            if outOfBoundsFlag==1,
%                 keyboard
                tempCrossingPoint = median(threshList);
                disp(['using Median CrossingPoint : ',num2str(crossingPoint)])
                outOfBoundsCounter = outOfBoundsCounter + 1
                if outOfBoundsCounter>3,
                    useMedianThresholdFlag = 1;
                    break
                end
            end
                
                
            if iterIND == 1,
                crossingPoint = (crossingPoint*2 + tempCrossingPoint)/3;
            else%if iterIND == 1,
                %exponential history with extra weight on current reading
                crossingPoint = (crossingPoint.*(iterIND-1) + tempCrossingPoint*3 )./(iterIND+2);
%                 crossingPoint = (crossingPoint*2 + tempCrossingPoint )./3;
            end%if iterIND == 1,

    

            %-----
            RelabelSplitPoint=crossingPoint;
            % figure;clf;plot(xsrange(2:end-1),ratioHistogram);hold

            % Keep track of split point history
            splitPointHistory(iterIND)=RelabelSplitPoint;


            
            if displayFlag == 1,
                f = figure(14);
                cutLine = ratioHistogram'.*0 + 10;
                crossingPointPos = find(abs(xRange-crossingPoint) == min(abs(xRange-crossingPoint)));
                cutLine(1:crossingPointPos) = 0;
                plot(xRange,ratioHistogram','r-');hold on
                plot(xRange,cutLine,'k-');hold off
        %         saveas(f,fullfile(neuronClassifyFilesFolder,['RatioCurve_Iteration_',num2str(iterIND),'.fig']));
                pause(0.01);
            end%if displayFlag == 1,

  
            % update all labels
            Relabels=newRatio>RelabelSplitPoint;
            numChangedLabels = sum(abs(Relabels-Labels));
            numNeuronsHistory(iterIND) = sum(Relabels);
            numNonNeuronsHistory(iterIND) = numLocalCellsForThreshEval - sum(Relabels);
            numChangedHistory(iterIND) = numChangedLabels;
            Labels = Relabels;

% %             mydisplay = [splitPointHistory(iterIND);numNeuronsHistory(iterIND)/1000;numNonNeuronsHistory(iterIND)/1000;numChangedHistory(iterIND)/1000];
% %             disp('splitPointHistory ; numNeuronsHistory *1k  ; numNonNeuronsHistory *1k ; numChangedLabels * 1k');
% %             disp(mydisplay);

            if numChangedLabels <= convergenceNumChangedAllowed,
                disp('fit converged...');
                break;
            end%if numChangedLabels <= convergenceNumChangedAllowed,

        end%for iterIND = 1:maxNumIterationsNeuronSegregation

        
        
        
        
        
        if iterIND==maxNumIterationsNeuronSegregation
            useMedianThresholdFlag = 1;
            disp(['NO CONVERGENCE - DEFAULTING TO MEDIAN CROSSING POINT : ',num2str(crossingPoint)])
        end%if iterIND==maxNumIterationsNeuronSegregation

        
        
        
        
        %if crossingPoint went out of bounds, set confidence of estimate to
        %zero by putting distance to infinity.
        if useMedianThresholdFlag==1,
            crossingPoint = median(threshList);
            currentZDist(:,2) = 1e12;
            disp(['DEFAULTING TO MEDIAN CROSSING POINT : ',num2str(crossingPoint)])
                     
            
            % update all labels
            RelabelSplitPoint = crossingPoint;
            Relabels=newRatio>RelabelSplitPoint;
            Labels = Relabels;
            
            
            if displayFlag == 1,
                f = figure(14);
                cutLine = ratioHistogram'.*0 + 10;
                crossingPointPos = find(abs(xRange-crossingPoint) == min(abs(xRange-crossingPoint)));
                cutLine(1:crossingPointPos) = 0;
                plot(xRange,ratioHistogram','r-');hold on
                plot(xRange,cutLine,'k-');hold off
            end%if displayFlag == 1,
           
           
           
        end%if outOfBoundsFlag==1,

        
        
        
        
        
        threshList = cat(2,threshList,crossingPoint);
        cellProgress(subCellList) = 1;
        %Replace the Label only if cell is close to middle of block than in
        %previous blocks
        for cellIter = 1:numLocalCellsForThreshEval,
            thisCellID = subCellList(cellIter);
            storedDist = allDist(thisCellID);
            thisDist = subZDist(cellIter);
            if thisDist<=storedDist,
                allLabels(thisCellID) = Labels(cellIter);
                allRatios(thisCellID) = newRatio(cellIter);
                allDist(thisCellID) = thisDist;
                allThresh(thisCellID) = crossingPoint;
            end%if thisDist<storedDist,
        end%for cellIter = 1:numLocalCellsForThreshEval,
    

thisSliceRatio = sum(Labels)./length(Labels);


disp(['Crossing Point: ',num2str(crossingPoint)])
disp(['This Slice Ratio: ',num2str(thisSliceRatio)]);
disp(['This Slice Cells: ',num2str(numZSliceCells)]);
disp(['Cells Completed: ',num2str(sum(cellProgress(:)))]);
neuronRatio = sum(allLabels(cellProgress))./sum(cellProgress);
disp(['Cumulative NeuronRatio: ',num2str(neuronRatio)]);
pause(0.5)

% % %     if mod(cycleCounter,50)==0,
% % %         saveSize = (getfield(whos('neuronRatio'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
% % %         save('ClassifyNeuronProgressFile.mat',scs);
% % %     end%if mod(cycleCounter,25)==0,

    else%if ~isempty(notAnalyzed),
        keepGoing = 0;
    end%if ~isempty(notAnalyzed),
    
    
    
    
end%while keepGoing == 1,



distFromSplit = abs(allRatios-allThresh);
neuronScore = 2*(allRatios>allThresh)-1;
neuronScore = 2 * neuronScore.*distFromSplit;

allRatios(sortInd) = allRatios;
allThresh(sortInd) = allThresh;
neuronScore(sortInd) = neuronScore;

outputParameters = inputParameters;
outputParameters.newRatio = allRatios;
outputParameters.crossingPoint = allThresh;
outputParameters.neuronScore = neuronScore;


% close all



