function outExp = DefaultClassicParameters(inExp)

experimentTemplate = inExp;


%FUNCTION-SPECIFIC PARAMETERS
%------------------------------------------------------------------------
%BASIC PARAMETERS          
%voxelSizeInMicrons: the pixel (voxel) size for the data. 2.0 means 2 micrometers per voxel; 0.5 means half micron per pixel
%basicParameters.numChannels : num of spectral channels (neurons, cells, vessels) in the data
basicParameters.voxelSizeInMicrons = 1.0;
basicParameters.numChannels = 3;
experimentTemplate.basicParameters = basicParameters;
%------------------------------------------------------------------------
%PREPROCESSING PARAMETERS
%fluoresceinGaussianFilterSizeIn Microns : 1/e-radius of gaussian filter for uniformity bath data file  
%filePrefix : set of identifier strings for the original .tif file channels -ChX.tif suffix. {'NEUN','HORD','FITC'}  
preProcessingParameters.fluoresceinGaussianFilterSizeInMicrons = 0; 
preProcessingParameters.filePrefix = {'NEUN','HORD','FITC'}; %in order of Ch1, Ch2, Ch3
experimentTemplate.preProcessingParameters = preProcessingParameters;
%--------------------------------------------------------------------------
%SUBBLOCK PARAMETERS
%subVolumeSize3DInMicrons : break into subblock of this maximum dimension [in microns].  Actual sub-blocks may be smaller to avoid oblong sub-blocks        
%subVolumeOverlapSizeInMicrons : desired overlap size between neighboring sub-blocks [in microns]    
subBlockParameters.subVolumeSize3DInMicrons = 600;
subBlockParameters.subVolumeOverlapSizeInMicrons = 40;
experimentTemplate.subBlockParameters = subBlockParameters;
%------------------------------------------------------------------------
%CELL SEGMENTATION PARAMETERS
%cellMatchFilterModelVarianceInMicronsSquard : Variance (stdev-squared) of gaussian used for cell-matched filter [in microns-squared].  9um2 = 3um 1/e radius     
%cellMatchFilterBackgroundVarianceInMicronsSquared : Variance (stdev-squared) of gaussian used for background filter [in microns-squared].  36um2 = 6um 1/e radius
%cellMatchFilterNumSigma : z-score (# of stdev's above mean) used on peak distribution for cell discrimination ; (generally, larger z-score = fewer cells detected)    
%cellMatchFilterMinimumVolumeInCubicMicrons : minimum (#) of contiguous cubic microns for a region to be counted as a cell         
%cellMatchFilterMaxVolumePercent : maximum contiguous percent size of subvolume that can be masked as a cell or cell cluster        
%numVoxelsMarginBlank : number of voxels to leave as a blank margin at edge of each subvolume when restitching.                  
cellSegParameters.cellMatchFilterModelVarianceInMicronsSquared = 8; 
cellSegParameters.cellMatchFilterBackgroundVarianceInMicronsSquared = 40; 
cellSegParameters.cellMatchFilterNumSigma = 5; 
cellSegParameters.cellMatchFilterMinimumVolumeInCubicMicrons = 100;
cellSegParameters.cellMatchFilterMaxVolumePercent = 0.1;   
cellSegParameters.numVoxelsMarginBlank=1;
experimentTemplate.cellSegParameters = cellSegParameters;
%------------------------------------------------------------------------
%VASC BLOCK CHANNEL SUB PARAMETERS
%channelSubFactorList : list of subtraction factors to run for cell-removal from vasc channel utlizing cell mask
%channelSubFitPolynomialDegree : determination of subtraction factor based on (#)-degree fit of a canceling metric versus subtraction factor    
%channelSubMaskSubSample :  for speed, the cell-masked region is subsampled by a factor of (#) for calculations            
%channelSubNonMaskSubSample : for speed, the non-cell-masked region is subsampled by a factor of (#) for calculations             
vascBlockChannelSubParameters.maxSubFraction = 0.5;
vascBlockChannelSubParameters.subFractionStepSize = 0.005;
vascBlockChannelSubParameters.channelSubFitPolynomialDegree = 6;
vascBlockChannelSubParameters.channelSubMaskSubSample = 10;
vascBlockChannelSubParameters.channelSubNonMaskSubSample = 50;
experimentTemplate.vascBlockChannelSubParameters = vascBlockChannelSubParameters;
%------------------------------------------------------------------------
%VASC SEGMENTATION PARAMETERS
%rodFilterHalfSizeInMicrons :  vessels are filtered with rod that a single voxel wide, and 2*(#) + 1 microns long     
%numPointsRodFilter : set=4 together with numTriangleRodFilter=2 to set number of rods spanning 3D space to be 82                           
%numTrianglesRodFilter : set=2 together with numPointsRodFilter=4 to set number of rods spanning 3D space to be 82             
vascSegParameters.vascRodFilterParameters.rodFilterHalfSizeInMicrons = 5;
vascSegParameters.vascRodFilterParameters.numPointsRodFilter = 4;                  %nptspu=4 and ntriags=2 produces ~82 rods to span unit sphere
vascSegParameters.vascRodFilterParameters.numTrianglesRodFilter = 2;               %nptspu=4 and ntriags=2 produces ~82 rods to span unit sphere
%-----
%enhMaskPrelimThresholdFraction :  preliminary threshold on rod-enhanced volume;  thresh = (#) * max_Intensity_in_volume     
%numBkgStdForMinThresh : lower limit on threshold for raw volume set at mode + (#) * stdev.  #=4.4 corresponds to 0.5percent outside; 3.85 ~ 1percent outside       
%maxDilationCyclesInMicrons : maximum dilation size [in microns] for of preliminary mask to determine background regions (avoid endless cycling for blank blocks)            
%gaussianRadiusForFilterInMicrons : 1/e-radius of large gaussian filter (in microns); a 4-x smaller radius filter is also used      
%fractionOfHighIntensityForMaxThresh :  upper limit on threshold for raw volume set at (#) * max_Intensity_in_volume     
%filtMultiplierForVarThresh : scale multiplier on local filtered values to set as threshold on raw volume; recommended (#) = 1.5    
%minVolForSmallVolRemovalInVoxels : min (#) of continguous voxels to count as a vessel segment; removes speckle noise             
%numCleanupCyclesInMicrons : number of rounds of "convexifying" rountine to run.  Automatically scales with resolution             
%minVolForCleanupInVoxels : determines the degree of "convexifying" performed by the algorithm.  Recommended (#) = 11             
vascSegParameters.vascRawBinarizeParameters.enhMaskPrelimThresholdFraction = 0.05;
vascSegParameters.vascRawBinarizeParameters.numBkgStdForMinThresh = 4.4; %4.4 corresponds to 0.5% outside; 3.85 corresponds to 1.01%
vascSegParameters.vascRawBinarizeParameters.maxDilationCyclesInMicrons = 20; %maximum number of dilations for isolating background
vascSegParameters.vascRawBinarizeParameters.gaussianRadiusForFilterInMicrons = 11;
vascSegParameters.vascRawBinarizeParameters.fractionOfHighIntensityForMaxThresh = 0.15;
vascSegParameters.vascRawBinarizeParameters.filtMultiplierForVarThresh = 1.5; 
vascSegParameters.vascRawBinarizeParameters.minVolForSmallVolRemovalInVoxels = 27;
vascSegParameters.vascRawBinarizeParameters.numCleanupCyclesInMicrons = 3;
vascSegParameters.vascRawBinarizeParameters.minVolForCleanupInVoxels = 11;
vascSegParameters.vascRawBinarizeParameters.smallGaussianSizeFactor = 4;
%-----
%-----
%numBkgStdForMinThresh : lower limit on threshold for enhanced volume set at mode + (#) * stdev.  #=4.4 corresponds to 0.5percent outside; 3.85 ~ 1percent outside       
%gaussianRadiusForFilterInMicrons : 1/e-radius of large gaussian filter (in microns); a 4-x smaller radius filter is also used      
%maxDilationCyclesInMicrons : maximum dilation size [in microns] for of preliminary mask to determine background regions (avoid endless cycling for blank blocks)            
%fractionOfHighIntensityForMaxThresh :  upper limit on threshold for enhanced volume set at (#) * max_Intensity_in_volume     
%filtMultiplierForVarThresh : scale multiplier on local filtered values to set as threshold on enhanced volume; recommended (#) = 1.1 %xxx :              
%minVolForSmallVolRemovalInVoxels : min (#) of continguous voxels to count as a vessel segment; removes speckle noise             
%numCleanupCyclesInMicrons : number of rounds of "convexifying" rountine to run.  Automatically scales with resolution             
%minVolForCleanupInVoxels : determines the degree of "convexifying" performed by the algorithm.  Recommended (#) = 11             
%postCleanUpCloseDiameterInMicrons : dilation size [in microns] performed on the final mask. Helps connect tiny gaps; do not over-dilate or merging occurs!                          
vascSegParameters.vascEnhancedBinarizeParameters.numBkgStdForMinThresh = 3; %4.4 corresponds to 0.5% outside; 3.85 corresponds to 1.01%
vascSegParameters.vascEnhancedBinarizeParameters.maxDilationCyclesInMicrons = 20; %maximum number of dilations for isolating background
vascSegParameters.vascEnhancedBinarizeParameters.gaussianRadiusForFilterInMicrons = 21;
vascSegParameters.vascEnhancedBinarizeParameters.fractionOfHighIntensityForMaxThresh = 0.10;
vascSegParameters.vascEnhancedBinarizeParameters.filtMultiplierForVarThresh = 0.7;
vascSegParameters.vascEnhancedBinarizeParameters.minVolForSmallVolRemovalInVoxels = 100;
vascSegParameters.vascEnhancedBinarizeParameters.numCleanupCyclesInMicrons = 1;
vascSegParameters.vascEnhancedBinarizeParameters.minVolForCleanupInVoxels = 11;
vascSegParameters.vascEnhancedBinarizeParameters.postCleanUpCloseDiameterInMicrons  = 1;%7
vascSegParameters.vascEnhancedBinarizeParameters.smallGaussianSizeFactor = 4;
vascSegParameters.enhancedVascRestitchFlag = 0;
experimentTemplate.vascSegParameters = vascSegParameters;
%--------------------------------------------------------------------------
%VASC CENTERLINING PARAMETERS
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx : 
centerlineEnhancedByConnectivityParameters.centerlineRecenteringFlag = 1; 
centerlineEnhancedByConnectivityParameters.hairTrimRadiusMultiplier = 1.0;
centerlineEnhancedByConnectivityParameters.hairTrimRadiusAdditiveInMicrons = 9;
centerlineEnhancedByConnectivityParameters.centeringIterMultiplier = 1.5;
centerlineEnhancedByConnectivityParameters.smallLoopCleanupSizeInMicrons = 3;
centerlineEnhancedByConnectivityParameters.minVolForConnectedCenterlineInMicrons = 7;       % throws away small centerlined islands
centerlineEnhancedByConnectivityParameters.preCloseDiameterInMicrons = 1;
centerlineEnhancedByConnectivityParameters.bwFiltKernelSizeInMicrons = 7;  
centerlineEnhancedByConnectivityParameters.maxFlips = 15;
centerlineEnhancedByConnectivityParameters.centeringFieldCircleDiameterInMicrons = 3;
centerlineEnhancedByConnectivityParameters.maxCenteringFieldFilterRangeInMicrons = 120;
centerlineEnhancedByConnectivityParameters.numCenteringFieldFilterCycles = -1; %set to -1 for automatic
experimentTemplate.centerlineEnhancedByConnectivityParameters = centerlineEnhancedByConnectivityParameters;
%--------------------------------------------------------------------------
%CELL CENTROID PARAMETERS
%xxx :              
%xxx :              
%xxx :              
%xxx :              
cellCentroidParameters.cellCentroidSubBlockSizeYInMicrons = 40; %y-block size for finding cell centroids
cellCentroidParameters.cellCentroidSubBlockBufferSizeYInMicrons = 10; %buffer zone on either side of each block
cellCentroidParameters.cellCentroidMergeDistanceInMicrons = 5;   %cell centroids closer than mergedistance will be merged
cellCentroidParameters.cellCentroidOverlayCircleDiameterInMicrons = 5;
experimentTemplate.cellCentroidParameters = cellCentroidParameters;
%--------------------------------------------------------------------------

%NEUN BLOCK CHANNEL SUBTRACTION PARAMETERS
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
neunBlockChannelSubParameters.maxSubFraction = 1.5;
neunBlockChannelSubParameters.subFractionStepSize = 0.01;
neunBlockChannelSubParameters.channelSubFitPolynomialDegree = 6;
neunBlockChannelSubParameters.channelSubMaskSubSampleFactor = 5;
neunBlockChannelSubParameters.channelSubNonMaskSubSampleFactor = 20;
neunBlockChannelSubParameters.neunCellFilterSizeInMicrons = 3;
neunBlockChannelSubParameters.topCellNeuNRatioFraction = 0.001;
experimentTemplate.neunBlockChannelSubParameters = neunBlockChannelSubParameters;
%--------------------------------------------------------------------------

%NEUN QUANTIFICATION PARAMETERS
%---
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%-----
%Instructions:    [a,b]=[0,+2] = shell from mask to dilate2 of mask
%                 [a,b]=[+2,+4] = shell from dilate2 of mask to dilate4 of mask
%                 [a,b]=[-2,+2] = shell from erode2 of mask to dilate2 of mask
%                 [a,b]=[-99,+3] = solid mask out to dilate3 of mask
neunQuantifyParameters.vascMaskFilterSizeInMicrons = 3;
neunQuantifyParameters.mainCellDilationInMicrons = [-99,+02];
neunQuantifyParameters.bkgShellDilationInMicrons = [+07,+08];
neunQuantifyParameters.otherCellDilationInMicrons = [-99,+04];
neunQuantifyParameters.excludeVascFromCellFlag = 1; %0 = no, 1 = yes
neunQuantifyParameters.excludeVascFromBkgFlag = 1; %0 = no, 1 = yes
neunQuantifyParameters.minCellAnalysisVolumeInVoxels = 9; 
neunQuantifyParameters.minBkgAnalysisVolumeInVoxels = 9;
experimentTemplate.neunQuantifyParameters = neunQuantifyParameters;
%--------------------------------------------------------------------------
%NEURON CLASSIFICATION PARAMETERS
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
neuronClassifyParameters.firstRoundParameters.minRatioThresh = 0.05;
neuronClassifyParameters.firstRoundParameters.maxRatioThresh = 0.85;
neuronClassifyParameters.firstRoundParameters.convergenceFraction = 0.01;
neuronClassifyParameters.firstRoundParameters.maxNumIterationsNeuronSegregation = 30;
neuronClassifyParameters.firstRoundParameters.ratioNeighborhoodXYInMicrons = 50;
neuronClassifyParameters.firstRoundParameters.ratioNeighborhoodZInMicrons = 10;
neuronClassifyParameters.firstRoundParameters.deltaZInMicrons = 1000;     %z-slice size to subslice the subblock; set large = no subslicing
neuronClassifyParameters.firstRoundParameters.minNumCells = 50;  %min number of cells within a z subslice.
neuronClassifyParameters.firstRoundParameters.minRatioRange = -1;
neuronClassifyParameters.firstRoundParameters.maxRatioRange = 2;
neuronClassifyParameters.firstRoundParameters.ratioRangeStepSize = 0.01;
experimentTemplate.neuronClassifyParameters = neuronClassifyParameters;
%--------------------------------------------------------------------------

%VASCULAR RADII PARAMETERS
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
%xxx :              
vascRadiiParameters.fractionalIntensityReconstructionParameters.minVolForConnectedCenterlineInVoxels = 7;       % throws away small centerlined islands
vascRadiiParameters.fractionalIntensityReconstructionParameters.neighborhoodSizeFactor = 6;             % KEY PARAMETER -- RELATED TO MASK CHOICE FOR R INITIALIZATION
vascRadiiParameters.fractionalIntensityReconstructionParameters.localRadiiMultiplerForSaturation = 1.5; % SENSITIVE?? how far out to saturate intensities distant from centerline point
vascRadiiParameters.fractionalIntensityReconstructionParameters.brightValueFraction = 0.9;              % makes sure that points far from center are < max val in center
vascRadiiParameters.fractionalIntensityReconstructionParameters.firstRoundThresholdFraction = 0.3890;   % bootstrap guess on fraction of max to set threshold
vascRadiiParameters.fractionalIntensityReconstructionParameters.maxFractionOfSubFramePixels = 0.25;     % make a function of neighborhoodSizeFactor
vascRadiiParameters.fractionalIntensityReconstructionParameters.bwDistSearchWindowSizeInMicrons = 3;             
vascRadiiParameters.fractionalIntensityReconstructionParameters.zSearchSizeInMicrons = 3;       
vascRadiiParameters.fractionalIntensityReconstructionParameters.minVesselRadiusSupportInMicrons = 0.5;
vascRadiiParameters.fractionalIntensityReconstructionParameters.maxVoxelsForNoiseBlock = 27;         % only introduced for noise blocks
vascRadiiParameters.fractionalIntensityReconstructionParameters.interpolateFactor = 1;                  % subpixel radius estimation
vascRadiiParameters.fractionalIntensityReconstructionParameters.minVolumeAfterReconstructionInVoxels = 0;       % only removes isolated points
vascRadiiParameters.fractionalIntensityReconstructionParameters.maxLocalRadiusInMicrons = 10;       
vascRadiiParameters.fractionalIntensityReconstructionParameters.closeDiameterInMicrons = 3;       
vascRadiiParameters.fractionalIntensityReconstructionParameters.minNeighborhoodSizeInCubicMicrons = 6;       
%-----
vascRadiiParameters.microVesselIsolationParameters.maxBlockSizeInMicrons = 300;
vascRadiiParameters.microVesselIsolationParameters.blockOverlapSizeInMicrons = 40;
vascRadiiParameters.microVesselIsolationParameters.morphologicalCloseSizeInMicrons = 1;%9
vascRadiiParameters.microVesselIsolationParameters.radiusListStartInMicrons = 3.5;
vascRadiiParameters.microVesselIsolationParameters.radiusListStopInMicrons = 7;
vascRadiiParameters.microVesselIsolationParameters.radiusListStepInMicrons = 0.5;
vascRadiiParameters.microVesselIsolationParameters.deltaRInMicrons = 3;
experimentTemplate.vascRadiiParameters = vascRadiiParameters;
%--------------------------------------------------------------------------
%REGISTRATION PARAMETERS
experimentTemplate.registrationParameters.reverseYBlockNumberingFlag = 0; %default is 0 (no) (data taken top to bottom)
experimentTemplate.registrationParameters.reverseXBlockNumberingFlag = 1;%default is 1 (yes) (data taken right to left)
experimentTemplate.registrationParameters.xcorrCh = 3;%channel used for correlation
experimentTemplate.registrationParameters.nominalBlockStepSizeRCInMicrons = 200; 
experimentTemplate.registrationParameters.nominalBlockStepSizeZInMicrons = 50; %either single number or list if step sizes vary;
experimentTemplate.registrationParameters.variableZBlockStepSizeFlag = 0;
experimentTemplate.registrationParameters.ySearchSizeInMicrons = 41;
experimentTemplate.registrationParameters.xSearchSizeInMicrons = 41;
experimentTemplate.registrationParameters.zSearchSizeInMicrons = 41;
experimentTemplate.registrationParameters.overlapfusion = 'quick'; % 'fixedzlayer', 'quick', 'sum' or 'centermost'
experimentTemplate.registrationParameters.checkBlockSize = 0;
experimentTemplate.registrationParameters.emptyBlockCorrectionFlag = 1; %set to 1 to default empty blocks to nominal position
experimentTemplate.registrationParameters.emptyBlockThreshold = 0.02; %threshold for determining if a block is empty.
experimentTemplate.registrationParameters.concatenateParameters.desiredOverlapSizeYXInMicrons = 3;
experimentTemplate.registrationParameters.concatenateParameters.desiredOverlapSizeZInMicrons = 3;
experimentTemplate.registrationParameters.concatenateParameters.desiredOverlapShiftYXInMicrons = 0;
experimentTemplate.registrationParameters.concatenateParameters.desiredOverlapShiftZInMicrons = 0;
%--------------------------------------------------------------------------
%STRANDIFY PARAMETERS
experimentTemplate.strandifyParameters.minSegmentLength = 5;
experimentTemplate.strandifyParameters.maxHairLengthToEvaluateForRemovalInMicrons = 0;
experimentTemplate.strandifyParameters.maxConnectingStrandLengthToEvaluateForRemovalInMicrons = 0;
experimentTemplate.strandifyParameters.intensityRatioThresholdForHairRemoval = 3;
experimentTemplate.strandifyParameters.intensityRatioThresholdForConnectingStrandRemoval = 3;
experimentTemplate.strandifyParameters.outputTifFilesFlag = 0;
%--------------------------------------------------------------------------




outExp = experimentTemplate;









