function outputParameters = VidaCreateUniformityFilter(inputParameters)
   
    fluoresceinGaussianFilterSize = 0;%noteToSelf

    
    vidaProject = inputParameters;
    vidaProjectRootName = vidaProject.vidaProjectRootName;
    vidaProjectFileName = vidaProject.vidaProjectFileName;
    %-----
    fluoresceinFileNameList{1} = fullfile(vidaProject.vidaProjectRootFolder,[vidaProjectRootName,'-fluorescein-Ch1.tif']);
    fluoresceinFileNameList{2} = fullfile(vidaProject.vidaProjectRootFolder,[vidaProjectRootName,'-fluorescein-Ch2.tif']);
    fluoresceinFileNameList{3} = fullfile(vidaProject.vidaProjectRootFolder,[vidaProjectRootName,'-fluorescein-Ch3.tif']);
    darkFileNameList{1} = fullfile(vidaProject.vidaProjectRootFolder,[vidaProjectRootName,'-dark-Ch1.tif']);
    darkFileNameList{2} = fullfile(vidaProject.vidaProjectRootFolder,[vidaProjectRootName,'-dark-Ch2.tif']);
    darkFileNameList{3} = fullfile(vidaProject.vidaProjectRootFolder,[vidaProjectRootName,'-dark-Ch3.tif']);
    uniformityFileNameList{1} = fullfile(vidaProject.vidaProjectRootFolder,[vidaProjectRootName,'-uniformityChannel1.tif']);
    uniformityFileNameList{2} = fullfile(vidaProject.vidaProjectRootFolder,[vidaProjectRootName,'-uniformityChannel2.tif']);
    uniformityFileNameList{3} = fullfile(vidaProject.vidaProjectRootFolder,[vidaProjectRootName,'-uniformityChannel3.tif']);
    
    
    if or(~exist(uniformityFileNameList{3},'file'),~isfield(vidaProject,'darkIntensityMode')),
        %Create uniformity filter and calculate background mode from dark
        %files
        disp('Creating Uniformity Filters');
        for ChIter = 1:3,
            darkData = loadmovie(darkFileNameList{ChIter});
            fluoresceinData = loadmovie(fluoresceinFileNameList{ChIter});
            darkFrame = mean(double(darkData),3);
            fluoresceinFrame = mean(double(fluoresceinData),3);
            if fluoresceinGaussianFilterSize>0,
                gaussianFilter = gaussian2d(3*fluoresceinGaussianFilterSize,fluoresceinGaussianFilterSize);
                darkSubtractedFrame = fluoresceinFrame - darkFrame;
                darkSubtractedFrame = imfilter(darkSubtractedFrame,gaussianFilter,'replicate');
            else
                darkSubtractedFrame = fluoresceinFrame - darkFrame;
            end
            uniformityFrame = uint8(256*(darkSubtractedFrame./max(max(darkSubtractedFrame))));
            maketiff(uniformityFrame,uniformityFileNameList{ChIter});
            reshapedDarkData = reshape(darkData,[1,numel(darkData)]);
            xRange = [1:2048];
            darkIntensityHistogram = hist(reshapedDarkData,xRange);
            darkHistogramMaximum = find(darkIntensityHistogram == max(darkIntensityHistogram));
            darkIntensityMode(ChIter) = xRange(darkHistogramMaximum(1));
        end%for ChIter = 1:3,
    else
        darkIntensityMode = vidaProject.darkIntensityMode;
    end%if ~exist(uniformityFileName{3},'file'),
    
   

    vidaProject.globalParameters.fluoresceinFileNameList = fluoresceinFileNameList;
    vidaProject.globalParameters.darkFileNameList = darkFileNameList;
    vidaProject.globalParameters.uniformityFileNameList = uniformityFileNameList;
    vidaProject.globalParameters.darkIntensityMode = darkIntensityMode;
    outputParameters= vidaProject;
    
end
