function outExp = VidaPreProcessing(inExp)

%Input Parameters
rootDataFolder = inExp.globalParameters.rootDataFolder;
fileRootName = inExp.globalParameters.fileRootName;
originalDataFolder = inExp.globalParameters.originalDataFolder;
processedDataFolder = inExp.globalParameters.processedDataFolder;
intermediateResultsFolder = inExp.globalParameters.intermediateResultsFolder;
voxelSizeInMicrons = inExp.globalParameters.voxelSizeInMicrons;
vidaProjectRootFolder = inExp.globalParameters.vidaProjectRootFolder;
vidaProjectRootName = inExp.globalParameters.vidaProjectRootName;
%------
fluoresceinGaussianFilterSizeInMicrons = inExp.preProcessingParameters.fluoresceinGaussianFilterSizeInMicrons;
fluoresceinGaussianFilterSize = fluoresceinGaussianFilterSizeInMicrons ./ voxelSizeInMicrons;
if isfield(inExp.globalParameters,'uniformityFileNameList'),
    uniformityFileNameList = inExp.globalParameters.uniformityFileNameList;
    fluoresceinFileNameList = inExp.globalParameters.fluoresceinFileNameList;
    darkFileNameList = inExp.globalParameters.darkFileNameList;
    darkIntensityMode = inExp.globalParameters.darkIntensityMode;
end
%-----
defaultFluoresceinFileName = fullfile(rootDataFolder,[fileRootName,'-fluorescein']);
defaultDarkFileName = fullfile(rootDataFolder,[fileRootName,'-dark']);
filePrefix = {'NEUN','HORD','FITC'};
%-----
[mkdirSuccess,mkdirMessage,mkdirId] = mkdir(originalDataFolder);
[mkdirSuccess,mkdirMessage,mkdirId] = mkdir(processedDataFolder);
[mkdirSuccess,mkdirMessage,mkdirId] = mkdir(intermediateResultsFolder);

darkFileName = defaultDarkFileName;
fluoresceinFileName = defaultFluoresceinFileName;
uniformityFolder = vidaProjectRootFolder;
%----------



%Check progress status:
%   if mpd files are still in main folder, progressStatus = 1;
%   if -uniformity files exists in main folder, progressStatus = 2
mpdFileQuery = dir(fullfile(uniformityFolder,'*.mpd'));
numMpdFiles = size(mpdFileQuery,1);
uniformityFileQuery = dir(fullfile(uniformityFolder,'*-uniformity*'));
numUniformityFiles = size(uniformityFileQuery,1);
%---
progressStatus = 3;
if numUniformityFiles > 0, progressStatus = 2; end
if numMpdFiles > 0, progressStatus = 1; end

%-----
%STEP 1 : convert MPD file to Tif, and move MPD files to originalDataFolder
if progressStatus <=1 ,
    mpdFileQuery = dir(fullfile(uniformityFolder,'*.mpd'));
    numMpdFiles = size(mpdFileQuery,1);
    %Convert MPD files to TIF files
    for fileIter = 1:numMpdFiles,
        disp(['Converting mpd file ',num2str(fileIter),' of ',num2str(numMpdFiles),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
        mpd2TiffParameters.inputFileName = fullfile(uniformityFolder,mpdFileQuery(fileIter).name);
        mpd2tiff(mpd2TiffParameters);
    end
    %-----
    %Move MPD files to originalDataFolder
    for fileIter = 1:numMpdFiles,
        disp(['Moving mpd file ',num2str(fileIter),' of ',num2str(numMpdFiles),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
        fullOriginalName = fullfile(uniformityFolder,mpdFileQuery(fileIter).name);
        fullFinalName = fullfile(originalDataFolder,mpdFileQuery(fileIter).name);
        systemMoveFile(fullOriginalName,fullFinalName);
      
        % % %movefile(fullOriginalName,fullFinalName);
    end%for fileIter = 1:numMpdFiles,
    %-----
end%if progressStatus <=1 ,


if numUniformityFiles == 0,
    %Create uniformity filter and calculate background mode from dark files
    disp('Creating Uniformity Filters');
    for ChIter = 1:3,
        darkData = loadmovie([darkFileName,'-Ch',num2str(ChIter),'.tif']);
        fluoresceinData = loadmovie([fluoresceinFileName,'-Ch',num2str(ChIter),'.tif']);
        darkFrame = mean(double(darkData),3);
        fluoresceinFrame = mean(double(fluoresceinData),3);
        if fluoresceinGaussianFilterSize>0,
            gaussianFilter = gaussian2d(3*fluoresceinGaussianFilterSize,fluoresceinGaussianFilterSize);
            darkSubtractedFrame = fluoresceinFrame - darkFrame;
            darkSubtractedFrame = imfilter(darkSubtractedFrame,gaussianFilter,'replicate');
        else
            darkSubtractedFrame = fluoresceinFrame - darkFrame;
        end
        uniformityFrame = uint8(256*(darkSubtractedFrame./max(max(darkSubtractedFrame))));
        uniformityFileNameList{ChIter} = fullfile(uniformityFolder,['uniformityFilterChannel',num2str(ChIter),'.tif']);
        maketiff(uniformityFrame,uniformityFileNameList{ChIter});
        reshapedDarkData = reshape(darkData,[1,prod(size(darkData))]);
        xRange = [1:2048];
        darkIntensityHistogram = hist(reshapedDarkData,xRange);
        darkHistogramMaximum = find(darkIntensityHistogram == max(darkIntensityHistogram));
        darkIntensityMode(ChIter) = xRange(darkHistogramMaximum(1));
    end%for ChIter = 1:3,
    %---
end%if numUniformityFiles = 0;

%-----
%STEP 2 : Subtract offset, and move orig tifs to originalDataFolder
if progressStatus <=2,
    %Perform offset subtraction on tif files
    mainTiffFileQuery = dir(fullfile(rootDataFolder,[vidaProjectRootName,'-Z*.tif']));
    numMainTiffFiles = size(mainTiffFileQuery,1);
    for fileIter = 1:numMainTiffFiles,
        disp(['Subtracting DC Offset on File ',num2str(fileIter),' of ',num2str(numMainTiffFiles),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
        currentFile = fullfile(rootDataFolder,mainTiffFileQuery(fileIter).name);
        outputFileName = [currentFile(1:end-4),'-off.tif'];
        if exist(outputFileName,'file') || ~isempty(findstr(currentFile,'-off')),
            disp('...Skipping');
        else%if exist(outputFileName,'file'),
            channelNamePos = findstr(currentFile,'-Ch');
            channelNum = str2num(currentFile(channelNamePos+3));
            subtractoffset2(currentFile,darkIntensityMode(channelNum)); %subtract the dark intensity mode from each frame
        end%if exist(outputFileName,'file'),
    end%for fileIter = 1:numMainTiffFiles
    %---
end%if progressStatus <=2,

%-----
%STEP 3 : Divide by uniformity filter, rename and move results to processedFilesFolder
%   Move remaining tif files (the -off files) to originalFilesFolder;
if progressStatus <= 3,
    %Divide by uniformity filter
    for channelIter = 1:3,
        darkSubtractedFiles = dir(fullfile(rootDataFolder,['*-Ch',num2str(channelIter),'-off.tif']));
        numDarkSubtractedFiles = size(darkSubtractedFiles,1);
        for fileIter = 1:numDarkSubtractedFiles;
            disp(['Uniformizing Ch ',num2str(channelIter),' file ',num2str(fileIter),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
            currentFile = darkSubtractedFiles(fileIter).name;
            dividefilter2(fullfile(rootDataFolder,currentFile),uniformityFileNameList{channelIter});
        end%for fileIter = 1:numDarkSubtractedFiles;
    end%for channelIter = 1:3,


    %-----
    %Move and rename files to processedFilesFolder
    filteredFiles = dir(fullfile(rootDataFolder,['*-off-divfilt.tif']));
    numFilteredFiles = size(filteredFiles,1);
    filteredFileNameList = cell(1,numFilteredFiles);
    for fileIter = 1:numFilteredFiles,
        disp(['Moving filtered file ',num2str(fileIter),' of ',num2str(numFilteredFiles),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
        tempName = filteredFiles(fileIter).name;
        tempLastNamePos = findstr(tempName,'-Ch');
        tempNameRoot = tempName(1:tempLastNamePos-1);
        channelNum = str2num(tempName(tempLastNamePos+3));
        newName = [filePrefix{channelNum},'-',tempNameRoot,'.tif'];
        fullOriginalName = [fullfile(rootDataFolder,tempName)];
        fullFinalName = [fullfile(processedDataFolder,newName)];
        filteredFileNameList{fileIter} = fullFinalName;
        systemMoveFile(fullOriginalName,fullFinalName);
        % % % movefile(fullOriginalName,fullFinalName)
    end%for fileIter = 1:numFilteredFiles,


    %-----
    %Cleanup -off.tif files into original file folder
    offSubFiles = dir(fullfile(rootDataFolder,'*-off.tif'));
    numOriginalFiles = size(offSubFiles,1);
    for fileIter = 1:numOriginalFiles,
        disp(['Deleting -offset file ',num2str(fileIter),' of ',num2str(numOriginalFiles),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
        fullOffSubFileName = [fullfile(rootDataFolder,offSubFiles(fileIter).name)];
        delete(fullOffSubFileName);
    end%for fileIter = 1:numOriginalFiles,
end%if progressStatus <= 3,

%DEFINE CHANNEL NAMES
stackedFileNameList{1} = fullfile(processedDataFolder,[fileRootName,'-Stacked-HORD.tif']);
stackedFileNameList{2} = fullfile(processedDataFolder,[fileRootName,'-Stacked-FITC.tif']);
stackedFileNameList{3} = fullfile(processedDataFolder,[fileRootName,'-Stacked-NEUN.tif']);

numFiles = length(filteredFileNameList);
for fileIter = 1:numFiles,
    currentFileName = filteredFileNameList{fileIter};
    if findstr(currentFileName,'HORD') , newFileName = stackedFileNameList{1}; end
    if findstr(currentFileName,'FITC') , newFileName = stackedFileNameList{2}; end
    if findstr(currentFileName,'NEUN') , newFileName = stackedFileNameList{3}; end
    imageStack = loadmovie(currentFileName);
    fullStackSize = size(imageStack);
    matFileName = regexprep(newFileName,'.tif','.mat');
    saveSize = (getfield(whos('imageStack'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(matFileName,'imageStack',scs);
    stackedFileNameList = regexprep(stackedFileNameList,'.tif','.mat');
    %Clean up extra file
    systemDeleteFile(currentFileName); %PB
end%for fileIter = 1:numFiles,
%-----
outExp = inExp;
outExp.globalParameters.stackedFileNameList = stackedFileNameList;
outExp.globalParameters.fullStackSize = fullStackSize;
outExp.completionDates.preProcessingCompletionDate = datestr(now);
return



