function outputParameters = VidaSortOriginalTifs(inputParameters)
    vidaProject = inputParameters;
    vidaProjectRootFolder = vidaProject.vidaProjectRootFolder;
    vidaProjectRootName = vidaProject.vidaProjectRootName;
    dosMoveFlag = 1;
    reverseXBlockNumberingFlag = vidaProject.globalParameters.reverseXBlockNumberingFlag;
    reverseYBlockNumberingFlag = vidaProject.globalParameters.reverseYBlockNumberingFlag;

    %-----
    %In case the field : imageStackFolderNameList exists and is not empty, assume
    %that the sorting is already done.
    if isfield(vidaProject,'imageStackFolderNameList') && ~isempty(vidaProject.imageStackFolderNameList)
        disp('Image Block Folders previously created');
        minR = vidaProject.rczMin(1);
        minC = vidaProject.rczMin(2);
        minZ = vidaProject.rczMin(3);
        maxR = vidaProject.rczMax(1);
        maxC = vidaProject.rczMax(2);
        maxZ = vidaProject.rczMax(3);
    else%if isfield(vidaProject,'imageStackFolderNameList') && ~isempty(vidaProject.imageStackFolderNameList)
        %for every Ch1 file, create a folder and move the files to it
        ch1NameKernel = fullfile(vidaProjectRootFolder,[vidaProjectRootName,'-Z*-Ch1.tif']);
        fileList = dir(ch1NameKernel);
        numImageStacks = size(fileList,1);
        %Sort the file list
        maxZ = 0; maxC = 0; maxR = 0;
        minZ = 9999;minC = 9999; minR = 9999;
        for fileIter = 1:numImageStacks
            currentName = fileList(fileIter).name;
            point1 = regexp(currentName,'-Z\d');
            point2 = regexp(currentName,'-C\d');
            point3 = regexp(currentName,'-R\d');
            point4 = regexp(currentName,'-Ch\d');
            zNum = str2num(currentName(point1+2:point2-1));
            cNum = str2num(currentName(point2+2:point3-1));
            rNum = str2num(currentName(point3+2:point4-1));
            minZ = min(minZ,zNum);
            minC = min(minC,cNum);
            minR = min(minR,rNum);
            maxZ = max(maxZ,zNum);
            maxC = max(maxC,cNum);
            maxR = max(maxR,rNum);
        end
        zList = [minZ:maxZ];
        cList = [minC:maxC];
        rList = [minR:maxR];
        
        if reverseXBlockNumberingFlag, cList = [maxC:-1:minC]; end
        if reverseYBlockNumberingFlag, rList = [maxR:-1:minR]; end
        counter = 0;
        for zIter = zList,
            for cIter = cList,
                for rIter = rList,
                    %-modified by Pablo to work with irregular block arrangements - 20110825
                    tmpFileName =  [vidaProjectRootName,'-Z',num2str(zIter,'%02d'),'-C',num2str(cIter,'%02d'),'-R',num2str(rIter,'%02d'),'-Ch1.tif'];
                    if exist(tmpFileName,'file')
                        counter = counter+1;
                        fileNameList{counter} = [vidaProjectRootName,'-Z',num2str(zIter,'%02d'),'-C',num2str(cIter,'%02d'),'-R',num2str(rIter,'%02d'),'-Ch1.tif'];
                    end%if exist(tmpfileName,'file')
                end%for rIter
            end%for cIter
        end%for zIter
        

        
        if numImageStacks == 0,
            disp('Image Block Folders previously created');
            dirList = dir(fullfile(vidaProjectRootFolder,'ImageStack*'));
            numDir = length(dirList),
            %Sort the directory list
            maxZ = 0; maxC = 0; maxR = 0;
            for dirIter = 1:numDir
                currentName = dirList(dirIter).name;
                point1 = regexp(currentName,'-Z\d');
                point2 = regexp(currentName,'-C\d');
                point3 = regexp(currentName,'-R\d');
                zNum = str2num(currentName(point1+2:point2-1));
                cNum = str2num(currentName(point2+2:point3-1));
                rNum = str2num(currentName(point3+2:end));
                minZ = min(minZ,zNum);
                minC = min(minC,cNum);
                minR = min(minR,rNum);
                maxZ = max(maxZ,zNum);
                maxC = max(maxC,cNum);
                maxR = max(maxR,rNum);
            end
            zList = [minZ:maxZ];
            cList = [minC:maxC];
            rList = [minR:maxR];
            if reverseXBlockNumberingFlag, cList = [maxC:-1:minC]; end
            if reverseYBlockNumberingFlag, rList = [maxR:-1:minR]; end
            counter = 0;
            for zIter = zList,
                for cIter = cList,
                    for rIter = rList,
                        counter = counter+1;
                        dirNameList{counter} = ['ImageStack-',vidaProjectRootName,'-Z',num2str(zIter,'%02d'),'-C',num2str(cIter,'%02d'),'-R',num2str(rIter,'%02d')];
                    end%for rIter
                end%for cIter
            end%for zIter
            %-----
            for imageStackIter = 1:numDir,
                currentDirName = dirNameList{imageStackIter};
                currentFullImageStackFolderName = fullfile(vidaProjectRootFolder,currentDirName);
                vidaProject.imageStackFolderNameList{imageStackIter} = currentFullImageStackFolderName;
                vidaProject.imageStackCompletedFlag(imageStackIter) = 0;
            end%for imageStackIter = 1:length(dirList),
        end%if numImageStack == 0,
        %-----

        for imageStackIter = 1:numImageStacks,
            currentCh1FileName = fileNameList{imageStackIter};
            currentImageStackRoot = currentCh1FileName(1:findstr(currentCh1FileName,'-Ch1.tif')-1);
            currentImageStackFolder =['ImageStack-',currentImageStackRoot];
            currentFullImageStackFolderName = fullfile(vidaProjectRootFolder,currentImageStackFolder);
            [mkdirSuc,mkdirMessage,mkdirMessageID] = mkdir(vidaProjectRootFolder,currentImageStackFolder);
            disp(['Moving Image Stack Set ',num2str(imageStackIter),' of ',num2str(numImageStacks),' : ', datestr(now,'mm-dd-yy,HH:MM.SS')]);
            vidaProject.imageStackCompletedFlag(imageStackIter) = 0;
            vidaProject.imageStackFolderNameList{imageStackIter} = currentFullImageStackFolderName;
            for channelIter = 1:1,
                currentFileName = [currentImageStackRoot,'-Ch',num2str(channelIter),'.tif'];
                sourceFileName = fullfile(vidaProjectRootFolder,currentFileName);
                destinationFileName = fullfile(currentFullImageStackFolderName,currentFileName);
                if dosMoveFlag == 1,
                    systemMoveFile(sourceFileName,destinationFileName);
                else%if dosMoveFlag == 1,
                    [moveStatus,moveMessage,moveMessageID] = movefile(sourceFileName,destinationFileName);
                end%if dosMoveFlag == 1,
            end%for channelIter = 1:3,
        end%for imageStackIter = 1:numImageStacks,
    end%if isfield(vidaProject,'imageStackFolderNameList') && ~isempty(vidaProject.imageStackFolderNameList)
    %-----
    %Check for layout of files
    
    vidaProject.numImageStacks = size(vidaProject.imageStackFolderNameList,2);
    vidaProject.globalParameters.rczMin = [minR,minC,minZ];
    vidaProject.globalParameters.rczMax = [maxR,maxC,maxZ];
    

    %-----
    outputParameters = vidaProject;
end
