function outExp = QuickTestVascSegRerun(inExp)
%last modified 12/05/08 by Phil Tsai

%-----
fileRootName = inExp.globalParameters.fileRootName;
rootDataFolder = inExp.globalParameters.rootDataFolder;
lastRerunStartDateNum = inExp.globalParameters.lastRerunStartDateNum;
processedDataFolder = inExp.globalParameters.processedDataFolder;
intermediateResultsFolder = inExp.globalParameters.intermediateResultsFolder;
numChannels = inExp.globalParameters.numChannels;
stackedFileNameList = inExp.globalParameters.stackedFileNameList;
fullStackSize = inExp.globalParameters.fullStackSize;
vascBlockFileNameList = inExp.globalParameters.vascBlockFileNameList;
blockGrid = inExp.globalParameters.blockGrid;
subVolumeOverlapSize = inExp.globalParameters.subVolumeOverlapSize;
voxelSizeInMicrons = inExp.globalParameters.voxelSizeInMicrons;
%-----
vascRodFilterParameters = inExp.vascSegParameters.vascRodFilterParameters;
vascRawBinarizeParameters = inExp.vascSegParameters.vascRawBinarizeParameters;
vascEnhancedBinarizeParameters = inExp.vascSegParameters.vascEnhancedBinarizeParameters;
%-----
enhancedVascRestitchFlag = inExp.vascSegParameters.enhancedVascRestitchFlag;
rerunRodFlag = inExp.rerunRodFlag;
rerunRawFlag = inExp.rerunRawFlag;
%-----
if isfield(inExp.globalParameters,'vascBlockEnhancedGrayscaleFileNameList'),
    vascBlockEnhancedGrayscaleFileNameList = inExp.globalParameters.vascBlockEnhancedGrayscaleFileNameList;
end
%---
if isfield(inExp.globalParameters,'vascBlockRawMaskFileNameList'),
    vascBlockRawMaskFileNameList = inExp.globalParameters.vascBlockRawMaskFileNameList;
end
%---
if isfield(inExp.globalParameters,'vascBlockEnhancedMaskFileNameList'),
    vascBlockEnhancedMaskFileNameList = inExp.globalParameters.vascBlockEnhancedMaskFileNameList;
end
%-----


vascSegFilesFolder = fullfile(processedDataFolder,'VascSegFiles');
[mkdirSuccess,mkdirMessage,mkdirId] = mkdir(vascSegFilesFolder);
%-----
%Flag creation of optional tif files for stitched raw and enhanced vasc masks
displayFlag = 0;%noteToSelf
%-----

%Allow for user to skip rod filtering and copy the raw mask as the enhanced mask
useRawMaskOnlyFlag = 0;
if isfield(inExp.vascSegParameters,'useRawMaskOnlyFlag'),
    useRawMaskOnlyFlag = inExp.vascSegParameters.useRawMaskOnlyFlag;
end%if isfield...
%-----
if useRawMaskOnlyFlag == 1,
    %setting rod size to 0 skips the rod filtering
    vascRodFilterParameters.rodFilterHalfSizeInMicrons = 0;
end%if useRawMaskOnlyFlag == 1,

%-----
% Filter Vessel Images with Rod Matched Filters
if rerunRodFlag == 1,
%     disp(['Starting Vessel Filtering : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
    if exist('vascBlockEnhancedGrayscaleFileNameList','var'),
        for k = 1:numel(vascBlockEnhancedGrayscaleFileNameList);
            delete(vascBlockEnhancedGrayscaleFileNameList{k});
        end%for k..
    end%if exist...
    vascRodFilterParameters.lastRerunStartDateNum = lastRerunStartDateNum;
    vascRodFilterParameters.fileRootName = fileRootName;
    vascRodFilterParameters.intermediateResultsFolder = intermediateResultsFolder;
    vascRodFilterParameters.vascSegFilesFolder = vascSegFilesFolder;
    vascRodFilterParameters.vascFileName = stackedFileNameList{2};
    vascRodFilterParameters.displayFlag = displayFlag;
    vascRodFilterParameters.vascBlockFileNameList = vascBlockFileNameList;
    vascRodFilterParameters.blockGrid = blockGrid;
    vascRodFilterParameters.voxelSizeInMicrons = voxelSizeInMicrons;
        vascRodFilterParameters = VidaVascBlockEnhance2(vascRodFilterParameters);
    vascBlockEnhancedGrayscaleFileNameList = vascRodFilterParameters.vascBlockEnhancedGrayscaleFileNameList;
end%if rerunRodFlag == 1,
%-----
%-----
% Binarize Raw Vessel Data
if rerunRawFlag == 1,
%     disp(['Starting Vessel Binarization : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
    if exist('vascBlockRawMaskFileNameList','var'),
        for k = 1:numel(vascBlockRawMaskFileNameList);
            delete(vascBlockRawMaskFileNameList{k});
        end%for k..
    end%if exist...
    vascRawBinarizeParameters.fileRootName = fileRootName;
    vascRawBinarizeParameters.lastRerunStartDateNum = lastRerunStartDateNum;
    vascRawBinarizeParameters.vascSegFilesFolder = vascSegFilesFolder;
    vascRawBinarizeParameters.intermediateResultsFolder = intermediateResultsFolder;
    vascRawBinarizeParameters.vascBlockFileNameList = vascBlockFileNameList;
    vascRawBinarizeParameters.vascBlockEnhancedGrayscaleFileNameList = vascRodFilterParameters.vascBlockEnhancedGrayscaleFileNameList;
    vascRawBinarizeParameters.voxelSizeInMicrons = voxelSizeInMicrons;
    vascRawBinarizeParameters.displayFlag = displayFlag;
        vascRawBinarizeParameters = VidaVascBinarizeRawVolume(vascRawBinarizeParameters);
    vascBlockRawMaskFileNameList = vascRawBinarizeParameters.vascBlockRawMaskFileNameList;
end%if rerunRawFlag == 1,
%-----
%-----
% Binarize Enhanced Vessel Data
% disp(['Starting Vessel Binarization : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
if exist('vascBlockEnhancedMaskFileNameList','var'),
    for k = 1:numel(vascBlockEnhancedMaskFileNameList);
        delete(vascBlockEnhancedMaskFileNameList{k});
    end%for k..
end%if exist...
vascEnhancedBinarizeParameters.fileRootName = fileRootName;
vascEnhancedBinarizeParameters.lastRerunStartDateNum = lastRerunStartDateNum;
vascEnhancedBinarizeParameters.vascSegFilesFolder = vascSegFilesFolder;
vascEnhancedBinarizeParameters.intermediateResultsFolder = intermediateResultsFolder;
vascEnhancedBinarizeParameters.vascBlockFileNameList = vascBlockFileNameList;
vascEnhancedBinarizeParameters.vascBlockEnhancedGrayscaleFileNameList = vascRodFilterParameters.vascBlockEnhancedGrayscaleFileNameList;
vascEnhancedBinarizeParameters.vascBlockRawMaskFileNameList = vascBlockRawMaskFileNameList;
vascEnhancedBinarizeParameters.voxelSizeInMicrons = voxelSizeInMicrons;
vascEnhancedBinarizeParameters.displayFlag = displayFlag;
vascEnhancedBinarizeParameters.rawFraction = vascRawBinarizeParameters.rawFraction;
vascEnhancedBinarizeParameters.useRawMaskOnlyFlag = useRawMaskOnlyFlag;
    vascEnhancedBinarizeParameters = VidaVascBinarizeEnhancedVolume(vascEnhancedBinarizeParameters);
vascBlockEnhancedMaskFileNameList = vascEnhancedBinarizeParameters.vascBlockEnhancedMaskFileNameList;
%-----
%-----    
% Stitch together all binarized vasc subvolume blocks
% disp(['Starting Raw Mask Restitch : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
vascRestitchParameters.fileRootName = fileRootName;
vascRestitchParameters.lastRerunStartDateNum = lastRerunStartDateNum;
vascRestitchParameters.vascSegFilesFolder = vascSegFilesFolder;
vascRestitchParameters.intermediateResultsFolder = intermediateResultsFolder;
vascRestitchParameters.vascStackFileName = stackedFileNameList{2};
vascRestitchParameters.fileSuffix = 'VascRawMask';
vascRestitchParameters.overlapsz = subVolumeOverlapSize;
vascRestitchParameters.blockGrid = blockGrid;
vascRestitchParameters.vascBlockMaskFileNameList = vascBlockRawMaskFileNameList;
vascRestitchParameters.fullStackSize = fullStackSize;
    vascRestitchParameters = VidaBinaryVascRestitch(vascRestitchParameters);   %previously called MarchBinaryCellRestitch(EXPERIMENT);
vascRawMaskFileName = vascRestitchParameters.vascMaskFileName;
%-----


%-----    
% Stitch together all binarized enhanced masks
% disp(['Starting Enhanced Mask Restitch : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
vascRestitchParameters.fileRootName = fileRootName;
vascRestitchParameters.lastRerunStartDateNum = lastRerunStartDateNum;
vascRestitchParameters.vascSegFilesFolder = vascSegFilesFolder;
vascRestitchParameters.intermediateResultsFolder = intermediateResultsFolder;
vascRestitchParameters.vascStackFileName = stackedFileNameList{2};
vascRestitchParameters.fileSuffix = 'VascEnhancedMask';
vascRestitchParameters.overlapsz = subVolumeOverlapSize;
vascRestitchParameters.blockGrid = blockGrid;
vascRestitchParameters.vascBlockMaskFileNameList = vascBlockEnhancedMaskFileNameList;
vascRestitchParameters.fullStackSize = fullStackSize;


%     vascRestitchParameters = VidaBinaryVascRestitch(vascRestitchParameters);   %previously called MarchBinaryCellRestitch(EXPERIMENT);
    disp('Now stitching enhanced mask by intersection');
    vascRestitchParameters = VidaBinaryVascRestitchByIntersect(vascRestitchParameters); %changed on 07/20/2010 - PST
    
vascEnhancedMaskFileName = vascRestitchParameters.vascMaskFileName;
%-----
enhancedVascRestitchFlag = 1;%noteToSelf
%-----    
% Stitch together all enhanced vasc subvolume blocks
if enhancedVascRestitchFlag == 1,
    disp(['Starting EnhancedVascRestitch : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
    vascEnhancedRestitchParameters.fileRootName = fileRootName;
    vascEnhancedRestitchParameters.lastRerunStartDateNum = lastRerunStartDateNum;
    vascEnhancedRestitchParameters.vascSegFilesFolder = vascSegFilesFolder;
    vascEnhancedRestitchParameters.intermediateResultsFolder = intermediateResultsFolder;
    vascEnhancedRestitchParameters.vascFileName = stackedFileNameList{2};
    vascEnhancedRestitchParameters.fileSuffix = '-VascEnhancedGrayscale';
    vascEnhancedRestitchParameters.overlapsz = subVolumeOverlapSize;
    vascEnhancedRestitchParameters.blockGrid = blockGrid;
    vascEnhancedRestitchParameters.vascBlockGrayscaleFileNameList = vascBlockEnhancedGrayscaleFileNameList;
    vascEnhancedRestitchParameters.fullStackSize = fullStackSize;
        vascEnhancedRestitchParameters = VidaEnhancedVascRestitch(vascEnhancedRestitchParameters);   %previously called MarchBinaryCellRestitch(EXPERIMENT);
    vascEnhancedGrayscaleFileName = vascEnhancedRestitchParameters.outputStackFileName;
else
    vascEnhancedRestitchParameters = [];
    vascEnhancedGrayscaleFileName = 'Enhanced Grayscale Volume not saved';
end%if enhancedVascRestitchFlag == 1,





%----
outExp = inExp;
%---
outExp.globalParameters.vascBlockEnhancedGrayscaleFileNameList = vascBlockEnhancedGrayscaleFileNameList;
outExp.globalParameters.vascBlockRawMaskFileNameList = vascBlockRawMaskFileNameList;
outExp.globalParameters.vascBlockEnhancedMaskFileNameList = vascBlockEnhancedMaskFileNameList;
%-----
outExp.globalParameters.vascRawMaskFileName = vascRawMaskFileName;
outExp.globalParameters.vascEnhancedMaskFileName = vascEnhancedMaskFileName;
outExp.globalParameters.vascEnhancedGrayscaleFileName = vascEnhancedGrayscaleFileName;


outExp.completionDates.vascSegCompletionDate = datestr(now,'mm/dd/yy HH:MM:SS');
%---
outExp.vascSegParameters.vascRodFilterParameters = vascRodFilterParameters;
outExp.vascSegParameters.vascEnhancedBinarizeParameters = vascEnhancedBinarizeParameters;
outExp.vascSegParameters.vascRawBinarizeParameters = vascRawBinarizeParameters;
outExp.vascSegParameters.vascEnhancedRestitchParameters = vascEnhancedRestitchParameters;
%---

return

