function outputStructure = QuickTestVascSegVida3Channel(varargin)

%This function requires that you be in a folder with a folder
%that contains a single block of 3-channel data
%PLUS dark and fluorescein bath files for all channels
%(9 files altogether - 3 data, 3 dark, 3 fluor)
%
%The code runs the VIDA suite using DefaultQuickTest3Channel parameter set
%up through the vasc channel subtraction.
%it then crops the corrected vasc volume to a user-chosen size
%and runs the QuickTestVidaVascOnlyVascSeg algorithm to help tune vascSeg parameters

%----
%Crop the file to desired size
prompt={'Y1','Y2','X1','X2','Z1','Z2'};
def={'51','200','51','200','26','75'};
dlgTitle='Crop Limits';
lineNo=1;
answer=inputdlg(prompt,dlgTitle,lineNo,def,'on');
cropVolumeLimits(1,1) = str2double(answer(1));
cropVolumeLimits(1,2) = str2double(answer(2));
cropVolumeLimits(2,1) = str2double(answer(3));
cropVolumeLimits(2,2) = str2double(answer(4));
cropVolumeLimits(3,1) = str2double(answer(5));
cropVolumeLimits(3,2) = str2double(answer(6));
vidaProject.cropVolumeLimits = cropVolumeLimits;
%-----
        

myDir = dir('*VidaProject.mat');
if numel(myDir)>0,
    load(myDir(1).name);
    imageStackFolderNameList{1} = vidaProject.imageStackFolderNameList;
else
    [SUCCESS,MESSAGE,MESSAGEID] = mkdir(cd,'OrigFiles'); 
    myDir2 = dir('*-dark-Ch1.tif');
    darkFileName1 = myDir2(1).name;
    rootName = darkFileName1(1:findstr(darkFileName1,'-dark')-1);
    myDir3 = dir([rootName,'*']);
    numFiles = numel(myDir3);
    for k = 1:numFiles,
        oldFileName = myDir3(k).name;
        darkFlag = ~isempty(findstr(oldFileName,'dark'));
        fluorFlag = ~isempty(findstr(oldFileName,'fluor'));
        mypics = loadmovie(oldFileName);
        cp = cropVolumeLimits;
        if or(darkFlag,fluorFlag),
            mypics = mypics(cp(1,1):cp(1,2),cp(2,1):cp(2,2),:);
        else
            mypics = mypics(cp(1,1):cp(1,2),cp(2,1):cp(2,2),cp(3,1):cp(3,2));
        end
        newFileName = regexprep(oldFileName,rootName,'QuickVS');
        moveFileName = fullfile('OrigFiles',oldFileName);
        [STATUS,MESSAGE,MESSAGEID] = movefile(oldFileName,moveFileName);
        maketiff(mypics,newFileName)
    end
end



outputStructure = [];
nin = nargin;
if nin == 0,
    commandVariable = 'current';
else%if nin==0
    commandVariable = varargin{1};
end%if nin==0
%---

parameterSetupFunctionName = commandVariable;

switch parameterSetupFunctionName
    case {'default','Default','DEFAULT','def','Def','DEF'};
        parameterSetupFunctionName = 'DefaultQuickTest3ChannelParameters.mat';
    case {'current','CURRENT','Current'}
        parameterSetupFunctionName = 'DefaultQuickTest3ChannelParameters.mat';
    otherwise
        parameterSetupFunctionName = parameterSetupFunctionName;
end%switch parameterSetupFunctionName
%-----------
%Automatically detect rootCodeFolder and add setupFunctions
rootCodeFolder = fileparts(which('VIDA'));
addpath(fullfile(rootCodeFolder,'SetupFunctionsFolder'))
%-----
    outputProject = VidaAutoDectectVidaProjectRootFolder([]);
vidaProjectRootName = outputProject.vidaProjectRootName;
vidaProjectRootFolder = outputProject.vidaProjectRootFolder;
%----------
%SetupParameters
defaultOverwriteFilesStateFlag = 0;
vidaProjectStyleFlag = 1;
useTifStorageFlag = 0; % use variable imageStack in Mat file
skipAutoUpgradeFlag = 0;
debugModeAvoidOverwriteFlag = 1; %set to 1 to avoid overwriting all files experiment file is deleted

deleteAfterCompletedFlags.deleteOffsetTif = 1;
deleteAfterCompletedFlags.deleteSubBlockFiles = 1;
deleteAfterCompletedFlags.deleteVascBlockChannelSubFiles = 1;
deleteAfterCompletedFlags.deleteNeunBlockChannelSubFiles = 1;
deleteAfterCompletedFlags.deleteCellSeg = 1;
deleteAfterCompletedFlags.deleteVascSegFiles = 0;
deleteAfterCompletedFlags.deleteCenterlineFiles = 0;
deleteAfterCompletedFlags.deleteVascRadiiFiles = 0;

createTifFlags.createRawMaskTif = 0;
createTifFlags.createEnhMaskTif = 0;
createTifFlags.createCenterlineMaskTif = 0;

%-----

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp([' ';' ';' ']);
disp(['VIDA Script Started at ',datestr(now)]);
disp([' ';' ';' ']);
%--------------------------------------------------------------------------


vidaProject.vidaProjectRootName = vidaProjectRootName;
vidaProject.vidaProjectRootFolder = vidaProjectRootFolder;
vidaProject.rootCodeFolder = rootCodeFolder;
%---
vidaProject.channelFlags.neun = 1;
vidaProject.channelFlags.cell = 1;
vidaProject.channelFlags.vasc = 1;
%---
vidaProject.progressFlags.initialized = 1;
vidaProject.progressFlags.sortOriginalTifs = 0;
vidaProject.progressFlags.createUniformityFilter = 0;
vidaProject.progressFlags.processImageStacksCompleted = 0;
vidaProject.progressFlags.multiStackProcessingCompleted = 0;


%Initialize the VidaProject file with folder names and setup Parameters
%This step initializes the vidaProject structure, but does NOT save to disk
vidaProject.setupParameters.vidaProjectRootName = vidaProjectRootName;
vidaProject.setupParameters.vidaProjectRootFolder = vidaProjectRootFolder;
vidaProject.setupParameters.parameterSetupFunctionName = parameterSetupFunctionName;
vidaProject.setupParameters.defaultOverwriteFilesStateFlag = defaultOverwriteFilesStateFlag;
vidaProject.setupParameters.useTifStorageFlag = useTifStorageFlag;
vidaProject.setupParameters.skipAutoUpgradeFlag = skipAutoUpgradeFlag;
vidaProject.setupParameters.defaultOverwriteFilesStateFlag = defaultOverwriteFilesStateFlag;
    vidaProject = VidaSetupVidaProjectFile(vidaProject);
revisedCodeFolders = vidaProject.setupParameters.revisedCodeFolders;
vidaProjectFileName = vidaProject.vidaProjectFileName;

%-----
experimentTemplate = vidaProject;
experimentTemplate.deleteAfterCompletedFlags = deleteAfterCompletedFlags;
experimentTemplate.createTifFlags = createTifFlags;
%-----
vidaProject.experimentTemplate = experimentTemplate;


%Add all code Folder to Matlab path
codeFolders = experimentTemplate.codeFolders;
codeFolderList = fieldnames(experimentTemplate.codeFolders);
numCodeFolders = size(codeFolderList,1);
for codeFolderIter = 1:numCodeFolders,
    currentField = codeFolderList{codeFolderIter};
    currentCodePath = experimentTemplate.codeFolders.(currentField);
    addpath(currentCodePath)
end
    


%LOAD PARAMETERS
%this step further initializes the vidaProject, but does NOT save to disk
%first, check to see if parameterSetupFunctionName is a .m file or .mat file
continueFlag = 0;
if strcmpi(parameterSetupFunctionName(end-1:end),'.m');
    [junkPath,fileNameOnly] = fileparts(parameterSetupFunctionName);
    loadParamCommand = ['experimentTemplate = ',fileNameOnly,'(experimentTemplate);'];
    eval(loadParamCommand)
    disp(['LOADING PARAMETER SET: ',parameterSetupFunctionName]);
    continueFlag = 1;
end%if parameterSetupFunctionName(end-1:end) == '.m',
%---
if strcmpi(parameterSetupFunctionName(end-3:end),'.mat'),
    load(parameterSetupFunctionName,'experimentTemplate');
    continueFlag = 1;
end%if parameterSetupFunctionName(end-3:end) == '.mat',
%---
if continueFlag == 0,
    disp('NOT A VALID PARAMETER SETUP FILE');
    keyboard
end

experimentTemplate.globalParameters = experimentTemplate.basicParameters;
vidaProject.globalParameters = experimentTemplate.basicParameters;
%-----
%update:
if ~isfield(experimentTemplate.basicParameters,'delayMultiplier')
    experimentTemplate.basicParameters.delayMultiplier = 1;
end%if ~isfield ...
%-----
vidaProject.experimentTemplate = experimentTemplate;

%Some specifc global-scale parameters may have been overwritten when the generic experiment Template
%was loaded.  Re-write these specific parameters to the experiment_Template
experimentTemplate.vidaProjectRootName = vidaProjectRootName;
experimentTemplate.vidaProjectRootFolder = vidaProjectRootFolder;
experimentTemplate.rootCodeFolder = rootCodeFolder;
experimentTemplate.setupParameters.vidaProjectRootName = vidaProjectRootName;
experimentTemplate.setupParameters.vidaProjectRootFolder = vidaProjectRootFolder;
experimentTemplate.setupParameters.parameterSetupFunctionName = parameterSetupFunctionName;
experimentTemplate.setupParameters.defaultOverwriteFilesStateFlag = defaultOverwriteFilesStateFlag;
experimentTemplate.setupParameters.useTifStorageFlag = useTifStorageFlag;
experimentTemplate.setupParameters.skipAutoUpgradeFlag = skipAutoUpgradeFlag;
experimentTemplate.setupParameters.defaultOverwriteFilesStateFlag = defaultOverwriteFilesStateFlag;
experimentTemplate.setupParameters.revisedCodeFolders = revisedCodeFolders;
experimentTemplate.vidaProjectFileName = vidaProjectFileName;
%---
experimentTemplate.deleteAfterCompletedFlags = deleteAfterCompletedFlags;
experimentTemplate.createTifFlags = createTifFlags;
experimentTemplate.codeFolders = codeFolders;
%-----
%-----



%AutoAdjust the rootfolder in any existing VidaProject file in case folder has moved
%this step saves the changes in the vidaProject file to disk
    vidaProject = VidaAutoAdjustVidaProjectRootFolder(vidaProject);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%DATE STAMP VIDAPROJECT FILE OR LOAD EXISTING VIDAPROJECT FILE
%This step overloads the iniatilzed vidaProject file if one exists on disk
vidaProject.setupParameters.debugModeAvoidOverwriteFlag = debugModeAvoidOverwriteFlag;
    vidaProject = VidaLoadVidaProjectFile(vidaProject);
vidaProject.codeFolders = revisedCodeFolders;
experimentTemplate = vidaProject.experimentTemplate;
experimentTemplate.codeFolders = revisedCodeFolders;
%-----
vidaProject.experimentTemplate = experimentTemplate;
%----------
%Copy the parameters relating to the full Project to the vidaProject structure
if ~isfield(vidaProject,'registrationParameters'),
    vidaProject.registrationParameters = experimentTemplate.registrationParameters;
end%if ~isfield(vidaProject,'registrationParameters'),
%---
if ~isfield(vidaProject,'strandifyParameters'),
    vidaProject.strandifyParameters = experimentTemplate.strandifyParameters;
end%if ~isfield(vidaProject,'strandifyParameters'),
%---
saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save(vidaProjectFileName,'vidaProject',scs);
preMatRoot = vidaProjectFileName(1:findstr(vidaProjectFileName,'.mat')-1);
save([preMatRoot,'Backup',datestr(now,'mmddyy-HHMM'),'.mat'],'vidaProject',scs);
%----------
%--------
%STAGE 1 : Determine number of image stacks - allocate each to a folder
%   SORT .TIF FILES INTO INDIVIDUAL FOLDERS IF NECESSARY
disp('Sorting Files');
if vidaProject.progressFlags.sortOriginalTifs == 0,
    vidaProject.globalParameters.reverseXBlockNumberingFlag = vidaProject.registrationParameters.reverseXBlockNumberingFlag;
    vidaProject.globalParameters.reverseYBlockNumberingFlag = vidaProject.registrationParameters.reverseYBlockNumberingFlag;
        vidaProject = VidaSortOriginalTifs(vidaProject);
    vidaProject.progressFlags.sortOriginalTifs = 1; 
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
end%if vidaProject.progressFlags.sortOriginalTifs == 0,
imageStackFolderNameList = vidaProject.imageStackFolderNameList;
numImageStacks = size(imageStackFolderNameList,2);

vidaProject.imageStackFolderNameList = imageStackFolderNameList;
vidaProject.setupParameters.currentImageStackFolderName = imageStackFolderNameList{1};
numImageStacks = 1;


%--------
%--------
%STAGE 2 : Create uniformity filter for preprocessing
disp('Uniformity files');
if vidaProject.progressFlags.createUniformityFilter == 0,
        vidaProject = VidaCreateUniformityFilter(vidaProject);
    vidaProject.progressFlags.createUniformityFilter = 1;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
end%if vidaProject.progressFlags.createUniformityFilter == 0,
%-----

%-----
%STAGE 4 : Check out an individual image stack, and run processing.
    fprintf('\nBegin Processing of individual image stacks\n\n');
    vidaProject.multiThreadProgress.imageStackProcessingFlag = 1;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
        vidaProject = QuickTestVascSegImageStack3Channel(vidaProject);
    vidaProject.multiThreadProgress.imageStackProcessingFlag = 2;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
 


