function outputParameters = GeneratePointerToBlocks(inputParameters)
%generate block names, padded with zeros to two digits

baseName = inputParameters.vidaProjectRootName;
rootFolder = inputParameters.vidaProjectRootFolder;
firstBlkRow = inputParameters.firstBlkRow;
firstBlkCol = inputParameters.firstBlkCol;
firstBlkZpos = inputParameters.firstBlkZpos;
reverseRows = inputParameters.reverseYBlockNumberingFlag;
reverseCols = inputParameters.reverseXBlockNumberingFlag;
nblks1 = inputParameters.nblks1;
nblks2 = inputParameters.nblks2;
nblks3 = inputParameters.nblks3;
maskFileToUseForRegistration = inputParameters.maskFileToUseForRegistration;


nBlocks = nblks1 * nblks2 * nblks3;
blockFileNameList = cell(nBlocks,1);

%generate block RC matrix
r = firstBlkRow : firstBlkRow + (nblks1-1);
c = firstBlkCol : firstBlkCol + (nblks2-1);
z = firstBlkZpos : firstBlkZpos + (nblks3-1);

if reverseRows; r = fliplr(r);end
if reverseCols; c = fliplr(c);end

%generate rcz index for each block, used for block naming
[R C] = meshgrid(r,c);
R = R';C = C';
R = repmat(R(:),nblks3,1);
C = repmat(C(:),nblks3,1);
Z = repmat(z,nblks1 * nblks2,1);
Z = Z(:);
useTiff = numel(maskFileToUseForRegistration) == 0; % required for on the fly nameing

RCZList = zeros(nBlocks,3);
for iBLOCK = 1 : nBlocks
    rStr = num2str(R(iBLOCK));if R(iBLOCK) <= 9; rStr = ['0' rStr];end %#ok<AGROW>
    cStr = num2str(C(iBLOCK));if C(iBLOCK) <= 9; cStr = ['0' cStr];end %#ok<AGROW>
    zStr = num2str(Z(iBLOCK));if Z(iBLOCK) <= 9; zStr = ['0' zStr];end %#ok<AGROW>
    blockName = [baseName '-Z' zStr  '-C' cStr '-R' rStr];
    completeBlockName = [blockName,'-',maskFileToUseForRegistration,'.mat'];
    path2block = fullfile(rootFolder,['ImageStack-',blockName],'ProcessedData',completeBlockName); %#ok<NASGU>
    blockFileNameList{iBLOCK} = path2block;
    RCZList(iBLOCK,:) = [R(iBLOCK),C(iBLOCK),Z(iBLOCK)];
end


outputParameters = inputParameters;
outputParameters.blockFileNameList = blockFileNameList;
outputParameters.RCZList = RCZList;
