function [dML,dML_proj] = GetMinEnergyPairwiseOffsetEstimates(MarginalOffsetEnergies)

dML = struct([]);
dML_proj = struct([]);

COMPUTE_2DPROJECTION_OFFSETS = false;

for iMOES = 1:numel(MarginalOffsetEnergies)
    if isfield(MarginalOffsetEnergies,'Offset3DEnergyMinPos'),
        %Minimum position stored in lieu of full 3D-MOES in MarginalOffsetEnergies
        Offset3DEnergyMinPos = MarginalOffsetEnergies(iMOES).Offset3DEnergyMinPos;
        Offset3DEnergySize = MarginalOffsetEnergies(iMOES).Offset3DEnergySize;
        minx1 = Offset3DEnergyMinPos(1);
        minx2 = Offset3DEnergyMinPos(2);
        minx3 = Offset3DEnergyMinPos(3);
    else%if isfield(MarginalOffsetEnergies,Offset3DMinPos),
        %Minimum position not stored - calculate from 3D-MOES now.
        Etmp = MarginalOffsetEnergies(iMOES).Offset3DEnergy;
        [minE,minEi] = min(Etmp(:));
        [minx1,minx2,minx3] = ind2sub(size(Etmp),minEi);
    end%if-else isfield(MarginalOffsetEnergies,Offset3DMinPos),
    % get the delta estimates for the 3D projection sum energies (i.e. subtract off the zero offset index).
    dML(iMOES).off = [minx1,minx2,minx3] - MarginalOffsetEnergies(iMOES).ZeroOffsetLoc3D;
    
    if COMPUTE_2DPROJECTION_OFFSETS
        % This section of code can compare the 3D max locations to the
        % projections the 3D energies were derived from
        [minE_x1x2,minE_x1x2i] = min(MarginalOffsetEnergies(iMOES).MOES_x1x2(:));
        [minx1_x1x2,minx2_x1x2,IGNOREminx3_x1x2] = ind2sub(size(MarginalOffsetEnergies(iMOES).MOES_x1x2),minE_x1x2i);
        [minE_x1x3,minE_x1x3i] = min(MarginalOffsetEnergies(iMOES).MOES_x1x3(:));
        [minx1_x1x3,IGNOREminx2_x1x3,minx3_x1x3] = ind2sub(size(MarginalOffsetEnergies(iMOES).MOES_x1x3),minE_x1x3i);
        [minE_x2x3,minE_x2x3i] = min(MarginalOffsetEnergies(iMOES).MOES_x2x3(:));
        [IGNOREminx1_x2x3,minx2_x2x3,minx3_x2x3] = ind2sub(size(MarginalOffsetEnergies(iMOES).MOES_x2x3),minE_x2x3i);
        
        % get the delta estimates for the projections to compare to those for
        % the 3D energies. If they are significantly different, understand the
        % cause.
        dML_proj(iMOES).off = [(minx1_x1x2 + minx1_x1x3)/2,(minx2_x1x2 + minx2_x2x3)/2,(minx3_x1x3 + minx3_x2x3)/2] - MarginalOffsetEnergies(iMOES).ZeroOffsetLoc3D;
        
        fprintf(1,'The 3D offset estimate is: %4.1f %4.1f %4.1f \n',dML(iMOES).off(1),dML(iMOES).off(2),dML(iMOES).off(3));
        fprintf(1,'The 2D offset estimate is: %4.1f %4.1f %4.1f \n',dML_proj(iMOES).off(1),dML_proj(iMOES).off(2),dML_proj(iMOES).off(3));
    end
end

if COMPUTE_2DPROJECTION_OFFSETS
    minE_residual_offset_diffs = [dML.off] - [dML_proj.off];
    fprintf(1,'The average absolute 3D/2D offset difference is: %4.3f \n',mean(abs(minE_residual_offset_diffs)));
    fprintf(1,'The maximum absolute 3D/2D offset difference is: %4.1f \n',max(abs(minE_residual_offset_diffs)));
end
