function MOES = GetXCorrelationViaSlidingWindows(Pi,Pj,MOESsz)
%         if Center_ij(iAB) == 1  % B_center is Bi
%             Delta_ij = (Blocks(Bi).corner_coords + off_cand) - Blocks(Bj).corner_coords - phi_BiBj + MarginalOffsetEnergies(BiBj_ind).ZeroOffsetLoc3D;
%         else                    % B_center is Bj
%             Delta_ij = Blocks(Bi).corner_coords - (Blocks(Bj).corner_coords + off_cand) - phi_BiBj + MarginalOffsetEnergies(BiBj_ind).ZeroOffsetLoc3D;
%         end

USE_LOWDIM_OVERLAP = true; % This speeds up indexing

Pi_normal = (Pi - mean(Pi(:)))/std(Pi(:));
Pj_normal = (Pj - mean(Pj(:)))/std(Pj(:));

if numel(size(Pi)) == 2
    %% COMPUTE MOES_x1x2
    MOES = zeros(MOESsz,MOESsz);
    midpt = ceil(MOESsz/2);
    for iM1 = 1:MOESsz
        off1 = iM1 - midpt;
        for iM2 = 1:MOESsz
            off2 = iM2 - midpt;
            if ~USE_LOWDIM_OVERLAP
                [xi1,xi2] = ndgrid(1:size(Pi,1),1:size(Pi,2));
                xj1 = xi1 - off1;    % cj = ci - phi_ij - Delta_ij
                xj2 = xi2 - off2;
                [Overlaps,Oi,Oj] = intersect([xi1(:),xi2(:)],[xj1(:),xj2(:)],'rows');
                % Compute the overlap region dimensions
                Overlap_r1 = max(Overlaps(:,1)) - min(Overlaps(:,1)) + 1;
                Overlap_r2 = max(Overlaps(:,2)) - min(Overlaps(:,2)) + 1;
                % Get the overlap data in block i
                Pi_overlap = single(reshape(Pi_normal(Oi),[Overlap_r2,Overlap_r1]));
                % Get the overlap data in block j
                Pj_overlap = single(reshape(Pj_normal(Oj),[Overlap_r2,Overlap_r1]));
            else
                xi1 = [1:size(Pi,1)];
                xi2 = [1:size(Pi,2)];
                xj1 = xi1 - off1;    % cj = ci - phi_ij - Delta_ij
                xj2 = xi2 - off2;
                [x1O,x1Oi,x1Oj] = intersect(xi1,xj1);
                [x2O,x2Oi,x2Oj] = intersect(xi2,xj2);
                % Compute the overlap region dimensions
                Overlap_r1 = max(x1O) - min(x1O) + 1;
                Overlap_r2 = max(x2O) - min(x2O) + 1;
                % Get the overlap data in block i
                Pi_overlap = single(Pi_normal(x1Oi,x2Oi));
                Pj_overlap = single(Pj_normal(x1Oj,x2Oj));
            end
            CCtmp = corrcoef(Pi_overlap(:),Pj_overlap(:));
            MOES(iM1,iM2) = CCtmp(1,2);
        end
    end
else
    if size(Pi,2) == 1
        %% COMPUTE MOES_x1x3
        MOES = zeros(MOESsz,1,MOESsz);
        midpt = ceil(MOESsz/2);
        for iM1 = 1:MOESsz
            off1 = iM1 - midpt;
            for iM3 = 1:MOESsz
                off3 = iM3 - midpt;
                if ~USE_LOWDIM_OVERLAP
                    [xi1,xi2,xi3] = ndgrid(1:size(Pi,1),1:size(Pi,2),1:size(Pi,3));
                    xj2 = xi2; % the 2nd dimension has no offset when computing MOES_x1x3
                    xj1 = xi1 - off1;    % cj = ci - phi_ij - Delta_ij
                    xj3 = xi3 - off3;
                    [Overlaps,Oi,Oj] = intersect([xi1(:),xi2(:),xi3(:)],[xj1(:),xj2(:),xj3(:)],'rows');
                    % Compute the overlap region dimensions
                    Overlap_r1 = max(Overlaps(:,1)) - min(Overlaps(:,1)) + 1;
                    Overlap_r2 = max(Overlaps(:,2)) - min(Overlaps(:,2)) + 1;
                    Overlap_r3 = max(Overlaps(:,3)) - min(Overlaps(:,3)) + 1;
                    % Get the overlap data in block i
                    Pi_overlap = single(reshape(Pi_normal(Oi),[Overlap_r3,Overlap_r2,Overlap_r1]));
                    % Get the overlap data in block j
                    Pj_overlap = single(reshape(Pj_normal(Oj),[Overlap_r3,Overlap_r2,Overlap_r1]));
                else
                    xi1 = [1:size(Pi,1)];
                    xi2 = [1:size(Pi,2)];
                    xi3 = [1:size(Pi,3)];
                    xj2 = xi2; % the 2nd dimension has no offset when computing MOES_x1x3
                    xj1 = xi1 - off1;    % cj = ci - phi_ij - Delta_ij
                    xj3 = xi3 - off3;
                    [x1O,x1Oi,x1Oj] = intersect(xi1,xj1);
                    [x3O,x3Oi,x3Oj] = intersect(xi3,xj3);
                    % Compute the overlap region dimensions
                    Overlap_r1 = max(x1O) - min(x1O) + 1;
                    Overlap_r3 = max(x3O) - min(x3O) + 1;
                    % Get the overlap data in block i
                    Pi_overlap = single(Pi_normal(x1Oi,1,x3Oi));
                    Pj_overlap = single(Pj_normal(x1Oj,1,x3Oj));
                end
                CCtmp = corrcoef(Pi_overlap(:),Pj_overlap(:));
                MOES(iM1,1,iM3) = CCtmp(1,2);
            end
        end
    else
        %% COMPUTE MOES_x2x3
        MOES = zeros(1,MOESsz,MOESsz);
        midpt = ceil(MOESsz/2);
        for iM2 = 1:MOESsz
            off2 = iM2 - midpt;
            for iM3 = 1:MOESsz
                off3 = iM3 - midpt;
                if ~USE_LOWDIM_OVERLAP
                    [xi1,xi2,xi3] = ndgrid(1:size(Pi,1),1:size(Pi,2),1:size(Pi,3));
                    xj1 = xi1; % the 1st dimension has no offset when computing MOES_x2x3
                    xj2 = xi2 - off2;    % cj = ci - phi_ij - Delta_ij
                    xj3 = xi3 - off3;
                    [Overlaps,Oi,Oj] = intersect([xi1(:),xi2(:),xi3(:)],[xj1(:),xj2(:),xj3(:)],'rows');
                    % Compute the overlap region dimensions
                    Overlap_r1 = max(Overlaps(:,1)) - min(Overlaps(:,1)) + 1;
                    Overlap_r2 = max(Overlaps(:,2)) - min(Overlaps(:,2)) + 1;
                    Overlap_r3 = max(Overlaps(:,3)) - min(Overlaps(:,3)) + 1;
                    % Get the overlap data in block i
                    Pi_overlap = single(reshape(Pi_normal(Oi),[Overlap_r3,Overlap_r2,Overlap_r1]));
                    % Get the overlap data in block j
                    Pj_overlap = single(reshape(Pj_normal(Oj),[Overlap_r3,Overlap_r2,Overlap_r1]));
                else
                    xi1 = [1:size(Pi,1)];
                    xi2 = [1:size(Pi,2)];
                    xi3 = [1:size(Pi,3)];
                    xj1 = xi1; % the 1st dimension has no offset when computing MOES_x2x3
                    xj2 = xi2 - off2;    % cj = ci - phi_ij - Delta_ij
                    xj3 = xi3 - off3;
                    [x2O,x2Oi,x2Oj] = intersect(xi2,xj2);
                    [x3O,x3Oi,x3Oj] = intersect(xi3,xj3);
                    % Compute the overlap region dimensions
                    Overlap_r2 = max(x2O) - min(x2O) + 1;
                    Overlap_r3 = max(x3O) - min(x3O) + 1;
                    % Get the overlap data in block i
                    Pi_overlap = single(Pi_normal(1,x2Oi,x3Oi));
                    Pj_overlap = single(Pj_normal(1,x2Oj,x3Oj));
                end
                CCtmp = corrcoef(Pi_overlap(:),Pj_overlap(:));
                MOES(1,iM2,iM3) = CCtmp(1,2);
            end
        end
    end
end

MOES = 1 - MOES;  % Make correlation coefficients into an energy surface with 0 as minimum
