function [Blocks,A,r,dMinE] = InitializeBlockCornersViaMOES(Blocks,BlockConfig,Offsets,MarginalOffsetEnergies)

% Create A matrix in Ac = d+r (where d+r are observations)
Atmp3 = [3*BlockConfig.all_block_pairs,3*(1:size(BlockConfig.all_block_pairs,1))'];
Atmp2 = Atmp3-1;
Atmp1 = Atmp3-2;
Ainds1 = [Atmp3(:,3);Atmp3(:,3);Atmp2(:,3);Atmp2(:,3);Atmp1(:,3);Atmp1(:,3)];
Ainds2 = [Atmp3(:,1);Atmp3(:,2);Atmp2(:,1);Atmp2(:,2);Atmp1(:,1);Atmp1(:,2)];
A1s = ones(size(Atmp3(:,1)));
Ainds3 = [A1s;-A1s;A1s;-A1s;A1s;-A1s];
A = sparse(Ainds1,Ainds2,Ainds3);

c = [Blocks.corner_coords]';
r = [Offsets.offs]';
% % If problems, check that this relation is true: Ac = r, so Ac-r = 0
% A*c - r

% A'A has reduced rank (n-3 center parameters) because one corner location must be set by
% hand... The first corner will be set to 1,1,1 by augmenting A on the LHS
% and by adding a constant c0 on the RHS
Aa = sparse(size(A,1)+3,size(A,2));
Aa(1:3,1:3) = speye(3);
Aa(4:end,:) = A;

% Augment relative offset vector and create c0 vector to fix c1(x,y,z)
ra = [zeros(3,1);r];
c0 = [c(1:3);zeros(size(r,1),1)];

% chat = inv(Aa'*Aa)*Aa'*(ra+c0);  % This should be the same as the initial c computed from the BlockConfig for zero offsets

% Adding in the least squares locations of all ML offset estimates:
dML = GetMinEnergyPairwiseOffsetEstimates(MarginalOffsetEnergies);
%dvec = [zeros(3,1);-[dML.off]'];   % SWAP SIGN OF Deltas TO MATCH MOES
dvec = [zeros(3,1);[dML.off]'];   
%dvec(3:3:end) = -dvec(3:3:end);     % this swaps the residual offset signs of dim 3 

% Set weighting matrix for WLS calculation of block corner locations
Rinv = speye(size(ra,1));
Corner1Weight = 1000;
Rinv(1:3,1:3) = Corner1Weight*Rinv(1:3,1:3);
%chat = round(inv(Aa'*Aa)*Aa'*(ra+c0+dvec));  % This should estimate block corner locations consistent with offsets (in a least squares sense)
chat = round(inv(Aa'*Rinv*Aa)*Aa'*Rinv*(ra+c0+dvec));  % This should estimate block corner locations consistent with offsets (in a least squares sense)
chat = reshape(chat,3,numel(Blocks));

% chat is a set of physical corner locations. r+dvec(4:end) is the list of
% pairwise offsets that would be chosen by each block pair if block
% pairs didn't have to eventually agree on individual corner locations
ML_pairwise_physical_conflicts = ((A*chat(:) - (r+dvec(4:end))));

LLSdistance = sqrt(sum(ML_pairwise_physical_conflicts.^2))/numel(ML_pairwise_physical_conflicts);
fprintf(1,'The RMS offset difference between the pairwise offset and physical corner locations is %3.3f \n',LLSdistance);

% Redistribute corner locations to blocks
for iBLOCK = 1:numel(Blocks)
    Blocks(iBLOCK).corner_coords(1) = chat(1,iBLOCK);
    Blocks(iBLOCK).corner_coords(2) = chat(2,iBLOCK);
    Blocks(iBLOCK).corner_coords(3) = chat(3,iBLOCK);
end

dMinE = [dML.off]';