function outputParameters = VidaCalculateSingleMOES(inputParameters)
multiThreadParameters = inputParameters;
registrationParameters = multiThreadParameters.registrationParameters;
MOESParameters = multiThreadParameters.MOESParameters;
%-----
emptyBlockCorrectionFlag = registrationParameters.emptyBlockCorrectionFlag;
emptyBlockThreshold = registrationParameters.emptyBlockThreshold;
%-----
Blocks = MOESParameters.Blocks;
BlockConfig = MOESParameters.BlockConfig;
Offsets = MOESParameters.Offsets;
yxzSearchSize = MOESParameters.yxzSearchSize;
currentAdjacentPairIndex = MOESParameters.currentAdjacentPairIndex;
registrationDataFileName = multiThreadParameters.registrationDataFileName;
%-----
MOESsz = yxzSearchSize(1);
%-----
MarginalOffsetEnergies = struct([]);
MOESmidpt = ceil(MOESsz/2);

USE_LOWDIM_INDEXING = true;

fprintf(1,'Populating Marginal Offset Energy %d of %d. %s\n',currentAdjacentPairIndex,numel(Offsets),datestr(now));
% Get indices of both blocks
Bi = Offsets(currentAdjacentPairIndex).offs_blocks(1);
Bj = Offsets(currentAdjacentPairIndex).offs_blocks(2);
% Get offset block i to block j
phi_BiBj = Offsets(currentAdjacentPairIndex).offs;
% Get each block's size
blksz = Blocks(Bi).block_size;
% Get each block's data
%------
loadedStruct = load(Blocks(Bi).block_data_fname);
loadedVariables = fieldnames(loadedStruct);
blockIData = loadedStruct.(loadedVariables{1});
if isstruct(blockIData),
    tempPixelList = blockIData.pixelList;
    tempMaskSize = blockIData.maskSize;
    blockIData = false(tempMaskSize);
    blockIData(tempPixelList) = 1;
    clear tempPixelList tempMaskSize
end%if isstruct(blockIData),
%-----
loadedStruct = load(Blocks(Bj).block_data_fname);
loadedVariables = fieldnames(loadedStruct);
blockJData = loadedStruct.(loadedVariables{1});
if isstruct(blockJData),
    tempPixelList = blockJData.pixelList;
    tempMaskSize = blockJData.maskSize;
    blockJData = false(tempMaskSize);
    blockJData(tempPixelList) = 1;
    clear tempPixelList tempMaskSize
end%if isstruct(blockJData),
%-----
if ~USE_LOWDIM_INDEXING
    % Choose block i's local coordinate system to begin at 1 1 1
    [x1i,x2i,x3i] = ndgrid(1:blksz(1),1:blksz(2),1:blksz(3));
    % Choose block j's local coordinate system in reference to i's
    [x1j,x2j,x3j] = ndgrid((1:blksz(1))-phi_BiBj(1),(1:blksz(2))-phi_BiBj(2),(1:blksz(3))-phi_BiBj(3));
    % Compute the overlap region
    [Overlaps,Oi,Oj] = intersect([x1i(:),x2i(:),x3i(:)],[x1j(:),x2j(:),x3j(:)],'rows');
    %clear x1i x2i x3i x1j x2j x3j
    % Compute the overlap region dimensions
    Overlap_r1 = max(Overlaps(:,1)) - min(Overlaps(:,1)) + 1;
    Overlap_r2 = max(Overlaps(:,2)) - min(Overlaps(:,2)) + 1;
    Overlap_r3 = max(Overlaps(:,3)) - min(Overlaps(:,3)) + 1;
    %clear Overlaps
    % Get the overlap data in block i
    blockIData = single(reshape(blockIData(Oi),[Overlap_r3,Overlap_r2,Overlap_r1]));
    %clear Oi
    % Get the overlap data in block j
    blockJData = single(reshape(blockJData(Oj),[Overlap_r3,Overlap_r2,Overlap_r1]));
    %clear Oj

    blockIData = permute(blockIData,[3 2 1]);  % fix dimension mix-up with reshape
    blockJData = permute(blockJData,[3 2 1]);  % fix dimension mix-up with reshape
else%if ~USE_LOWDIM_INDEXING
    x1i = [1:blksz(1)];
    x2i = [1:blksz(2)];
    x3i = [1:blksz(3)];
    x1j = x1i-phi_BiBj(1);
    x2j = x2i-phi_BiBj(2);
    x3j = x3i-phi_BiBj(3);
    [x1O,x1Oi,x1Oj] = intersect(x1i,x1j);
    [x2O,x2Oi,x2Oj] = intersect(x2i,x2j);
    [x3O,x3Oi,x3Oj] = intersect(x3i,x3j);
    blockIData = blockIData(x1Oi,x2Oi,x3Oi);
    blockJData = blockJData(x1Oj,x2Oj,x3Oj);
end%if-else ~USE_LOWDIM_INDEXING

% Get projections of each block in the overlap region
Pi_12 = sum(blockIData,3);
Pi_13 = sum(blockIData,2);
Pi_23 = sum(blockIData,1);
Pj_12 = sum(blockJData,3);
Pj_13 = sum(blockJData,2);
Pj_23 = sum(blockJData,1);

% Slide projections according to MOES energy lookup tables -- populate MOES
MOES_x1x2 = GetXCorrelationViaSlidingWindows(Pi_12,Pj_12,MOESsz);
MOES_x1x3 = GetXCorrelationViaSlidingWindows(Pi_13,Pj_13,MOESsz);
MOES_x2x3 = GetXCorrelationViaSlidingWindows(Pi_23,Pj_23,MOESsz);

adjacentPairEnergy.MOES_x1x2 = MOES_x1x2;
adjacentPairEnergy.MOES_x1x3 = MOES_x1x3;
adjacentPairEnergy.MOES_x2x3 = MOES_x2x3;
adjacentPairEnergy.blocks = BlockConfig.all_block_pairs(currentAdjacentPairIndex,:);
adjacentPairEnergy.dims.x1x2 = [1 2]; % indices of dimensions for MOES_x1x2
adjacentPairEnergy.dims.x1x3 = [1 3]; % indices of dimensions for MOES_x1x3
adjacentPairEnergy.dims.x2x3 = [2 3]; % indices of dimensions for MOES_x2x3
adjacentPairEnergy.dim_names = {'x1x2','x1x3','x2x3'}; % dimension name for MOES_x1x2, MOES_x1x3, MOES_x2x3
adjacentPairEnergy.offs_to_MOESloc_x1x2.MOESloc = [MOESmidpt MOESmidpt 1];
adjacentPairEnergy.offs_to_MOESloc_x1x3.MOESloc = [MOESmidpt 1 MOESmidpt];
adjacentPairEnergy.offs_to_MOESloc_x2x3.MOESloc = [1 MOESmidpt MOESmidpt];
adjacentPairEnergy.offs_to_MOESloc_x1x2.offs = [blksz(1) blksz(2) 0];
adjacentPairEnergy.offs_to_MOESloc_x1x3.offs = [blksz(1) 0 blksz(3)];
adjacentPairEnergy.offs_to_MOESloc_x2x3.offs = [0 blksz(2) blksz(3)];









%% COMPUTE 3D OFFSET ENERGY LUT (CONSTANT STRUCT)
                        
MOES = adjacentPairEnergy;
% get all joint offsets (x1s,x2s,x3s) in search extents
searchextents = max([size(adjacentPairEnergy.MOES_x1x2,1),size(adjacentPairEnergy.MOES_x1x3,1),size(adjacentPairEnergy.MOES_x2x3,1);...
    size(adjacentPairEnergy.MOES_x1x2,2),size(adjacentPairEnergy.MOES_x1x3,2),size(adjacentPairEnergy.MOES_x2x3,2);...
    size(adjacentPairEnergy.MOES_x1x2,3),size(adjacentPairEnergy.MOES_x1x3,3),size(adjacentPairEnergy.MOES_x2x3,3)],[],2)';
allblksi = 1:prod(searchextents);
[x1s,x2s,x3s] = ind2sub(searchextents,allblksi);
%-----
E_MOESi = zeros(searchextents);
for iSEARCH = 1:numel(x1s)
    E12 = adjacentPairEnergy.MOES_x1x2(x1s(iSEARCH),x2s(iSEARCH),1); 
    E13 = adjacentPairEnergy.MOES_x1x3(x1s(iSEARCH),1,x3s(iSEARCH));
    E23 = adjacentPairEnergy.MOES_x2x3(1,x2s(iSEARCH),x3s(iSEARCH));
    %-----
    E_MOESi(x1s(iSEARCH),x2s(iSEARCH),x3s(iSEARCH)) = E12 + E13 + E23; % 3D energy is sum of 2D MOES after normalization
end%for iSEARCH = 1:numel(x1s)
[minE,minEi] = min(E_MOESi(:));
[minx1,minx2,minx3] = ind2sub(size(E_MOESi),minEi);
adjacentPairEnergy.Offset3DEnergy = E_MOESi;
adjacentPairEnergy.ZeroOffsetLoc3D = max([adjacentPairEnergy.offs_to_MOESloc_x1x2.MOESloc;adjacentPairEnergy.offs_to_MOESloc_x1x3.MOESloc;adjacentPairEnergy.offs_to_MOESloc_x2x3.MOESloc]);
adjacentPairEnergy.Offset3DEnergySize = size(E_MOESi);
adjacentPairEnergy.Offset3DEnergyMinPos = [minx1,minx2,minx3];

%-----
%Check for correlation to "Empty" blocks:
if emptyBlockCorrectionFlag == 1,
    E = adjacentPairEnergy.Offset3DEnergy;
    minE = min(E(:));
    p1 = E(1,:,:);
    p2 = E(:,1,:); 
    p3 = E(:,:,1);
    p4 = E(end,:,:); 
    p5 = E(:,end,:); 
    p6 = E(:,:,end);
    perimE = [p1(:);p2(:);p3(:);p4(:);p5(:);p6(:)];
    minPerimE = min(perimE(:));
    validMetric = minPerimE - minE;
    
    if or(validMetric < emptyBlockThreshold , isnan(validMetric))
        blockIDs = adjacentPairEnergy.blocks;
        myMessage = ['Interface ',num2str(currentAdjacentPairIndex),' between blocks ',num2str(blockIDs),' appears Blank - defaulting'];
        disp(myMessage);
        ESize = size(E);
        midPoint = ceil(ESize/2);
        defaultE = zeros(ESize);
        defaultE(midPoint(1),midPoint(2),midPoint(3)) = 1;
        defaultE = (bwdist(defaultE).^2);
        normDefaultE = defaultE./max(defaultE(:));
        adjacentPairEnergy.Offset3DEnergy = normDefaultE;
    end%if validMetric < emptyBlockThreshold,
end%if emptyBlockCorrection == 1,

%-----
adjacentPairEnergy.emptyBlockCorrectionFlag = emptyBlockCorrectionFlag;
adjacentPairEnergy.emptyBlockThreshold = emptyBlockThreshold;
%-----
save(registrationDataFileName,'adjacentPairEnergy');
%-----
outputParameters = inputParameters;
outputParameters.adjacentPairEnergy = adjacentPairEnergy;
