function outputParameters = VidaPrepareMOES(inputParameters)
registrationParameters = inputParameters;
blockFileNameList = registrationParameters.blockFileNameList;
adjacentBlockPairList = registrationParameters.adjacentBlockPairList;
adjacentBlockPairOffsetList = registrationParameters.adjacentBlockPairOffsetList;
emptyBlockCorrectionFlag = registrationParameters.emptyBlockCorrectionFlag;
emptyBlockThreshold = registrationParameters.emptyBlockThreshold;
%-----
MOESsz = 41;%noteToSelf



%% COMPUTE CORNER LOCATIONS ASSUMING BLOCK 1 IS AT 1,1,1
% Preallocate corner locations
cornercoords1 = zeros(numel(blockFileNameList),1);
cornercoords2 = zeros(numel(blockFileNameList),1);
cornercoords3 = zeros(numel(blockFileNameList),1);

% Set the first corner to be at 1,1,1
cornercoords1(1) = 1;
cornercoords2(1) = 1;
cornercoords3(1) = 1;

% Estimate all other corner locations for initial config based on corner
% 1--assume all subsequent corner locations of blocks in the 2nd column of
% adjacentBlockPairList are defined by prior corner locations.
for iAB = 1:size(adjacentBlockPairList,1)
    cornercoords1(adjacentBlockPairList(iAB,2)) = cornercoords1(adjacentBlockPairList(iAB,1)) - adjacentBlockPairOffsetList(iAB,1);
    cornercoords2(adjacentBlockPairList(iAB,2)) = cornercoords2(adjacentBlockPairList(iAB,1)) - adjacentBlockPairOffsetList(iAB,2);
    cornercoords3(adjacentBlockPairList(iAB,2)) = cornercoords3(adjacentBlockPairList(iAB,1)) - adjacentBlockPairOffsetList(iAB,3);
end

% Infer BlockConfig from adjacency and offsets (ASSUMES REGULAR GRID
% OFFSETTING ON DIMENSIONS 1 AND 2) 3rd dimension can be nonuniformly
% spaced (must be monotonic).
% Assumes grid forms a lattice aligned with 3 coordinate directions (i.e.
% nominal offsets do not skew (at all)). Some drift is acceptable, but
% total drift over any dimension cannot exceed the spacing for that
% direction.

MaxAbsAdjacentOffsets = max(abs(adjacentBlockPairOffsetList),[],1);
% Minimum offset determines what range of corner coordinates will lead to a
% specific block configuration
MinimumOffsetRequiredForBlockIndexing = 15;%noteToSelf

if MaxAbsAdjacentOffsets(1) > MinimumOffsetRequiredForBlockIndexing
    BlockInds1 = round(cornercoords1/MaxAbsAdjacentOffsets(1)) + 1;
else
    BlockInds1 = ones(size(cornercoords1));
end


if MaxAbsAdjacentOffsets(2) > MinimumOffsetRequiredForBlockIndexing
    BlockInds2 = round(cornercoords2/MaxAbsAdjacentOffsets(2)) + 1;
else
    BlockInds2 = ones(size(cornercoords2));
end

if MaxAbsAdjacentOffsets(3) > MinimumOffsetRequiredForBlockIndexing
    %BlockInds3 = round(cornercoords3/MaxAbsAdjacentOffsets(3)) + 1;
    BlockInds3 = round(cornercoords3/MinimumOffsetRequiredForBlockIndexing) + 1;
    [bis3,bis3s] = unique(BlockInds3);
    if bis3s(1) == min(bis3s)    % if when sorted the first number is the lowest (closest to index 1), then indexing is "positive"
        [ubis3,ubis3s] = sort(bis3s,'ascend');
    else
        [ubis3,ubis3s] = sort(bis3s,'descend');
    end
    BlockInds3 = ubis3s;
else
    BlockInds3 = ones(size(cornercoords3));
end

 

% Get block indices for grid


% Break up volume into 4x4x4 grid
BlockConfig.nblks1 = numel(unique(BlockInds1));
BlockConfig.nblks2 = numel(unique(BlockInds2));
BlockConfig.nblks3 = numel(unique(BlockInds3));
BlockConfig.nblks = [BlockConfig.nblks1,BlockConfig.nblks2,BlockConfig.nblks3];
BlockConfig.BlockArrangement = reshape(1:prod(BlockConfig.nblks),BlockConfig.nblks);
% Make a list of all block pairs
BlockConfig.all_block_pairs = adjacentBlockPairList;

% Compute sizes and ranges of blocks (ASSUME BLOCKS ARE REGULAR AND BLOCK 1
% IS REPRESENTATIVE)
%[blockData] = GetBlockVolumeData(blockFileNameList{1},'VascMask');

loadedStruct = load(blockFileNameList{1});
loadedVariables = fieldnames(loadedStruct);
blockData = loadedStruct.(loadedVariables{1});
if isstruct(blockData),
    tempPixelList = blockData.pixelList;
    tempMaskSize = blockData.maskSize;
    blockData = false(tempMaskSize);
    blockData(tempPixelList) = 1;
    clear tempPixelList tempMaskSize
end%if isstruct(blockData),
%-----    
bsz = size(blockData);
BlockConfig.bsz1 = bsz(1);
BlockConfig.bsz2 = bsz(2);
BlockConfig.bsz3 = bsz(3);
BlockConfig.blksz = [BlockConfig.bsz1,BlockConfig.bsz2,BlockConfig.bsz3];
BlockConfig.blksz1 = 0:(BlockConfig.bsz1 - 1);
BlockConfig.blksz2 = 0:(BlockConfig.bsz2 - 1);
BlockConfig.blksz3 = 0:(BlockConfig.bsz3 - 1);


%% GENERATE BLOCKS STRUCT
Blocks = struct([]);
% Initialize block corner locations
for iBLK = 1:numel(BlockConfig.BlockArrangement)
    Blocks(iBLK).corner_coords = [cornercoords1(iBLK) cornercoords2(iBLK) cornercoords3(iBLK)];
end

for iBLOCK = 1:numel(Blocks)
    loadedStruct = load(blockFileNameList{1});
    loadedVariables = fieldnames(loadedStruct);
    blockData = loadedStruct.(loadedVariables{1});
    if isstruct(blockData),
        tempPixelList = blockData.pixelList;
        tempMaskSize = blockData.maskSize;
        blockData = false(tempMaskSize);
        blockData(tempPixelList) = 1;
        clear tempPixelList tempMaskSize
    end%if isstruct(blockData),
    %-----
    % Blocks(iBLOCK).block_data_fname = ['Block_',num2str(iBLOCK),'.tif'];
    Blocks(iBLOCK).block_data_fname = blockFileNameList{iBLOCK};
    Blocks(iBLOCK).block_size = size(blockData);
    
    Blocks(iBLOCK).all_block_pair_indices = find(sum((BlockConfig.all_block_pairs == iBLOCK),2) > 0);
    abtmp = BlockConfig.all_block_pairs(Blocks(iBLOCK).all_block_pair_indices,:);
    Blocks(iBLOCK).adjacent_blocks = abtmp(abtmp ~= iBLOCK);
    for iAB = 1:numel(Blocks(iBLOCK).adjacent_blocks)
        Blocks(iBLOCK).MOESindices(iAB).adjacent_block = Blocks(iBLOCK).adjacent_blocks(iAB);
        Blocks(iBLOCK).MOESindices(iAB).x1x2 = 1;
        Blocks(iBLOCK).MOESindices(iAB).x1x3 = 1;
        Blocks(iBLOCK).MOESindices(iAB).x2x3 = 1;
    end
end
BlocksNominal = Blocks;

%% INITIALIZE OFFSETS
    Offsets = InitializeOffsets(Blocks,BlockConfig);

    
%%
registrationParameters.BlocksNominal = BlocksNominal;
registrationParameters.Blocks = Blocks;
registrationParameters.BlockConfig = BlockConfig;
registrationParameters.Offsets = Offsets;
%-----
outputParameters = registrationParameters;
return