function outProject = VidaRegisterBlocksByDefault(inProject)

vidaProject = inProject;
vidaProjectRootName = vidaProject.vidaProjectRootName;
vidaProjectFileName = vidaProject.vidaProjectFileName;
imageStackFolderNameList = vidaProject.imageStackFolderNameList;
adjacentPairRegistrationDataFolder = vidaProject.globalParameters.adjacentPairRegistrationDataFolder;
multiThreadProgress = vidaProject.multiThreadProgress;
multiThreadRequestFolder = multiThreadProgress.multiThreadRequestFolder;
%-----
experimentTemplate = vidaProject.experimentTemplate;
currentExperiment = vidaProject.currentExperiment;
fullStackSize = currentExperiment.globalParameters.fullStackSize;
registrationParameters = experimentTemplate.registrationParameters;
reverseYBlockNumberingFlag = registrationParameters.reverseYBlockNumberingFlag;
reverseXBlockNumberingFlag = registrationParameters.reverseXBlockNumberingFlag;
xcorrCh = registrationParameters.xcorrCh;
nominalBlockStepSizeRCInMicrons = registrationParameters.nominalBlockStepSizeRCInMicrons;
nominalBlockStepSizeZInMicrons = registrationParameters.nominalBlockStepSizeZInMicrons;
variableZBlockStepSizeFlag = registrationParameters.variableZBlockStepSizeFlag;
ySearchSizeInMicrons = registrationParameters.ySearchSizeInMicrons;
xSearchSizeInMicrons = registrationParameters.xSearchSizeInMicrons;
zSearchSizeInMicrons = registrationParameters.zSearchSizeInMicrons;
experimentTemplate.registrationParameters.emptyBlockCorrectionFlag = 1; %set to 1 to default empty blocks to nominal position
emptyBlockCorrectionFlag = registrationParameters.emptyBlockCorrectionFlag;
emptyBlockThreshold = registrationParameters.emptyBlockThreshold;
vidaProjectRootName = vidaProject.vidaProjectRootName;
vidaProjectRootFolder = vidaProject.vidaProjectRootFolder;
rczMin = vidaProject.globalParameters.rczMin;
rczMax = vidaProject.globalParameters.rczMax;
voxelSizeInMicrons = vidaProject.globalParameters.voxelSizeInMicrons;
%-----
delayMultiplier = vidaProject.experimentTemplate.basicParameters.delayMultiplier;
%-----
%Convert microns to voxels
nominalBlockStepSizeRC = round(nominalBlockStepSizeRCInMicrons ./ voxelSizeInMicrons);
nominalBlockStepSizeZ = round(nominalBlockStepSizeZInMicrons ./ voxelSizeInMicrons);
ySearchSize = round(ySearchSizeInMicrons ./ voxelSizeInMicrons);
xSearchSize = round(xSearchSizeInMicrons ./ voxelSizeInMicrons);
zSearchSize = round(zSearchSizeInMicrons ./ voxelSizeInMicrons);
yxzSearchSize = [ySearchSize,xSearchSize,zSearchSize];
%-----
%Add converted or global variables to registrationParameters for ease of
%passing information to subfunction calls
registrationParameters.vidaProjectRootName = vidaProjectRootName;
registrationParameters.vidaProjectRootFolder = vidaProjectRootFolder;
registrationParameters.nominalBlockStepSizeRC = nominalBlockStepSizeRC;
registrationParameters.nominalBlockStepSizeZ = nominalBlockStepSizeZ;
registrationParameters.yxzSearchSize = yxzSearchSize;
%-----
registrationParameters.maskFileToUseForRegistration = 'VascEnhancedMask';%noteToSelf
%-----
%-----

numImageStacks = size(imageStackFolderNameList);
if numImageStacks == 1,
    registrationParameters.fullStackSize = fullStackSize;
    registrationParameters.blockRegistration = [1 1 1];
    registrationParameters.numBlocksRCZ = [1 1 1];
    registrationParameters.adjacentBlockPairList = [];
    vidaProject.registrationParameters = registrationParameters;
    vidaProject.globalParameters.blockRegistration = [1 1 1];
    vidaProject.globalParameters.RCZList = [1 1 1];
    vidaProject.registrationParameters.RCZList = [1 1 1];
    vidaProject.globalParameters.adjacentBlockPairList = [];
    outProject = vidaProject;
    return% skip the rest of the function
end%if numImageStacks == 1,


%% SET BLOCK CONFIGURATION (CONSTANT STRUCT)

numRBlocks = rczMax(1) - rczMin(1) +1;
numCBlocks = rczMax(2) - rczMin(2) +1;
numZBlocks = rczMax(3) - rczMin(3) +1;
%-----
registrationParameters.nblks1 = numRBlocks;
registrationParameters.nblks2 = numCBlocks;
registrationParameters.nblks3 = numZBlocks;
registrationParameters.firstBlkRow = rczMin(1);
registrationParameters.firstBlkCol = rczMin(2);
registrationParameters.firstBlkZpos = rczMin(3);

%%
%BUILD STRUCTURE
    registrationParameters = GeneratePointerToBlocks(registrationParameters);
blockFileNameList = registrationParameters.blockFileNameList;
RCZList = registrationParameters.RCZList;
    registrationParameters = GetBlockAdjacency(registrationParameters);
adjacentBlockPairList = registrationParameters.adjacentBlockPairList;
    registrationParameters = GetBlockOffsets(registrationParameters);
adjacentBlockPairOffsetList = registrationParameters.adjacentBlockPairOffsetList;
blockCornerCoordList = registrationParameters.blockCornerCoordList;
    registrationParameters = VidaPrepareMOES(registrationParameters);

   
%-----
Blocks = registrationParameters.Blocks;
BlocksNominal = registrationParameters.BlocksNominal;
BlockConfig = registrationParameters.BlockConfig;
Offsets = registrationParameters.Offsets;

%-----
numAdjacentPairs = size(adjacentBlockPairList,1);


%% STAGE XXX: USE MOES MINIMA TO ESTIMATE INITIAL BLOCK LOCATIONS
 

Blocks0 = Blocks;   % Set the initial configuration of Blocks to be the least squares solution

estimatedRegistration.BlocksNominal = BlocksNominal;
estimatedRegistration.Blocks0 = Blocks0;
estimatedRegistration.Blocks = Blocks;
estimatedRegistration.BlockConfig = BlockConfig;
estimatedRegistration.Offsets = Offsets;




blockRegistration = reshape([estimatedRegistration.Blocks.corner_coords],[3,length(blockFileNameList)])';

numBlocksRCZ = [numRBlocks,numCBlocks,numZBlocks];

registrationParameters.fullStackSize = fullStackSize;
registrationParameters.BlocksNominal = BlocksNominal;
registrationParameters.Blocks0 = Blocks0;
registrationParameters.Blocks = Blocks;
registrationParameters.blockRegistration = blockRegistration;
registrationParameters.numBlocksRCZ = numBlocksRCZ;
vidaProject.registrationParameters = registrationParameters;
vidaProject.globalParameters.blockRegistration = blockRegistration;
vidaProject.globalParameters.RCZList = RCZList;
vidaProject.globalParameters.adjacentBlockPairList = adjacentBlockPairList;
outProject = vidaProject;




























