function outputParameters = VidaRegistrationInitStackBlocksInto3DVol(inputParameters)
% Compute distance transform for each block contribution to each montage slice.
% % % vidaProject = inputParameters;
% % % prmts = vidaProject.registrationParameters;

prmts = inputParameters;
if isfield(prmts,'blockStacking'),
    blockStacking = prmts.blockStacking;
end
%-----
blockStacking.rawoffsets = prmts.blockRegistration;
blockStacking.blockconfig = prmts.numBlocksRCZ;
blockStacking.zlayers = kron(1:blockStacking.blockconfig(3),ones(1,blockStacking.blockconfig(1)*blockStacking.blockconfig(2)));
blockStacking.datadir = prmts.path2dir;
blockStacking.blockFileNames = prmts.blocks2StitchFullFileNameList;
blockStacking.numblocks = numel(blockStacking.blockFileNames);
blockStacking.overlapfusion = prmts.overlapfusion; 
%-----
checkBlockSize = prmts.checkBlockSize;
minConfig = prmts.blockRegistration;


%% Get all 3D tif sizes
if checkBlockSize
    fprintf('\nChecking block sizes');
    blockStacking.volsizes = cell(1,blockStacking.numblocks);
    for iFNAME = 1:blockStacking.numblocks
        thisBlockFileName = regexprep(blockStacking.blockFileNames{iFNAME},'Mask','');

        tmpinfo = imfinfo(fullfile(blockStacking.datadir,thisBlockFileName));
        tmpwidth = unique([tmpinfo.Width]);
        tmpheight = unique([tmpinfo.Height]);
        tmpdepth = numel(tmpinfo);
        tmpbitdepth = unique([tmpinfo.BitDepth]);
        if (numel(tmpwidth) ~= 1) || (numel(tmpheight) ~= 1) || (numel(tmpdepth) ~= 1)
            error(['Block ',blockStacking.blockFileNames{iFNAME},' does not have consistent sizes']);
        else
            blockStacking.volsizes{iFNAME} = [tmpwidth tmpheight tmpdepth];
            blockStacking.bitdepth{iFNAME} = tmpbitdepth;
        end
    end
    fprintf('\t Done');
else
    thisBlockFileName = blockStacking.blockFileNames{1};
    loadedStruct = load(thisBlockFileName);
    loadedVariables = fieldnames(loadedStruct);
    numVariables = numel(loadedVariables);
    varByteList = [];
    for varIter = 1:numVariables,
        currentVar = loadedStruct.(loadedVariables{varIter});
        varProps = whos('currentVar');
        varByteList(varIter) = varProps.bytes;
    end;
    [maxByte,maxPos] = max(varByteList);
    thisBlock = loadedStruct.(loadedVariables{maxPos});
    if isstruct(thisBlock),
        tempPixelList = thisBlock.pixelList;
        tempMaskSize = thisBlock.maskSize;
        radiusFlag = isfield(thisBlock,'radiusList');
        thisBlock = false(tempMaskSize);
        thisBlock(tempPixelList) = 1;
        if radiusFlag,
            thisBlock = single(thisBlock);
        end
        clear tempPixelList tempMaskSize
    end%if isstruct(blockData),
    %-----
    tmpwidth = size(thisBlock,2);
    tmpheight = size(thisBlock,1);
    tmpdepth = size(thisBlock,3);
    blockStacking.volsizes = num2cell(repmat([tmpheight tmpwidth tmpdepth],blockStacking.numblocks,1),2);
    blockStacking.voxeltype = class(thisBlock);
end


% % %% Interpret bitdepth
% % if unique([blockStacking.bitdepth{:}]) == 8
% %     fprintf('8-bit imagery... Assuming binary voxels.\n');
% %     blockStacking.voxeltype = 'uint8';
% % else
% %     fprintf('Not 8-bit imagery... Assuming 16-bit voxels.\n');
% %     blockStacking.voxeltype = 'uint16';
% % end


%% Create 1-based indexing so all indices into .tifs are positive
blockStacking.zerobasedoffsets = minConfig - repmat(min(minConfig),size(minConfig,1),1);

%% Create each block's integer lattice coordinate ranges
for iFNAME = 1:blockStacking.numblocks
    blockStacking.latticecoords.x1{iFNAME} = blockStacking.zerobasedoffsets(iFNAME,1) + (1:blockStacking.volsizes{iFNAME}(1));
    blockStacking.latticecoords.x2{iFNAME} = blockStacking.zerobasedoffsets(iFNAME,2) + (1:blockStacking.volsizes{iFNAME}(2));
    blockStacking.latticecoords.x3{iFNAME} = blockStacking.zerobasedoffsets(iFNAME,3) + (1:blockStacking.volsizes{iFNAME}(3));
end

%% Compute total number of slices in volume via max extent in 3rd
% coordinate
blockStacking.lastslice = max([blockStacking.latticecoords.x3{:}]);

%% Make distance maps for each block size--assume most block sizes will be
% repeated for efficiency.
if strcmp(blockStacking.overlapfusion,'centermost')
    fprintf('Computing distance transforms for centermost overlap method.\n');
    % Get all 3D block sizes
    vszs = reshape([blockStacking.volsizes{:}],3,numel(blockStacking.volsizes))';
    [uvszs,uvszsi,uvszsj] = unique(vszs,'rows');
    blockStacking.distancemappings = uvszsj;
    for iVSZ = 1:size(uvszs,1)
        % Downsample volume by 4 for approximate distance transform
        Dtmp = ones(round(uvszs(iVSZ,1)/4),round(uvszs(iVSZ,2)/4),round(uvszs(iVSZ,3)/4));
        Dtmp(2:end-1,2:end-1,2:end-1) = 0; % zero the inside
        % compute distance transform of reduced size block
        Dtmp = bwdist(Dtmp);

        % Interpolate in 2 orthogonal directions back up to original volume
        % size
        Dtmp2 = zeros(uvszs(iVSZ,1),uvszs(iVSZ,2),round(uvszs(iVSZ,3)/4));
        for iZD = 1:size(Dtmp,3) % interpolate each slice first
            Dtmp2(:,:,iZD) = imresize(Dtmp(:,:,iZD),[uvszs(iVSZ,1),uvszs(iVSZ,2)],'bilinear');
        end
        Dtmp = zeros(uvszs(iVSZ,1),uvszs(iVSZ,3),round(uvszs(iVSZ,2)));
        Dtmp2 = permute(Dtmp2,[1 3 2]); % rotate distance transform
        for iZD = 1:size(Dtmp2,3) % interpolate each orthoganal slice second
            Dtmp(:,:,iZD) = imresize(Dtmp2(:,:,iZD),[uvszs(iVSZ,1),uvszs(iVSZ,3)],'bilinear');
        end
        Dtmp2 = permute(Dtmp,[1 3 2]);
        zgrad = zeros(size(Dtmp2,3),1);   % create an upside-down V vs depth to scale each 3D distance xform slice
        zgrad(round(size(Dtmp2,3)/2)) = 1;
        zgrad = bwdist(zgrad);
        zgrad = max(zgrad(:)) - zgrad;
        zgrad = zgrad/max(zgrad(:))+.01;
        for iZ = 1:size(Dtmp2,3)
            Dtmp2(:,:,iZ) = zgrad(iZ)*Dtmp2(:,:,iZ);
        end
        clear Dtmp
        Dtmp2 = uint8(255*(Dtmp2/max(Dtmp2(:)))); % store distance as uint8 to fit slices in cell array
        blockStacking.DistanceTransform{iVSZ} = Dtmp2; % store bwdist ordering as uint8 for first ~50 voxels from edge
    end
    fprintf('Finished computing distance transforms for centermost overlap method.\n');
elseif strcmp(blockStacking.overlapfusion,'fixedzlayer')
    fprintf('...Computing distance transforms for fixed-z layer overlap method.\n');
    % Get all 3D block sizes
    vszs = reshape([blockStacking.volsizes{:}],3,numel(blockStacking.volsizes))';
    [uvszs,uvszsi,uvszsj] = unique(vszs,'rows');
    blockStacking.distancemappings = uvszsj;
    for iVSZ = 1:size(uvszs,1)
        % Downsample volume by 4 for approximate distance transform
        Dtmp = ones(round(uvszs(iVSZ,1)/4),round(uvszs(iVSZ,2)/4),round(uvszs(iVSZ,3)/4));
        Dtmp(2:end-1,2:end-1,2:end-1) = 0; % zero the inside
        % compute distance transform of reduced size block
        Dtmp = bwdist(Dtmp);
        % Interpolate in 2 orthogonal directions back up to original volume
        % size
        Dtmp2 = zeros(uvszs(iVSZ,1),uvszs(iVSZ,2),round(uvszs(iVSZ,3)/4));
        for iZD = 1:size(Dtmp,3) % interpolate each slice first
            Dtmp2(:,:,iZD) = imresize(Dtmp(:,:,iZD),[uvszs(iVSZ,1),uvszs(iVSZ,2)],'bilinear');
        end
        Dtmp = zeros(uvszs(iVSZ,1),uvszs(iVSZ,3),round(uvszs(iVSZ,2)));
        Dtmp2 = permute(Dtmp2,[1 3 2]); % rotate distance transform
        for iZD = 1:size(Dtmp2,3) % interpolate each orthoganal slice second
            Dtmp(:,:,iZD) = imresize(Dtmp2(:,:,iZD),[uvszs(iVSZ,1),uvszs(iVSZ,3)],'bilinear');
        end
        Dtmp2 = permute(Dtmp,[1 3 2]);
        clear Dtmp
        Dtmp2 = uint8(255*(Dtmp2/max(Dtmp2(:)))); % store distance as uint8 to fit slices in cell array
        blockStacking.DistanceTransform{iVSZ} = Dtmp2; % store bwdist ordering as uint8 for first ~50 voxels from edge
    end
end


outputParameters = inputParameters;
outputParameters.blockStacking = blockStacking;


