

function flowProject = CalculateDownstreamPathsFromVectorProject(flowProject);

baseStrandIDList = flowProject.baseStrandIDList;
maxBranchSearch = flowProject.maxBranchSearch;
stopVoltage = flowProject.stopVoltage;
minimumFluxFraction = flowProject.minimumFluxFraction;


verboseFlag = false;

downstreamPathList = [];
numStrands = numel(flowProject.Strands);
downstreamBranchList = zeros(numStrands,numel(baseStrandIDList)) + Inf;
numBaseStrands = numel(baseStrandIDList);

for baseStrandIter = 1:numBaseStrands;
    if verboseFlag == true, baseStrandIter, end
    baseStrandID = baseStrandIDList(baseStrandIter);
    baseStrand = flowProject.Strands(baseStrandID);
    bsiv = baseStrand.InteriorVertices;
    baseStrandCoordList = flowProject.Vertices.AllVerts(bsiv,:);
    strandNodes = [baseStrand.StartVertexIndex,baseStrand.EndVertexIndex];
    nodeVoltages = flowProject.Vertices.AllVoltages(strandNodes);
    [highVoltage,highVoltagePos] = max(nodeVoltages);
    [lowVoltage,lowVoltagePos] = min(nodeVoltages);
    highNode = strandNodes(highVoltagePos);
    lowNode = strandNodes(strandNodes~=highNode);

    pathActiveFlagList = false(1,100000);
    pathActiveFlagList(1) = 1;
    numActivePath = sum(pathActiveFlagList);
    strandPathList = {};
    nodePathList = {};
    fractionPathList = {};
    strandPathList{1} = baseStrandID;
    nodePathList{1} = [highNode lowNode];
    fractionPathList{1} = 1;
    strandFlowList = zeros(numStrands,1);
    strandFlowList(baseStrandID) = 1;
    strandVoltageList = zeros(numStrands,1);
    strandVoltageList(1) = mean([highVoltage,lowVoltage]);
    pathCounter = 1;
    keepGoingFlag = 1;
    branchCounter = 0;

    while (keepGoingFlag == 1),
        myCount = 0;
        branchCounter = branchCounter+1;
        for pathIter = find(pathActiveFlagList == 1);
            if verboseFlag == 1, disp([pathIter,sum(pathActiveFlagList)]);end
            currentPathID = pathIter;
            currentStrandPath = strandPathList{pathIter};
            currentNodePath = nodePathList{pathIter};
            currentFractionPath = fractionPathList{pathIter};
            currentStrandID = currentStrandPath(end);
            currentStrand = flowProject.Strands(currentStrandID);
            %-----
            strandNodes = [currentStrand.StartVertexIndex,currentStrand.EndVertexIndex];
            nodeVoltages = flowProject.Vertices.AllVoltages(strandNodes);
            [highVoltage,highVoltagePos] = max(nodeVoltages);
            [lowVoltage,lowVoltagePos] = min(nodeVoltages);
            highNode = strandNodes(highVoltagePos);
            lowNode = strandNodes(strandNodes~=highNode);
            lowAttachedStrands = flowProject.Vertices.StrandIndices(lowNode,:);
            lowAttachedStrands = lowAttachedStrands(lowAttachedStrands~=0);
            lowAttachedStrands = lowAttachedStrands(lowAttachedStrands~=currentStrandID);
            numDaughterPaths = numel(lowAttachedStrands);
            strandVoltageList(currentStrandID) = mean([highVoltage,lowVoltage]);
            stopTracingFlag = 0;
            if numDaughterPaths == 0, stopTracingFlag = 1; end
            if lowVoltage < stopVoltage, stopTracingFlag = 1; end
            if currentFractionPath(end) < minimumFluxFraction, stopTracingFlag = 1; end
            if stopTracingFlag == 1,
                pathActiveFlagList(currentPathID) = 0;
            else%if stopTracingFlag == 1,
                motherCurrent = abs(currentStrand.strandCurrent);
                validDaughterCounter = 0;
                validDaughterStrandList = [];
                validDaughterCurrentList = zeros(1,numDaughterPaths);
                validDaughterPathIndexList = [];
                for daughterIter = 1:numDaughterPaths,
                    %Check to see if the daughter strand leads to a lower voltage
                    candidateDaughterID = lowAttachedStrands(daughterIter);
                    candidateDaughterStrand = flowProject.Strands(candidateDaughterID);
                    candidateDaughterNodes = [candidateDaughterStrand.StartVertexIndex,candidateDaughterStrand.EndVertexIndex];
                    candidateDaughterOtherNode = candidateDaughterNodes(candidateDaughterNodes~=lowNode);
                    candidateDaughterOtherVoltage = flowProject.Vertices.AllVoltages(candidateDaughterOtherNode);
                    if candidateDaughterOtherVoltage < lowVoltage,
                        %if so, add the strand to a path
                        myCount = myCount + 1;
                        validDaughterCounter = validDaughterCounter + 1;
                        validDaughterStrandList(validDaughterCounter) = candidateDaughterID;
                        validDaughterCurrentList(validDaughterCounter) = abs(candidateDaughterStrand.strandCurrent);
                        if validDaughterCounter == 1,
                            %if this is the first valid daughter, add to current path
                            strandPathList{currentPathID} = [currentStrandPath, lowAttachedStrands(daughterIter)];
                            nodePathList{currentPathID} = [currentNodePath, candidateDaughterOtherNode];
                            validDaughterPathIndexList(1) = currentPathID;
                        else%if validDaughterCounter == 1,
                            %if not the first daughter, duplicate path up to this point, and
                            %make new path
                            pathCounter = pathCounter + 1;
                            strandPathList{pathCounter} = [currentStrandPath, lowAttachedStrands(daughterIter)];
                            nodePathList{pathCounter} = [currentNodePath, candidateDaughterOtherNode];
                            fractionPathList{pathCounter} = currentFractionPath;
                            pathActiveFlagList(pathCounter) = 1;  
                            validDaughterPathIndexList(validDaughterCounter) = pathCounter;
                        end%if-else validDaughterCounter == 1,
                    end%if candidateDaughterOtherVoltage < lowVoltage,
                end%for daughterIter = 1:numDaughterPaths,
                clear daughterIter
                %-----
                daughterSumCurrent = sum(validDaughterCurrentList);
                fractionalDaughterCurrent = validDaughterCurrentList./daughterSumCurrent;
                for validDaughterIter = 1:validDaughterCounter;
                    validDaughterSegmentID = validDaughterStrandList(validDaughterIter);
                    validDaughterFraction = currentFractionPath(end).*fractionalDaughterCurrent(validDaughterIter);
                    fractionPathList{validDaughterPathIndexList(validDaughterIter)} = [currentFractionPath , validDaughterFraction];
                    strandFlowList(validDaughterSegmentID) = strandFlowList(validDaughterSegmentID) + validDaughterFraction;
                end%for validDaughterIter = 1:validDaughterCounter;
            end%if-else stopTracingFlag == 1,
        end%for pathIter = find(pathActiveFlagList == 1);
        if sum(pathActiveFlagList) == 0, keepGoingFlag = 0;end
        if myCount == 0, keepGoingFlag = 0;end
        if branchCounter >= maxBranchSearch, keepGoingFlag = 0; end
    end%while (keepGoingFlag == 1),
    downstreamPathList{baseStrandIter} = strandPathList;
end%for baseStrandIter = 1:numBaseStrands;

for baseStrandIter = 1:numBaseStrands;
    thisPathSet = downstreamPathList{baseStrandIter};
    for pathIter = 1:numel(thisPathSet),
        thisPath = thisPathSet{pathIter};
        thisPath = thisPath(2:end);
        posList = [1:numel(thisPath)]';
        downstreamBranchList(thisPath,baseStrandIter) = min(downstreamBranchList(thisPath,baseStrandIter),posList);
    end%for pathIter = 1:numel(thisPathSet);
end%for baseStrandIter = ...
     
flowProject.downstreamPathList = downstreamPathList;
flowProject.downstreamBranchList = downstreamBranchList;

