


%Formula for Flux:
% Q = dP * pi * r^4 / (8 * mu * L)
%   where Q = volume flux [m3/s]
%   dP = pressure difference in Pascal [kg/m s2]
%   r = vessel radius [m]
%   L = vessel length [m]
%   mu = viscosity [Pa*s] = [kg / m s]
%       mu = mu0 * muRelative

%in caluclating strand currents, I used the formula:
%strandCurrent = myQ = myP * conductanceMultiplier * myR^4 / (muRelative * myL)
%   where myP = pressure in [mmHg]
%   conductanceMultiplier = 1e9
%   myR = vessel radius [um]
%   myL = vessel length [um]
%   muRelative = unitless
%and thus, strandCurrent has the odd units of : [mmHg um^3]
%which is not in the correct units of volume flux because we left out the asolute
%viscosity mu0 and has also been artificially multiplied by conductanceMultiplier = 1e9
%also: compared to Q, there is a missing (pi/8) factor:

%define: Qodd = (pi/8) * strandCurrent /( mu0 * conductance multiplier)
%              = (0.3927) * strandCurrent / (1.5e-3 [kg/ms]* 1e9)
%              = 2.618e-7 * strandCurrent 
%and Qodd now has the odd units of:  [mmHg um3 ms/kg]
%   which has hybrid units, but has all the correct units and prefactors to match Q

%the conversion from mmHg to Pascal [kg/ms2] is : 133.33 Pa / mmHg
%the conversion from um3 to m3 is 1e-18 m3/um3
%so that:
%Qtrue = Qodd * 133.33 * 1e-18
%   and Qtrue has the units of [Pa m3 kg/ms] = [(kg/ms2) * m3 * (ms/kg)] = [m3/s];
%   and Qtrue is now the true flux in m3/s through each segment
%   assuming dP was given in mmHg in the circuit calculation
%           r,L were given in microns in the circuit calculation
%           only muRelative was used in the circuit calulation
%           a conductanceMultiplier of 1e9 was used in the circuit calculation
%           the viscosity of blood plasma is assumed to be mu0 = 1.5e-3 kg/ms

%finally, blood velocity is taken as volumeFlux[m3/s] / vesselArea [m2]
%   to yield velocity in [m/s], which is then multiplied by 1000 to yield mm/s velocities


%First do the un-adjusted vessels in mouse dataset db
vp = allProject.db;
maxDepth = 600;

vs = vp.vectorizedStructure;
strandIndices = vs.Vertices.StrandIndices;
strands = vs.Strands;
numStrands = numel(strands);
strandZList = vs.strandMidYXZList(:,3);

strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;
rootName = vectorProject.vectorProjectRootName;
findDash = findstr(rootName,'-');
shortName = rootName(1:findDash(1)-1);

strandMidYXZList = vectorProject.vectorizedStructure.strandMidYXZList;

strandLabelList = [strands(:).MedianLabel];
surfList = find(and(strandLabelList>=300,strandLabelList<=399)); 
zSurf = strandMidYXZList(surfList,3);
medianZSurf = median(zSurf);
minZSurf = min(zSurf);

strandMidYXZList(:,3) = max(strandMidYXZList(:,3)-minZSurf,0);
allPlungingVenuleList = vectorProject.vectorizedStructure.allPlungingVenuleList;
allPlungingArterioleList = vectorProject.vectorizedStructure.allPlungingArterioleList;

eff = vectorProject.vectorizedStructure.effectiveStrandRadiusList;

capFlagList = ~ismember(strandLabelList,[330:349,430:449]);
arterioleFlagList = ismember(strandLabelList,[330:339,430:439]);
venuleFlagList = ismember(strandLabelList,[340:349,440:449]);

arterioleFlagList = ismember(strandLabelList,[430:439]);
venuleFlagList = ismember(strandLabelList,[440:449]);





fluxList = [];    
velocityList = [];
numVessels = numel(strands);
for qq = 1:numVessels,
    vesselRadius = vs.effectiveStrandRadiusList(qq);
    vesselCurrent = abs(strands(qq).strandCurrent);
    
    vesselLength = vs.strandLengthList(qq);
    dV = abs(strands(qq).VoltageDrop);% in Pascal = kjg*m/s2

    
    Qodd = vesselCurrent * 2.618e-7;
    Qtrue = Qodd*133*1e-18;
    
 
    vesselArea = pi*vesselRadius.^2;
    vesselAreaM2 = vesselArea /1e12;
    
    vesselVelocityMs = Qtrue ./ vesselAreaM2; %m/s
    vesselVelocityMMs = vesselVelocityMs * 1000;
    
    fluxList(qq) = Qtrue;
    velocityList(qq) = vesselVelocityMMs;

end%for qq    
figure(2);semilogy(strandZList,velocityList,'r.')


ff = find(strandZList<maxDepth);
subV = velocityList(ff);
median(subV)



%%



diameterList =  vectorProject.vectorizedStructure.effectiveStrandRadiusList*2;


capD = diameterList(capFlagList); capV = velocityList(capFlagList);
artD = diameterList(arterioleFlagList); artV = velocityList(arterioleFlagList);
venD = diameterList(venuleFlagList); venV = velocityList(venuleFlagList);

subCap = 20;
figure(1);loglog(capD(1:subCap:end),capV(1:subCap:end),'ko');hold on
figure(1);loglog(artD,artV,'ro');
figure(1);loglog(venD,venV,'bo');hold off
set(gca,'YLim',[1e-3 1e2]);
set(gca,'XLim',[2 100]);






%%
%%
%%
%%
%%
%%



%Formula for Flux:
% Q = dP * pi * r^4 / (8 * mu * L)
%   where Q = volume flux [m3/s]
%   dP = pressure difference in Pascal [kg/m s2]
%   r = vessel radius [m]
%   L = vessel length [m]
%   mu = viscosity [Pa*s] = [kg / m s]
%       mu = mu0 * muRelative

%in caluclating strand currents, I used the formula:
%strandCurrent = myQ = myP * conductanceMultiplier * myR^4 / (muRelative * myL)
%   where myP = pressure in [mmHg]
%   conductanceMultiplier = 1e9
%   myR = vessel radius [um]
%   myL = vessel length [um]
%   muRelative = unitless
%and thus, strandCurrent has the odd units of : [mmHg um^3]
%which is not in the correct units of volume flux because we left out the asolute
%viscosity mu0 and has also been artificially multiplied by conductanceMultiplier = 1e9
%also: compared to Q, there is a missing (pi/8) factor:

%define: Qodd = (pi/8) * strandCurrent /( mu0 * conductance multiplier)
%              = (0.3927) * strandCurrent / (1.5e-3 [kg/ms]* 1e9)
%              = 2.618e-7 * strandCurrent 
%and Qodd now has the odd units of:  [mmHg um3 ms/kg]
%   which has hybrid units, but has all the correct units and prefactors to match Q

%the conversion from mmHg to Pascal [kg/ms2] is : 133.33 Pa / mmHg
%the conversion from um3 to m3 is 1e-18 m3/um3
%so that:
%Qtrue = Qodd * 133.33 * 1e-18
%   and Qtrue has the units of [Pa m3 kg/ms] = [(kg/ms2) * m3 * (ms/kg)] = [m3/s];
%   and Qtrue is now the true flux in m3/s through each segment
%   assuming dP was given in mmHg in the circuit calculation
%           r,L were given in microns in the circuit calculation
%           only muRelative was used in the circuit calulation
%           a conductanceMultiplier of 1e9 was used in the circuit calculation
%           the viscosity of blood plasma is assumed to be mu0 = 1.5e-3 kg/ms

%finally, blood velocity is taken as volumeFlux[m3/s] / vesselArea [m2]
%   to yield velocity in [m/s], which is then multiplied by 1000 to yield mm/s velocities


%Now repeat for "rat-like" vessel diameters;
load('F:\Phil\VectorProjects\OcclusionFiles\RatLikePlunging1\pbDB-Unperturbed.mat')

vp = allProject.db;
maxDepth = 600;

vs = vp.vectorizedStructure;
strandIndices = vs.Vertices.StrandIndices;
strands = vs.Strands;
numStrands = numel(strands);
strandZList = vs.strandMidYXZList(:,3);

strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.vectorizedStructure.maskSize;
rootName = vectorProject.vectorProjectRootName;
findDash = findstr(rootName,'-');
shortName = rootName(1:findDash(1)-1);

strandMidYXZList = vectorProject.vectorizedStructure.strandMidYXZList;

strandLabelList = [strands(:).MedianLabel];
surfList = find(and(strandLabelList>=300,strandLabelList<=399)); 
zSurf = strandMidYXZList(surfList,3);
medianZSurf = median(zSurf);
minZSurf = min(zSurf);

strandMidYXZList(:,3) = max(strandMidYXZList(:,3)-minZSurf,0);
allPlungingVenuleList = vectorProject.vectorizedStructure.allPlungingVenuleList;
allPlungingArterioleList = vectorProject.vectorizedStructure.allPlungingArterioleList;

eff = vectorProject.vectorizedStructure.effectiveStrandRadiusList;

capFlagList = ~ismember(strandLabelList,[330:349,430:449]);
arterioleFlagList = ismember(strandLabelList,[330:339,430:439]);
venuleFlagList = ismember(strandLabelList,[340:349,440:449]);

arterioleFlagList = ismember(strandLabelList,[430:439]);
venuleFlagList = ismember(strandLabelList,[440:449]);



ratRadiusList = vs.effectiveStrandRadiusList;
ratRadiusList(capFlagList) = ratRadiusList(capFlagList) *2;
ratRadiusList(arterioleFlagList) = ratRadiusList(arterioleFlagList) *1.5;
ratRadiusList(venuleFlagList) = ratRadiusList(venuleFlagList) *4;
              


fluxList = [];    
velocityList = [];
numVessels = numel(strands);
for qq = 1:numVessels,
%    vesselRadius = vs.effectiveStrandRadiusList(qq);
%     vesselCurrent = abs(strands(qq).strandCurrent);
    vesselRadius = ratRadiusList(qq);
    vesselCurrent = origCurrentList(qq);
    
    vesselLength = vs.strandLengthList(qq);
    dV = abs(strands(qq).VoltageDrop);% in Pascal = kjg*m/s2

    
    Qodd = vesselCurrent * 2.618e-7;
    Qtrue = Qodd*133*1e-18;
    
 
    vesselArea = pi*vesselRadius.^2;
    vesselAreaM2 = vesselArea /1e12;
    
    vesselVelocityMs = Qtrue ./ vesselAreaM2; %m/s
    vesselVelocityMMs = vesselVelocityMs * 1000;
    
    fluxList(qq) = Qtrue;
    velocityList(qq) = vesselVelocityMMs;

end%for qq    
figure(2);semilogy(strandZList,velocityList,'r.')
% figure(2);semilogy(strandZList,fluxList,'b.')


ff = find(strandZList<maxDepth);
subV = velocityList(ff);
median(subV)



%%



% diameterList =  vectorProject.vectorizedStructure.effectiveStrandRadiusList*2;
diameterList = ratRadiusList*2;

% %  effTemp = vectorProject.vectorizedStructure.effectiveStrandRadiusList;
% % cRad = effTemp(capFlagList);
% % aRad = effTemp(arterioleFlagList);
% % vRad = effTemp(venuleFlagList);
% % 
% % 
% % figure(1);semilogy(cRad*2*2,'ko');
% % figure(2);semilogy(aRad*2*1.5,'ro');
% % figure(3);semilogy(vRad*2*4,'bo');

capD = diameterList(capFlagList); capV = velocityList(capFlagList);
artD = diameterList(arterioleFlagList); artV = velocityList(arterioleFlagList);
venD = diameterList(venuleFlagList); venV = velocityList(venuleFlagList);

subCap = 20;
figure(11);loglog(capD(1:subCap:end),capV(1:subCap:end),'ko');hold on
figure(11);loglog(artD,artV,'ro');
figure(11);loglog(venD,venV,'bo');hold off

set(gca,'YLim',[5e-3 50]);
set(gca,'XLim',[3 300]);











