function ShowNeighboringStrands (varargin)
%ShowNeighboringStrands(vectorProject,centerStrandID,[blockSize],[radiusDisplayMultiplier]);

if nargin < 2,
    disp('To use ShowNeighboringStrands you must pass:');
    disp('     vectorProject, centerStrandID')
    disp('optional 3rd and 4th variables are :')
    disp('     [blockSize (1x1 or 1x3)] and [radiusDisplayMultiplier (1-10)]');
    return
end%if varargin <2

vectorProject = varargin{1};
centerStrandID = varargin{2};
blockSize = 100;
radiusDisplayMultiplier = 3;
if nargin >2 ,
    blockSize = varargin{3};
end%if nargin>1
if nargin >3,
    radiusDisplayMultiplier = varargin{4};
end


if numel(blockSize) == 1, blockSize = [blockSize,blockSize,blockSize]; end
if numel(blockSize) == 2, blockSize(3) = min(blockSize);end
halfBlockSize = blockSize/2;

strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
allVerts = vertices.AllVerts;
strandIndices = vertices.StrandIndices;
calcStatsFlag = 0;
if ~isfield(vectorProject.vectorizedStructure,'strandMidYXZList'); calcStatsFlag = 1; end
if ~isfield(vectorProject.vectorizedStructure,'adjustedStrandRadiusList'); calcStatsFlag = 1; end

if calcStatsFlag == 1,    
    numStrands = numel(strands);
    strandMidYXZList = zeros(numStrands,3);
    strandRadiusList = zeros(numStrands,1);
    strandLabelList = zeros(numStrands,1);
    for strandIter = 1:numStrands,
        stei = strands(strandIter).StartToEndIndices;
        cc = vertices.AllVerts(stei,:);
        rr = vertices.AllRadii(stei,:);
        ll = vertices.AllLabels(stei,:);
        yxzLim(:,1) = min(cc,[],1)';
        yxzLim(:,2) = max(cc,[],1)';
        strandMidYXZList(strandIter,:) = mean(yxzLim,2);
        strandRadiusList(strandIter) = median(rr);
        strandLabelList(strandIter) = median(ll);
        strands(strandIter).MedianLabel = median(ll);
% %         if mod(strandIter,100) ==0, disp([strandIter,numStrands]);end
    end%for strandIter = 1:numStrands,
    vectorProject.vectorizedStructure.Strands = strands;
    vectorProject.vectorizedStructure.strandMidYXZList = strandMidYXZList;
    vectorProject.vectorizedStructure.adjustedStrandRadiusList = strandRadiusList;
    
end%if calcStatsFlag == 1,

yxzStrandList = vectorProject.vectorizedStructure.strandMidYXZList;
strandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;

%Locate the coords of the chosen centerStrand
centerStrandCoords = yxzStrandList(centerStrandID,:);
y1 = centerStrandCoords(1)-halfBlockSize(1);
y2 = centerStrandCoords(1)+halfBlockSize(1);
x1 = centerStrandCoords(2)-halfBlockSize(2);
x2 = centerStrandCoords(2)+halfBlockSize(2);
z1 = centerStrandCoords(3)-halfBlockSize(3);
z2 = centerStrandCoords(3)+halfBlockSize(3);

%Find all vertices within a bounding box of that center
subFlag1 = and(allVerts(:,1)>y1,allVerts(:,1)<y2);
subFlag2 = and(allVerts(:,2)>x1,allVerts(:,2)<x2);
subFlag3 = and(allVerts(:,3)>z1,allVerts(:,3)<z2);
subFlagAll = and(and(subFlag1,subFlag2),subFlag3);
% subFlagAll(strands(centerStrandID).StartToEndIndices) = 0;
subVertList = find(subFlagAll);

%Find the list of strands to which those vertices belong
subStrandList = unique(strandIndices(subFlagAll,1));
numSubStrands = numel(subStrandList);

%Plot strand-by-strand
pSubList = zeros(numSubStrands);
for strandIter = 1:numSubStrands,
    thisStrandID = subStrandList(strandIter);
    thisStrand = strands(thisStrandID);
    thisStrandVertList = thisStrand.StartToEndIndices;
    thisStrandLabel = thisStrand.MedianLabel;
    thisStrandRadius = strandRadiusList(thisStrandID);
    %pull only those verts that are within the bounding box
    thisStrandSubVertList = thisStrandVertList(ismember(thisStrandVertList,subVertList));
    cc = allVerts(thisStrandSubVertList,:);
    figure(123);pSub = plot3(cc(:,1),cc(:,2),-1*cc(:,3),'k.');hold on
    pSubList(strandIter) = pSub;
    
    %Set the marker size to match the radius
    set(pSub,'MarkerEdgeColor',[0 0 0]);
    markerSize = (thisStrandRadius*radiusDisplayMultiplier);
    set(pSub,'MarkerSize',max(1,markerSize));

    %Color by label
    if ismember(thisStrandLabel,[330:339]),set(pSub,'Marker','o','MarkerEdgeColor','red');end%surface supply vessel
    if ismember(thisStrandLabel,[340:349]),set(pSub,'Marker','o','MarkerEdgeColor','blue');end%surface drain vessel
    if ismember(thisStrandLabel,[430:439]),set(pSub,'Marker','x','MarkerEdgeColor','red');end%plunging supply vessel
    if ismember(thisStrandLabel,[440:439]),set(pSub,'Marker','x','MarkerEdgeColor','blue');end%plunging drain vessel
    if ismember(thisStrandLabel,[401:429,440:499]),set(pSub,'Marker','x','MarkerEdgeColor','black');end%other plunging vessel
    
    %Color the centerStrand Green
    if thisStrandID == centerStrandID,
        set(pSub,'MarkerEdgeColor','green');
    end
    
    %Deposit StrandInfo into the plot
    strandData = thisStrand;
    strandData.strandID = thisStrandID;
    strandData.strandRadius = thisStrandRadius;
    set(pSub,'UserData',strandData);
    set(pSub,'ButtonDownFcn','FV_GetStrandInfo');
    
% %     h = datacursormode(f123);
% %     set(h,'UpdateFcn',@SNS_GetStrandInfo,'DisplayStyle','window')

    
    
end%for strandIter
hold off; axis equal

