function ShowTheseStrands (varargin)
%ShowNeighboringStrands(vectorProject,strandList,figNum,[radiusDisplayMultiplier]);

if nargin < 2,
    disp('To use ShowNeighboringStrands you must pass:');
    disp('     vectorProject, centerStrandID')
    disp('optional 3rd and 4th variables are :')
    disp('     [blockSize (1x1 or 1x3)] and [radiusDisplayMultiplier (1-10)]');
    return
end%if varargin <2

vectorProject = varargin{1};
strandList = varargin{2};
figNum = 1;
radiusDisplayMultiplier = 3;
if nargin >2 ,
    figNum = varargin{3};
end%if nargin>1
if nargin >3,
    radiusDisplayMultiplier = varargin{4};
end



strands = vectorProject.vectorizedStructure.Strands;
vertices = vectorProject.vectorizedStructure.Vertices;
allVerts = vertices.AllVerts;
strandIndices = vertices.StrandIndices;
calcStatsFlag = 0;
if ~isfield(vectorProject.vectorizedStructure,'strandMidYXZList'); calcStatsFlag = 1; end
if ~isfield(vectorProject.vectorizedStructure,'adjustedStrandRadiusList'); calcStatsFlag = 1; end

if calcStatsFlag == 1,    
    numStrands = numel(strands);
    strandMidYXZList = zeros(numStrands,3);
    strandRadiusList = zeros(numStrands,1);
    strandLabelList = zeros(numStrands,1);
    for strandIter = 1:numStrands,
        stei = strands(strandIter).StartToEndIndices;
        cc = vertices.AllVerts(stei,:);
        rr = vertices.AllRadii(stei,:);
        ll = vertices.AllLabels(stei,:);
        yxzLim(:,1) = min(cc,[],1)';
        yxzLim(:,2) = max(cc,[],1)';
        strandMidYXZList(strandIter,:) = mean(yxzLim,2);
        strandRadiusList(strandIter) = median(rr);
        strandLabelList(strandIter) = median(ll);
        strands(strandIter).MedianLabel = median(ll);
% %         if mod(strandIter,100) ==0, disp([strandIter,numStrands]);end
    end%for strandIter = 1:numStrands,
    vectorProject.vectorizedStructure.Strands = strands;
    vectorProject.vectorizedStructure.strandMidYXZList = strandMidYXZList;
    vectorProject.vectorizedStructure.adjustedStrandRadiusList = strandRadiusList;
    
end%if calcStatsFlag == 1,

yxzStrandList = vectorProject.vectorizedStructure.strandMidYXZList;
strandRadiusList = vectorProject.vectorizedStructure.adjustedStrandRadiusList;


%Find the list of strands to which those vertices belong
numStrands = numel(strandList);

%Plot strand-by-strand
pSubList = zeros(numStrands);
for strandIter = 1:numStrands,
    thisStrandID = strandList(strandIter);
    thisStrand = strands(thisStrandID);
    thisStrandVertList = thisStrand.StartToEndIndices;
    thisStrandLabel = thisStrand.MedianLabel;
    thisStrandRadius = strandRadiusList(thisStrandID);
    cc = allVerts(thisStrandVertList,:);
    figure(figNum);pSub = plot3(cc(:,1),cc(:,2),-1*cc(:,3),'k.');hold on
    pSubList(strandIter) = pSub;
    
    %Set the marker size to match the radius
    set(pSub,'MarkerEdgeColor',[0 0 0]);
    markerSize = (thisStrandRadius*radiusDisplayMultiplier);
    set(pSub,'MarkerSize',max(1,markerSize));

    %Color by label
    if ismember(thisStrandLabel,[330:339]),set(pSub,'Marker','o','MarkerEdgeColor','red');end%surface supply vessel
    if ismember(thisStrandLabel,[340:349]),set(pSub,'Marker','o','MarkerEdgeColor','blue');end%surface drain vessel
    if ismember(thisStrandLabel,[430:439]),set(pSub,'Marker','x','MarkerEdgeColor','red');end%plunging supply vessel
    if ismember(thisStrandLabel,[440:439]),set(pSub,'Marker','x','MarkerEdgeColor','blue');end%plunging drain vessel
    if ismember(thisStrandLabel,[401:429,440:499]),set(pSub,'Marker','x','MarkerEdgeColor','black');end%other plunging vessel
    
% %     %Color the centerStrand Green
% %     if thisStrandID == centerStrandID,
% %         set(pSub,'MarkerEdgeColor','green');
% %     end
    
    %Deposit StrandInfo into the plot
    strandData = thisStrand;
    strandData.strandID = thisStrandID;
    strandData.strandRadius = thisStrandRadius;
    set(pSub,'UserData',strandData);
    set(pSub,'ButtonDownFcn','SNS_GetStrandInfo');
    
% %     h = datacursormode(f123);
% %     set(h,'UpdateFcn',@SNS_GetStrandInfo,'DisplayStyle','window')

    
    
end%for strandIter
hold off; axis equal

