%This code finds plunging vessels attached to surface vessels

%Load a globalDataStruct
%
%Note:  no connectivity changes can have occured since the latest restrandify round
%Changes to labels are okay.
%

%Labels: 100=Unlabeled; 200=Temp ; 300=Surface ; 400=Plunging ; 500=Micro ;
%         10=Unlabeled ; 20=Temp ; 30=Supply ; 40=Draining ;  50=Connecting ;
%
%Therefore: SurfaceArtery=330; SurfaceVein = 340;

cutoffRadius = 2.1;
minLength = 30;
plungingAspectRatioCutoff = 0.66;
plungingDimension = 3;

Strands = globalDataStruct.vectorizedStructure.Strands;
cpMatrix = globalDataStruct.cpMatrix;
ar = globalDataStruct.ar;
numStrands = size(Strands,2);

%consider all strands that are mostly vertical
%and of sufficient median diameter to be plunging
candidateList = zeros(size(Strands));
for kk = 1:numStrands,
    cs = Strands(kk);%currentStrand
    iv = cs.InteriorVertices;
    strandLength = size(iv,2);
    cp1 = cpMatrix(cs.StartVertexIndex,1:3);
    cp2 = cpMatrix(cs.EndVertexIndex,1:3);
    cpRange = abs(cp1-cp2);
    plungingAspectRatio = cpRange(plungingDimension)./sqrt(sum(cpRange.^2));
    cr = ar(iv);
    medianRadius = median(cr);
    if plungingAspectRatio > plungingAspectRatioCutoff
        if medianRadius > cutoffRadius,
            if strandLength > minLength,
                candidateList(kk) = 1;
            end
        end
    end

end

candidateIDList = find(candidateList);
numCandidates = sum(candidateList(:));

figure(1);
hold off
for k = 1:numCandidates,
    currentCandidate = candidateIDList(k);
    cs = Strands(currentCandidate);
    iv = cs.InteriorVertices;
    %---
    cc1 = cpMatrix(iv,1);
    cc2 = cpMatrix(iv,2);
    cc3 = cpMatrix(iv,3);
    p3 = plot3(cc1,cc2,-1*cc3,'ro');
    hold on
end
    
pause
pause
pause
pause


for k = 1:numCandidates,
    currentCandidate = candidateIDList(k);
    cs = Strands(currentCandidate);
    iv = cs.InteriorVertices;
    %---
    currentLabel = cpMatrix(iv,7);
    newLabel = 400+mod(currentLabel,100);
    cpMatrix(iv,7) = newLabel;
end
globalDataStruct.cpMatrix = cpMatrix;




fileRootName = globalDataStruct.fileRootName;
dateString = datestr(now,'yyyy_mm_dd-HH_MM_SS');
[pathName,fileName,ext] = fileparts(fileRootName);
saveFileName = fullfile(pathName,['SE',dateString,'-PlungingLabeled','-StrandEdit-',fileName,ext]);
saveBox = warndlg('Saving Progress - Please Wait');
pause(0.1);
save(saveFileName,'globalDataStruct');
close(gcf);




