function outExp = PrepareSE2ForRestrandify(globalDataStruct);%

if ~exist('variableCleanupFlag','var'), 
    variableCleanupFlag = 0;
end

%This function takes as input, an experiment file that contains the following:
%   globalDataStruct.strandifyParameters
%   strandifyExp.centerlineRadiusStructure : which contains the fields:
%   centerlineRadiusStructure.pixelList = pixelList;
%       maskSize
%       radiusList
%       labelList
%       confidenceList
%       noteList        
%           
%
%the output strandifyExp will contain the new vectorizedStructure and the
%   strandifyParameters



%Hardcode
bestGuessOffset = [0 0 0];%offset from labeledCP to newCP's : offset = newCoord - labeledCoord (labeled = currentCPMatrix)
%-----

%Grab the current centerline points, radii, and labels
currentCPMatrix = globalDataStruct.cpMatrix;
currentMaskSize = globalDataStruct.maskSize;
ar = globalDataStruct.ar;
labeledIndexList = find(currentCPMatrix(:,7)~=110);%where in the currentCPMatrix is the labeled point.
numLabeledPoints = size(labeledIndexList,1);%how many points were labeled
transferredIndexList = zeros(size(labeledIndexList));%0 or 1 = transferred/not yet
%-----
%Convert the information to the centerlineRadiusStructure format and run it through the
%Strandify algorithm
activeFlags = currentCPMatrix(:,6)==1;
cc1 = currentCPMatrix(activeFlags,1);
cc2 = currentCPMatrix(activeFlags,2);
cc3 = currentCPMatrix(activeFlags,3);
ccr = ar(activeFlags);
centerlineRadiusStructure = [];
centerlineRadiusStructure.pixelList = sub2ind(currentMaskSize,cc1,cc2,cc3);
centerlineRadiusStructure.maskSize = currentMaskSize;
centerlineRadiusStructure.radiusList = ccr;
newVectorizedStructure = JustStrandifyCenterlineRadiusStructure2(centerlineRadiusStructure);
%-----
%Construct a newCPMatrix from the newly re-strandified structure
newMaskSize = newVectorizedStructure.maskSize;
av = newVectorizedStructure.Vertices.AllVerts;
ar = newVectorizedStructure.Vertices.AllRadii;
si = newVectorizedStructure.Vertices.StrandIndices;
jpTemp = newVectorizedStructure.Vertices.JunctionPoints;
fepTemp = newVectorizedStructure.Vertices.FreeEndPoints;
jp = av(jpTemp==1,:);
fep = av(fepTemp==1,:);
newCPMatrix = [av,fepTemp,jpTemp,ones(size(jpTemp)),zeros(size(jpTemp))+110];
numNewCP = size(newCPMatrix,1);
%-----
%Create a sparse reverse look-up table of the newCPMatrix coordinates
%to improve speed over performing multiple find operatations.
%Sparse matrices are limited to 10^9 in size: therefore we break it up
%into a different sparse matrix for each dim1_dim2 plane
disp('Generating Reverse Lookup Table');
maxCoords = max(newMaskSize,currentMaskSize);
maxPlanarLookup = maxCoords(1)*maxCoords(2);
coordMultiplier = 10.^ceil(log10(maxCoords));
maxLookup = coordMultiplier(2) + maxCoords(2).*coordMultiplier(3) + maxCoords(3);
reverseLUT = {};
for kk = 1:newMaskSize(3);
    inPlaneNewCPMatrixIndexList = find(newCPMatrix(:,3)==kk);
    planarNewCPMatrix = newCPMatrix(inPlaneNewCPMatrixIndexList,1:3);
    [uniquePlanarNewCPMatrix,ui,uj] = unique(planarNewCPMatrix,'rows');
    numPlanarCP = size(uniquePlanarNewCPMatrix,1);
    ii = zeros(numPlanarCP,1);
    jj = ones(numPlanarCP,1);
    ss = inPlaneNewCPMatrixIndexList(ui);
    for qq = 1:numPlanarCP,
        currentPlanarCP = uniquePlanarNewCPMatrix(qq,1:3);
        currentPlanarLookup = currentPlanarCP(1) + (currentPlanarCP(2)-1) * maxCoords(1);
        ii(qq) = currentPlanarLookup;
    end%for qq = 1:numPlanarCP,
    planarReverseLUT = sparse(ii,jj,ss);
    reverseLUT{kk} = planarReverseLUT;
end%for kk = 1:newMaskSize(3);
disp('....done.');
%-----
%Go through each labeled centerline point in the original currentCPMatrix and attempt to transfer
%it to an identically coordinated point in the newCPMatrix
%-----
disp('Transferring Labels on Exact Matches');
counter = 0;
for kk = 1:numLabeledPoints,
    origCoords = currentCPMatrix(labeledIndexList(kk),1:3);
    origLabel = currentCPMatrix(labeledIndexList(kk),7);
    adjustedCoords = origCoords + bestGuessOffset;
    adjustedCoords = max(1,min(maxCoords,adjustedCoords));
    currentPlanarLookup = adjustedCoords(1) + (adjustedCoords(2)-1) * maxCoords(1);
    planarReverseLUT = reverseLUT{adjustedCoords(3)};
    planarReverseLUTSize = size(planarReverseLUT,1);
    if currentPlanarLookup <= planarReverseLUTSize,
        newIndex = planarReverseLUT(currentPlanarLookup);
        if newIndex ~= 0,
            counter = counter+1;
            newCPMatrix(newIndex,7) = origLabel;
            transferredIndexList(newIndex) =  1;
        end%if newIndex ~= 0,
    end%if currentPlanarLookup <= planarReverseLUTSize,
end%for kk = 1:numLabeledPoints,    
disp('....done.');
%-----
%If any of the labeled points did not get transferred, search within a small region
%around that coordinate for a unlabeled point to associate.  This is done in case
%the re-strandification process moved points by a few pixels (especially during the
%node collaspse algorithm)
%
%Not yet implemented
%
%
%-----
%Go strand-by-strand, and make the majority label of the strand the label for all points
%on that strand.
newStrands = newVectorizedStructure.Strands;
numStrands = size(newStrands,2);
for kk = 1:numStrands,
    currentStrand = newStrands(kk);
    interiorVertices = currentStrand.InteriorVertices;
    interiorLabels = newCPMatrix(interiorVertices,7);
    uniqueLabels = unique(interiorLabels);
    if size(uniqueLabels,1) > 1,
        labelHist = hist(interiorLabels,uniqueLabels);
        [histMax,maxPos] = max(labelHist);
        majorityLabel = uniqueLabels(maxPos);
        newCPMatrix(interiorVertices,7) = majorityLabel;
    end%if size(uniqueLabels,1) > 1,
end%for kk = 1:numStrands,
%-----
%Place new Strand and label data back into globalDataStruct, replacing the old data
globalDataStruct.strands = newStrands;
% globalDataStruct.vectorizedStructure = newVectorizedStructure;
globalDataStruct.maskSize = newMaskSize;
globalDataStruct.cpMatrix = newCPMatrix;
globalDataStruct.strands = newVectorizedStructure.Strands;
globalDataStruct.av = av;
globalDataStruct.ar = ar;
globalDataStruct.jp = jp;
globalDataStruct.fep = fep;
globalDataStruct.si = si;
%-----
fileRootName = globalDataStruct.fileRootName;
dateString = datestr(now,'yyyy_mm_dd-HH_MM_SS');
[pathName,fileName,ext] = fileparts(fileRootName);
saveFileName = fullfile(pathName,['SE',dateString,'-VectorizedStructure-',fileName,ext]);
save(saveFileName,'newVectorizedStructure');
saveFileName = fullfile(pathName,['SE',dateString,'-Restrandified','-StrandEdit-',fileName,ext]);
save(saveFileName,'globalDataStruct');

if variableCleanupFlag == 1,
    clear activeFlags adjustedCoords ar av bestGuessOffset cc1 cc2 cc3 ccr
    clear centerlineRadiusStructure coordMultiplier counter currentCPMatrix
    clear currentMaskSize currentPlanarCP currentPlanarLookup currentStrand
    clear dateString ext fep fepTemp fileName fileRootName histMax ii
    clear inPlaneNewCPMatrixIndexList interiorLabels interiorVertices jj
    clear jp jpTemp kk labelHist labeledIndexList majorityLabel maxCoords
    clear maxLookup maxPlanarLookup maxPos newCPMatrix newIndex newMaskSize
    clear newStrands newVectorizedStructure numLabeledPoints numNewCP numPlanarCP                  
    clear numStrands origCoords origLabel pathName planarNewCPMatrix planarReverseLUT  
    clear planarReverseLUTSize qq reverseLUT saveFileName si  
    clear ss transferredIndexList ui uj uniqueLabels uniquePlanarNewCPMatrix
end%if variableCleanupFlag == 1
