%This code takes a StrandEditor globalDataStruct
%and subsamples it by a factor of 3 in each dimension
%by taking a maximum filter and then subsampling, and then re-centerlining and
%restrandifying.  A "junk" grayscale is created so that StrandEditor can run
%on the smaller set.  Alternatively, you could subsample the original grayscale
%and manually change the pointer in globalDataStruct to that file instead.


load('F:\Phil\AOH Datasets\Dataset_DB\ThirdSize\SE2010_10_18-13_17_34-Mod-StrandEdit-pbDB-VidaProject.mat')
reduceFactor = 3;
cropRegion =    [1 1200 ; ...
                101 1350 ; ...
                91 1300];

cpMatrix = globalDataStruct.cpMatrix;
ar = globalDataStruct.ar;
maskSize = globalDataStruct.maskSize;

numCP = size(cpMatrix,1);
newMaskSize = (cropRegion(:,2)-cropRegion(:,1) + 1)';
reducedMaskSize = ceil(newMaskSize./3);
reducedRadiusMap = zeros(reducedMaskSize,'single');
reducedLabelMap = zeros(reducedMaskSize,'uint16');
for cpIter = 1:numCP;
    if cpMatrix(cpIter,6) == 1 %active
        cy = cpMatrix(cpIter,1);
        cx = cpMatrix(cpIter,2);
        cz = cpMatrix(cpIter,3);
        cl = cpMatrix(cpIter,7);
        crad = ar(cpIter);
        %---
        inFlag1 = and(cy>=cropRegion(1,1) , cy<=cropRegion(1,2));
        inFlag2 = and(cx>=cropRegion(2,1) , cx<=cropRegion(2,2));
        inFlag3 = and(cz>=cropRegion(3,1) , cz<=cropRegion(3,2));
        inFlagAll = and(and(inFlag1,inFlag2),inFlag3);
        if inFlagAll == 1,
            rcy = ceil((cy-cropRegion(1,1)+1)/reduceFactor);
            rcx = ceil((cx-cropRegion(2,1)+1)/reduceFactor);
            rcz = ceil((cz-cropRegion(3,1)+1)/reduceFactor);
            rcrad = crad/reduceFactor;
            currentReducedRadius = reducedRadiusMap(rcy,rcx,rcz);
            reducedRadiusMap(rcy,rcx,rcz) = max(currentReducedRadius,rcrad);
            currentReducedLabel = reducedLabelMap(rcy,rcx,rcz);
            reducedLabelMap(rcy,rcx,rcz) = max(currentReducedLabel,cl);
        end%if inFlagAll == 1,
    end%if cpMatrix(cpIter,6) == 1 %active
end%for cpIter = 1:numCP;
        

myMask = reducedLabelMap>0;
%Run recentering code
RecenteringCode
%result is stored in centerlineMask



%Reconstruct the cpMatrix
cpCounter = 0;
ff = find(reducedLabelMap>0);
numReducedPoints = numel(ff);
reducedCPMatrix = zeros(numReducedPoints,7);
reducedRadiusList = zeros(numReducedPoints,1);
[rcyList,rcxList,rczList] = ind2sub(reducedMaskSize,ff);
for reducedCPIter = 1:numReducedPoints,
    thisIndex = ff(reducedCPIter);
    thisRad = reducedRadiusMap(thisIndex);
    thisLabel = reducedLabelMap(thisIndex);
    rcy = rcyList(reducedCPIter);
    rcx = rcxList(reducedCPIter);
    rcz = rczList(reducedCPIter);
    if centerlineMask(rcy,rcx,rcz)>0,
        cpCounter = cpCounter+1;
        reducedCPMatrix(cpCounter,:) = [rcy, rcx, rcz, 0 , 0 , 1 , thisLabel];
        reducedRadiusList(cpCounter) = thisRad;
    end%if centerlineMask(rcy,rcx,rcz)>0,
end%for reducedCPIter = 1:numReducedPoints,
reducedCPMatrix = reducedCPMatrix(1:cpCounter,:);
reducedRadiusList = reducedRadiusList(1:cpCounter);

%Make a junk random grayscale file
junkGray = uint8(rand(reducedMaskSize)>0.99)*255;
junkGrayFileName = fullfile(cd,'junkGray.mat');
save(junkGrayFileName,'junkGray');

%Initialize the globalDataStruct
clear global globalDataStruct
grayThresh = 100;
globalDataStruct.maskSize = reducedMaskSize;
globalDataStruct.fileRootName = 'pbDB_subSample3';
globalDataStruct.grayscaleFileName = junkGrayFileName;
globalDataStruct.cpMatrix = reducedCPMatrix;
globalDataStruct.ar = reducedRadiusList;
globalDataStruct.grayThresh = grayThresh;
InitializeGlobalDataStruct

%Save the globalDataStruct
fileRootName = globalDataStruct.fileRootName;
dateString = datestr(now,'yyyy_mm_dd-HH_MM_SS');
[pathName,fileName,ext] = fileparts(fileRootName);
saveFileName = fullfile(pathName,['SE',dateString,'-SubSampled','-StrandEdit-',fileName,ext]);
save(saveFileName,'globalDataStruct');

%%
RestrandifyAndTransferLabels


