%This code takes a globalDataStruct from the StrandEditor
%and performs a circuit analysis on it.  It takes the following
%general steps:
%1) Restrandify to ensure any added points are "strandified"
%2) Crop a region rectangular, if so desired
%3) Remove all free ends that are not sources or sinks
%4) Grab only the largest connected element (in case cropping
%       caused an island to form
%5) Restrandify again to ensure a proper strandification
%6) Convert from a strand-based representation to node-based
%7) Populate the conductance matrix and invert to find node voltages
%8) Create masks for visualization
%-----------------------------------
cropYXZRange = [1 1200; 100 1350; 90 1300]
startStrandID = 101 %start searching strand for single conncted volume
sourceLabelList = [330 430]
groundLabelList = [340 440]
sourceVoltage = 100
groundVoltage = 10
unitLengthConductance = 1e9
variableCleanupFlag = 1;

%-----------------------------------
if ~exist('globalDataStruct','var'),
    disp('Load a StrandEdit file');
    pause
    pause
end


%Stage 1) Restrandify
disp(['Stage 1: Restrandifying Round 1 : ',datestr(now)]);
RestrandifyAndTransferLabels


%Stage 2) Crop a rectangular region if desired
disp(['Stage 2: Cropping rectangular region : ',datestr(now)]);
maskSize = globalDataStruct.maskSize;
minY = max(1,cropYXZRange(1,1));
minX = max(1,cropYXZRange(2,1));
minZ = max(1,cropYXZRange(3,1));
maxY = min(cropYXZRange(1,2),maskSize(1));
maxX = min(cropYXZRange(2,2),maskSize(2));
maxZ = min(cropYXZRange(3,2),maskSize(3));
cpMatrix = globalDataStruct.cpMatrix;
fy = find(or(cpMatrix(:,1)>maxY,cpMatrix(:,1)<minY));
fx = find(or(cpMatrix(:,2)>maxX,cpMatrix(:,2)<minX));
fz = find(or(cpMatrix(:,3)>maxZ,cpMatrix(:,3)<minZ));
cpMatrix(fy,6) = 0; %make inactive;
cpMatrix(fx,6) = 0; %make inactive;
cpMatrix(fz,6) = 0; %make inactive;
globalDataStruct.cpMatrix = cpMatrix;
globalDataStruct.fileRootName = [globalDataStruct.fileRootName,'-crop'];
if variableCleanupFlag == 1,
    clear minY minX minZ maxY maxX maxZ fy fx fz
end

%Stage 3) Remove free ends not attached specificed vessel types
disp(['Stage 3: Removing Free Ends : ',datestr(now)]);
safeLabelList = cat(2,sourceLabelList,groundLabelList);
RemoveFreeEnds

%Stage 4) Retain only single connected neighborhood
disp(['Stage 4: Retaining only single connected neighborhood : ',datestr(now)]);
voltageStrandLabelList = sourceLabelList;
groundStrandLabelList = groundLabelList;
FindSingleConnectedNeighborhood

%Stage 5) Restrandify again
disp(['Stage 5: Restrandify Round 2 : ',datestr(now)]);
RestrandifyAndTransferLabels
% % % % 
% % % % %Stage 6 & 7) Convert to node connectivity representation and circuit analysis
% % % % disp(['Stage 3: Convert to node structure and circuit analysis : ',datestr(now)]);
% % % % Ev = sourceVoltage;
% % % % Eg = groundVoltage;
% % % % conductanceMultiplier = unitLengthConductance;
% % % % ConvertToNodeAndAnalyzeCircuit;
% % % % 
% % % % %Stage 8) Create Masks for visualization
% % % % disp(['Stage 8 : Create masks for visulization',datestr(now)]);
% % % % disp(['Stage 8a : VoltageMask',datestr(now)]);
% % % % GenerateCircuitVoltageMask
% % % % disp(['Stage 8b : VoltageMask HalfSize',datestr(now)]);
% % % % MakeHalfSizeMasks
% % % % disp(['Writing Mask : ',datestr(now)]);
% % % % maketiff(halfMask,'halfVoltageMask.tif');
% % % % clear halfMask
% % % % disp(['Stage 8c : CurrentMask',datestr(now)]);
% % % % GenerateCircuitCurrentMask
% % % % disp(['Stage 8d : CurrentMask HalfSize',datestr(now)]);
% % % % MakeHalfSizeMasks
% % % % maketiff(halfMask,'halfCurrentMask.tif');
% % % % disp(['Writing Mask : ',datestr(now)]);
% % % % clear halfMask




