%Load vectorizedStructure from the larger dataset
%Load globalDataStruct from the StrandEditor file for the smaller dataset
%Change the newCroppedScaleMatFileName (below) to point to the grayscale mat file for the
    %larger dataset
%Change the dataOffset to account for any global shifts between the two datasets
%Change the cropLimits to be the size of the smaller dataset
%   the (unlabeled) points in this region of the larger dataset will be removed and
%   replaced with the (labeled) points in this region of the smaller dataset

dataOffset = [0 0 0];%offset from labeledCP to newCP's : offset = newCoord - labeledCoord (labeled = currentCPMatrix)
%-----
cropLimits = [1 1119 1 1081 1 299];% remove CP's within these limits in the (unlabeled) joiner dataset
newCroppedGrayScaleMatFileName = 'F:\Phil\AOH_Datasets\pbAU_Trials\pbAUAnalysis\Surface\pbAU-registeredVascGrayscale.mat';


%-----
%Grab the current centerline points, radii, and labels
currentCPMatrix = globalDataStruct.cpMatrix;
currentMaskSize = globalDataStruct.maskSize;
ar = globalDataStruct.ar;
labeledIndexList = find(currentCPMatrix(:,7)~=110);%where in the currentCPMatrix is the labeled point.
numLabeledPoints = size(labeledIndexList,1);%how many points were labeled
transferredIndexList = zeros(size(labeledIndexList));%0 or 1 = transferred/not yet
%-----
%Convert the information to the centerlineRadiusStructure format:
%First grab the existing (labeled) centerline points
activeFlags = currentCPMatrix(:,6)==1;
cc1 = currentCPMatrix(activeFlags,1);
cc2 = currentCPMatrix(activeFlags,2);
cc3 = currentCPMatrix(activeFlags,3);
ccr = ar(activeFlags);
%-----
%Grab the (unlabeled) joiner centerline points from the dataset's vectorizedStructure;
allJoinerVerts = vectorizedStructure.Vertices.AllVerts;
allJoinerRadii = vectorizedStructure.Vertices.AllRadii;
fullMaskSize = vectorizedStructure.maskSize;
cropFlag1 = and(allJoinerVerts(:,1)>=cropLimits(1) , allJoinerVerts(:,1)<=cropLimits(2));
cropFlag2 = and(allJoinerVerts(:,2)>=cropLimits(3) , allJoinerVerts(:,2)<=cropLimits(4));
cropFlag3 = and(allJoinerVerts(:,3)>=cropLimits(5) , allJoinerVerts(:,3)<=cropLimits(6));
cropFlagAll = and(and(cropFlag1,cropFlag2),cropFlag3);
keepFlagAll = ~cropFlagAll;
%Compile the two lists, but adjust coordinates to match the fullDataSet coordinate frame
cc1 = [cc1+dataOffset(1);allJoinerVerts(keepFlagAll,1)];
cc2 = [cc2+dataOffset(2);allJoinerVerts(keepFlagAll,2)];
cc3 = [cc3+dataOffset(3);allJoinerVerts(keepFlagAll,3)];
ccr = [ccr;allJoinerRadii(keepFlagAll)];
%-----
%Trim the mask to ignore all points that now fall outside the fullMaskSize
flag1 = cc1>0 && cc1<fullMaskSize(1);
flag2 = cc2>0 && cc2<fullMaskSize(2);
flag3 = cc3>0 && cc3<fullMaskSize(3);
allFlag = and(flag1,and(flag2,flag3));
cc1 = cc1(allFlag);
cc2 = cc2(allFlag);
cc3 = cc3(allFlag);
ccr = ccr(allFlag);
%-----
centerlineRadiusStructure = [];
centerlineRadiusStructure.pixelList = sub2ind(fullMaskSize,cc1,cc2,cc3);
centerlineRadiusStructure.maskSize = fullMaskSize;
centerlineRadiusStructure.radiusList = ccr;
%Run the joined centerlineRadiusStructure through the Strandify algorithm
newVectorizedStructure = JustStrandifyCenterlineRadiusStructure(centerlineRadiusStructure);
%-----
%Construct a newCPMatrix from the newly re-strandified structure
newMaskSize = newVectorizedStructure.maskSize;
av = newVectorizedStructure.Vertices.AllVerts;
ar = newVectorizedStructure.Vertices.AllRadii;
si = newVectorizedStructure.Vertices.StrandIndices;
jpTemp = newVectorizedStructure.Vertices.JunctionPoints;
fepTemp = newVectorizedStructure.Vertices.FreeEndPoints;
jp = av(jpTemp==1,:);
fep = av(fepTemp==1,:);
newCPMatrix = [av,fepTemp,jpTemp,ones(size(jpTemp)),zeros(size(jpTemp))+110];
numNewCP = size(newCPMatrix,1);
%-----
%Create a sparse reverse look-up table of the newCPMatrix coordinates
%to improve speed over performing multiple find operatations.
%Sparse matrices are limited to 10^9 in size: therefore we break it up
%into a different sparse matrix for each dim1_dim2 plane
disp('Generating Reverse Lookup Table');
maxCoords = max(newMaskSize,currentMaskSize);
maxPlanarLookup = maxCoords(1)*maxCoords(2);
coordMultiplier = 10.^ceil(log10(maxCoords));
maxLookup = coordMultiplier(2) + maxCoords(2).*coordMultiplier(3) + maxCoords(3);
reverseLUT = {};
for kk = 1:newMaskSize(3);
    inPlaneNewCPMatrixIndexList = find(newCPMatrix(:,3)==kk);
    planarNewCPMatrix = newCPMatrix(inPlaneNewCPMatrixIndexList,:);
    numPlanarCP = size(planarNewCPMatrix,1);
    ii = zeros(numPlanarCP,1);
    jj = ones(numPlanarCP,1);
    ss = inPlaneNewCPMatrixIndexList;
    for qq = 1:numPlanarCP,
        currentPlanarCP = planarNewCPMatrix(qq,1:3);
        currentPlanarLookup = currentPlanarCP(1) + (currentPlanarCP(2)-1) * maxCoords(1);
        ii(qq) = currentPlanarLookup;
    end%for qq = 1:numPlanarCP,
    planarReverseLUT = sparse(ii,jj,ss);
    reverseLUT{kk} = planarReverseLUT;
end%for kk = 1:newMaskSize(3);
%-----
%Go through each labeled centerline point in the original currentCPMatrix and attempt to transfer
%it to an identically coordinated point in the newCPMatrix
%-----
counter = 0;
for kk = 1:numLabeledPoints,
    origCoords = currentCPMatrix(labeledIndexList(kk),1:3);
    origLabel = currentCPMatrix(labeledIndexList(kk),7);
    adjustedCoords = origCoords + dataOffset;
    adjustedCoords = max(1,min(maxCoords,adjustedCoords));
    currentPlanarLookup = adjustedCoords(1) + (adjustedCoords(2)-1) * maxCoords(1);
    planarReverseLUT = reverseLUT{adjustedCoords(3)};
    planarReverseLUTSize = size(planarReverseLUT,1);
    if currentPlanarLookup <= planarReverseLUTSize,
        newIndex = planarReverseLUT(currentPlanarLookup);
        if newIndex ~= 0,
            counter = counter+1;
            newCPMatrix(newIndex,7) = origLabel;
            transferredIndexList(newIndex) =  1;
        end%if newIndex ~= 0,
    end%if currentPlanarLookup <= planarReverseLUTSize,
end%for kk = 1:numLabeledPoints,    
%-----
%If any of the labeled points did not get transferred, search within a small region
%around that coordinate for a unlabeled point to associate.  This is done in case
%the re-strandification process moved points by a few pixels (especially during the
%node collaspse algorithm)
%
%Not yet implemented
keyboard
%
%
%-----
%Go strand-by-strand, and make the majority label of the strand the label for all points
%on that strand.
newStrands = newVectorizedStructure.Strands;
numStrands = size(newStrands,2);
for kk = 1:numStrands,
    currentStrand = newStrands(kk);
    interiorVertices = currentStrand.InteriorVertices;
    interiorLabels = newCPMatrix(interiorVertices,7);
    uniqueLabels = unique(interiorLabels);
    if size(uniqueLabels,1) > 1,
        labelHist = hist(interiorLabels,uniqueLabels);
        [histMax,maxPos] = max(labelHist);
        majorityLabel = uniqueLabels(maxPos);
        newCPMatrix(interiorVertices,7) = majorityLabel;
    end%if size(uniqueLabels,1) > 1,
end%for kk = 1:numStrands,
%-----
%Place new Strand and label data back into globalDataStruct, replacing the old data
globalDataStruct.strands = newStrands;
globalDataStruct.vectorizedStructure = newVectorizedStructure;
globalDataStruct.maskSize = newMaskSize;
globalDataStruct.cpMatrix = newCPMatrix;
globalDataStruct.strands = newVectorizedStructure.Strands;
globalDataStruct.av = av;
globalDataStruct.ar = ar;
globalDataStruct.jp = jp;
globalDataStruct.fep = fep;
globalDataStruct.si = si;
globalDataStruct.grayscaleFileName = newCroppedGrayScaleMatFileName;
%-----
fileRootName = globalDataStruct.fileRootName;
dateString = datestr(now,'yyyy_mm_dd-HH_MM_SS');
[pathName,fileName,ext] = fileparts(fileRootName);
saveFileName = fullfile(pathName,['SE',dateString,'-Joined','-StrandEdit-',fileName,ext]);
saveBox = warndlg('Saving Progress - Please Wait');
pause(0.1);
save(saveFileName,'globalDataStruct');
close(saveBox);
