%Transfer labels, and mark differences between datasets without merging

%Load vectorizedStructure from the larger dataset
% load('P:\AOH\pbDB\VidaSuiteProcessing\pbDB-VidaProject.mat');

%Load globalDataStruct from the StrandEditor file for the smaller dataset
% load('G:\AOH\pbDB_surface_allRC\Mod-2010_06_08_16_45_18-StrandEdit-pbDB-VidaProject.mat');

%Change the newCroppedScaleMatFileName (below) to point to the grayscale mat file for the
%larger dataset
%Change the dataOffset to account for any global shifts between the two datasets
%Change the cropLimits to be the size of the smaller dataset
%   the (unlabeled) points in this region of the larger dataset will be removed and
%   replaced with the (labeled) points in this region of the smaller dataset
clc

% % % % dataOffset = [4 22 5];%offset from currentCPs to newCPs
dataOffsetGuess = [ -10    -7   -16];%offset from currentCPs to newCPs
dataOffset = dataOffsetGuess;
searchForBestOffset = 0;


%-----
% cropLimits = [1 120 1 508 1 586];% remove CP's within these limits in the (unlabeled) joiner dataset
if isunix
    newCroppedGrayScaleMatFileName = '';
else
    newCroppedGrayScaleMatFileName = 'P:\AOH\pbDB\VidaSuiteProcessing\RegisteredFiles\StitchedFilesFolder\pbDB-registeredVascGrayscale.mat';
end


vectorizedStructure = vidaProject.globalParameters.vectorizedStructure;
fullMaskSize = vectorizedStructure.maskSize;

%Grab the current centerline points, radii, and labels
currentCPMatrix = globalDataStruct.cpMatrix;
currentMaskSize = globalDataStruct.maskSize;

%labeled ctps
labeledIndexList = find(currentCPMatrix(:,7)~=110);%where in the currentCPMatrix is the labeled point.
numLabeledPoints = size(labeledIndexList,1);%how many points were labeled
transferredIndexList = zeros(size(labeledIndexList));%0 or 1 = transferred/not yet

%inactive ctps
inactiveLabeledList = find(currentCPMatrix(:,6)==0);
numInactivePoints = numel(inactiveLabeledList);
inactiveIndexList = zeros(numInactivePoints,1);%0 or 1 = transferred/not yet
inacetiveLabel = 620;  % Marked for revision - need latest Strand Editor edition

%% convert unlabeled strandset to cpMatrix
%Construct a newCPMatrix from the newly re-strandified structure
% newMaskSize = vectorizedStructure.maskSize;
av = vectorizedStructure.Vertices.AllVerts;
ar = vectorizedStructure.Vertices.AllRadii;
% si = vectorizedStructure.Vertices.StrandIndices;
jpTemp = vectorizedStructure.Vertices.JunctionPoints;
fepTemp = vectorizedStructure.Vertices.FreeEndPoints;
% jp = av(jpTemp==1,:);
% fep = av(fepTemp==1,:);
newCPMatrix = [av,fepTemp,jpTemp,ones(size(jpTemp)),zeros(size(jpTemp))+110];
% numNewCP = size(newCPMatrix,1);


%% find matching point by reverse LUT


%Create a sparse reverse look-up table of the newCPMatrix coordinates
%to improve speed over performing multiple find operatations.
%Sparse matrices are limited to 10^9 in size: therefore we break it up
%into a different sparse matrix for each dim1_dim2 plane
fprintf('\nGenerating Reverse Lookup Table');
maxCoords = fullMaskSize;
maxPlanarLookup = maxCoords(1)*maxCoords(2);
coordMultiplier = 10.^ceil(log10(maxCoords));
maxLookup = coordMultiplier(2) + maxCoords(2).*coordMultiplier(3) + maxCoords(3);
reverseLUT = {};
for kk = 1:fullMaskSize(3);
    inPlaneNewCPMatrixIndexList = find(newCPMatrix(:,3)==kk);
    planarNewCPMatrix = newCPMatrix(inPlaneNewCPMatrixIndexList,:);
    numPlanarCP = size(planarNewCPMatrix,1);
    ii = zeros(numPlanarCP,1);
    jj = ones(numPlanarCP,1);
    ss = inPlaneNewCPMatrixIndexList;
    for qq = 1:numPlanarCP,
        currentPlanarCP = planarNewCPMatrix(qq,1:3);
        currentPlanarLookup = currentPlanarCP(1) + (currentPlanarCP(2)-1) * maxCoords(1);
        ii(qq) = currentPlanarLookup;
    end%for qq = 1:numPlanarCP,
    planarReverseLUT = sparse(ii,jj,ss);
    reverseLUT{kk} = planarReverseLUT; %#ok<AGROW>
end%for kk = 1:newMaskSize(3);

%% perform a greedy search to find the offset that maximizes label transfer
if searchForBestOffset
    x1Range = -10:10;
    x2Range = -10:10;
    x3Range = -25:25;
    x1sz = numel(x1Range);
    x2sz = numel(x2Range);
    x3sz = numel(x3Range);
    res = zeros(x1sz*x2sz*x3sz,4);
    iCount = 1;
    for iX1 = x1Range
        for iX2 = x2Range
            for iX3 = x3Range
                bestGuessOffset = [iX1 iX2 iX3];
                fprintf('\nOffset = %d,%d,%d',bestGuessOffset)


                %-exact matching
                transferredIndexList = zeros(size(transferredIndexList));
                counter = 0;
                for kk = 1:numLabeledPoints,
                    origCoords = currentCPMatrix(labeledIndexList(kk),1:3);
                    origLabel = currentCPMatrix(labeledIndexList(kk),7);
                    adjustedCoords = origCoords + bestGuessOffset;
                    adjustedCoords = max(1,min(maxCoords,adjustedCoords));
                    currentPlanarLookup = adjustedCoords(1) + (adjustedCoords(2)-1) * maxCoords(1);
                    planarReverseLUT = reverseLUT{adjustedCoords(3)};
                    planarReverseLUTSize = size(planarReverseLUT,1);
                    if currentPlanarLookup <= planarReverseLUTSize,
                        newIndex = planarReverseLUT(currentPlanarLookup);
                        if newIndex ~= 0,
                            counter = counter+1;
                            newCPMatrix(newIndex,7) = origLabel;
                            transferredIndexList(newIndex) =  1;
                        end%if newIndex ~= 0,
                    end%if currentPlanarLookup <= planarReverseLUTSize,
                end%for kk = 1:numLabeledPoints,

                success = 1-(numLabeledPoints-sum(transferredIndexList))/numLabeledPoints;

                fprintf('\t %3.2f%%labels transferred',success*100);
                res(iCount,:) = [bestGuessOffset success];
                iCount = iCount + 1;
                pause(0.01)
            end
        end
    end

    [sMax iMax] = max(res(:,4));
    fprintf('\nBest offsets: Offset = %d,%d,%d (%f)',res(iMax,:)');
    dataOffset = res(iMax,1:3);
    plot(res(:,1),res(:,4),'r.',res(:,2),res(:,4),'g.',res(:,3),res(:,4),'b.')
    % scatter3(res(:,1),res(:,2),res(:,3),exp(res(:,4)./max(res(:,4)))
end
%% find inactive (deleted) ctps, this should be labeled for display
fprintf('\nExact matching for inactive ctps');
%Go through each labeled centerline point in the original currentCPMatrix and attempt to transfer
%it to an identically coordinated point in the newCPMatrix
%-----
counter = 0;
for kk = 1:numInactivePoints,
    origCoords = currentCPMatrix(inactiveLabeledList(kk),1:3);
    origLabel = currentCPMatrix(inactiveLabeledList(kk),7);
    adjustedCoords = origCoords + dataOffset;
    adjustedCoords = max(1,min(maxCoords,adjustedCoords));
    currentPlanarLookup = adjustedCoords(1) + (adjustedCoords(2)-1) * maxCoords(1);
    planarReverseLUT = reverseLUT{adjustedCoords(3)};
    planarReverseLUTSize = size(planarReverseLUT,1);
    if currentPlanarLookup <= planarReverseLUTSize,
        newIndex = planarReverseLUT(currentPlanarLookup);
        if newIndex ~= 0,
            counter = counter+1;
            newCPMatrix(newIndex,7) = inacetiveLabel;
            inactiveIndexList(newIndex) =  1;
        end%if newIndex ~= 0,
    end%if currentPlanarLookup <= planarReverseLUTSize,
end%for kk = 1:numLabeledPoints,
fprintf('\t %3.2f%%inactive ctps labeled',numInactivePoints/sum(inactiveIndexList)*100);
% figure;
% plot3(currentCPMatrix(:,1),currentCPMatrix(:,2),currentCPMatrix(:,3),'y.','markerSize',3)
% hold on
% plot3(currentCPMatrix(inactiveLabeledList,1),currentCPMatrix(inactiveLabeledList,2),currentCPMatrix(inactiveLabeledList,3),'ro')

%%
fprintf('\nExact matching for lableled ctps');
%Go through each labeled centerline point in the original currentCPMatrix and attempt to transfer
%it to an identically coordinated point in the newCPMatrix
%-----
counter = 0;
labeledIndexListFailed = zeros(size(labeledIndexList));%flag untransfered
for kk = 1:numLabeledPoints,
    origCoords = currentCPMatrix(labeledIndexList(kk),1:3);
    origLabel = currentCPMatrix(labeledIndexList(kk),7);
    adjustedCoords = origCoords + dataOffset;
    adjustedCoords = max(1,min(maxCoords,adjustedCoords));
    currentPlanarLookup = adjustedCoords(1) + (adjustedCoords(2)-1) * maxCoords(1);
    planarReverseLUT = reverseLUT{adjustedCoords(3)};
    planarReverseLUTSize = size(planarReverseLUT,1);
    if currentPlanarLookup <= planarReverseLUTSize,
        newIndex = planarReverseLUT(currentPlanarLookup);
        if newIndex ~= 0,
            counter = counter+1;
            newCPMatrix(newIndex,7) = origLabel;
            transferredIndexList(newIndex) =  1;
        else
            labeledIndexListFailed(kk) = 1;
        end%if newIndex ~= 0,
    end%if currentPlanarLookup <= planarReverseLUTSize,
end%for kk = 1:numLabeledPoints,
fprintf('\t %3.2f%%labels transferred',(sum(transferredIndexList)-sum(labeledIndexListFailed))/numLabeledPoints*100);
% % % % % % % % % % %% expand ctps list with the transfer-failed labeled points, restrandify, and repeat label transfer then restrandify
% % % % % % % % % % labeledIndexListFailed = logical(labeledIndexListFailed);
% % % % % % % % % % labeledIndexListFailed = labeledIndexListFailed & currentCPMatrix(labeledIndexList,6); %only keep active ones
% % % % % % % % % % labeledIndexListFailed = labeledIndexList(logical(labeledIndexListFailed));
% % % % % % % % % % cc1 = [newCPMatrix(:,1);currentCPMatrix(labeledIndexListFailed,1)];
% % % % % % % % % % cc2 = [newCPMatrix(:,2);currentCPMatrix(labeledIndexListFailed,2)];
% % % % % % % % % % cc3 = [newCPMatrix(:,3);currentCPMatrix(labeledIndexListFailed,3)];
% % % % % % % % % % ccr = [ar;zeros(size(labeledIndexListFailed))];

%% All new added strands should be transfered (aka appended to the cpt list prior to restrandify).
%New strands have only StartToEndIndices field populated, all others are empty

nCurrentStrands = numel(globalDataStruct.strands);
strand2transferIndexList = zeros(nCurrentStrands,1);
for iSTRAND = 1 : nCurrentStrands
    strand2transferIndexList(iSTRAND) = numel(globalDataStruct.strands(iSTRAND).StartVertexIndex) == 0;
end

%concatenate list of all ctps to transfer - keep only active ones (in case a new strand has been created then deleted)
strand2transferIndexList = find(strand2transferIndexList);
newStrandsCtpsCandidates = [globalDataStruct.strands(strand2transferIndexList).StartToEndIndices]';
newStrandsCtpsCandidates = newStrandsCtpsCandidates(currentCPMatrix(newStrandsCtpsCandidates,6)==1);

%alter first digit of lables to tag them for review
tmp = currentCPMatrix(newStrandsCtpsCandidates,7); %use second extra color
tmpidx = tmp>300 & tmp < 400;
tmp(tmpidx) = tmp(tmpidx) + 400;
tmpidx = tmp>400 & tmp < 500;
tmp(tmpidx) = tmp(tmpidx) + 300;

currentCPMatrix(newStrandsCtpsCandidates,7) = tmp;
newCPMatrix = [newCPMatrix;currentCPMatrix(newStrandsCtpsCandidates,:)];





%% restrandify
cc1 = newCPMatrix(:,1);
cc2 = newCPMatrix(:,2);
cc3 = newCPMatrix(:,3);
ccr = [ar;zeros(size(newStrandsCtpsCandidates))];

%validate coordinates
cc1(cc1<1)=1;cc1(cc1>fullMaskSize(1)) = fullMaskSize(1);
cc2(cc2<1)=1;cc2(cc2>fullMaskSize(2)) = fullMaskSize(2);
cc3(cc3<1)=1;cc3(cc3>fullMaskSize(3)) = fullMaskSize(3);

%swap current and new, easier to reuse code this way
currentCPMatrix = newCPMatrix;

centerlineRadiusStructure = [];
centerlineRadiusStructure.pixelList = sub2ind(fullMaskSize,cc1,cc2,cc3);
centerlineRadiusStructure.maskSize = fullMaskSize;
centerlineRadiusStructure.radiusList = ccr;
%Run the joined centerlineRadiusStructure through the Strandify algorithm
newVectorizedStructure = JustStrandifyCenterlineRadiusStructure(centerlineRadiusStructure);
%-----
%-----
%Construct a newCPMatrix from the newly re-strandified structure
newMaskSize = newVectorizedStructure.maskSize;
av = newVectorizedStructure.Vertices.AllVerts;
ar = newVectorizedStructure.Vertices.AllRadii;
si = newVectorizedStructure.Vertices.StrandIndices;
jpTemp = newVectorizedStructure.Vertices.JunctionPoints;
fepTemp = newVectorizedStructure.Vertices.FreeEndPoints;
jp = av(jpTemp==1,:);
fep = av(fepTemp==1,:);
newCPMatrix = [av,fepTemp,jpTemp,ones(size(jpTemp)),zeros(size(jpTemp))+110];
numNewCP = size(newCPMatrix,1);

%% Re-transfer labels to newly strandify representation
bestGuessOffset = dataOffset;%offset from currentCP to newCP's : offset = newCoord - currentCoord
labeledIndexList = find(currentCPMatrix(:,7)~=110);%where in the currentCPMatrix is the labeled point.
numLabeledPoints = size(labeledIndexList,1);%how many points were labeled
transferredIndexList = zeros(size(labeledIndexList));%0 or 1 = transferred/not yet
%-----

%-----
%Create a sparse reverse look-up table of the newCPMatrix coordinates
%to improve speed over performing multiple find operatations.
%Sparse matrices are limited to 10^9 in size: therefore we break it up
%into a different sparse matrix for each dim1_dim2 plane
disp('Generating Reverse Lookup Table');
maxCoords = max(newMaskSize,currentMaskSize);
maxPlanarLookup = maxCoords(1)*maxCoords(2);
coordMultiplier = 10.^ceil(log10(maxCoords));
maxLookup = coordMultiplier(2) + maxCoords(2).*coordMultiplier(3) + maxCoords(3);
reverseLUT = {};
for kk = 1:newMaskSize(3);
    inPlaneNewCPMatrixIndexList = find(newCPMatrix(:,3)==kk);
    planarNewCPMatrix = newCPMatrix(inPlaneNewCPMatrixIndexList,1:3);
    [uniquePlanarNewCPMatrix,ui,uj] = unique(planarNewCPMatrix,'rows');
    numPlanarCP = size(uniquePlanarNewCPMatrix,1);
    ii = zeros(numPlanarCP,1);
    jj = ones(numPlanarCP,1);
    ss = inPlaneNewCPMatrixIndexList(ui);
    for qq = 1:numPlanarCP,
        currentPlanarCP = uniquePlanarNewCPMatrix(qq,1:3);
        currentPlanarLookup = currentPlanarCP(1) + (currentPlanarCP(2)-1) * maxCoords(1);
        ii(qq) = currentPlanarLookup;
    end%for qq = 1:numPlanarCP,
    planarReverseLUT = sparse(ii,jj,ss);
    reverseLUT{kk} = planarReverseLUT;
end%for kk = 1:newMaskSize(3);
disp('....done.');
%-----
%Go through each labeled centerline point in the original currentCPMatrix and attempt to transfer
%it to an identically coordinated point in the newCPMatrix
%-----
counter = 0;
for kk = 1:numLabeledPoints,
    origCoords = currentCPMatrix(labeledIndexList(kk),1:3);
    origLabel = currentCPMatrix(labeledIndexList(kk),7);
    adjustedCoords = origCoords + bestGuessOffset;
    adjustedCoords = max(1,min(maxCoords,adjustedCoords));
    currentPlanarLookup = adjustedCoords(1) + (adjustedCoords(2)-1) * maxCoords(1);
    planarReverseLUT = reverseLUT{adjustedCoords(3)};
    planarReverseLUTSize = size(planarReverseLUT,1);
    if currentPlanarLookup <= planarReverseLUTSize,
        newIndex = planarReverseLUT(currentPlanarLookup);
        if newIndex ~= 0,
            counter = counter+1;
            newCPMatrix(newIndex,7) = origLabel;
            transferredIndexList(newIndex) =  1;
        end%if newIndex ~= 0,
    end%if currentPlanarLookup <= planarReverseLUTSize,
end%for kk = 1:numLabeledPoints,
%-----
%If any of the labeled points did not get transferred, search within a small region
%around that coordinate for a unlabeled point to associate.  This is done in case
%the re-strandification process moved points by a few pixels (especially during the
%node collaspse algorithm)
%
%Not yet implemented
%
%
%-----
%Go strand-by-strand, and make the majority label of the strand the label for all points
%on that strand.
newStrands = newVectorizedStructure.Strands;
numStrands = size(newStrands,2);
for kk = 1:numStrands,
    currentStrand = newStrands(kk);
    interiorVertices = currentStrand.InteriorVertices;
    interiorLabels = newCPMatrix(interiorVertices,7);
    uniqueLabels = unique(interiorLabels);
    if size(uniqueLabels,1) > 1,
        labelHist = hist(interiorLabels,uniqueLabels);
        [histMax,maxPos] = max(labelHist);
        majorityLabel = uniqueLabels(maxPos);
        newCPMatrix(interiorVertices,7) = majorityLabel;
    end%if size(uniqueLabels,1) > 1,
end%for kk = 1:numStrands,
%-----
%Place new Strand and label data back into globalDataStruct, replacing the old data
globalDataStruct.strands = newStrands;
globalDataStruct.vectorizedStructure = newVectorizedStructure;
globalDataStruct.maskSize = newMaskSize;
globalDataStruct.cpMatrix = newCPMatrix;
globalDataStruct.strands = newVectorizedStructure.Strands;
globalDataStruct.av = av;
globalDataStruct.ar = ar;
globalDataStruct.jp = jp;
globalDataStruct.fep = fep;
globalDataStruct.si = si;
%-----
fileRootName = globalDataStruct.fileRootName;
dateString = datestr(now,'yyyy_mm_dd_HH_MM_SS');
[pathName,fileName,ext] = fileparts(fileRootName);
saveFileName = fullfile(pathName,['Restrandified-',dateString,'-StrandEdit-',fileName,ext]);
saveBox = warndlg('Saving Progress - Please Wait');
pause(0.1);
save(saveFileName,'globalDataStruct');
close(saveBox);