%This code tracks vessels back to their closest labeled surface vessel.

%Load a globalDataStruct
%
%Note:  no connectivity changes can have occured since the latest restrandify round
%Changes to labels are okay.
%

%Labels: 100=Unlabeled; 200=Temp ; 300=Surface ; 400=Plunging ; 500=Micro ;
%         10=Unlabeled ; 20=Temp ; 30=Supply ; 40=Draining ;  50=Connecting ;
%
%Therefore: SurfaceArtery=330; SurfaceVein = 340;

targetLabels = [330 340];%the list of labels you are tracking to
associatedLabels = [130 140];%the list of label to associate with strands tracked to targetLabels
maxSearchDepth = 50;


Strands = globalDataStruct.vectorizedStructure.Strands;
cpMatrix = globalDataStruct.cpMatrix;
numStrands = size(Strands,2);
%-----
%Generate a label for each strand based on the cpMatrix
strandTracker = [];
%neighborStruct will keep track of the search, arranged by strandID
neighborStruct = [];
for strandIter = 1:numStrands,
    neighborStruct(strandIter).searchDepth = Inf;
    neighborStruct(strandIter).completedFlag = 0;
    currentStrand = Strands(strandIter);
    interiorVertices = currentStrand.InteriorVertices;
    interiorLabels = cpMatrix(interiorVertices,7);
    if isempty(interiorLabels),
        uniqueLabels = 110;
    else%if isempty(interiorLabels),
        uniqueLabels = unique(interiorLabels);
    end%if isempty(interiorLabels),
    %-----
    tempDepthToTarget = Inf;
    tempBestLabel = 110;
    tempBestPath = [];
    %-----
    if size(uniqueLabels,1) > 1,
        labelHist = hist(interiorLabels,uniqueLabels);
        [histMax,maxPos] = max(labelHist);
        majorityLabel = uniqueLabels(maxPos);
        currentLabel = majorityLabel;
    else%if size(uniqueLabels,1) > 1,
        currentLabel = uniqueLabels(1);
    end%if-else size(uniqueLabels,1) > 1,
    strandTracker(strandIter).originalLabel = currentLabel;
    strandTracker(strandIter).trackedLabel = 0;
    strandTracker(strandIter).distanceToTarget = Inf;
    strandTracker(strandIter).pathToTarget = [];
    currentTracker = strandTracker(strandIter);
    if ismember(currentTracker.originalLabel,targetLabels),
%         currentTracker.trackedLabel = associatedLabels(currentTracker.originalLabel==targetLabels);
        currentTracker.trackedLabel = currentLabel;
        currentTracker.distanceToTarget = 0;
        strandTracker(strandIter) = currentTracker;
    end%if ismember(currentTracker.originalLabel,targetLabels),
end%for strandIter = 1:numStrands,
clear strandIter
%-----
%Begin tracking
disp('Begin Tracking');
numChar = 0;
disp('WARNING! NOT STARTING AT STRAND 1 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!');
for startStrandIter = 1:numStrands,
    tic
% %     fprintf(repmat('\b',[1,numChar]));
% %     numChar = fprintf(num2str(startStrandIter));

    %%deleteMe=0;disp([' ']);
    %%deleteMe=0;disp(['startStrandIter = ',num2str(startStrandIter)]);
    %%deleteMe=0;ivTemp = Strands(startStrandIter).InteriorVertices;
    %%deleteMe=0;myCoords = cpMatrix(ivTemp,1:3);
    %%deleteMe=0;minCoords = min(myCoords,[],1);
    %%deleteMe=0;maxCoords = max(myCoords,[],1);
    %%deleteMe=0;disp(['minCoords: ',num2str(minCoords)]);
    %%deleteMe=0;disp(['maxCoords: ',num2str(maxCoords)]);
    
    %---
    strandCompletedFlag = 0;
    tempDepthToTarget = Inf;
    tempBestLabel = 110;
    tempBestPath = [];
    currentPath = [];
    %---
    startingStrand = Strands(startStrandIter);
    currentTracker = strandTracker(startStrandIter);
    searchDepthCounter = 1;%keeps track of how far (in units of strands) you are from the starting strand
    %-----
    %First check to see if strand has already been associated by a previous search or
    %if it itself is a target strand
    %%deleteMe=0;disp(['originalLabel = ',num2str(currentTracker.originalLabel)]);
    if ismember(currentTracker.originalLabel,targetLabels),
        %%deleteMe=0;disp(['target!']);
        tempDepthToTarget = 0;
        tempBestLabel = associatedLabels(currentTracker.originalLabel==targetLabels);
        tempBestPath = [];
        strandCompletedFlag = 1;
        %%pause
    end%if ismember(currentTracker.originalLabel,targetLabels),
    %----
    if ismember(currentTracker.trackedLabel,associatedLabels),
        %%deleteMe=0;disp(['associatedLabel!']);
        tempDepthToTarget = currentTracker.distanceToTarget;
        tempBestLabel = currentTracker.trackedLabel;
        tempBestPath = currentTracker.pathToTarget;
        strandCompletedFlag = 1;
        %%pause
    end%if ismember(currentTracker.trackedLabel,associatedLabels),
    strandTracker(startStrandIter) = currentTracker;
    %------
    %-----
    neighborStrands = [startingStrand.StartVertexNeighborStrands,startingStrand.EndVertexNeighborStrands];
    startAttachedStrands = unique(neighborStrands(neighborStrands~=startStrandIter));
    numAttached = size(startAttachedStrands,2);
    if isempty(startAttachedStrands) || strandCompletedFlag == 1,
        %%deleteMe=0;disp(['possible empty startAttachedStrands']);
        strandCompletedFlag = 1;
        trackedToTargetFlag = 0;
    else%if isempty(startAttachedStrands) || strandCompletedFlag == 0,,
        currentStrandID = startAttachedStrands(1);
        currentStrand = Strands(currentStrandID);
        startOrEndFlag = ismember(startStrandIter,currentStrand.EndVertexNeighborStrands);
            %startOrEndFlag = 0 if we got to this strand from its startVertex
            %               = 1 if we got to this strand from its endVertex
        tempDepthToTarget = Inf;
        currentPath = [currentStrandID];
        tempBestPath = [];
       
    end%if-else isempty(startAttachedStrands) || strandCompletedFlag == 0,,
    %%deleteMe=0;disp(['startAttachedStrands = ',num2str(startAttachedStrands)]);
    %-----
    tempCounter = 0;
    while strandCompletedFlag == 0,
        tempCounter = tempCounter + 1;
%         numChar = numChar + fprintf([' : ',num2str(searchDepthCounter)]);
        trackedToTargetFlag = 0; 
        currentStrand = Strands(currentStrandID);
        currentTracker = strandTracker(currentStrandID);
        neighborStruct(currentStrandID).searchDepth = searchDepthCounter;
        neighborStruct(currentStrandID).startOrEndFlag = startOrEndFlag;  
       
        %%deleteMe=0;disp(['CurrentPath = ',num2str(currentPath)]);
        %%deleteMe=0;disp(['CurrentStrandID = ',num2str(currentStrandID)]);
        %%deleteMe=0;disp(['CurrentStrandStart = ',num2str(currentStrand.StartVertexNeighborStrands)]);
        %%deleteMe=0;disp(['CurrentStrandEnd = ',num2str(currentStrand.EndVertexNeighborStrands)]);
      
        
        %-----
        %Check if the currentStrand is already associated
        if ismember(currentTracker.trackedLabel,associatedLabels),
            %%deleteMe=0;disp(['CurrentStrand Previously associated']);
            %%pause
            thisPathDepth = currentTracker.distanceToTarget+searchDepthCounter;
            if thisPathDepth < tempDepthToTarget,
                tempDepthToTarget = thisPathDepth;
                tempBestPath = [currentPath,currentTracker.pathToTarget];
                tempBestLabel = currentTracker.trackedLabel;
                neighborStruct(currentStrandID).completedFlag = 1;
                trackedToTargetFlag = 1;
                %%deleteMe=0;disp(['CurrentStrand is a associated!!!!!!']);
                %%deleteMe=0;disp(['tempBestPath = ',num2str(tempBestPath)]);
%                 pause
            end%if thisPathDepth < tempDepthToTarget,
        else%if ismember(currentTracker.trackedLabel,associatedLabels),
            %Check if the currentStrand is a targetStrand
            if ismember(currentTracker.originalLabel,targetLabels),
                thisPathDepth = searchDepthCounter;
                tempDepthToTarget = thisPathDepth;
                tempBestPath = currentPath;
                tempBestLabel = associatedLabels(currentTracker.originalLabel==targetLabels);
                neighborStruct(currentStrandID).completedFlag = 1;
                trackedToTargetFlag = 1;
                %%deleteMe=0;disp(['CurrentStrand is a target!!!!!!!!!!']);
                %%deleteMe=0;disp(['tempBestPath = ',num2str(tempBestPath)]);
%                 pause
            end%if ismember(currentTracker.originalLabel,targetLabels),
        end%if-else ismember(currentTracker.trackedLabel,associatedLabels),
        %-----
        if trackedToTargetFlag == 0 && searchDepthCounter < tempDepthToTarget
            %the current strand is not a target nor has it been tracked to a target
            %pick one of the its attached strands for the next round
            %increment the searchDepth by one, and continue search.
            %%deleteMe=0;disp(['Current Strand is not a target']);
            startOrEndFlag = neighborStruct(currentStrandID).startOrEndFlag;
            if startOrEndFlag == 0,%reached this strand by its startVertex
                potentialNextStrandList = currentStrand.EndVertexNeighborStrands;
            else%if startOrEndFlag == 0,
                potentialNextStrandList = currentStrand.StartVertexNeighborStrands;
            end
            
            
            %-----
            %Eliminate backtracking 
            for kk = 1:size(currentPath,2);
                potentialNextStrandList = potentialNextStrandList(potentialNextStrandList~=currentPath(kk));
            end
            %Eliminate already-processed strands, but only if it's recorded searchDepth is
            %greater or equal to the current searchDepth
            tempNextStrandList = [];
            for kk = 1:size(potentialNextStrandList,2),
                currentPotentialStrandID = potentialNextStrandList(kk);
                recordedSearchDepth = neighborStruct(currentPotentialStrandID).searchDepth;
                if searchDepthCounter + 1  < recordedSearchDepth,%try this path again, now a shorter distance from start
                    tempNextStrandList = [tempNextStrandList,potentialNextStrandList(kk)];
                end%if searchDepthCounter < recordedSearchDepth,
            end%for kk = 1:size(potentialNextStrandList,2),
            potentialNextStrandList = tempNextStrandList;
% %             potentialNextStrandList = potentialNextStrandList(~[neighborStruct([potentialNextStrandList]).completedFlag]);
            %----
            tempNextStrandList = [];
            %Eliminate strands that are already on the search path (no circling back)
            for kk = 1:size(potentialNextStrandList,2),
                 currentPotentialStrandID = potentialNextStrandList(kk);
                 if ~ismember(currentPotentialStrandID,currentPath),
                     tempNextStrandList = [tempNextStrandList,potentialNextStrandList(kk)];
                 end%if ~ismember(currentPotentialStrandID,currentPath),
            end%for kk = 1:size(potentialNextStrandList,2),
            potentialNextStrandList = tempNextStrandList;
            %-----
            
            %%deleteMe=0;disp(['potentialNextStrandList: ' , num2str(potentialNextStrandList)]);
            %%pause
            %-----
            if isempty(potentialNextStrandList),
                trackedToTargetFlag = 1; %no more viable moves along this path
            else%if isempty(potentialNextStrandList),
                searchDepthCounter = searchDepthCounter + 1;

                
                
                zCoord = [];
                %Pick the strand that end up closest to the surface (in this case,
                %smallest coordinate#3)
                for kk = 1:size(potentialNextStrandList,2);
                    tempStrandID = potentialNextStrandList(kk);
                    tempStrand = Strands(tempStrandID);
                    tempSOEFlag = ismember(currentStrandID,tempStrand.EndVertexNeighborStrands);
                    if tempSOEFlag == 0, otherEndVertex = tempStrand.EndVertexIndex; end
                    if tempSOEFlag == 1, otherEndVertex = tempStrand.StartVertexIndex; end
                    zCoord(kk) = cpMatrix(otherEndVertex,3);
                end%forkk
                [minVal,minPos] = min(zCoord);
                
                nextStrandID = potentialNextStrandList(end);
% %                 nextStrandID = potentialNextStrandList(minPos);
                
                
                nextStrand = Strands(nextStrandID);
                neighborStruct(nextStrandID).searchDepth = searchDepthCounter;
                currentPath = [currentPath,nextStrandID];
                startOrEndFlag = ismember(currentStrandID,nextStrand.EndVertexNeighborStrands);
                %%deleteMe=0;disp(['searchDepthCounter : ' , num2str(searchDepthCounter)]);
                %%deleteMe=0;disp(['nextStrandID : ' , num2str(nextStrandID)]);
            end%if-else isempty(potentialNextStrandList),
        end%if trackedToTargetFlag == 0 && searchDepthCounter < tempDepthToTarget
        %-----
        if trackedToTargetFlag == 1 || searchDepthCounter >= tempDepthToTarget || searchDepthCounter > maxSearchDepth,
            %either we've reached a target, or we've gone farther than some other
            %path to a target,(and since we're looking for the shortest path - we stop)
            searchDepthCounter = searchDepthCounter - 1;
            if searchDepthCounter == 0; %we're back at the startingStrand
                %%deleteMe=0;disp(['returned to starting point']);
                if size(startAttachedStrands,2)>1,
                    startAttachedStrands = startAttachedStrands(2:end);
                    nextStrandID = startAttachedStrands(1);
                    nextStrand = Strands(nextStrandID);
                    currentPath = [nextStrandID];
                    searchDepthCounter = 1;
                    startOrEndFlag = ismember(currentStrandID,nextStrand.EndVertexNeighborStrands);
                else%if size(startAttachedStrands,2)>1,
                    %%deleteMe=0;disp(['strandCompleted']);
                    strandCompletedFlag = 1;
                    searchDepthCounter = 1;
                end%if size(startAttachedStrands,2)>1,
                
               
            else%if searchDepthCounter == 0;
                neighborStruct(nextStrandID).completedFlag = 1;
                currentPath = currentPath(1:end-1);
                nextStrandID = currentPath(end);
                startOrEndFlag = neighborStruct(nextStrandID).startOrEndFlag;
            end%if-else searchDepthCounter == 0;
        end%if trackedToTargetFlag == 0 || searchDepthCounter >= tempDepthToTarget,
        neighborStruct(currentStrandID).completedFlag = 1;
        currentStrandID = nextStrandID;
        

        
% %         if mod(tempCounter,100)==0,
% %             disp(searchDepthCounter);
% %         end%if mod(tempCounter,100)==0,
        
    end%while strandCompletedFlag == 0,
    %----
    %Record the shortest path to target found for this strand
    strandTracker(startStrandIter).distanceToTarget = tempDepthToTarget;
    strandTracker(startStrandIter).pathToTarget = tempBestPath;
    strandTracker(startStrandIter).trackedLabel = tempBestLabel;
    %-----
    %Go along the path and associate all the strands with the target
    for kk = 1:size(tempBestPath,2)-1,
        tempStrandID = tempBestPath(kk);
        strandTracker(tempStrandID).distanceToTarget = tempDepthToTarget-kk;
        strandTracker(tempStrandID).trackedLabel = tempBestLabel;
        strandTracker(tempStrandID).pathToTarget = tempBestPath(kk+1:end);
    end%for kk = 1:size(tempBestPath,2)-1,
    %%deleteMe=0;disp(['completedStrand : ']);
    %%deleteMe=0;disp(strandTracker(startStrandIter));
    %%pause
    disp([startStrandIter,strandTracker(startStrandIter).distanceToTarget]);
    toc
    if strandTracker(startStrandIter).distanceToTarget == Inf,
        %%pause
    end
end%for startStrandIter = 1:numStrands,







