function outputParameters = VidaCenterlineToNodeSegList2d(inputParameters)
centerlineRadiusStructure = inputParameters.centerlineRadiusStructure;
%-----
pixelList = centerlineRadiusStructure.pixelList;
maskSize = centerlineRadiusStructure.maskSize;
radiusList = centerlineRadiusStructure.radiusList;
labelList = centerlineRadiusStructure.labelList;
confidenceList = centerlineRadiusStructure.confidenceList;
% % noteList = centerlineRadiusStructure.noteList;
%-----
%These delta are used to generate a 3x3x3 neighborhood on-the-fly
%using the centerpoint list and the reverse lookup table.
delta1 = [-1 +0 +1 -1 +0 +1 -1 +0 +1 -1 +0 +1 -1 +0 +1 -1 +0 +1 -1 +0 +1 -1 +0 +1 -1 +0 +1];
delta2 = [-1 -1 -1 +0 +0 +0 +1 +1 +1 -1 -1 -1 +0 +0 +0 +1 +1 +1 -1 -1 -1 +0 +0 +0 +1 +1 +1];
delta3 = [-1 -1 -1 -1 -1 -1 -1 -1 -1 +0 +0 +0 +0 +0 +0 +0 +0 +0 +1 +1 +1 +1 +1 +1 +1 +1 +1]; 
%----
%Remove duplicates:
[pixelList,ui,uj] = unique(double(pixelList));%note: default setting of unique returns LAST occurence of the unique value
radiusList = radiusList(ui);
labelList = labelList(ui);
confidenceList = confidenceList(ui);
% % noteList = noteList(ui);
%----
%Blank out all voxels at the edge of the block so that 3x3x3 neighborhoods
%do not go outside the block
disp(['Blanking Edge Voxels : ',datestr(now)]);tic
numCenterlinePoints = size(pixelList,1);
croppedFlagList = false(numCenterlinePoints,1);
[c1List,c2List,c3List] = ind2sub(maskSize,double(pixelList));
for cpIter = 1:size(pixelList,1);
    c1 = c1List(cpIter);
    c2 = c2List(cpIter);
    c3 = c3List(cpIter);
    if c1>1 && c2>1 && c3>1 && c1<maskSize(1) && c2<maskSize(2) && c3<maskSize(3),
        croppedFlagList(cpIter) = 1;
    end
end


pixelList = pixelList(croppedFlagList,:);
radiusList = radiusList(croppedFlagList);
labelList = labelList(croppedFlagList);
confidenceList = confidenceList(croppedFlagList);
% % noteList = noteList(croppedFlagList);
numCenterlinePoints = size(pixelList,1);
clear croppedCenterlinePointIndexList croppedRadiusList
fprintf('   Completed. ');toc
%----
%cpList structure:
%   [positionIndex, numNeighbors, Segment#(+)Node#(-)orUnused(0), neighborsProcessed]
cpList = zeros(numCenterlinePoints,5);
cpList(:,1) = pixelList;
%-----
%Create a reverse look-up table in a sparse matrix
disp(['Creating Reverse Lookup Table : ',datestr(now)]);tic
linearMaskSize = prod(maskSize);
tic
iSP = cpList(:,1);
jSP = ones(numCenterlinePoints,1);
sSP = [1:numCenterlinePoints];
mSP = linearMaskSize;
nSP = 1;
reverseLUT = sparse(iSP,jSP,sSP,mSP,nSP);
fprintf('   Completed. ');toc
%-----
disp(['Identifying neighbors : ',datestr(now)]);tic
%Calculate the number of neighbors for each centerline point

ni = sub2ind(maskSize,delta1+2,delta2+2,delta3+2);
relativeNeighborIndexAddon = ni-ni(14);
relativeNeighborIndexAddon = relativeNeighborIndexAddon(relativeNeighborIndexAddon~=0);
for pixelIter = 1:numCenterlinePoints,
    neighborhoodIndices = cpList(pixelIter,1)+relativeNeighborIndexAddon;
%     neighborhoodValues = reverseLUT(neighborhoodIndices)>0;
%     numNeighbors = sum(neighborhoodValues(:)) - 1;
    numNeighbors = sum(reverseLUT(neighborhoodIndices)>0);

    cpList(pixelIter,2) = numNeighbors;
end%for pixelIter = 1:numCenterlinePoints,


% % % [cp1List,cp2List,cp3List] = ind2sub(maskSize,cpList(:,1));
% % % for pixelIter = 1:numCenterlinePoints,
% % %     cp1 = cp1List(pixelIter);
% % %     cp2 = cp2List(pixelIter);
% % %     cp3 = cp3List(pixelIter);
% % %     np1 = delta1+cp1;
% % %     np2 = delta2+cp2;
% % %     np3 = delta3+cp3;
% % %     neighborhoodIndices = sub2ind(maskSize,np1,np2,np3);
% % %     neighborhoodValues = reverseLUT(neighborhoodIndices)>0;
% % %     numNeighbors = sum(neighborhoodValues(:)) - 1;
% % %     cpList(pixelIter,2) = numNeighbors;
% % % end
%-----
%find all endpoint and nodes
endPointList = find(cpList(:,2)==1);
nodePointList = find(cpList(:,2)>2);
numEndPoints = size(endPointList,1);
numNodePoints = size(nodePointList,1);
previousPointer = 0;
debugCounter = 0;
fprintf('   Completed. ');toc
%-----
disp(['Strandifying : ',datestr(now)]);tic
%initialize counters and parameters
segmentCounter = 0;
nodeCounter = 0;
growingSegment = zeros(1,1000);
growInd = 1;
segmentList = cell(1,numCenterlinePoints);
nodeList = cell(1,numCenterlinePoints);
continueTrackingFlag = 1; 
stillPointsLeftFlag = 1;
charCount=0;
lowEndPointMarker = 1;
lowNodePointMarker = 1;
t1 = now;
rem1 = sum((cpList(:,3)==0));
%-----
while stillPointsLeftFlag == 1,
    %Find a starting point
    startPointPointer = 0;
    nextNeighborPointer = 0;
    previousPointer = 0;
    %First look for any remaining unprocessed endpoints
    for epIter = lowEndPointMarker:numEndPoints,
        endPointPointer = endPointList(epIter);
        if cpList(endPointPointer,3)==0, %Unused point
            %found an unprocessed endPoint
            lowEndPointMarker = epIter;
            startPointPointer = endPointPointer;
            segmentCounter = segmentCounter +1;
            growingSegment = zeros(1,1000);%[startPointPointer];
            growInd = 1;
            cpList(startPointPointer,3) = segmentCounter;
            break%break out of epIter loop
        end% if cpList(endPointPointer,3)==0,
    end%for epIter = 1:numEndPoints,
    %----
    if startPointPointer == 0,
       %Found no more endpoints, look for any not-fully-processed nodes
       for nodeIter = lowNodePointMarker:numNodePoints,
           nodePointPointer = nodePointList(nodeIter);
           if cpList(nodePointPointer,4) < cpList(nodePointPointer,2), %neighborsProcessed<numNeighbors
               %found a good node to start from
               nodeID = cpList(nodePointPointer,3) * -1;
               lowNodePointMarker = nodeIter;
               if nodeID == 0,
                   %this is a new node.
                   startPointPointer = nodePointPointer;
                   segmentCounter = segmentCounter + 1;
                   growingSegment = zeros(1,1000);%[startPointPointer];
                   growInd = 1;
                   nodeCounter = nodeCounter + 1;
                   nodeList{nodeCounter} = [];
                   cpList(startPointPointer,3) = -1 * nodeCounter;
               else%if nodeID == 0,
                   %this is an established node.
                   startPointPointer = nodePointPointer;
                   segmentCounter = segmentCounter + 1;
                   growingSegment = zeros(1,1000);%[startPointPointer];
                   growInd = 1;
               end%if-else nodeID == 0,
               break%break out of for nodeIter loop
           end%if cpList(nodePointPointer,4) < cpList(nodePointPointer,2),
       end%for nodeIter = 1:numNodePoints,
    end%if startPointPointer == 0,
    %-----
    if startPointPointer == 0,
        %nothing left to process.
        stillPointsLeftFlag = 0;
    end%if startPointPointer == 0,
    %-----
    if stillPointsLeftFlag,
        continueTrackingFlag = 1;
        endOfSegmentReachedFlag = 0;
        endOfTrackAndSegmentReachedFlag = 0;
        currentListPointer = startPointPointer;
        %-----
        if mod(segmentCounter,100)==0,
            numStillEmpty = sum((cpList(:,3)==0));
            t2 = now;
            dt = (t2-t1)*24*60*60;
            t1 = t2;
            dRem = rem1 - numStillEmpty;
            rem1 = numStillEmpty;
            remainingTime = dt * numStillEmpty/(dRem);
            fprintf(repmat('\b',[1,charCount]));
            pause(0.001);
            charCount = fprintf(['Points Left: ',num2str(numStillEmpty),'. Estimated time: ',num2str(remainingTime),' sec']);
        end%if mod(numStillEmpty,10)==0,
        %-----
        while continueTrackingFlag == 1,
            %add the current point to the growingSegment list
            growingSegment(growInd) = currentListPointer;
            growInd = growInd+1;
            
            %What type of point are we on?
            if cpList(currentListPointer,2) == 0, currentPointType = 'dotPoint';end
            if cpList(currentListPointer,2) == 1, currentPointType = 'endPoint';end
            if cpList(currentListPointer,2) == 2, currentPointType = 'segPoint';end
            if cpList(currentListPointer,2) > 2, currentPointType = 'nodePoint';end
            %FIRST FIND A VALID CONNECTION
            %Grab a 3x3x3 neighborhood, and determine neighbors.


            %Search neighbors for unfilled points
            currentPixelIndex = cpList(currentListPointer,1);
            neighborIndexList = relativeNeighborIndexAddon + currentPixelIndex;
            neighborPointerList = full(reverseLUT(neighborIndexList));
            previousPointerFlagList = neighborPointerList==previousPointer;%disallow backtracking
            validNeighborFlagList = and(neighborPointerList>0 , ~previousPointerFlagList);
            validNeighborIndexList = neighborIndexList(validNeighborFlagList);
            validNeighborPointerList = neighborPointerList(validNeighborFlagList);
%             numNeighbors = numel(validNeighborIndexList);
            neighborValList = zeros(size(neighborIndexList));
            neighborUnfilledList = zeros(size(neighborIndexList));
%             neighborValList(validNeighborFlagList) = cpList(validNeighborPointerList,2);%0/1 = endpoint, 2=seg, 3+ = node.
            neighborUnfilledList(validNeighborFlagList) =(cpList(validNeighborPointerList,2) - cpList(validNeighborPointerList,4)) > 0;% how many of this neighbor's attachments are yet-unfilled 
            neighborUnfilledList = (neighborUnfilledList(validNeighborFlagList))';

            %---
            if sum(neighborUnfilledList(:))==0, %no unfilled neighbors. terminate track.
                endOfTrackAndSegmentReachedFlag = 1;
            else%if sum(neighborUnfilledList(:))==0,
                %Pick the first viable neighbor
                %Check that no connection has already been made
                findUnfilled = find(neighborUnfilledList>0);
                numUnfilled = size(findUnfilled,1);
                segmentList{segmentCounter} = growingSegment(growingSegment>0);
                for unfilledIter = 1:numUnfilled,
                    alreadyLinkedFlag = 0;
                    testNeighborPointer = validNeighborPointerList(findUnfilled(unfilledIter));
                    %Check that a connection to this point doesn't already exit.
                    testPointID = cpList(testNeighborPointer,3);
                    if testPointID == 0,
                        %this neighbor point is un-initiated - no connections.
                        nextNeighborPointer = testNeighborPointer;
                        break% break out of unfilledIter loop
                    end%if testPointID == 0,
                    %---
                    if testPointID > 0,
                        %this neighbor point is a segment or endpoint - add.
                        nextNeighborPointer = testNeighborPointer;
                        break% break out of unfilledIter loop
                    end%if testPointID > 0,
                    %---
                    if testPointID < 0,
                        %this neighbor point is a node.
                        %check that connection doesn't already exist
                        segmentsAttachedToNeighbor = nodeList{-1*testPointID};
                        numSegmentsAttachedToNeighbor = size(segmentsAttachedToNeighbor,2);
                        for testSegmentIter = 1:numSegmentsAttachedToNeighbor,
                            currentTestSegment = segmentsAttachedToNeighbor(testSegmentIter);
                            currentTestSegmentPoints = segmentList{currentTestSegment};
                            
                            numTestSegmentPoints = size(currentTestSegmentPoints,2);
                            q = find(currentTestSegmentPoints==testNeighborPointer);
                            q1 = max(1,q-1);
                            q2 = min(q+1,numTestSegmentPoints);
                            directlyLinkedTestSegmentPoints = currentTestSegmentPoints(q1:q2);
                            %foundLink = ismember(currentListPointer,currentTestSegmentPoints);
                            foundLink = ismember(currentListPointer,directlyLinkedTestSegmentPoints);
                            if foundLink,
                                alreadyLinkedFlag = 1;
                                break%break out of testSegmentIter Loop
                            end%if foundLink
                            alreadyLinkedFlag = 0;
                        end%for testSegmentIter = ...
                    end%if testPointID<0
                    %-----
                    if alreadyLinkedFlag == 0,
                        nextNeighborPointer = testNeighborPointer;
                        break%break out of unfilledIter loop
                    end%if alreadyLinkedFlag == 0,
                    %-----
                    if unfilledIter == numUnfilled,
                        %looked at all viable neighbors - no legitimate links
                        removeMe=1;disp('This should not occur!'); keyboard
                        endOfTrackAndSegmentReachedFlag = 1;
                    end%if unfilledIter == numUnfilled
                end%for unfilledIter = 1:numUnfilled
            end%if sum(neighborUnfilledList(:))==0, 
            %----------
            %----------
            if nextNeighborPointer == 0,
                %no valid neighbor found.  terminate track
                endOfTrackAndSegmentReachedFlag = 1;
            else%if nextNeighborPointer == 0,
                %NEXT, MARK THE CONNECTION TO THE VALID NEIGHBOR
                %mark the connection as made on BOTH points.
                cpList(currentListPointer,4) = cpList(currentListPointer,4) + 1;
                cpList(nextNeighborPointer,4) = cpList(nextNeighborPointer,4) + 1;
                %-----
                %Now take care of tagging the current point

                switch currentPointType,
                    case{'endPoint'},
                        %This should only occur for a starting point.
                        %Tag this point with the segmentID
                        cpList(currentListPointer,3) = segmentCounter;
                    case{'segPoint'}
                        %Tag this point with the segmentID
                        cpList(currentListPointer,3) = segmentCounter;
                    case{'nodePoint'}
                        %if it reached this point in the code, this node should have remaining connections.
                        %(if not, it would have broken out as an end-of-segment)
                        %it must be a starting node, or a continuing node
                        %either way, do not increment the nodeCount - that was done either at the start point.
                        %or at the last stage, when this node was the next-neighbor.
                        %so, here, simply attach the current segment to the nodeList
                        currentNodeID = -1 * cpList(currentListPointer,3);
                        nodeList{currentNodeID} = [nodeList{currentNodeID},segmentCounter];
                end%switch currentPointType
                %-----
                %Next, take care of the next neighbor point
                %First determine what type of point it is
                if cpList(nextNeighborPointer,2) == 0, nextNeighborType = 'dotPoint';end
                if cpList(nextNeighborPointer,2) == 1, nextNeighborType = 'endPoint';end
                if cpList(nextNeighborPointer,2) == 2, nextNeighborType = 'segPoint';end
                if cpList(nextNeighborPointer,2) > 2, nextNeighborType = 'nodePoint';end
                switch nextNeighborType,
                    case{'endPoint'},
                        %The track ends here.  Segment termination dealt with in track-ending code below.
                        %Tag the nextNeighbor point with the segmentID
                        cpList(nextNeighborPointer,3) = segmentCounter;
                        endOfTrackAndSegmentReachedFlag = 1;
                    case{'segPoint'}
                        %Continue onward to next cycle
                        previousPointer = currentListPointer;
                        currentListPointer = nextNeighborPointer;
                        nextNeighborPointer = 0;
                    case{'nodePoint'}
                        %Determine the remaining connections on that node
                        %(this is after the current connection is already ascribed)
                        numRemainingConnectionsOnNeighborNode = cpList(nextNeighborPointer,2) - cpList(nextNeighborPointer,4);
                        if numRemainingConnectionsOnNeighborNode == 0,
                            %this node is now full. Track ends here
                            %Segment termination dealt with in track-ending code below 
                            endOfTrackAndSegmentReachedFlag = 1;
                        else%if numRemainingConnectionsOnNeighborNode == 0,
                            %this node has more neighbors.  Continue track, but end this segment.
                            %Segment termination dealt with in segment-ending code below.
                            endOfSegmentReachedFlag = 1;
                        end%if-else numRemainingConnectionsOnNeighborNode == 0,
                        %----------
                        %If the neighbor node is full, add the current segment to its nodeList now 
                        %because the code will break out of the tracking loop after this.
                        nodeID = cpList(nextNeighborPointer,3) * -1;
                        if numRemainingConnectionsOnNeighborNode == 0,
                            %the neighbor node is now full (and must therefore also be established)
                            nodeList{nodeID} = [nodeList{nodeID},segmentCounter];
                        else%if numRemainingConnectionsOnNeighborNode == 0,
                            %the neighbor node is still not full.  Is it new or established?
                            if nodeID == 0,
                                %node is new
                                %increment the node count here.
                                %the node count is only incremented 2 places - at a startpoint node
                                %and here when the node is a next-neighbor; not when it is the currentPoint.
                                %initiate this nodeList with the current segment. 
                                nodeCounter = nodeCounter + 1;
                                cpList(nextNeighborPointer,3) = -1*nodeCounter;
                                nodeList{nodeCounter} = [segmentCounter];
                            else%if nodeID == 0,
                                %node is established
                                %add the current segment to this node
                                nodeList{nodeID} = [nodeList{nodeID},segmentCounter];
                            end%if-else nodeID == 0,
                        end%if-else numRemainingConnectionsOnNeighborNode == 0,
                end%switch-case nextNeighborType
            end%if-else nextNeighborPointer == 0,
            %-----
            if endOfSegmentReachedFlag==1
                %Our next step is to a node (end of segment)but we can still track onwards,
                %   add the neighbor node to the growingSegment, and send it to the segmentList.
                %   increment the segmentCounter
                %   initiate a new growingSegment with the neighborPoint
                growingSegment(growInd) = nextNeighborPointer;
                segmentList{segmentCounter} = growingSegment(growingSegment>0);
                segmentCounter = segmentCounter+1;
                growingSegment = zeros(1,1000);
                growInd = 1;
                previousPointer = currentListPointer;
                currentListPointer = nextNeighborPointer;
                nextNeighborPointer = 0;
                endOfSegmentReachedFlag = 0;
            end%if endOfSegmentReachedFlag==1 && noValidNeighborFlag==0,
            %-----
            if endOfTrackAndSegmentReachedFlag==1,
                %Our next step is to an termination of the current track.
                %   add the neighbor node to the growingSegment, and send it to the segmentList.
                %   do NOT increment the segmentCounter
                %       (the search for a new start point will incrementit)
                %   initiate a new growingSegment as empty.
                %       (the search for a new start point with seed it)
                growingSegment(growInd) = nextNeighborPointer;
                segmentList{segmentCounter} = growingSegment(growingSegment>0);
                growingSegment = zeros(1,1000);
                growInd = 1;
                endOfTrackAndSegmentReachedFlag = 0;
                continueTrackingFlag = 0;
            end%if endOfTrackAndSegmentReachedFlag==1,
        end%while continueTrackingFlag == 1,
    end%if stillPointsLeftFlag,
end%while stillPointsLeftFlag==1,
fprintf(repmat('\b',[1,charCount]));
% fprintf('\n\n');  
fprintf('   Completed. ');toc
%-------
nodeList = nodeList(1:nodeCounter);
segmentList = segmentList(1:segmentCounter);
nodeSegStructure.cpList = cpList;
nodeSegStructure.radiusList = radiusList;
nodeSegStructure.labelList = labelList;
nodeSegStructure.confidenceList = confidenceList;
% % nodeSegStructure.noteList = noteList;
nodeSegStructure.segmentList = segmentList;
nodeSegStructure.nodeList = nodeList;
nodeSegStructure.maskSize = maskSize;

%---
outputParameters = inputParameters;
outputParameters.nodeSegStructure = nodeSegStructure;



