function outputParameters = VidaCleanUpNodeSegList2(inputParameters)
nodeSegStructure = inputParameters.nodeSegStructure;
    cpList = nodeSegStructure.cpList;
    radiusList = nodeSegStructure.radiusList;
    labelList = nodeSegStructure.labelList;
    confidenceList = nodeSegStructure.confidenceList;
% %     noteList = nodeSegStructure.noteList;
    segmentList = nodeSegStructure.segmentList;
    nodeList = nodeSegStructure.nodeList;
    maskSize = nodeSegStructure.maskSize;


%-----
%Clean up lists by removing empty entries
%Strategy:
%   In cpList: find all IDTag = 0 rows, and remove, generating a lookup
%       table oldCPPointer->newCPPointer because the segmentList makes
%       reference to these pointers
%   In segmentList : remove all empty segments
%       and change all oldCPPointers->newCPPointers in non-blank segmentLists.
%       but keep track of oldSegmentPointer->newSegmentPointer lookup
%       because the nodeList and cpList makes references to these pointers
%   In nodeList : remove all empty nodes, but keep track in lookup table
%       of oldNodePointer->newNodePointer because the cpList refers to these.
%       Go through all nodeLists, and change oldSegmentPointer->newSegmentPointers
%   In cpList : replace all oldSegmentPointers->newSegmentPointers
%       and replace all oldNodePointers->newNodePointers;
disp(['Cleaning up NodeSegList : ',datestr(now)]);tic
startCPListSize = size(cpList,1);
startSegmentListSize = size(segmentList,2);
startNodeListSize = size(nodeList,2);
%---
cpListOldToNewLookup = zeros(1,startCPListSize);
segmentListOldToNewLookup = zeros(1,startCPListSize);
OldToNewLookup = zeros(1,startCPListSize);
%---
finalCPListSize = sum(cpList(:,3)~=0);
cpDim2 = size(cpList,2);
newCPList = zeros(finalCPListSize,cpDim2);
newRadiusList = zeros(finalCPListSize,1);
newLabelList = zeros(finalCPListSize,1) + 111;
newConfidenceList = zeros(finalCPListSize,1) + Inf;
newNoteList = cell(finalCPListSize,1);

newCPCounter = 0;
for cpIter = 1:startCPListSize,
    if cpList(cpIter,3) ~= 0,
        newCPCounter = newCPCounter+1;
        cpListOldToNewLookup(cpIter) = newCPCounter;
        newCPList(newCPCounter,:) = cpList(cpIter,:);
        newRadiusList(newCPCounter) = radiusList(cpIter);
        newLabelList(newCPCounter) = labelList(cpIter);
        newConfidenceList(newCPCounter) = confidenceList(cpIter);
%         newNoteList{newCPCounter} = noteList{cpIter};
    else%if cpList(cpIter,3) ~= 0,
        %empty row - skip
        cpListOldToNewLookup(cpIter) = 0;
    end%if-else cpList(cpIter,3) ~= 0,
end%for cpIter = 1:startCPListSize,
%---
newSegmentList = cell(1,startSegmentListSize);
segmentListOldToNewLookup = zeros(1,startSegmentListSize);
newSegmentCounter = 0;
for segmentIter = 1:startSegmentListSize,
    currentSegment = segmentList{segmentIter};
    if ~isempty(currentSegment),
        newSegmentCounter = newSegmentCounter+1;
        segmentListOldToNewLookup(segmentIter) = newSegmentCounter;
        modifiedSegment = cpListOldToNewLookup(currentSegment);
        
if ismember(0,modifiedSegment);keyboard;end
        
        newSegmentList{newSegmentCounter} = modifiedSegment;
    else%if ~isempty(currentSegment),
        %emptySegment - skip
        segmentListOldToNewLookup(segmentIter) = 0;
    end%if-else ~isempty(currentSegment),
end%for segmentIter = 1:startSegmentListSize,
newSegmentList = newSegmentList(1:newSegmentCounter);
%---


newNodeList = cell(1,startNodeListSize);
newNodeCounter = 0;
nodeListOldToNewLookup = zeros(1,startNodeListSize);
for nodeIter = 1:startNodeListSize,
    currentNode = nodeList{nodeIter};
    if ~isempty(currentNode),
        newNodeCounter = newNodeCounter+1;
        nodeListOldToNewLookup(nodeIter) = newNodeCounter;
        modifiedNode = segmentListOldToNewLookup(currentNode);
        newNodeList{newNodeCounter} = modifiedNode;
    else%if ~isempty(currentNode),
        %empty node - skip
        nodeListOldToNewLookup(nodeIter) = 0;
    end%if-else ~isempty(currentNode),
end%for nodeIter = 1:startNodeListSize,
newNodeList = newNodeList(1:newNodeCounter);
%---
%replace all segmentPointers and nodePointers in newCPList
for cpIter = 1:size(newCPList,1),
    currentTag = newCPList(cpIter,3);
    if currentTag>0,
        %tag is a segment
        modifiedTag = segmentListOldToNewLookup(currentTag);
    else%if currentTag>0,
        %tag is a node
        modifiedTag = -1 * nodeListOldToNewLookup(-1*currentTag);
    end%if currentTag>0
    newCPList(cpIter,3) = modifiedTag;
end%for cpIter = 1:size(newCPList,1),
%-----
%-----
cpList = newCPList;
radiusList = newRadiusList;
labelList = newLabelList;
segmentList = newSegmentList;
nodeList = newNodeList;
fprintf('   Completed. ');toc
%-----
nodeSegStructure.cpList = newCPList;
nodeSegStructure.radiusList = newRadiusList;
nodeSegStructure.labelList = newLabelList;
nodeSegStructure.confidenceList = newConfidenceList;
% % nodeSegStructure.noteList = newNoteList;
nodeSegStructure.segmentList = newSegmentList;
nodeSegStructure.nodeList = newNodeList;
%---
outputParameters = inputParameters;
outputParameters.nodeSegStructure = nodeSegStructure;

