function outputParameters = VidaRemoveFalseNodes2(inputParameters)
nodeSegStructure = inputParameters.nodeSegStructure;
    cpList = nodeSegStructure.cpList;
    segmentList = nodeSegStructure.segmentList;
    nodeList = nodeSegStructure.nodeList;
%-----
%Clean up "accidental" nodes with only 2 neighbors
%This can happen when a node cluster collapses
%Strategy:  find all nodes with 2 neighbors
%   Pick the first strand - make sure its segmentList runs UP TO the node
%   Take the second strand - make sure its segmentList runs FROM the node
%   Concatenate the second to the first, and store it in the first.
%   Find the OTHER node attached to the second strand
%   Change the entry in nodeList from the second strand to the first strand
%   Change all the cpList points on the second strand to have the value
%       of the first strand - including the old node point
%   Clear the node from the nodeList
%   Clear the second strand from the segmentList

disp(['Removing false nodes : ',datestr(now)]);tic
falseNodePointers = find(and((cpList(:,2) == 2),(cpList(:,3)<0)));
numFalseNodes = size(falseNodePointers,1);
for falseNodeIter = 1:numFalseNodes,
    currentFalseNodeCPListPointer = falseNodePointers(falseNodeIter);
    currentFalseNodeTag = cpList(currentFalseNodeCPListPointer,3) * -1;
    attachedSegments = nodeList{currentFalseNodeTag};

    
    
if numel(attachedSegments) == 0, keyboard; end
    


    
    segment1Pointer = attachedSegments(1);
    segment2Pointer = attachedSegments(2);
    segment1Members = segmentList{segment1Pointer};
    segment2Members = segmentList{segment2Pointer};
    %---
    if segment1Members(end) ~= currentFalseNodeCPListPointer,
        segment1Members = fliplr(segment1Members);
        %doublecheck:
        if segment1Members(end)~=currentFalseNodeCPListPointer,disp('err'),keyboard;end
    end%if segment1Members(end) ~= currentFalseNodeCPListPointer,
    %---
    if segment2Members(1) ~= currentFalseNodeCPListPointer,
        segment2Members = fliplr(segment2Members);
        %doublecheck:
        if segment2Members(1)~=currentFalseNodeCPListPointer,disp('err'),keyboard;end
    end%if segment1Members(end) ~= currentFalseNodeCPListPointer,
    %-----
    combinedSegmentMembers = [segment1Members,segment2Members(2:end)];
    segment2OtherNodeCPListPointer = combinedSegmentMembers(end);
    segment2OtherNodeTag = cpList(segment2OtherNodeCPListPointer,3);
    if segment2OtherNodeTag < 0,
        %this is a node - update its connection
        segment2OtherNodeStrandList = nodeList{segment2OtherNodeTag * -1};
        segment2OtherNodeStrandList(segment2OtherNodeStrandList==segment2Pointer) = segment1Pointer;
        nodeList{segment2OtherNodeTag*-1} = segment2OtherNodeStrandList;
    else%if segment2OtherNodeTag < 0,
        %this is an endpoint - update
        cpList(segment2OtherNodeCPListPointer,3) = segment1Pointer;
    end%if-else segment2OtherNodeTag < 0,
    combinedInteriorSegmentMembers = combinedSegmentMembers(2:end-1);
    cpList(combinedInteriorSegmentMembers,3) = segment1Pointer;
    %-----
    nodeList{currentFalseNodeTag} = [];
    segmentList{segment2Pointer} = [];
    segmentList{segment1Pointer} = combinedSegmentMembers;
end
%-----

disp(['   Completed removal of ',num2str(numFalseNodes),' false nodes.']);toc
fprintf('   Completed. ');toc
%-----
nodeSegStructure.cpList = cpList;
nodeSegStructure.segmentList = segmentList;
nodeSegStructure.nodeList = nodeList;
%---
outputParameters = inputParameters;
outputParameters.nodeSegStructure = nodeSegStructure;
