function outputParameters = VidaRemoveFalseStrands2(inputParameters)
nodeSegStructure = inputParameters.nodeSegStructure;
    cpList = nodeSegStructure.cpList;
    radiusList = nodeSegStructure.radiusList;
    labelList = nodeSegStructure.labelList;
    confidenceList = nodeSegStructure.confidenceList;
% %     noteList = nodeSegStructure.noteList;
    segmentList = nodeSegStructure.segmentList;
    nodeList = nodeSegStructure.nodeList;
    maskSize = nodeSegStructure.maskSize;
maxHairLengthToEvaluateForRemovalInMicrons = inputParameters.maxHairLengthToEvaluateForRemovalInMicrons;
maxConnectingStrandLengthToEvaluateForRemovalInMicrons = inputParameters.maxConnectingStrandLengthToEvaluateForRemovalInMicrons;

if maxHairLengthToEvaluateForRemovalInMicrons~=0 && maxConnectingStrandLengthToEvaluateForRemovalInMicrons~=0,
    minSegmentLength = inputParameters.minSegmentLength;
    registeredVascGrayscaleFileName = inputParameters.registeredVascGrayscaleFileName;
    intensityRatioThresholdForHairRemoval = inputParameters.intensityRatioThresholdForHairRemoval;
    intensityRatioThresholdForConnectingStrandRemoval = inputParameters.intensityRatioThresholdForConnectingStrandRemoval;
    voxelSizeInMicrons = inputParameters.voxelSizeInMicrons;
    maxHairLengthToEvaluateForRemoval = maxHairLengthToEvaluateForRemovalInMicrons ./ voxelSizeInMicrons;
    maxConnectingStrandLengthToEvaluateForRemoval = maxConnectingStrandLengthToEvaluateForRemovalInMicrons ./ voxelSizeInMicrons;


    load(registeredVascGrayscaleFileName);
    if ~exist('registeredVascGrayscale');
        if exist('imageStack'),
            registeredVascGrayscale = imageStack;
            clear imageStack
        end%if exist('imageStack'),
        %---
        if exist('vascBlock'),
            registeredVascGrayscale = vascBlock;
            clear vascBlock
        end%if exist('vascBlock'),
    end%if ~exist('registeredVascGrayscale');




    %First, look at the hairs - remove if too dim
    numSegments = size(segmentList,2);
    segLength = zeros(1,numSegments);
    hairRemovalCounter = 0;
    for segIter = 1:numSegments,
        thisSeg = segmentList{segIter};
        segLength(segIter) = size(thisSeg,2);
        if segLength(segIter)>0 && segLength(segIter)<maxHairLengthToEvaluateForRemoval,
            firstPoint = thisSeg(1);
            lastPoint = thisSeg(end);
            firstPointEndPointFlag = cpList(firstPoint,2)== 1;
            lastPointEndPointFlag = cpList(lastPoint,2)== 1;
            hairFlag = (firstPointEndPointFlag + lastPointEndPointFlag) == 1;
            if hairFlag == 1,

                otherPoints = [];
                if cpList(firstPoint,3)<0,
                    attachedSegList = nodeList{-1*cpList(firstPoint,3)};
                    otherSegList = attachedSegList(attachedSegList~=segIter);
                    for otherIter = 1:size(otherSegList,2),
                        newPoints = cpList(segmentList{otherSegList(otherIter)},1);
                        otherPoints = cat(1,otherPoints,newPoints);
                    end%for otherIter = 1:size(otherSegList,2),
                end%if cpList(firstPoint,3)<0,
                %---
                if cpList(lastPoint,3)<0,
                    attachedSegList = nodeList{-1*cpList(lastPoint,3)};
                    otherSegList = attachedSegList(attachedSegList~=segIter);
                    for otherIter = 1:size(otherSegList,2),
                        newPoints = cpList(segmentList{otherSegList(otherIter)},1);
                        otherPoints = cat(1,otherPoints,newPoints);
                    end%for otherIter = 1:size(otherSegList,2),
                end%if cpList(firstPoint,3)<0,
                thisSegPoints = cpList(thisSeg,1);
                thisSegBrightnessList = registeredVascGrayscale(thisSegPoints);
                thisMeanBrightness = mean(thisSegBrightnessList);
                otherBrightnessList = registeredVascGrayscale(otherPoints);
                otherMeanBrightness = mean(otherBrightnessList);
                brightnessRatio = thisMeanBrightness/otherMeanBrightness;
                if brightnessRatio <intensityRatioThresholdForHairRemoval,
                    %REMOVE THE HAIR
                    if firstPointEndPointFlag,
                        nonNodeList = thisSeg(1:end-1);
                        nodeCPEntry = lastPoint;
                        attachedNode = -1*cpList(lastPoint,3);
                        hairRemovalCounter = hairRemovalCounter+1;
                    else%if firstPointEndPointFlag,
                        nonNodeList = thisSeg(2:end);
                        nodeCPEntry = firstPoint;
                        attachedNode = -1*cpList(firstPoint,3);
                    end%if-else firstPointEndPointFlag,
                    cpList(nonNodeList,3) = 0;
                    segmentList{segIter} = [];
                    oldNodeListEntry = nodeList{attachedNode};
                    newNodeListEntry = oldNodeListEntry(oldNodeListEntry~=segIter);
                    nodeList{attachedNode} = newNodeListEntry;
                    cpList(nodeCPEntry,2) = cpList(nodeCPEntry,2) - 1;
                    cpList(nodeCPEntry,4) = cpList(nodeCPEntry,4) - 1;
                end%if brightnessRatio <intensityRatioThresholdForHairRemoval,
            end%if HairFlag == 1,
        end%if segLength(segIter)<maxHairLengthToEvaluateForRemoval,
    end%for segIter = 1:numSegments,
    disp([num2str(hairRemovalCounter),' hairs removed']);



    %Next, look at the connecting strands - remove if too dim
    numSegments = size(segmentList,2);
    segLength = zeros(1,numSegments);
    falseConnectionCounter = 0;
    for segIter = 1:numSegments,
        thisSeg = segmentList{segIter};
        segLength(segIter) = size(thisSeg,2);
        if segLength(segIter)>0 && segLength(segIter)<maxConnectingStrandLengthToEvaluateForRemoval,
            firstPoint = thisSeg(1);
            lastPoint = thisSeg(end);
            firstPointEndPointFlag = cpList(firstPoint,2)== 1;
            lastPointEndPointFlag = cpList(lastPoint,2)== 1;
            connectionFlag = (firstPointEndPointFlag + lastPointEndPointFlag) == 0;
            if connectionFlag == 1,
                otherPoints = [];
                if cpList(firstPoint,3)<0,
                    attachedSegList = nodeList{-1*cpList(firstPoint,3)};
                    otherSegList = attachedSegList(attachedSegList~=segIter);
                    for otherIter = 1:size(otherSegList,2),
                        newPoints = cpList(segmentList{otherSegList(otherIter)},1);
                        otherPoints = cat(1,otherPoints,newPoints);
                    end%for otherIter = 1:size(otherSegList,2),
                end%if cpList(firstPoint,3)<0,
                %---
                if cpList(lastPoint,3)<0,
                    attachedSegList = nodeList{-1*cpList(lastPoint,3)};
                    otherSegList = attachedSegList(attachedSegList~=segIter);
                    for otherIter = 1:size(otherSegList,2),
                        newPoints = cpList(segmentList{otherSegList(otherIter)},1);
                        otherPoints = cat(1,otherPoints,newPoints);
                    end%for otherIter = 1:size(otherSegList,2),
                end%if cpList(firstPoint,3)<0,
                thisSegPoints = cpList(thisSeg,1);
                thisSegBrightnessList = registeredVascGrayscale(thisSegPoints);
                thisMeanBrightness = mean(thisSegBrightnessList);
                otherBrightnessList = registeredVascGrayscale(otherPoints);
                otherMeanBrightness = mean(otherBrightnessList);
                brightnessRatio = thisMeanBrightness/otherMeanBrightness;
                if brightnessRatio <intensityRatioThresholdForConnectingStrandRemoval,
                    %REMOVE THE CONNECTION
                    nonNodeList = thisSeg(2:end-1);
                    cpList(nonNodeList,3) = 0;
                    segmentList{segIter} = [];
                    %take care of frontNode
                    attachedNode = -1*cpList(firstPoint,3);
                    oldNodeListEntry = nodeList{attachedNode};
                    newNodeListEntry = oldNodeListEntry(oldNodeListEntry~=segIter);
                    nodeList{attachedNode} = newNodeListEntry;
                    cpList(firstPoint,2) = cpList(firstPoint,2) - 1;
                    cpList(firstPoint,4) = cpList(firstPoint,4) - 1;
                    %take care of backNode
                    attachedNode = -1*cpList(lastPoint,3);
                    oldNodeListEntry = nodeList{attachedNode};
                    newNodeListEntry = oldNodeListEntry(oldNodeListEntry~=segIter);
                    nodeList{attachedNode} = newNodeListEntry;
                    cpList(lastPoint,2) = cpList(lastPoint,2) - 1;
                    cpList(lastPoint,4) = cpList(lastPoint,4) - 1;
                    falseConnectionCounter = falseConnectionCounter+1;
                end%if brightnessRatio <intensityRatioThresholdForHairRemoval,
            end%if connectionFlag == 1,
        end%if segLength(segIter)>0 && segLength(segIter)<maxConnectingStrandLengthToEvaluateForRemoval,
    end%for segIter = 1:numSegments,
    disp([num2str(falseConnectionCounter),' false connections removed']);





    nodeSegStructure.cpList = cpList;
    nodeSegStructure.segmentList = segmentList;
    nodeSegStructure.nodeList = nodeList;
end%if maxHairLengthToEvaluateForRemoval~=0 && maxConnectingStrandLengthToEvaluateForRemoval~=0,


%---
outputParameters = inputParameters;
outputParameters.nodeSegStructure = nodeSegStructure;


