function outputProject = VidaBreakupVascOnly(inputProject)

vidaProject = inputProject;
vidaProjectRootFolder = vidaProject.vidaProjectRootFolder;
globalParameters = vidaProject.globalParameters;
experimentTemplate = vidaProject.experimentTemplate;
subBlockParameters = experimentTemplate.subBlockParameters;
registrationParameters = experimentTemplate.registrationParameters;
subVolumeSize3DInMicrons = subBlockParameters.subVolumeSize3DInMicrons;
subVolumeOverlapSizeInMicrons = subBlockParameters.subVolumeOverlapSizeInMicrons;
nominalBlockStepSizeRCInMicrons = registrationParameters.nominalBlockStepSizeRCInMicrons;
nominalBlockStepSizeZInMicrons = registrationParameters.nominalBlockStepSizeZInMicrons;
voxelSizeInMicrons = globalParameters.voxelSizeInMicrons;
%-----
nominalBlockStepSizeRC = round(nominalBlockStepSizeRCInMicrons ./ voxelSizeInMicrons);
nominalBlockStepSizeZ = round(nominalBlockStepSizeZInMicrons ./ voxelSizeInMicrons);
subVolumeOverlapSize = round(subVolumeOverlapSizeInMicrons ./ voxelSizeInMicrons);
%-----
subBlockStepSize = [nominalBlockStepSizeRC,nominalBlockStepSizeRC,nominalBlockStepSizeZ];
overlapSize = [subVolumeOverlapSize,subVolumeOverlapSize,subVolumeOverlapSize];
%-----
myDir = dir(fullfile(vidaProjectRootFolder,'*.tif'));
inputBlockFileName = myDir(1).name;



blockFileRoot = inputBlockFileName(1:end-4);
bigBlock = loadmovie(inputBlockFileName);
bigBlockSize = size(bigBlock);
variableClass = class(bigBlock);

numSubBlocks = max(1,round((bigBlockSize-overlapSize)./subBlockStepSize));
subBlockStepSizePrime = ceil((bigBlockSize-overlapSize)./numSubBlocks);
%----
%force square subblocks
biggerSize = max(subBlockStepSizePrime(1:2));
subBlockStepSizePrime(1) = biggerSize;
subBlockStepSizePrime(2) = biggerSize;
%-----
subBlockSize = subBlockStepSizePrime+overlapSize;
fullPaddedSize = numSubBlocks .* subBlockStepSizePrime + overlapSize;
frontPadSize = floor((fullPaddedSize-bigBlockSize)/2);
backPadSize = (fullPaddedSize-bigBlockSize) - frontPadSize;
nonOverlapSize = subBlockStepSizePrime;

for kk = 1:3,
    if bigBlockSize(kk) < subBlockSize(kk),
        subBlockSize(kk) = bigBlockSize(kk);
        numSubBlocks(kk) = 1;
        fullPaddedSize(kk) = bigBlockSize(kk);
        frontPadSize(kk) = 0;
        backPadSize(kk) = 0;
    end%if bigBlockSize(kk) < subBlockSize(kk),
end%for kk = 1:3,

numZ = numSubBlocks(3);
numColumns = numSubBlocks(2);
numRows = numSubBlocks(1);

blockCounter = 0;
for zIter = 1:numZ
    switch zIter
    case 1
        z1 = 1;
        z2 = subBlockSize(3)-frontPadSize(3);
        z1prime = 1+frontPadSize(3);
        z2prime = subBlockSize(3);
    case numZ
        z1 = 1 + (zIter-1)*nonOverlapSize(3)-frontPadSize(3);
        z2 = bigBlockSize(3);
        z1prime = 1;
        z2prime = subBlockSize(3) - backPadSize(3);
    otherwise
        z1 = 1 + (zIter-1)*nonOverlapSize(3)-frontPadSize(3);
        z2 = z1 + subBlockSize(3) - 1;
        z1prime = 1;
        z2prime = subBlockSize(3);
    end%switch columnIter
    %-----
    for columnIter = 1:numColumns
        switch columnIter
            case 1
                c1 = 1;
                c2 = subBlockSize(2)-frontPadSize(2);
                c1prime = 1+frontPadSize(2);
                c2prime = subBlockSize(2);
            case numColumns
                c1 = 1 + (columnIter-1)*nonOverlapSize(2)-frontPadSize(2);
                c2 = bigBlockSize(2);
                c1prime = 1;
                c2prime = subBlockSize(2) - backPadSize(2);
            otherwise
                c1 = 1 + (columnIter-1)*nonOverlapSize(2)-frontPadSize(2);
                c2 = c1 + subBlockSize(2) - 1;
                c1prime = 1;
                c2prime = subBlockSize(2);
        end%switch columnIter
        %-----
        for rowIter = 1:numRows
            switch rowIter
                case 1
                    r1 = 1;
                    r2 = subBlockSize(1)-frontPadSize(1);
                    r1prime = 1+frontPadSize(1);
                    r2prime = subBlockSize(1);
                case numRows
                    r1 = 1 + (rowIter-1)*nonOverlapSize(1)-frontPadSize(1);
                    r2 = bigBlockSize(1);
                    r1prime = 1;
                    r2prime = subBlockSize(1) - backPadSize(1);
                otherwise
                    r1 = 1 + (rowIter-1)*nonOverlapSize(1)-frontPadSize(1);
                    r2 = r1 + subBlockSize(1) - 1;
                    r1prime = 1;
                    r2prime = subBlockSize(1);
            end%switch rowIter
            %-----
            blockCounter = blockCounter + 1;
            subBlock = zeros(subBlockSize,variableClass);
            subBlock(r1prime:r2prime,c1prime:c2prime,z1prime:z2prime) = bigBlock(r1:r2,c1:c2,z1:z2);
            subBlockFileName = [blockFileRoot,'-Z',num2str(zIter,'%02d'),'-C',num2str(columnIter,'%02d'),'-R',num2str(rowIter,'%02d'),'-Ch1.tif'];
            disp(['Writing SubBlock ',num2str(blockCounter),' of ',num2str(prod(numSubBlocks))]);
            maketiff(subBlock,subBlockFileName);
            
        end%for rowIter
    end%for columnIter
end%for zIter

[rootDir,fileName,ext] = fileparts(inputBlockFileName);
[scc,msg,msgID] = mkdir(vidaProjectRootFolder,'OriginalVascBlockTif');

destinationFileName = fullfile(vidaProjectRootFolder,'OriginalVascBlockTif',[fileName,ext]);
systemMoveFile(inputBlockFileName,destinationFileName);

outputProject = inputProject;
outputProject.registrationParameters.nominalBlockStepSizeRCInMicrons = subBlockStepSizePrime(1);
outputProject.registrationParameters.nominalBlockStepSizeZInMicrons = subBlockStepSizePrime(3);
outputProject.experimentTemplate.registrationParameters.nominalBlockStepSizeRCInMicrons = subBlockStepSizePrime(1);
outputProject.experimentTemplate.registrationParameters.nominalBlockStepSizeZInMicrons = subBlockStepSizePrime(3);
end
        
            
    



