function outExp = VidaSubBlock(inExp)
%last modified 05/08/08 by Phil Tsai

%-----
fileRootName = inExp.globalParameters.fileRootName;
rootDataFolder = inExp.globalParameters.rootDataFolder;
lastRerunStartDateNum = inExp.globalParameters.lastRerunStartDateNum;
processedDataFolder = inExp.globalParameters.processedDataFolder;
intermediateResultsFolder = inExp.globalParameters.intermediateResultsFolder;
numChannels = inExp.globalParameters.numChannels;
stackedFileNameList = inExp.globalParameters.stackedFileNameList;
voxelSizeInMicrons = inExp.globalParameters.voxelSizeInMicrons;
fullStackSize = inExp.globalParameters.fullStackSize;
%-----
blockVariableNames = {'cellBlock','vascBlock','neunBlock'};
%-----
subVolumeSize3DInMicrons = inExp.subBlockParameters.subVolumeSize3DInMicrons;              
subVolumeOverlapSizeInMicrons = inExp.subBlockParameters.subVolumeOverlapSizeInMicrons;    
%-----
subBlockFilesFolder = fullfile(processedDataFolder,'subBlockFiles');
[mkdirSuccess,mkdirMessage,mkdirId] = mkdir(subBlockFilesFolder);
%-----
displayFlag = 0;
%-----
subVolumeSize3D = round(subVolumeSize3DInMicrons ./ voxelSizeInMicrons);
subVolumeOverlapSize = round(subVolumeOverlapSizeInMicrons ./ voxelSizeInMicrons);




%DEFINE CHANNEL NAMES
[tempPath,tempFileName,tempExt,tempVer] = fileparts(stackedFileNameList{1});
dashPos = findstr(tempFileName,'-');
lastDash = dashPos(end);
channels.cellChannelString = tempFileName(lastDash+1:end);
[tempPath,tempFileName,tempExt,tempVer] = fileparts(stackedFileNameList{2});
dashPos = findstr(tempFileName,'-');
lastDash = dashPos(end);
channels.vascChannelString = tempFileName(lastDash+1:end);
channels.neunChannelString = '';
if numChannels >2,
    [tempPath,tempFileName,tempExt,tempVer] = fileparts(stackedFileNameList{3});
    dashPos = findstr(tempFileName,'-');
    lastDash = dashPos(end);
    channels.neunChannelString = tempFileName(lastDash+1:end);
end

%-----
% Extract subblocks from .tifs, order, and store pointers to .mat files for vasc data
disp(['Starting Cell Grid Subblocking : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
gridParameters.lastRerunStartDateNum = lastRerunStartDateNum;
gridParameters.subBlockFilesFolder = subBlockFilesFolder;
gridParameters.stackedFileName = stackedFileNameList{1};
gridParameters.blockVariableName = blockVariableNames{1};
gridParameters.maskSize = fullStackSize;
gridParameters.sz3D=subVolumeSize3D;
gridParameters.overlapsz = subVolumeOverlapSize;
gridParameters.dispon = displayFlag;
%-----
    gridParameters = VidaSegmentationGrid(gridParameters);      
%-----
blockGrid = gridParameters.blockGrid;
cellBlockFileNameList = gridParameters.blockFileNameList;
cellGrayscaleFileName = stackedFileNameList{1};
%---------
%---------
disp(['Starting Vasc Grid Subblocking : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
gridParameters.stackedFileName = stackedFileNameList{2};
gridParameters.blockVariableName = blockVariableNames{2};
%-----
    gridParameters = VidaSegmentationGrid(gridParameters);      
%-----
vascBlockFileNameList = gridParameters.blockFileNameList;
%---------
%---------
disp(['Starting Neun Grid Subblocking : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
gridParameters.stackedFileName = stackedFileNameList{3};
gridParameters.blockVariableName = blockVariableNames{3};
%-----
    gridParameters = VidaSegmentationGrid(gridParameters);      
%-----
neunBlockFileNameList = gridParameters.blockFileNameList;
%---------
%---------

outExp = inExp;
outExp.globalParameters.channels = channels;
%---
outExp.globalParameters.blockGrid = blockGrid;
outExp.globalParameters.subVolumeOverlapSize = subVolumeOverlapSize;
outExp.globalParameters.cellGrayscaleFileName = cellGrayscaleFileName;
outExp.globalParameters.cellBlockFileNameList = cellBlockFileNameList;
outExp.globalParameters.vascBlockFileNameList = vascBlockFileNameList;
outExp.globalParameters.neunBlockFileNameList = neunBlockFileNameList;
outExp.completionDates.subBlockCompletionDate = datestr(now);
%---
%---
return


