function outputParameters = VidaFractionalIntensityReconstruction(inputParameters)

lastRerunStartDateNum = inputParameters.lastRerunStartDateNum;
fileRootName = inputParameters.fileRootName;
processedDataFolder = inputParameters.processedDataFolder;
intermediateResultsFolder = inputParameters.intermediateResultsFolder;
vascRadiusFilesFolder = inputParameters.vascRadiusFilesFolder;
vascBlockFileNameList = inputParameters.vascBlockFileNameList;
vascBlockRawMaskFileNameList = inputParameters.vascBlockRawMaskFileNameList;
vascBlockCenterlineMaskFileNameList = inputParameters.vascBlockCenterlineMaskFileNameList;
%-----
minVolForConnectedCenterlineInVoxels = inputParameters.minVolForConnectedCenterlineInVoxels;      % throws away small centerlined islands
neighborhoodSizeFactor = inputParameters.neighborhoodSizeFactor;
localRadiiMultiplerForSaturation = inputParameters.localRadiiMultiplerForSaturation;
brightValueFraction = inputParameters.brightValueFraction;
firstRoundThresholdFraction = inputParameters.firstRoundThresholdFraction;
maxFractionOfSubFramePixels = inputParameters.maxFractionOfSubFramePixels;
maxVoxelsForNoiseBlock = inputParameters.maxVoxelsForNoiseBlock; 
interpolateFactor = inputParameters.interpolateFactor;
bwDistSearchWindowSizeInMicrons = inputParameters.bwDistSearchWindowSizeInMicrons;
minVesselRadiusSupportInMicrons = inputParameters.minVesselRadiusSupportInMicrons;
minVolumeAfterReconstructionInVoxels = inputParameters.minVolumeAfterReconstructionInVoxels;
voxelSizeInMicrons = inputParameters.voxelSizeInMicrons;
%-----
maxLocalRadiusInMicrons = inputParameters.maxLocalRadiusInMicrons;
closeDiameterInMicrons = inputParameters.closeDiameterInMicrons;
zSearchSizeInMicrons = inputParameters.zSearchSizeInMicrons;
minNeighborhoodSizeInCubicMicrons = inputParameters.minNeighborhoodSizeInCubicMicrons;     
%-----
histGaussianFilter = gaussian1d(7,5);
crossFilter = double([0 1 0 ; 1 1 1 ; 0 1 0]);
%-----
bwDistSearchWindowSize = round(bwDistSearchWindowSizeInMicrons ./ voxelSizeInMicrons);
minVesselRadiusSupport = minVesselRadiusSupportInMicrons ./ voxelSizeInMicrons;
maxLocalRadius = maxLocalRadiusInMicrons ./ voxelSizeInMicrons;
closeDiameter = closeDiameterInMicrons ./ voxelSizeInMicrons;
zSearchSize = round(zSearchSizeInMicrons ./ voxelSizeInMicrons);
minNeighborhoodSize = round(minNeighborhoodSizeInCubicMicrons ./ (voxelSizeInMicrons.^3));
%-----
maxFractionOfSubFramePixels = 1.5 * (1/neighborhoodSizeFactor);%noteToSelf
interpolateMultiplier = 2^(interpolateFactor);%noteToSelf
%-----
overrideFileNumber = 0;
if isfield(inputParameters,'overrideFileNumber'),
    overrideFileNumber = inputParameters.overrideFileNumber;
end
%-----
outputFilePrefix = fullfile(vascRadiusFilesFolder,'VascBlockRadiusMap');
if isfield(inputParameters,'overrideOutputFilePrefix'),
    outputFilePrefix = inputParameters.overrideOutputFilePrefix;
end
%-----               
%new results from Even-pixel PSF simulation           
secondRatio = [0.2836 0.2836 0.2836 0.3053 0.3404 0.3549 0.3662 0.3833 0.3899 0.4098 ...
             0.4118 0.4175 0.4286 0.4331 0.4339 0.4400 0.4450 0.4500 0.4550 0.4599 ...
             0.4620 0.4640 0.4660 0.4680 0.4700 0.4720 0.4740 0.4760 0.4780 0.4800, repmat(0.4800,[1,1000])];           


         
numBlocks = size(vascBlockCenterlineMaskFileNameList,2);
vascBlockRadiusMapFileNameList = cell(numBlocks);
for blockIter = 1:numBlocks,
    vascBlockFileName = vascBlockFileNameList{blockIter};
    vascBlockRawMaskFileName = vascBlockRawMaskFileNameList{blockIter};
    vascBlockCenterlineMaskFileName = vascBlockCenterlineMaskFileNameList{blockIter};
    vascBlockRadiusMapFileName = [outputFilePrefix,num2str(blockIter),'.mat'];
    %---
    if overrideFileNumber ~=0,
        vascBlockRadiusMapFileName = [outputFilePrefix,num2str(overrideFileNumber),'.mat'];
    end%if overrideFileNumber ~=0,
    %---
    disp(['Reconstructing Vasc Mask Block ',num2str(blockIter),' of ',num2str(numBlocks),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
    %-----
    vascBlockRadiusMapFileNameList{blockIter} = vascBlockRadiusMapFileName;
    %-----
    overwriteFlag = 0;
    fileExistsFlag = (exist(vascBlockRadiusMapFileName,'file') >0 );
    if fileExistsFlag,
        fileProps = dir(vascBlockRadiusMapFileName);
        fileDateNum = datenum(fileProps.date);
        if fileDateNum < lastRerunStartDateNum,
            overwriteFlag = 1;
        end%if fileDateNum < lastRerunStartDateNum,
    end%if fileExistsFlag
    %---
    if and(fileExistsFlag==1 , overwriteFlag==0),
        disp(['...previously computed on ',datestr(fileDateNum,'mm/dd/yy HH:MM:SS')]);
    else%if and(fileExistsFlag==1,overwriteFlag==0),
        %Load the original FITC block and centerline mask
        %---
        blockStruct = load(vascBlockFileName);
        blockVariables = fieldnames(blockStruct);
        vascBlock = blockStruct.(blockVariables{1});
        if isstruct(vascBlock),
            pixelList = vascBlock.pixelList;
            maskSize = vascBlock.maskSize;
            vascBlock = false(maskSize);
            vascBlock(pixelList) = 1;
        end%if isstruct(vascBlock),
        vascBlock = single(vascBlock);
        %---
        blockStruct = load(vascBlockRawMaskFileName);
        blockVariables = fieldnames(blockStruct);
        vascBlockRawMask = blockStruct.(blockVariables{1});
        if isstruct(vascBlockRawMask),
            pixelList = vascBlockRawMask.pixelList;
            maskSize = vascBlockRawMask.maskSize;
            vascBlockRawMask = false(maskSize);
            vascBlockRawMask(pixelList) = 1;
        end%if isstruct(vascBlockRawMask),
        %---
        blockStruct = load(vascBlockCenterlineMaskFileName);
        blockVariables = fieldnames(blockStruct);
        vascBlockCenterlineMask = blockStruct.(blockVariables{1});
        if isstruct(vascBlockCenterlineMask),
            pixelList = vascBlockCenterlineMask.pixelList;
            maskSize = vascBlockCenterlineMask.maskSize;
            vascBlockCenterlineMask = false(maskSize);
            vascBlockCenterlineMask(pixelList) = 1;
        end%if isstruct(vascBlockCenterlineMask),
        %---
        blockSize = size(vascBlock);
        [ySize,xSize,zSize] = size(vascBlock);
        %----------
        %Eliminate contiguous centerline regions of size  [minvol] or less
        L=bwlabeln(vascBlockCenterlineMask,26);
        stmp=regionprops(L,'Area');
        LUT=[0,[stmp.Area]>minVolForConnectedCenterlineInVoxels];
        L=LUT(L+1);
        vascBlockCenterlineMask = L>0;
        clear L
        %----------
        %Calculate 2D bwdists on the original rawmask
        vascBlockRawMask(vascBlockCenterlineMask)=1;
        bwRaw = single(vascBlockRawMask.*0);
        for frameIter = 1:zSize,
            currentFrame = vascBlockRawMask(:,:,frameIter);
            currentFrame = imfill(currentFrame,4,'holes');
            bwFrame = bwdist(~currentFrame);
            bwRaw(:,:,frameIter) = bwFrame;
        end
        radiusListRaw = bwRaw(vascBlockCenterlineMask);
        %-----
        %For every centerline Pixel, extract a 2D local subframe & Calculate 2D bwDist
        centerlinePixelList = find(vascBlockCenterlineMask);
        numCenterlinePixels = size(centerlinePixelList,1);
        radiusListCalc = zeros(numCenterlinePixels,1);
     
        disp(['Calculating Thresholds : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
        numChar = 0;
        for pixelIter = 1:numCenterlinePixels,
            fprintf(repmat('\b',[1 numChar]));
            myMessage = ['Centerline Pixel ',num2str(pixelIter),' of ',num2str(numCenterlinePixels)];
            numChar = fprintf(myMessage);
            %Isolate a subBlock and subFrame
            currentPixel = centerlinePixelList(pixelIter);
            currentRadiusRaw = radiusListRaw(pixelIter);
            currentRadiusRaw = max(currentRadiusRaw,3);
            [cY,cX,cZ] = ind2sub(blockSize,currentPixel);
            neighborhoodSizeXY = max(ceil(neighborhoodSizeFactor.*currentRadiusRaw),minNeighborhoodSize);
            neighborhoodSizeZ = ceil(neighborhoodSizeXY/2);
            neighborhoodSizeZ = zSearchSize;
            
            %-----
            %x1,x2 = limits of subBlock in the larger block
            y1 = max(cY-neighborhoodSizeXY,1);
            y2 = min(cY+neighborhoodSizeXY,ySize);
            x1 = max(cX-neighborhoodSizeXY,1);
            x2 = min(cX+neighborhoodSizeXY,xSize);
            z1 = max(cZ-neighborhoodSizeZ,1);
            z2 = min(cZ+neighborhoodSizeZ,zSize);
            %-----
            subBlock = vascBlock(y1:y2,x1:x2,z1:z2);
            subFrame = vascBlock(y1:y2,x1:x2,cZ);
            [ySubSize,xSubSize,zSubSize] = size(subBlock);
            cYPrime = (cY-y1)+1;
            cXPrime = (cX-x1)+1;
            cZPrime = (cZ-z1)+1;
            %----------
            %Saturate bright regions that are away from the centerlinePoint
            localMaxSize = ceil(currentRadiusRaw*localRadiiMultiplerForSaturation);
            localMaxSize = min(localMaxSize,maxLocalRadius);
            ly1 = max(cYPrime-localMaxSize,1);
            ly2 = min(cYPrime+localMaxSize,ySubSize);
            lx1 = max(cXPrime-localMaxSize,1);
            lx2 = min(cXPrime+localMaxSize,xSubSize);
            localRegion = subFrame(ly1:ly2,lx1:lx2);
            maxLocalVal = max(localRegion(:));
            subFrame(subFrame>maxLocalVal)=maxLocalVal;
            if max(subFrame(:))>0,
                xRange = linspace(0,maxLocalVal,100);
                h = hist(subFrame(:),xRange);
                h([1,end]) = 0;
                hFilt = h;
                hFilt(2:end-1) = imfilter(h(2:end-1),histGaussianFilter,'replicate');
                [histMax,maxPos] = max(hFilt);
                bkgLevel = xRange(maxPos(1));
                %-----
                %first pass threshold based on brightest pixel * prelimary fraction
                threshold  = firstRoundThresholdFraction*(maxLocalVal-bkgLevel) + bkgLevel;
                %-----
                %second pass threshold based on "high range" pixel * prelimary fraction
                brightRegions = subFrame(subFrame>threshold);
                numBrightPixels = length(brightRegions);
                brightPos = ceil(brightValueFraction*numBrightPixels);
                sortBright = sort(brightRegions);
                brightVal = sortBright(brightPos);
                %-----
                threshold = firstRoundThresholdFraction*(brightVal-bkgLevel) + bkgLevel;
                bwBlock = subBlock > threshold;
                %-----
                bwFrame = bwBlock(:,:,cZPrime);
                %Close the volume with a small kernel
                closedFrame = imclose(bwFrame,circle2d(closeDiameter));
                %-----
                for interpolateIter = 1:interpolateFactor,
                    %Expand (supersample) the binary image to improve radius calculation resolution
                    expandedBinFrame = imresize(closedFrame,2,'nearest');
                    %Compensate for resize distortion to bwdistances
                    closedFrame = or(expandedBinFrame,imfilter(double(expandedBinFrame),crossFilter)>1);
                end
                expandedBW2D = bwdist(~closedFrame);
                %-----
                %Look in a small bwDistSearchWindowSize around the acutal
                %centerline Pixel to pick the largest bwDist value
                %This is the first round estimate of the vessel radius
                ny1 = max(cYPrime*interpolateMultiplier-bwDistSearchWindowSize*interpolateMultiplier,1);
                ny2 = min(cYPrime*interpolateMultiplier+bwDistSearchWindowSize*interpolateMultiplier,ySubSize*interpolateMultiplier);
                nx1 = max(cXPrime*interpolateMultiplier-bwDistSearchWindowSize*interpolateMultiplier,1);
                nx2 = min(cXPrime*interpolateMultiplier+bwDistSearchWindowSize*interpolateMultiplier,xSubSize*interpolateMultiplier);
                searchRegion = expandedBW2D(ny1:ny2,nx1:nx2);
                maxSearchValue = max(searchRegion(:));
                calculatedRadius = ceil(maxSearchValue)./interpolateMultiplier;
                calculatedRadius = max(calculatedRadius,minVesselRadiusSupport);
                %----------
                %----------
                %Perform a second round estimate using the Look-up-Table based
                %on Gaussian PSF's and cylindrical vessels to determine the
                %optimal threshold fraction for this particular vessel size
                
                firstRoundCalculatedRadius = calculatedRadius;

              try
                secondRoundThreshFraction = secondRatio(round(firstRoundCalculatedRadius*2));%noteToSelf
                catch
                    if firstRoundCalculatedRadius == inf, %either full field is bright or vessel is dim
                        if mean(subFrame(:)) > mean(vascBlock(:)) + 2*std(vascBlock(:)); %all bright
                            %set radius to full subblock size
                            firstRoundCalculatedRadius = max([y2-y1,x2-x1,z2-z1])*sqrt(2);
                        else%if mean(subFrame(:)) > mean(vascBlock(:))
                            firstRoundCalculatedRadius = 0.5;
                        end%if-else mean(subFrame(:)) > mean(vascBlock(:))
                    else%if firstRoundCalculatedRadius == inf, 
                    keyboard    
                    end%if-else firstRoundCalculatedRadius == inf, %either full field is bright or vessel is dim
                    secondRoundThreshFraction = secondRatio(round(firstRoundCalculatedRadius*2));%noteToSelf
               
                end
                
                
                localRegionBlock = subBlock(ly1:ly2,lx1:lx2,:);
                maxLocalVal = max(localRegionBlock(:));
                xRange = linspace(0,maxLocalVal,100);
                h = hist(subBlock(:),xRange);
                h([1,end]) = 0;
                hFilt = h;
                hFilt(2:end-1) = imfilter(h(2:end-1),histGaussianFilter,'replicate');
                [histMax,maxPos] = max(hFilt);
                bkgLevel = xRange(maxPos(1));
                %-----
                prelimThreshold = secondRoundThreshFraction*(maxLocalVal-bkgLevel) + bkgLevel;
                brightRegions = subBlock(subBlock>prelimThreshold);
                numBrightPixels = length(brightRegions);
                   if numBrightPixels > 0,
                    brightPos = ceil(brightValueFraction*numBrightPixels);
                    sortBright = sort(brightRegions);
                    brightVal = sortBright(brightPos);
                    secondRoundThresh = secondRoundThreshFraction*(brightVal-bkgLevel) + bkgLevel;
                    bwBlock = subBlock > secondRoundThresh;
                else%if numBrightPixels > 0,
                    bwBlock(:) = 0;
                end%if numBrightPixels > 0,
 
                %-----
                %-----
                %Remove Isolated Pixels,
                cleanBlock = (imfilter(double(bwBlock),ones(3,3)).*double(bwBlock))>1;

                %Close the volume with a 3x3 square kernel
%                 closedBlock = imclose(bwBlock,circle2d(closeDiameter));
                closedBlock = imclose(cleanBlock,circle2d(closeDiameter));
                for interpolateIter = 1:interpolateFactor,
                    %Expand (supersample) the binary image to improve radius calculation resolution
                    expandedBinBlock = imresize(closedBlock,2,'nearest');
                    %Compensate for resize distortion to bwdistances
                    closedBlock = or(expandedBinBlock,imfilter(double(expandedBinBlock),crossFilter)>1);
                end
                correctedExpandedBinBlock = closedBlock;

                zMaxList = [];
                for zIter = -zSearchSize:zSearchSize
                    currentZ = min(zSubSize,max(1,(cZPrime + zIter)));
                    correctedExpandedBinFrame = correctedExpandedBinBlock(:,:,currentZ);
                    %-----
                    %-----
                    if min(correctedExpandedBinFrame(:)) == 1,
                        correctedExpandedBinFrame(:) = 0;
                    end
                    %If the centerline Pixel is in emptySpace / noise, avoid masking noise
                    %by setting a maximum allowed mask fraction for the
                    %subFrame
                    if sum(correctedExpandedBinFrame(:))/numel(correctedExpandedBinFrame) > maxFractionOfSubFramePixels,
                        subFrame = subBlock(:,:,currentZ);
                        bwFrame = bwBlock(:,:,currentZ);
                        bkgRegion = subFrame(~bwFrame);
                        maskRegion = subFrame(bwFrame);
                        meanBkg = mean(bkgRegion);
                        stdBkg = std(bkgRegion);
                        meanMask = mean(maskRegion);
                        if meanBkg + 5*stdBkg > meanMask,
                            sortPixels = sort(subFrame(:));
                            cutoff = ceil(numel(sortPixels) .*  (1-maxFractionOfSubFramePixels));
                            thresholdOverride = sortPixels(cutoff);
                            bwFrame = subFrame > thresholdOverride;
                            %Eliminate contiguous regions of size  [minvol] or less
                            L=bwlabeln(bwFrame,4);
                            stmp=regionprops(L,'Area');
                            LUT=[0,[stmp.Area]>maxVoxelsForNoiseBlock ];
                            L=LUT(L+1);
                            bwFrame = L>0;
                            closedFrame = imclose(bwFrame,circle2d(closeDiameter));
                            for interpolateIter = 1:interpolateFactor,
                                %Expand (supersample) the binary image to improve radius calculation resolution
                                expandedBinFrame = imresize(closedFrame,2,'nearest');
                                %Compensate for resize distortion to bwdistances
                                closedFrame = or(expandedBinFrame,imfilter(double(expandedBinFrame),crossFilter)>1);
                            end
                            correctedExpandedBinFrame = closedFrame;
                        end%if meanBkg+5*stdBkg > meanMask,
                    end%if sum(correctedExpandedBinFrame(:))/numel(correctedExpandedBinFrame) > maxFractionOfSubFramePixels,
                    expandedBW2D = bwdist(~correctedExpandedBinFrame);
                    %-----
                    %Look in a small bwDistSearchWindowSize around the acutal
                    %centerline Pixel to pick the largest bwDist value
                    ny1 = max(cYPrime*interpolateMultiplier-bwDistSearchWindowSize*interpolateMultiplier,1);
                    ny2 = min(cYPrime*interpolateMultiplier+bwDistSearchWindowSize*interpolateMultiplier,ySubSize*interpolateMultiplier);
                    nx1 = max(cXPrime*interpolateMultiplier-bwDistSearchWindowSize*interpolateMultiplier,1);
                    nx2 = min(cXPrime*interpolateMultiplier+bwDistSearchWindowSize*interpolateMultiplier,xSubSize*interpolateMultiplier);
                    searchRegion = expandedBW2D(ny1:ny2,nx1:nx2);
                    maxSearchValue = max(searchRegion(:));
                    calculatedRadius = maxSearchValue./interpolateMultiplier;
                    zMaxList = [zMaxList,calculatedRadius];
                end%for zIter = ...
                calculatedRadius = max(zMaxList);
                radiusListCalc(pixelIter) = calculatedRadius;
            else%if max(subFrame(:))>0,
                radiusListCalc(pixelIter) = 0;
            end%if max(subFrame(:))>0,
        end%for pixelIter = 1:numCenterlinePixels,
        radiusListCalc = max(radiusListCalc,minVesselRadiusSupport);
        %Eliminate redunancies in the list
        [centerlinePixelList,ui,uj] = unique(centerlinePixelList);
        radiusListCalc = radiusListCalc(ui);
        numCenterlinePixels = size(centerlinePixelList,1);
        %----------
        %Create a 3D block with the calculated radius placed at each centerline point
        radiusMaskCalc = (vascBlockCenterlineMask*.0);
        for pixelIter = 1:numCenterlinePixels,
            currentPixel = centerlinePixelList(pixelIter);
            currentRadiusCalc = radiusListCalc(pixelIter);
            [cY,cX,cZ] = ind2sub(blockSize,currentPixel);
            radiusMaskCalc(cY,cX,cZ) = currentRadiusCalc;
        end

        %----------
        %----------
        %RECONSTRUCT VASC MASK FROM THE CENTERLINE MASK AND BWDIST RADII
        vascBlockReconstructedMask = false(size(vascBlockCenterlineMask));
        numCenterlinePixels = size(centerlinePixelList,1);
        %Create a storage space for spheres to avoid re-creating them
        sphereCell = cell([10000,1]);
        disp(' ');
        disp(['Reconstructing Mask : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
        numChar = 0;
        for pixelIter = 1:numCenterlinePixels,
            fprintf(repmat('\b',[1 numChar]));
            myMessage = ['Centerline Pixel ',num2str(pixelIter),' of ',num2str(numCenterlinePixels)];
            numChar = fprintf(myMessage);
            currentPixel = centerlinePixelList(pixelIter);
            currentRadius = radiusListCalc(pixelIter);
            [cY,cX,cZ] = ind2sub(blockSize,currentPixel);
            sphereRadius = double(currentRadius);
            sphereRadiusIndex = int16(ceil(sphereRadius*10));
            if sphereRadius>0,
                %Fill in with a sphere
                if isempty(sphereCell{sphereRadiusIndex}),
                    sphereDiameter = sphereRadius * 2;
                    myhalf = 1+round(sphereDiameter-1)/2;
                    [x,y,z] = meshgrid(-myhalf:myhalf,-myhalf:myhalf,-myhalf:myhalf);
                    myR = sqrt(x.^2 + y.^2 +z.^2);
                    mySphere = myR<=(sphereDiameter/2);
                    if sphereDiameter <= 2,mySphere = cat(3,[1 1; 1 1],[1 1; 1 1]);end
                    if sphereDiameter <= 1,mySphere = 1; end
                    sphereCell{sphereRadiusIndex} = mySphere;
                end%if isempty(sphereCell(sphereRadiusIndex))
                mySphere = sphereCell{sphereRadiusIndex};
                cubeSize = size(mySphere,1);
                halfCubeMinus = int16(floor(cubeSize/2));
                halfCubePlus = cubeSize-halfCubeMinus-1;
                %---
                xminus = cX-halfCubeMinus;
                x1Block = max(1,xminus);
                x1Sphere = max(1,((halfCubeMinus + 2) - cX));
                xplus = cX+halfCubePlus;
                x2Block = min(xplus,xSize);
                x2Sphere = min(cubeSize,xSize+cubeSize-xplus);
                %---
                yminus = cY-halfCubeMinus;
                y1Block = max(1,yminus);
                y1Sphere = max(1,((halfCubeMinus + 2 ) - cY));
                yplus = cY+halfCubePlus;
                y2Block = min(yplus,ySize);
                y2Sphere = min(cubeSize,ySize+cubeSize-yplus);
                %---
                zminus = cZ-halfCubeMinus;
                z1Block = max(1,zminus);
                z1Sphere = max(1,((halfCubeMinus + 2) - cZ));
                zplus = cZ+halfCubePlus;
                z2Block = min(zplus,zSize);
                z2Sphere = min(cubeSize,zSize+cubeSize-zplus);
                %---
                cutSphere = mySphere(y1Sphere:y2Sphere,x1Sphere:x2Sphere,z1Sphere:z2Sphere);
                cutBlock = vascBlockReconstructedMask(y1Block:y2Block,x1Block:x2Block,z1Block:z2Block);
                unionCutBlock = or(cutSphere,cutBlock);
                vascBlockReconstructedMask(y1Block:y2Block,x1Block:x2Block,z1Block:z2Block) = unionCutBlock;
            end%if sphereRadius>0
        end%for pixelIter = 1:numCenterlinePixels,
        disp(' ');
        %----------
        %----------
        %Eliminate contiguous regions of size  [minvol] or less
        L=bwlabeln(vascBlockReconstructedMask,6);
        stmp=regionprops(L,'Area');
        LUT=[0,[stmp.Area]>minVolumeAfterReconstructionInVoxels];
        L=LUT(L+1);
        vascBlockReconstructedMask = L>0;
        %-----
        blockSize = [ySize, xSize, zSize];
        maskSize = size(vascBlockReconstructedMask);
        extraData = 'extraData variable use for Mask storage option only - not used for List Storage option';
        vascBlockRadiusMap = [];
        vascBlockRadiusMap.pixelList = centerlinePixelList;
        vascBlockRadiusMap.maskSize = maskSize;
        vascBlockRadiusMap.radiusList = radiusListCalc;
        saveSize = (getfield(whos('vascBlockRadiusMap'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(vascBlockRadiusMapFileName,'vascBlockRadiusMap','extraData',scs);

        
        %-----
    end %if-else and(fileExistsFlag==1,overwriteFlag==0),
end% for blockIter = 1:numBlocks,
%-----
outputParameters = inputParameters;
outputParameters.vascBlockRadiusMapFileNameList = vascBlockRadiusMapFileNameList;
return


