function outputParameters=VidaVascBinarizeRawVolume(inputParameters)
%Algorithm flow and logic listed at bottom of file

%-----      
fileRootName = inputParameters.fileRootName;
lastRerunStartDateNum = inputParameters.lastRerunStartDateNum;
intermediateResultsFolder = inputParameters.intermediateResultsFolder;
vascSegFilesFolder = inputParameters.vascSegFilesFolder;
%---
enhMaskPrelimThresholdFraction = inputParameters.enhMaskPrelimThresholdFraction;
numBkgStdForMinThresh = inputParameters.numBkgStdForMinThresh;
fractionOfHighIntensityForMaxThresh = inputParameters.fractionOfHighIntensityForMaxThresh;
filtMultiplierForVarThresh = inputParameters.filtMultiplierForVarThresh;
minVolForSmallVolRemoval = inputParameters.minVolForSmallVolRemovalInVoxels;
minVolForCleanup = inputParameters.minVolForCleanupInVoxels;
numCleanupCyclesInMicrons = inputParameters.numCleanupCyclesInMicrons;
gaussianRadiusForFilterInMicrons = inputParameters.gaussianRadiusForFilterInMicrons;
smallGaussianSizeFactor = inputParameters.smallGaussianSizeFactor;
maxDilationCyclesInMicrons = inputParameters.maxDilationCyclesInMicrons;
voxelSizeInMicrons = inputParameters.voxelSizeInMicrons;
%-----
numCleanupCycles = round(numCleanupCyclesInMicrons ./ voxelSizeInMicrons);
gaussianRadiusForFilter = gaussianRadiusForFilterInMicrons ./ voxelSizeInMicrons;
maxDilationCycles = round(maxDilationCyclesInMicrons ./ voxelSizeInMicrons);

%-----
displayFlag = inputParameters.displayFlag;
%---
vascBlockFileNameList=inputParameters.vascBlockFileNameList;
vascBlockEnhancedGrayscaleFileNameList = inputParameters.vascBlockEnhancedGrayscaleFileNameList;
%-----
numBlocks = length(vascBlockFileNameList);
vascBlockRawMaskFileNameList = cell(1,numBlocks);
vascBlockRawMaskPixelListFileNameList = cell(1,numBlocks);
vascBlockRawMaskBinarizeDetailsFileNameList = cell(1,numBlocks);
for i=1:numBlocks,
  [pathName,fileName,ext] = fileparts(vascBlockFileNameList{i});
  vascBlockRawMaskFileNameList{i} = fullfile(vascSegFilesFolder,[fileName,'-RawMask.mat']);
  vascBlockRawMaskPixelListFileNameList{i} = fullfile(vascSegFilesFolder,[fileName,'-RawMaskPixelList.mat']);
  vascBlockRawMaskBinarizeDetailsFileNameList{i} = fullfile(vascSegFilesFolder,[fileName,'-RawBinDetails.mat']);
end
%-----
numBlocks = length(vascBlockFileNameList);
vbthresh = cell(1,numBlocks);
rawFraction = cell(1,numBlocks);

%-----
for blockIter=1:numBlocks,
    disp(['Vasc Binarize Raw subvolume ',num2str(blockIter),' of ',num2str(length(vascBlockFileNameList)),' : ',datestr(now,'mm/dd/yy HH:MM:SS')]);
    %-----
    currentMaskFileName = vascBlockRawMaskFileNameList{blockIter};
    %-----
    overwriteFlag = 0;
    fileExistsFlag = (exist(currentMaskFileName,'file')==2);
    if fileExistsFlag,
        fileProps = dir(currentMaskFileName);
        fileDateNum = datenum(fileProps.date);
        if fileDateNum < lastRerunStartDateNum,
            overwriteFlag = 1;
        end%if fileDateNum < lastRerunStartDateNum,
    end%if if fileExistsFlag,
    %-----
    if and(fileExistsFlag==1,overwriteFlag==0),
        disp('...Skipping-Previously calculated');
        load(vascBlockRawMaskBinarizeDetailsFileNameList{blockIter},'vbthresh','rawFraction');
    else%if and(fileExistsFlag==1,overwriteFlag==0),
        loadedStruct = load(vascBlockFileNameList{blockIter});
        loadedVariables = fieldnames(loadedStruct);
        vascBlock = loadedStruct.(loadedVariables{1});
        %---
        loadedStruct = load(vascBlockEnhancedGrayscaleFileNameList{blockIter},'vascBlockEnhancedGrayscale','-mat');
        loadedVariables = fieldnames(loadedStruct);
        vascBlockEnhancedGrayscale = loadedStruct.(loadedVariables{1});
        %-----  
        %Generate a preliminary mask of the vessel regions from the rod-filtered data.
        maxVal = max(vascBlockEnhancedGrayscale(:));
        sortEnh = sort(vascBlockEnhancedGrayscale(vascBlockEnhancedGrayscale>0));%sort the non-zero pixels
        numNonZero = numel(sortEnh);
        %   take the bottom 25% of the sorted pixels as a rough background
        enhBkg = sortEnh(1:ceil(numNonZero*0.25));
        bkgMean = mean(enhBkg);
        bkgStd = std(enhBkg);
        %   set a threshold as some small fraction (e.g. 5%) of the max Intensity Value 
        threshVal = maxVal*enhMaskPrelimThresholdFraction;
        %   set a minimum thresh as 1 stdev above the mean background
        minThresh = bkgMean + 1*bkgStd;
        threshVal = max(threshVal,minThresh);
        enhMask = vascBlockEnhancedGrayscale>(threshVal);
        %-----
        %Generate bkg regions by Dilating up the preliminary Mask to exclude all vessel regions        
        vesselRegionMask = enhMask;
        nonZeroPixelsCount = sum(vascBlockEnhancedGrayscale(:)>0);
        keepGoing = 1;
        circleDiameter3 = circle3d(3);
        cycleCounter = 0;
        while keepGoing==1,
        cycleCounter=cycleCounter+1;
            previousRoundMask = vesselRegionMask;
            vesselRegionMask = imfilter(previousRoundMask,double(circleDiameter3))>0;
            newFraction = sum(vesselRegionMask(:))./nonZeroPixelsCount;
            if newFraction>0.80, keepGoing = 0; end
            if cycleCounter >maxDilationCycles, keepGoing = 0; disp('==========DILATION LIMIT REACHED');end
         end
        enhMask = previousRoundMask;
        nonVesselRegions = single(vascBlock(~enhMask));
        %-----
        %Calculate a mode and standard deviation of the background regions
%         modeVal = mode(round(nonVesselRegions(nonVesselRegions>0)));
        modeVal = mean(nonVesselRegions(nonVesselRegions>0));       
        stdCutBlock = std(nonVesselRegions(nonVesselRegions>0));

        %-----
        %Set a minimum threshold that is several standard deviations above the background.
        %Set a maximum threshold as a fraction of the brightest rod-filtered pixel intensity.
        minThresh = modeVal+numBkgStdForMinThresh*stdCutBlock;
        maxThresh = fractionOfHighIntensityForMaxThresh * maxVal;
        %-----
        %Determine a variable threshold by performing local mean field subtraction.
        %   The local mean field is determined as the minimum of the mean value using a large
        %   and small gaussian filter. 
        gaussianBlockDiameter = 6*gaussianRadiusForFilter+1;
        gaussianBlockDiameter = min(min(size(vascBlockEnhancedGrayscale))-5,gaussianBlockDiameter);
        largeGaussian = gaussian3d(gaussianBlockDiameter,gaussianRadiusForFilter);
        largeGaussianSum = sum(largeGaussian(:));
        filtBlock1 = VidaConv3DFreq(vascBlock,largeGaussian)./largeGaussianSum;
        %-----
        smallGaussian = gaussian3d(gaussianBlockDiameter,gaussianRadiusForFilter/smallGaussianSizeFactor);
        smallGaussianSum = sum(smallGaussian(:));
        filtBlock2 = VidaConv3DFreq(vascBlock,smallGaussian)./smallGaussianSum;
        %-----
        %Set the threshold to a multiplier times the local mean field
        %   subject to the min and max threshold constraints
        threshBlock = min(filtBlock1,filtBlock2)*filtMultiplierForVarThresh;
        threshBlock = min(threshBlock,maxThresh);
        threshBlock = max(threshBlock,minThresh);
        vascBlockRawMask = vascBlock>threshBlock;
        %-----
        vbthresh{blockIter} = minThresh;
        rawFraction{blockIter} = sum(vascBlockRawMask(:))./numel(vascBlockRawMask);
             

        %----------
        %----------
        %Eliminate contiguous 6-connected regions of size  [minVolForSmallVolRemoval] or less
        %-----
        L=bwlabeln(vascBlockRawMask,6);
        stmp=regionprops(L,'Area');
        LUT=[0,[stmp.Area]>=minVolForSmallVolRemoval];
        L=LUT(L+1);
        vascBlockRawMask = L>0;
        %-----
        %Close the mask with a 5-diameter circle
        vascBlockRawMask = imclose(vascBlockRawMask,circle3d(5));
        %-----
        %Run multiple iterations of a convexify-ing routine.
        %   Remove pixels "hairs" that have fewer than [minVol] of 26-connected neighbors.
        %   but also add any pixels that are more than [minVol] of 26-connected neighbors.
        %   
        for cleanIter = 1:numCleanupCycles,
            filtMask = imfilter(double(vascBlockRawMask),ones(3,3,3),'replicate');
            vascBlockRawMask = filtMask>=minVolForCleanup;
        end%for cleanIter = 1:numCleanupCycles,  
        %---
        saveSize = (getfield(whos('vbthresh'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(vascBlockRawMaskBinarizeDetailsFileNameList{blockIter},'vbthresh','rawFraction',scs);
        pixelList = find(vascBlockRawMask);
        maskSize = size(vascBlockRawMask);
        vascBlockRawMask = [];
        vascBlockRawMask.pixelList = pixelList;
        vascBlockRawMask.maskSize = maskSize;
        saveSize = (getfield(whos('vascBlockRawMask'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(vascBlockRawMaskFileNameList{blockIter},'vascBlockRawMask',scs);
    end%if and(fileExistsFlag==1,overwriteFlag==0),
end%for blockIter=1:length(vascBlockFileNameList)

outputParameters = inputParameters;
outputParameters.vascBlockThresh = vbthresh;
outputParameters.rawFraction = rawFraction;
outputParameters.vascBlockRawMaskFileNameList = vascBlockRawMaskFileNameList;



%ALGORITHM FLOW AND LOGIC
%The goal is to binarize the raw data to make a raw mask
%Start with the rod-enhanced grayscale volume.
%   Take the lower quarter of the non-zero intensity values as a tentative background distribution and get a mean and stdev of the bkg.
%   Take as preliminary threshold the greater of (0.05) * maximal intensity ,and bkg_mean + 1* bkg_stdev
%   Dilate this mask and take its inverse as a way to determine background regions that are away from vessels
%Apply this backgound mask to the raw data; calculate a new bkg_std and bkg_mode    
%Set an upper threshold limit as (0.15) * the maximal intensity of ENHANCED volume (filtered - so no bright single pixels)            
%Set a lower threshold limit as bkg_mode + (4.4) * bkg_stdev.
%Find a voxel-by-voxel threshold on the raw volume:
%   Filter with a (18)-micron FWHM 3d isotropic gaussian, and also with a (4.5)-micron-FWHM 3d isotropic gaussian
%   Take (1.5) * the minimum between these two values; subject to the upper and lower threshold limits determined earlier
%Eliminate contigous regions smaller than (27) voxels - to remove pixel noise
%Close the mask with a 5-voxel wide sphere
%Run a convexifying routine (3) times :  sum-filter the mask over a 3x3x3 cube, and keep voxel values > (11)   
%
%Logic : Large, bright vessels:  both sized filters  will give high values, so 1.5*Value may be too high : upper thresh limit kicks in
%Logic : Medium vessels : The small(4.5) filter may still have high values, but the large(16) filter will filter in lots of background
%   so its value is low, and 1.5*Value still catches the vessel
%Logic : Small vessels : barely caught by small(4.5) filter; but if this is a dim vessel in a dark region, the low threshold limit
%   will catch it, as the low threshold limit is set to the upper range(far_from_vessel) background 
%Logic : Small vessel in "haze" of large vessel:
            %In the small vessel : small filter will catch it
            %In the large vessel : upper limit on threshold kicks in
            %In the "haze" between : small filter works because in even a mild dip between the big and small bumps
            %   the mean value is pulled high enough that 1.5*Value will yield space between the vessels in the haze                 
            %By contrast, the big filter alone could lose the small vessel
            %   The small filter alone would hollow out the large vessel
            %   The small filter alone with the upper limit MAY work.

            
            
            