function outProject = VidaMultiThreadImageStackChild(inProject)

vidaProject = inProject;
vidaProjectRootName = vidaProject.vidaProjectRootName;
imageStackFolderNameList = vidaProject.imageStackFolderNameList;
multiThreadProgress = vidaProject.multiThreadProgress;
multiThreadRequestFolder = multiThreadProgress.multiThreadRequestFolder;

%-----
numImageStacks = size(imageStackFolderNameList,2);
%-----
%STAGE 4 - ChildProcess : Request a image stack folder for processing
%   Generate a 6-digit random number as a processor ID
rand('twister', (sum(1000*clock)+(cputime*10000)));
childProcessorID = ceil(rand(1)*(999998));
childProcessorIDString = num2str(childProcessorID,'%06d');
allImageStacksCompletedSignal = 0;
waitForParentCounter = 0;
waitForParentMaxLoops = 200;
waitForParentDelayInSec = 15;%seconds
completedResponseIDList = [];
%-----
disp(['ChildProcessor ID : ',childProcessorIDString]);
while allImageStacksCompletedSignal == 0,
    %Check for a response file - if none exists, create a request file
    requestFileName = fullfile(multiThreadRequestFolder,['MultiThreadRequest',childProcessorIDString,'.mat']);
    responseFileName = fullfile(multiThreadRequestFolder,['MultiThreadResponse',childProcessorIDString,'.mat']);
    deliveryFileName = fullfile(multiThreadRequestFolder,['MultiThreadDelivery',childProcessorIDString,'.mat']);
    if ~exist(responseFileName,'file'),
        if ~exist(requestFileName,'file'),
            %Generate a request file
            disp(['Sending multiThread request as child-processor # ',childProcessorIDString,' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            requestVariable = [];
            save(requestFileName,'requestVariable');
            pause(3);
        end%if ~exist(requestFileName,'file'),
    end%if ~exist(responseFileName,'file'),
    %-----
    %Check for a response file, and process the Image stack.
    %Look in request folder for delivery files
    if exist(responseFileName,'file'),
        try
            disp('Detected Response File. Attempting Load.');
            load(responseFileName,'multiThreadResponse');
            assignmentIndex = multiThreadResponse.assignmentIndex;
            assignmentString = multiThreadResponse.assignmentString;
            responseID = multiThreadResponse.responseID;
            disp(['Load successful, Assignment Index = ',num2str(assignmentIndex)])
        catch
            assignmentIndex = 0;
            responseID = 0;
            assignmentString = '';
            disp('Load Failed');
        end%try-catch
        %-----
        if assignmentIndex == -1,
            %This stage is completed.  Move on to next multi-thread stage
            disp(['Image Stack Stage completion signal received : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            allImageStacksCompletedSignal = 1;
        end%if assignmentIndex == -1,
        %-----
        %If this reponse has already been delivered, just wait for next response.
        if any(responseID == completedResponseIDList)
            assignmentIndex = 0;
        end%if any(responseID == completedResponseIDList)
        %-----
        if assignmentIndex == 0,
            %No stack assigned right now - wait and keep requesting
            disp(['Resending and waiting for reponse to Image Stack request as child-processor #',childProcessorIDString,' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            requestVariable = [];
            save(requestFileName,'requestVariable');
            pause(waitForParentDelayInSec);
            waitForParentCounter = waitForParentCounter + 1;
        end%if assignmentIndex == 0,
        %-----
        if waitForParentCounter >= waitForParentMaxLoops,
            %No stack assigned right now - wait and keep requesting
            disp(['Parent process failing to repond.  Moving to next stage : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            pause(waitForParentDelayInSec);
            assignmentIndex = -1;
        end%if if waitForParentCounter >= waitForParentMaxLoops,
        %-----
        if assignmentIndex > 0,
            waitForParentCounter = 0;
            %-----
            %Before processing, send the next request file
            %This eliminates excessive waiting for parent process.
            if ~exist(requestFileName,'file'),
                disp(['Sending next multiThread request as child-processor # ',childProcessorIDString,' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
                requestVariable = [];
                save(requestFileName,'requestVariable');
                pause(3);
            end%if ~exist(requestFileName,'file'),
            %-----
            %*******HERE IS WHERE YOU ACTUALLY PROCESS THE IMAGE BLOCK*****
            %*******HERE IS WHERE YOU ACTUALLY PROCESS THE IMAGE BLOCK*****
            %*******HERE IS WHERE YOU ACTUALLY PROCESS THE IMAGE BLOCK*****
            fprintf('\n\n');
            disp(['Child-Process prcoessing Stack # ',num2str(assignmentIndex),' of ',num2str(numImageStacks),' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            fprintf('\n\n');
            currentImageStackFolderName = imageStackFolderNameList{assignmentIndex};
            vidaProject.setupParameters.currentImageStackFolderName = currentImageStackFolderName;
                vidaProject = VidaProcessImageStackSL(vidaProject);
            fprintf('\n\n');
            disp(['Completed Image Stack # ',num2str(assignmentIndex),' of ',num2str(numImageStacks),' : ',datestr(now)]);
            fprintf('\n\n');
            %----- 
            %Generate delivery file
            deliveryData = [];
            deliveryData.IDNum = assignmentIndex;
            deliveryCompleted = 0;
            while deliveryCompleted == 0,
                if ~exist(deliveryFileName,'file'),
                    save(deliveryFileName,'deliveryData');
                    deliveryCompleted = 1;
                else%if ~exist(deliveryFileName,'file'),
                    pause(10);
                end%if-else ~exist(deliveryFileName,'file'),
            end%while deliveryCompleted == 0,
            %-----
            completedResponseIDList = [completedResponseIDList;responseID];
            assignmentIndex = 0 ; %set this processor to wait.
        end%if assignmentIndex > 0,
    end%if exist(responseFileName,'file'),
end%while allImageStacksCompletedSignal == 0,
%-----
%All Image Stacks Completed.
outProject = vidaProject;

