function outProject = VidaMultiThreadRegSubBlockChild(inProject)

vidaProject = inProject;
vidaProjectRootName = vidaProject.vidaProjectRootName;
vidaProjectFileName = vidaProject.vidaProjectFileName;
multiThreadProgress = vidaProject.multiThreadProgress;
multiThreadRequestFolder = multiThreadProgress.multiThreadRequestFolder;
registeredExperiment = vidaProject.registeredExperiment;
%-----
registeredVascProcessFlagList = multiThreadProgress.registeredVascProcessFlagList;
blockGrid = registeredExperiment.globalParameters.blockGrid;
vascBlockEnhancedMaskFileNameList = registeredExperiment.globalParameters.vascBlockEnhancedMaskFileNameList;
vascBlockRawMaskFileNameList = registeredExperiment.globalParameters.vascBlockRawMaskFileNameList;
vascBlockFileNameList = registeredExperiment.globalParameters.vascBlockFileNameList;
%-----
numSubBlocks = size(blockGrid,2);



%-----
%STAGE 4 - Child : Request a registered subblockfor processing
%   Generate a 6-digit random number as a processor ID
rand('twister', (sum(1000*clock)+(cputime*10000)));
childProcessorID = ceil(rand(1)*(999998));
childProcessorIDString = num2str(childProcessorID,'%06d');
allSubBlocksCompletedSignal = 0;
waitForParentCounter = 0;
waitForParentMaxLoops = 90;
waitForParentDelayInSec = 60;%seconds
completedResponseIDList = [];
%-----
disp(['Child-Processor ID : ',childProcessorIDString]);
while allSubBlocksCompletedSignal == 0,
    %Check for a response file - if none exists, create a request file
    requestFileName = fullfile(multiThreadRequestFolder,['MultiThreadRequest',childProcessorIDString,'.mat']);
    responseFileName = fullfile(multiThreadRequestFolder,['MultiThreadResponse',childProcessorIDString,'.mat']);
    deliveryFileName = fullfile(multiThreadRequestFolder,['MultiThreadDelivery',childProcessorIDString,'.mat']);
    if ~exist(responseFileName,'file'),
        if ~exist(requestFileName,'file'),
            %Generate a request file
            disp(['Sending multiThread request as child-processor # ',childProcessorIDString,' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            requestVariable = [];
            save(requestFileName,'requestVariable');
            pause(3);
        end%if ~exist(requestFileName,'file'),
    end%if ~exist(responseFileName,'file'),
    %-----
    %Check for a response file, and process the Image stack.
    %Look in request folder for delivery files
    if exist(responseFileName,'file'),
        try
            disp('Detected Response File. Attempting Load.');
            load(responseFileName,'multiThreadResponse');
            assignmentIndex = multiThreadResponse.assignmentIndex;
            assignmentString = multiThreadResponse.assignmentString;
            responseID = multiThreadResponse.responseID;
            disp(['Load successful, Assignment Index = ',num2str(assignmentIndex)])
        catch
            assignmentIndex = 0;
            responseID = 0;
            assignmentString = '';
            disp('Load Failed');
        end%try-catch
        %-----
        if assignmentIndex == -1,
            %This stage is completed.  Move on to next multi-thread stage
            disp(['Image Stack Stage completion signal received : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            allSubBlocksCompletedSignal = 1;
        end%if assignmentIndex == -1,
        %-----
        %If this response has already been delivered, just wait for next response.
        if any(responseID == completedResponseIDList)
            disp('Response file was already delivered.');
            assignmentIndex = 0;
        end%if any(responseID == completedResponseIDList)
        %-----
        if assignmentIndex == 0,
            %No stack assigned right now - wait and keep requesting
            disp(['No subblock was assigned']);
            disp(['Resending and waiting for response to subBlock request as child-processor #',childProcessorIDString,' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            requestVariable = [];
            save(requestFileName,'requestVariable');
            pause(waitForParentDelayInSec);
            waitForParentCounter = waitForParentCounter + 1;
        end%if assignmentIndex == 0,
        %-----
        if waitForParentCounter >= waitForParentMaxLoops,
            %No stack assigned right now - wait and keep requesting
            disp(['Parent process failing to repond.  Moving to next stage : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            pause(waitForParentDelayInSec);
            assignmentIndex = -1;
        end%if if waitForParentCounter >= waitForParentMaxLoops,
        %-----
        if assignmentIndex > 0,
            waitForParentCounter = 0;
            %-----
            %Before processing, send the next request file
            %This eliminates excessive waiting for parent process.
            if ~exist(requestFileName,'file'),
                disp(['Sending next multiThread request as child-processor # ',childProcessorIDString,' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
                requestVariable = [];
                save(requestFileName,'requestVariable');
                pause(3);
            end%if ~exist(requestFileName,'file'),
            %-----
            %*******HERE IS WHERE YOU ACTUALLY PROCESS THE SUBBLOCK*****
            %*******HERE IS WHERE YOU ACTUALLY PROCESS THE SUBBLOCK*****
            %*******HERE IS WHERE YOU ACTUALLY PROCESS THE SUBBLOCK*****
            fprintf('\n\n');
            disp(['Child Processing SubBlock # ',num2str(assignmentIndex),' of ',num2str(numSubBlocks),' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            currentRegVascBlockFileName = vascBlockFileNameList{assignmentIndex};
            currentRegVascBlockRawMaskFileName = vascBlockRawMaskFileNameList{assignmentIndex};
            currentRegVascBlockEnhancedMaskFileName = vascBlockEnhancedMaskFileNameList{assignmentIndex};
            vidaProject.setupParameters.currentRegVascBlockFileName = currentRegVascBlockFileName;
            vidaProject.setupParameters.currentRegVascBlockRawMaskFileName = currentRegVascBlockRawMaskFileName;
            vidaProject.setupParameters.currentRegVascBlockEnhancedMaskFileName = currentRegVascBlockEnhancedMaskFileName;
            vidaProject.setupParameters.assignmentNumber = assignmentIndex;
                vidaProject = VidaProcessSingleRegSubBlock(vidaProject);
            disp([' ';' ';' ']);
            disp(['Completed SubBlock # ',num2str(assignmentIndex),' of ',num2str(numSubBlocks),' : ',datestr(now)]);
            disp([' ';' ';' ']);
            %----- 
            %Generate delivery file
            deliveryData = [];
            deliveryData.IDNum = assignmentIndex;
            deliveryCompleted = 0;
            while deliveryCompleted == 0,
                if ~exist(deliveryFileName,'file'),
                    save(deliveryFileName,'deliveryData');
                    deliveryCompleted = 1;
                else%if ~exist(deliveryFileName,'file'),
                    pause(10);
                end%if-else ~exist(deliveryFileName,'file'),
            end%while deliveryCompleted == 0,
            %-----
            completedResponseIDList = [completedResponseIDList;responseID];
            assignmentIndex = 0 ; %set this processor to wait.
        end%if assignmentIndex > 0,
    end%if exist(responseFileName,'file'),
end%while allSubBlocksCompletedSignal == 0,
%-----
%All Image Stacks Completed.
outProject = vidaProject;



