function outProject = VidaMultiThreadRegSubBlockParent(inProject)

vidaProject = inProject;
vidaProjectRootName = vidaProject.vidaProjectRootName;
vidaProjectFileName = vidaProject.vidaProjectFileName;
multiThreadProgress = vidaProject.multiThreadProgress;
multiThreadRequestFolder = multiThreadProgress.multiThreadRequestFolder;
registeredExperiment = vidaProject.registeredExperiment;
%-----
registeredVascProcessFlagList = multiThreadProgress.registeredVascProcessFlagList;
blockGrid = registeredExperiment.globalParameters.blockGrid;
vascBlockEnhancedMaskFileNameList = registeredExperiment.globalParameters.vascBlockEnhancedMaskFileNameList;
vascBlockRawMaskFileNameList = registeredExperiment.globalParameters.vascBlockRawMaskFileNameList;
vascBlockFileNameList = registeredExperiment.globalParameters.vascBlockFileNameList;
%-----
numSubBlocks = size(blockGrid,2);
%-----
delayMultiplier = vidaProject.experimentTemplate.basicParameters.delayMultiplier;
%-----
%STAGE 4 : Assign a registered subblock, and run centerline and vasc radius.
%Look in request folder : find all delivery files
%   Log results from each delivery file and delete it.
%Search for all request files
%For each request file, write a response file, assigning an subBlock
%   Delete the request file
%Take an imagestack and process it.

allSubBlocksAssignedFlag = 0;
allSubBlocksCompletedFlag = 0;
waitForChildCounter = 0;
waitForChildMaxLoops = round(20 * delayMultiplier + 1);%note that each loop is 3x of waitForChildDelayInSec
waitForChildDelayInSec = round(60 * delayMultiplier + 1);%seconds

%%-----
%Initial Delay to allow child requests to come in
global skipDelayButtonFlag
skipDelayButtonFlag = 0;
delayFigureHandle = VidaGuiSkipDelay({'VIDA parent - allow time for child requests'});
disp('Pausing momentarily for child requests');
for delayLoop = 1:waitForChildDelayInSec,
    pause(1);
    if skipDelayButtonFlag == 1, break; end
end%for delayLoop = 1:waitForChildDelayInSec,
if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
    close(delayFigureHandle);
end%if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
pause(0.5);
%-----
while allSubBlocksCompletedFlag == 0,
    %Look in request folder for delivery files
    searchString = fullfile(multiThreadRequestFolder,'MultiThreadDelivery*.mat');
    myDir = dir(searchString);
    numDeliveryFiles = size(myDir,1);
    for deliveryIter = 1:numDeliveryFiles,
        currentDeliveryFileName = myDir(deliveryIter).name;
        fullDeliveryFileName = fullfile(multiThreadRequestFolder,currentDeliveryFileName);
        load(fullDeliveryFileName,'deliveryData');
        registeredVascProcessFlagList(deliveryData.IDNum) = 2;
            %0=not,1=started(assigned),2=completed.
        pause(0.03);
        %Delete both the delivery file and the corresponding response file
        numberPos = regexp(currentDeliveryFileName,'\d');
        currentDeliveryID = currentDeliveryFileName(numberPos(1):numberPos(end));
        delete(fullDeliveryFileName);
        correspondingResponseFileName = fullfile(multiThreadRequestFolder,['MultiThreadResponse',currentDeliveryID,'.mat']);
        try
            load(correspondingResponseFileName,'multiThreadResponse');
            if  deliveryData.IDNum == multiThreadResponse.assignmentIndex,
                delete(correspondingResponseFileName);
            end%if  deliveryData.IDNum == multiThreadResponse.assignmentIndex,
        catch
        end%try-catch
        pause(0.03);
    end
    %-----
    %Check for unassigned stacks
    if isempty(find(registeredVascProcessFlagList==0)),
        allSubBlocksAssignedFlag = 1;
        numCompleted = sum(registeredVascProcessFlagList==2);
        if numCompleted == numSubBlocks,
            allSubBlocksCompletedFlag = 1;
        end%if numCompleted == numSubBlocks,
    end%if isempty(find(registeredVascProcessFlagList==0)),
    %-----
    %Pick an assignment for parent process
    notAssignedList = find(registeredVascProcessFlagList==0);
    if ~isempty(notAssignedList),
        parentAssignment = notAssignedList(1);
        registeredVascProcessFlagList(parentAssignment) = 1;%indicates assigned
    else%if ~isempty(notAssignedList),
        parentAssignment = 0;
    end%if-else ~isempty(notAssignedList),
    %-----
    %Look for request files, and generate response files
    for requestLoopIter = 1:2,
        %loop twice to give child processes a chance to accept a response, and send the
        %next round request before starting
        searchString = fullfile(multiThreadRequestFolder,'MultiThreadRequest*.mat');
        myDir = dir(searchString);
        numRequestFiles = size(myDir,1);
        requesterIDList = [];
        for requestIter = 1:numRequestFiles,
            multiThreadResponse = [];
            multiThreadResponse.stageNum = 1;
            multiThreadResponse.stageString = 'adjacentPair';
            %-----
            currentRequestFileName = myDir(requestIter).name;
            numberPos = regexp(currentRequestFileName,'\d');
            currentRequesterID = currentRequestFileName(numberPos(1):numberPos(end));
            correspondingResponseFileName = fullfile(multiThreadRequestFolder,['MultiThreadResponse',currentRequesterID,'.mat']);
            requesterIDList{requestIter} = currentRequesterID;
            notAssignedList = find(registeredVascProcessFlagList==0);
            if ~exist(correspondingResponseFileName),
                if isempty(notAssignedList),
                    allAdjacentPairsAssignedFlag = 1;
                    multiThreadResponse.assignmentIndex = 0;%indicates no assignment right now - wait.
                    multiThreadResponse.responseID = 0;
                    multiThreadResponse.assignmentString = '';
                else%if isempty(notAssignedList),
                    nextUnassignedIndex = notAssignedList(1);
                    multiThreadResponse.assignmentIndex = nextUnassignedIndex;
                    rand('twister', (sum(1000*clock)+(cputime*10000)));
                    multiThreadResponse.responseID = ceil(rand(1)*(999999998));%unique ID for this request response;
                    registeredVascProcessFlagList(nextUnassignedIndex) = 1;
                    %indicates assigned
                    multiThreadResponse.assignmentString = '';
                end%if-else isempty(notAssignedList),
                %-----
                currentResponseFileName = fullfile(multiThreadRequestFolder,['MultiThreadResponse',currentRequesterID,'.mat']);
                currentRequestFullFileName = fullfile(multiThreadRequestFolder,['MultiThreadRequest',currentRequesterID,'.mat']);
                save(currentResponseFileName,'multiThreadResponse');
                pause(0.03);
                if exist(currentRequestFullFileName),
                    delete(currentRequestFullFileName);
                end%if exist(currentRequestFullFileName),
            end%if ~exist(correspondingResponseFileName),
        end%for requestIter = 1:numRequestFiles,
        %pause to allow time for child processes to queue up next-round request
        if requestLoopIter < 2,
            global skipDelayButtonFlag
            disp(['Waiting for child processor to queue next requests  : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            skipDelayButtonFlag = 0;
            delayFigureHandle = VidaGuiSkipDelay({'VIDA parent - allow time for child requests'});
            for delayLoop = 1:waitForChildDelayInSec,
                pause(1);
                if skipDelayButtonFlag == 1, break; end;
            end%for delayLoop = 1:waitForChildDelayInSec,
            if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
                close(delayFigureHandle);
            end%if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
            pause(0.5);
        end%if requestLoopIter <2
    end%for requestLoopIter = 1:2
    %-----
    %Recheck for unassigned stacks
    if isempty(find(registeredVascProcessFlagList==0)),
        allSubBlocksAssignedFlag = 1;
        numCompleted = sum(registeredVascProcessFlagList==2);
        if numCompleted == numSubBlocks,
            allSubBlocksCompletedFlag = 1;
        end%if numCompleted == numSubBlocks,
    end%if isempty(find(registeredVascProcessFlagList==0)),
    %-----
    if parentAssignment == 0,
        %If not all completed, wait, and then recheck
        if allSubBlocksCompletedFlag == 0;
            waitForChildCounter = waitForChildCounter + 1;
            %%-----
            global skipDelayButtonFlag
            disp(['All subBlocks assigned - waiting for delivery : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            skipDelayButtonFlag = 0;
            delayFigureHandle = VidaGuiSkipDelay({'Waiting on deliveries - skip to reassign image stacks'});
            for delayLoop = 1:waitForChildDelayInSec,
                pause(1);
                if skipDelayButtonFlag == 1, 
                    waitForChildCounter = waitForChildMaxLoops;
                    break;
                end%if skipDelayButtonFlag == 1, 
            end%for delayLoop = 1:waitForChildDelayInSec,
            if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
                close(delayFigureHandle);
            end%if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
            pause(0.5);
            %%-----
        end%if allSubBlocksCompletedFlag == 0;
    else%if parentAssignment == 0,
        %-----
        %Run the parent assignment
        %*******HERE IS WHERE YOU ACTUALLY PROCESS THE IMAGE BLOCK*****
        %*******HERE IS WHERE YOU ACTUALLY PROCESS THE IMAGE BLOCK*****
        %*******HERE IS WHERE YOU ACTUALLY PROCESS THE IMAGE BLOCK*****
        fprintf('\n\n');
        disp(['Parent Processing SubBlock # ',num2str(parentAssignment),' of ',num2str(numSubBlocks),' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
        currentRegVascBlockFileName = vascBlockFileNameList{parentAssignment};
        currentRegVascBlockRawMaskFileName = vascBlockRawMaskFileNameList{parentAssignment};
        currentRegVascBlockEnhancedMaskFileName = vascBlockEnhancedMaskFileNameList{parentAssignment};
        vidaProject.setupParameters.currentRegVascBlockFileName = currentRegVascBlockFileName;
        vidaProject.setupParameters.currentRegVascBlockRawMaskFileName = currentRegVascBlockRawMaskFileName;
        vidaProject.setupParameters.currentRegVascBlockEnhancedMaskFileName = currentRegVascBlockEnhancedMaskFileName;
        vidaProject.setupParameters.assignmentNumber = parentAssignment;
            vidaProject = VidaProcessSingleRegSubBlock(vidaProject);
        disp([' ';' ';' ']);
        disp(['Completed SubBlock # ',num2str(parentAssignment),' of ',num2str(numSubBlocks),' : ',datestr(now)]);
        disp([' ';' ';' ']);
        %-----
        registeredVascProcessFlagList(parentAssignment) = 2;%indicates completed
    end%if-else parentAssignment == 0,
    %-----
    if waitForChildCounter >= waitForChildMaxLoops,
        %Assume a child processor crashed.  Un-assign all uncompleted stacks.
        registeredVascProcessFlagList(registeredVascProcessFlagList==1) = 0;
        waitForChildCounter = 0;
        disp(['Re-assigning uncompleted image stacks : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
        pause(3);
        %Delete all existing request and response files
        searchString1 = fullfile(multiThreadRequestFolder,'MultiThreadRequest*.mat');
        myDir1 = dir(searchString1);
        searchString2 = fullfile(multiThreadRequestFolder,'MultiThreadResponse*.mat');
        myDir2 = dir(searchString2);
        pause(3);
        %-----
        numResidualRequestFiles = size(myDir1,1);
        for fileIter = 1:numResidualRequestFiles
            currentFileName = myDir1(fileIter).name;
            currentFullFileName = fullfile(multiThreadRequestFolder,currentFileName);
            delete(currentFullFileName);
        end%for fileIter = 1:numResidualRequestFiles
        %-----
        numResidualResponseFiles = size(myDir2,1);
        for fileIter = 1:numResidualResponseFiles
            currentFileName = myDir2(fileIter).name;
            currentFullFileName = fullfile(multiThreadRequestFolder,currentFileName);
            delete(currentFullFileName);
        end%for fileIter = 1:numResidualResponseFiles
        %-----
    end%if waitForChildCounter >= waitForChildMaxLoops,
    %Save VidaProject to file
    multiThreadProgress.registeredVascProcessFlagList = registeredVascProcessFlagList;
    vidaProject.multiThreadProgress = multiThreadProgress;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
end%while allSubBlocksCompletedFlag == 0,
%-----
%Wait for all child processes who are "waiting for a response" to recycle
%   their requests - and then reply to them with an "all done" signal
%-----
%Wait for all child processes who are "waiting for a response" to recycle
%   their requests - and then reply to them with an "all done" signal
global skipDelayButtonFlag
disp(['Waiting for final round of child processor requests  : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
skipDelayButtonFlag = 0;
delayFigureHandle = VidaGuiSkipDelay({'VIDA parent - allow time for child requests'});
for delayLoop = 1:waitForChildDelayInSec,
    pause(1);
    if skipDelayButtonFlag == 1, break; end;
end%for delayLoop = 1:waitForChildDelayInSec,
if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
    close(delayFigureHandle);
end%if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
pause(0.5);
%-----
disp(['Sending All-done signal  : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
searchString = fullfile(multiThreadRequestFolder,'MultiThreadRequest*.mat');
myDir = dir(searchString);
numRequestFiles = size(myDir,1);
disp([num2str(numRequestFiles),' final request files detected']);
requesterIDList = [];
for requestIter = 1:numRequestFiles,
    multiThreadResponse = [];
    multiThreadResponse.responseID = 0;
    multiThreadResponse.stageNum = 1;
    multiThreadResponse.stageString = 'SubBlock';
    %-----
    currentRequestFileName = myDir(requestIter).name;
    numberPos = regexp(currentRequestFileName,'\d');
    currentRequesterID = currentRequestFileName(numberPos(1):numberPos(end));
    requesterIDList{requestIter} = currentRequesterID;
    multiThreadResponse.assignmentIndex = -1;%indicates stage completed - all done.
    multiThreadResponse.assignmentString = '';
    %-----
    currentResponseFileName = fullfile(multiThreadRequestFolder,['MultiThreadResponse',currentRequesterID,'.mat']);
    currentRequestFullFileName = fullfile(multiThreadRequestFolder,['MultiThreadRequest',currentRequesterID,'.mat']);
    save(currentResponseFileName,'multiThreadResponse');
    pause(0.03);
    if exist(currentRequestFileName,'file'),
        delete(currentRequestFullFileName);
    end%if exist(currentRequestFileName,'file'),
end%for requestIter = 1:numRequestFiles,
%-----
%All sub blocks Completed.
%-----
%Clear out the request folder
%-----
global skipDelayButtonFlag
disp(['Allowing time for receipt of all-done signal  : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
skipDelayButtonFlag = 0;
delayFigureHandle = VidaGuiSkipDelay({'VIDA parent - allow time for receipt of all-done signal'});
for delayLoop = 1:waitForChildDelayInSec,
    pause(1);
    if skipDelayButtonFlag == 1, break; end;
end%for delayLoop = 1:waitForChildDelayInSec,
if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
    close(delayFigureHandle);
end%if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
pause(0.5);
%%-----
disp(['Clearing request folder  : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
searchString = fullfile(multiThreadRequestFolder,'MultiThread*.mat');
myDir = dir(searchString);
numResidualFiles = size(myDir,1);
for residualIter = 1:numResidualFiles,
    currentResidualFileName = myDir(residualIter).name;
    fullResidualFileName = fullfile(multiThreadRequestFolder,currentResidualFileName);
    delete(fullResidualFileName);
end%for residualIter = 1:numResidualFiles,
%-----
%All sub blocks Completed.
%-----
disp(['All sub blocks completed : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
multiThreadProgress.registeredVascProcessFlagList = registeredVascProcessFlagList;
vidaProject.multiThreadProgress = multiThreadProgress;
outProject = vidaProject;

