function outProject = VidaRegisterBlocksParentSL(inProject)

vidaProject = inProject;
vidaProjectRootName = vidaProject.vidaProjectRootName;
vidaProjectFileName = vidaProject.vidaProjectFileName;
imageStackFolderNameList = vidaProject.imageStackFolderNameList;
adjacentPairRegistrationDataFolder = vidaProject.globalParameters.adjacentPairRegistrationDataFolder;
multiThreadProgress = vidaProject.multiThreadProgress;
multiThreadRequestFolder = multiThreadProgress.multiThreadRequestFolder;
%-----
experimentTemplate = vidaProject.experimentTemplate;
currentExperiment = vidaProject.currentExperiment;
fullStackSize = currentExperiment.globalParameters.fullStackSize;
registrationParameters = experimentTemplate.registrationParameters;
reverseYBlockNumberingFlag = registrationParameters.reverseYBlockNumberingFlag;
reverseXBlockNumberingFlag = registrationParameters.reverseXBlockNumberingFlag;
xcorrCh = registrationParameters.xcorrCh;
nominalBlockStepSizeRCInMicrons = registrationParameters.nominalBlockStepSizeRCInMicrons;
nominalBlockStepSizeZInMicrons = registrationParameters.nominalBlockStepSizeZInMicrons;
variableZBlockStepSizeFlag = registrationParameters.variableZBlockStepSizeFlag;
ySearchSizeInMicrons = registrationParameters.ySearchSizeInMicrons;
xSearchSizeInMicrons = registrationParameters.xSearchSizeInMicrons;
zSearchSizeInMicrons = registrationParameters.zSearchSizeInMicrons;
experimentTemplate.registrationParameters.emptyBlockCorrectionFlag = 1; %set to 1 to default empty blocks to nominal position
emptyBlockCorrectionFlag = registrationParameters.emptyBlockCorrectionFlag;
emptyBlockThreshold = registrationParameters.emptyBlockThreshold;
vidaProjectRootName = vidaProject.vidaProjectRootName;
vidaProjectRootFolder = vidaProject.vidaProjectRootFolder;
rczMin = vidaProject.globalParameters.rczMin;
rczMax = vidaProject.globalParameters.rczMax;
voxelSizeInMicrons = vidaProject.globalParameters.voxelSizeInMicrons;
%-----
delayMultiplier = vidaProject.experimentTemplate.basicParameters.delayMultiplier;
%-----
%Convert microns to voxels
nominalBlockStepSizeRC = round(nominalBlockStepSizeRCInMicrons ./ voxelSizeInMicrons);
nominalBlockStepSizeZ = round(nominalBlockStepSizeZInMicrons ./ voxelSizeInMicrons);
ySearchSize = round(ySearchSizeInMicrons ./ voxelSizeInMicrons);
xSearchSize = round(xSearchSizeInMicrons ./ voxelSizeInMicrons);
zSearchSize = round(zSearchSizeInMicrons ./ voxelSizeInMicrons);
yxzSearchSize = [ySearchSize,xSearchSize,zSearchSize];
%-----
%Add converted or global variables to registrationParameters for ease of
%passing information to subfunction calls
registrationParameters.vidaProjectRootName = vidaProjectRootName;
registrationParameters.vidaProjectRootFolder = vidaProjectRootFolder;
registrationParameters.nominalBlockStepSizeRC = nominalBlockStepSizeRC;
registrationParameters.nominalBlockStepSizeZ = nominalBlockStepSizeZ;
registrationParameters.yxzSearchSize = yxzSearchSize;
%-----
registrationParameters.maskFileToUseForRegistration = 'VascEnhancedMask';%noteToSelf
%-----
%-----

numImageStacks = size(imageStackFolderNameList);
if numImageStacks == 1,
    registrationParameters.fullStackSize = fullStackSize;
    registrationParameters.blockRegistration = [1 1 1];
    registrationParameters.numBlocksRCZ = [1 1 1];
    registrationParameters.adjacentBlockPairList = [];
    vidaProject.registrationParameters = registrationParameters;
    vidaProject.globalParameters.blockRegistration = [1 1 1];
    vidaProject.globalParameters.RCZList = [1 1 1];
    vidaProject.registrationParameters.RCZList = [1 1 1];
    vidaProject.globalParameters.adjacentBlockPairList = [];
    outProject = vidaProject;
    return% skip the rest of the function
end%if numImageStacks == 1,


%% SET BLOCK CONFIGURATION (CONSTANT STRUCT)

numRBlocks = rczMax(1) - rczMin(1) +1;
numCBlocks = rczMax(2) - rczMin(2) +1;
numZBlocks = rczMax(3) - rczMin(3) +1;
%-----
registrationParameters.nblks1 = numRBlocks;
registrationParameters.nblks2 = numCBlocks;
registrationParameters.nblks3 = numZBlocks;
registrationParameters.firstBlkRow = rczMin(1);
registrationParameters.firstBlkCol = rczMin(2);
registrationParameters.firstBlkZpos = rczMin(3);

%%
%BUILD STRUCTURE
    registrationParameters = GeneratePointerToBlocks(registrationParameters);
blockFileNameList = registrationParameters.blockFileNameList;
RCZList = registrationParameters.RCZList;
    registrationParameters = GetBlockAdjacency(registrationParameters);
adjacentBlockPairList = registrationParameters.adjacentBlockPairList;
    registrationParameters = GetBlockOffsets(registrationParameters);
adjacentBlockPairOffsetList = registrationParameters.adjacentBlockPairOffsetList;
blockCornerCoordList = registrationParameters.blockCornerCoordList;
    registrationParameters = VidaPrepareMOES(registrationParameters);

    
%%

vidaProject.experimentTemplate.registrationParameters = registrationParameters;
saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save(vidaProjectFileName,'vidaProject',scs);
    
    
    
%-----
Blocks = registrationParameters.Blocks;
BlocksNominal = registrationParameters.BlocksNominal;
BlockConfig = registrationParameters.BlockConfig;
Offsets = registrationParameters.Offsets;
%-----
MOESParameters.Blocks = Blocks;
MOESParameters.BlockConfig = BlockConfig;
MOESParameters.Offsets = Offsets;
MOESParameters.yxzSearchSize = yxzSearchSize;


%-----
numAdjacentPairs = size(adjacentBlockPairList,1);
adjacentPairRegistrationResultsFilePrefix = fullfile(adjacentPairRegistrationDataFolder,[vidaProjectRootName,'-RegData']);

        
%%
%Begin Multi-thread processing of registration


%Initialize the check-out list if not already existent.
if ~isfield(vidaProject.multiThreadProgress,'adjacentPairProgressFlagList')
    vidaProject.multiThreadProgress.adjacentPairProgressFlagList = zeros(numAdjacentPairs,1);
end%if ~isfield(vidaProject.multiThreadProgress,'adjacentPairProgressFlagList')
adjacentPairProgressFlagList = vidaProject.multiThreadProgress.adjacentPairProgressFlagList;

%Search for previously created registrationDataFiles in adjacentPairRegistrationDataFolder
%   and populate the adjacentPairProgressFlagList appropriately
for adjacentPairIter = 1:numAdjacentPairs,
    currentRegistrationDataFileName = [adjacentPairRegistrationResultsFilePrefix,num2str(adjacentPairIter,'%06d'),'.mat'];
    if exist(currentRegistrationDataFileName,'file'),
        adjacentPairProgressFlagList(adjacentPairIter) = 2;
    else%if exist(currentRegistrationDataFileName,'file'),
        adjacentPairProgressFlagList(adjacentPairIter) = 0;
    end%if-else exist(currentRegistrationDataFileName,'file'),
end%for adjacentPairIter = 1:numAdjacentPairs,

        

%-----
%STAGE 7 : Assign an individual adjacent pair, and run processing.
%Look in request folder : find all delivery files
%   Log results from each delivery file and delete it.
%Search for all request files
%For each request file, write a response file, assigning an adjacentPair
%   Delete the request file
%Take an adjacentPair and process it.
allAdjacentPairsAssignedFlag = 0;
allAdjacentPairsCompletedFlag = 0;
waitForChildCounter = 0;
waitForChildMaxLoops = round(50 * delayMultiplier + 1);%note that each loop is 3x of waitForChildDelayInSec
waitForChildDelayInSec = round(30 * delayMultiplier + 1);%seconds

%%-----
%Initial Delay to allow child requests to come in
global skipDelayButtonFlag
skipDelayButtonFlag = 0;
delayFigureHandle = VidaGuiSkipDelay({'VIDA parent - allow time for child requests'});
disp('Pausing momentarily for child requests');
for delayLoop = 1:waitForChildDelayInSec,
    pause(1);
    if skipDelayButtonFlag == 1, break; end
end%for delayLoop = 1:waitForChildDelayInSec,
if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
    close(delayFigureHandle);
end%if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
pause(0.5);
%-----
while allAdjacentPairsCompletedFlag == 0,
    %Look in request folder for delivery files
    searchString = fullfile(multiThreadRequestFolder,'MultiThreadDelivery*.mat');
    myDir = dir(searchString);
    numDeliveryFiles = size(myDir,1);
    for deliveryIter = 1:numDeliveryFiles,
        currentDeliveryFileName = myDir(deliveryIter).name;
        fullDeliveryFileName = fullfile(multiThreadRequestFolder,currentDeliveryFileName);
        load(fullDeliveryFileName,'deliveryData');
        adjacentPairProgressFlagList(deliveryData.IDNum) = 2;
            %0=not,1=started(assigned),2=completed.
        pause(0.03);
        %Delete both the delivery file and the corresponding response file
        numberPos = regexp(currentDeliveryFileName,'\d');
        currentDeliveryID = currentDeliveryFileName(numberPos(1):numberPos(end));
        delete(fullDeliveryFileName);
        correspondingResponseFileName = fullfile(multiThreadRequestFolder,['MultiThreadResponse',currentDeliveryID,'.mat']);
        try
            load(correspondingResponseFileName,'multiThreadResponse');
            if  deliveryData.IDNum == multiThreadResponse.assignmentIndex,
                delete(correspondingResponseFileName);
            end%if  deliveryData.IDNum == multiThreadResponse.assignmentIndex,
        catch
        end%try-catch
        pause(0.03);
    end
    %-----
    %Check for unassigned stacks
    if isempty(find(adjacentPairProgressFlagList==0)),
        allAdjacentPairsAssignedFlag = 1;
        numCompleted = sum(adjacentPairProgressFlagList==2);
        if numCompleted == numAdjacentPairs,
            allAdjacentPairsCompletedFlag = 1;
        end%if numCompleted == numAdjacentPairs,
    end%if isempty(find(adjacentPairProgressFlagList==0)),
    %-----
    %Pick an assignment for parent process
    notAssignedList = find(adjacentPairProgressFlagList==0);
    if ~isempty(notAssignedList),
        parentAssignment = notAssignedList(1);
        adjacentPairProgressFlagList(parentAssignment) = 1;%indicates assigned
    else%if ~isempty(notAssignedList),
        parentAssignment = 0;
    end%if-else ~isempty(notAssignedList),
    %-----
    %Look for request files, and generate response files
    for requestLoopIter = 1:2,
        %loop twice to give child processes a chance to accept a response, and send the
        %next round request before starting
        searchString = fullfile(multiThreadRequestFolder,'MultiThreadRequest*.mat');
        myDir = dir(searchString);
        numRequestFiles = size(myDir,1);
        requesterIDList = [];
        for requestIter = 1:numRequestFiles,
            multiThreadResponse = [];
            multiThreadResponse.stageNum = 1;
            multiThreadResponse.stageString = 'adjacentPair';
            %-----
            currentRequestFileName = myDir(requestIter).name;
            numberPos = regexp(currentRequestFileName,'\d');
            currentRequesterID = currentRequestFileName(numberPos(1):numberPos(end));
            correspondingResponseFileName = fullfile(multiThreadRequestFolder,['MultiThreadResponse',currentRequesterID,'.mat']);
            requesterIDList{requestIter} = currentRequesterID;
            notAssignedList = find(adjacentPairProgressFlagList==0);
            if ~exist(correspondingResponseFileName),
                if isempty(notAssignedList),
                    allAdjacentPairsAssignedFlag = 1;
                    multiThreadResponse.assignmentIndex = 0;%indicates no assignment right now - wait.
                    multiThreadResponse.responseID = 0;
                    multiThreadResponse.assignmentString = '';
                else%if isempty(notAssignedList),
                    nextUnassignedIndex = notAssignedList(1);
                    multiThreadResponse.assignmentIndex = nextUnassignedIndex;
                    rand('twister', (sum(1000*clock)+(cputime*10000)));
                    multiThreadResponse.responseID = ceil(rand(1)*(999999998));%unique ID for this request response;
                    adjacentPairProgressFlagList(nextUnassignedIndex) = 1;
                    %indicates assigned
                    multiThreadResponse.assignmentString = '';
                end%if-else isempty(notAssignedList),
                %-----
                currentResponseFileName = fullfile(multiThreadRequestFolder,['MultiThreadResponse',currentRequesterID,'.mat']);
                currentRequestFullFileName = fullfile(multiThreadRequestFolder,['MultiThreadRequest',currentRequesterID,'.mat']);
                save(currentResponseFileName,'multiThreadResponse');
                pause(0.03);
                if exist(currentRequestFullFileName),
                    delete(currentRequestFullFileName);
                end%if exist(currentRequestFullFileName),
            end%if ~exist(correspondingResponseFileName),
        end%for requestIter = 1:numRequestFiles,
        %pause to allow time for child processes to queue up next-round request
        if requestLoopIter < 2,
            global skipDelayButtonFlag
            disp(['Waiting for child processor to queue next requests  : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            skipDelayButtonFlag = 0;
            delayFigureHandle = VidaGuiSkipDelay({'VIDA parent - allow time for child requests'});
            for delayLoop = 1:waitForChildDelayInSec,
                pause(1);
                if skipDelayButtonFlag == 1, break; end;
            end%for delayLoop = 1:waitForChildDelayInSec,
            if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
                close(delayFigureHandle);
            end%if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
            pause(0.5);
        end%if requestLoopIter <2
    end%for requestLoopIter = 1:2
    %-----
    %Recheck for unassigned stacks
    if isempty(find(adjacentPairProgressFlagList==0)),
        allAdjacentPairsAssignedFlag = 1;
        numCompleted = sum(adjacentPairProgressFlagList==2);
        if numCompleted == numAdjacentPairs,
            allAdjacentPairsCompletedFlag = 1;
        end%if numCompleted == numAdjacentPairs,
    end%if isempty(find(adjacentPairProgressFlagList==0)),
    %-----
    if parentAssignment == 0,
        %If not all completed, wait, and then recheck
        if allAdjacentPairsCompletedFlag == 0;
            waitForChildCounter = waitForChildCounter + 1;
            %%-----
            global skipDelayButtonFlag
            disp(['All ajacent pairs assigned - waiting for delivery : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
            skipDelayButtonFlag = 0;
            delayFigureHandle = VidaGuiSkipDelay({'Waiting on deliveries - skip to reassign image stacks'});
            for delayLoop = 1:waitForChildDelayInSec,
                pause(1);
                if skipDelayButtonFlag == 1, 
                    waitForChildCounter = waitForChildMaxLoops;
                    break;
                end%if skipDelayButtonFlag == 1, 
            end%for delayLoop = 1:waitForChildDelayInSec,
            if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
                close(delayFigureHandle);
            end%if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
            pause(0.5);
            %%-----
        end%if allAdjacentPairsCompletedFlag == 0;
    else%if parentAssignment == 0,
        %Take a stack for the parent process to run
        %-----
        %*******HERE IS WHERE YOU ACTUALLY PROCESS THE AJACENT PAIR*****
        %*******HERE IS WHERE YOU ACTUALLY PROCESS THE AJACENT PAIR*****
        %*******HERE IS WHERE YOU ACTUALLY PROCESS THE AJACENT PAIR*****
        fprintf('\n\n');
        disp(['Parent Processing Adjacent Pair # ',num2str(parentAssignment),' of ',num2str(numAdjacentPairs),' : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
        MOESParameters.currentAdjacentPairIndex = parentAssignment;
        multiThreadParameters.MOESParameters = MOESParameters;
        multiThreadParameters.registrationParameters = registrationParameters;
        multiThreadParameters.registrationDataFileName = [adjacentPairRegistrationResultsFilePrefix,num2str(parentAssignment,'%06d'),'.mat'];
            multiThreadParameters = VidaCalculateSingleMOES(multiThreadParameters);
        %-----
        adjacentPairProgressFlagList(parentAssignment) = 2;%indicates completed
    end%if-else parentAssignment == 0,
    %-----
    if waitForChildCounter >= waitForChildMaxLoops,
        %Assume a child processor crashed.  Un-assign all uncompleted stacks.
        adjacentPairProgressFlagList(adjacentPairProgressFlagList==1) = 0;
        waitForChildCounter = 0;
        disp(['Re-assigning uncompleted ajacent pairs : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
        pause(3);
        %Delete all existing request and response files
        searchString1 = fullfile(multiThreadRequestFolder,'MultiThreadRequest*.mat');
        myDir1 = dir(searchString1);
        searchString2 = fullfile(multiThreadRequestFolder,'MultiThreadResponse*.mat');
        myDir2 = dir(searchString2);
        pause(3);
        %-----
        numResidualRequestFiles = size(myDir1,1);
        for fileIter = 1:numResidualRequestFiles
            currentFileName = myDir1(fileIter).name;
            currentFullFileName = fullfile(multiThreadRequestFolder,currentFileName);
            delete(currentFullFileName);
        end%for fileIter = 1:numResidualRequestFiles
        %-----
        numResidualResponseFiles = size(myDir2,1);
        for fileIter = 1:numResidualResponseFiles
            currentFileName = myDir2(fileIter).name;
            currentFullFileName = fullfile(multiThreadRequestFolder,currentFileName);
            delete(currentFullFileName);
        end%for fileIter = 1:numResidualResponseFiles
        %-----
    end%if waitForChildCounter >= waitForChildMaxLoops,
    %Save VidaProject to file
    multiThreadProgress.adjacentPairProgressFlagList = adjacentPairProgressFlagList;
    vidaProject.multiThreadProgress = multiThreadProgress;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
end%while allAdjacentPairsCompletedFlag == 0,
%-----
%Wait for all child processes who are "waiting for a response" to recycle
%   their requests - and then reply to them with an "all done" signal
global skipDelayButtonFlag
disp(['Waiting for final round of child processor requests  : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
skipDelayButtonFlag = 0;
delayFigureHandle = VidaGuiSkipDelay({'VIDA parent - allow time for child requests'});
for delayLoop = 1:waitForChildDelayInSec,
    pause(1);
    if skipDelayButtonFlag == 1, break; end;
end%for delayLoop = 1:waitForChildDelayInSec,
if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
    close(delayFigureHandle);
end%if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
pause(0.5);
%-----
disp(['Sending All-done signal  : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
searchString = fullfile(multiThreadRequestFolder,'MultiThreadRequest*.mat');
myDir = dir(searchString);
numRequestFiles = size(myDir,1);
disp([num2str(numRequestFiles),' final request files detected']);
requesterIDList = [];
for requestIter = 1:numRequestFiles,
    multiThreadResponse = [];
    multiThreadResponse.responseID = 0;
    multiThreadResponse.stageNum = 1;
    multiThreadResponse.stageString = 'adjacentPair';
    %-----
    currentRequestFileName = myDir(requestIter).name;
    numberPos = regexp(currentRequestFileName,'\d');
    currentRequesterID = currentRequestFileName(numberPos(1):numberPos(end));
    requesterIDList{requestIter} = currentRequesterID;
    correspondingResponseFileName = fullfile(multiThreadRequestFolder,['MultiThreadResponse',currentRequesterID,'.mat']);
    multiThreadResponse.assignmentIndex = -1;%indicates stage completed - all done.
    multiThreadResponse.assignmentString = '';
    %-----
    currentResponseFullFileName = fullfile(multiThreadRequestFolder,['MultiThreadResponse',currentRequesterID,'.mat']);
    currentRequestFullFileName = fullfile(multiThreadRequestFolder,['MultiThreadRequest',currentRequesterID,'.mat']);
    save(currentResponseFullFileName,'multiThreadResponse');
    pause(0.01);
    if exist(currentRequestFileName,'file'),
        delete(currentRequestFullFileName);
    end%if exist(currentRequestFileName,'file'),
end%for requestIter = 1:numRequestFiles,
%-----
%-----
%All Adjacent Pairs Completed.
%-----
%Clear out the request folder
%-----
global skipDelayButtonFlag
disp(['Allowing time for receipt of all-done signal  : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
skipDelayButtonFlag = 0;
delayFigureHandle = VidaGuiSkipDelay({'VIDA parent - allow time for receipt of all-done signal'});
for delayLoop = 1:waitForChildDelayInSec,
    pause(1);
    if skipDelayButtonFlag == 1, break; end;
end%for delayLoop = 1:waitForChildDelayInSec,
if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
    close(delayFigureHandle);
end%if delayLoop == waitForChildDelayInSec && ishandle(delayFigureHandle)
pause(0.5);
%%-----
disp(['Clearing request folder  : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
searchString = fullfile(multiThreadRequestFolder,'MultiThread*.mat');
myDir = dir(searchString);
numResidualFiles = size(myDir,1);
for residualIter = 1:numResidualFiles,
    currentResidualFileName = myDir(residualIter).name;
    fullResidualFileName = fullfile(multiThreadRequestFolder,currentResidualFileName);
    delete(fullResidualFileName);
end%for residualIter = 1:numResidualFiles,
%-----
%All ajacent pairs Completed.
disp(['All ajacent pairs completed : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
multiThreadProgress.adjacentPairProgressFlagList = adjacentPairProgressFlagList;
vidaProject.multiThreadProgress = multiThreadProgress;
%-----
vidaProject.multiThreadProgress.registeringFlag = 2;
saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
save(vidaProjectFileName,'vidaProject',scs);


%%

%Collect all relevant offset energy information
registrationDataFileNameList = [];
for adjacentPairIter = 1:numAdjacentPairs,
    currentRegistrationDataFileName = [adjacentPairRegistrationResultsFilePrefix,num2str(adjacentPairIter,'%06d'),'.mat'];
    load(currentRegistrationDataFileName,'adjacentPairEnergy');
    currentMarginalOffsetEnergies = [];
    currentMarginalOffsetEnergies.registrationDataFileName = currentRegistrationDataFileName;
    currentMarginalOffsetEnergies.blocks = adjacentPairEnergy.blocks;
    currentMarginalOffsetEnergies.dims = adjacentPairEnergy.dims;
    currentMarginalOffsetEnergies.dim_names = adjacentPairEnergy.dim_names;
    currentMarginalOffsetEnergies.offs_to_MOESloc_x1x2 = adjacentPairEnergy.offs_to_MOESloc_x1x2;
    currentMarginalOffsetEnergies.offs_to_MOESloc_x1x3 = adjacentPairEnergy.offs_to_MOESloc_x1x3;
    currentMarginalOffsetEnergies.offs_to_MOESloc_x2x3 = adjacentPairEnergy.offs_to_MOESloc_x2x3;
    currentMarginalOffsetEnergies.ZeroOffsetLoc3D = adjacentPairEnergy.ZeroOffsetLoc3D;
    currentMarginalOffsetEnergies.Offset3DEnergySize = adjacentPairEnergy.Offset3DEnergySize;
    currentMarginalOffsetEnergies.Offset3DEnergyMinPos = adjacentPairEnergy.Offset3DEnergyMinPos;
    MarginalOffsetEnergies(adjacentPairIter) = currentMarginalOffsetEnergies;
end
      


%% STAGE XXX: USE MOES MINIMA TO ESTIMATE INITIAL BLOCK LOCATIONS
 
[Blocks,A,r,dML] = InitializeBlockCornersViaMOES(Blocks,BlockConfig,Offsets,MarginalOffsetEnergies);

% compute the residual deltas from the least squares solution
dLS = A*[Blocks.corner_coords]' - r;

BlockConfig.cross3D = Make3DCross;
[a1,a2,a3] = ind2sub(size(BlockConfig.cross3D),find(BlockConfig.cross3D));
a1=a1-2;a2=a2-2;a3=a3-2;
a1=[a1;0];a2=[a2;0];a3=[a3;0];
OffsetSearch = [a1(:),a2(:),a3(:)];

disp('Skipping Nosearchdims step in MaximizeProjectedBlockCorrelationSum.m line203');
%noteToSelf
% % % % nosearchdims = BlockConfig.nblks == 1;
% % % % OffsetSearch((abs(OffsetSearch(:,nosearchdims)) ~= 0),:) = [];  % remove any search dimension that only has one block layer

Blocks0 = Blocks;   % Set the initial configuration of Blocks to be the least squares solution
num_it = 12;

disp('Skipping Block Descent');
disp('Skipping Block Descent');
disp('Skipping Block Descent');
disp('Skipping Block Descent');
disp('Skipping Block Descent');
disp('Skipping Block Descent');
disp('Skipping Block Descent');
disp('Skipping Block Descent');
disp('Skipping Block Descent');
num_it = 0;%noteToSelf

%% RUN DESCENT FROM BlocksPert num_ensembles TIMES, TAKING THE BEST RESULT (by E)
num_ensembles = 10;

num_ensembles = 0;%noteTo Self


if num_ensembles >0,   
    %A_trials = cell(1,num_ensembles);
    CornerEnsembles = cell(1,num_ensembles);
    E_iEA = zeros(1,num_ensembles);
    tic
    E_init = ComputeTotalEnergy(Blocks0,Offsets,MarginalOffsetEnergies);    % Contains the energy of the initial configuration
    for iEA = 1:num_ensembles
        disp(['search ',num2str(iEA)]);
        %[Blocks,AO,TotalMotion,E_it] = BlockDescentByRandomBlockLocalOffsetSearch(OffsetSearch,Blocks0,Offsets,MarginalOffsetEnergies,num_it);
        [Blocks] = BlockDescentByRandomBlockLocalOffsetSearch(OffsetSearch,Blocks0,Offsets,MarginalOffsetEnergies,num_it);
        [E_iEA(iEA)] = ComputeTotalEnergy(Blocks,Offsets,MarginalOffsetEnergies);
        CornerEnsembles{iEA} = reshape([Blocks.corner_coords],3,prod(BlockConfig.nblks))';
    end
    toc

    % Get the lowest energy config of all the ensemble trials; store that config in Blocks
    [minE,minE_configi] = min(E_iEA);
    best_corner_config = CornerEnsembles{minE_configi};
    for iB = 1:numel(Blocks); Blocks(iB).corner_coords = best_corner_config(iB,:); end
end%if num_ensembles >0,
    
    
% compute the residual deltas from the least squares solution
dLS_Search = A*[Blocks.corner_coords]' - r;

% Set the returned datastructure
estimatedRegistration.BlocksNominal = BlocksNominal;
estimatedRegistration.Blocks0 = Blocks0;
estimatedRegistration.Blocks = Blocks;
estimatedRegistration.BlockConfig = BlockConfig;
estimatedRegistration.Offsets = Offsets;
estimatedRegistration.MarginalOffsetEnergies = MarginalOffsetEnergies;
estimatedRegistration.dML = dML;
estimatedRegistration.dLS = dLS;
estimatedRegistration.dLS_Search = dLS_Search;



%StepThroughMarginalOffsetEnergies(estimatedRegistration.MarginalOffsetEnergies);

% compute the total motion from the least squares answer to the final
% answer after energy minimization:
TotalMotion_LS_EMin = sum(abs([estimatedRegistration.Blocks.corner_coords] - [estimatedRegistration.Blocks0.corner_coords]));
fprintf('After the least squares solution, the entire set of blocks moved a total of %d voxels to the min energy solution found in the search.\n',TotalMotion_LS_EMin);

%ShowAllPairwiseRegisteredBlocks(estimatedRegistration.BlocksNominal,estimatedRegistration.Offsets);
%ShowAllPairwiseRegisteredBlocks(estimatedRegistration.Blocks0,estimatedRegistration.Offsets);
% ShowAllPairwiseRegisteredBlocks(estimatedRegistration.Blocks,estimatedRegistration.Offsets);

% outputParameters.estimatedRegistration = estimatedRegistration;


blockRegistration = reshape([estimatedRegistration.Blocks.corner_coords],[3,length(blockFileNameList)])';
numBlocksRCZ = [numRBlocks,numCBlocks,numZBlocks];

registrationParameters.fullStackSize = fullStackSize;
registrationParameters.BlocksNominal = BlocksNominal;
registrationParameters.Blocks0 = Blocks0;
registrationParameters.Blocks = Blocks;
registrationParameters.blockRegistration = blockRegistration;
registrationParameters.numBlocksRCZ = numBlocksRCZ;
vidaProject.registrationParameters = registrationParameters;
vidaProject.globalParameters.blockRegistration = blockRegistration;
vidaProject.globalParameters.RCZList = RCZList;
vidaProject.globalParameters.adjacentBlockPairList = adjacentBlockPairList;
outProject = vidaProject;




























