function vectorProject = VidaRegisteredPermuteParentSL(vidaProject)

vidaProject = vidaProject;
vidaProjectFileName = vidaProject.vidaProjectFileName;
vidaProjectRootName = vidaProject.vidaProjectRootName;
vidaProjectRootFolder = vidaProject.vidaProjectRootFolder;
globalParameters = vidaProject.globalParameters;
registrationParameters = vidaProject.registrationParameters;
strandifyParameters = vidaProject.strandifyParameters;
thresholdRelaxationParameters = vidaProject.thresholdRelaxationParameters;
imageStackFolderNameList = vidaProject.imageStackFolderNameList;
registeredFilesFolder = globalParameters.registeredFilesFolder;
voxelSizeInMicrons = globalParameters.voxelSizeInMicrons;
%-----
permuteSequence = registrationParameters.permuteSequence;
generatePermutedTifFlagList = registrationParameters.generatePermutedTifFlagList;
%-----





fileToRotateList{1} = vidaProject.globalParameters.registeredVascGrayscaleFileName;
variableNameList{1} = 'registeredVascGrayscale';
fileToRotateList{2} = vidaProject.globalParameters.registeredVascEnhancedGrayscaleFileName;
variableNameList{2} = 'registeredVascEnhancedGrayscale';
fileToRotateList{3} = vidaProject.globalParameters.registeredVascRawMaskFileName;
variableNameList{3} = 'registeredVascRawMask';
fileToRotateList{4} = vidaProject.globalParameters.registeredVascEnhancedMaskFileName;
variableNameList{4} = 'registeredVascEnhancedMask';
fileToRotateList{5} = vidaProject.globalParameters.registeredVascRadiusMapFileName;
variableNameList{5} = 'vascRadiusMap';
fileToRotateList{6} = vidaProject.globalParameters.registeredVascReconstructedMaskFileName;
variableNameList{6} = 'vascReconstructedMask';






%-----
%Create a subFolder for vectorProject 
vectorProjectFolder = fullfile(vidaProjectRootFolder,'VectorProjectFolder');
vidaProject.globalParameters.vectorProjectFolder = vectorProjectFolder;
if ~exist(vectorProjectFolder,'dir'),
    mkdir(vectorProjectFolder);
end
%-----
%Create a subFolder for permuted volumes
permutedFilesFolder = fullfile(vectorProjectFolder,'PermutedFilesFolder');
vidaProject.globalParameters.permutedFilesFolder = permutedFilesFolder;
if ~exist(permutedFilesFolder,'dir'),
    mkdir(permutedFilesFolder);
end

%-----
vectorProject.note = 'Permuted Coord. System';
vectorProject.permuteSequence = permuteSequence;
%-----
vectorProject.vectorProjectRootName = vidaProject.vidaProjectRootName;
%-----
vectorProject.vectorProjectFolder = vidaProject.globalParameters.vectorProjectFolder;
%-----
vectorProject.rootCodeFolder = vidaProject.rootCodeFolder;
%-----
origMaskSize = vidaProject.globalParameters.vectorizedStructure.maskSize;
vectorProject.maskSize = origMaskSize(permuteSequence);
%-----
vectorProject.voxelSizeInMicrons = vidaProject.globalParameters.voxelSizeInMicrons;
%-----
vectorProject.progressFlags.permuteCoordCompleted = 0;
vectorProject.progressFlags.thresholdRelaxationCompleted = 0;
vectorProject.progressFlags.addTRBridgesCompleted = 0;
%-----
origVectorizedStructure = vidaProject.globalParameters.vectorizedStructure;
origMaskSize = origVectorizedStructure.maskSize;
origVertices = origVectorizedStructure.Vertices;
origStrands = origVectorizedStructure.Strands;
tempVertices = origVertices;
tempVertices.AllVerts(:,1) = origVertices.AllVerts(:,permuteSequence(1));
tempVertices.AllVerts(:,2) = origVertices.AllVerts(:,permuteSequence(2));
tempVertices.AllVerts(:,3) = origVertices.AllVerts(:,permuteSequence(3));
tempVertices.AllLabels = zeros(size(origVertices.AllRadii),'int16') + 111; %See Label Notes Below
tempVertices.AllConfidences = zeros(size(origVertices.AllRadii),'single') + Inf;
% % tempVertices.AllNotes = cell(size(origVertices.AllRadii));
vectorProject.vectorizedStructure.note = 'Permuted Coord. System';
vectorProject.vectorizedStructure.maskSize = origMaskSize(permuteSequence);
vectorProject.vectorizedStructure.Vertices = tempVertices;
vectorProject.vectorizedStructure.Strands = origStrands;
%-----
%-----
vectorProject.thresholdRelaxationParameters = thresholdRelaxationParameters;
%-----
vectorProject.strandifyParameters = vidaProject.strandifyParameters;

clear vidaProject globalParameters origVectorizedStructure origCCL 
clear tempCCL tempVertices registrationParameters origVertices origStrands


numFilesToRotate = numel(fileToRotateList);
volumeFileNames = [];
for fileIter = 1:numFilesToRotate,
    currentPathFileName = fileToRotateList{fileIter};
    currentVariableName = variableNameList{fileIter};
    disp(['Permuting ',currentVariableName,' : ',datestr(now,'mm-dd-yy HH:MM.SS')]);
    fileSepPosList = findstr(currentPathFileName,filesep);
    currentFileName = currentPathFileName(fileSepPosList(end)+1:end);
    permutedFileName = regexprep(currentFileName,'-registered','-permuted');
    permutedPathFileName = fullfile(permutedFilesFolder,permutedFileName);
    permutedVariableName = regexprep(currentVariableName,'registered','permuted');
    volumeFileNames.(permutedVariableName) = permutedFileName; 
    if exist(permutedPathFileName,'file'),
        disp([permutedFileName,' already created.  Skipping....']);
    else%if exist(permutedPathFileName,'file'),
        if isempty(findstr(permutedVariableName,'permuted')),
            permutedVariableName = ['permuted',upper(permutedVariableName(1)),permutedVariableName(2:end)];
        end%if isempty(findstr(permutedVariableName,'permuted')),
        %-----
        try
            loadedVariable = load(currentPathFileName);
        catch
            disp(['Unable to locate : ',currentFileName]);
            disp(['Please manually locate the file']);
            [manualFileName,manualPath] = uigetfile({['*',currentFileName];'*.mat'},['Locate : ',currentFileName]);
            loadedVariable = load(fullfile(manualPath,manualFileName));
        end%try-catch
        
        eval(['tempVariable = loadedVariable.',currentVariableName,';']);
        clear loadedVariable;
        if isstruct(tempVariable),
            origMaskSize = tempVariable.maskSize;
            permutedMaskSize = origMaskSize(permuteSequence);
            [oi1,oi2,oi3] = ind2sub(origMaskSize,tempVariable.pixelList);
            radiusFlag = isfield(tempVariable,'radiusList');
            if radiusFlag == 1;
                holderStruct.radiusList = tempVariable.radiusList;
            end%if radiusFlag == 1;
            clear tempVariable
            oic = [oi1,oi2,oi3];
            clear oi1 oi2 oi3
            ps = permuteSequence;
            holderStruct.pixelList = sub2ind(permutedMaskSize,oic(:,ps(1)),oic(:,ps(2)),oic(:,ps(3)));
            clear oic
            holderStruct.maskSize = permutedMaskSize;
            eval([permutedVariableName,' = holderStruct;']);
            clear holderStruct
        else%if isstruct(tempVariable),
            %Data is stored as a simple matrix
            eval([permutedVariableName,' = permute(tempVariable,[',num2str(permuteSequence),']);']);
            clear tempVariable;
        end%if-else isstruct(tempVariable),
        %-----
        saveSize = (getfield(whos(permutedVariableName),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(permutedPathFileName,permutedVariableName,scs);
        %-----
        if generatePermutedTifFlagList(fileIter) == 1,
            eval(['tempVariable = ',permutedVariableName,';']);
            clear(permutedVariableName)
            dotPos = findstr(permutedFileName,'.');
            if isempty(dotPos),
                tifFileName = [permutedFileName,'.tif'];
            else%if isempty(dotPos),
                tifFileName = [permutedFileName(1:dotPos),'tif'];
            end%if-else isempty(dotPos),
            tifPathFileName = fullfile(permutedFilesFolder,tifFileName);
            if isstruct(tempVariable),
                myMask = false(tempVariable.maskSize);
                myMask(tempVariable.pixelList) = 1;
                maketiff(myMask,tifPathFileName);
                clear myMask
            else%if isstruct(tempVariable),
                maketiff(tempVariable,tifPathFileName);
            end%if-else isstruct(tempVariable),
        end%if generatePermutedTifFlagList(fileIter) == 1,
        clear('tempVariable');
    end%if-else exist(permutedPathFileName,'file'),
    clear(permutedVariableName)
end%for fileIter = 1:numFilesToRotate,
vectorProject.progressFlags.permuteCoordCompleted = 1;   
vectorProject.volumeFileNames = volumeFileNames;
    
% % vectorProjectFileName = fullfile(vectorProjectFolder,[vectorProject.vectorProjectRootName,'-VectorProject.mat']);
% % saveSize = (getfield(whos('vectorProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
% % save(vectorProjectFileName,'vectorProject',scs);
% %        








%Label Structure
%100's Labels : Topology   : 100=Unlabeled, 200=Temporary, 300=Surface, 400=Plunging, 500=Connecting
%10's Labels  : Physiology :  10=Unlabeled,  20=Temporary,  30=Supply ,  40=Draining,  50=Microvessel
%1's Labels   : WhenAdded  :   1=Original ,   2=TR Added ,   3=Manual




