function outProject = VidaRegisteredProcessCenterlineChildSL(inProject)

vidaProject = inProject;
vidaProjectFileName = vidaProject.vidaProjectFileName;
vidaProjectRootName = vidaProject.vidaProjectRootName;
vidaProjectRootFolder = vidaProject.vidaProjectRootFolder;
globalParameters = vidaProject.globalParameters;
registrationParameters = vidaProject.registrationParameters;
strandifyParameters = vidaProject.strandifyParameters;
registeredFilesFolder = globalParameters.registeredFilesFolder;
voxelSizeInMicrons = globalParameters.voxelSizeInMicrons;
remoteVidaProjectRootFolder = vidaProject.remoteVidaProjectRootFolder;
%-----
registrationParameters.vidaProjectRootFolder = vidaProjectRootFolder;
registrationParameters.voxelSizeInMicrons = voxelSizeInMicrons;
%-----
%Convert vidaProjectFileName to remote folder
[origPath,fileName,myExt] = fileparts(vidaProjectFileName);
vidaProjectFileName = fullfile(remoteVidaProjectRootFolder,[fileName,myExt]);

%-----
%STAGE 2: Wait until Parent process begins regSubBlockProcessing, load updated VidaProject
disp('Pausing to check parent process status : registered subBlock processing');
parentRegSubBlockProcessingStartedFlag = 0;
while parentRegSubBlockProcessingStartedFlag == 0,
    try
        load(vidaProjectFileName,'vidaProject');
        check = vidaProject.multiThreadProgress.subBlockProcessingFlag;
    catch
        disp('Read error - will try again momentarily');
        pause(5);
    end%try-catch
    %---
    if vidaProject.multiThreadProgress.subBlockProcessingFlag > 0,
        parentRegSubBlockProcessingStartedFlag = 1;
        pause(10);
    end%if vidaProject.multiThreadProgress.subBlockProcessingFlag > 0,
end%while parentRegSubBlockProcessingStartedFlag == 0,        
originalVidaProjectRootFolder = vidaProject.vidaProjectRootFolder;


%-----
%STAGE 3: MODIFY LOCAL PARAMETERS. 
%Modify the necessary local VidaProject parameters to reflect remote filename on another computer
multiThreadRequestFolder = fullfile(remoteVidaProjectRootFolder,'MultiThreadRequestFiles');
registeredFilesFolder = fullfile(remoteVidaProjectRootFolder,'RegisteredFiles');
vidaProject.originalVidaProjectRootFolder = originalVidaProjectRootFolder;
vidaProject.remoteVidaProjectRootFolder = remoteVidaProjectRootFolder;
vidaProject.vidaProjectRootFolder = remoteVidaProjectRootFolder;
vidaProject.experimentTemplate.globalParameters.vidaProjectRootFolder = remoteVidaProjectRootFolder;
vidaProject.currentExperiment.globalParameters.vidaProjectRootFolder = remoteVidaProjectRootFolder;
vidaProject.multiThreadProgress.multiThreadRequestFolder = multiThreadRequestFolder;
vidaProject.globalParameters.registeredFilesFolder = registeredFilesFolder;
%-----
regCenterlineFilesFolder = fullfile(registeredFilesFolder,'CenterlineFiles');
regVascRadiusFilesFolder = fullfile(registeredFilesFolder,'VascRadiusFiles');
regCenterlineBlockFilePrefix = fullfile(regCenterlineFilesFolder,'RegVascBlockCenterlineMask');
regVascRadiusBlockFilePrefix = fullfile(regVascRadiusFilesFolder,'RegVascBlockRadiusMap');
vidaProject.registeredExperiment.globalParameters.regCenterlineFilesFolder = regCenterlineFilesFolder;
vidaProject.registeredExperiment.globalParameters.regVascRadiusFilesFolder = regVascRadiusFilesFolder;
vidaProject.registeredExperiment.globalParameters.regCenterlineBlockFilePrefix = regCenterlineBlockFilePrefix;
vidaProject.registeredExperiment.globalParameters.regVascRadiusBlockFilePrefix = regVascRadiusBlockFilePrefix;
%-----
pathLength = size(originalVidaProjectRootFolder,2);
vascBlockEnhancedMaskFileNameList = vidaProject.registeredExperiment.globalParameters.vascBlockEnhancedMaskFileNameList;
vascBlockRawMaskFileNameList = vidaProject.registeredExperiment.globalParameters.vascBlockRawMaskFileNameList;
vascBlockFileNameList = vidaProject.registeredExperiment.globalParameters.vascBlockFileNameList;
%---
blockGrid = vidaProject.registeredExperiment.globalParameters.blockGrid;
numSubBlocks = size(blockGrid,2);
for blockIter = 1:numSubBlocks,
    currentBlockName = vascBlockEnhancedMaskFileNameList{blockIter};
    newBlockName = [remoteVidaProjectRootFolder,currentBlockName(pathLength+1:end)];
    vidaProject.registeredExperiment.globalParameters.vascBlockEnhancedMaskFileNameList{blockIter} = newBlockName;
    %---
    currentBlockName = vascBlockRawMaskFileNameList{blockIter};
    newBlockName = [remoteVidaProjectRootFolder,currentBlockName(pathLength+1:end)];
    vidaProject.registeredExperiment.globalParameters.vascBlockRawMaskFileNameList{blockIter} = newBlockName;
    %---
    currentBlockName = vascBlockFileNameList{blockIter};
    newBlockName = [remoteVidaProjectRootFolder,currentBlockName(pathLength+1:end)];
    vidaProject.registeredExperiment.globalParameters.vascBlockFileNameList{blockIter} = newBlockName;
end%for blockIter = 1:numSubBlocks,

%-----
%-----
%STAGE 4: Request Subblocks from parent process.
%   Take an unassigned sublblock and run centerline and radius
if vidaProject.multiThreadProgress.subBlockProcessingFlag == 2,
    fprintf('\nProcessing of registered subBlocks previously completed\n\n');
else%if vidaProject.multiThreadProgress.subBlockProcessingFlag == 2,  
    fprintf('\nBegin Processing of registered subBlocks\n\n');
    vidaProject.multiThreadProgress.subBlockProcessingFlag = 1;
        vidaProject = VidaMultiThreadRegSubBlockChildSL(vidaProject);
    vidaProject.multiThreadProgress.subBlockProcessingFlag = 2;
end%if vidaProject.multiThreadProgress.subBlockProcessingFlag == 2,




disp([' ';' ';' ']);
disp(['Completed Child-Processing of Registered Mask : ',datestr(now)]);
disp([' ';' ';' ']);
%----


outProject = vidaProject;
