function outProject = VidaRegisteredProcessCenterlineParentSL(inProject)

vidaProject = inProject;
vidaProjectFileName = vidaProject.vidaProjectFileName;
vidaProjectRootName = vidaProject.vidaProjectRootName;
vidaProjectRootFolder = vidaProject.vidaProjectRootFolder;
globalParameters = vidaProject.globalParameters;
registrationParameters = vidaProject.registrationParameters;
strandifyParameters = vidaProject.strandifyParameters;
imageStackFolderNameList = vidaProject.imageStackFolderNameList;
registeredFilesFolder = globalParameters.registeredFilesFolder;
voxelSizeInMicrons = globalParameters.voxelSizeInMicrons;
channelFlags = vidaProject.channelFlags;
cellChannelFlag = isfield(channelFlags,'cell') && channelFlags.cell == 1;
neunChannelFlag = isfield(channelFlags,'neun') && channelFlags.neun == 1;
vascChannelFlag = isfield(channelFlags,'vasc') && channelFlags.vasc == 1;
%-----




%STAGE 2: Subblock the registered files

fprintf('\n\n\n\n\n');
disp(['Start Processing of Registered Mask : ',datestr(now,'mm-dd-yy,HH:MM.SS')]);
fprintf('\n\n');
currentImageStackFolderName = vidaProject.imageStackFolderNameList{end};
%-----
registeredExperiment = vidaProject.experimentTemplate;
vidaProjectRootFolder = vidaProject.vidaProjectRootFolder;
vidaProjectRootName = vidaProject.vidaProjectRootName;
rootCodeFolder = registeredExperiment.codeFolders.rootCodeFolder;
registeredVascRawMaskFileName = vidaProject.globalParameters.registeredVascRawMaskFileName;
registeredVascEnhancedMaskFileName = vidaProject.globalParameters.registeredVascEnhancedMaskFileName;
registeredVascGrayscaleFileName = vidaProject.globalParameters.registeredVascGrayscaleFileName;
registeredStackSize = vidaProject.globalParameters.registeredStackSize;
subVolumeOverlapSizeInMicrons = registeredExperiment.subBlockParameters.subVolumeOverlapSizeInMicrons;
subVolumeOverlapSize = subVolumeOverlapSizeInMicrons./voxelSizeInMicrons;
%-----
%Set up internal folder and file names
fileRootName = [vidaProjectRootName,'-Reg'];
rootDataFolder = registeredFilesFolder;
originalDataFolder = rootDataFolder;
processedDataFolder = rootDataFolder;
intermediateResultsFolder = rootDataFolder;
registeredExperiment.globalParameters.fileRootName = fileRootName;
registeredExperiment.globalParameters.rootDataFolder = rootDataFolder;
registeredExperiment.globalParameters.originalDataFolder = originalDataFolder;
registeredExperiment.globalParameters.processedDataFolder = processedDataFolder;
registeredExperiment.globalParameters.intermediateResultsFolder = intermediateResultsFolder;
registeredExperimentFileName = fullfile(registeredExperiment.globalParameters.processedDataFolder,[fileRootName,'-Experiment.mat']);
registeredExperiment.globalParameters.registeredExperimentFileName = registeredExperimentFileName;
%-----
registeredExperiment.globalParameters.vidaProjectRootFolder = vidaProjectRootFolder;
registeredExperiment.globalParameters.vidaProjectRootName = vidaProjectRootName;
registeredExperiment.globalParameters.fileRootName = fileRootName;
registeredExperiment.globalParameters.registeredExperimentFileName = registeredExperimentFileName;
registeredExperiment.globalParameters.rootDataFolder = rootDataFolder;
registeredExperiment.globalParameters.originalDataFolder = originalDataFolder;
registeredExperiment.globalParameters.processedDataFolder = processedDataFolder;
registeredExperiment.globalParameters.intermediateResultsFolder = intermediateResultsFolder;
registeredExperiment.globalParameters.fullStackSize = registeredStackSize;
registeredExperiment.globalParameters.imageStackFolderNameList = imageStackFolderNameList;
%-----
registeredExperiment.globalParameters.registeredVascGrayscaleFileName = registeredVascGrayscaleFileName;
registeredExperiment.globalParameters.registeredVascRawMaskFileName = registeredVascRawMaskFileName;
registeredExperiment.globalParameters.vascEnhancedMaskFileName = registeredVascEnhancedMaskFileName;
registeredExperiment.globalParameters.channels.vascChannelString = 'FITC';
%----------
registeredExperiment.globalParameters.lastRerunStartDateNum = now()-5000;
registeredExperiment.globalParameters.stackedFileNameList = {registeredVascGrayscaleFileName,registeredVascRawMaskFileName,registeredVascEnhancedMaskFileName,};
registeredExperiment.registrationParameters = registrationParameters;
registeredExperiment.progressFlags.initialized = 1;
registeredExperiment.registrationParameters = vidaProject.registrationParameters;
%----------
if exist(registeredExperimentFileName),
    load(registeredExperimentFileName,'registeredExperiment');
end%if exist(registeredExperimentFileName),

%----------
%SET UP AND RUN SUBBLOCKING
vidaProject.multiThreadProgress.subBlockingFlag = 1;
stageParameters.regVascSubBlock.currentStageString = 'RegisteredSubBlock';
stageParameters.regVascSubBlock.dataStructureVariableName = 'registeredExperiment';
stageParameters.regVascSubBlock.dataStructureFileName = registeredExperimentFileName;
stageParameters.regVascSubBlock.mainFunctionCallString = 'VidaSubBlockRegSL';
%%------
disp(['RootDataFolder = ',rootDataFolder]);
disp(['FileRootName = ',fileRootName]);
[registeredExperiment] = VidaRunSaveOrSkip3(registeredExperiment,stageParameters.regVascSubBlock);
blockGrid = registeredExperiment.globalParameters.blockGrid;
%-----
numSubBlocks = size(blockGrid,2);
% % % % if numSubBlocks == 1,
% % % %     keyboard%noteToSelf
% % % %     outputParameters = vidaProject;
% % % %     return% skip the rest of the function
% % % % end%if numImageStacks == 1,
% % % % 



%STAGE 3: Prepare for multiThread processing of Centerline and
%   vasc radius of registered subblock
%-----
%Initialize the check-out list if not already existent.
if ~isfield(vidaProject.multiThreadProgress,'registeredVascProcessFlagList')
    vidaProject.multiThreadProgress.registeredVascProcessFlagList = zeros(numSubBlocks,1);
end%if ~isfield(vidaProject.multiThreadProgress,'registeredVascProcessFlagList')
registeredVascProcessFlagList = vidaProject.multiThreadProgress.registeredVascProcessFlagList;
%-----
regCenterlineFilesFolder = fullfile(registeredFilesFolder,'CenterlineFiles');
regVascRadiusFilesFolder = fullfile(registeredFilesFolder,'VascRadiusFiles');
regCenterlineBlockFilePrefix = fullfile(regCenterlineFilesFolder,'RegVascBlockCenterlineMask');
regVascRadiusBlockFilePrefix = fullfile(regVascRadiusFilesFolder,'RegVascBlockRadiusMap');
%-----
if ~exist(regCenterlineFilesFolder,'file');
    [mds,mdm,mdmid] = mkdir(registeredFilesFolder,'CenterlineFiles');
end%if ~exist(regCenterlineFilesFolder,'file');
%-----
if ~exist(regVascRadiusFilesFolder,'file');
    [mds,mdm,mdmid] = mkdir(registeredFilesFolder,'VascRadiusFiles');
end%if ~exist(regVascRadiusFilesFolder,'file');
%-----
%Search for previously created registrationDataFiles in adjacentPairRegistrationDataFolder
%   and populate the adjacentPairProgressFlagList appropriately
regCenterlineBlockFileNameList = {};
regVascRadiusBlockFileNameList = {};
for subBlockIter = 1:numSubBlocks,
    currentRegCenterlineBlockFileName = [regCenterlineBlockFilePrefix,num2str(subBlockIter),'.mat'];
    currentRegVascRadiusBlockFileName = [regVascRadiusBlockFilePrefix,num2str(subBlockIter),'.mat'];
    regCenterlineBlockFileNameList{subBlockIter} = currentRegCenterlineBlockFileName;
    regVascRadiusBlockFileNameList{subBlockIter} = currentRegVascRadiusBlockFileName;
    if exist(currentRegCenterlineBlockFileName,'file') && exist(currentRegVascRadiusBlockFileName,'file'),
        registeredVascProcessFlagList(subBlockIter) = 2;
    else%if exist(currentRegCenterlineBlockFileName,'file') && exist(currentRegRadiusBlockFileName,'file'),
        registeredVascProcessFlagList(subBlockIter) = 0;
    end%if-else exist(currentRegCenterlineBlockFileName,'file') && exist(currentRegRadiusBlockFileName,'file'),
end%for adjacentPairIter = 1:numAdjacentPairs,
vidaProject.multiThreadProgress.registeredVascProcessFlagList = registeredVascProcessFlagList;
%-----

%STAGE 4: Assign Subblocks to child-processors.
%   Take an unassigned subblock and run centerline and radius
registeredExperiment.globalParameters.regCenterlineFilesFolder = regCenterlineFilesFolder;
registeredExperiment.globalParameters.regVascRadiusFilesFolder = regVascRadiusFilesFolder;
registeredExperiment.globalParameters.regCenterlineBlockFilePrefix = regCenterlineBlockFilePrefix;
registeredExperiment.globalParameters.regVascRadiusBlockFilePrefix = regVascRadiusBlockFilePrefix;
vidaProject.registeredExperiment = registeredExperiment;
vidaProject.multiThreadProgress.registeredVascProcessFlagList = registeredVascProcessFlagList;
vidaProject.multiThreadProgress.subBlockingFlag = 2;
%-----
if vidaProject.multiThreadProgress.subBlockProcessingFlag == 2,
    fprintf('\nProcessing of registered subBlocks previously completed\n\n');
else%if vidaProject.multiThreadProgress.subBlockProcessingFlag == 2,  
    fprintf('\nBegin Processing of registered subBlocks\n\n');
    vidaProject.multiThreadProgress.subBlockProcessingFlag = 1;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
        vidaProject = VidaMultiThreadRegSubBlockParent(vidaProject);
    vidaProject.multiThreadProgress.subBlockProcessingFlag = 2;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
end%if vidaProject.multiThreadProgress.subBlockProcessingFlag == 2,
%-----


%STAGE 5 : Re-stitch reconstructed vasc mask
if vidaProject.multiThreadProgress.subBlockRestitchingFlag == 2,
% % %     fprintf('\nRestitching of registered subBlocks and cell centroids previously completed\n\n');
% % %     registeredVascRadiusMapFileName = fullfile(registeredFilesFolder,[vidaProjectRootName,'-registeredVascRadiusMap.mat']);
% % %     if cellChannelFlag == 1,
% % %         registeredCellCentroidAndLabelList = vidaProject.globalParameters.registeredCellCentroidAndLabelList;
% % %     end%if cellChannelFlag == 1,
else%if vidaProject.multiThreadProgress.subBlockRestitchingFlag == 2,
    fprintf('\nBegin Restitching of registered subBlocks and cell centroids\n\n');
    vidaProject.multiThreadProgress.subBlockRestitchingFlag = 1;
    regVascRadiusBlockFilePrefix = vidaProject.registeredExperiment.globalParameters.regVascRadiusBlockFilePrefix;
    registeredVascRadiusMapFileName = fullfile(registeredFilesFolder,[vidaProjectRootName,'-registeredVascRadiusMap.mat']);
    registeredVascReconstructedMaskFileName = fullfile(registeredFilesFolder,[vidaProjectRootName,'-registeredVascReconstructedMask']);
    for blockIter = 1:numSubBlocks,
        vascBlockRadiusMapFileNameList{blockIter} = [regVascRadiusBlockFilePrefix,num2str(blockIter),'.mat'];
    end%for blockIter = 1:numSubBlocks,
    %-----
    regRestitchParameters.fileRootName = vidaProjectRootName;
    regRestitchParameters.intermediateResultsFolder = intermediateResultsFolder;
    regRestitchParameters.processedDataFolder = processedDataFolder;
    regRestitchParameters.vascBlockRadiusMapFileNameList = vascBlockRadiusMapFileNameList;
    regRestitchParameters.fullStackSize = registeredStackSize;
    regRestitchParameters.blockGrid = blockGrid;
    regRestitchParameters.subVolumeOverlapSize = subVolumeOverlapSize;
    regRestitchParameters.overrideVascRadiusMapFullFileName = registeredVascRadiusMapFileName;
        regRestitchParameters = VidaRadiusMapRestitch(regRestitchParameters);
    vidaProject.globalParameters.registeredVascRadiusMapFileName = registeredVascRadiusMapFileName;
    %-----
    %Generate the registeredVascReconstructedMask from the radius map.
    reconstructParameters.inputRadiusMapFullFileName = registeredVascRadiusMapFileName;
    reconstructParameters.outputReconstructedMaskFullFileName = registeredVascReconstructedMaskFileName;
        reconstructParameters = VidaReconstructVascFromRadiusMap(reconstructParameters);
    vidaProject.globalParameters.registeredVascReconstructedMaskFileName = registeredVascReconstructedMaskFileName;
    %-----
    if cellChannelFlag == 1,
        %Compile Cell Data
        stageParameters.regCellCentroidMerge.currentStageString = 'registeredCellCentroidMerge';
        stageParameters.regCellCentroidMerge.dataStructureVariableName = 'registeredExperiment';
        stageParameters.regCellCentroidMerge.dataStructureFileName = registeredExperimentFileName;
        stageParameters.regCellCentroidMerge.mainFunctionCallString = 'VidaMultiStackMergeCellCentroids';
        %-----
        registeredCellCentroidAndLabelList = [];
        if isfield(registeredExperiment.registrationParameters,'blockStacking');
            if ~isfield(registeredExperiment.globalParameters,'cellCentroidAndLabelList')
                    [registeredExperiment] = VidaRunSaveOrSkip3(registeredExperiment,stageParameters.regCellCentroidMerge);%noteToSelf
                registeredCellCentroidAndLabelList = registeredExperiment.globalParameters.cellCentroidAndLabelList;%noteToSelf
                vidaProject.globalParameters.registeredCellCentroidAndLabelList = registeredCellCentroidAndLabelList;
                cellCentroidListOutputFileName = fullfile(vidaProjectRootFolder,[vidaProjectRootName,'-CellCentroidandLabelList.mat']);
                saveSize = (getfield(whos('registeredCellCentroidAndLabelList'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
                save(cellCentroidListOutputFileName,'registeredCellCentroidAndLabelList',scs);
            end%if ~isfield(registeredExperiment.globalParameters,'cellCentroidAndLabelList')
        else%if%isfield(registeredExperiment.registrationParameters,'blockStacking');
            currentRootName = currentImageStackFolderName(findstr(currentImageStackFolderName,'ImageStack-')+11:end);
            currentExpName = fullfile(fullfile(currentImageStackFolderName,'ProcessedData'),[currentRootName,'-Experiment.mat']);
            load(currentExpName,'experiment');
            vidaProject.globalParameters.registeredCellCentroidAndLabelList = experiment.globalParameters.cellCentroidAndLabelList;
        end%if%isfield(registeredExperiment.registrationParameters,'blockStacking');
        
    end%if cellChannelFlag == 1,
    vidaProject.registeredExperiment = registeredExperiment;
    %-----
    saveSize = (getfield(whos('registeredExperiment'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(registeredExperimentFileName,'registeredExperiment',scs);
    %-----
    vidaProject.multiThreadProgress.subBlockRestitchingFlag = 2;
    saveSize = (getfield(whos('vidaProject'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
    save(vidaProjectFileName,'vidaProject',scs);
end%if-else vidaProject.multiThreadProgress.subBlockRestitchingFlag == 2,


outProject = vidaProject;
