function outputParameters = VidaThresholdRelaxation2(inputParameters)

if ~exist('inputParameters'),
    inputParameters = [];
    vectorProjectFolder = cd;
    myDir = dir('*VectorProject.mat');
    vectorProjectFileName = fullfile(vectorProjectFolder,myDir(1).name);
    disp(['Loading VidaProject : ',datestr(now,'mmddyy_HH:MM.SS')]);
    load(vectorProjectFileName);
    inputParameters = vectorProject;
end%if ~exist(inputParameters)

vectorProject = inputParameters;
clear inputParameters


vectorProjectRootName = vectorProject.vectorProjectRootName;
vectorProjectFolder = vectorProject.vectorProjectFolder;
rootCodeFolder = vectorProject.rootCodeFolder;
vectorizedStructure = vectorProject.vectorizedStructure;
Strands = vectorProject.vectorizedStructure.Strands;
Vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.maskSize;
thresholdRelaxationParameters = vectorProject.thresholdRelaxationParameters;
augmentTRBlockEdgesFlag = thresholdRelaxationParameters.augmentTRBlockEdgesFlag;
edgeAugmentationGridStepSize = thresholdRelaxationParameters.edgeAugmentationGridStepSize;
%-----
voxelSizeInMicrons = vectorProject.voxelSizeInMicrons;
maxGapDist_um = thresholdRelaxationParameters.maxGapDist_um;
maxGapDist = maxGapDist_um / voxelSizeInMicrons;
thresholdRelaxationParameters.maxGapDist = maxGapDist;
vectorProject.thresholdRelaxationParameters = thresholdRelaxationParameters;

%-----
vsz1 = maskSize(1);
vsz2 = maskSize(2);
nslices = maskSize(3);
%-----
addpath(fullfile(rootCodeFolder,'GapConnectionFunctionsFolder'));
permutedFilesFolder = fullfile(vectorProjectFolder,'PermutedFilesFolder');

%% define file names 

GrayscaleVesselVolumeMatFname = fullfile(permutedFilesFolder,[vectorProjectRootName,'-permutedVascGrayscale.mat']);
EnhancedVesselVolumeMatFname = fullfile(permutedFilesFolder,[vectorProjectRootName,'-permutedVascEnhancedGrayscale.mat']);
ReconstructedVesselMaskMatFname = fullfile(permutedFilesFolder,[vectorProjectRootName,'-permutedVascReconstructedMask.mat']);
VectorizedMV3DFilename =  fullfile(vectorProjectFolder,[vectorProjectRootName,'-StrandSet.mv3d']);


%%




%Optional Augmentation Step:
%   Augment the data block by adding a 2d mesh of active points on every endface of the
%   data block.  Without this, endpoints near the block boundary may never find a
%   connection even if the (sub-threshold) grayscale indicates a connection to the edge of
%   the block.  By adding a mesh of valid anchor points on the block faces, these points
%   will connect with appropriate confidences.

if augmentTRBlockEdgesFlag == 1,
    disp(['Augmenting points with edge points: ',datestr(now,'mmddyy_HH:MM.SS')]);
    augmentedFileName = fullfile(vectorProjectFolder,[vectorProjectRootName,'-AugmentedPoints.mat']);
    if exist(augmentedFileName,'file');
        disp(['Loading Previous edge augmented file : ',datestr(now,'mmddyy_HH:MM.SS')]);
        load(augmentedFileName,'augmentedStruct'),
    else%if exist(augmentedFileName,'file');
        augmentedStruct = VidaTRAugmentBlockEdges(vectorizedStructure,edgeAugmentationGridStepSize);
        saveSize = (getfield(whos('augmentedStruct'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(augmentedFileName,'augmentedStruct',scs);
    end%if-else exist(augmentedFileName,'file');
    Strands = augmentedStruct.Strands;
    Vertices = augmentedStruct.Vertices;
end%if augmentTRBlockEdgesFlag == 1,











%%
%Generate MV3DFile

if ~exist(VectorizedMV3DFilename,'file');
    if ~WriteMV3DFile(Strands,Vertices,VectorizedMV3DFilename);
        fprintf('\nFailed to wirte %s!\nHalting',VectorizedMV3DFilename);
    end
end



%% ConnectGaps
%% Set parameters for test suite
VesselRepresentations.RawVesselVolumeMatFname = GrayscaleVesselVolumeMatFname;
VesselRepresentations.EnhancedVesselVolumeMatFname = EnhancedVesselVolumeMatFname;


TRFilesFolder = fullfile(vectorProjectFolder,'TRFilesFolder');
[success,message,messageID] = mkdir(vectorProjectFolder,'TRFilesFolder');

VesselRepresentations.IsotropicVesselMaskMatFname = ReconstructedVesselMaskMatFname;
VesselRepresentations.VectorizedMV3DFilename = VectorizedMV3DFilename;
VesselRepresentations.maskSize = maskSize;
VesselRepresentations.rootFolder = vectorProjectFolder;
VesselRepresentations.rootName = vectorProjectRootName;
VesselRepresentations.TRFilesFolder = TRFilesFolder;
VesselRepresentations.thresholdRelaxationParameters = thresholdRelaxationParameters;
VesselRepresentations.thresholdRelaxationParameters.boundBoxDimXYZ = maskSize;


    [GapConnectionStruct] = LocalThresholdRelaxationConnectionPrime3(VesselRepresentations);
vectorProject.GapConnectionStruct = GapConnectionStruct;
outputParameters = vectorProject;

