function outputParameters = VidaThresholdRelaxation3(inputParameters)

if ~exist('inputParameters'),
    inputParameters = [];
    vectorProjectFolder = cd;
    myDir = dir('*VectorProject.mat');
    vectorProjectFileName = fullfile(vectorProjectFolder,myDir(1).name);
    disp(['Loading VidaProject : ',datestr(now,'mmddyy_HH:MM.SS')]);
    load(vectorProjectFileName);
    inputParameters = vectorProject;
end%if ~exist(inputParameters)

vectorProject = inputParameters;
clear inputParameters


vectorProjectRootName = vectorProject.vectorProjectRootName;
vectorProjectFolder = vectorProject.vectorProjectFolder;
rootCodeFolder = vectorProject.rootCodeFolder;
vectorizedStructure = vectorProject.vectorizedStructure;
Strands = vectorProject.vectorizedStructure.Strands;
Vertices = vectorProject.vectorizedStructure.Vertices;
maskSize = vectorProject.maskSize;
thresholdRelaxationParameters = vectorProject.thresholdRelaxationParameters;
augmentTRBlockEdgesFlag = thresholdRelaxationParameters.augmentTRBlockEdgesFlag;
edgeAugmentationGridStepSize = thresholdRelaxationParameters.edgeAugmentationGridStepSize;
%-----
voxelSizeInMicrons = vectorProject.voxelSizeInMicrons;
maxGapDist_um = thresholdRelaxationParameters.maxGapDist_um;
maxGapDist = maxGapDist_um / voxelSizeInMicrons;
thresholdRelaxationParameters.maxGapDist = maxGapDist;
vectorProject.thresholdRelaxationParameters = thresholdRelaxationParameters;

%-----
vsz1 = maskSize(1);
vsz2 = maskSize(2);
nslices = maskSize(3);
%-----
addpath(fullfile(rootCodeFolder,'GapConnectionFunctionsFolder'));
permutedFilesFolder = fullfile(vectorProjectFolder,'PermutedFilesFolder');

%% define file names 

vascGrayscaleFileName = fullfile(permutedFilesFolder,[vectorProjectRootName,'-permutedVascGrayscale.mat']);
vascEnhancedGrayscaleFileName = fullfile(permutedFilesFolder,[vectorProjectRootName,'-permutedVascEnhancedGrayscale.mat']);
vascReconstructedMaskFileName = fullfile(permutedFilesFolder,[vectorProjectRootName,'-permutedVascReconstructedMask.mat']);


%Optional Augmentation Step:
%   Augment the data block by adding a 2d mesh of active points on every endface of the
%   data block.  Without this, endpoints near the block boundary may never find a
%   connection even if the (sub-threshold) grayscale indicates a connection to the edge of
%   the block.  By adding a mesh of valid anchor points on the block faces, these points
%   will connect with appropriate confidences.

augmentTRBlockEdgesFlag = 0,disp('VidaThresholdRelaxation3.m_line56')%Hardcode
augmentTRBlockEdgesFlag = 0,disp('VidaThresholdRelaxation3.m_line56')%Hardcode
augmentTRBlockEdgesFlag = 0,disp('VidaThresholdRelaxation3.m_line56')%Hardcode
augmentTRBlockEdgesFlag = 0,disp('VidaThresholdRelaxation3.m_line56')%Hardcode
augmentTRBlockEdgesFlag = 0,disp('VidaThresholdRelaxation3.m_line56')%Hardcode



if augmentTRBlockEdgesFlag == 1,
    disp(['Augmenting points with edge points: ',datestr(now,'mmddyy_HH:MM.SS')]);
    augmentedFileName = fullfile(vectorProjectFolder,[vectorProjectRootName,'-AugmentedPoints.mat']);
    if exist(augmentedFileName,'file');
        disp(['Loading Previous edge augmented file : ',datestr(now,'mmddyy_HH:MM.SS')]);
        load(augmentedFileName,'augmentedStruct'),
    else%if exist(augmentedFileName,'file');
        augmentedStruct = VidaTRAugmentBlockEdges2(vectorizedStructure,edgeAugmentationGridStepSize);
        saveSize = (getfield(whos('augmentedStruct'),'bytes')>1.9e9);switch saveSize, case 0,scs='-v7';case 1,scs='-v7.3';end
        save(augmentedFileName,'augmentedStruct',scs);
    end%if-else exist(augmentedFileName,'file');
    Strands = augmentedStruct.Strands;
    Vertices = augmentedStruct.Vertices;
end%if augmentTRBlockEdgesFlag == 1,

%% ConnectGaps
%% Set parameters for test suite

TRFilesFolder = fullfile(vectorProjectFolder,'TRFilesFolder');
[success,message,messageID] = mkdir(vectorProjectFolder,'TRFilesFolder');

thresholdRelaxationParameters.vascGrayscaleFileName = vascGrayscaleFileName;
thresholdRelaxationParameters.vascEnhancedGrayscaleFileName = vascEnhancedGrayscaleFileName;
thresholdRelaxationParameters.vascReconstructedMaskFileName = vascReconstructedMaskFileName;
thresholdRelaxationParameters.maskSize = maskSize;
thresholdRelaxationParameters.Strands = Strands;
thresholdRelaxationParameters.Vertices = Vertices;
thresholdRelaxationParameters.rootFolder = vectorProjectFolder;
thresholdRelaxationParameters.rootName = vectorProjectRootName;
thresholdRelaxationParameters.TRFilesFolder = TRFilesFolder;
thresholdRelaxationParameters.boundBoxDimXYZ = maskSize;
    [GapConnectionStruct] = LocalThresholdRelaxationConnectionPrime4(thresholdRelaxationParameters);
vectorProject.GapConnectionStruct = GapConnectionStruct;
outputParameters = vectorProject;

